//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Collections.Specialized;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Resources;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.Biz;
using System.Collections.Generic;

public partial class AppSettingMgmt : System.Web.UI.UserControl
{
    #region Resources
    private const string _controlName = "AppSettingMgmt.ascx";
    public struct ResKeys
    {
        public const string DomainLabel = "DomainLabel";
        public const string LogFileLabel = "LogFileLabel";
        public const string ServiceUserLoginLabel = "ServiceUserLoginLabel";
        public const string ConnectionStringLabel = "ConnectionStringLabel";
        public const string AttachmentFolderLabel = "AttachmentFolderLabel";
        public const string AttachmentBufferSizeLabel = "AttachmentBufferSizeLabel";
        public const string EmailMessagingEnabledCheckBox = "EmailMessagingEnabledCheckBox";
        public const string EmailMessagingServerLabel = "EmailMessagingServerLabel";
        public const string EmailMessagingAccountLabel = "EmailMessagingAccountLabel";
        public const string EmailMessagingPasswordLabel = "EmailMessagingPasswordLabel";
        public const string EmailMessagingSSLEnabled = "EmailMessagingSSLEnabled";
        public const string IMWebServiceEnabledCheckBox = "IMWebServiceEnabledCheckBox";
        public const string IMWebServiceLabel = "IMWebServiceLabel";
        public const string SaveButton = "SaveButton";
        public const string ResetApplication = "ResetApplication";
    }
    protected ResourceManager _rm;
    #endregion

    protected void Page_Init(object sender, EventArgs e)
    {
        _rm = ResourceHelper.Current.GetResourceManagerForPage(_controlName);
    }

    protected class AppSettingMgmtSessionData
    {
        public NameValueCollection AppSettings;
    }

    protected AppSettingMgmtSessionData _sessionData = new AppSettingMgmtSessionData();

    protected string GenerateNewGuid()
    {
        System.Guid guid = System.Guid.NewGuid();
        return guid.ToString();
    }

    protected UserSession _us;

    protected void Page_Load(object sender, EventArgs e)
    {
        _us = (UserSession) Session[SessionStateKeys.UserSession];
        if (!IsPostBack)
        {
            if (_us.UserInfo.UserIsAdministrator())
            {
                _sessionData.AppSettings = (new AppSettingOperator()).Load(DCAppSettings.Current.SettingFile);

                BindControls();
                hidSessionId.Value = GenerateNewGuid();
                Session[hidSessionId.Value] = _sessionData;
                DataBind();
            }
            else
            {
                Response.Redirect("ErrorMessage.aspx?ResourceKey=AccessDenied");
            }
        }
        else
        {
            _sessionData = (AppSettingMgmtSessionData)Session[hidSessionId.Value];
        }

    }

    protected void BindControls()
    {
        uiDomain.Text = _sessionData.AppSettings[SettingFileKeys.Domain];
        uiLogFile.Text = _sessionData.AppSettings[SettingFileKeys.LogFile];
        uiServiceUserLogin.Text = _sessionData.AppSettings[SettingFileKeys.ServiceUserLogin];
        uiConnectionString.Text = _sessionData.AppSettings[SettingFileKeys.ConnectionString];
        uiAttachmentBufferSize.Text = _sessionData.AppSettings[SettingFileKeys.AttachmentBufferSize];
        uiAttachmentFolder.Text = _sessionData.AppSettings[SettingFileKeys.AttachmentFolder];
        uiEmailMessagingServer.Text = _sessionData.AppSettings[SettingFileKeys.EmailMessagingServer];
        uiEmailMessagingAccount.Text = _sessionData.AppSettings[SettingFileKeys.EmailMessagingAccount];
        uiEmailMessagingPassword.Text = _sessionData.AppSettings[SettingFileKeys.EmailMessagingPassword];
        uiIMWebService.Text = _sessionData.AppSettings[SettingFileKeys.IMWebService];
        uiIMWebServiceEnabled.Checked = 
            (_sessionData.AppSettings[SettingFileKeys.IMWebServiceEnabled].ToLower() == "true");
        uiEmailMessagingEnabled.Checked = 
            (_sessionData.AppSettings[SettingFileKeys.EmailMessagingEnabled].ToLower() == "true");
        uiEmailMessagingSSLEnabled.Checked =
            (_sessionData.AppSettings[SettingFileKeys.EmailMessagingSSLEnabled].ToLower() == "true");
    }

    protected void BindData()
    {
        _sessionData.AppSettings[SettingFileKeys.Domain] = uiDomain.Text;
        _sessionData.AppSettings[SettingFileKeys.LogFile] = uiLogFile.Text;
        _sessionData.AppSettings[SettingFileKeys.ServiceUserLogin] = uiServiceUserLogin.Text;
        _sessionData.AppSettings[SettingFileKeys.ConnectionString] = uiConnectionString.Text;
        _sessionData.AppSettings[SettingFileKeys.AttachmentBufferSize] = uiAttachmentBufferSize.Text;
        _sessionData.AppSettings[SettingFileKeys.AttachmentFolder] = uiAttachmentFolder.Text;
        _sessionData.AppSettings[SettingFileKeys.EmailMessagingServer] = uiEmailMessagingServer.Text;
        _sessionData.AppSettings[SettingFileKeys.EmailMessagingAccount] = uiEmailMessagingAccount.Text;
        _sessionData.AppSettings[SettingFileKeys.EmailMessagingPassword] = uiEmailMessagingPassword.Text;
        _sessionData.AppSettings[SettingFileKeys.IMWebService] = uiIMWebService.Text;

        if(uiIMWebServiceEnabled.Checked)
        {
            _sessionData.AppSettings[SettingFileKeys.IMWebServiceEnabled] = "true";
        }
        else
        {
            _sessionData.AppSettings[SettingFileKeys.IMWebServiceEnabled] = "false";
        }

        if (uiEmailMessagingEnabled.Checked)
        {
            _sessionData.AppSettings[SettingFileKeys.EmailMessagingEnabled] = "true";
        }
        else
        {
            _sessionData.AppSettings[SettingFileKeys.EmailMessagingEnabled] = "false";
        }

        if (uiEmailMessagingSSLEnabled.Checked)
        {
            _sessionData.AppSettings[SettingFileKeys.EmailMessagingSSLEnabled] = "true";
        }
        else
        {
            _sessionData.AppSettings[SettingFileKeys.EmailMessagingSSLEnabled] = "false";
        }
    }

    protected void uiSave_Click(object sender, EventArgs e)
    {
        BindData();
        (new AppSettingOperator()).Save(DCAppSettings.Current.SettingFile, _sessionData.AppSettings);
    }

    private void ResetApplication()
    {
        string currentFolder = this.Context.Request.MapPath(this.Context.Request.ApplicationPath);
        ((Global)this.ApplicationInstance).EndApplication();
        DCAppSettings.Current.LoadSettings(currentFolder);

        Dictionary<string, object> appStateDict = (new BizGlobal()).Init(
            this.Server.MachineName,
            this.Context.Request.ApplicationPath,
            currentFolder,
            this.Context.Request.Url.AbsoluteUri,
            this.Context.Request.Url.Port
            );

        foreach (string key in appStateDict.Keys)
        {
            Application[key] = appStateDict[key];
        }

        Response.Redirect("LoginForUser.aspx");
    }

    protected void uiResetApplication_Click(object sender, EventArgs e)
    {
        ResetApplication();
        //((Global)this.ApplicationInstance).EndSession();
        //((Global)this.ApplicationInstance).EndApplication();

        //(new BizGlobal()).Init(
        //    this.Server.MachineName,
        //    this.Context.Request.ApplicationPath,
        //    this.Context.Request.MapPath(this.Context.Request.ApplicationPath),
        //    this.Context.Request.Url.AbsoluteUri
        //    );
        //((Global)this.ApplicationInstance).StartApplication();
        //((Global)this.ApplicationInstance).StartSession();
        //Response.Redirect("Page01.aspx");
    }
}
