//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Data;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Web;
using System.Web.SessionState;
using System.IO;
using System.Configuration;
using System.DirectoryServices;
using System.Text;
using System.Reflection;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.Biz;
using Evpo.DocCirc.UI;

//using MSIMLib = MSIMCliSDKLib;

//namespace Evpo.DocCirc
//{
	/// <summary>
	/// Summary description for Global.
	/// </summary>
	public class Global : System.Web.HttpApplication
	{
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.IContainer components = null;

		public Global()
		{
			InitializeComponent();
            //_appStateWrapper = new AppStateWrapper(this);
		}
        
        public void StartApplication()
        {
            DCAppSettings.Current.LoadSettings(
                this.Context.Request.MapPath(this.Context.Request.ApplicationPath)
                );
            
            BizGlobal bizGlobal = new BizGlobal();
            Dictionary<string, object> appStateDict = null;

            bool isSetupNeeded = bizGlobal.GetIsSetupNeeded();

            if (!isSetupNeeded)
            {
                appStateDict = bizGlobal.Init(
                      this.Server.MachineName,
                      this.Context.Request.ApplicationPath,
                      this.Context.Request.MapPath(this.Context.Request.ApplicationPath),
                      this.Context.Request.Url.AbsoluteUri,
                      this.Context.Request.Url.Port  
                      );
            }
            else
            {
                SettingKeeper sk = new SettingKeeper();
                bizGlobal.InitializeResources(
                    this.Context.Request.MapPath(this.Context.Request.ApplicationPath),
                    sk
                    );
                this.Application[AppStateKeys.SettingKeeper] = sk;
                appStateDict = new Dictionary<string, object>(1);
            }

            appStateDict.Add(AppStateKeys.IsSetupNeeded, isSetupNeeded);

            foreach (string key in appStateDict.Keys)
            {
                Application[key] = appStateDict[key];
            }

            try
            {//temporary for debug
            }
            catch (Exception ex)
            {
                if (LogHelper.Current != null)
                {
                    LogHelper.Current.WriteException(ex);
                }
                else
                {
                    throw ex;
                }
            }
        }

		protected void Application_Start(Object sender, EventArgs e)
		{

            StartApplication();
		}


        public void StartSession()
        {
            object value = Application[AppStateKeys.IsSetupNeeded];

            if (value == null)
            {
                StartApplication();
                value = Application[AppStateKeys.IsSetupNeeded];
            }

            bool isSetupNeeded = (bool)value;

            if (this.Context.User.Identity.IsAuthenticated && !isSetupNeeded)
            {
                SettingKeeper sk = (SettingKeeper)Application[AppStateKeys.SettingKeeper];
                DataConnection con = new DataConnection(sk.ConnectionString);
                bool logged = true;
                UserToken token = null;
                con.Open();
                try
                {
                    if (Context.User.Identity.AuthenticationType == "Forms")
                    {
                        token = UserHelper.Current.GetUserTokenByLogin(Context.User.Identity.Name, con);
                        logged = true;
                    }
                    else
                    {
                        int userId =
                            UserHelper.Current.GetUserIdByExternalIdentity(Context.User.Identity.Name, con);

                        UserProperties prop = UserHelper.Current.GetUserProperties(userId, con);

                        if (!prop.ExternalAuthorization)
                        {
                            logged = false;
                            //throw new DcObjectNotFoundException(
                            //    String.Format(
                            //        "User {0} cannot use external authentication. See user properties!",
                            //        Context.User.Identity.Name
                            //        )
                            //    );
                        }

                        token = new UserToken(prop.UserId.Value, prop.Login);
                    }
                }
                catch (DcObjectNotFoundException ex)
                {
                    logged = false;
                }

                if (logged)
                {
                    UserSession session = new UserSession(
                        token,
                        con,
                        sk.ConnectionString, // make separate conn string for a user in the future
                        (UserInfo)Application[AppStateKeys.ServiceUserInfo],
                        (SettingKeeper)Application[AppStateKeys.SettingKeeper]
                        );


                    this.Session[SessionStateKeys.UserSession] = session;
                }
                else
                {
                    Response.Redirect("LoginForUser.aspx");
                }
                con.Close();

            }
        }
         
 
		protected void Session_Start(Object sender, EventArgs e)
		{
            StartSession();
       	}

		protected void Application_BeginRequest(Object sender, EventArgs e)
		{

		}

		protected void Application_EndRequest(Object sender, EventArgs e)
		{

		}

		protected void Application_AuthenticateRequest(Object sender, EventArgs e)
		{

		}

		protected void Application_Error(Object sender, EventArgs e)
		{

		}

		protected void Session_End(Object sender, EventArgs e)
		{
            EndSession();
		}

		protected void Application_End(Object sender, EventArgs e)
		{
            EndApplication();
		}

        public void EndSession()
        {
            UserSession us = (UserSession)Session[SessionStateKeys.UserSession];
            us.Close();
        }

        public void EndApplication()
        {
            ((Notificator)Application[AppStateKeys.Notificator]).Close();
            ((DataConnection)Application[AppStateKeys.ApplicationDataConnection]).Close();
            LogHelper.Current.AddStr(ResourceHelper.StrForLog[LogResKeys.ApplicationEnded]);
        }

		#region Web Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.components = new System.ComponentModel.Container();
		}
		#endregion
	}

    

   
    
//}

