//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.IO;
using Evpo.DocCirc.Biz;
using Evpo.DocCirc;
using Evpo.DocCirc.Common;

public partial class Attachment : System.Web.UI.Page
{
    protected struct PageParameter
    {
        public const string File = "file";
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];

        string sFile = null;
        if (us.SettingKeeper.AttachmentSavingDialogEnabled)
        {
            sFile = this.Request.Params[PageParameter.File];

        }
        else
        {
            sFile = this.Request.PathInfo;
        }

        if (String.IsNullOrEmpty(sFile))
        {
            throw new ArgumentException("Request is invalid");
        }

        long id = -1;
        if (us.SettingKeeper.AttachmentSavingDialogEnabled)
        {
            id = long.Parse(sFile);
        }
        else
        {
            string[] pathInfoItems = sFile.Split('/');
            id = long.Parse(pathInfoItems[1]);
        }
        
        AttachmentData ad = new AttachmentData(us.UserInfo, us.DataConnection, us.SettingKeeper);
        AttachmentItem item = ad.GetAttachmentItem(id);

        if (!us.SettingKeeper.AttachmentSavingDialogEnabled && sFile.Split('/').Length == 2)
        {
            Response.Redirect(
            String.Format("{0}/{1}", Request.RawUrl, HttpUtility.UrlPathEncode(item.FileName)));
        }
            
        Response.Clear();
        Response.ContentType = item.ContentType;

        if (us.SettingKeeper.AttachmentSavingDialogEnabled)
        {
            Response.AddHeader(
                "Content-Disposition",
                String.Format("attachment; filename=\"{0}\"", item.FileName)
                );
        }
        //string htmlFileName = HttpUtility.UrlPathEncode(item.FileName);
        //Response.AddHeader(
        //    "Content-Disposition",
        //    String.Format("attachment; filename={0}", htmlFileName)
        //    );

        Stream attStream = ad.OpenAttachment(item);

        byte[] buffer = new byte[us.SettingKeeper.AttachmentBufferSize];
        long bytesToRead = attStream.Length;
        while (bytesToRead > 0 && Response.IsClientConnected)
        {
            int bytesRead = attStream.Read(buffer, 0, us.SettingKeeper.AttachmentBufferSize);
            Response.OutputStream.Write(buffer, 0, bytesRead);
            Response.Flush();
            bytesToRead -= bytesRead;
        }
        attStream.Close();
        Response.OutputStream.Close();
        Response.Close();
    }
}
