//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Collections.Generic;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Resources;
using Evpo.DocCirc.UI;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.Biz;
using Evpo.DocCirc;

public partial class CustomQuery : System.Web.UI.UserControl
{
    #region Resources
    private const string _controlName = "CustomQuery.ascx";
    public struct ResKeys
    {
        public const string ConsideringUser = "ConsideringUser";
        public const string SearchTypeExact = "SearchTypeExact";
        public const string SearchTypeBegins = "SearchTypeBegins";
        public const string SearchTypeContains = "SearchTypeContains";
        public const string ConditionLabel = "ConditionLabel";
        public const string QueryLabel = "QueryLabel";
        public const string AddCondition = "AddCondition";
        public const string StartQuery = "StartQuery";
        public const string DeleteCondition = "DeleteCondition";
        public const string FromToPeriod = "FromToPeriod";
    }
    protected ResourceManager _rm;
    #endregion

    protected struct ControlConst
    {
        public const string ConsideringUserField = "ConsideringUser";
    }

    protected class CustomQuerySessionData
    {
        public List<DisplayableFilterItem> FilterItems = new List<DisplayableFilterItem>();
        public bool ListIsVisible = false;
        public List<FieldListItem> FieldListItems = new List<FieldListItem>();
    }
    protected CustomQuerySessionData _sessionData = new CustomQuerySessionData();

    protected class FieldListItem
    {
        private string _fieldDisplayName;

	    public string FieldDisplayName
	    {
		    get { return _fieldDisplayName;}
		    set { _fieldDisplayName = value;}
	    }

        private string _fieldName;

        public string FieldName
        {
            get { return _fieldName; }
            set { _fieldName = value; }
        }

        private string _controlId;

        public string ControlId
        {
            get { return _controlId; }
            set { _controlId = value; }
        }

        private Type _valueType;

        public Type ValueType
        {
            get { return _valueType; }
            set { _valueType = value; }
        }


        public void OnSelect(object sender)
        {
            
            Select(sender, new EventArgs());
        }

        public event EventHandler Select;

        public FieldListItem(string fieldName, string fieldDisplayName, string controlId, Type valueType)
        {
            _fieldName = fieldName;
            _fieldDisplayName = fieldDisplayName;
            _controlId = controlId;
            _valueType = valueType;
        }
	
    }

    protected class DisplayableFilterItem
    {

        private DocCardFilterItem _filterItem = null;

        public DocCardFilterItem FilterItem
        {
            get { return _filterItem; }
            set { _filterItem = value; }
        }

        private int? _consideringUser = null;

        public int? ConsideringUser
        {
            get { return _consideringUser; }
            set { _consideringUser = value; }
        }

        private string _fieldDisplayName;

        public string FieldDisplayName
        {
            get { return _fieldDisplayName; }
            set { _fieldDisplayName = value; }
        }

        private string _displayValue;

        public string DisplayValue
        {
            get { return _displayValue; }
            set { _displayValue = value; }
        }

        public DisplayableFilterItem(int consideringUser, string fieldDisplayName, string displayValue)
        {
            _consideringUser = consideringUser;
            _displayValue = displayValue;
            _fieldDisplayName = fieldDisplayName;
        }

        public DisplayableFilterItem(DocCardFilterItem filterItem, string fieldDisplayName, string displayValue)
        {
            _filterItem = filterItem;
            _displayValue = displayValue;
            _fieldDisplayName = fieldDisplayName;
        }
    }


    protected UserSession _us;

    protected string GenerateNewGuid()
    {
        System.Guid guid = System.Guid.NewGuid();
        return guid.ToString();
    }
    
    protected void Page_Init(object sender, EventArgs e)
    {
        _rm = ResourceHelper.Current.GetResourceManagerForPage(_controlName);
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        _us = (UserSession)Session[SessionStateKeys.UserSession];
        if (!IsPostBack)
        {
            hidSessionId.Value = GenerateNewGuid();
            InitializeUI();
            CreateFieldList();
            FillFieldList();

            Session[hidSessionId.Value] = _sessionData;
        }
        else
        {
            _sessionData = (CustomQuerySessionData)Session[hidSessionId.Value];
        }

        this.DataBind();
    }

    public override void DataBind()
    {
        foreach (Control cnt in Controls)
        {
            if (cnt.ID != uiSelection.ID && cnt.ID != uiConditions.ID)
            {
                cnt.DataBind();
            }
        }
    }

    private void FillFieldList()
    {
        foreach (FieldListItem listItem in _sessionData.FieldListItems)
        {
            uiField.Items.Add(new ListItem(listItem.FieldDisplayName));
        }

        _sessionData.FieldListItems[0].OnSelect(this);
    }

    private void InitializeUI()
    {
        // Search types
        uiSearchType.Items.Add(
            new ListItem(
                _rm.GetString(ResKeys.SearchTypeExact), 
                Enum.GetName(typeof(DocCardSearchType), DocCardSearchType.Exact)
                )
            );
        uiSearchType.Items.Add(
            new ListItem(
                _rm.GetString(ResKeys.SearchTypeBegins),
                Enum.GetName(typeof(DocCardSearchType), DocCardSearchType.Begins)
                )
            );
        uiSearchType.Items.Add(
            new ListItem(_rm.GetString(ResKeys.SearchTypeContains),
            Enum.GetName(typeof(DocCardSearchType), DocCardSearchType.Contains)
            )
            );
    }

    protected void CreateFieldList()
    {
        ResourceManager rm = ResourceHelper.Current.GetResourceManagerForPage(GlobalStringConstants.DocCardFieldNameResourcesId);
        FieldListItem fieldItem = new FieldListItem(
            DocCardFieldName.InternalNumber,
            rm.GetString(DocCardFieldName.InternalNumber),
            uiCustomText.ID,
            typeof(string)
            );
        fieldItem.Select += new EventHandler(CustomTextField_OnSelect);
        _sessionData.FieldListItems.Add(fieldItem);

        fieldItem = new FieldListItem(
            DocCardFieldName.Subject,
            rm.GetString(DocCardFieldName.Subject),
            uiCustomText.ID,
            typeof(string)
            );
        fieldItem.Select += new EventHandler(CustomTextField_OnSelect);
        _sessionData.FieldListItems.Add(fieldItem);

        fieldItem = new FieldListItem(
            DocCardFieldName.RegisteringUser,
            rm.GetString(DocCardFieldName.RegisteringUser),
            uiDropDown.ID,
            typeof(int)
            );
        fieldItem.Select += new EventHandler(UserListField_OnSelect);
        _sessionData.FieldListItems.Add(fieldItem);

        fieldItem = new FieldListItem(
            DocCardFieldName.DocType,
            rm.GetString(DocCardFieldName.DocType),
            uiDropDown.ID,
            typeof(int)
            );
        fieldItem.Select += new EventHandler(DocTypeListField_OnSelect);
        _sessionData.FieldListItems.Add(fieldItem);

        fieldItem = new FieldListItem(
            DocCardFieldName.OriginalNumber,
            rm.GetString(DocCardFieldName.OriginalNumber),
            uiCustomText.ID,
            typeof(string)
            );
        fieldItem.Select += new EventHandler(CustomTextField_OnSelect);
        _sessionData.FieldListItems.Add(fieldItem);

        fieldItem = new FieldListItem(
            ControlConst.ConsideringUserField,
            _rm.GetString(ResKeys.ConsideringUser),
            uiDropDown.ID,
            typeof(int)
            );
        fieldItem.Select += new EventHandler(UserListField_OnSelect);
        _sessionData.FieldListItems.Add(fieldItem);

        fieldItem = new FieldListItem(
            DocCardFieldName.DocStatus,
            rm.GetString(DocCardFieldName.DocStatus),
            uiDropDown.ID,
            typeof(int)
            );
        fieldItem.Select += new EventHandler(DocStatusListField_OnSelect);
        _sessionData.FieldListItems.Add(fieldItem);
    }

    void DocStatusListField_OnSelect(object sender, EventArgs e)
    {
        CustomQuery cq = (CustomQuery)sender;

        cq.uiDropDown.Visible = true;
        cq.uiCustomText.Visible = false;
        cq.uiTimeSpan.Visible = false;
        cq.uiSearchType.Visible = false;

        cq.uiDropDown.Items.Clear();
        cq.uiDropDown.Items.Add(
            new ListItem(
                ResourceHelper.StrForUI[UIResKeys.DraftDocStatus], 
                DBConstKeeper.Current.DocStatus.Draft.ToString()
                )
            );

        cq.uiDropDown.Items.Add(
            new ListItem(
                ResourceHelper.StrForUI[UIResKeys.UnderConsDocStatus],
                DBConstKeeper.Current.DocStatus.UnderCons.ToString()
                )
            );

        cq.uiDropDown.Items.Add(
            new ListItem(
                ResourceHelper.StrForUI[UIResKeys.FinalizedDocStatus],
                DBConstKeeper.Current.DocStatus.Finalized.ToString()
                )
            );
    }

    void DocTypeListField_OnSelect(object sender, EventArgs e)
    {
        CustomQuery cq = (CustomQuery)sender;

        cq.uiDropDown.Visible = true;
        cq.uiCustomText.Visible = false;
        cq.uiTimeSpan.Visible = false;
        cq.uiSearchType.Visible = false;

        cq.uiDropDown.Items.Clear();

        DocTypeData dtd = new DocTypeData(_us.UserInfo, _us.DataConnection);
        foreach (DocTypeItem item in dtd.GetDocTypes())
        {
            cq.uiDropDown.Items.Add(new ListItem(item.Name, item.Id.ToString()));
        }

        (new ListItemSorter()).SortByText(cq.uiDropDown.Items);
    }

    void UserListField_OnSelect(object sender, EventArgs e)
    {
        CustomQuery cq = (CustomQuery)sender;
        cq.uiDropDown.Visible = true;
        cq.uiCustomText.Visible = false;
        cq.uiTimeSpan.Visible = false;
        cq.uiSearchType.Visible = false;

        cq.uiDropDown.Items.Clear();

        UserProperties[] props = UserHelper.Current.GetAllUserProperties(_us.DataConnection);
        foreach (UserProperties prop in props)
        {
            cq.uiDropDown.Items.Add(new ListItem(prop.DisplayName, prop.UserId.ToString()));
        }
    }

    protected void CustomTextField_OnSelect(object sender, EventArgs e)
    {
        CustomQuery cq = (CustomQuery)sender;

        cq.uiDropDown.Visible = false;
        cq.uiCustomText.Visible = true;
        cq.uiTimeSpan.Visible = false;
        cq.uiText.Text = "";
        cq.uiSearchType.Visible = true;
    }

    protected void uiField_SelectedIndexChanged(object sender, EventArgs e)
    {
        _sessionData.FieldListItems[uiField.SelectedIndex].OnSelect(this);
    }

    protected void uiAddCondition_Click(object sender, EventArgs e)
    {
        AddCondition();  
    }

    protected void AddCondition()
    {
        FieldListItem fieldItem = _sessionData.FieldListItems[uiField.SelectedIndex];

        //if the field has already been added, remove previous
        
        DisplayableFilterItem dispItemToRemove = null;
        foreach (DisplayableFilterItem dispItem in _sessionData.FilterItems)
        {
            if (dispItem.FilterItem != null && dispItem.FilterItem.FieldName == fieldItem.FieldName)
            {
                dispItemToRemove = dispItem;
                break;
            }

            if (fieldItem.FieldName == ControlConst.ConsideringUserField && dispItem.ConsideringUser != null)
            {
                dispItemToRemove = dispItem;
                break;
            }
        }

        if (dispItemToRemove != null)
        {
            _sessionData.FilterItems.Remove(dispItemToRemove);
        }


        bool isConsideringUser = false;
        if (fieldItem.FieldName == ControlConst.ConsideringUserField)
        {
            isConsideringUser = true;
        }

        DisplayableFilterItem displayItem = null;

        if (fieldItem.ControlId == uiDropDown.ID)
        {

            if (isConsideringUser)
            {
                displayItem = new DisplayableFilterItem(
                    int.Parse(uiDropDown.SelectedValue),
                    fieldItem.FieldDisplayName,
                    uiDropDown.SelectedItem.Text
                    );
            }
            else
            {
                object value = ParseString(uiDropDown.SelectedValue, fieldItem.ValueType);
                DocCardFilterItem filterItem = new DocCardFilterItem(fieldItem.FieldName, value);
                displayItem =
                    new DisplayableFilterItem(filterItem, fieldItem.FieldDisplayName, uiDropDown.SelectedItem.Text);
            }
        }
        else if (fieldItem.ControlId == uiCustomText.ID)
        {
            object value = ParseString(uiText.Text, fieldItem.ValueType);
            DocCardSearchType searchType = 
                (DocCardSearchType)Enum.Parse(typeof(DocCardSearchType), uiSearchType.SelectedValue);
            DocCardFilterItem filterItem = new DocCardFilterItem(fieldItem.FieldName, value, searchType);
            displayItem = new DisplayableFilterItem(
                filterItem, fieldItem.FieldDisplayName, 
                String.Format("({0}) {1}", uiSearchType.SelectedItem.Text, uiText.Text)
                );
        }
        else if (fieldItem.ControlId == uiTimeSpan.ID)
        {
            DocCardFilterItem filterItem = new PeriodDocCardFilterItem(
                fieldItem.FieldName,
                DateTime.Parse(uiFrom.Text),
                DateTime.Parse(uiTo.Text)
                );

            displayItem = new DisplayableFilterItem(
                filterItem,
                fieldItem.FieldDisplayName,
                String.Format(_rm.GetString(ResKeys.FromToPeriod), uiFrom.Text, uiTo.Text)
                );
        }

        _sessionData.FilterItems.Add(displayItem);

        uiConditions.DataSource = _sessionData.FilterItems;
        uiConditions.DataBind();
    }

    protected object ParseString(string value, Type type)
    {
        object retVal = null;
        
        if(type == typeof(int))
        {
            retVal = int.Parse(value);
        }
        else if(type == typeof(long))
        {
            retVal = long.Parse(value);
        }
        else if(type == typeof(string))
        {
            retVal = value;
        }
        else
        {
            throw new ApplicationException("Unexpected type!");
        }

        return retVal;
    }

    protected void uiDeleteCondition_Click(object sender, EventArgs e)
    {
        Control cnt = (Control)sender;
        RepeaterItem repItem = (RepeaterItem)cnt.Parent;

        _sessionData.FilterItems.RemoveAt(repItem.ItemIndex);

        uiConditions.DataSource = _sessionData.FilterItems;
        uiConditions.DataBind();

    }

    protected void uiStartQuery_Click(object sender, EventArgs e)
    {
        if (_sessionData.FilterItems.Count == 0)
        {
            AddCondition();
        }

        List<int> consideringUsers = new List<int>();
        List<DocCardFilterItem> filterItems = new List<DocCardFilterItem>();

        foreach (DisplayableFilterItem displayItem in _sessionData.FilterItems)
        {
            if (displayItem.ConsideringUser != null)
            {
                consideringUsers.Add(displayItem.ConsideringUser.Value);
            }
            else if (displayItem.FilterItem != null)
            {
                filterItems.Add(displayItem.FilterItem);
            }
        }

        if (!uiSelection.Visible)
        {
            uiSelection.Visible = true;
        }

        uiSelection.SetConsideringUsers(consideringUsers.ToArray());
        uiSelection.SetFilterItems(filterItems.ToArray());
        uiSelection.RefreshTable();

        
    }
}
