//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Resources;
using System.Collections.Generic;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.Biz;
using Evpo.DocCirc;

public partial class Deputy : System.Web.UI.UserControl
{


    #region Resources
    private const string _controlName = "Deputy.ascx";
    public struct ResKeys
    {
        public const string DeputyColumnHeader = "DeputyColumnHeader";
        public const string NotifyColumnHeader = "NotifyColumnHeader";
        public const string DeleteColumnHeader = "DeleteColumnHeader";
        public const string AddDeputyField = "AddDeputyField";
        public const string AddDeputyButton = "AddDeputyButton";
        public const string ApplyButton = "ApplyButton";
        public const string AllowDeputyNotification = "AllowDeputyNotification";
        public const string DeputyIsAlreadyAddedMessage = "DeputyIsAlreadyAddedMessage";
        public const string SaveButton = "SaveButton";

    }
    protected ResourceManager _rm;
    #endregion

    protected class DisplayDeputyItem : DeputyItem
    {
        private string _userDisplayName;

        public string UserDisplayName
        {
            get { return _userDisplayName; }
            set { _userDisplayName = value; }
        }

        private string _deputyDisplayName;

        public string DeputyDisplayName
        {
            get { return _deputyDisplayName; }
            set { _deputyDisplayName = value; }
        }

        private bool _deleteMark;

        public bool DeleteMark
        {
            get { return _deleteMark; }
            set { _deleteMark = value; }
        }

        public DisplayDeputyItem(DeputyItem item, DataConnection con)
            : base(item.DeputyId, item.User, item.Deputy, item.Notify)
        {
            _userDisplayName = UserHelper.Current.GetUserProperties(item.User, con).DisplayName;
            _deputyDisplayName = UserHelper.Current.GetUserProperties(item.Deputy, con).DisplayName;
            _deleteMark = false;
        }
    }

    protected class DeputySessionData
    {
        private List<DisplayDeputyItem> _deputyItems = new List<DisplayDeputyItem>();
	    
        public List<DisplayDeputyItem> DeputyItems
	    {
		    get { return _deputyItems;}
		    set { _deputyItems = value;}
	    }

        private UserProperties _userProperties;

        public UserProperties UserProperties
        {
            get { return _userProperties; }
            set { _userProperties = value; }
        }

        private ResourceManager _resourceManager;

        public ResourceManager ResourceManager
        {
            get { return _resourceManager; }
            set { _resourceManager = value; }
        }
    }

    protected DeputySessionData _sessionData = new DeputySessionData();

    protected void Page_Load(object sender, EventArgs e)
    {
        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];

        if (!IsPostBack)
        {
            _rm = ResourceHelper.Current.GetResourceManagerForPage(_controlName);
            DeputyItem[] items = UserHelper.Current.GetUserDeputies(us.UserInfo.UserId, us.DataConnection);
            foreach (DeputyItem item in items)
            {
                _sessionData.DeputyItems.Add(new DisplayDeputyItem(item, us.DataConnection));
            }

            _sessionData.UserProperties = us.UserInfo.UserProperties;
            _sessionData.ResourceManager = _rm;

            FillAddDeputyList();
            

            uiWebFormId.Value = (new GuidGenerator()).Generate();
            Session[uiWebFormId.Value] = _sessionData;
        }
        else
        {
            _sessionData = (DeputySessionData)Session[uiWebFormId.Value];
            _rm = _sessionData.ResourceManager;
            BindToData();

            if (uiMessage.Visible)
            {
                uiMessage.Visible = false;
            }
        }

        uiDeputyGrid.DataSource = _sessionData.DeputyItems;
        
        this.DataBind();
    }

    private void FillAddDeputyList()
    {
        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];
        UserProperties[] users = UserHelper.Current.GetAllUserProperties(us.DataConnection);
        foreach (UserProperties user in users)
        {
            if (user.UserId != us.UserInfo.UserId)
            {
                uiAddDeputyList.Items.Add(new ListItem(user.DisplayName, user.UserId.ToString()));
            }
        }
    }

    protected void BindToData()
    {
        int i = 0;
        foreach (DataGridItem gridItem in uiDeputyGrid.Items)
        {
            _sessionData.DeputyItems[i].DeleteMark = ((CheckBox)gridItem.FindControl("uiDeleteMark")).Checked;
            _sessionData.DeputyItems[i].Notify = ((CheckBox)gridItem.FindControl("uiNotify")).Checked;
            i++;
        }

        _sessionData.UserProperties.AllowDeputyNotification = uiAllowDeputyNotification.Checked;
    }

    protected bool DeputyIsAlreadyAdded(int userId)
    {
        bool retVal = false;
        foreach (DisplayDeputyItem item in _sessionData.DeputyItems)
        {
            if (item.Deputy == userId)
            {
                retVal = true;
                break;
            }
        }
        return retVal;
    }

    protected void uiAddDeputyButton_Click(object sender, EventArgs e)
    {
        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];
        int addedUser = int.Parse(uiAddDeputyList.SelectedValue);

        if (!DeputyIsAlreadyAdded(addedUser))
        {
            bool notify = true;
            DeputyItem item = new DeputyItem(us.UserInfo.UserId, addedUser, notify);
            _sessionData.DeputyItems.Add(new DisplayDeputyItem(item, us.DataConnection));
            uiDeputyGrid.DataBind();
        }
        else
        {
            uiMessage.Text = _rm.GetString(ResKeys.DeputyIsAlreadyAddedMessage);
            uiMessage.Visible = true;
        }
    }

    protected void ApplyDeputyGrid()
    {
        for (int i = _sessionData.DeputyItems.Count - 1; i >= 0; i--)
        {
            DisplayDeputyItem item = _sessionData.DeputyItems[i];
            if (item.DeleteMark)
            {
                _sessionData.DeputyItems.Remove(item);
            }
        }
        uiDeputyGrid.DataBind();
    }

    protected void uiApply_Click(object sender, EventArgs e)
    {
        ApplyDeputyGrid();
    }

    protected void uiSave_Click(object sender, EventArgs e)
    {
        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];
        ApplyDeputyGrid();
        UserHelper.Current.SaveUserDeputies(
            us.UserInfo.UserId, 
            _sessionData.DeputyItems.ToArray(), 
            us.DataConnection
            );
        UserHelper.Current.SaveUserProperties(_sessionData.UserProperties, us.DataConnection);
    }
}
