//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Resources;
using Evpo.DocCirc;
using Evpo.DocCirc.UI;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.Biz;

public partial class DocCardDirectory : System.Web.UI.UserControl
{
    #region Resources
    private const string _controlName = "DocCardDirectory.ascx";
    public struct ResKeys
    {
        public const string CreateDocCard = "CreateDocCard";
    }
    protected ResourceManager _rm;
    #endregion

    protected DocCardDirectoriesSessionData _sessionData;

    protected class DocCardDirectoriesSessionData
    {
        public DcFolder DocCardRootFolder;
    }

    protected class DocCardFolderTreeNode : TreeNode
    {
        private DcFolder _DocCardFolder;
        public DcFolder DocCardFolder
        {
            get { return _DocCardFolder; }
        }

        public DocCardFolderTreeNode(string text, DcFolder DocCardFolder, int orderNum)
            : base(text, orderNum.ToString())
        {
            _DocCardFolder = DocCardFolder;
            for (int i = 0; i < _DocCardFolder.Folders.Length; i++)
            {
                DcFolder fld = _DocCardFolder.Folders[i];
                this.ChildNodes.Add(new DocCardFolderTreeNode(fld.Name, fld, i));
            }
        }
    }


    private const string _createNewDocCardUrl = "~/DocumentCard.aspx?cmd=new&folder={0}";

    protected void Page_Init(object sender, EventArgs e)
    {
        _rm = ResourceHelper.Current.GetResourceManagerForPage(_controlName);
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        if (!this.IsPostBack)
        {
            _sessionData = new DocCardDirectoriesSessionData();
            
            UserSession us = (UserSession)Session[SessionStateKeys.UserSession];
            
            FolderData fd = new FolderData(us.UserInfo, us.DataConnection);
            DcFolder rootFolder = fd.GetFolderHierarchy(DBConstKeeper.Current.FolderType.Document);
            _sessionData.DocCardRootFolder = rootFolder;
            string nameResKey = rootFolder.Name.Replace("^", "");
            DocCardFolderTreeNode node =
                new DocCardFolderTreeNode(ResourceHelper.StrForUI[nameResKey], rootFolder, 0);
            uiDirectoryTree.Nodes.Add(node);
            node.Select();
            uiDirectoryTree.CollapseAll();
            uiDirectoryTree.Nodes[0].Expand();

            ((DocCardList)uiDocCardList).SetFilterItems(
                new DocCardFilterItem[] { new DocCardFilterItem(DocCardFieldName.Folder, rootFolder.Id) }
                );
            

            ApplyFolderSecurity(rootFolder.Id);

            uiCreateDocCard.NavigateUrl = String.Format(_createNewDocCardUrl, rootFolder.Id.ToString());

            Session[_controlName] = _sessionData;
        }
        else
        {
            _sessionData = (DocCardDirectoriesSessionData)Session[_controlName];
        }

        this.DataBind();
    }

    public override void DataBind()
    {
        foreach (Control cnt in Controls)
        {
            if (cnt.ID != "uiDocCardList")
            {
                cnt.DataBind();
            }
        }
        
    }

    private void ApplyFolderSecurity(int folderId)
    {
        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];

        SecurityData sd = new SecurityData(us.UserInfo, us.DataConnection);
        SecurityItem si = sd.GetFolderSecurity(folderId);

        if (!si.Create)
        {
            uiCreateDocCard.Enabled = false;
        }
        else
        {
            uiCreateDocCard.Enabled = true;
        }

        if (!si.Delete && !si.DeleteOwn)
        {
            ((DocCardList)uiDocCardList).SetDeletePermission(false);
        }
        else
        {
            ((DocCardList)uiDocCardList).SetDeletePermission(true);
        }
    }

    protected DcFolder GetFolderByPath(string idPath)
    {
        string[] pathIds = idPath.Split(new char[] { '/' });
        DcFolder folder = _sessionData.DocCardRootFolder;
        for (int i = 1; i < pathIds.Length; i++)
        {
            folder = folder.Folders[int.Parse(pathIds[i])];
        }
        return folder;
    }

    protected void uiDirectoryTree_SelectedNodeChanged(object sender, EventArgs e)
    {
        UserSession us = (UserSession)this.Session[SessionStateKeys.UserSession];
        DocTypeData dtd = new DocTypeData(us.UserInfo, us.DataConnection);

        DcFolder folder = GetFolderByPath(uiDirectoryTree.SelectedNode.ValuePath);
        ((DocCardList)uiDocCardList).SetFilterItems(
        new DocCardFilterItem[] { new DocCardFilterItem(DocCardFieldName.Folder, folder.Id) }
        );
            
        ((DocCardList)uiDocCardList).RefreshTable();
        uiCreateDocCard.NavigateUrl = String.Format(_createNewDocCardUrl, folder.Id.ToString());
        ApplyFolderSecurity(folder.Id);
    }
}
