//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Text;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Collections.Generic;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Xml.Serialization;
using System.Resources;
using Evpo.DocCirc;
using Evpo.DocCirc.UI;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.Biz;

public partial class DocCardList : System.Web.UI.UserControl, IParameterReceiver
{
    #region Resources
    private const string _controlName = "DocCardList.ascx";
    
    public struct ResKeys
    {
        public const string SortedAscending = "SortedAscending";
        public const string SortedDescending = "SortedDescending";
        public const string DocCardsTotalCount = "DocCardsTotalCount";
        public const string DeleteCheckedDocCards = "DeleteCheckedDocCards";
        public const string Previous = "Previous";
        public const string Next = "Next";
        public const string DocStatusToolTip = "DocStatusToolTip";
        public const string DocCardColumnName = "DocCardColumnName";
        public const string InternalNumColumnName = "InternalNumColumnName";
        public const string SubjectColumnName = "SubjectColumnName";
        public const string RegisteringUserColumnName = "RegisteringUserColumnName";
        public const string ConsideringEmployeesColumnName = "ConsideringEmployeesColumnName";
        public const string LastMovementColumnName = "LastMovementColumnName";
        public const string DaysAfterMovementColumnName = "DaysAfterMovementColumnName";
        public const string AllDocStatuses = "AllDocStatuses";
        public const string RefreshButton = "RefreshButton";

        
        

    }
    protected ResourceManager _rm;
    #endregion

    protected const string _parameterName = "QueryType";

    protected struct ParameterValues
    {
        public const string DocumentsConsideredByCurrentUser = "DocumentsConsideredByCurrentUser";
        public const string CurrentUserDocumentsUnderCons = "CurrentUserDocumentsUnderCons";
        public const string AllDocumentsUnderCons = "AllDocumentsUnderCons";
        public const string CurrentUserDrafts = "CurrentUserDrafts";
    }

    public struct ViewStateKeys
    {
        public const string Sort = "Sort";
    }

    protected struct ExtendedFieldName
    {
        public const string ConsideringEmployeeNames = "ConsideringEmployeeNames";
        public const string RegisteringUserName = "RegisteringUserName";
        public const string DaysAfterMovement = "DaysAfterMovement";
        public const string Url = "Url";
        public const string Checked = "Checked";
    }

    protected class DocCardListSessionData
    {
        public DataTable DocCardTable;
        public DataView CurDataView;
        public Evpo.DocCirc.UI.MenuItemParameter[] Parameters = new Evpo.DocCirc.UI.MenuItemParameter[0];
        public DocCardFilterItem[] FilterItems = new DocCardFilterItem[0];
        public int[] ConsideringEmployees = new int[0];
    }

    protected DocCardListSessionData _sessionData = new DocCardListSessionData();

    protected string[] _fields;

    protected SortedList _orderPictures;

    private const string _openDocCardUrl = "DocumentCard.aspx?cmd=open&id={0}";

    protected void Page_Init(object sender, EventArgs e)
    {
        _rm = ResourceHelper.Current.GetResourceManagerForPage(_controlName);

        _fields = new string[]
        {
            DocCardFieldName.Id,
            DocCardFieldName.InternalNumber,
            DocCardFieldName.Subject,
            DocCardFieldName.RegisteringUser,
            DocCardFieldName.LastMovement,
            DocCardFieldName.DocStatus
        };
    
        _orderPictures = new SortedList();
        _orderPictures[DocCardFieldName.InternalNumber] = "";
        _orderPictures[DocCardFieldName.Subject] = "";
        _orderPictures[DocCardFieldName.RegisteringUser] = "";
        _orderPictures[DocCardFieldName.LastMovement] = "";
        _orderPictures["ConsideringPerson"] = "";
        _orderPictures["DaysAfterMovement"] = "";

        if (!IsPostBack)
        {
            uiStatus.Items.Add(new ListItem(_rm.GetString(ResKeys.AllDocStatuses), "all"));

            uiStatus.Items.Add(
                new ListItem(
                    ResourceHelper.StrForUI[UIResKeys.DraftDocStatus],
                    DBConstKeeper.Current.DocStatus.Draft.ToString()
                    )
                );

            uiStatus.Items.Add(
                new ListItem(
                    ResourceHelper.StrForUI[UIResKeys.UnderConsDocStatus],
                    DBConstKeeper.Current.DocStatus.UnderCons.ToString()
                    )
                );

            uiStatus.Items.Add(
                new ListItem(
                    ResourceHelper.StrForUI[UIResKeys.FinalizedDocStatus],
                    DBConstKeeper.Current.DocStatus.Finalized.ToString()
                    )
                );
        }
    }

    protected void CreateFilterItems(MenuItemParameter[] parameters)
    {
        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];

        if (parameters[0].Name != _parameterName)
        {
            throw new ApplicationException(
                String.Format("Unknown parameter name: {0}!", parameters[0].Name)
                );
        }
        switch (parameters[0].Value)
        {
            case ParameterValues.AllDocumentsUnderCons:
                _sessionData.FilterItems = new DocCardFilterItem[]
                        {
                            new DocCardFilterItem(
                                DocCardFieldName.DocStatus,
                                DBConstKeeper.Current.DocStatus.UnderCons
                                )
                        };
                break;
            case ParameterValues.CurrentUserDocumentsUnderCons:
                _sessionData.FilterItems = new DocCardFilterItem[]
                        {
                            new DocCardFilterItem(
                                DocCardFieldName.DocStatus,
                                DBConstKeeper.Current.DocStatus.UnderCons
                                ),
                            new DocCardFilterItem(
                                DocCardFieldName.RegisteringUser,
                                us.UserToken.UserId
                                )
                        };
                break;
            case ParameterValues.DocumentsConsideredByCurrentUser:
                _sessionData.FilterItems = new DocCardFilterItem[]
                        {
                            new DocCardFilterItem(
                                DocCardFieldName.DocStatus,
                                DBConstKeeper.Current.DocStatus.UnderCons
                                )
                        };
                _sessionData.ConsideringEmployees = new int[]
                        {
                            us.UserToken.UserId
                        };
                break;
            case ParameterValues.CurrentUserDrafts:
                _sessionData.FilterItems = new DocCardFilterItem[]
                        {
                            new DocCardFilterItem(
                                DocCardFieldName.DocStatus,
                                DBConstKeeper.Current.DocStatus.Draft
                                ),
                            new DocCardFilterItem(
                                DocCardFieldName.RegisteringUser,
                                us.UserInfo.UserId
                                )
                        };
                break;
            default:
                throw new ApplicationException(
                    String.Format("Unknown parameter value: {0}!", parameters[0].Value)
                    );

        }
    }

    #region IParameterReceiver Members

    public void SetParameters(Evpo.DocCirc.UI.MenuItemParameter[] parameters)
    {
        _sessionData.Parameters = parameters;
    }
    #endregion

    public void SetFilterItems(DocCardFilterItem[] filterItems)
    {
        _sessionData.FilterItems = filterItems;
    }

    public void SetConsideringUsers(int[] users)
    {
        _sessionData.ConsideringEmployees = users;
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];

        if (!IsPostBack)
        {

            if (_sessionData.Parameters.Length > 0)
            {
                CreateFilterItems(_sessionData.Parameters);
            }

            BindDocStatus(BindDirection.ToControls);

            RefreshTable();

            Session[_controlName] = _sessionData;

            
        }
        else
        {

            _sessionData = (DocCardListSessionData)Session[_controlName];
            DataView dv = _sessionData.CurDataView;
            if (dv.Count > 0)
            {
                dv.AllowEdit = true;

                for (int i = 0; i < uiMainGrid.Items.Count; i++)
                {
                    DataGridItem item = uiMainGrid.Items[i];
                    CheckBox box = item.Cells[0].FindControl("uiDeleteMark") as CheckBox;
                    int dvIndex = uiMainGrid.CurrentPageIndex * uiMainGrid.PageSize + i;
                    if ((bool)dv[dvIndex]["Checked"] != box.Checked)
                    {
                        dv[dvIndex].BeginEdit();
                        dv[dvIndex]["Checked"] = box.Checked;
                        dv[dvIndex].EndEdit();
                    }
                }
            }
        }

        if (_sessionData.DocCardTable.Rows.Count == 0)
        {
            uiMainGrid.Visible = false;
        }
        else
        {
            uiMainGrid.Visible = true;
        }

        this.DataBind();
    }

    public override void DataBind()
    {
        foreach (Control cnt in Controls)
        {
            if (cnt.ID != "uiMainGrid")
            {
                cnt.DataBind();
            }
        }
    }

    protected void ExtendTable(DataTable table)
    {
        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];

        table.Columns.Add(ExtendedFieldName.ConsideringEmployeeNames, typeof(string));
        table.Columns.Add(ExtendedFieldName.RegisteringUserName, typeof(string));
        table.Columns.Add(ExtendedFieldName.DaysAfterMovement, typeof(int));
        table.Columns.Add(ExtendedFieldName.Url, typeof(string));
        table.Columns.Add(ExtendedFieldName.Checked, typeof(bool));

        UserProperties[] userProps = UserHelper.Current.GetAllUserProperties(us.DataConnection);
        SortedList<int, UserProperties> userPropList = new SortedList<int, UserProperties>(userProps.Length);
        foreach (UserProperties prop in userProps)
        {
            userPropList.Add(prop.UserId.Value, prop);
        }

        DocCardData dcd = new DocCardData(us.UserInfo, us.DataConnection, us.SettingKeeper);
        DocCardConsiderationInfo[] consInfoItems = dcd.GetDocumentConsiderationInfo(_sessionData.FilterItems);
        SortedList<long, DocCardConsiderationInfo> consInfoList = 
            new SortedList<long, DocCardConsiderationInfo>(consInfoItems.Length);
        foreach (DocCardConsiderationInfo consInfoItem in consInfoItems)
        {
            consInfoList.Add(consInfoItem.DocCardId, consInfoItem);
        }

        foreach (DataRow row in table.Rows)
        {
            row[ExtendedFieldName.Checked] = false;

            row[ExtendedFieldName.RegisteringUserName] = userPropList[(int)row[DocCardFieldName.RegisteringUser]].DisplayName;
            
            long docCardId = (long)row[DocCardFieldName.Id];

            DocCardConsiderationInfo consInfo = null;
            if (consInfoList.ContainsKey(docCardId))
            {
                consInfo = consInfoList[docCardId];
            }

            int[] employees = new int[0];

            if (consInfo != null)
            {
                employees = consInfo.Employees;
            }

            StringBuilder sb = new StringBuilder();
            foreach (int employee in employees)
            {
                if (sb.Length > 0)
                {
                    sb.Append(", ");
                }
                sb.Append(userPropList[employee].DisplayName);
            }

            row[ExtendedFieldName.ConsideringEmployeeNames] = sb.ToString();

            if (row[DocCardFieldName.LastMovement] != DBNull.Value)
            {
                TimeSpan span = DateTime.Now - ((DateTime)row[DocCardFieldName.LastMovement]);
                row[ExtendedFieldName.DaysAfterMovement] = span.Days;
            }

            row[ExtendedFieldName.Url] = String.Format(_openDocCardUrl, docCardId);
        }

    }

    public void SetDeletePermission(bool enable)
    {
        uiDeleteItems.Enabled = enable;
    }

    public void RefreshTable()
    {
        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];
        DocCardData dcd = new DocCardData(us.UserInfo, us.DataConnection, us.SettingKeeper);
        _sessionData.DocCardTable = dcd.GetDocumentTable(_fields, _sessionData.FilterItems, _sessionData.ConsideringEmployees);
        ExtendTable(_sessionData.DocCardTable);
        DataView dv = new DataView(_sessionData.DocCardTable);
        _sessionData.CurDataView = dv;

        if (dv.Count > 0)
        {
            string sortCol = DocCardFieldName.InternalNumber;
            string order = "DESC";
            dv.Sort = sortCol + " " + order;

            this.ViewState[ViewStateKeys.Sort] = dv.Sort;
            UpdateOrderPics();
            uiMainGrid.DataSource = dv;
            uiMainGrid.CurrentPageIndex = 0;
            uiMainGrid.DataBind();
            uiDeleteItems.Visible = true;
            uiMainGrid.Visible = true;
        }
        else
        {
            uiDeleteItems.Visible = false;
            uiMainGrid.Visible = false;
        }

        UpdatePager();

    }

    protected void UpdateOrderPics()
    {

        string sort = (string)this.ViewState[ViewStateKeys.Sort];
        string column = sort.Substring(0, sort.IndexOf(" "));
        string order = sort.Substring(sort.IndexOf(" ") + 1);

        _orderPictures = new SortedList();
        foreach (DataColumn dataCol in _sessionData.DocCardTable.Columns)
        {
            _orderPictures[dataCol.ColumnName] = "";
            if (dataCol.ColumnName == column)
            {
                if (order == "ASC")
                {
                    _orderPictures[dataCol.ColumnName] =
                        String.Format(
                            "<img src='g/i-arrdn.gif' width='24' height='16' alt='{0}' border='0' align='absmiddle'>",
                            _rm.GetString(ResKeys.SortedAscending)
                        );
                }
                else
                {
                    _orderPictures[dataCol.ColumnName] = String.Format(
                        "<img src='g/i-arrup.gif' width='24' height='16' alt='{0}' border='0' align='absmiddle'>",
                        _rm.GetString(ResKeys.SortedDescending)
                        );
                }
            }
        }
    }

    public string GetImageForStatus(int status)
    {
        string image = "";
        string title = "";
        if (status == DBConstKeeper.Current.DocStatus.UnderCons)
        {
            image = "g/undercons.gif";
        }
        else if (status == DBConstKeeper.Current.DocStatus.Finalized)
        {
            image = "g/finalized.gif";
        }
        else if (status == DBConstKeeper.Current.DocStatus.Draft)
        {
            image = "g/draft.gif";
        }

        string retVal = "<img src='" + image + "'" +
        " border='0' align='absmiddle' title='" + status + "' />";
        return retVal;
    }

    protected void uiMainGridHdr_Command(object sender, CommandEventArgs e)
    {
        string cmd = e.CommandName;

        string sort = this.ViewState[ViewStateKeys.Sort] as string;
        string column = sort.Substring(0, sort.IndexOf(" "));
        string order = sort.Substring(sort.IndexOf(" ") + 1);
        string newOrder = "";
        if (column == cmd)
        {
            if (order == "ASC")
            {
                newOrder = "DESC";
                _sessionData.CurDataView.Sort = cmd + " " + newOrder;
            }
            else
            {
                newOrder = "ASC";
                _sessionData.CurDataView.Sort = cmd + " " + newOrder;
            }
        }
        else
        {
            newOrder = "ASC";
            _sessionData.CurDataView.Sort = e.CommandName + " " + newOrder;
        }

        this.uiMainGrid.DataSource = _sessionData.CurDataView;
        this.ViewState[ViewStateKeys.Sort] = _sessionData.CurDataView.Sort;
        UpdateOrderPics();
        this.uiMainGrid.DataBind();
        
    }

    protected void UpdatePager()
    {

        int rowCount = _sessionData.CurDataView.Count;

        int pageSize = uiMainGrid.PageSize;
        int curPage = uiMainGrid.CurrentPageIndex + 1;

        int firstDoc = (curPage - 1) * pageSize + 1;

        int lastDoc = 0;
        if (rowCount > pageSize)
        {
            lastDoc = firstDoc + pageSize - 1;
            if (lastDoc > rowCount)
            {
                lastDoc = rowCount;
            }
        }
        else
        {
            lastDoc = rowCount;
        }


        uiPrevPage.Visible = false;
        uiNextPage.Visible = false;
        uiDocSpan.Visible = false;

        if (rowCount > 0)
        {
            uiDocSpan.Visible = true;
            uiDocSpan.Text = firstDoc.ToString() + "..." + lastDoc.ToString();
            if (firstDoc > 1)
            {
                uiPrevPage.Visible = true;
            }

            if (lastDoc < rowCount)
            {
                uiNextPage.Visible = true;
            }
        }

        uiDocQuantity.Text = _sessionData.CurDataView.Count.ToString();
    }

    protected void uiPrevPage_Click(object sender, EventArgs e)
    {
        uiMainGrid.CurrentPageIndex--;
        _sessionData.CurDataView.Sort = this.ViewState[ViewStateKeys.Sort].ToString();
        uiMainGrid.DataSource = _sessionData.CurDataView;
        UpdateOrderPics();
        uiMainGrid.DataBind();
        UpdatePager();
    }

    protected void uiNextPage_Click(object sender, EventArgs e)
    {
        uiMainGrid.CurrentPageIndex++;
        _sessionData.CurDataView.Sort = this.ViewState[ViewStateKeys.Sort].ToString();
        uiMainGrid.DataSource = _sessionData.CurDataView;
        UpdateOrderPics();
        uiMainGrid.DataBind();
        UpdatePager();
    }

    protected void uiDeleteItems_Click(object sender, EventArgs e)
    {
        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];
        DocCardData dcd = new DocCardData(us.UserInfo, us.DataConnection, us.SettingKeeper);


        DataView dv = _sessionData.CurDataView;
        for (int i = dv.Count - 1; i >= 0; i--)
        {
            if ((bool)dv[i][ExtendedFieldName.Checked])
            {
                dcd.DeleteDocCard((long)dv[i][DocCardFieldName.Id]);
                dv[i].Delete();
            }
        }

        if (dv.Count > 0)
        {
            int pageCount = dv.Count / uiMainGrid.PageSize;
            if (dv.Count % uiMainGrid.PageSize > 0)
            {
                pageCount++;
            }

            if (uiMainGrid.CurrentPageIndex > pageCount - 1)
            {
                uiMainGrid.CurrentPageIndex = pageCount - 1;
            }

            uiMainGrid.DataSource = dv;
            UpdateOrderPics();
            uiMainGrid.DataBind();
            UpdatePager();
        }
        else
        {
            uiDeleteItems.Visible = false;
            uiMainGrid.Visible = false;
            UpdatePager();
        }
    }

    protected string MakeDateString(DateTime dateTime)
    {
        string retVal = "";
        if(dateTime != null)
        {
            retVal = dateTime.ToString();
        }
        return retVal;
    }

    protected enum BindDirection
    {
        ToControls,
        ToData
    }

    protected void BindDocStatus(BindDirection direction)
    {
        if (direction == BindDirection.ToControls)
        {
            DocCardFilterItem foundItem = null;
            foreach (DocCardFilterItem filterItem in _sessionData.FilterItems)
            {
                if (filterItem.FieldName == DocCardFieldName.DocStatus)
                {
                    foundItem = filterItem;
                    break;
                }
            }

            if (foundItem != null)
            {
                uiStatus.SelectedValue = ((int)foundItem.FieldValue).ToString();
            }
            else
            {
                uiStatus.SelectedValue = "all";
            }
        }
        else // direction == ToData
        {
            DocCardFilterItem foundItem = null;
            int i = 0;
            foreach (DocCardFilterItem filterItem in _sessionData.FilterItems)
            {
                if( filterItem.FieldName == DocCardFieldName.DocStatus)
                {
                    foundItem = filterItem;
                    break;
                }
                i++;
            }

            if (uiStatus.SelectedValue == "all" && foundItem != null)
            {
                DocCardFilterItem[] newArr = new DocCardFilterItem[_sessionData.FilterItems.Length - 1];
                Array.Copy(_sessionData.FilterItems, 0, newArr, 0, i);
                Array.Copy(_sessionData.FilterItems, i + 1, newArr, i, _sessionData.FilterItems.Length - i - 1);
                _sessionData.FilterItems = newArr;
            }
            else if (uiStatus.SelectedValue != "all" && foundItem != null)
            {
                foundItem.FieldValue = int.Parse(uiStatus.SelectedValue);
            }
            else if (uiStatus.SelectedValue != "all")
            {
                DocCardFilterItem[] newArr = new DocCardFilterItem[_sessionData.FilterItems.Length + 1];
                _sessionData.FilterItems.CopyTo(newArr, 0);
                newArr[newArr.Length - 1] = new DocCardFilterItem(
                    DocCardFieldName.DocStatus,
                    int.Parse(uiStatus.SelectedValue)
                    );
                _sessionData.FilterItems = newArr;
            }
            else
            {
            }

        }
    }

    protected void uiStatus_SelectedIndexChanged(object sender, EventArgs e)
    {
        BindDocStatus(BindDirection.ToData);
        RefreshTable();
    }
    protected void uiRefresh_Click(object sender, EventArgs e)
    {
        RefreshTable();
    }
}
