//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Resources;
using Evpo.DocCirc;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.Biz;
using Evpo.DocCirc.UI;

public partial class DocTypeDirectories : System.Web.UI.UserControl
{
    #region Resources
    private const string _controlName = "DocTypeDirectories.ascx";
    public struct ResKeys
    {
        public const string CreateDocType = "CreateDocType";
        public const string RefreshDocTypeList = "RefreshDocTypeList";
    }
    protected ResourceManager _rm;
    #endregion


    protected DocTypeDirectoriesSessionData _sessionData;

    public class DocTypeDirectoriesSessionData
    {
        public DcFolder DocTypeRootFolder;
    }


    private const string _createNewDocTypeURL = "~/DocumentType.aspx?cmd=new&folder={0}";



    //public class TreeNodeForDir : TreeNode, INodeOperator
    //{
    //    public TreeNodeForDir(string text, string value)
    //        : base(text, value)
    //    {
    //    }

    //    #region INodeOperator Members

    //    public INodeOperator AddNode(string name, string data)
    //    {
    //        TreeNodeForDir node = new TreeNodeForDir(name, data);
    //        this.ChildNodes.Add(node);
    //        return node;
    //    }

    //    #endregion
    //}

    class DocTypeFolderTreeNode : TreeNode
    {
        private DcFolder _docTypeFolder;
        public DcFolder DocTypeFolder
        {
            get { return _docTypeFolder; }
        }

        public DocTypeFolderTreeNode(string text, DcFolder docTypeFolder, int orderNum)
            : base(text, orderNum.ToString())
        {
            _docTypeFolder = docTypeFolder;
            for (int i = 0; i < _docTypeFolder.Folders.Length; i++)
            {
                DcFolder fld = _docTypeFolder.Folders[i];
                this.ChildNodes.Add(new DocTypeFolderTreeNode(fld.Name, fld, i));
            }
        }
    }

    protected void Page_Init(object sender, EventArgs e)
    {
        _rm = ResourceHelper.Current.GetResourceManagerForPage(_controlName);
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        if (!this.IsPostBack)
        {
            _sessionData = new DocTypeDirectoriesSessionData();
            
            UserSession us = (UserSession)Session[SessionStateKeys.UserSession];
            
            FolderData fd = new FolderData(us.UserInfo, us.DataConnection);
            DcFolder rootFolder = fd.GetFolderHierarchy(DBConstKeeper.Current.FolderType.DocType);
            _sessionData.DocTypeRootFolder = rootFolder;
            string nameResKey = rootFolder.Name.Replace("^", "");
            DocTypeFolderTreeNode node =
                new DocTypeFolderTreeNode(ResourceHelper.StrForUI[nameResKey], rootFolder, 0);
            uiDirectoryTree.Nodes.Add(node);
            node.Select();
            uiDirectoryTree.CollapseAll();
            uiDirectoryTree.Nodes[0].Expand();

            DocTypeData dtd = new DocTypeData(us.UserInfo, us.DataConnection);
            ((DocTypeList)uiDocTypeList).SetDocTypeTable(dtd.GetDocTypes(rootFolder.Id));
            ApplyFolderSecurity(rootFolder.Id);
            uiCreateDocType.NavigateUrl = String.Format(_createNewDocTypeURL, rootFolder.Id.ToString());

            Session[_controlName] = _sessionData;
            //try
            //{
            //    string docTypePath = this.Application["docTypePath"].ToString();
            //    uiCreateDocType.NavigateUrl = _createNewDocTypeURL + Server.UrlEncode(docTypePath);

            //    ConsideredDocs consDocs = new ConsideredDocs(
            //        this.Session["sessionId"].ToString(),
            //        this.Session["userName"].ToString(),
            //        this.Application,
            //        this.Page.User
            //        );

            //    string folderName = "";
            //    if (docTypePath.Contains("/"))
            //    {
            //        folderName = docTypePath.Substring(docTypePath.LastIndexOf("/") + 1);
            //    }

            //    TreeNodeForDir node = new TreeNodeForDir(folderName, docTypePath);
            //    uiDirectoryTree.Nodes.Add(node);
            //    node.Select();
            //    consDocs.GetDirectoryHierarchy(
            //        docTypePath,
            //        node
            //        );
            //    consDocs.DocumentStatus = "";
            //    (uiDocTypeList as DocTypeList).DocTypeTable = consDocs.FillDocTypeTable(docTypePath);
            //    consDocs.CloseConnection();
            //    uiDirectoryTree.CollapseAll();
            //    uiDirectoryTree.Nodes[0].Expand();
            //}
            //catch (Exception ex)
            //{
            //    LogHelper.WriteException(ex);
            //}
        }
        else
        {
            _sessionData = (DocTypeDirectoriesSessionData)Session[_controlName];
        }

        this.DataBind();
    }

    public override void DataBind()
    {
        foreach (Control cnt in Controls)
        {
            if (cnt.ID != "uiDocTypeList")
            {
                cnt.DataBind();
            }
        }
    }

    private void ApplyFolderSecurity(int folderId)
    {
        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];

        SecurityData sd = new SecurityData(us.UserInfo, us.DataConnection);
        SecurityItem si = sd.GetFolderSecurity(folderId);

        if (!si.Create)
        {
            uiCreateDocType.Enabled = false;
        }
        else
        {
            uiCreateDocType.Enabled = true;
        }

        if (!si.Delete && !si.DeleteOwn)
        {
            ((DocTypeList)uiDocTypeList).SetDeletePermission(false);
        }
        else
        {
            ((DocTypeList)uiDocTypeList).SetDeletePermission(true);
        }
    }

    protected DcFolder GetFolderByPath(string idPath)
    {
        string[] pathIds = idPath.Split(new char[] { '/' });
        DcFolder folder = _sessionData.DocTypeRootFolder;
        for (int i = 1; i < pathIds.Length; i++)
        {
            folder = folder.Folders[int.Parse(pathIds[i])];
        }
        return folder;
    }

    protected void uiDirectoryTree_SelectedNodeChanged(object sender, EventArgs e)
    {
        UserSession us = (UserSession)this.Session[SessionStateKeys.UserSession];
        DocTypeData dtd = new DocTypeData(us.UserInfo, us.DataConnection);

        DcFolder folder = GetFolderByPath(uiDirectoryTree.SelectedNode.ValuePath);
        ((DocTypeList)uiDocTypeList).SetDocTypeTable(dtd.GetDocTypes(folder.Id));
        ((DocTypeList)uiDocTypeList).ResetTable();
        uiCreateDocType.NavigateUrl = String.Format(_createNewDocTypeURL, folder.Id.ToString());
        ApplyFolderSecurity(folder.Id);
        //try
        //{
        //    ConsideredDocs consDocs = new ConsideredDocs(
        //        this.Session["sessionId"].ToString(),
        //        this.Session["userName"].ToString(),
        //        this.Application,
        //        this.Page.User
        //        );
        //    (uiDocTypeList as DocTypeList).DocTypeTable = consDocs.FillDocTypeTable(uiDirectoryTree.SelectedNode.Value);
        //    (uiDocTypeList as DocTypeList).ResetTable();

        //    consDocs.CloseConnection();
        //    uiCreateDocType.NavigateUrl =
        //        _createNewDocTypeURL + Server.UrlEncode(uiDirectoryTree.SelectedNode.Value);
        //}
        //catch (Exception ex)
        //{
        //    LogHelper.WriteException(ex);
        //}
    }

    protected void uiRefresh_Click(object sender, EventArgs e)
    {
        UserSession us = (UserSession)this.Session[SessionStateKeys.UserSession];
        DocTypeData dtd = new DocTypeData(us.UserInfo, us.DataConnection);

        DcFolder folder = GetFolderByPath(uiDirectoryTree.SelectedNode.ValuePath);
        ((DocTypeList)uiDocTypeList).SetDocTypeTable(dtd.GetDocTypes(folder.Id));
        ((DocTypeList)uiDocTypeList).ResetTable();

        //try
        //{
        //    ConsideredDocs consDocs = new ConsideredDocs(
        //        this.Session["sessionId"].ToString(),
        //        this.Session["userName"].ToString(),
        //        this.Application,
        //        this.Page.User
        //        );
        //    (uiDocTypeList as DocTypeList).DocTypeTable = consDocs.FillDocTypeTable(uiDirectoryTree.SelectedNode.Value);
        //    (uiDocTypeList as DocTypeList).ResetTable();

        //    consDocs.CloseConnection();
        //}
        //catch (Exception ex)
        //{
        //    LogHelper.WriteException(ex);
        //}
    }


}
