//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Text;
using System.Resources;
using Evpo.DocCirc;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.Biz;
using Evpo.DocCirc.UI;

public partial class DocTypeList : System.Web.UI.UserControl
{
    #region Resources
    private const string _controlName = "DocTypeList.ascx";
    public struct ResKeys
    {
        public const string SortedAscending = "SortedAscending";
        public const string SortedDescending = "SortedDescending";
        public const string DocTypesTotalCount = "DocTypesTotalCount";
        public const string DeleteCheckedDocTypes = "DeleteCheckedDocTypes";
        public const string Previous = "Previous";
        public const string Next = "Next";
        public const string DocTypeColumnName = "DocTypeColumnName";
        public const string PrefixColumnName = "PrefixColumnName";

    }
    protected ResourceManager _rm;
    #endregion

    public struct ViewStateKeys
    {
        public const string Sort = "Sort";
    }

    protected class DocTypeListSessionData
    {
        public DataTable DocTypeTable;
        public DataView CurDataView;
    }

    protected DocTypeListSessionData _sessionData = new DocTypeListSessionData();

    private const string _openDocTypeURL = "DocumentType.aspx?cmd=open&id={0}";

    protected SortedList _orderPictures;

    protected void Page_Init(object sender, EventArgs e)
    {
        _rm = ResourceHelper.Current.GetResourceManagerForPage(_controlName);

        DataTable docTypeTable = new DataTable();
        _sessionData.DocTypeTable = docTypeTable;

        docTypeTable.Columns.Add("DocType", typeof(string));
        docTypeTable.Columns.Add("Href", typeof(string));
        docTypeTable.Columns.Add("Prefix", typeof(string));
        docTypeTable.Columns.Add("ItemId", typeof(int));
        docTypeTable.Columns.Add("Checked", typeof(bool));

        _orderPictures = new SortedList();
        _orderPictures["DocType"] = "";
        _orderPictures["Prefix"] = "";
    }

    public void SetDocTypeTable(DocTypeItem[] docTypes)
    {
        _sessionData.DocTypeTable.Clear();

        foreach (DocTypeItem item in docTypes)
        {
            DataRow row = _sessionData.DocTypeTable.NewRow();
            row["DocType"] = item.Name;
            row["Href"] = String.Format(_openDocTypeURL, item.Id.ToString());
            row["Prefix"] = item.Prefix;
            row["ItemId"] = item.Id;
            row["Checked"] = false;
            _sessionData.DocTypeTable.Rows.Add(row);
        }
    }

    public void SetDeletePermission(bool enable)
    {
        uiDeleteItems.Enabled = enable;
    }

    /// <summary>
    /// Columns: 
    /// DocType - name of a document type
    /// Href - URL of a document type card
    /// Prefix - prefix of a document type
    /// ItemID - item ID in the data store
    /// Checked - marked for an action
    /// </summary>
    //public DataTable DocTypeTable
    //{
    //    get { return _docTypeTable; }
    //    set {
    //        DataTable dt = (DataTable)value;
    //        object[][] cols = new object[][]
    //        {
    //            new object[]{"DocType", typeof(string)},
    //            new object[]{"Href", typeof(string)},
    //            new object[]{"Prefix", typeof(string)},
    //            new object[]{"ItemID", typeof(string)},
    //            new object[]{"Checked", typeof(bool)}
    //        };

    //        bool rightCols = true;
    //        foreach(object[] col in cols)
    //        {
    //            DataColumn dc = dt.Columns[col[0].ToString()];
    //            if (dc == null || dc.DataType != (Type)col[1])
    //            {
    //                rightCols = false;
    //                break;
    //            }
    //        }

    //        if (!rightCols)
    //        {
    //            throw new Exception("DocCirc: DocTypeList: set_DocTypeTable: Some table columns are wrong");
    //        }

    //        _docTypeTable = dt; 
    //    }
    //}

    public void ResetTable()
    {
        if (_sessionData.DocTypeTable == null)
        {
            throw new Exception("DocCirc: DocTypeList: ResetTable: _docTypeTable can not be null");
        }
        DataView dv = new DataView(_sessionData.DocTypeTable);
        _sessionData.CurDataView = dv;

        if (dv.Count > 0)
        {
            string sortCol = "DocType";
            string order = "ASC";
            dv.Sort = sortCol + " " + order;

            this.ViewState[ViewStateKeys.Sort] = dv.Sort;
            UpdateOrderPics();
            uiMainGrid.DataSource = dv;
            uiMainGrid.CurrentPageIndex = 0;
            uiMainGrid.DataBind();
            uiDeleteItems.Visible = true;
            uiMainGrid.Visible = true;
        }
        else
        {
            uiDeleteItems.Visible = false;
            uiMainGrid.Visible = false;
        }

        UpdatePager();

    }

    private void UpdatePager()
    {
        DataTable dataTable = _sessionData.DocTypeTable;
        DataView dv = _sessionData.CurDataView;

        int rowCount = dv.Count;

        int pageSize = uiMainGrid.PageSize;
        int curPage = uiMainGrid.CurrentPageIndex + 1;

        int firstDoc = (curPage - 1) * pageSize + 1;

        int lastDoc = 0;
        if (rowCount > pageSize)
        {
            lastDoc = firstDoc + pageSize - 1;
            if (lastDoc > rowCount)
            {
                lastDoc = rowCount;
            }
        }
        else
        {
            lastDoc = rowCount;
        }


        uiPrevPage.Visible = false;
        uiNextPage.Visible = false;
        uiDocTypeSpan.Visible = false;

        if (rowCount > 0)
        {
            uiDocTypeSpan.Visible = true;
            uiDocTypeSpan.Text = firstDoc.ToString() + "..." + lastDoc.ToString();
            if (firstDoc > 1)
            {
                uiPrevPage.Visible = true;
            }

            if (lastDoc < rowCount)
            {
                uiNextPage.Visible = true;
            }
        }

        uiDocTypeQuantity.Text = dv.Count.ToString();
    }


    protected void Page_Load(object sender, EventArgs e)
    {
        if (!this.IsPostBack)
        {
            this.Session[_controlName] = _sessionData;
            ResetTable();
            
        }
        else
        {
            _sessionData = (DocTypeListSessionData)Session[_controlName];
            DataView dv = _sessionData.CurDataView;
            if (dv.Count > 0)
            {
                dv.AllowEdit = true;

                for (int i = 0; i < uiMainGrid.Items.Count; i++)
                {
                    DataGridItem item = uiMainGrid.Items[i];
                    CheckBox box = item.Cells[0].FindControl("uiDeleteMark") as CheckBox;
                    int dvIndex = uiMainGrid.CurrentPageIndex * uiMainGrid.PageSize + i;
                    if ((bool)dv[dvIndex]["Checked"] != box.Checked)
                    {
                        dv[dvIndex].BeginEdit();
                        dv[dvIndex]["Checked"] = box.Checked;
                        dv[dvIndex].EndEdit();
                    }
                }
            }

        }

        if (_sessionData.DocTypeTable.Rows.Count == 0)
        {
            uiMainGrid.Visible = false;
        }
        else
        {
            uiMainGrid.Visible = true;
        }

        this.DataBind();
        
    }

    public override void DataBind()
    {
        foreach (Control cnt in Controls)
        {
            if (cnt.ID != "uiMainGrid")
            {
                cnt.DataBind();
            }
        }
    }

    protected void UpdateOrderPics()
    {
        string sort = (string)this.ViewState[ViewStateKeys.Sort];
        string column = sort.Substring(0, sort.IndexOf(" "));
        string order = sort.Substring(sort.IndexOf(" ") + 1);

        _orderPictures = new SortedList();
        foreach (DataColumn dataCol in _sessionData.DocTypeTable.Columns)
        {
            _orderPictures[dataCol.ColumnName] = "";
            if (dataCol.ColumnName == column)
            {
                if (order == "ASC")
                {
                    _orderPictures[dataCol.ColumnName] =
                        String.Format(
                            "<img src='g/i-arrdn.gif' width='24' height='16' alt='{0}' border='0' align='absmiddle'>",
                            _rm.GetString(ResKeys.SortedAscending)
                        );
                }
                else
                {
                    _orderPictures[dataCol.ColumnName] = String.Format(
                        "<img src='g/i-arrup.gif' width='24' height='16' alt='{0}' border='0' align='absmiddle'>",
                        _rm.GetString(ResKeys.SortedDescending)
                        );
                }
            }
        }
    }

    protected void uiDeleteItems_Click(object sender, EventArgs e)
    {
        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];
        DocTypeData dtd = new DocTypeData(us.UserInfo, us.DataConnection);
        

        DataView dv = _sessionData.CurDataView;
        for (int i = dv.Count - 1; i >= 0; i--)
        {
            if ((bool)dv[i]["Checked"])
            {
                dtd.DeleteDocType((int)dv[i]["ItemId"]);
                dv[i].Delete();
            }
        }

        if (dv.Count > 0)
        {
            int pageCount = dv.Count / uiMainGrid.PageSize;
            if (dv.Count % uiMainGrid.PageSize > 0)
            {
                pageCount++;
            }

            if (uiMainGrid.CurrentPageIndex > pageCount - 1)
            {
                uiMainGrid.CurrentPageIndex = pageCount - 1;
            }

            uiMainGrid.DataSource = dv;
            UpdateOrderPics();
            uiMainGrid.DataBind();
            UpdatePager();
        }
        else
        {
            uiDeleteItems.Visible = false;
            uiMainGrid.Visible = false;
            UpdatePager();
        }
      

        //bool unauthorizedAccess = false;
        //for (int i = dv.Count - 1; i >= 0; i--)
        //{
        //    if ((bool)dv[i]["Checked"])
        //    {

        //        try
        //        {
        //            consDocs.DeleteDocumentType(dv[i]["ItemID"].ToString());
        //            dv[i].Delete();
        //        }
        //        catch (UnauthorizedAccessException ex)
        //        {
        //            unauthorizedAccess = true;
        //        }
        //        catch (DocCircReferenceIntegrityException ex)
        //        {
        //            DCItems.DCItemsTblDataTable dt = (DCItems.DCItemsTblDataTable)ex.ReferencingItems;

        //            StringBuilder sb = new StringBuilder();
        //            sb.AppendFormat(
        //                "  <b>{0}</b>    ,      :<br/>",
        //                dv[i]["DocType"].ToString()
        //                );

        //            bool firstOne = true;
        //            int docNumsInRow = 8;
        //            int restInRow = docNumsInRow;
        //            foreach(DCItems.DCItemsTblRow row in dt.Rows)
        //            {
        //                sb.AppendFormat("<a href='{0}' target='_blank' style='color:red'>{1}</a>", row.Href, row.InsideNum);
        //                sb.Append(" ");

        //                restInRow--;
        //                if (restInRow == 0)
        //                {
        //                    restInRow = docNumsInRow;
        //                    sb.Append("<br />");
        //                }
        //            }
        //            sb.Append("<br/>");
        //            uiMessage.Text += sb.ToString();
        //        }

        //    }
        //}
        //consDocs.CloseConnection();

        //if (unauthorizedAccess)
        //{
        //    uiMessage.Text += "      !    .<br/>";
        //}

        //int pageCount = dv.Count / uiMainGrid.PageSize;
        //if (dv.Count % uiMainGrid.PageSize > 0)
        //{
        //    pageCount++;
        //}

        //if (uiMainGrid.CurrentPageIndex > pageCount - 1)
        //{
        //    uiMainGrid.CurrentPageIndex = pageCount - 1;
        //}

        //uiMainGrid.DataSource = dv;
        //UpdateOrderPics();
        //uiMainGrid.DataBind();
        //UpdatePager();
            
    }

    protected void uiPrevPage_Click(object sender, EventArgs e)
    {
        uiMainGrid.CurrentPageIndex--;
        DataView dv = _sessionData.CurDataView;
        dv.Sort = this.ViewState[ViewStateKeys.Sort].ToString();
        uiMainGrid.DataSource = dv;
        UpdateOrderPics();
        uiMainGrid.DataBind();
        UpdatePager();
    }

    protected void uiNextPage_Click(object sender, EventArgs e)
    {
        uiMainGrid.CurrentPageIndex++;
        DataView dv = _sessionData.CurDataView;
        dv.Sort = this.ViewState[ViewStateKeys.Sort].ToString();
        uiMainGrid.DataSource = dv;
        UpdateOrderPics();
        uiMainGrid.DataBind();
        UpdatePager();
    }

    protected void uiMainGridHdr_Command(object sender, CommandEventArgs e)
    {
        DataView dv = _sessionData.CurDataView;

        string cmd = e.CommandName;

        string sort = (string)this.ViewState[ViewStateKeys.Sort];
        string column = sort.Substring(0, sort.IndexOf(" "));
        string order = sort.Substring(sort.IndexOf(" ") + 1);
        string newOrder = "";
        if (column == cmd)
        {
            if (order == "ASC")
            {
                newOrder = "DESC";
                dv.Sort = cmd + " " + newOrder;
            }
            else
            {
                newOrder = "ASC";
                dv.Sort = cmd + " " + newOrder;
            }
        }
        else
        {
            newOrder = "ASC";
            dv.Sort = e.CommandName + " " + newOrder;
        }

        this.uiMainGrid.DataSource = dv;
        this.ViewState[ViewStateKeys.Sort] = dv.Sort;
        UpdateOrderPics();
        this.uiMainGrid.DataBind();
    }
}
