//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Resources;
using System.Collections.Generic;
using System.Text;
using Evpo.DocCirc.Biz;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.UI;
using Evpo.DocCirc;

public partial class DocumentCard : System.Web.UI.Page
{
    protected enum BindingDirection
    {
        BindToControls,
        BindToData
    }

    protected struct PageParameter
    {
        public const string Cmd = "cmd";
        public const string Id = "id";
        public const string Folder = "folder";
    }

    protected struct PageParameterCmd
    {
        public const string New = "new";
        public const string Open = "open";
    }

    #region Resources
    private const string _controlName = "DocumentCard.aspx";
    public struct ResKeys
    {
        public const string PageTitle = "PageTitle";
        public const string SaveAndCloseButton = "SaveAndCloseButton";
        public const string CancelButton = "CancelButton";

        public const string DocumentTab = "DocumentTab";
        public const string RouteTab = "RouteTab";
        public const string EventTab = "EventTab";
        public const string AttachmentTab = "AttachmentTab";

        public const string SubjectLabel = "SubjectLabel";
        public const string DocTypeLabel = "DocTypeLabel";
        public const string OriginalDateLabel = "OriginalDateLabel";
        public const string DocStatusLabel = "DocStatusLabel";
        public const string OriginalNumberLabel = "OriginalNumberLabel";
        public const string ConsideringUserLabel = "ConsideringUserLabel";
        public const string SendingCompanyLabel = "SendingCompanyLabel";
        public const string AnotherCompanyOption = "AnotherCompanyOption";
        public const string InternalNumberLabel = "InternalNumberLabel";
        public const string ReceivingCompanyLabel = "ReceivingCompanyLabel";
        public const string LastMovementLabel = "LastMovementLabel";
        public const string DepartmentLabel = "DepartmentLabel";
        public const string RegistrationDateLabel = "RegistrationDateLabel";
        public const string RegisteringUserNameLabel = "RegisteringUserNameLabel";
        public const string FinalizingDateLabel = "FinalizingDateLabel";
        public const string AttachmentsAreNotChangeableLabel = "AttachmentsAreNotChangeableLabel";
        public const string RouteLabel = "RouteLabel";
        public const string RoutePointNumLabel = "RoutePointNumLabel";
        public const string UserLabel = "UserLabel";
        public const string DecisionLabel = "DecisionLabel";
        public const string ReturnToRoutePoint = "ReturnToRoutePoint";
        public const string SignDecisionLabel = "SignDecisionLabel";
        public const string DecisionNoteLabel = "DecisionNoteLabel";
        public const string FileColumnHeaderLabel = "FileColumnHeaderLabel";
        public const string DeleteColumnHeaderLabel = "DeleteColumnHeaderLabel";
        public const string RouteItemListRow = "RouteItemListRow";

        public const string NoteLabel = "NoteLabel";
        public const string SetStateToUnderConsLabel = "SetStateToUnderConsLabel";
        public const string ReturnToRoutePointLabel = "ReturnToRoutePointLabel";


        public const string PassedResult = "PassedResult";
        public const string UnderConsResult = "UnderConsResult";

        public const string AddAttachments = "AddAttachments";
        public const string AttachmentInstruction = "AttachmentInstruction";

        public const string ReturnToDraft = "ReturnToDraft";

        public const string AdministratorDeputyLabel = "AdministratorDeputyLabel";
        public const string DeputyLabel = "DeputyLabel";

        public const string AnotherCompany = "AnotherCompany";
        public const string RefreshCompanyToolTip = "RefreshCompanyToolTip";

        public const string AddReferenceButton = "AddReferenceButton";
        public const string ReferenceTab = "ReferenceTab";
        public const string RegisteringUserInReferenceList = "RegisteringUserInReferenceList";
        public const string RemoveCheckBox = "RemoveCheckBox";

        public const string DocCardHasBeenAddedToReferenceListError = "DocCardHasBeenAddedToReferenceListError";
        public const string InaccessibleReferenceRow = "InaccessibleReferenceRow";
        public const string ReferenceRow = "ReferenceRow";

        public const string EmailReferenceToThisDocCard = "EmailReferenceToThisDocCard";
        public const string FolderNameLabel = "FolderNameLabel";
        public const string OriginalDateFormatError = "OriginalDateFormatError";
        public const string SubjectIsEmpty = "SubjectIsEmpty";


    }
    protected ResourceManager _rm;
    #endregion

    protected struct PageConstant
    {
        public const string AttachmentUrlForSavingDialog = "Attachment.aspx?file={0}";
        public const string AttachmentUrl = "Attachment.aspx/{0}";
        public const string DocCardUrl = "DocumentCard.aspx?cmd=open&id={0}";
        public const string ErrorMessageUrl = "ErrorMessage.aspx?ResourceKey={0}";
        public const string AccessDeniedErrorMessageKey = "AccessDenied";
        public const string DocumentNotFoundErrorMessageKey = "DocumentNotFound";
        public const string DocTypeListIsEmpty = "DocTypeListIsEmpty";
    }

    protected class DocumentCardSessionData
    {
        public DocCardItem DocCardItem;
        public AttachmentInterfaceItem[] AttachmentInterfaceItems;
        public List<InterfaceReferenceItem> InterfaceReferenceItems = new List<InterfaceReferenceItem>();
        public bool AttachmentIsReadOnly = true;
        public bool AttachmentReadOnlyCheckBoxIsReadOnly = false;
        public bool DocumentTabIsReadOnly = true;
        public bool RouteTabIsVisible = true;
        public bool RouteTabIsReadOnly = true;
        public bool CanBeSaved = false;
        public bool RemovePassRouteDecisionOption = false;
        public bool CanBeSeen = true;
        public int? UserWhoseDeputyCurrentUserIs = null;
        public bool ReferenceTabIsOpened = false;
        public string FolderName = "";
    }
    
    protected DocumentCardSessionData _sessionData = new DocumentCardSessionData();

    protected class InterfaceReferenceItem
    {
        private DocCardReferenceItem _referenceItem;

        public DocCardReferenceItem ReferenceItem
        {
            get { return _referenceItem; }
            set { _referenceItem = value; }
        }

        public long? Id
        {
            get { return _referenceItem.Id; }
        }

        private string _url;

        public string Url
        {
            get { return _url; }
            set { _url = value; }
        }

        private string _text;

        public string Text
        {
            get { return _text; }
            set { _text = value; }
        }

        private bool _inaccessible;

        public bool Inaccessible
        {
            get { return _inaccessible; }
            set { _inaccessible = value; }
        }


        public InterfaceReferenceItem(
            DocCardReferenceItem refItem,
            string text,
            string url,
            bool inaccesible
            )
        {
            _referenceItem = refItem;
            _text = text;
            _url = url;
            _inaccessible = inaccesible;
        }

    }

    protected UserSession _us;

    protected class AttachmentInterfaceItem
    {
        private long _id;
	    public long Id
	    {
		    get { return _id;}
		    set { _id = value;}
	    }

        private string _url;

        public string Url
        {
            get { return _url; }
            set { _url = value; }
        }

        private string _displayName;

        public string DisplayName
        {
            get { return _displayName; }
            set { _displayName = value; }
        }

        private bool _deleteMark;

        public bool DeleteMark
        {
            get { return _deleteMark; }
            set { _deleteMark = value; }
        }

        public AttachmentInterfaceItem(long id, string url, string displayName, bool deleteMark)
        {
            _id = id;
            _url = url;
            _displayName = displayName;
            _deleteMark = deleteMark;
        }
    }

    protected class TabHref : Control, IPostBackEventHandler
    {
        #region IPostBackEventHandler Members

        public void RaisePostBackEvent(string eventArgument)
        {
            OnClick(new EventArgs());
        }

        #endregion

        public event EventHandler Click;

        protected void OnClick(EventArgs e)
        {
            if (Click != null)
            {
                Click(this, e);
            }
        }

        private string _clientStatement = "";

        public string ClientStatement
        {
            get { return _clientStatement; }
            set { _clientStatement = value; }
        }

        private string _text = "";
        
        public string Text
        {
            get { return _text; }
            set { _text = value; }
        }

        protected override void Render(HtmlTextWriter writer)
        {
            writer.Write(
                String.Format(
                    "<a id='{0}' href='javascript:{1}' style='font-weight: bold;color: navy'>{2}</a>",
                    this.ID,
                    _clientStatement,
                    Text
                    )
                );
        }
    }

    protected bool _userDataIsCorrect = true;

    protected override void InitializeCulture()
    {
        _rm = ResourceHelper.Current.GetResourceManagerForPage(_controlName);
        base.InitializeCulture();
    }

    protected string GenerateNewGuid()
    {
        System.Guid guid = System.Guid.NewGuid();
        return guid.ToString();
    }

    
   

    protected void Page_Load(object sender, EventArgs e)
    {
        int? id = null;
        int? folder = null;
        bool isNew = false;
        if (
            this.Request.Params[PageParameter.Cmd] != null &&
            this.Request.Params[PageParameter.Cmd] == PageParameterCmd.New)
        {
            isNew = true;
            folder = int.Parse(this.Request.Params[PageParameter.Folder]);
        }
        else
        {
            id = int.Parse(this.Request.Params[PageParameter.Id]);
        }

        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];
        _us = us;

        if (!this.IsPostBack)
        {
            try
            {
                hidSessionId.Value = this.InitCardSession(id, folder, isNew);
            }
            catch (Evpo.DocCirc.Common.DcUnathorizedAccessException ex)
            {
                Response.Redirect(
                    String.Format(PageConstant.ErrorMessageUrl, PageConstant.AccessDeniedErrorMessageKey)
                    );
            }
            catch (Evpo.DocCirc.Common.DcObjectNotFoundException ex)
            {
                Response.Redirect(
                    String.Format(PageConstant.ErrorMessageUrl, PageConstant.DocumentNotFoundErrorMessageKey)
                    );
            }
            catch (Exception ex)
            {
                throw ex;
            }

            this.Title = String.Format(
                    _rm.GetString(ResKeys.PageTitle),
                    _sessionData.DocCardItem.InternalNumber,
                    _sessionData.DocCardItem.Subject
                    );

            LoadRegisteringUserOptions();
            LoadDocTypeOptions();
            if (
                (_sessionData.DocCardItem.DocStatus == DBConstKeeper.Current.DocStatus.NotSaved ||
                _sessionData.DocCardItem.DocStatus == DBConstKeeper.Current.DocStatus.Draft) &&
                uiDocumentType.Items.Count == 0
            )
            {
                Response.Redirect(
                    String.Format(PageConstant.ErrorMessageUrl, PageConstant.DocTypeListIsEmpty)
                    );
            }


            InitEventList();
            InitAttachmentTable();
            InitReferenceList();
            
            SetDocCardToUserRelation();
            InitializeUIControls();
            BindControls(BindingDirection.BindToControls);


            this.DataBind();
            Session[hidSessionId.Value] = _sessionData;
        }
        else
        {
            _sessionData = (DocumentCardSessionData)Session[hidSessionId.Value];
            
            _userDataIsCorrect = CheckUserData();
            if (_userDataIsCorrect)
            {
                BindControls(BindingDirection.BindToData);
            }
        }

        
        ReferencePageHrefInitialize();
        
    }

    protected void Page_PreRender(object sender, EventArgs e)
    {
        this.litScript.Text +=
            "<script type='text/javascript'>" + Environment.NewLine +
            "var pref = \"\";" + Environment.NewLine +
            "var referenceTabIsOpened = " + _sessionData.ReferenceTabIsOpened.ToString().ToLower() + ";" +
                Environment.NewLine +
            "</script>" + Environment.NewLine;

    }

    private void ReferencePageHrefInitialize()
    {
        uiReferencePageHref = new TabHref();
        uiReferencePageHref.ID = "uiReferencePageHref";
        uiReferencePageHref.Text = _rm.GetString(ResKeys.ReferenceTab);
        uiReferencePageHref.ClientStatement = "SetPage(\"uiReferencePage\")";
        uiReferencePageHref.Click += new EventHandler(uiReferencePageHref_Click);
        uiReferencePagePlaceHolder.Controls.Add(uiReferencePageHref);
    }

    private void InitEventList()
    {
        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];
        EventData ed = new EventData(us.UserInfo, us.DataConnection);

        EventHtmlComposer composer = 
            new EventHtmlComposer(us.SettingKeeper, us.DataConnection, us.ServiceUserInfo);

        StringBuilder sb = new StringBuilder();
        foreach (EventItem ei in ed.GetDocCardEventItems(_sessionData.DocCardItem.Id))
        {
            sb.Append("<br/>");
            sb.Append(composer.ComposeHtml(ei));
        }

        uiEvent.Text = sb.ToString();
    }

    protected void SaveDocCard()
    {
        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];

        if (_userDataIsCorrect)
        {
            if (_sessionData.DocCardItem.DocStatus == DBConstKeeper.Current.DocStatus.NotSaved)
            {
                _sessionData.DocCardItem.DocStatus = DBConstKeeper.Current.DocStatus.Draft;

                DocTypeData dtd = new DocTypeData(us.ServiceUserInfo, us.DataConnection);
                _sessionData.DocCardItem.InternalNumber =
                    dtd.GetNextInternalNumber(_sessionData.DocCardItem.DocType.Value);
            }

            DocCardData dcd = new DocCardData(us.UserInfo, us.DataConnection, us.SettingKeeper);

            DocCardMovementController cnt = new DocCardMovementController(us.UserInfo, us.DataConnection);

            int activeUser = us.UserInfo.UserId;
            if (_sessionData.UserWhoseDeputyCurrentUserIs != null)
            {
                activeUser = _sessionData.UserWhoseDeputyCurrentUserIs.Value;
            }

            bool asAdmin = _sessionData.RemovePassRouteDecisionOption;

            if (
                (_sessionData.DocCardItem.DocStatus == DBConstKeeper.Current.DocStatus.NotSaved ||
                _sessionData.DocCardItem.DocStatus == DBConstKeeper.Current.DocStatus.Draft) &&
                uiSubmitCard.Checked
                )
            {
                dcd.LoadDocCardRoute(_sessionData.DocCardItem);
                cnt.StartConsideration(_sessionData.DocCardItem, activeUser);
            }

            if (
                _sessionData.DocCardItem.DocStatus == DBConstKeeper.Current.DocStatus.UnderCons && 
                !_sessionData.RouteTabIsReadOnly &&
                uiSubmitDecision.Checked
                )
            {
                Decision decision = (Decision)Enum.Parse(typeof(Decision), uiDecision.SelectedValue);
                if (decision == Decision.Pass)
                {
                    cnt.Pass(_sessionData.DocCardItem, activeUser, uiDecisionNote.Text);
                }
                else if (decision == Decision.Postpone)
                {
                    cnt.Postpone(_sessionData.DocCardItem, activeUser, uiDecisionNote.Text, asAdmin);
                }
                else if (decision == Decision.Reject)
                {
                    int pointToReturn = int.Parse(uiReturnToPoint.SelectedValue);
                    cnt.Reject(_sessionData.DocCardItem, activeUser, pointToReturn, uiDecisionNote.Text, asAdmin);
                }
            }

            if (cnt.Events.Count > 0)
            {
                EventData ed = new EventData(us.UserInfo, us.DataConnection);
                foreach (EventItem ei in cnt.Events)
                {
                    ed.AddEvent(ei);
                }
            }

            
            dcd.SaveDocCard(_sessionData.DocCardItem, us.DocumentDrafts);
            SaveAttachments();
            SaveReferenceItems();
            

            if (cnt.Notifications.Count > 0)
            {
                Notificator notificator = (Notificator)Application[AppStateKeys.Notificator];                    
                
                foreach (NotificationItem ni in cnt.Notifications)
                {
                    notificator.Notify(ni);
                }
            }
        }

        if (_userDataIsCorrect)
        {
            this.litScript.Text += "<script language='javascript'>window.close();</script>";
        }
    }

    private void SaveReferenceItems()
    {
        DocCardReferenceData rd = new DocCardReferenceData(_us.UserInfo, _us.DataConnection, _us.SettingKeeper);
        DocCardReferenceItem[] refItems = new DocCardReferenceItem[_sessionData.InterfaceReferenceItems.Count];
        for(int i = 0; i < refItems.Length; i ++)
        {
            refItems[i] = _sessionData.InterfaceReferenceItems[i].ReferenceItem;
        }

        rd.SaveDocCardReferences(_sessionData.DocCardItem.Id, refItems);
    }

    private void FillCompanies()
    {
        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];
        InstitutionData instData = new InstitutionData(us.UserInfo, us.DataConnection);

        uiSendingCompany.Items.Clear();
        uiReceivingCompany.Items.Clear();
        uiSendingCompany.Items.Add(new ListItem(_rm.GetString(ResKeys.AnotherCompany), "another"));
        uiReceivingCompany.Items.Add(new ListItem(_rm.GetString(ResKeys.AnotherCompany), "another"));

        if (
                (_sessionData.DocCardItem.DocStatus == DBConstKeeper.Current.DocStatus.NotSaved ||
                _sessionData.DocCardItem.DocStatus == DBConstKeeper.Current.DocStatus.Draft) &&
                _sessionData.CanBeSaved
                )
        {
            InstitutionToken[] tokens = instData.GetAllInstitutionTokents();
            foreach (InstitutionToken token in tokens)
            {
                uiSendingCompany.Items.Add(new ListItem(token.Name, token.Id.ToString()));
                uiReceivingCompany.Items.Add(new ListItem(token.Name, token.Id.ToString()));
            }
        }
        else
        {
            if (_sessionData.DocCardItem.SendingCompany != null)
            {
                InstitutionItem item = 
                    instData.LoadInstitutionItem(_sessionData.DocCardItem.SendingCompany.Value);
                
                uiSendingCompany.Items.Add(new ListItem(item.Name, item.InstitutionId.ToString()));
            }
            
            if (_sessionData.DocCardItem.ReceivingCompany != null)
            {
                InstitutionItem item =
                    instData.LoadInstitutionItem(_sessionData.DocCardItem.ReceivingCompany.Value);

                uiReceivingCompany.Items.Add(new ListItem(item.Name, item.InstitutionId.ToString()));
            }

        }

        // bind companies
        if (_sessionData.DocCardItem.SendingCompany != null)
        {
            uiSendingCompany.SelectedValue = _sessionData.DocCardItem.SendingCompany.Value.ToString();
        }
        else
        {
            uiSendingCompany.SelectedValue = "another";
        }

        if (_sessionData.DocCardItem.ReceivingCompany != null)
        {
            uiReceivingCompany.SelectedValue = _sessionData.DocCardItem.ReceivingCompany.Value.ToString();
        }
        else
        {
            uiReceivingCompany.SelectedValue = "another";
        }
    }

    protected TabHref uiReferencePageHref;

    private void InitializeUIControls()
    {
        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];

        FillCompanies();

        if (
                _sessionData.DocCardItem.DocStatus != DBConstKeeper.Current.DocStatus.NotSaved &&
                _sessionData.DocCardItem.DocStatus != DBConstKeeper.Current.DocStatus.Draft
                )
        {
            uiSubmitCard.Checked = true;
            uiSubmitCard.Enabled = false;
        }

        if (!_sessionData.RemovePassRouteDecisionOption)
        {
            uiDecision.Items.Add(
                new ListItem(ResourceHelper.StrForUI[UIResKeys.PassDecision], Enum.GetName(typeof(Decision), Decision.Pass))
                );
        }
        // decision drop down
        uiDecision.Items.Add(
         new ListItem(ResourceHelper.StrForUI[UIResKeys.PostponeDecision], Enum.GetName(typeof(Decision), Decision.Postpone))
         );
        uiDecision.Items.Add(
            new ListItem(ResourceHelper.StrForUI[UIResKeys.RejectDecision], Enum.GetName(typeof(Decision), Decision.Reject))
            );

        // return to point
        for(int i = _sessionData.DocCardItem.RoutePointNum - 1; i > 0; i --)
        {
            uiReturnToPoint.Items.Add(new ListItem(i.ToString(), i.ToString()));
        }

        uiReturnToPoint.Items.Add(
            new ListItem(_rm.GetString(ResKeys.ReturnToDraft), (0).ToString())
            );

        // user labels
        uiUser.Text = us.UserInfo.DisplayName;

        string mainUserDisplayName = "";
        if (_sessionData.UserWhoseDeputyCurrentUserIs != null)
        {
            UserProperties prop = UserHelper.Current.GetUserProperties(
                    _sessionData.UserWhoseDeputyCurrentUserIs.Value,
                    us.DataConnection
                    );
            mainUserDisplayName = prop.DisplayName;
        }

        string deputyLabel = "";
        if (_sessionData.RemovePassRouteDecisionOption && _sessionData.UserWhoseDeputyCurrentUserIs != null)
        {
            deputyLabel = _rm.GetString(ResKeys.AdministratorDeputyLabel);
        }
        else if (!_sessionData.RemovePassRouteDecisionOption && _sessionData.UserWhoseDeputyCurrentUserIs != null)
        {
            deputyLabel = _rm.GetString(ResKeys.DeputyLabel);
        }

        if (deputyLabel != "")
        {
            deputyLabel = String.Format(deputyLabel, mainUserDisplayName);
        }
        uiDeputyLabel.Text = deputyLabel;

        uiFolderName.Text = _sessionData.FolderName;
    }

    void uiReferencePageHref_Click(object sender, EventArgs e)
    {
        if (!_sessionData.ReferenceTabIsOpened)
        {
            InitializeReferenceTab();
            _sessionData.ReferenceTabIsOpened = true;
        }
    }

    private void SetDocCardToUserRelation()
    {
        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];

        DocCardRelationToUserResolver resolver = new DocCardRelationToUserResolver(us.UserInfo, us.DataConnection);


        DocCardRelationToUser relation = resolver.GetDocCardRelationToUser(_sessionData.DocCardItem);
        _sessionData.AttachmentIsReadOnly = relation.AttachmentIsReadOnly;
        _sessionData.AttachmentReadOnlyCheckBoxIsReadOnly = relation.AttachmentReadOnlyCheckBoxIsReadOnly;
        _sessionData.CanBeSaved = relation.CanBeSaved;
        _sessionData.CanBeSeen = relation.CanBeSeen;
        _sessionData.DocumentTabIsReadOnly = relation.DocumentTabIsReadOnly;
        _sessionData.RemovePassRouteDecisionOption = relation.RemovePassRouteDecisionOption;
        _sessionData.RouteTabIsReadOnly = relation.RouteTabIsReadOnly;
        _sessionData.RouteTabIsVisible = relation.RouteTabIsVisible;
        _sessionData.UserWhoseDeputyCurrentUserIs = relation.UserWhoseDeputyCurrentUserIs;
    }

    private void SaveAttachments()
    {
        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];
        AttachmentData ad = new AttachmentData(us.UserInfo, us.DataConnection, us.SettingKeeper);

        // select item to delete
        List<AttachmentInterfaceItem> itemsToDeleteList = new List<AttachmentInterfaceItem>();
        for (int i = 0; i < uiAttachmentWell.Rows.Count; i++)
        {
            GridViewRow row = uiAttachmentWell.Rows[i];
            CheckBox uiCheckBox = (CheckBox)row.FindControl("uiDelete");
            if (uiCheckBox.Checked)
            {
                itemsToDeleteList.Add(_sessionData.AttachmentInterfaceItems[i]);
            }
        }

        // add new attachments
        AddAttachment(ad, uiFile1, itemsToDeleteList);
        AddAttachment(ad, uiFile2, itemsToDeleteList);
        AddAttachment(ad, uiFile3, itemsToDeleteList);
        AddAttachment(ad, uiFile4, itemsToDeleteList);

        foreach (AttachmentInterfaceItem intItem in itemsToDeleteList)
        {
            AttachmentItem item = ad.GetAttachmentItem(intItem.Id);
            ad.DeleteAttachment(item);
        }

    }

    private void AddAttachment(
        AttachmentData attachmentData, FileUpload fileUpload, List<AttachmentInterfaceItem> itemsToDelete
        )
    {
        if (fileUpload.HasFile)
        {
            AttachmentInterfaceItem itemToReplace = null;
            foreach (AttachmentInterfaceItem intItem in itemsToDelete)
            {
                if (intItem.DisplayName.ToUpper() == fileUpload.FileName.ToUpper())
                {
                    itemToReplace = intItem;
                    break;
                }
            }

            AttachmentItem ai = null;
            if (itemToReplace == null)
            {
                ai = new AttachmentItem(
                    _sessionData.DocCardItem.Id,
                    fileUpload.FileName,
                    fileUpload.PostedFile.ContentType,
                    fileUpload.PostedFile.ContentLength,
                    DateTime.Now
                    );
            }
            else
            {
                itemsToDelete.Remove(itemToReplace);
                ai = attachmentData.GetAttachmentItem(itemToReplace.Id);
                ai.ContentType = fileUpload.PostedFile.ContentType;
                ai.Size = fileUpload.PostedFile.ContentLength;
                ai.UploadTime = DateTime.Now;
            }

            attachmentData.SaveAttachment(ai, fileUpload.PostedFile.InputStream);
            fileUpload.PostedFile.InputStream.Close();
        }
    }

    private void InitReferenceList()
    {
        DocCardReferenceData rd = new DocCardReferenceData(_us.UserInfo, _us.DataConnection, _us.SettingKeeper);
        DocCardReferenceItem[] items = rd.GetDocCardReferences(_sessionData.DocCardItem.Id);
        foreach (DocCardReferenceItem item in items)
        {
            _sessionData.InterfaceReferenceItems.Add(GetInterfaceReferenceItem(item));
        }

        uiReferenceList.DataSource = _sessionData.InterfaceReferenceItems;
        uiReferenceList.DataBind();
    }

    private void InitAttachmentTable()
    {
        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];
        AttachmentData ad = new AttachmentData(us.UserInfo, us.DataConnection, us.SettingKeeper);
        AttachmentItem[] items = ad.GetDocCardAttachmentItems(_sessionData.DocCardItem.Id);
        AttachmentInterfaceItem[] attIntItems = new AttachmentInterfaceItem[items.Length];
        
        string sAttachmentUrl = 
            us.SettingKeeper.AttachmentSavingDialogEnabled ? 
            PageConstant.AttachmentUrlForSavingDialog : 
            PageConstant.AttachmentUrl;

        for (int i = 0; i < items.Length; i ++)
        {
            AttachmentItem item = items[i];
            attIntItems[i] = new AttachmentInterfaceItem(
                item.Id.Value,
                String.Format(sAttachmentUrl, item.Id.ToString()),
                item.FileName,
                false
                );
        }
        _sessionData.AttachmentInterfaceItems = attIntItems;
        uiAttachmentWell.DataSource = attIntItems;
        uiAttachmentWell.DataBind();
    }

    protected void LoadRegisteringUserOptions()
    {
        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];
        if (
            _sessionData.DocCardItem.DocStatus == DBConstKeeper.Current.DocStatus.NotSaved ||
            _sessionData.DocCardItem.DocStatus == DBConstKeeper.Current.DocStatus.Draft
            )
        {
            UserProperties[] users = UserHelper.Current.GetAllUserProperties(us.DataConnection);
            foreach (UserProperties user in users)
            {
                uiRegisteringUser.Items.Add(new ListItem(user.DisplayName, user.UserId.ToString()));
            }
        }
        else
        {
            UserProperties user = UserHelper.Current.GetUserProperties(
                _sessionData.DocCardItem.RegisteringUser, 
                us.DataConnection
                );
            uiRegisteringUser.Items.Add(new ListItem(user.DisplayName, user.UserId.ToString()));
        }
    }

    protected void LoadDocTypeOptions()
    {
        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];
        DocTypeData dtd = new DocTypeData(us.UserInfo, us.DataConnection);
        
        bool addDocType = true;

        if (
            _sessionData.DocCardItem.DocStatus == DBConstKeeper.Current.DocStatus.NotSaved ||
            _sessionData.DocCardItem.DocStatus == DBConstKeeper.Current.DocStatus.Draft
            )
        {
            int? lastDocType = null;

            if (_sessionData.DocCardItem.IsNew)
            {
                addDocType = false;
                
                UserProperties prop = UserHelper.Current.GetUserProperties(_us.UserInfo.UserId, _us.DataConnection);
                lastDocType = prop.LastDocType;
            }
            
            bool useTmpDocTypeSecurity = true;
            DocTypeItem[] docTypeItems = dtd.GetDocTypesWithCreatePermission(!useTmpDocTypeSecurity);

            foreach (DocTypeItem docTypeItem in docTypeItems)
            {
                uiDocumentType.Items.Add(new ListItem(docTypeItem.Name, docTypeItem.Id.ToString()));

                if (addDocType && docTypeItem.Id == _sessionData.DocCardItem.DocType)
                {
                    addDocType = false;
                }

                if (lastDocType != null && docTypeItem.Id == lastDocType.Value)
                {
                    uiDocumentType.SelectedValue = lastDocType.ToString();
                }
            }

            

        }
        else
        {
            addDocType = true;
        }

        if (addDocType)
        {
            bool loadRoute = true;
            DocTypeItem docTypeItem = dtd.LoadDocTypeItem(_sessionData.DocCardItem.DocType.Value, !loadRoute);
            uiDocumentType.Items.Add(new ListItem(docTypeItem.Name, docTypeItem.Id.ToString()));
        }
    }

    protected bool CheckUserData()
    {
        bool originalDateIsOK = true, subjectIsOK = true;
        if (uiOriginalDate.Text.Length != 0)
        {
            try
            {
                DateTime date = DateTime.Parse(uiOriginalDate.Text);
            }
            catch (FormatException ex)
            {
                originalDateIsOK = false;
            }
        }

        if (uiSubject.Text.Trim().Length == 0)
        {
            subjectIsOK = false;
        }

        StringBuilder sb = new StringBuilder();
        if (!originalDateIsOK)
        {
            sb.Append(_rm.GetString(ResKeys.OriginalDateFormatError));
        }

        if (!subjectIsOK)
        {
            if (sb.Length > 0)
            {
                sb.Append("<br/>");
            }
            sb.Append(_rm.GetString(ResKeys.SubjectIsEmpty));
        }

        bool retVal = true;
        if (!originalDateIsOK || !subjectIsOK)
        {
            uiDocumentPageError.Text = sb.ToString();
            uiDocumentPageError.Visible = true;
            retVal = false;
        }

        return retVal;
    }

    protected string InitCardSession(int? id, int? folder, bool isNew)
    {
        string cardSessionId = GenerateNewGuid();

        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];
        DocCardData dcd = new DocCardData(us.UserInfo, us.DataConnection, us.SettingKeeper);

        if (isNew)
        {
            _sessionData.DocCardItem = dcd.CreateDocCard(folder.Value, us.DocumentDrafts);
        }
        else
        {
            bool loadRouteItems = true;
            _sessionData.DocCardItem = dcd.LoadDocCard(id.Value, loadRouteItems);
        }

        FolderData fd = new FolderData(_us.UserInfo, _us.DataConnection);
        _sessionData.FolderName = fd.GetFolderItem(_sessionData.DocCardItem.Folder).Name;
        if (_sessionData.FolderName.StartsWith("^"))
        {
            string resKey = _sessionData.FolderName.Replace("^", "");
            _sessionData.FolderName = ResourceHelper.StrForUI[resKey];
        }

        return cardSessionId;
    }

    protected void BindControls(BindingDirection direction)
    {
        DocCardItem item = _sessionData.DocCardItem;
        if (direction == BindingDirection.BindToControls)
        {
            // document tab
            if (item.DocType != null)
            {
                uiDocumentType.SelectedValue = item.DocType.ToString();
            }
            
            string statusResKey = (new ResKeyResolver()).GetDocStatusResKey(item.DocStatus);
            uiStatus.Text = ResourceHelper.StrForUI[statusResKey];

            uiRegisteringUser.SelectedValue = item.RegisteringUser.ToString();

            if (item.LastMovement != null)
            {
                uiLastMovement.Text = PresentDateTime(item.LastMovement.Value); // R-O
            }
            uiSubject.Text = item.Subject;
            uiInternalNumber.Text = item.InternalNumber; // R-O
            uiRegistrationDate.Text = PresentDateTime(item.RegistrationTime); // R-O
            if (item.FinalizingTime != null)
            {
                uiFinalizingDate.Text = PresentDateTime(item.FinalizingTime.Value); // R-O
            }
            uiLockContent.Checked = !item.AttachmentsAreChangeable;
            if (
                item.DocStatus == DBConstKeeper.Current.DocStatus.NotSaved ||
                item.DocStatus == DBConstKeeper.Current.DocStatus.Draft
               )
            {
                uiSubmitCard.Checked = false;
            }
            else
            {
                uiSubmitCard.Checked = true;
            }

            if (item.OriginalDate != null)
            {
                uiOriginalDate.Text = PresentDate(item.OriginalDate.Value);
            }

            uiOriginalNumber.Text = item.OriginalNumber;
            uiSendingCompanyData.Text = item.CustomSendingCompany;
            uiReceivingCompanyData.Text = item.CustomReceivingCompany;

            if (item.SendingCompany != null)
            {
                uiSendingCompany.SelectedValue = item.SendingCompany.Value.ToString();
            }
            else
            {
                uiSendingCompany.SelectedValue = "another";
            }

            if (item.ReceivingCompany != null)
            {
                uiReceivingCompany.SelectedValue = item.ReceivingCompany.Value.ToString();
            }
            else
            {
                uiReceivingCompany.SelectedValue = "another";
            }

            uiNote.Text = item.Note;

            // route tab
            uiRoutePointNum.Text = item.RoutePointNum.ToString();

            if (!item.IsNew)
            {
                FillRoute();
            }

        }
        else //BindingDirection.BindToData
        {
            item.DocType = int.Parse(uiDocumentType.SelectedValue);
            item.Subject = uiSubject.Text;
            item.AttachmentsAreChangeable = !uiLockContent.Checked;
            if (uiOriginalDate.Text.Trim() != "")
            {
                item.OriginalDate = GetDateTime(uiOriginalDate.Text);
            }
            item.OriginalNumber = uiOriginalNumber.Text;
            item.CustomSendingCompany = uiSendingCompanyData.Text;
            item.CustomReceivingCompany = uiReceivingCompanyData.Text;

            if (uiSendingCompany.SelectedValue != "another")
            {
                item.SendingCompany = int.Parse(uiSendingCompany.SelectedValue);
            }
            else
            {
                item.SendingCompany = null;
            }

            if (uiReceivingCompany.SelectedValue != "another")
            {
                item.ReceivingCompany = int.Parse(uiReceivingCompany.SelectedValue);
            }
            else
            {
                item.ReceivingCompany = null;
            }

            item.Note = uiNote.Text;
        }
    }

    protected void FillRoute()
    {
        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];
        foreach (RoutePoint rp in _sessionData.DocCardItem.RoutePoints)
        {
            foreach (RouteItem ri in rp.RouteItems)
            {
                UserProperties userProps = UserHelper.Current.GetUserProperties(ri.User, us.DataConnection); 
                DocCardRouteItem item = (DocCardRouteItem) ri;
                string result = "";
                if(item.Passed)
                {
                    result = _rm.GetString(ResKeys.PassedResult);
                }
                else if(item.UnderCons)
                {
                    result = _rm.GetString(ResKeys.UnderConsResult);
                }

                string rowText = String.Format(
                    _rm.GetString(ResKeys.RouteItemListRow), 
                    rp.OrderNum.ToString(),
                    userProps.DisplayName,
                    result
                    );
                uiRoute.Items.Add(rowText);
            }
        }
    }

    protected DateTime GetDateTime(string dateTimeStr)
    {
        return DateTime.Parse(dateTimeStr);
    }

    protected string PresentDateTime(DateTime dateTime)
    {
        return dateTime.ToString();
    }

    protected string PresentDate(DateTime dateTime)
    {

        return dateTime.Date.ToString("d");
    }

    protected void uiSave_Click(object sender, EventArgs e)
    {
        SaveDocCard();
    }

    protected void uiRefresh2_Click(object sender, EventArgs e)
    {
        this.FillCompanies();        
    }

    private void InitializeReferenceTab()
    {
        DocTypeData dtd = new DocTypeData(_us.UserInfo, _us.DataConnection);
        foreach(DocTypeItem docType in dtd.GetDocTypes())
        {
            uiDocTypeListInRef.Items.Add(new ListItem(docType.Name, docType.Id.ToString()));
        }

        if(uiDocTypeListInRef.Items.Count > 0)
        {
            uiDocTypeListInRef.SelectedIndex = 0;
            FillInternalNumList();
        }
    }

    protected void FillInternalNumList()
    {
        if (uiDocTypeListInRef.Items.Count > 0)
        {
            int docType = int.Parse(uiDocTypeListInRef.SelectedValue);
            DocCardData dcd = new DocCardData(_us.UserInfo, _us.DataConnection, _us.SettingKeeper);
            DataTable dt = dcd.GetDocumentTable(
                new string[] { DocCardFieldName.Id, DocCardFieldName.InternalNumber, DocCardFieldName.Subject },
                new DocCardFilterItem[] { new DocCardFilterItem(DocCardFieldName.DocType, docType) }
                );

            uiInternalNumList.Items.Clear();
            foreach (DataRow row in dt.Rows)
            {
                uiInternalNumList.Items.Add(
                    new ListItem(
                        String.Format(
                            "{0} : {1}", 
                            row[DocCardFieldName.InternalNumber].ToString(), 
                            row[DocCardFieldName.Subject].ToString()
                            ),
                        row[DocCardFieldName.Id].ToString()
                        )
                    );
            }
        }

        uiAddReference.Enabled = (uiInternalNumList.Items.Count > 0);
        
    }

    protected void uiDocTypeListInRef_SelectedIndexChanged(object sender, EventArgs e)
    {
        FillInternalNumList();
    }

    protected void uiRemoveReference_OnCheckedChanged(object sender, EventArgs e)
    {
        RepeaterItem repItem = (RepeaterItem)((Control)sender).Parent;

        _sessionData.InterfaceReferenceItems.RemoveAt(repItem.ItemIndex);
        
        uiReferenceList.DataSource = _sessionData.InterfaceReferenceItems;
        uiReferenceList.DataBind();
    }

    protected void uiAddReference_Click(object sender, EventArgs e)
    {
        AddDocCardToReferenceList(long.Parse(uiInternalNumList.SelectedValue));
        uiReferenceList.DataSource = _sessionData.InterfaceReferenceItems;
        uiReferenceList.DataBind();
    }

    protected void AddDocCardToReferenceList(long docCardId)
    {
        bool hasBeenAdded = false;
        foreach(InterfaceReferenceItem curItem in _sessionData.InterfaceReferenceItems)
        {
            if (curItem.ReferenceItem.Reference == docCardId)
            {
                hasBeenAdded = true;
                break;
            }
        }

        if (!hasBeenAdded)
        {
            DocCardReferenceItem refItem = new DocCardReferenceItem(
                _sessionData.DocCardItem.Id,
                docCardId
                );
            InterfaceReferenceItem interfaceItem = GetInterfaceReferenceItem(refItem);
            _sessionData.InterfaceReferenceItems.Add(interfaceItem);
        }
        else
        {
            uiReferenceAdditionErrorMessage.Text =
                   _rm.GetString(ResKeys.DocCardHasBeenAddedToReferenceListError);
            uiReferenceAdditionErrorMessage.Visible = true;
        }

    }

    protected InterfaceReferenceItem GetInterfaceReferenceItem(DocCardReferenceItem refItem)
    {
        DocCardData dcd = new DocCardData(_us.ServiceUserInfo, _us.DataConnection, _us.SettingKeeper);
        bool loadRoute = true;
        DocCardItem item = dcd.LoadDocCard(refItem.Reference, !loadRoute);
        SecurityData sd = new SecurityData(_us.UserInfo, _us.DataConnection);
        bool isAccess = sd.GetDocumentCardSecurity(item).Read;

        string registeringUserDisplayName =
            UserHelper.Current.GetUserProperties(item.RegisteringUser, _us.DataConnection).DisplayName;
        

        InterfaceReferenceItem interfaceItem = null;

        if(isAccess)
        {
            interfaceItem = new InterfaceReferenceItem(
                refItem,
                String.Format(
                    _rm.GetString(ResKeys.ReferenceRow), 
                    item.InternalNumber, 
                    item.Subject, 
                    registeringUserDisplayName
                    ),
                String.Format(PageConstant.DocCardUrl, item.Id),
                !isAccess
                );
        }
        else
        {
            interfaceItem = new InterfaceReferenceItem(
                refItem,
                String.Format(_rm.GetString(ResKeys.InaccessibleReferenceRow), item.InternalNumber),
                "",
                isAccess
                );
        }

        return interfaceItem;
    }

    protected string GetDocumentCardUrl()
    {
        SettingKeeper sk = (SettingKeeper)Application[AppStateKeys.SettingKeeper];
        return String.Format("{0}/DocumentCard.aspx?id={1}", sk.ApplicationUrl, _sessionData.DocCardItem.Id);
    }

}
