//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Collections.Generic;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Globalization;
using System.Security.Principal;
using System.Text;
using System.Text.RegularExpressions;
using System.Resources;
using System.Xml.Serialization;
using System.Xml;
using System.Xml.Schema;
using System.IO;
using Evpo.DocCirc;
using Evpo.DocCirc.Biz;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.UI;

public partial class DocumentType : System.Web.UI.Page
{
    
    
    private enum BindingDirection
    {
        BindToControls,
        BindToData
    }

    protected struct PageParameter
    {
        public const string Cmd = "cmd";
        public const string Id = "id";
        public const string Folder = "folder";
    }

    protected struct PageParameterCmd
    {
        public const string New = "new";
        public const string Open = "open";
    }

    #region Resources
    private const string _controlName = "DocumentType.aspx";
    public struct ResKeys
    {
        public const string PageTitle = "PageTitle";
        public const string DocsWillBeReturnedToDrafts = "DocsWillBeReturnedToDrafts";
        public const string SaveAndCloseButton = "SaveAndCloseButton";
        public const string CancelButton = "CancelButton";
        public const string MainTab = "MainTab";
        public const string RouteTab = "RouteTab";
        
        public const string DocTypeName = "DocTypeName";
        public const string Description = "Description";
        public const string Prefix = "Prefix";
        public const string NextNumber = "NextNumber";

        public const string RoutePointInPanel = "RoutePointInPanel";
        public const string DeleteInPanel = "DeleteInPanel";

        public const string EmployeeList = "EmployeeList";
        public const string RoutePointSelect = "RoutePointSelect";

        public const string AddRouteItemButton = "AddRouteItemButton";
        public const string RouteTableTitle = "RouteTableTitle";
        public const string Remove = "Remove";
        public const string RouteInstruction = "RouteInstruction";

        public const string DecisionRouteItemType = "DecisionRouteItemType";
        public const string NotificationRouteItemType = "NotificationRouteItemType";
        
        public const string DocumentTypePermission = "DocumentTypePermission";
    }
    protected ResourceManager _rm;
    #endregion

    protected class SaveAndCloseButton : Control, IPostBackEventHandler
    {
        #region IPostBackEventHandler Members

        public void RaisePostBackEvent(string eventArgument)
        {
            OnClick(new EventArgs());
        }
        #endregion

        public event EventHandler Click;

        protected void OnClick(EventArgs e)
        {
            if (Click != null)
            {
                Click(this, e);
            }
        }

        private string _clientStatement = "";

        public string ClientStatement
        {
            get { return _clientStatement; }
            set { _clientStatement = value; }
        }

        private string _text = "";

        public string Text
        {
            get { return _text; }
            set { _text = value; }
        }

        private bool _disabled = false;

        public bool Disabled
        {
            get { return _disabled; }
            set { _disabled = value; }
        }


        protected override void Render(HtmlTextWriter writer)
        {
            string disabledInjection = "";
            if (_disabled)
            {
                disabledInjection = "disabled='true'";
            }

            writer.Write(
                String.Format(
                    "<input type='button' id='{0}' runat='server' value='{1}' onclick='javascript:{2}' {3} />"
                    ,
                    this.ID,
                    Text,
                    _clientStatement,
                    disabledInjection
                    )
                );
        }

    }

    protected class RoutePointInterfaceItem
    {
        private string _displayName;
        public string DisplayName
        {
            get { return _displayName; }
            set { _displayName = value; }
        }

        private int _routePointNum;

        public int RoutePointNum
        {
            get { return _routePointNum; }
            set { _routePointNum = value; }
        }

        private int _employeeId;

        public int EmployeeId
        {
            get { return _employeeId; }
            set { _employeeId = value; }
        }

        private int _routeItemType;

        public int RouteItemType
        {
            get { return _routeItemType; }
            set { _routeItemType = value; }
        }

        private int? _routePoint;
        
        public int? RoutePoint
        {
            get { return _routePoint; }
            set { _routePoint = value; }
        }


        private int? _routeItem;

        public int? RouteItem
        {
            get { return _routeItem; }
            set { _routeItem = value; }
        }


        public RoutePointInterfaceItem(int? routePoint, int? routeItem, int employeeId, int routePointNum, int routeItemType, string displayName)
        {
            _routePoint = routePoint;
            _routeItem = routeItem;
            _employeeId = employeeId;
            _routePointNum = routePointNum;
            _routeItemType = routeItemType;
            _displayName = displayName;
        }

    }

    protected class DocumentTypeSessionData
    {
        public DocTypeItem DocTypeItem;
        public List<RoutePointInterfaceItem> RoutePointInterfaceItems;
    }


    protected DocumentTypeSessionData _sessionData = new DocumentTypeSessionData();

    protected override void InitializeCulture()
    {
        _rm = ResourceHelper.Current.GetResourceManagerForPage(_controlName);
        base.InitializeCulture();
    }

    protected string GenerateNewGuid()
    {
        System.Guid guid = System.Guid.NewGuid();
        return guid.ToString();
    }


    //private DocTypeAccess _dtAccess;
    //Hashtable _bindTable;
    private string _errorMessage = "";
    private bool _readOnly = false;

    protected void ReadOnlyState(bool readOnly)
    {
        set_ReadOnlyState("uiDocTypeName", readOnly);
        set_ReadOnlyState("uiEmployee", readOnly);
        set_ReadOnlyState("uiDescription", readOnly);
        set_ReadOnlyState("uiPrefix", readOnly);
        set_ReadOnlyState("uiRoutePoint", readOnly);
        set_ReadOnlyState("uiNextNumber", readOnly);
        set_ReadOnlyState("uiRouteItemType", readOnly);
        uiAddRouteItem.Enabled = !readOnly;
        uiSaveAndClose.Disabled = readOnly;
        _readOnly = readOnly;
    }

    protected SaveAndCloseButton uiSaveAndClose;

    protected void Page_Load(object sender, EventArgs e)
    {
        bool unauthorized = false;
        
        int? id = null;
        int? folder = null;
        bool isNew = false;
        if (
            this.Request.Params[PageParameter.Cmd] != null && 
            this.Request.Params[PageParameter.Cmd] == PageParameterCmd.New)
        {
            isNew = true;
            folder = int.Parse(this.Request.Params[PageParameter.Folder]);
        }
        else
        {
            id = int.Parse(this.Request.Params[PageParameter.Id]);
        }

        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];
        
        if (!this.IsPostBack)
        {
            string cardSessionId = "";
            try
            {
                cardSessionId = this.InitCardSession(id, folder, isNew);
            }
            catch (DcUnathorizedAccessException ex)
            {
                _errorMessage = ResourceHelper.StrForUI[UIResKeys.AccessDenied];
                unauthorized = true;
            }

            if (!unauthorized)
            {
                this.hidSessionId.Value = cardSessionId;

                SaveAndCloseButtonInitialize();
                RouteItemTypeInitialize();

                SecurityData sd = new SecurityData(us.UserInfo, us.DataConnection);
                SecurityItem si = sd.GetFolderSecurity(_sessionData.DocTypeItem.Folder);

                if (!si.Update && !(si.UpdateOwn && us.UserInfo.UserId == _sessionData.DocTypeItem.Owner))
                {
                    ReadOnlyState(true);
                }
                
                if (!_sessionData.DocTypeItem.IsNew)
                {
                    set_ReadOnlyState("uiDocTypeName", true);
                }

                this.Title = String.Format(
                    _rm.GetString(ResKeys.PageTitle),
                    _sessionData.DocTypeItem.Name
                    );

                InitRouteTable();
                UpdateRoutePointDropDown();
                uiRoute.DataSource = _sessionData.RoutePointInterfaceItems;
                uiRoute.DataBind();
                //// debug purposes
                //if (isNew)
                //{
                //    _sessionData.DocTypeItem.RouteItems.Add(new RouteItem(1, 1, 1));
                //}

                BindControls(BindingDirection.BindToControls);

                UserProperties[] users = UserHelper.Current.GetAllUserProperties(us.DataConnection);
                
                foreach(UserProperties user in users)
                {
                    uiEmployee.Items.Add(new ListItem(user.DisplayName, user.UserId.ToString()));
                }

                Session[this.hidSessionId.Value] = _sessionData;
            }

            this.DataBind();
        }
        else // PostBack
        {
            string cardSessionId = this.hidSessionId.Value;
            _sessionData = (DocumentTypeSessionData)Session[this.hidSessionId.Value];
            SaveAndCloseButtonInitialize();
            
        }

        if (!unauthorized)
        {
            this.litScript.Text +=
                    "<script type='text/javascript'>" + Environment.NewLine +
                    "var pref = \"\";" + Environment.NewLine +
                //"var thisCardHref = \"" + _dtAccess.DataUrl + "\";" + Environment.NewLine +
                    "var lReadOnly = " + _readOnly.ToString().ToLower() + ";" + Environment.NewLine +
                    "var nDecisionRouteType = " + 
                        DBConstKeeper.Current.RouteItemType.Decision.ToString() +
                        ";" + Environment.NewLine +
                    "</script>" + Environment.NewLine;
        }

        if (_errorMessage != "")
        {
            string msg = _errorMessage;
            //this.ShowErrorMessage(msg);
        }

        
    }

    

    protected void SaveAndCloseButtonInitialize()
    {
        uiSaveAndClose = new SaveAndCloseButton();
        uiSaveAndClose.Text = _rm.GetString(ResKeys.SaveAndCloseButton);
        uiSaveAndClose.ClientStatement = "SubmitPage()";
        uiSaveAndClose.ID = "uiSaveAndClose";
        uiSaveAndClose.Click += new EventHandler(uiSaveAndClose_Click);
        uiSaveAndClosePlaceHolder.Controls.Add(uiSaveAndClose);
    }

    protected void RouteItemTypeInitialize()
    {
        uiRouteItemType.Items.Add(
            new ListItem(
                _rm.GetString(ResKeys.DecisionRouteItemType),
                DBConstKeeper.Current.RouteItemType.Decision.ToString()
                )
            );

        uiRouteItemType.Items.Add(
            new ListItem(
                _rm.GetString(ResKeys.NotificationRouteItemType),
                DBConstKeeper.Current.RouteItemType.Notification.ToString()
                )
            );
        uiRouteItemType.SelectedIndex = 0;

    }

    protected void UpdateRoutePointDropDown()
    {
        int maxRoutePoint = 0;
        if (_sessionData.RoutePointInterfaceItems.Count > 0)
        {
            maxRoutePoint =
                _sessionData.RoutePointInterfaceItems[_sessionData.RoutePointInterfaceItems.Count - 1].RoutePointNum;
        }

        uiRoutePoint.Items.Clear();
        for (int i = 1; i <= maxRoutePoint + 1; i++)
        {
            uiRoutePoint.Items.Add(new ListItem(i.ToString(), i.ToString()));
        }

        uiRoutePoint.SelectedValue = (maxRoutePoint + 1).ToString();
    }


    protected void uiSaveAndClose_Click(object sender, EventArgs e)
    {
        bool userDataIsCorrect = true;

        BindControls(BindingDirection.BindToData);
        BindRouteInterfaceCollectionToDocTypeItem();
        userDataIsCorrect = CheckUserData();

        if (userDataIsCorrect)
        {
            SaveDocumentType();
        }


        if (userDataIsCorrect)
        {
            this.litScript.Text += "<script language='javascript'>window.close();</script>";
        }
    }

    private void BindRouteInterfaceCollectionToDocTypeItem()
    {
        List<RoutePoint> routePoints = new List<RoutePoint>();
        int i = 0;
        while (i < _sessionData.RoutePointInterfaceItems.Count)
        {
            int routePointNum = _sessionData.RoutePointInterfaceItems[i].RoutePointNum;
            int? routePoint = _sessionData.RoutePointInterfaceItems[i].RoutePoint;
            StringBuilder expression = new StringBuilder("");
            List<RouteItem> routeItems = new List<RouteItem>();
            int riIndex = 0;
            while (
                i < _sessionData.RoutePointInterfaceItems.Count && 
                routePointNum == _sessionData.RoutePointInterfaceItems[i].RoutePointNum
                )
            {
                RoutePointInterfaceItem item = _sessionData.RoutePointInterfaceItems[i];
                routeItems.Add(new RouteItem(item.RouteItem, item.EmployeeId, item.RouteItemType));
                if (item.RouteItemType == DBConstKeeper.Current.RouteItemType.Decision)
                {
                    if (expression.Length != 0)
                    {
                        expression.Append("&&");
                    }

                    expression.AppendFormat("[{0}]", riIndex);
                }
                i++;
                riIndex++;
            }
            RoutePoint rp = new RoutePoint(routePoint, expression.ToString(), routePointNum);
            rp.RouteItems = routeItems;
            routePoints.Add(rp);
        }

        _sessionData.DocTypeItem.RoutePoints = routePoints;
        
    }

    protected void SaveDocumentType()
    {
        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];

        DocTypeData dtd = new DocTypeData(us.UserInfo, us.DataConnection);
        
        dtd.ProcessDocCardsAfterRouteChange(
            _sessionData.DocTypeItem, 
            us.ServiceUserInfo, 
            us.SettingKeeper,
            (Notificator)Application[AppStateKeys.Notificator]
            );

        dtd.SaveDocTypeItem(_sessionData.DocTypeItem, us.DocTypeDrafts);

        
    }

    protected bool CheckUserData()
    {

        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];
        CheckerAnswer answer =
            (new DocTypeChecker(us.ServiceUserInfo, us.DataConnection)).CheckDocTypeBeforeSave(
                _sessionData.DocTypeItem
                );

        if (!answer.Success)
        {
            uiMessage.Text = answer.TextForUser;
            uiMessage.Visible = true;
        }

        return answer.Success;
    }

    protected void InitRouteTable()
    {
        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];
        
        _sessionData.RoutePointInterfaceItems =
            new List<RoutePointInterfaceItem>(_sessionData.DocTypeItem.RoutePoints.Count);

        foreach (RoutePoint rp in _sessionData.DocTypeItem.RoutePoints)
        {
            foreach (RouteItem ri in rp.RouteItems)
            {
                _sessionData.RoutePointInterfaceItems.Add(
                    new RoutePointInterfaceItem(
                        rp.Id,
                        ri.Id,
                        ri.User,
                        rp.OrderNum,
                        ri.RouteItemType,
                        UserHelper.Current.GetUserProperties(ri.User, us.DataConnection).DisplayName
                        )
                    );
            }
        }

    }

    protected int _notificationRouteItemType = DBConstKeeper.Current.RouteItemType.Notification;

    protected string InitCardSession(int? id, int? folder, bool isNew)
    {
        string cardSessionId = GenerateNewGuid();
        

        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];
        DocTypeData dtd = new DocTypeData(us.UserInfo, us.DataConnection);

        //try
        //{
        if (isNew)
        {
            _sessionData.DocTypeItem = dtd.CreateDocType(folder.Value, us.DocTypeDrafts);
        }
        else
        {
            bool loadRouteItems = true;
            _sessionData.DocTypeItem = dtd.LoadDocTypeItem(id.Value, loadRouteItems);
        }

        
        
        return cardSessionId;
    }

    private void BindControls(BindingDirection direction)
    {
        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];
        RoutePointSerializer ser = new RoutePointSerializer(us.DataConnection);        

        if (direction == BindingDirection.BindToControls)
        {
            this.uiDocTypeName.Text = _sessionData.DocTypeItem.Name;
            this.uiPrefix.Text = _sessionData.DocTypeItem.Prefix;
            this.uiDescription.Text = _sessionData.DocTypeItem.Description;
            this.uiNextNumber.Text = _sessionData.DocTypeItem.NextNumber.ToString();
        }
        else
        {
            _sessionData.DocTypeItem.Name = this.uiDocTypeName.Text;
            _sessionData.DocTypeItem.Prefix = this.uiPrefix.Text;
            _sessionData.DocTypeItem.Description = this.uiDescription.Text;
            _sessionData.DocTypeItem.NextNumber = int.Parse(this.uiNextNumber.Text);
        }
    }

    public bool get_ReadOnlyState(string interfaceName)
    {
        bool retVal = false;
        Control cnt = this.FindControl(interfaceName);
        if (cnt is TextBox)
        {
            retVal = ((TextBox)cnt).ReadOnly;
        }
        else if (cnt is CheckBox)
        {
            AttributeCollection atts = ((CheckBox)cnt).InputAttributes;
            retVal = (atts["disabled"] != null);
        }
        else if (cnt is DropDownList)
        {
            AttributeCollection atts = ((DropDownList)cnt).Attributes;
            retVal = (atts["disabled"] != null);
        }
        else if (cnt is Label)
        {
            retVal = true;
        }
        else
        {
            throw new Exception("DocCirc: get_ReadOnlyState: Unknown cnt type!");
        }

        return retVal;
    }

    public void set_ReadOnlyState(string interfaceName, bool Value)
    {
        Control cnt = this.FindControl(interfaceName);

        AttributeCollection atts = null;
        if (cnt is TextBox)
        {
            ((TextBox)cnt).ReadOnly = Value;
        }
        else if (cnt is CheckBox)
        {
            atts = ((CheckBox)cnt).InputAttributes;
            if (Value)
            {
                if (atts["disabled"] == null)
                {
                    atts.Add("disabled", "disabled");
                }
            }
            else
            {
                if (atts["disabled"] != null)
                {
                    atts.Remove("disabled");
                }
            }
        }
        else if (cnt is DropDownList || cnt is HtmlSelect)
        {
            if (cnt is DropDownList)
            {
                atts = ((DropDownList)cnt).Attributes;
            }
            else
            {
                atts = ((HtmlSelect)cnt).Attributes;
            }
            if (Value)
            {
                if (atts["disabled"] == null)
                {
                    atts.Add("disabled", "disabled");
                }
            }
            else
            {
                if (atts["disabled"] != null)
                {
                    atts.Remove("disabled");
                }
            }
        }
        else
        {
            throw new Exception("DocCirc: get_ReadOnlyState: Unknown cnt type!");
        }
    }


    public override void Dispose()
    {
        base.Dispose();
    }

    protected void uiAddRouteItem_Click(object sender, EventArgs e)
    {
        int? routePoint = null;
        int routePointNum = int.Parse(uiRoutePoint.SelectedValue);
        if(routePointNum <= _sessionData.DocTypeItem.RoutePoints.Count)
        {
            routePoint = _sessionData.DocTypeItem.RoutePoints[routePointNum - 1].Id;
        }

        RoutePointInterfaceItem addedItem = new RoutePointInterfaceItem(
            routePoint,
            null,
            int.Parse(uiEmployee.SelectedValue),
            routePointNum,
            int.Parse(uiRouteItemType.SelectedValue),
            uiEmployee.Items[uiEmployee.SelectedIndex].Text
            );

        int i = 0;
        bool nextRoutePointFound = false;
        foreach (RoutePointInterfaceItem item in _sessionData.RoutePointInterfaceItems)
        {
            if (item.RoutePointNum > addedItem.RoutePointNum)
            {
                nextRoutePointFound = true;
                break;
            }
            i++;
        }

        if (nextRoutePointFound)
        {
            _sessionData.RoutePointInterfaceItems.Insert(i, addedItem);
        }
        else
        {
            _sessionData.RoutePointInterfaceItems.Add(addedItem);
        }
        uiRoute.DataSource = _sessionData.RoutePointInterfaceItems;
        uiRoute.DataBind();
        UpdateRoutePointDropDown();

        uiRouteItemType.SelectedIndex = 0;
    }

    protected void uiRemove_OnCheckedChanged(object sender, EventArgs e)
    {
        RepeaterItem item = (RepeaterItem)((Control)sender).Parent;
        _sessionData.RoutePointInterfaceItems.RemoveAt(item.ItemIndex);
        uiRoute.DataSource = _sessionData.RoutePointInterfaceItems;
        uiRoute.DataBind();
        UpdateRoutePointDropDown();
    }
}
