//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Collections.Generic;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Resources;
using Evpo.DocCirc;
using Evpo.DocCirc.UI;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.Biz;

public partial class FolderMgmt : System.Web.UI.UserControl, IParameterReceiver
{
    #region Resources
    private const string _controlName = "FolderMgmt.ascx";
    public struct ResKeys
    {
        public const string NewFolderButton = "NewFolderButton";
        public const string NewFolderName = "NewFolderName";
        public const string NewFolderNameLabel = "NewFolderNameLabel";
        public const string EditInGrid = "EditInGrid";
        public const string DeleteInGrid = "DeleteInGrid";
        public const string UpdateInGrid = "UpdateInGrid";
        public const string CancelInGrid = "CancelInGrid";
        public const string PermissionsInGrid = "PermissionsInGrid";
        public const string RootFolderPermissions = "RootFolderPermissions";
        public const string FolderHasChildFoldersError = "FolderHasChildFoldersError";

    }
    protected ResourceManager _rm;
    #endregion

    protected struct ControlConst
    {
        public const string FolderTypeParameterName = "FolderType";
        public const string DocTypeFolder = "DocTypeFolder";
        public const string DocumentFolder = "DocumentFolder";
        public const string PermissionMgmtUrl = "PermissionMgmt.aspx?ID={0}&ItemType={1}";
        public const int GridCommandFieldIndex = 2;

    }

    protected class FolderMgmtSessionData
    {
        public DcFolder RootFolder;
        public int FolderType;
        public List<FolderItem> FolderItems;
        
    }

    protected FolderMgmtSessionData _sessionData = new FolderMgmtSessionData();

    

    protected class FolderTreeNode : TreeNode
    {
        private DcFolder _folder;

        public DcFolder Folder
        {
            get { return _folder; }
            set { _folder = value; }
        }

        public FolderTreeNode(string text, DcFolder folder, int orderNum)
            :base(text, orderNum.ToString())
        {
            _folder = folder;
            for (int i = 0; i < _folder.Folders.Length; i++)
            {
                DcFolder fld = _folder.Folders[i];
                this.ChildNodes.Add(new FolderTreeNode(fld.Name, fld, i));
            }
        }
    }


    protected UserSession _us;

    protected string GenerateNewGuid()
    {
        System.Guid guid = System.Guid.NewGuid();
        return guid.ToString();
    }

    protected void Page_Init(object sender, EventArgs e)
    {
        _rm = ResourceHelper.Current.GetResourceManagerForPage(_controlName);
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        _us = (UserSession)Session[SessionStateKeys.UserSession];
        if (!IsPostBack)
        {
            if (_us.UserInfo.UserIsAdministrator())
            {
                FolderData fd = new FolderData(_us.UserInfo, _us.DataConnection);
                _sessionData.RootFolder = fd.GetFolderHierarchy(_sessionData.FolderType);
                string nameResKey = _sessionData.RootFolder.Name.Replace("^", "");
                FolderTreeNode node = new FolderTreeNode(
                    ResourceHelper.StrForUI[nameResKey], 
                    _sessionData.RootFolder, 
                    0
                    );
                uiDirectoryTree.Nodes.Add(node);
                node.Select();
                uiDirectoryTree.CollapseAll();
                uiDirectoryTree.Nodes[0].Expand();

                LocalizeFolderGrid();
                LoadFolderItems();
                UpdateFolderGrid();
                DataBind();
                
                
                hidSessionId.Value = GenerateNewGuid();
                Session[hidSessionId.Value] = _sessionData;
            }
            else
            {
                Response.Redirect("AccessDenied.aspx");
            }

        }
        else
        {
            _sessionData = (FolderMgmtSessionData)Session[hidSessionId.Value];
        }
        
    }

    private void LocalizeFolderGrid()
    {
        int i = ControlConst.GridCommandFieldIndex; 
        ((CommandField)uiFolderGrid.Columns[i]).EditText = _rm.GetString(ResKeys.EditInGrid);
        ((CommandField)uiFolderGrid.Columns[i]).DeleteText = _rm.GetString(ResKeys.DeleteInGrid);
        ((CommandField)uiFolderGrid.Columns[i]).UpdateText = _rm.GetString(ResKeys.UpdateInGrid);
        ((CommandField)uiFolderGrid.Columns[i]).CancelText = _rm.GetString(ResKeys.CancelInGrid);
    }

    public override void DataBind()
    {
        foreach (Control cnt in Controls)
        {
            if (!object.ReferenceEquals(cnt, uiFolderGrid))
            {
                cnt.DataBind();
            }
        }
    }

    protected DcFolder GetFolderByPath(string idPath)
    {
        string[] pathIds = idPath.Split(new char[] { '/' });
        DcFolder folder = _sessionData.RootFolder;
        for (int i = 1; i < pathIds.Length; i++)
        {
            folder = folder.Folders[int.Parse(pathIds[i])];
        }
        return folder;
    }

    private void LoadFolderItems()
    {
        DcFolder folder = GetFolderByPath(uiDirectoryTree.SelectedNode.ValuePath);
        FolderData fd = new FolderData(_us.UserInfo, _us.DataConnection);
        FolderItem[] items = fd.GetFolderItemsInFolder(folder.Id);
        _sessionData.FolderItems = new List<FolderItem>(items.Length);
        foreach (FolderItem item in items)
        {
            _sessionData.FolderItems.Add(item);
        }
    }

    protected void UpdateFolderGrid()
    {
        uiFolderGrid.DataSource = _sessionData.FolderItems;
        uiFolderGrid.DataBind();
    }

    protected string GetPermissionMgmtUrl()
    {
        int rootFolder = DBConstKeeper.Current.RootFolder.Document;
        if (_sessionData.FolderType == DBConstKeeper.Current.FolderType.DocType)
        {
            rootFolder = DBConstKeeper.Current.RootFolder.DocType;
        }

        return GetPermissionMgmtUrl(rootFolder);
    }

    protected string GetPermissionMgmtUrl(int folderId)
    {
        string folderType = ControlConst.DocumentFolder;
        if (_sessionData.FolderType == DBConstKeeper.Current.FolderType.DocType)
        {
            folderType = ControlConst.DocTypeFolder;
        }

        return String.Format(
            ControlConst.PermissionMgmtUrl,
            folderId.ToString(),
            folderType
            );
    }

    protected void uiDirectoryTree_SelectedNodeChanged(object sender, EventArgs e)
    {
        LoadFolderItems();
        UpdateFolderGrid();
    }

    #region IParameterReceiver Members

    public void SetParameters(MenuItemParameter[] parameters)
    {
        if (parameters.Length > 0 && parameters[0].Name == ControlConst.FolderTypeParameterName)
        {
            if (parameters[0].Value == ControlConst.DocTypeFolder)
            {
                _sessionData.FolderType = DBConstKeeper.Current.FolderType.DocType;
            }
            else if(parameters[0].Value == ControlConst.DocumentFolder)
            {
                _sessionData.FolderType = DBConstKeeper.Current.FolderType.Document;
            }
        }
    }

    #endregion

    protected void AddNewFolder()
    {
        DcFolder folder = GetFolderByPath(uiDirectoryTree.SelectedNode.ValuePath);
        FolderItem fi = 
            new FolderItem(folder.Id, uiNewFolderName.Text, _sessionData.FolderType);
        FolderData fd = new FolderData(_us.UserInfo, _us.DataConnection);
        fd.SaveFolderItem(fi);
        DcFolder[] folders = folder.Folders;
        
        folder.Folders = new DcFolder[folders.Length + 1];
        folders.CopyTo(folder.Folders, 0);
        int lastFolderIndex = folder.Folders.Length - 1;
        folder.Folders[lastFolderIndex] = new DcFolder(fi.FolderId.Value, fi.Parent, fi.Name);

        uiDirectoryTree.SelectedNode.ChildNodes.Add(new FolderTreeNode(fi.Name, folder.Folders[lastFolderIndex], lastFolderIndex));
        LoadFolderItems();
        UpdateFolderGrid();

    }

    protected void uiNewFolderButton_Click(object sender, EventArgs e)
    {
        AddNewFolder();
        uiNewFolderName.Text = "";
    }

    protected void uiFolderGrid_RowEditing(object sender, GridViewEditEventArgs e)
    {
        uiFolderGrid.EditIndex = e.NewEditIndex;
        
        UpdateFolderGrid();
    }

    protected void uiFolderGrid_RowDeleting(object sender, GridViewDeleteEventArgs e)
    {
        if (uiDirectoryTree.SelectedNode.ChildNodes[e.RowIndex].ChildNodes.Count > 0 )
        {
            uiErrorMessage.Text = _rm.GetString(ResKeys.FolderHasChildFoldersError);
            uiErrorMessage.Visible = true;
        }
        else
        {

            FolderData fd = new FolderData(_us.UserInfo, _us.DataConnection);
            fd.DeleteFolder(_sessionData.FolderItems[e.RowIndex].FolderId.Value); // in database

            _sessionData.FolderItems.RemoveAt(e.RowIndex); // in folder items

            uiDirectoryTree.SelectedNode.ChildNodes.RemoveAt(e.RowIndex);

            UpdateFolderGrid();
        }
        
    }

    protected void uiFolderGrid_RowUpdating(object sender, GridViewUpdateEventArgs e)
    {
        TextBox tb = (TextBox)uiFolderGrid.Rows[e.RowIndex].FindControl("uiFolderNameEdit");

        // update in FolderItems
        _sessionData.FolderItems[e.RowIndex].Name = tb.Text;
        uiFolderGrid.EditIndex = -1;

        // update in DataBase
        FolderData fd = new FolderData(_us.UserInfo, _us.DataConnection);
        fd.SaveFolderItem(_sessionData.FolderItems[e.RowIndex]);

        //update in tree
        TreeNode node = 
            uiDirectoryTree.FindNode(String.Format("{0}/{1}", uiDirectoryTree.SelectedNode.ValuePath, e.RowIndex));
        node.Text = tb.Text;

        
        UpdateFolderGrid();
    }

    protected void uiFolderGrid_RowCancelingEdit(object sender, GridViewCancelEditEventArgs e)
    {
        uiFolderGrid.EditIndex = -1;
        UpdateFolderGrid();
    }

    protected void uiFolderGrid_RowUpdated(object sender, GridViewUpdatedEventArgs e)
    {
        
    }
}
