//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Resources;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.Biz;
using System.Collections.Generic;
using Evpo.DocCirc.MySqlDL;
using System.IO;
using System.Collections.Specialized;
using System.Text;

public partial class Install : System.Web.UI.Page
{
    #region Resources
    private const string _controlName = "Install.aspx";
    public struct ResKeys
    {
        public const string Title = "Title";
        public const string Introduction = "Introduction";
        public const string MySqlServerMachineName = "MySqlServerMachineName";
        public const string MySqlServerMachineNameComment = "MySqlServerMachineNameComment";
        public const string MySqlPort = "MySqlPort";
        public const string MySqlPortComment = "MySqlPortComment";
        public const string DatabaseAdministratorAccount = "DatabaseAdministratorAccount";
        public const string DatabaseAdministratorAccountComment = "DatabaseAdministratorAccountComment";
        public const string DatabaseAdministratorPassword = "DatabaseAdministratorPassword";
        public const string ConnectionError = "ConnectionError";
        public const string ExceptionMessage = "ExceptionMessage";
        public const string DatabaseConnectionStep = "DatabaseConnectionStep";
        public const string CreateNewDatabase = "CreateNewDatabase";
        public const string CreateDatabaseIntroduction = "CreateDatabaseIntroduction";
        public const string DatabaseNameLabel = "DatabaseNameLabel";
        public const string CreateNewDatabaseComment = "CreateNewDatabaseComment";
        public const string CreateNewDatabaseCheckBox = "CreateNewDatabaseCheckBox";
        public const string DatabaseCreationError = "DatabaseCreationError";
        public const string CreateUserIntroduction = "CreateUserIntroduction";
        public const string ServiceUserLabel = "ServiceUserLabel";
        public const string ServiceUserPasswordLabel = "ServiceUserPasswordLabel";
        public const string ApplicationUserLabel = "ApplicationUserLabel";
        public const string ApplicationUserPasswordLabel = "ApplicationUserPasswordLabel";
        public const string ServiceUserComment = "ServiceUserComment";
        public const string ApplicationUserComment = "ApplicationUserComment";
        public const string UserCreationError = "UserCreationError";
        public const string ServiceUserPasswordCannotBeEmpty = "ServiceUserPasswordCannotBeEmpty";
        public const string ApplicationUserPasswordCannotBeEmpty = "ApplicationUserPasswordCannotBeEmpty";
        public const string AttachmentFolderIntroduction = "AttachmentFolderIntroduction";
        public const string AttachmentFolderLabel = "AttachmentFolderLabel";
        public const string AttachmentFolderComment = "AttachmentFolderComment";
        public const string FolderCreationError = "FolderCreationError";

        public const string NotificationSenderIntroduction = "NotificationSenderIntroduction";
        public const string EmailNotificationEnabledCheckBox = "EmailNotificationEnabledCheckBox";
        public const string EmailNotificationEnabledComment = "EmaiNotificationEnabledComment";
        public const string SmtpServerLabel = "SmtpServerLabel";
        public const string SmtpServerComment = "SmtpServerComment";
        public const string SmtpAccountLabel = "SmtpAccountLabel";
        public const string SmtpPasswordLabel = "SmtpPasswordLabel";
        public const string SmtpSSLEnabledCheckBox = "SmtpSSLEnabledCheckBox";
        public const string SmtpAccountComment = "SmtpAccountComment";

        public const string PreviousWizardButton = "PreviousWizardButton";
        public const string NextWizardButton = "NextWizardButton";
        public const string FinishWizardButton = "FinishWizardButton";

        public const string SmtpServerIsEmpty = "SmtpServerIsEmpty";
        public const string SmtpAccountIsEmpty = "SmtpAccountIsEmpty";
        public const string SmtpPasswordIsEmpty = "SmtpPasswordIsEmpty";
        public const string SmtpPasswordsAreNotEqual = "SmtpPasswordsAreNotEqual";

        public const string CannotSaveSettings = "CannotSaveSettings";

        public const string CommentBeforeSave = "CommentBeforeSave";
        public const string ConfirmPassword = "ConfirmPassword";

    }
    protected ResourceManager _rm;
    #endregion

    protected struct ControlConst
    {
    }

    protected enum WizardSteps : int
    {
        DatabaseServerConnection = 0,
        DatabaseCreation = 1,
        DatabaseUserCreation = 2,
        AttachmentFolderCreation = 3,
        EmailNotificationSetup = 4
    }

    protected class InstallSessionData
    {
        public DatabaseInstaller DatabaseInstaller = new DatabaseInstaller();
        public string DatabaseAdministratorPassword = "";
        public string InitialConnectionString = "";
        public string ServiceUserPassword = "";
        public string ApplicationUserPassword = "";
        public string AttachmentFolderFullPath = "";
        public string CurrentFolder = "";
        public string SmtpPassword = "";
    }
    protected InstallSessionData _sessionData = new InstallSessionData();
    
    protected UserSession _us = null;

    protected override void InitializeCulture()
    {
        base.InitializeCulture();
        _rm = ResourceHelper.Current.GetResourceManagerForPage(_controlName);
    }

    protected string GenerateNewGuid()
    {
        System.Guid guid = System.Guid.NewGuid();
        return guid.ToString();
    }
    
    protected void Page_Load(object sender, EventArgs e)
    {
        _us = (UserSession)Session[SessionStateKeys.UserSession];
        if (!IsPostBack)
        {
            hidSessionId.Value = GenerateNewGuid();
            Session[hidSessionId.Value] = _sessionData;

            _sessionData.CurrentFolder = this.Context.Request.MapPath(this.Context.Request.ApplicationPath);
            SetDefaultAttachmentFolder();
            SetAttachmentFolderComment();
            SetCommentBeforeSave();

            this.DataBind();
        }
        else
        {
            _sessionData = (InstallSessionData)Session[hidSessionId.Value];
        }
    }

    private void SetAttachmentFolderComment()
    {
        uiAttachmentFolderComment.Text = String.Format(
            _rm.GetString(ResKeys.AttachmentFolderComment),
            GetCurrentUserName()
            );
    }

    private string GetCurrentUserName()
    {
        return System.Security.Principal.WindowsIdentity.GetCurrent().Name;
    }

    private void SetCommentBeforeSave()
    {
        uiCommentBeforeSave.Text = String.Format(
            _rm.GetString(ResKeys.CommentBeforeSave),
            DCAppSettings.Current.SettingFile,
            GetCurrentUserName()
            );
    }

    protected void SetDefaultAttachmentFolder()
    {
        string defaultAttachmentFolder = 
            Path.Combine(_sessionData.CurrentFolder, "..");
        defaultAttachmentFolder = 
            Path.Combine(defaultAttachmentFolder, DatabaseInstaller.Const.AttachmentDirectory);
        uiAttachmentFolder.Text = Path.GetFullPath(defaultAttachmentFolder);
        
        //string defaultAttachmentFolder = _sessionData.CurrentFolder.Substring(
        //        0,
        //        _sessionData.CurrentFolder.LastIndexOf(Path.DirectorySeparatorChar)
        //        );
        //defaultAttachmentFolder = Path.Combine(defaultAttachmentFolder, DatabaseInstaller.Const.AttachmentDirectory);
        
    }

    protected void Wizard1_NextButtonClick(object sender, WizardNavigationEventArgs e)
    {
        if (e.CurrentStepIndex == (int)WizardSteps.DatabaseServerConnection)
        {
            _sessionData.DatabaseAdministratorPassword = uiDatabaseAdministratorPassword.Text;
            _sessionData.InitialConnectionString = _sessionData.DatabaseInstaller.GetConnectionString(
                "mysql",
                uiMySqlMachineName.Text,
                uiDatabaseAdministrator.Text,
                _sessionData.DatabaseAdministratorPassword,
                DatabaseInstaller.Const.ConnectionTimeOut,
                uiMySqlPort.Text
                );

            bool result = _sessionData.DatabaseInstaller.CheckConnection(_sessionData.InitialConnectionString);
            if (!result)
            {
                ShowError(_rm.GetString(ResKeys.ConnectionError));
                e.Cancel = true;
            }
        }
        else if (e.CurrentStepIndex == (int)WizardSteps.DatabaseCreation)
        {
            if (uiCreateDatabase.Checked)
            {
                bool result = _sessionData.DatabaseInstaller.CreateDatabase(
                    uiDatabaseName.Text,
                    _sessionData.InitialConnectionString
                    );

                if (!result)
                {
                    ShowError(_rm.GetString(ResKeys.DatabaseCreationError));
                    e.Cancel = true;
                }
            }
        }
        else if (e.CurrentStepIndex == (int)WizardSteps.DatabaseUserCreation)
        {
            if (uiApplicationUserPassword.Text == String.Empty)
            {
                uiError.Text = _rm.GetString(ResKeys.ApplicationUserPasswordCannotBeEmpty);
                uiError.Visible = true;
                e.Cancel = true;
            }
            
            if (uiServiceUserPassword.Text == String.Empty)
            {
                if (uiError.Text != String.Empty)
                {
                    uiError.Text += "<br/>";
                }

                uiError.Text += _rm.GetString(ResKeys.ServiceUserPasswordCannotBeEmpty);
                uiError.Visible = true;
                e.Cancel = true;
            }

            if(!e.Cancel)
            {

                bool isLocalhost = (uiMySqlMachineName.Text.ToUpper().Trim() == "LOCALHOST");

                _sessionData.ServiceUserPassword = uiServiceUserPassword.Text;
                _sessionData.ApplicationUserPassword = uiApplicationUserPassword.Text;

                bool result = _sessionData.DatabaseInstaller.CreateUsers(
                    _sessionData.InitialConnectionString,
                    uiDatabaseName.Text,
                    uiApplicationUser.Text,
                    _sessionData.ApplicationUserPassword,
                    uiServiceUser.Text,
                    _sessionData.ServiceUserPassword,
                    isLocalhost
                    );

                if (!result)
                {
                    ShowError(_rm.GetString(ResKeys.UserCreationError));
                    e.Cancel = true;
                }
            }
        }
        else if (e.CurrentStepIndex == (int)WizardSteps.AttachmentFolderCreation)
        {
            if(Path.IsPathRooted(uiAttachmentFolder.Text))
            {
                _sessionData.AttachmentFolderFullPath = uiAttachmentFolder.Text;
            }
            else
            {
                _sessionData.AttachmentFolderFullPath = Path.Combine(
                    this.Context.Request.MapPath(this.Context.Request.ApplicationPath),
                    uiAttachmentFolder.Text
                    );
            }

            bool result = _sessionData.DatabaseInstaller.CreateAndCheckAttachmentDirectory(_sessionData.AttachmentFolderFullPath);
            if (!result)
            {
                ShowError(_rm.GetString(ResKeys.FolderCreationError));
                e.Cancel = true;
            }
        }
        else if (e.CurrentStepIndex == (int)WizardSteps.EmailNotificationSetup)
        {
            if (uiEmailNotificationEnabledCheckBox.Checked)
            {
                StringBuilder errorText = new StringBuilder();

                if (uiSmtpServer.Text == String.Empty)
                {
                    errorText.AppendFormat("{0}<br/>", (_rm.GetString(ResKeys.SmtpServerIsEmpty)));
                }

                if (uiSmtpAccount.Text == String.Empty)
                {
                    errorText.AppendFormat("{0}<br/>", (_rm.GetString(ResKeys.SmtpAccountIsEmpty)));
                }

                if (uiSmtpPassword.Text == String.Empty)
                {
                    errorText.AppendFormat("{0}<br/>", (_rm.GetString(ResKeys.SmtpPasswordIsEmpty)));
                }

                if (uiSmtpPassword.Text != uiConfirmSmtpPassword.Text)
                {
                    errorText.AppendFormat("{0}<br/>", (_rm.GetString(ResKeys.SmtpPasswordsAreNotEqual)));
                }

                _sessionData.SmtpPassword = uiSmtpPassword.Text;

                if (errorText.Length > 0)
                {
                    e.Cancel = true;
                    uiError.Visible = true;
                    uiError.Text = errorText.ToString();
                }
            }
        }
    }

    protected void Wizard1_FinishButtonClick(object sender, WizardNavigationEventArgs e)
    {
        bool result = SaveSettings();
        if (result)
        {
            ResetApplication();
        }
        else
        {
            e.Cancel = true;
        }
    }

    private void ResetApplication()
    {
        Application[AppStateKeys.IsSetupNeeded] = false;

        DCAppSettings.Current.LoadSettings(_sessionData.CurrentFolder);

        Dictionary<string, object> appStateDict = (new BizGlobal()).Init(
            this.Server.MachineName,
            this.Context.Request.ApplicationPath,
            this.Context.Request.MapPath(this.Context.Request.ApplicationPath),
            this.Context.Request.Url.AbsoluteUri,
            this.Context.Request.Url.Port
            );
        
        foreach (string key in appStateDict.Keys)
        {
            Application[key] = appStateDict[key];
        }

        Response.Redirect("LoginForUser.aspx");
    }
    
    /// <summary>
    /// 
    /// </summary>
    /// <returns> false if operation is not succesfull </returns>
    private bool SaveSettings()
    {

        StringBuilder errorText = new StringBuilder();

        string settingFilePath = DCAppSettings.Current.SettingFile;

        if (!File.Exists(settingFilePath))
        {
            try
            {
                using (FileStream fs = File.Create(settingFilePath))
                {
                    fs.Close();
                }
            }
            catch(Exception ex)
            {
                errorText.AppendFormat("{0} <br/>", ex.Message);
            }
        }

        // was here
        AppSettingOperator op = new AppSettingOperator();
            NameValueCollection nameValues = op.Load(settingFilePath);

            nameValues[SettingFileKeys.LogFile] = Path.Combine(_sessionData.AttachmentFolderFullPath, DatabaseInstaller.Const.LogFileName);
            nameValues[SettingFileKeys.ConnectionString] = _sessionData.DatabaseInstaller.GetConnectionString(
                uiDatabaseName.Text,
                uiMySqlMachineName.Text,
                uiApplicationUser.Text,
                _sessionData.ApplicationUserPassword,
                DatabaseInstaller.Const.ConnectionTimeOut,
                uiMySqlPort.Text
                );

            nameValues[SettingFileKeys.ServiceConnectionString] = _sessionData.DatabaseInstaller.GetConnectionString(
                uiDatabaseName.Text,
                uiMySqlMachineName.Text,
                uiServiceUser.Text,
                _sessionData.ServiceUserPassword,
                DatabaseInstaller.Const.ConnectionTimeOut,
                uiMySqlPort.Text
                );

            nameValues[SettingFileKeys.AttachmentFolder] = _sessionData.AttachmentFolderFullPath;
            
            nameValues[SettingFileKeys.LogFile] = Path.Combine(
                _sessionData.AttachmentFolderFullPath,
                DatabaseInstaller.Const.LogFileName
                );

            nameValues[SettingFileKeys.EmailMessagingEnabled] = uiEmailNotificationEnabledCheckBox.Checked ? "true" : "false";
            nameValues[SettingFileKeys.EmailMessagingServer] = uiSmtpServer.Text;
            nameValues[SettingFileKeys.EmailMessagingAccount] = uiSmtpAccount.Text;
            nameValues[SettingFileKeys.EmailMessagingPassword] = _sessionData.SmtpPassword;
            nameValues[SettingFileKeys.EmailMessagingSSLEnabled] = uiSmtpSSLEnabled.Checked ? "true" : "false";
            nameValues[SettingFileKeys.ServiceUserLogin] = DefaultSettingValues.ServiceUserLogin;

            op.Save(settingFilePath, nameValues);
            try
            {

        }
        catch (Exception ex)
        {
            errorText.AppendFormat("{0}<br/>", ex.Message);
        }

        bool retVal = true;
        if (errorText.Length > 0)
        {
            uiError.Text = _rm.GetString(ResKeys.CannotSaveSettings);
            uiError.Visible = true;
            uiExceptionMessage.Visible = true;
            uiExceptionMessage.Text = errorText.ToString();
            retVal = false;
        }

        return retVal;
    }

    private void ShowError(string errorText)
    {
        uiError.Text = errorText;
        uiError.Visible = true;
        uiExceptionMessage.Visible = true;
        uiExceptionMessage.Text = String.Format(
            _rm.GetString(ResKeys.ExceptionMessage),
            _sessionData.DatabaseInstaller.DatabaseError
            );
    }
    
    
}
