//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Resources;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.Biz;
using Evpo.DocCirc.UI;
using Evpo.DocCirc;

public partial class Institution : System.Web.UI.Page
{

    #region Resources
    private const string _controlName = "Institution.aspx";

    public struct ResKeys
    {
        public const string PageTitle = "PageTitle";
        public const string NameField = "NameField";
        public const string Phone1Field = "Phone1Field";
        public const string Phone2Field = "Phone2Field";
        public const string Address1Field = "Address1Field";
        public const string Address2Field = "Address2Field";
        public const string Email1Field = "Email1Field";
        public const string Email2Field = "Email2Field";
        public const string WebSiteField = "WebSiteField";
        public const string NoteField = "NoteField";
        public const string SaveAndClose = "SaveAndClose";
        public const string Cancel = "Cancel";
        public const string NameIsRequired = "NameIsRequired";
    }
    protected ResourceManager _rm;
    #endregion

    protected enum BindingDirection
    {
        BindToControls,
        BindToData
    }

    protected struct PageParameter
    {
        public const string Cmd = "cmd";
        public const string Id = "id";
    }

    protected struct PageParameterCmd
    {
        public const string New = "new";
    }

    protected class InstitutionSessionData
    {
        public InstitutionItem InstitutionItem;

    }

    protected InstitutionSessionData _sessionData = new InstitutionSessionData();

    protected UserSession _us;

    protected string GenerateNewGuid()
    {
        System.Guid guid = System.Guid.NewGuid();
        return guid.ToString();
    }

    protected override void InitializeCulture()
    {
        _rm = ResourceHelper.Current.GetResourceManagerForPage(_controlName);
        base.InitializeCulture();
    }

    protected InstitutionItem CreateNewInstitutionItem()
    {
        return new InstitutionItem(_us.UserInfo.UserId, "", "", "", "", "", "", "", "", "");
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        _us = (UserSession)Session[SessionStateKeys.UserSession];

        int? id = null;
        bool isNew = false;
        if (
                   this.Request.Params[PageParameter.Cmd] != null &&
                   this.Request.Params[PageParameter.Cmd] == PageParameterCmd.New)
        {
            isNew = true;
        }
        else
        {
            id = int.Parse(this.Request.Params[PageParameter.Id]);
        }

        if (!IsPostBack)
        {
            hidSessionId.Value = GenerateNewGuid();

            if (isNew)
            {
                _sessionData.InstitutionItem = CreateNewInstitutionItem();
            }
            else
            {
                InstitutionData instData = new InstitutionData(_us.UserInfo, _us.DataConnection);
                _sessionData.InstitutionItem = instData.LoadInstitutionItem(id.Value);
            }

            this.Title = String.Format(ResKeys.PageTitle, _sessionData.InstitutionItem.Name);

            BindControls(BindingDirection.BindToControls);
            this.DataBind();
            Session[hidSessionId.Value] = _sessionData;
        }
        else
        {
            _sessionData = (InstitutionSessionData)Session[hidSessionId.Value];
            BindControls(BindingDirection.BindToData);
        }
    }

    private void BindControls(BindingDirection bindingDirection)
    {
        if (bindingDirection == BindingDirection.BindToControls)
        {
            uiName.Text = _sessionData.InstitutionItem.Name;
            uiPhone1.Text = _sessionData.InstitutionItem.Phone1;
            uiPhone2.Text = _sessionData.InstitutionItem.Phone2;
            uiAddress1.Text = _sessionData.InstitutionItem.Address1;
            uiAddress2.Text = _sessionData.InstitutionItem.Address2;
            uiEmail1.Text = _sessionData.InstitutionItem.Email1;
            uiEmail2.Text = _sessionData.InstitutionItem.Email2;
            uiWebSite.Text = _sessionData.InstitutionItem.WebSite;
            uiNote.Text = _sessionData.InstitutionItem.Note;
        }
        else
        {
            _sessionData.InstitutionItem.Name = uiName.Text;
            _sessionData.InstitutionItem.Phone1 = uiPhone1.Text;
            _sessionData.InstitutionItem.Phone2 = uiPhone2.Text;
            _sessionData.InstitutionItem.Address1 = uiAddress1.Text;
            _sessionData.InstitutionItem.Address2 = uiAddress2.Text;
            _sessionData.InstitutionItem.Email1 = uiEmail1.Text;
            _sessionData.InstitutionItem.Email2 = uiEmail2.Text;
            _sessionData.InstitutionItem.WebSite = uiWebSite.Text;
            _sessionData.InstitutionItem.Note = uiNote.Text;
        }
    }

    protected void uiSave_Click(object sender, EventArgs e)
    {
        InstitutionData instData = new InstitutionData(_us.UserInfo, _us.DataConnection);
        instData.SaveInstituion(_sessionData.InstitutionItem);
        this.litScript.Text += "<script language='javascript'>window.close();</script>";
    }
}
