//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Resources;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.UI;
using Evpo.DocCirc.Biz;
using Evpo.DocCirc;

public partial class InstitutionList : System.Web.UI.UserControl
{
    #region Resources
    private const string _controlName = "InstitutionList.ascx";
    public struct ResKeys
    {
        public const string CreateInstitution = "CreateInstitution";
        public const string RefreshList = "RefreshList";
        public const string DeleteChecked = "DeleteChecked";
        public const string ListHeaderColumnName = "ListHeaderColumnName";
        public const string InstitutionsTotalCount = "InstitutionsTotalCount";

    }
    protected ResourceManager _rm;
    #endregion

    protected struct ControlConst
    {
        public const int ItemsOnPage = 25;
        public const string InstitutionItemUrl = "Institution.aspx?Id={0}";
        public const string NewInstitutionItemUrl = "Institution.aspx?cmd=new";
    }

    protected class InstitutionListSessionData
    {
        public InstitutionToken[] InstitutionTokens;
        public int FirstVisibleToken = 0;
    }

    protected InstitutionListSessionData _sessionData = new InstitutionListSessionData();

    protected UserSession _us;

    protected void Page_Init(object sender, EventArgs e)
    {
        _rm = ResourceHelper.Current.GetResourceManagerForPage(_controlName);
    }

    protected string GenerateNewGuid()
    {
        System.Guid guid = System.Guid.NewGuid();
        return guid.ToString();
    }

    protected void ResetInstitutionList()
    {
        InstitutionData instData = new InstitutionData(_us.UserInfo, _us.DataConnection);
        _sessionData.InstitutionTokens = instData.GetAllInstitutionTokents();
        uiInstitutionQuantity.Text = _sessionData.InstitutionTokens.Length.ToString();
        _sessionData.FirstVisibleToken = 0;
        ShowInstituions();
    }

    protected void ShowInstituions()
    {
        if (_sessionData.InstitutionTokens.Length > 0)
        {
            int lastVisibleInstitution =
                _sessionData.FirstVisibleToken + ControlConst.ItemsOnPage - 1;

            bool lastPage = false;

            if (lastVisibleInstitution >= _sessionData.InstitutionTokens.Length - 1)
            {
                lastVisibleInstitution = _sessionData.InstitutionTokens.Length - 1;
                lastPage = true;
            }

            InstitutionToken[] visibleTokens =
                new InstitutionToken[lastVisibleInstitution - _sessionData.FirstVisibleToken + 1];
            Array.Copy(
                _sessionData.InstitutionTokens,
                _sessionData.FirstVisibleToken,
                visibleTokens,
                0,
                lastVisibleInstitution - _sessionData.FirstVisibleToken + 1
                );
            uiList.Visible = true;
            uiList.DataSource = visibleTokens;
            uiList.DataBind();

            uiInstitutionSpan.Text = String.Format(
                "{0}...{1}",
                _sessionData.FirstVisibleToken + 1,
                lastVisibleInstitution + 1
                );

            uiPrevPage.Visible = (_sessionData.FirstVisibleToken > 0);
            uiNextPage.Visible = !lastPage;
        }
        else
        {
            uiInstitutionSpan.Visible = false;
            uiPrevPage.Visible = false;
            uiNextPage.Visible = false;
        }
    }

    public override void DataBind()
    {
        foreach (Control cnt in Controls)
        {
            if (cnt.ID != "uiList")
            {
                cnt.DataBind();
            }
        }
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        _us = (UserSession)Session[SessionStateKeys.UserSession];
        if (!IsPostBack)
        {
            hidSessionId.Value = GenerateNewGuid();
            ResetInstitutionList();
            Session[hidSessionId.Value] = _sessionData;
        }
        else
        {
            _sessionData = (InstitutionListSessionData)Session[hidSessionId.Value];
        }

        this.DataBind();
    }
    protected void uiPrevPage_Click(object sender, EventArgs e)
    {
        _sessionData.FirstVisibleToken -= ControlConst.ItemsOnPage;
        ShowInstituions();
    }


    protected void uiNextPage_Click(object sender, EventArgs e)
    {
        _sessionData.FirstVisibleToken += ControlConst.ItemsOnPage;
        ShowInstituions();
    }

    protected void uiRefresh_Click(object sender, EventArgs e)
    {
        ResetInstitutionList();
    }
    protected void uiDeleteChecked_Click(object sender, EventArgs e)
    {
        InstitutionData instData = new InstitutionData(_us.UserInfo, _us.DataConnection);
        int i = 0;
        foreach (DataListItem listItem in uiList.Items)
        {
            CheckBox checkBox = (CheckBox)listItem.FindControl("uiDeleteMark");
            if (checkBox.Checked)
            {
                instData.DeleteInstitution(_sessionData.InstitutionTokens[i].Id);
            }
            i++;
        }

        ResetInstitutionList();
    }
}
