//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Resources;
using Evpo.DocCirc;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.Biz;
using Evpo.DocCirc.UI;

public partial class LoginForUser : System.Web.UI.Page
{
    public struct ResKeys
    {
        public const string LoginFormHeaderCaption = "LoginFormHeaderCaption";
        public const string User = "User";
        public const string Password = "Password";
        public const string LoginButton = "LoginButton";
        public const string InvalidPasswordOrUsername = "InvalidPasswordOrUsername";
        public const string SavePasswordCheckbox = "SavePasswordCheckbox";

    }

    private const string _controlName = "LoginForUser.aspx";
    protected ResourceManager _rm;

    protected override void InitializeCulture()
    {
        _rm = ResourceHelper.Current.GetResourceManagerForPage(_controlName);
        base.InitializeCulture();
    }

    private void LoadUserOptions()
    {
        SettingKeeper sk = (SettingKeeper)this.Application[AppStateKeys.SettingKeeper];
        DataConnection dc = new DataConnection(sk.ConnectionString);
        dc.Open();
        foreach (UserProperties user in UserHelper.Current.GetAllUserProperties(dc))
        {
            uiUser.Items.Add(new ListItem(user.Login));
        }
        dc.Close();
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        if ((bool)this.Application[AppStateKeys.IsSetupNeeded])
        {
            FormsAuthentication.RedirectFromLoginPage("Install user", false);
        }
        else
        {
            LoadUserOptions();
        }
        //if (Request.IsAuthenticated)
        //{
        //    Response.Redirect("~/DebugStart.aspx");
        //}
        this.DataBind();
    }

    protected void uiLogin_Click(object sender, EventArgs e)
    {
        bool loggedIn = false;
        SettingKeeper sk = (SettingKeeper)this.Application[AppStateKeys.SettingKeeper];
        DataConnection dc = new DataConnection(sk.ConnectionString);
        dc.Open();
        
        UserToken token = UserHelper.Current.AuthorizeUser(
            uiUser.SelectedItem.Text,
            uiPassword.Text,
            dc
            );
        
        if (token != null)
        {
            UserSession session = new UserSession(
            token,
            dc,
            sk.ConnectionString,  // Make separate connection string for each user in the future
            (UserInfo)this.Application[AppStateKeys.ServiceUserInfo],
            (SettingKeeper)this.Application[AppStateKeys.SettingKeeper]
            );

            this.Session[SessionStateKeys.UserSession] = session;
            loggedIn = true;
        }

        dc.Close();

        if (loggedIn)
        {
            FormsAuthentication.RedirectFromLoginPage(token.Login, uiSavePassword.Checked);
        }
        else
        {
            uiResultText.Text = _rm.GetString(ResKeys.InvalidPasswordOrUsername);
            uiResultText.Visible = true;
        }


    }
}
