//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Resources;
using System.IO;
using Evpo.DocCirc;
using Evpo.DocCirc.UI;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.Biz;


public partial class Page01 : System.Web.UI.Page //without unit testing
{
    protected const string _menuXsdFileName = "Page01.MainMenu.xsd";
    protected const string _menuXmlFileName = "Page01.MainMenu.xml";
    protected const string _menuIdParameter = "menuId";

    #region Resources
    private const string _controlName = "Page01.aspx";
    public struct ResKeys
    {
        public const string PageTitle = "PageTitle";
        public const string HomeHref = "HomeHref";
        public const string UserNameInscr = "UserNameInscr";
        public const string DepartmentInscr = "DepartmentInscr";
        public const string ChangeUser = "ChangeUser";
    }
    protected ResourceManager _rm;
    protected override void InitializeCulture()
    {
        _rm = ResourceHelper.Current.GetResourceManagerForPage(_controlName);
        base.InitializeCulture();
    }
    #endregion

    protected MenuKeeper _menuKeeper;

    protected string GetMenuItemUrl(string id)
    {
        return String.Format("{0}?{1}={2}", _controlName, _menuIdParameter,  id);;
    }

    protected void SetMenuKeeper()
    {
        _menuKeeper = (MenuKeeper)this.Application[AppStateKeys.Page01MainMenuKeeper];
        if (_menuKeeper == null)
        {
            SettingKeeper sk = (SettingKeeper)this.Application[AppStateKeys.SettingKeeper];
            StreamReader xsdFile = new StreamReader(
                Path.Combine(sk.ApplicationFolder, _menuXsdFileName)
                );
            StreamReader xmlFile = new StreamReader(
                Path.Combine(sk.ApplicationFolder, _menuXmlFileName)
                );
            _menuKeeper = new MenuKeeper(xmlFile, xsdFile);
            xsdFile.Close();
            xmlFile.Close();
            this.Application[AppStateKeys.Page01MainMenuKeeper] = _menuKeeper;
        }
    }

    protected void Page_Init(object sender, EventArgs e)
    {
        UserSession userSession = (UserSession)this.Session[SessionStateKeys.UserSession];
        uiUserName.Text = userSession.UserInfo.DisplayName;
        
        SetMenuKeeper();
        uiMainMenu.DataSource = _menuKeeper.MenuData.MenuItems;
        string menuId = null;
        if (Request.Params[_menuIdParameter] != null)
        {
            menuId = Request.Params[_menuIdParameter];
        }
        else
        {
            menuId = _menuKeeper.MenuData.DefaultMenuId;
        }
        
        DcMenuItem menuItem = _menuKeeper.GetMenuItem(menuId);
        uiCurrentMenu.Text = _rm.GetString(menuItem.NameResKey);
        if (menuItem.MenuItems.Length > 0)
        {
            uiSubMenu.DataSource = menuItem.MenuItems;
            uiSubMenu.Visible = true;
        }
        else
        {
            if (uiContent.Controls.Count > 0)
            {
                uiContent.Controls.Clear();
            }
            Control cnt = Page.LoadControl(menuItem.ControlFile);
            if(menuItem.Parameters.Length > 0)
            {
                IParameterReceiver paramReceiver = cnt as IParameterReceiver;
                if (paramReceiver != null)
                {
                    paramReceiver.SetParameters(menuItem.Parameters);
                }
            }
            uiContent.Controls.Add(cnt);
        }
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        if (!IsPostBack)
        {
            this.DataBind();
        }
    }

    /// <summary>
    /// overriden for not binding the uiContent. Allow it to bind itself
    /// </summary>
    public override void DataBind()
    {
        foreach (Control cnt in this.Controls)
        {
            if (cnt.ID != "uiForm")
            {
                cnt.DataBind();
            }
        }

        foreach (Control cnt in this.Form.Controls)
        {
            if (cnt.ID != "uiContent")
            {
                cnt.DataBind();
            }
        }
    }
}
