//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Collections.Generic;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Resources;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.Biz;
using Evpo.DocCirc.UI;
using Evpo.DocCirc;

public partial class PermissionMgmt : System.Web.UI.Page
{
    #region Resources
    private const string _controlName = "PermissionMgmt.aspx";
    public struct ResKeys
    {
        public const string SelectRole = "SelectRole";
        public const string AddRoleButton = "AddRoleButton";
        public const string SavePermissionButton = "SavePermissionButton";
        public const string CancelButton = "CancelButton";
        public const string RemoveCheckBox = "RemoveCheckBox";
        public const string CreateCheckBox = "CreateCheckBox";
        public const string ReadCheckBox = "ReadCheckBox";
        public const string UpdateCheckBox = "UpdateCheckBox";
        public const string DeleteCheckBox = "DeleteCheckBox";
        public const string BrowseCheckBox = "BrowseCheckBox";
        public const string ReadOwnCheckBox = "ReadOwnCheckBox";
        public const string UpdateOwnCheckBox = "UpdateOwnCheckBox";
        public const string DeleteOwnCheckBox = "DeleteOwnCheckBox";
        public const string Title = "Title";
        public const string RoleDropDownIsEmpty = "RoleDropDownIsEmpty";

    }
    protected ResourceManager _rm;
    #endregion

    protected struct RequestParam
    {
        public const string ItemId = "ID";
        public const string ItemType = "ItemType";
    }

    protected struct ItemTypeOption
    {
        public const string DocType = "DocType";
        public const string DocTypeFolder = "DocTypeFolder";
        public const string DocumentFolder = "DocumentFolder";
    }

    protected class PermissionMgmtSessionData
    {
        public List<PermissionBase> Permissions;
        public int ItemId;
        public string ItemType;

    }

    protected PermissionMgmtSessionData _sessionData;

    protected UserSession _us;

    protected string GenerateNewGuid()
    {
        System.Guid guid = System.Guid.NewGuid();
        return guid.ToString();
    }

    protected override void InitializeCulture()
    {
        base.InitializeCulture();
        _rm = ResourceHelper.Current.GetResourceManagerForPage(_controlName);
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        _us = (UserSession)Session[SessionStateKeys.UserSession];
        
        if (!IsPostBack)
        {
            if (_us.UserInfo.UserIsAdministrator())
            {
                _sessionData = new PermissionMgmtSessionData();
                _sessionData.ItemId = int.Parse(Request.Params[RequestParam.ItemId]);
                _sessionData.ItemType = Request.Params[RequestParam.ItemType];

                

                LoadPermissions();
                FillRoleDropDown();
                UpdatePermissionList();
                hidSessionId.Value = GenerateNewGuid();
                Session[hidSessionId.Value] = _sessionData;

                DataBind();
            }
            else
            {
                Response.Redirect("AccessDenied.aspx");
            }
        }
        else
        {
            _sessionData = (PermissionMgmtSessionData)Session[hidSessionId.Value];
        }

        SetTitle();
    }

    protected void SetTitle()
    {
        string itemName = "";
        if (_sessionData.ItemType == ItemTypeOption.DocType)
        {
            DocTypeData dtd = new DocTypeData(_us.UserInfo, _us.DataConnection);
            bool routePoints = true;
            itemName = dtd.LoadDocTypeItem(_sessionData.ItemId, !routePoints).Name;
        }
        else
        {
            FolderData fd = new FolderData(_us.UserInfo, _us.DataConnection);
            itemName = fd.GetFolderItem(_sessionData.ItemId).Name;
            
        }
        Title = String.Format(_rm.GetString(ResKeys.Title), itemName);
    }

    protected void FillRoleDropDown()
    {
        List<int> addedRoles = new List<int>(_sessionData.Permissions.Count);
        foreach (PermissionBase perm in _sessionData.Permissions)
        {
            addedRoles.Add(perm.Role.RoleId.Value);
        }

        RoleData rd = new RoleData(_us.DataConnection);
        UserRole[] roles = rd.GetAllUserRoles();
        uiRoleDropDown.Items.Clear();
        foreach (UserRole role in roles)
        {
            if (!addedRoles.Contains(role.RoleId.Value))
            {
                uiRoleDropDown.Items.Add(new ListItem(role.RoleName, role.RoleId.ToString()));
            }
        }

        (new ListItemSorter()).SortByText(uiRoleDropDown.Items);
    }

    protected void BindToData()
    {
        for (int i = 0; i < uiPermissionList.Items.Count; i ++ )
        {
            CheckBox cb = (CheckBox)uiPermissionList.Items[i].FindControl("uiCreateRight");
            _sessionData.Permissions[i].SecurityItem.Create = cb.Checked;
            cb = (CheckBox)uiPermissionList.Items[i].FindControl("uiReadRight");
            _sessionData.Permissions[i].SecurityItem.Read = cb.Checked;
            cb = (CheckBox)uiPermissionList.Items[i].FindControl("uiUpdateRight");
            _sessionData.Permissions[i].SecurityItem.Update = cb.Checked;
            cb = (CheckBox)uiPermissionList.Items[i].FindControl("uiDeleteRight");
            _sessionData.Permissions[i].SecurityItem.Delete = cb.Checked;
            cb = (CheckBox)uiPermissionList.Items[i].FindControl("uiBrowseRight");
            _sessionData.Permissions[i].SecurityItem.Browse = cb.Checked;
            cb = (CheckBox)uiPermissionList.Items[i].FindControl("uiReadOwnRight");
            _sessionData.Permissions[i].SecurityItem.ReadOwn = cb.Checked;
            cb = (CheckBox)uiPermissionList.Items[i].FindControl("uiUpdateOwnRight");
            _sessionData.Permissions[i].SecurityItem.UpdateOwn = cb.Checked;
            cb = (CheckBox)uiPermissionList.Items[i].FindControl("uiDeleteOwnRight");
            _sessionData.Permissions[i].SecurityItem.DeleteOwn = cb.Checked;
        }
    }


    public override void DataBind()
    {
        foreach (Control cnt in Controls)
        {
            if (!object.ReferenceEquals(cnt, uiPermissionList))
            {
                cnt.DataBind();
            }
        }
    }

    private void UpdatePermissionList()
    {
        uiPermissionList.DataSource = _sessionData.Permissions;
        uiPermissionList.DataBind();
    }

    protected void LoadPermissions()
    {
        SecurityData sd = new SecurityData(_us.UserInfo, _us.DataConnection);

        PermissionBase[] perms = null;

        if (_sessionData.ItemType == ItemTypeOption.DocType)
        {
            perms = sd.GetDocTypePermissions(_sessionData.ItemId);
        }
        else if (
            _sessionData.ItemType == ItemTypeOption.DocTypeFolder || 
            _sessionData.ItemType == ItemTypeOption.DocumentFolder
            )
        {
            perms = sd.GetFolderPermissions(_sessionData.ItemId);
        }

        _sessionData.Permissions = new List<PermissionBase>();
        foreach (PermissionBase perm in perms)
        {
            _sessionData.Permissions.Add(perm);
        }

    }

    /// <summary>
    /// Factory method
    /// </summary>
    /// <param name="role"></param>
    /// <returns></returns>
    protected PermissionBase GetPermissionBase(UserRole role)
    {
        PermissionBase retVal = null;
        SecurityItem si = new SecurityItem(true, true, true, true, true, true, true, true);
        if (_sessionData.ItemType == ItemTypeOption.DocType)
        {
            si.Browse = false;
            retVal = new DocTypePermission(_sessionData.ItemId, role, si);
        }
        else
        {
            retVal = new FolderPermission(_sessionData.ItemId, role, si);
        }

        return retVal;
    }

    protected void uiRemove_CheckedChanged(object sender, EventArgs e)
    {
        Control cnt = (Control)sender;
        RepeaterItem ri = (RepeaterItem)cnt.Parent;
        UserRole removedRole = _sessionData.Permissions[ri.ItemIndex].Role;
        _sessionData.Permissions.RemoveAt(ri.ItemIndex);
        uiRoleDropDown.Items.Add(new ListItem(removedRole.RoleName, removedRole.RoleId.ToString()));
        (new ListItemSorter()).SortByText(uiRoleDropDown.Items);
        UpdatePermissionList();
    }

    protected void uiAddRole_Click(object sender, EventArgs e)
    {
        if (uiRoleDropDown.Items.Count > 0)
        {
            int i = uiRoleDropDown.SelectedIndex;
            UserRole role = new UserRole(
                int.Parse(uiRoleDropDown.Items[i].Value),
                uiRoleDropDown.Items[i].Text
                );
            _sessionData.Permissions.Add(GetPermissionBase(role));
            uiRoleDropDown.Items.Remove(
                uiRoleDropDown.Items.FindByValue(role.RoleId.ToString())
                );
            (new ListItemSorter()).SortByText(uiRoleDropDown.Items);
            UpdatePermissionList();
        }
        else
        {
            uiErrorMessage.Text = _rm.GetString(ResKeys.RoleDropDownIsEmpty);
            uiErrorMessage.Visible = true;
        }
    }


    protected void uiSavePermission_Click(object sender, EventArgs e)
    {
        BindToData();
        SecurityData sd = new SecurityData(_us.UserInfo, _us.DataConnection);
        if (_sessionData.ItemType == ItemTypeOption.DocType)
        {
            sd.SaveDocTypePermissions(
                _sessionData.ItemId,
                GetDocTypePermissions()
                );
        }
        else
        {
            sd.SaveFolderPermissions(
                _sessionData.ItemId,
                GetFolderPermissions()
                );
        }

        this.ClientScript.RegisterStartupScript(typeof(string), "", "window.close();", true);
    }

    protected DocTypePermission[] GetDocTypePermissions()
    {
        if (_sessionData.ItemType != ItemTypeOption.DocType)
        {
            throw new ApplicationException("Can't cast to DocTypePermission!");
        }

        DocTypePermission[] retVal = new DocTypePermission[_sessionData.Permissions.Count];

        for (int i = 0; i < _sessionData.Permissions.Count; i ++)
        {
            retVal[i] = (DocTypePermission)_sessionData.Permissions[i];
        }
        return retVal;
    }

    protected FolderPermission[] GetFolderPermissions()
    {
        if (_sessionData.ItemType == ItemTypeOption.DocType)
        {
            throw new ApplicationException("Can't cast to FolderPermission!");
        }

        FolderPermission[] retVal = new FolderPermission[_sessionData.Permissions.Count];

        for (int i = 0; i < _sessionData.Permissions.Count; i++)
        {
            retVal[i] = (FolderPermission)_sessionData.Permissions[i];
        }
        return retVal;
    }
}
