//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Collections.Generic;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Resources;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.UI;
using Evpo.DocCirc.Biz;
using Evpo.DocCirc;

public partial class RoleMgmt : System.Web.UI.UserControl
{
    #region Resources
    private const string _controlName = "RoleMgmt.ascx";
    public struct ResKeys
    {
        public const string CreateRole = "CreateRole";
        public const string DeleteRole = "DeleteRole";
        public const string SaveButton = "SaveButton";
        public const string CancelButton = "CancelButton";
        public const string RoleNameLabel = "RoleNameLabel";
        public const string NewUserLabel = "NewUserLabel";
        public const string AddNewUserButton = "AddNewUserButton";
        public const string UserListHeader = "UserListHeader";
        public const string RemoveUser = "RemoveUser";
        public const string SelectRoleLabel = "SelectRoleLabel";
        public const string NewRoleName = "NewRoleName";



        /*
        public const string NewUserLogin = "NewUserFullLogin";
        public const string SelectRoleLabel = "SelectRoleLabel";
        public const string AddUserLabel = "AddUserLabel";
        public const string AddNewUserButton = "AddNewUserButton";
        
        
         * */

    }
    protected ResourceManager _rm;
    #endregion

    protected class RoleMgmtSessionData
    {
        public List<UserRole> Roles;
        public UserRole CurrentRole;
        public List<UserToken> CurrentRoleUsers;
    }

    protected RoleMgmtSessionData _sessionData;

    protected UserSession _us;

    protected string GenerateNewGuid()
    {
        System.Guid guid = System.Guid.NewGuid();
        return guid.ToString();
    }

    protected void Page_Init(object sender, EventArgs e)
    {
        _rm = ResourceHelper.Current.GetResourceManagerForPage(_controlName);
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        _us = (UserSession)Session[SessionStateKeys.UserSession];

        if (!IsPostBack)
        {
            if (_us.UserInfo.UserIsAdministrator())
            {
                _sessionData = new RoleMgmtSessionData();
                RoleData rd = new RoleData(_us.DataConnection);
                UserRole[] roles = rd.GetAllUserRoles();

                _sessionData.Roles = new List<UserRole>(roles.Length);
                foreach (UserRole role in roles)
                {
                    _sessionData.Roles.Add(role);
                }

                BindRoleList();

                DataBind();
                
                FillUserDropDown();
                
                _sessionData.CurrentRoleUsers = new List<UserToken>();
                if (_sessionData.Roles.Count > 0)
                {
                    _sessionData.CurrentRole = _sessionData.Roles[0];
                    uiRoleList.SelectedIndex = 0;
                    LoadCurrentRoleUsers();
                }
                DataToControlBind();

                hidSessionId.Value = GenerateNewGuid();
                Session[hidSessionId.Value] = _sessionData;
            }
            else
            {
                Response.Redirect("AccessDenied.aspx");
            }
        }
        else
        {
            _sessionData = (RoleMgmtSessionData)Session[hidSessionId.Value];

        }
       


    }

    protected void FillUserDropDown()
    {
        UserProperties[] props = UserHelper.Current.GetAllUserProperties(_us.DataConnection);
        List<UserProperties> propList = new List<UserProperties>(props.Length);
        for (int i = 0; i < props.Length; i++)
        {
            propList.Add(props[i]);
        }

        (new UserPropertiesOperator()).SortUserPropertiesListByLogin(propList);

        foreach (UserProperties prop in propList)
        {
            uiUserDropDown.Items.Add(new ListItem(prop.Login, prop.UserId.ToString()));
        }
    }

    private void DataToControlBind()
    {
        uiRoleName.Text = _sessionData.CurrentRole.RoleName;
    }

    private void ControlToDataBind()
    {
        _sessionData.CurrentRole.RoleName = uiRoleName.Text;
    }

    private void LoadCurrentRoleUsers()
    {
        RoleData rd = new RoleData(_us.DataConnection);
        UserToken[] users = rd.GetUserRoleUsers(_sessionData.CurrentRole.RoleId.Value);
        _sessionData.CurrentRoleUsers = new List<UserToken>(users.Length);
        foreach (UserToken user in users)
        {
            _sessionData.CurrentRoleUsers.Add(user);
        }
        UpdateUserList();
    }

    private void BindRoleList()
    {
        uiRoleList.Items.Clear();
        foreach (UserRole role in _sessionData.Roles)
        {
            if (role.RoleId != null)
            {
                uiRoleList.Items.Add(new ListItem(role.RoleName, role.RoleId.ToString()));
            }
            else
            {
                uiRoleList.Items.Add(new ListItem(role.RoleName, GenerateNewGuid()));
            }
        }
    }

    protected void UpdateUserList()
    {
        uiUserList.DataSource = _sessionData.CurrentRoleUsers;
        uiUserList.DataBind();
    }

    protected void uiRoleList_SelectedIndexChanged(object sender, EventArgs e)
    {
        _sessionData.CurrentRole = _sessionData.Roles[uiRoleList.SelectedIndex];
        DataToControlBind();

        uiUserDropDown.SelectedIndex = 0;

        if (_sessionData.CurrentRole.RoleId != null)
        {
            LoadCurrentRoleUsers();
        }
    }

    protected void uiCreate_Click(object sender, EventArgs e)
    {
        UserRole role = new UserRole(
            null,
            _rm.GetString(ResKeys.NewRoleName)
            );
        _sessionData.CurrentRole = role;
        _sessionData.Roles.Add(role);
        _sessionData.CurrentRoleUsers.Clear();

        BindRoleList();
        uiRoleList.SelectedIndex = uiRoleList.Items.Count - 1;
        DataToControlBind();

        UpdateUserList();

    }

    protected void uiDelete_Click(object sender, EventArgs e)
    {
        RoleData rd = new RoleData(_us.DataConnection);
        rd.DeleteUserRole(_sessionData.CurrentRole.RoleId.Value);
        int i = _sessionData.Roles.IndexOf(_sessionData.CurrentRole);
        _sessionData.Roles.RemoveAt(i);

        if (i > 0)
        {
            i--;
        }

        _sessionData.CurrentRole = _sessionData.Roles[i];
        uiRoleList.SelectedIndex = i;
        LoadCurrentRoleUsers();

        BindRoleList();

        DataToControlBind();

        UpdateUserList();
    }

    protected void uiRemoveUser_CheckedChanged(object sender, EventArgs e)
    {
        Control cnt = (Control)sender;
        int i = ((RepeaterItem)cnt.Parent).ItemIndex;
        _sessionData.CurrentRoleUsers.RemoveAt(i);
        UpdateUserList();
    }

    protected void uiSave_Click(object sender, EventArgs e)
    {
        ControlToDataBind();

        bool isNew = false;
        if (_sessionData.CurrentRole.RoleId == null)
        {
            isNew = true;
        }

        RoleData rd = new RoleData(_us.DataConnection);
        rd.SaveUserRole(_sessionData.CurrentRole);

        rd.SetUserRoleUsers(_sessionData.CurrentRole.RoleId.Value, _sessionData.CurrentRoleUsers.ToArray());

        int i = _sessionData.Roles.IndexOf(_sessionData.CurrentRole);
        if (isNew)
        {
            
            uiRoleList.Items[i].Value = _sessionData.CurrentRole.RoleId.ToString();
        }

        BindRoleList();
        uiRoleList.SelectedIndex = i;
    }

    protected void uiCancel_Click(object sender, EventArgs e)
    {
        DataToControlBind();
    }

    protected void uiAddNewUser_Click(object sender, EventArgs e)
    {
        _sessionData.CurrentRoleUsers.Add(
            new UserToken(
                int.Parse(uiUserDropDown.SelectedValue),
                uiUserDropDown.Items[uiUserDropDown.SelectedIndex].Text
                )
            );
        UpdateUserList();
        uiUserDropDown.SelectedIndex = 0;
    }
}
