//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Collections.Generic;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Resources;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.UI;
using Evpo.DocCirc.Biz;
using Evpo.DocCirc;


public partial class UserMgmt : System.Web.UI.UserControl
{
    #region Resources
    private const string _controlName = "UserMgmt.ascx";
    public struct ResKeys
    {
        public const string CreateUser = "CreateUser";
        public const string DeleteUser = "DeleteUser";
        public const string LoginLabel = "LoginLabel";
        public const string FullNameLabel = "FullNameLabel";
        public const string AutomaticPassLabel = "AutomaticPassLabel";
        public const string AllowDeputyNotificationLabel = "AllowDeputyNotificationLabel";
        public const string InstantMessagingLabel = "InstantMessagingLabel";
        public const string ImAddressLabel = "ImAddressLabel";
        public const string ChangePwdNextTimeLabel = "ChangePwdNextTimeLabel";
        public const string ExternalAuthorizationLabel = "ExternalAuthorizationLabel";
        public const string ExternalIdentityLabel = "ExternalIdentityLabel";
        public const string SaveButton = "SaveButton";
        public const string CancelButton = "CancelButton";
        public const string NewUserFullName = "NewUserFullName";
        public const string SelectUserLabel = "SelectUserLabel";
        public const string AddEmailLabel = "AddEmailLabel";
        public const string AddNewEmailButton = "AddNewEmailButton";
        public const string EmailListHeader = "EmailListHeader";
        public const string RemoveEmail = "RemoveEmail";
        public const string ResetPasswordCheckBox = "ResetPasswordCheckBox";
        public const string PasswordConfirmationError = "PasswordConfirmationError";


    }
    protected ResourceManager _rm;
    #endregion

    protected class UserMgmtSessionData
    {
        public List<UserProperties> UserProps;
        public UserProperties CurrentUserProp;
        public List<string> CurrentUserEmails;

    }

    protected UserMgmtSessionData _sessionData;

    protected UserSession _us;

    protected void Page_Init(object sender, EventArgs e)
    {
        _rm = ResourceHelper.Current.GetResourceManagerForPage(_controlName);
    }

    protected string GenerateNewGuid()
    {
        System.Guid guid = System.Guid.NewGuid();
        return guid.ToString();
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        _us = (UserSession)Session[SessionStateKeys.UserSession];



        if (!IsPostBack)
        {
            if (_us.UserInfo.UserIsAdministrator())
            {
                _sessionData = new UserMgmtSessionData();
                UserProperties[] props = UserHelper.Current.GetAllUserProperties(_us.DataConnection);
                _sessionData.UserProps = new List<UserProperties>(props.Length);
                foreach (UserProperties prop in props)
                {
                    _sessionData.UserProps.Add(prop);
                }

                BindUserList();

                DataBind();

                _sessionData.CurrentUserEmails = new List<string>();
                if (_sessionData.UserProps.Count > 0)
                {
                    _sessionData.CurrentUserProp = _sessionData.UserProps[0];
                    uiUserList.SelectedIndex = 0;
                    LoadCurrentUserEmails();
                }
                DataToControlBind();

                hidSessionId.Value = GenerateNewGuid();
                Session[hidSessionId.Value] = _sessionData;
            }
            else
            {
                Response.Redirect("AccessDenied.aspx");
            }
        }
        else
        {
            _sessionData = (UserMgmtSessionData)Session[hidSessionId.Value];
            
        }


    }

    private void LoadCurrentUserEmails()
    {
        string[] emails = 
            UserHelper.Current.GetUserEmails(_sessionData.CurrentUserProp.UserId.Value, _us.DataConnection);
        
        _sessionData.CurrentUserEmails.Clear();
        foreach (string email in emails)
        {
            _sessionData.CurrentUserEmails.Add(email);
        }

        UpdateUserEmailList();
    }

    private void UpdateUserEmailList()
    {
        uiEmailList.DataSource = _sessionData.CurrentUserEmails;
        uiEmailList.DataBind();
    }

    private void SaveCurrentUserEmails()
    {
        UserHelper.Current.SaveUserEmails(
            _sessionData.CurrentUserProp.UserId.Value, 
            _sessionData.CurrentUserEmails.ToArray(), 
            _us.DataConnection
            );
    }

    protected void BindUserList()
    {
        (new UserPropertiesOperator()).SortUserPropertiesListByLogin(_sessionData.UserProps);
        
        uiUserList.Items.Clear();
        foreach (UserProperties prop in _sessionData.UserProps)
        {
            string val = null;
            if(prop.UserId != null)
            {
                val = prop.UserId.ToString();
            }
            else
            {
                val = GenerateNewGuid();
            }

            uiUserList.Items.Add(new ListItem(prop.Login, val));
        }

        if (_sessionData.CurrentUserProp != null)
        {
            uiUserList.SelectedIndex = _sessionData.UserProps.IndexOf(_sessionData.CurrentUserProp);
        }
    }

    protected void uiUserList_SelectedIndexChanged(object sender, EventArgs e)
    {
        _sessionData.CurrentUserProp = _sessionData.UserProps[uiUserList.SelectedIndex];
        DataToControlBind();

        uiNewEmail.Text = "";
        uiResetPassword.Checked = false;
        if (_sessionData.CurrentUserProp.UserId != null)
        {
            LoadCurrentUserEmails();
        }
    }

    protected void DataToControlBind()
    {
        UserProperties prop = _sessionData.CurrentUserProp;
        uiLogin.Text = prop.Login;
        uiFullName.Text = prop.FullName;
        uiAutomaticPass.Checked = prop.AutomaticPass;
        uiAllowDeputyNotification.Checked = prop.AllowDeputyNotification;
        uiInstantMessaging.Checked = prop.InstantMessaging;
        uiImAddress.Text = prop.ImAddress;
        uiChangePwdNextTime.Checked = prop.ChangePwdNextTime;
        uiExternalAuthorization.Checked = prop.ExternalAuthorization;
        uiExternalIdentity.Text = prop.ExternalIdentity;
    }

    protected void ControlToDataBind()
    {
        UserProperties prop = _sessionData.CurrentUserProp;
        prop.Login = uiLogin.Text;
        prop.FullName = uiFullName.Text;
        prop.AutomaticPass = uiAutomaticPass.Checked;
        prop.AllowDeputyNotification = uiAllowDeputyNotification.Checked;
        prop.InstantMessaging = uiInstantMessaging.Checked;
        prop.ImAddress = uiImAddress.Text;
        prop.ChangePwdNextTime = uiChangePwdNextTime.Checked;
        prop.ExternalAuthorization = uiExternalAuthorization.Checked;
        prop.ExternalIdentity = uiExternalIdentity.Text;
    }

    protected bool CheckInput()
    {
        bool retVal = true;
        if (uiResetPassword.Checked && uiPassword.Text != uiPasswordConfirmation.Text)
        {
            uiErrorMessage.Text = _rm.GetString(ResKeys.PasswordConfirmationError);
            uiErrorMessage.Visible = true;
            retVal = false;
        }
        return retVal;
    }

    protected void uiSave_Click(object sender, EventArgs e)
    {
        bool inputChecked = CheckInput();

        if (inputChecked)
        {
            ControlToDataBind();

            UserProperties prop = _sessionData.CurrentUserProp;
            bool isNew = false;
            if (prop.UserId == null)
            {
                isNew = true;
            }

            UserHelper.Current.SaveUserProperties(prop, _us.DataConnection);
            SaveCurrentUserEmails();

            if (uiResetPassword.Checked)
            {
                UserHelper.Current.SetUserPassword(prop.UserId.Value, uiPassword.Text, _us.DataConnection);
            }

            if (isNew)
            {
                int i = _sessionData.UserProps.IndexOf(prop);
                uiUserList.Items[i].Value = prop.UserId.ToString();
            }

            BindUserList();
        }
    }


    protected void uiCancel_Click(object sender, EventArgs e)
    {
        DataToControlBind();
    }

    protected void uiCreate_Click(object sender, EventArgs e)
    {
        UserProperties prop = new UserProperties(
            _rm.GetString(ResKeys.NewUserFullName),
            _rm.GetString(ResKeys.NewUserFullName),
            false,
            true,
            DBConstKeeper.Current.CommonDepartment,
            false,
            "",
            false,
            false,
            "",
            null
            );

        _sessionData.CurrentUserProp = prop;
        _sessionData.UserProps.Add(prop);
        _sessionData.CurrentUserEmails.Clear();

        BindUserList();
        DataToControlBind();
        
        UpdateUserEmailList();
    }

    protected void uiRemoveEmail_CheckedChanged(object sender, EventArgs e)
    {
        Control cnt = (Control)sender;
        int i = ((RepeaterItem)cnt.Parent).ItemIndex;
        _sessionData.CurrentUserEmails.RemoveAt(i);
        UpdateUserEmailList();
    }

    protected void uiAddNewEmail_Click(object sender, EventArgs e)
    {
        if (uiNewEmail.Text != "")
        {
            _sessionData.CurrentUserEmails.Add(uiNewEmail.Text);
        }

        UpdateUserEmailList();
        uiNewEmail.Text = "";
    }

    protected void uiDelete_Click(object sender, EventArgs e)
    {
        UserHelper.Current.DeleteUser(_sessionData.CurrentUserProp.UserId.Value, _us.DataConnection);
        int i = _sessionData.UserProps.IndexOf(_sessionData.CurrentUserProp);
        _sessionData.UserProps.RemoveAt(i);

        if (i > 0)
        {
            i--;
        }

        _sessionData.CurrentUserProp = _sessionData.UserProps[i];
        uiUserList.SelectedIndex = i;
        LoadCurrentUserEmails();

        BindUserList();
        DataToControlBind();

        UpdateUserEmailList();
    }
}
