//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.IO;
using System.Resources;
using Evpo.DocCirc;
using Evpo.DocCirc.Messaging;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.UI;
using Evpo.DocCirc.Biz;

public partial class UserNotificationList : System.Web.UI.UserControl
{

    #region Resources
    private const string _controlName = "UserNotificationList.ascx";

    public struct ResKeys
    {
        public const string NotificationsTotalCount = "NotificationsTotalCount";
        public const string Previous = "Previous";
        public const string Next = "Next";
        public const string RefreshButton = "RefreshButton";
    }
    protected ResourceManager _rm;
    #endregion

    protected struct ControlConst
    {
        public const int NotificationsOnPageCount = 20;
    }

    protected struct ControlFiles
    {
        public const string NotificationTemplate = "UserNotificationList.file1.htm";
    }
    
    protected class UserNotificationListSessionData
    {
        public EmailHtmlBodyComposer HtmlComposer;
        public NotificationItem[] NotifItems;
        public MessageCreator MessageCreator;
        public string NotificationTemplate;
        public int FirstVisibleNotification;
    }

    protected UserNotificationListSessionData _sessionData = new UserNotificationListSessionData();

    protected string GenerateNewGuid()
    {
        System.Guid guid = System.Guid.NewGuid();
        return guid.ToString();
    }

    protected struct TemplateSubstitutions
    {
        public const string InternalNum = "^InternalNum^";
        public const string Subject = "^Subject^";
        public const string NotifId = "^NotifId^";
        public const string MessageHtmlText = "^MessageHtmlText^";
        public const string NotifTime = "^NotifTime^";
    }

    protected string ComposeNotification(NotificationItem item)
    {
        NotificationMessage notifMsg =
            _sessionData.MessageCreator.CreateNotificationMessage(item);

        string htmlText = _sessionData.HtmlComposer.CreateHtmlBody(notifMsg);
        string retVal = 
            _sessionData.NotificationTemplate.Replace(TemplateSubstitutions.InternalNum, notifMsg.InternalNum);
        retVal = retVal.Replace(TemplateSubstitutions.Subject, notifMsg.DocCardSubject);
        retVal = retVal.Replace(TemplateSubstitutions.NotifId, notifMsg.UserNotificationId.ToString());
        retVal = retVal.Replace(TemplateSubstitutions.MessageHtmlText, htmlText);
        retVal = retVal.Replace(TemplateSubstitutions.NotifTime, notifMsg.NotificationTime.ToString());

        return retVal;
    }

    protected void Page_Init(object sender, EventArgs e)
    {
        _rm = ResourceHelper.Current.GetResourceManagerForPage(_controlName);
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        
        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];
        if (!IsPostBack)
        {
            hidSessionId.Value = GenerateNewGuid();
            _sessionData.HtmlComposer = new EmailHtmlBodyComposer(us.SettingKeeper);
            _sessionData.MessageCreator = new MessageCreator(us.DataConnection, us.ServiceUserInfo, us.SettingKeeper);

            StreamReader reader = File.OpenText(
                Path.Combine(us.SettingKeeper.ApplicationFolder, ControlFiles.NotificationTemplate)
                );
            _sessionData.NotificationTemplate = reader.ReadToEnd();
            reader.Close();
            ResetNotificationList();
            Session[hidSessionId.Value] = _sessionData;
        }
        else
        {
            _sessionData = (UserNotificationListSessionData)Session[hidSessionId.Value];
        }

        this.DataBind();
    }

    protected void ResetNotificationList()
    {
        UserSession us = (UserSession)Session[SessionStateKeys.UserSession];

        UserNotificationData und = new UserNotificationData(us.UserInfo, us.DataConnection, us.SettingKeeper);
        _sessionData.NotifItems = und.GetUserNotifications();
        
        uiNotifQuantity.Text = _sessionData.NotifItems.Length.ToString();

        _sessionData.FirstVisibleNotification = 0;
        ShowNotificationItems();
    }

    protected void ShowNotificationItems()
    {
        if (_sessionData.NotifItems.Length > 0)
        {
            int lastVisibleNotification =
                _sessionData.FirstVisibleNotification + ControlConst.NotificationsOnPageCount - 1;

            bool lastPage = false;
            if (lastVisibleNotification >= _sessionData.NotifItems.Length - 1)
            {
                lastVisibleNotification = _sessionData.NotifItems.Length - 1;
                lastPage = true;
            }

            NotificationItem[] visibleItems =
                new NotificationItem[lastVisibleNotification - _sessionData.FirstVisibleNotification + 1];
            Array.Copy(
                _sessionData.NotifItems,
                _sessionData.FirstVisibleNotification,
                visibleItems,
                0,
                lastVisibleNotification - _sessionData.FirstVisibleNotification + 1
                );
            uiNotificationList.Visible = true;
            uiNotificationList.DataSource = visibleItems;
            uiNotificationList.DataBind();

            uiNotifSpan.Text = String.Format(
                "{0}...{1}",
                _sessionData.FirstVisibleNotification + 1,
                lastVisibleNotification + 1
                );

            uiPrevPage.Visible = (_sessionData.FirstVisibleNotification > 0);
            uiNextPage.Visible = !lastPage;
        }
        else
        {
            uiNotifSpan.Visible = false;
            uiPrevPage.Visible = false;
            uiNextPage.Visible = false;
        }


    }

    public override void DataBind()
    {
        foreach (Control cnt in Controls)
        {
            if (cnt.ID != "uiNotificationList")
            {
                cnt.DataBind();
            }
        }
    }

    protected void uiPrevPage_Click(object sender, EventArgs e)
    {
        _sessionData.FirstVisibleNotification -= ControlConst.NotificationsOnPageCount;
        ShowNotificationItems();
    }


    protected void uiNextPage_Click(object sender, EventArgs e)
    {
        _sessionData.FirstVisibleNotification += ControlConst.NotificationsOnPageCount;
        ShowNotificationItems();
    }
}
