//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using Evpo.DocCirc.MySqlDL;
using Evpo.DocCirc.Common;

namespace Evpo.DocCirc.Biz
{
    public class DocCardData
    {
        protected DocCardAccess _docCardAccess;
        protected UserInfo _userInfo;
        protected DataConnection _con;
        protected SettingKeeper _sk;

        /// <summary>
        /// for unit testing
        /// </summary>
        public DocCardData()
        {
        }

        public DocCardData(UserInfo userInfo, DataConnection con, SettingKeeper sk)
        {
            _docCardAccess = new DocCardAccess(userInfo.UserId, con.DBConnection, sk);
            _userInfo = userInfo;
            _con = con;
            _sk = sk;
        }

        public DataTable GetDocumentTable(string[] fields, DocCardFilterItem[] filterItems)
        {
            return GetDocumentTable(fields, filterItems, new int[0]);
        }

        public DataTable GetDocumentTable(
            string[] fields, 
            DocCardFilterItem[] filterItems, 
            int[] consideringEmployees
            )
        {
            return _docCardAccess.GetDocumentTable(fields, filterItems, consideringEmployees);
        }

        public DocCardConsiderationInfo[] GetDocumentConsiderationInfo(DocCardFilterItem[] filterItems)
        {
            return _docCardAccess.GetDocumentConsiderationInfo(filterItems);
        }

        public void DeleteDocCard(long id)
        {
            bool loadRoute = true;
            DocCardItem item = _docCardAccess.LoadDocCard(id, !loadRoute);

            _con.BeginTransaction();
            try
            {
                _docCardAccess.DeleteDocCard(id);
            }
            catch (Exception ex)
            {
                _con.RollBackTransaction();
                LogHelper.Current.WriteException(ex);
                throw ex;
            }
            _con.CommitTransaction();

            LogHelper.Current.AddStr(String.Format(
                ResourceHelper.StrForLog[LogResKeys.DocCardDeleted],
                item.InternalNumber,
                item.Subject
                ),
                _userInfo.Login
                );
        }

        public DocCardItem CreateDocCard(int folder, List<long> documentDrafts)
        {
            DocCardItem retVal = _docCardAccess.CreateDocCard(folder);
            documentDrafts.Add(retVal.Id);

            EventData ed = new EventData(_userInfo, _con);
            ed.AddEvent(
                new EventItem(
                    retVal.Id,
                    DBConstKeeper.Current.EventType.DocCardRegistration,
                    _userInfo.UserId,
                    null,
                    0,
                    DateTime.Now,
                    "",
                    ""
                    )
                );

            LogHelper.Current.AddStr(String.Format(
                ResourceHelper.StrForLog[LogResKeys.DocCardCreated],
                retVal.Id
                ),
                _userInfo.Login
                );

            return retVal;
        }

        public virtual DocCardItem LoadDocCard(long id, bool loadRoute)
        {
            return _docCardAccess.LoadDocCard(id, loadRoute);
        }

        public virtual void LoadDocCardRoute(DocCardItem item)
        {
            _docCardAccess.LoadDocCardRoute(item);
        }

        public void SaveDocCard(DocCardItem item, List<long> documentDrafts)
        {
            if (item.IsNew)
            {
                documentDrafts.Remove(item.Id);

                //set user's LastDocType
                UserProperties prop = UserHelper.Current.GetUserProperties(_userInfo.UserId, _con);
                prop.LastDocType = item.DocType;
                UserHelper.Current.SaveUserProperties(prop, _con);
            }

            _con.BeginTransaction();
            try
            {
                _docCardAccess.SaveDocCard(item);
            }
            catch (Exception ex)
            {
                _con.RollBackTransaction();
                LogHelper.Current.WriteException(ex);
                throw ex;
            }
            _con.CommitTransaction();

            LogHelper.Current.AddStr(String.Format(
                ResourceHelper.StrForLog[LogResKeys.DocCardSaved],
                item.InternalNumber,
                item.Subject
                ),
                _userInfo.Login
                );
        }
    }
}
