//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.MySqlDL;

namespace Evpo.DocCirc.Biz
{
    public class DocCardRelationToUserResolver
    {
        protected DocCardItem _docCardItem;
        protected UserInfo _userInfo;
        protected DataConnection _con;

        public DocCardRelationToUserResolver(UserInfo userInfo, DataConnection con)
        {
            _userInfo = userInfo;
            _con = con;
        }

        protected virtual SecurityData GetSecurityData(UserInfo userInfo, DataConnection con)
        {
            return new SecurityData(userInfo, con);
        }

        public DocCardRelationToUser GetDocCardRelationToUser(DocCardItem item)
        {
            DocCardRelationToUser retVal = new DocCardRelationToUser();
            int i = 0;

            retVal.AttachmentIsReadOnly = true;
            retVal.AttachmentReadOnlyCheckBoxIsReadOnly = true;
            retVal.CanBeSaved = false;
            retVal.CanBeSeen = false;
            retVal.DocumentTabIsReadOnly = true;
            retVal.RemovePassRouteDecisionOption = false;
            retVal.RouteTabIsReadOnly = true;
            retVal.RouteTabIsVisible = false;
            retVal.UserWhoseDeputyCurrentUserIs = null;


            SecurityData sd = GetSecurityData(_userInfo, _con);
            SecurityItem si = sd.GetDocumentCardSecurity(item);
            retVal.CanBeSeen = si.Read;

            bool canUpdate = si.Update;

            if(
                !canUpdate && 
                item.DocStatus != DBConstKeeper.Current.DocStatus.Draft && 
                item.DocStatus != DBConstKeeper.Current.DocStatus.NotSaved
                )
            {
                retVal.RouteTabIsVisible = true;
            }

            if(canUpdate)
            {
                if (
                    item.DocStatus == DBConstKeeper.Current.DocStatus.NotSaved ||
                    item.DocStatus == DBConstKeeper.Current.DocStatus.Draft
                    )
                {
                    retVal.AttachmentIsReadOnly = false;
                    retVal.AttachmentReadOnlyCheckBoxIsReadOnly = false;
                    retVal.CanBeSaved = true;
                    retVal.DocumentTabIsReadOnly = false;
                    retVal.RemovePassRouteDecisionOption = false;
                    retVal.RouteTabIsReadOnly = true;
                    retVal.RouteTabIsVisible = false;
                    retVal.UserWhoseDeputyCurrentUserIs = null;
                }
                else if (item.DocStatus == DBConstKeeper.Current.DocStatus.Finalized)
                {
                    retVal.RouteTabIsVisible = true;
                }
                else // UnderCons
                {
                    retVal.RouteTabIsVisible = true;
                    
                    DeputyItem[] deputyItems = UserHelper.Current.GetWhomUserIsDeputyFor(_userInfo.UserId, _con);
                    int[] usersWhoseDeputyCurrentUserIs = new int[deputyItems.Length];
                    i = 0;
                    foreach(DeputyItem deputyItem in deputyItems)
                    {
                        usersWhoseDeputyCurrentUserIs[i] = deputyItem.User;
                        i++;
                    }

                    int? mainUserForAdmin = null;
                    int? mainUser = null;
                    bool cons = false;
                    foreach (DocCardRouteItem docCardRi in item.RoutePoints[item.RoutePointNum - 1].RouteItems)
                    {
                        if (docCardRi.UnderCons)
                        {
                            if (mainUserForAdmin == null && mainUser == null && _userInfo.UserIsAdministrator())
                            {
                                mainUserForAdmin = docCardRi.User;
                            }

                            if (docCardRi.User == _userInfo.UserId)
                            {
                                cons = true;
                                mainUser = null;
                                mainUserForAdmin = null;
                                break;
                            }
                            else
                            {
                                i = Array.BinarySearch<int>(usersWhoseDeputyCurrentUserIs, docCardRi.User);
                                if (i >= 0)
                                {
                                    mainUser = usersWhoseDeputyCurrentUserIs[0];
                                    mainUserForAdmin = null;
                                }
                            }
                        }
                    }

                    if (cons || mainUser != null || mainUserForAdmin != null )
                    {
                        retVal.AttachmentIsReadOnly = !item.AttachmentsAreChangeable;
                        retVal.AttachmentReadOnlyCheckBoxIsReadOnly = !item.AttachmentsAreChangeable;
                        retVal.RouteTabIsReadOnly = false;
                        retVal.CanBeSaved = true;

                        if (mainUser != null)
                        {
                            retVal.UserWhoseDeputyCurrentUserIs = mainUser;
                        }

                        if (mainUserForAdmin != null)
                        {
                            retVal.UserWhoseDeputyCurrentUserIs = mainUserForAdmin;
                            retVal.RemovePassRouteDecisionOption = true;
                        }
                    }

                }

            }

            return retVal;
        }

    }

    public class DocCardRelationToUser
    {
        public bool AttachmentIsReadOnly;
        public bool AttachmentReadOnlyCheckBoxIsReadOnly;
        public bool DocumentTabIsReadOnly;
        public bool RouteTabIsVisible;
        public bool RouteTabIsReadOnly;
        public bool CanBeSaved;
        public bool RemovePassRouteDecisionOption;
        public int? UserWhoseDeputyCurrentUserIs;
        public bool CanBeSeen;
    }
}
