//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using System.Resources;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.MySqlDL;
using Evpo.DocCirc.Messaging;

namespace Evpo.DocCirc.Biz
{
    public class MessageCreator
    {
        #region Resources
        private const string _controlName = "MessageCreator";
        public struct ResKeys
        {
            public const string AcquaintanceRequestNotificationType = "AcquaintanceRequestNotificationType";
            public const string DecisionAfterReturnNotificationType = "DecisionAfterReturnNotificationType";
            public const string DeputyNextDecisionNotificationType = "DeputyNextDecisionNotificationType";
            public const string FinalizationNotificationType = "FinalizationNotificationType";
            public const string NextDecisionNotificationType = "NextDecisionNotificationType";
            public const string NextNotificationNotificationType = "NextNotificationNotificationType";
            public const string NotificationAfterReturnNotificationType = "NotificationAfterReturnNotificationType";
            public const string ReturnToDraftDueToDecisionNotificationType = "ReturnToDraftDueToDecisionNotificationType";
            public const string ReturnToDraftDueToRouteChangeNotificationType = "ReturnToDraftDueToRouteChangeNotificationType";
            public const string DraftStateTarget = "DraftStateTarget";
            public const string RoutePointTarget = "RoutePointTarget";
        }
        protected ResourceManager _rm;

        #endregion

        protected string GetNotificationTypeResKey(int notificationType)
        {
            string retVal = "";
            if (notificationType == DBConstKeeper.Current.NotificationType.AcquaintanceRequest)
            {
                retVal = ResKeys.AcquaintanceRequestNotificationType;
            }
            else if (notificationType == DBConstKeeper.Current.NotificationType.DecisionAfterReturn)
            {
                retVal = ResKeys.DecisionAfterReturnNotificationType;
            }
            else if (notificationType == DBConstKeeper.Current.NotificationType.Finalization)
            {
                retVal = ResKeys.FinalizationNotificationType;
            }
            else if (notificationType == DBConstKeeper.Current.NotificationType.NextDecision)
            {
                retVal = ResKeys.NextDecisionNotificationType;
            }
            else if (notificationType == DBConstKeeper.Current.NotificationType.DeputyNextDecision)
            {
                retVal = ResKeys.DeputyNextDecisionNotificationType;
            }
            else if (notificationType == DBConstKeeper.Current.NotificationType.NextNotification)
            {
                retVal = ResKeys.NextNotificationNotificationType;
            }
            else if (notificationType == DBConstKeeper.Current.NotificationType.NotificationAfterReturn)
            {
                retVal = ResKeys.NotificationAfterReturnNotificationType;
            }
            else if (notificationType == DBConstKeeper.Current.NotificationType.ReturnToDraftDueToDecision)
            {
                retVal = ResKeys.ReturnToDraftDueToDecisionNotificationType;
            }
            else if (notificationType == DBConstKeeper.Current.NotificationType.ReturnToDraftDueToRouteChange)
            {
                retVal = ResKeys.ReturnToDraftDueToRouteChangeNotificationType;
            }
            else
            {
                throw new ApplicationException("Unknown notification type!");
            }
            return retVal;
        }

        protected DataConnection _con;
        protected UserInfo _serviceUser;
        protected SettingKeeper _sk;

        protected virtual ResourceManager GetResourceManager()
        {
            return ResourceHelper.Current.GetResourceManagerForPage(_controlName);
        }

        protected virtual DocCardData GetDocCardData(UserInfo userInfo, DataConnection con, SettingKeeper sk)
        {
            return new DocCardData(userInfo, con, sk);
        }

        protected virtual DocTypeData GetDocTypeData(UserInfo userInfo, DataConnection con)
        {
            return new DocTypeData(userInfo, con);
        }

        public MessageCreator(DataConnection con, UserInfo serviceUser, SettingKeeper sk)
        {
            _con = con;
            _serviceUser = serviceUser;
            _sk = sk;
            _rm = GetResourceManager();
        }

        private string CombineUri(string uri1, string uri2)
        {
            string retVal = uri1;
            if (!uri1.EndsWith("/"))
            {
                retVal += "/";
            }
            retVal += uri2;
            return retVal;
        }

        public NotificationMessage CreateNotificationMessage(NotificationItem item)
        {
            NotificationMessage retVal = new NotificationMessage();
            retVal.UserNotificationId = item.Id.Value;

            retVal.Recipients = UserHelper.Current.GetUserEmails(item.User, _con);
            retVal.UserLogin = UserHelper.Current.GetUserProperties(item.User, _con).Login;
            retVal.ServiceUserEmail = _serviceUser.Emails[0];
            retVal.ServiceUserDisplayName = _serviceUser.UserProperties.DisplayName;
            retVal.NotificationType = item.NotificationType;
            
            retVal.Text = item.Text;
            retVal.UserNote = item.UserNote;
            UserProperties userProps = UserHelper.Current.GetUserProperties(item.User, _con);
            retVal.InstantMessaging = userProps.InstantMessaging;
            retVal.ImAddress = userProps.ImAddress;

            if (item.LastActionUser != null)
            {
                retVal.LastActionUserDisplayName =
                    UserHelper.Current.GetUserProperties(item.LastActionUser.Value, _con).DisplayName;
            }
            retVal.NotificationTime = item.NotificationTime;
            retVal.RoutePointNum = item.RoutePointNum;

            string notifText = _rm.GetString(GetNotificationTypeResKey(item.NotificationType));
            if(retVal.NotificationType == DBConstKeeper.Current.NotificationType.AcquaintanceRequest)
            {
                notifText = String.Format(notifText, retVal.LastActionUserDisplayName);
            }
            else if (retVal.NotificationType == DBConstKeeper.Current.NotificationType.DecisionAfterReturn)
            {
                string target = "";
                if(retVal.RoutePointNum == 0)
                {
                    target = _rm.GetString(ResKeys.DraftStateTarget);
                }
                else
                {
                    target = String.Format(
                        _rm.GetString(ResKeys.RoutePointTarget),
                        retVal.RoutePointNum.ToString()
                        );
                }

                notifText = String.Format(
                    notifText,
                    retVal.LastActionUserDisplayName,
                    target
                    );
            }
            else if (retVal.NotificationType == DBConstKeeper.Current.NotificationType.DeputyNextDecision)
            {
                string userDisplayName = UserHelper.Current.GetUserProperties(item.User, _con).DisplayName;
                notifText = String.Format(
                    _rm.GetString(ResKeys.DeputyNextDecisionNotificationType),
                    userDisplayName
                    );
            }
            else if (
               retVal.NotificationType == DBConstKeeper.Current.NotificationType.NextNotification ||
               retVal.NotificationType == DBConstKeeper.Current.NotificationType.NotificationAfterReturn
               )
            {
                notifText = String.Format(
                    _rm.GetString(ResKeys.NextNotificationNotificationType),
                    retVal.RoutePointNum
                    );
            }

            retVal.NotificationTypeText = notifText;

            if (item.DocCardId != null)
            {
                DocCardData dcd = GetDocCardData(_serviceUser, _con, _sk);
                bool loadRoute = true;
                DocCardItem docCard = dcd.LoadDocCard(item.DocCardId.Value, !loadRoute);
                retVal.DocCardSubject = docCard.Subject;
                retVal.InternalNum = docCard.InternalNumber;

                retVal.DocCardUrl = CombineUri(
                    _sk.ApplicationUrl,
                    String.Format("DocumentCard.aspx?id={0}", docCard.Id)
                    );

                retVal.RegistratorDisplayName =
                    UserHelper.Current.GetUserProperties(docCard.RegisteringUser, _con).DisplayName;

                DocTypeData dtd = GetDocTypeData(_serviceUser, _con);
                bool loadRoutePoints = true;
                retVal.DocTypeName = dtd.LoadDocTypeItem(docCard.DocType.Value, !loadRoutePoints).Name;
                string docStatusResKey = (new ResKeyResolver()).GetDocStatusResKey(docCard.DocStatus);
                retVal.DocStatusText = ResourceHelper.StrForUI[docStatusResKey];
            }

            return retVal;

        }


    }
}
