//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.Messaging;
using Evpo.DocCirc.MySqlDL;

namespace Evpo.DocCirc.Biz
{
    public class Notificator
    {
        protected DataConnection _con;
        protected UserInfo _serviceUserInfo;
        protected SettingKeeper _sk;
        protected EmailMessenger _emailMessanger;
        protected InstantMessenger _instantMessanger;
        protected NotificationAccess _notifAccess;

        /// <summary>
        /// for unit testing
        /// </summary>
        public Notificator()
        {
        }

        public Notificator(UserInfo serviceUserInfo, SettingKeeper sk, DataConnection con)
        {
            _serviceUserInfo = serviceUserInfo;
            _sk = sk;
            _con = con;

            _emailMessanger = new EmailMessenger(_sk);
            _instantMessanger = new InstantMessenger(_sk);
            _notifAccess = new NotificationAccess(serviceUserInfo.UserId, con.DBConnection, sk);
        }

        public virtual void Notify(NotificationItem item)
        {
            _notifAccess.AddUserNotification(item);
            NotificationMessage msg =
                (new MessageCreator(_con, _serviceUserInfo, _sk)).CreateNotificationMessage(item);

            if (_sk.EmailMessagingEnabled)
            {
                _emailMessanger.SendMessage(msg);
            }

            if (msg.InstantMessaging && _sk.IMWebServiceEnabled)
            {
                _instantMessanger.SendMessage(msg);
            }
        }

        public void Close()
        {
            _instantMessanger.Close();
        }
    }
}
