//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using Evpo.DocCirc.MySqlDL;
using Evpo.DocCirc.Common;

namespace Evpo.DocCirc.Biz
{
    public class SecurityData
    {
        protected UserInfo _userInfo;
        protected DataConnection _con;
        protected SecurityAccess _securityAccess;

        protected virtual SecurityAccess GetSecurityAccess(int userId, DBConnection con)
        {
            return new SecurityAccess(userId, con);
        }

        /// <summary>
        /// for unit tests
        /// </summary>
        public SecurityData()
        {
        }

        public SecurityData(UserInfo userInfo, DataConnection con)
        {
            _userInfo = userInfo;
            _con = con;
            _securityAccess = GetSecurityAccess(_userInfo.UserId, con.DBConnection);
        }

        public virtual void CreateFolderSecurityTable()
        {
            _securityAccess.CreateFolderSecurityTable();
        }

        public virtual SecurityItem GetFolderSecurity(int folderId)
        {
            return _securityAccess.GetFolderSecurity(folderId);
        }

        public virtual SecurityItem GetDocTypeDocumentsSecurity(int docTypeId)
        {
            return _securityAccess.GetDocTypeDocumentsSecurity(docTypeId);
        }

        public virtual SecurityItem GetDocumentCardSecurity(DocCardItem item)
        {
            return _securityAccess.GetDocumentCardSecurity(item);
        }

        public DocTypePermission[] GetDocTypePermissions(int docType)
        {
            return _securityAccess.GetDocTypePermissions(docType);
        }

        public void SaveDocTypePermissions(int docType, DocTypePermission[] permissions)
        {
            _securityAccess.SaveDocTypePermissions(docType, permissions);
        }

        public FolderPermission[] GetFolderPermissions(int folderId)
        {
            return _securityAccess.GetFolderPermissions(folderId);
        }

        public void SaveFolderPermissions(int folderId, FolderPermission[] permissions)
        {
            _securityAccess.SaveFolderPermissions(folderId, permissions);
        }

    }


}
