//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.MySqlDL;

namespace Evpo.DocCirc.Biz
{
    public class UserInfo
    {
        public UserProperties UserProperties
        {
            get { return _userProperties; }
        }

        protected UserProperties _userProperties;
        public virtual int UserId
        {
            get { return _userProperties.UserId.Value; }
        }

        public virtual string Login
        {
            get { return _userProperties.Login; }
        }

        protected string _fullName;
        public virtual string FullName
        {
            get { return _userProperties.FullName; }
        }

        public string DisplayName
        {
            get { return _userProperties.DisplayName; }
        }

        protected string[] _emails;
        public virtual string[] Emails
        {
            get { return _emails; }
        }

        private UserRole[] _userRoles;
        public virtual UserRole[] UserRoles
        {
            get { return _userRoles; }
        }

        public virtual bool UserIsAdministrator()
        {
            bool retVal = false;
            foreach(UserRole role in _userRoles)
            {
                if (role.RoleId == DBConstKeeper.Current.MainRole.Administrators)
                {
                    retVal = true;
                    break;
                }
            }
            return retVal;
        }

        protected virtual UserAccess GetUserAccess(DBConnection con)
        {
            return new UserAccess(con);
        }

        public UserInfo()
        {
        }

        public UserInfo(string userLogin, DataConnection con)
        {
            UserAccess ua = GetUserAccess(con.DBConnection);
            int userId = ua.GetUserIdByLogin(userLogin);
            InitUser(userId, con);
        }

        public UserInfo(int userId, DataConnection con)
        {
            InitUser(userId, con);
        }

        protected void InitUser(int userId, DataConnection con)
        {
            UserAccess ua = GetUserAccess(con.DBConnection);
            _userProperties = ua.GetUserProperties(userId);
            _emails = ua.GetUserEmails(userId);
            _userRoles = ua.GetUserRoles(userId);
        }
    }

    public class UserHelper
    {
        public static UserHelperInstance Current;
        static UserHelper()
        {
            Current = new UserHelperInstance();
        }
    }

    public class UserHelperInstance
    {
        protected virtual UserAccess GetUserAccess(DBConnection con)
        {
            return new UserAccess(con);
        }

        public UserToken AuthorizeUser(string login, string password, DataConnection con)
        {
            UserAccess ua = GetUserAccess(con.DBConnection);
            UserToken retVal = ua.AuthorizeUser(login, password);
            return retVal;
        }

        public UserToken GetUserTokenByLogin(string login, DataConnection con)
        {
            UserAccess ua = GetUserAccess(con.DBConnection);
            return new UserToken(ua.GetUserIdByLogin(login), login);
        }

        public UserToken GetUserTokenByExternalIdentity(string identity, DataConnection con)
        {
            UserAccess ua = GetUserAccess(con.DBConnection);
            UserProperties prop = ua.GetUserProperties(ua.GetUserIdByExternalIdentity(identity));

            return new UserToken(prop.UserId.Value, prop.Login);
        }

        public int GetUserIdByExternalIdentity(string identity, DataConnection con)
        {
            UserAccess ua = GetUserAccess(con.DBConnection);
            return ua.GetUserIdByExternalIdentity(identity);
        }

        public UserToken[] GetUsers(DataConnection con)
        {
            //UserAccess ua = new UserAccess(con.DBConnection);
            //return ua.GetAllUsers();
            throw new NotImplementedException();
        }

        public UserProperties[] GetAllUserProperties(DataConnection con)
        {
            UserAccess ua = new UserAccess(con.DBConnection);
            return ua.GetAllUserProperties();
        }

        public virtual UserProperties GetUserProperties(int userId, DataConnection con)
        {
            UserAccess ua = new UserAccess(con.DBConnection);
            return ua.GetUserProperties(userId);
        }

        public virtual string[] GetUserEmails(int userId, DataConnection con)
        {
            UserAccess ua = new UserAccess(con.DBConnection);
            return ua.GetUserEmails(userId);
        }

        public virtual void SaveUserEmails(int userId, string[] emails, DataConnection con)
        {
            UserAccess ua = new UserAccess(con.DBConnection);
            ua.SaveUserEmails(userId, emails);
        }

        public virtual DeputyItem[] GetUserDeputies(int userId, DataConnection con)
        {
            UserAccess ua = new UserAccess(con.DBConnection);
            return ua.GetUserDeputies(userId);
        }

        public virtual DeputyItem[] GetWhomUserIsDeputyFor(int userId, DataConnection con)
        {
            UserAccess ua = new UserAccess(con.DBConnection);
            return ua.GetWhomUserIsDeputyFor(userId);
        }

        public virtual void SaveUserDeputies(int userId, DeputyItem[] deputies, DataConnection con)
        {
            UserAccess ua = new UserAccess(con.DBConnection);
            ua.SaveUserDeputies(userId, deputies);
        }

        public virtual void SaveUserProperties(UserProperties userProp, DataConnection con)
        {
            UserAccess ua = new UserAccess(con.DBConnection);
            ua.SaveUserProperties(userProp);
        }

        public virtual void DeleteUser(int userId, DataConnection con)
        {
            UserAccess ua = new UserAccess(con.DBConnection);
            ua.DeleteUser(userId);
        }

        public virtual void SetUserPassword(int userId, string password, DataConnection con)
        {
            UserAccess ua = new UserAccess(con.DBConnection);
            ua.SetUserPassword(userId, password);
        }

    }
}
