//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using Evpo.DocCirc.Common;

namespace Evpo.DocCirc.Biz
{
    public class UserSession:IDisposable
    {
        protected UserToken _userToken;
        public UserToken UserToken
        {
            get { return _userToken; }
        }

        private UserInfo _serviceUserInfo;
        public UserInfo ServiceUserInfo
        {
            get { return _serviceUserInfo; }
        }

        private SettingKeeper _settingKeeper;

        public SettingKeeper SettingKeeper
        {
            get { return _settingKeeper; }
        }


        private UserInfo _userInfo;
        public UserInfo UserInfo
        {
            get { return _userInfo; }
        }

        protected DataConnection _dataConnection;
        public DataConnection DataConnection
        {
            get { return _dataConnection; }
        }

        protected virtual UserInfo GetUserInfo(int userId, DataConnection con)
        {
            return new UserInfo(userId, con);
        }

        protected virtual DataConnection GetDataConnection(string connString)
        {
            return new DataConnection(connString);
        }

        protected virtual SecurityData GetSecurityData(UserInfo userInfo, DataConnection con)
        {
            return new SecurityData(userInfo, con);
        }

        public UserSession(
            UserToken token, DataConnection con, string userConnString, 
            UserInfo serviceUserInfo, SettingKeeper settingKeeper
            )
        {
            _userToken = token;
            _settingKeeper = settingKeeper;
            _userInfo = GetUserInfo(token.UserId, con);
            _serviceUserInfo = serviceUserInfo;
            _dataConnection = GetDataConnection(userConnString);
            _dataConnection.Open();

            //SecurityData sd = GetSecurityData(_userInfo, _dataConnection);
            //sd.CreateFolderSecurityTable();

            _docTypeDrafts = new List<int>();
            _documentDrafts = new List<long>();

            LogHelper.Current.AddStr(
                ResourceHelper.StrForLog[LogResKeys.UserSessionOpened],
                _userToken.Login
                );

        }

        public void Close()
        {
            DocTypeData dtd = new DocTypeData(_serviceUserInfo, _dataConnection);
            foreach (int docTypeId in _docTypeDrafts)
            {
                dtd.DeleteDocType(docTypeId);
            }
            _docTypeDrafts.Clear();

            DocCardData dcd = new DocCardData(_serviceUserInfo, _dataConnection, _settingKeeper);
            foreach (long docCardId in _documentDrafts)
            {
                dcd.DeleteDocCard(docCardId);
            }
            _documentDrafts.Clear();

            if (_dataConnection.IsOpen)
            {
                _dataConnection.Close();
            }

            LogHelper.Current.AddStr(
                ResourceHelper.StrForLog[LogResKeys.UserSessionClosed],
                _userToken.Login
                );
        }

        protected List<long> _documentDrafts;
        public List<long> DocumentDrafts
        {
            get { return _documentDrafts; }
        }


        protected List<int> _docTypeDrafts;
        public List<int> DocTypeDrafts
        {
            get { return _docTypeDrafts; }
        }

        #region IDisposable Members

        public void Dispose()
        {
            //Close();
        }

        #endregion
    }

    
}
