//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using System.Xml.Serialization;
using System.IO;

namespace Evpo.DocCirc.Common
{
    [Serializable]
    public class AppParameter
    {
        private string _name;
        [XmlAttribute()]
        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }

        private string _value;
        [XmlAttribute()]
        public string Value
        {
            get { return _value; }
            set { _value = value; }
        }

        public AppParameter()
        {
        }

        public AppParameter(string name, string val)
        {
            _name = name;
            _value = val;
        }


    }

    public class AppSettingOperator
    {
        protected const string _rootAttributeName = "EvpoDCSettings";

        public NameValueCollection Load(string fileName)
        {
            XmlSerializer ser = new XmlSerializer(
                typeof(AppParameter[]), 
                new XmlRootAttribute(_rootAttributeName)
                );
            TextReader reader = File.OpenText(fileName);
            AppParameter[] pars = (AppParameter[])ser.Deserialize(reader);
            reader.Close();
            NameValueCollection retVal = new NameValueCollection();
            foreach (AppParameter par in pars)
            {
                retVal.Add(par.Name, par.Value);
            }

            return retVal;
        }

        public void Save(string fileName, NameValueCollection nameValCol)
        {
            AppParameter[] pars = new AppParameter[nameValCol.Count];
            for(int i = 0; i < nameValCol.Count; i ++)
            {
                pars[i] = new AppParameter(nameValCol.Keys[i], nameValCol[nameValCol.Keys[i]]);
            }
            
            
            XmlSerializer ser = new XmlSerializer(
                typeof(AppParameter[]),
                new XmlRootAttribute(_rootAttributeName)
                );

            using (FileStream stm = File.OpenWrite(fileName))
            {
                ser.Serialize(stm, pars);
                stm.Flush();
                stm.Close();
            }
        }
    }
}
