//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Data;
using System.Configuration;
using System.Reflection;
using System.Collections.Specialized;
using System.IO;

namespace Evpo.DocCirc.Common
{
    #region ConfigurationSettings wrapper
    public class DcAppSettingFileKeyNotFound : ApplicationException
    {
    }

    /// <summary>
    /// Singleton for ConfigurationSettings
    /// </summary>
    public class DCAppSettings
    {
        protected const string _settingFileKey = "AppSettingFile";
        protected NameValueCollection _appSettings;

        private string _settingFile;

        public string SettingFile
        {
            get { return _settingFile; }
            set { _settingFile = value; }
        }

        public virtual void LoadSettings(string appFolder)
        {
            string settingFile = ConfigurationSettings.AppSettings[_settingFileKey];
            if (settingFile == null)
            {
                throw new DcAppSettingFileKeyNotFound();
            }
            
            if (!Path.IsPathRooted(settingFile))
            {
                settingFile = Path.Combine(appFolder, settingFile);
            }
            _settingFile = settingFile;
            _appSettings = (new AppSettingOperator()).Load(settingFile);
        }

        public virtual string this[string key]
        {
            get {
                
                return _appSettings[key]; 
            }
        }

        private static DCAppSettings _current = new DCAppSettings();
        public static DCAppSettings Current
        {
            get { return _current; }
            set { _current = value; }
        }
    }
    #endregion
    
    #region FileWrapper

    public class FileWrapper
    {

        static FileWrapper()
        {
            _current = new FileWrapper();
        }

        private static FileWrapper _current;
        public static FileWrapper Current
        {
            get { return _current; }
            set { _current = value; }
        }

        public virtual bool Exists(string path)
        {
            return File.Exists(path);
        }

        public virtual FileStreamWrapper Create(string path)
        {
            FileStream fs = File.Create(path);
            fs.Close();

            return new FileStreamWrapper(
                new FileStream(path, FileMode.Append, FileAccess.Write, FileShare.Read)
                );
        }

        public virtual FileStreamWrapper Open(string path, FileMode mode)
        {
            return new FileStreamWrapper(
                new FileStream(path, mode, FileAccess.Write, FileShare.Read)
                );
        }
    }
    #endregion

    #region FileStreamWrapper
    public class FileStreamWrapper
    {
        private FileStream _fileStream;

        public FileStreamWrapper()
        {
        }

        public FileStreamWrapper(FileStream fileStream)
        {
            _fileStream = fileStream;
        }

        public virtual void Close()
        {
            _fileStream.Close();
        }

        public virtual void Write(byte[] array, int offset, int count)
        {
            _fileStream.Write(array, offset, count);
        }

        public virtual void Flush()
        {
            _fileStream.Flush();
        }
    }
    #endregion

    #region AssemblyWrapper
    public class AssemblyWrapper
    {
        private Assembly _assembly;
        public Assembly Assembly
        {
            get { return _assembly; }
        }

        public AssemblyWrapper()
        {
        }

        public AssemblyWrapper(Assembly assembly)
        {
            _assembly = assembly;
        }
    }
    #endregion
}