//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;

namespace Evpo.DocCirc.Common
{
    public class FolderTypeDirectory
    {
        private int _docType;
        public int DocType
        {
            get { return _docType; }
        }

        private int _document;
        public int Document
        {
            get { return _document; }
        }

        public FolderTypeDirectory(int document, int docType)
        {
            _document = document;
            _docType = docType;
        }
    }

    public class RootFolderDirectory
    {
        private int _document;
        public int Document
        {
            get { return _document; }
        }

        private int _docType;
        public int DocType
        {
            get { return _docType; }
        }

        public RootFolderDirectory(int document, int docType)
        {
            _document = document;
            _docType = docType;
        }
    }

    public class MainRoleDirectory
    {
        private int _users;
        public int Users
        {
            get { return _users; }
        }


        private int _administrators;
        public int Administrators
        {
            get { return _administrators; }
        }

        public MainRoleDirectory(int administrators, int users)
        {
            _administrators = administrators;
            _users = users;
        }
    }

    public class RouteItemTypeDirectory
    {
        private int _decision;
        public int Decision
        {
            get { return _decision; }
        }

        private int _notification;
        public int Notification
        {
            get { return _notification; }
        }

        public RouteItemTypeDirectory(int decision, int notification)
        {
            _decision = decision;
            _notification = notification;
        }
    }

    public class DocStatusDirectory
    {
        private int _notSaved;

        public int NotSaved
        {
            get { return _notSaved; }
            set { _notSaved = value; }
        }

        private int _draft;

        public int Draft
        {
            get { return _draft; }
            set { _draft = value; }
        }

        private int _underCons;

        public int UnderCons
        {
            get { return _underCons; }
            set { _underCons = value; }
        }

        private int _finalized;

        public int Finalized
        {
            get { return _finalized; }
            set { _finalized = value; }
        }

        public DocStatusDirectory(
            int notSaved,
            int draft,
            int underCons,
            int finalized
            )
        {
            _notSaved = notSaved;
            _draft = draft;
            _underCons = underCons;
            _finalized = finalized;
        }
    }

    public class NotificationTypeDirectory
    {
        private int _nextDecision;

        public int NextDecision
        {
            get { return _nextDecision; }
            set { _nextDecision = value; }
        }

        private int _deputyNextDecision;

        public int DeputyNextDecision
        {
            get { return _deputyNextDecision; }
            set { _deputyNextDecision = value; }
        }


        private int _nextNotification;

        public int NextNotification
        {
            get { return _nextNotification; }
            set { _nextNotification = value; }
        }

        private int _decisionAfterReturn;

        public int DecisionAfterReturn
        {
            get { return _decisionAfterReturn; }
            set { _decisionAfterReturn = value; }
        }

        private int _notificationAfterReturn;

        public int NotificationAfterReturn
        {
            get { return _notificationAfterReturn; }
            set { _notificationAfterReturn = value; }
        }

        private int _finalization;

        public int Finalization
        {
            get { return _finalization; }
            set { _finalization = value; }
        }

        private int _returnToDraftDueToDecision;

        public int ReturnToDraftDueToDecision
        {
            get { return _returnToDraftDueToDecision; }
            set { _returnToDraftDueToDecision = value; }
        }

        private int _returnToDraftDueToRouteChange;

        public int ReturnToDraftDueToRouteChange
        {
            get { return _returnToDraftDueToRouteChange; }
            set { _returnToDraftDueToRouteChange = value; }
        }

        private int _acquaintanceRequest;

        public int AcquaintanceRequest
        {
            get { return _acquaintanceRequest; }
            set { _acquaintanceRequest = value; }
        }


        public NotificationTypeDirectory() { }

        public NotificationTypeDirectory(
            int nextDecision,
            int deputyNextDecision,
            int nextNotification,
            int decisionAfterReturn,
            int notificationAfterReturn,
            int finalization,
            int returnToDraftDueToDecision,
            int returnToDraftDueToRouteChange,
            int acquaintanceRequest
            )
        {
            _nextDecision = nextDecision;
            _deputyNextDecision = deputyNextDecision;
            _nextNotification = nextNotification;
            _decisionAfterReturn = decisionAfterReturn;
            _notificationAfterReturn = notificationAfterReturn;
            _finalization = finalization;
            _returnToDraftDueToDecision = returnToDraftDueToDecision;
            _returnToDraftDueToRouteChange = returnToDraftDueToRouteChange;
            _acquaintanceRequest = acquaintanceRequest;
        }
    }

    public class EventTypeDirectory
    {
        private int _passDecision;
        /// <summary>
        /// User decides to pass the document card. It can leave on the current route point
        /// because other users on it might not have passed it.
        /// </summary>
        public int PassDecision
        {
            get { return _passDecision; }
            set { _passDecision = value; }
        }

        private int _passDecisionAutomatically;

        public int PassDecisionAutomatically
        {
            get { return _passDecisionAutomatically; }
            set { _passDecisionAutomatically = value; }
        }

        private int _passDecisionByDeputy;

        public int PassDecisionByDeputy
        {
            get { return _passDecisionByDeputy; }
            set { _passDecisionByDeputy = value; }
        }

        private int _rejectDecision;
        
        public int RejectDecision
        {
            get { return _rejectDecision; }
            set { _rejectDecision = value; }
        }

        private int _rejectDecisionByDeputy;

        public int RejectDecisionByDeputy
        {
            get { return _rejectDecisionByDeputy; }
            set { _rejectDecisionByDeputy = value; }
        }

        private int _rejectDecisionByAdministrator;

        public int RejectDecisionByAdministrator
        {
            get { return _rejectDecisionByAdministrator; }
            set { _rejectDecisionByAdministrator = value; }
        }

        private int _postponeDecision;
        
        public int PostponeDecision
        {
            get { return _postponeDecision; }
            set { _postponeDecision = value; }
        }

        private int _postponeDecisionByDeputy;

        public int PostponeDecisionByDeputy
        {
            get { return _postponeDecisionByDeputy; }
            set { _postponeDecisionByDeputy = value; }
        }

        private int _postponeDecisionByAdministrator;

        public int PostponeDecisionByAdministrator
        {
            get { return _postponeDecisionByAdministrator; }
            set { _postponeDecisionByAdministrator = value; }
        }

        private int _returnToDraftDueToRouteChange;

        public int ReturnToDraftDueToRouteChange
        {
            get { return _returnToDraftDueToRouteChange; }
            set { _returnToDraftDueToRouteChange = value; }
        }

        private int _docCardRegistration;

        public int DocCardRegistration
        {
            get { return _docCardRegistration; }
            set { _docCardRegistration = value; }
        }

        private int _docCardFinalization;

        public int DocCardFinalization
        {
            get { return _docCardFinalization; }
            set { _docCardFinalization = value; }
        }

        private int _considerationStarted;

        public int ConsiderationStarted
        {
            get { return _considerationStarted; }
            set { _considerationStarted = value; }
        }

        private int _passedToNextRoutePoint;
        /// <summary>
        /// 1. When everybody passed the document card on the previous route point it moves to the next route point 
        /// 2. When somebody returned it to the last route point
        /// 3. When consideration started
        /// </summary>
        public int PassedToNextRoutePoint
        {
            get { return _passedToNextRoutePoint; }
            set { _passedToNextRoutePoint = value; }
        }

        public EventTypeDirectory() { }

        public EventTypeDirectory(
            int passDecision,
            int passDecisionAutomatically,
            int passDecisionByDeputy,
            int rejectDecision,
            int rejectDecisionByDeputy,
            int rejectDecisionByAdministrator,
            int postponeDecision,
            int postponeDecisionByDeputy,
            int postponeDecisionByAdministrator,
            int returnToDraftDueToRouteChange,
            int docCardRegistration,
            int docCardFinalization,
            int considerationStarted,
            int passedToNextRoutePoint
            )
        {
            _passDecision = passDecision;
            _passDecisionAutomatically = passDecisionAutomatically;
            _passDecisionByDeputy = passDecisionByDeputy;
            _rejectDecision = rejectDecision;
            _rejectDecisionByAdministrator = rejectDecisionByAdministrator;
            _rejectDecisionByDeputy = rejectDecisionByDeputy;
            _postponeDecision = postponeDecision;
            _postponeDecisionByAdministrator = postponeDecisionByAdministrator;
            _postponeDecisionByDeputy = postponeDecisionByDeputy;
            _returnToDraftDueToRouteChange = returnToDraftDueToRouteChange;
            _docCardRegistration = docCardRegistration;
            _docCardFinalization = docCardFinalization;
            _considerationStarted = considerationStarted;
            _passedToNextRoutePoint = passedToNextRoutePoint;
        }


    }


    public enum Decision
    {
        Pass,
        Postpone,
        Reject
    }
}
