//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;


namespace Evpo.DocCirc.Common
{
    public struct DocCardFieldName
    {
        public const string Id = "Id";
        public const string Folder = "Folder";
        public const string DocType = "DocType";
        public const string DocStatus = "DocStatus";
        public const string RegisteringUser = "RegisteringUser";
        public const string Owner = "Owner";
        public const string LastMovement = "LastMovement";
        public const string Subject = "Subject";
        public const string InternalNumber = "InternalNumber";
        public const string RegistrationTime = "RegistrationTime";
        public const string FinalizingTime = "FinalizingTime";
        public const string AttachmentsAreChangeable = "AttachmentsAreChangeable";
        public const string OriginalDate = "OriginalDate";
        public const string OriginalNumber = "OriginalNumber";
        public const string CustomSendingCompany = "CustomSendingCompany";
        public const string CustomReceivingCompany = "CustomReceivingCompany";
        public const string SendingCompany = "SendingCompany";
        public const string ReceivingCompany = "ReceivingCompany";
        public const string Note = "Note";
        public const string RoutePointNum = "RoutePointNum";
    }

    public enum DocCardSearchType
    {
        Exact,
        Begins,
        Contains
    }

    public class DocCardFilterItem
    {
        protected string _fieldName;

        public string FieldName
        {
            get { return _fieldName; }
            set { _fieldName = value; }
        }

        private object _fieldValue;

        public object FieldValue
        {
            get { return _fieldValue; }
            set { _fieldValue = value; }
        }

        private DocCardSearchType _searchType;

        public DocCardSearchType SearchType
        {
            get { return _searchType; }
            set { _searchType = value; }
        }

        public DocCardFilterItem(string fieldName, object fieldValue)
            :this(fieldName, fieldValue, DocCardSearchType.Exact)
        {
        }

        public DocCardFilterItem(string fieldName, object fieldValue, DocCardSearchType searchType)
        {
            _fieldName = fieldName;
            _fieldValue = fieldValue;
            _searchType = searchType;
        }

    }

    public class PeriodDocCardFilterItem : DocCardFilterItem
    {
        private DateTime _startOfPeriod;

        public DateTime StartOfPeriod
        {
            get { return _startOfPeriod; }
            set { _startOfPeriod = value; }
        }

        private DateTime _endOfPeriod;

        public DateTime EndOfPeriod
        {
            get { return _endOfPeriod; }
            set { _endOfPeriod = value; }
        }

        public PeriodDocCardFilterItem(string fieldName, DateTime startOfPeriod, DateTime endOfPeriod):
            base(fieldName, null)
        {
            _fieldName = fieldName;
            _startOfPeriod = startOfPeriod;
            _endOfPeriod = endOfPeriod;
        }
    }
}
