//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;

namespace Evpo.DocCirc.Common
{
    public class DocCardItem
    {
        private long _id;

        public long Id
        {
            get { return _id; }
            set { _id = value; }
        }

        private int _folder;

        public int Folder
        {
            get { return _folder; }
            set { _folder = value; }
        }

        private int? _docType = null;

        public int? DocType
        {
            get { return _docType; }
            set { _docType = value; }
        }

        private int _docStatus;

        public int DocStatus
        {
            get { return _docStatus; }
            set { _docStatus = value; }
        }

        private int _registeringUser;

        public int RegisteringUser
        {
            get { return _registeringUser; }
            set { _registeringUser = value; }
        }

        private int _owner;

        public int Owner
        {
            get { return _owner; }
            set { _owner = value; }
        }

        private DateTime? _lastMovement = null;

        public DateTime? LastMovement
        {
            get { return _lastMovement; }
            set { _lastMovement = value; }
        }

        private string _subject;

        public string Subject
        {
            get { return _subject; }
            set { _subject = value; }
        }

        private string _internalNumber;

        public string InternalNumber
        {
            get { return _internalNumber; }
            set { _internalNumber = value; }
        }

        private DateTime _registrationTime;

        public DateTime RegistrationTime
        {
            get { return _registrationTime; }
            set { _registrationTime = value; }
        }

        private DateTime? _finalizingTime = null;

        public DateTime? FinalizingTime
        {
            get { return _finalizingTime; }
            set { _finalizingTime = value; }
        }

        private bool _attachmentsAreChangeable;

        public bool AttachmentsAreChangeable
        {
            get { return _attachmentsAreChangeable; }
            set { _attachmentsAreChangeable = value; }
        }

        private DateTime? _originalDate = null;

        public DateTime? OriginalDate
        {
            get { return _originalDate; }
            set { _originalDate = value; }
        }

        private string _originalNumber;

        public string OriginalNumber
        {
            get { return _originalNumber; }
            set { _originalNumber = value; }
        }

        private int? _sendingCompany = null;

        public int? SendingCompany
        {
            get { return _sendingCompany; }
            set { _sendingCompany = value; }
        }

        private int? _receivingCompany = null;

        public int? ReceivingCompany
        {
            get { return _receivingCompany; }
            set { _receivingCompany = value; }
        }

        private string _customSendingComany;

        public string CustomSendingCompany
        {
            get { return _customSendingComany; }
            set { _customSendingComany = value; }
        }

        private string _customReceivingCompany;

        public string CustomReceivingCompany
        {
            get { return _customReceivingCompany; }
            set { _customReceivingCompany = value; }
        }

        private string _note;

        public string Note
        {
            get { return _note; }
            set { _note = value; }
        }


        private bool _isNew;

        public bool IsNew
        {
            get { return _isNew; }
            set { _isNew = value; }
        }

        private int _routePointNum;

        public int RoutePointNum
        {
            get { return _routePointNum; }
            set { _routePointNum = value; }
        }

        private bool _routeIsLoaded;

        public bool RouteIsLoaded
        {
            get { return _routeIsLoaded; }
            set { _routeIsLoaded = value; }
        }

        private RoutePoint[] _routePoints;
        public RoutePoint[] RoutePoints
        {
            get { return _routePoints; }
            set { _routePoints = value; }
        }

        public void SetRouteIsLoaded(bool routeIsLoaded)
        {
            _routeIsLoaded = routeIsLoaded;
        }

        public DocCardItem()
        {
        }

        public DocCardItem(DocCardItem item)
        {
            _attachmentsAreChangeable = item.AttachmentsAreChangeable;
            _sendingCompany = item.SendingCompany;
            _receivingCompany = item.ReceivingCompany;
            _customReceivingCompany = item.CustomReceivingCompany;
            _customSendingComany = item.CustomSendingCompany;
            _docStatus = item.DocStatus;
            _docType = item.DocType;
            _finalizingTime = item.FinalizingTime;
            _folder = item.Folder;
            _id = item.Id;
            _internalNumber = item.InternalNumber;
            _isNew = item.IsNew;
            _lastMovement = item.LastMovement;
            _note = item.Note;
            _originalDate = item.OriginalDate;
            _originalNumber = item.OriginalNumber;
            _owner = item.Owner;
            _registeringUser = item.RegisteringUser;
            _registrationTime = item.RegistrationTime;
            _routeIsLoaded = item.RouteIsLoaded;
            _routePointNum = item.RoutePointNum;
            _subject = item.Subject;

            // child objects
            if (item.RouteIsLoaded)
            {
                _routePoints = new RoutePoint[item.RoutePoints.Length];
                int i = 0;
                foreach (RoutePoint rp in item.RoutePoints)
                {
                    _routePoints[i] = new RoutePoint(rp.Id, rp.Expression, rp.OrderNum);
                    _routePoints[i].RouteItems = new List<RouteItem>();
                    foreach (DocCardRouteItem ri in rp.RouteItems)
                    {
                        _routePoints[i].RouteItems.Add(
                            new DocCardRouteItem(ri.DocRouteItemId, ri.Passed, ri.UnderCons, ri)
                            );
                    }

                    i++;
                }
            }

        }
    }

    public class DocCardRouteItem : RouteItem
    {
        private long? _docRouteItemId;
        public long? DocRouteItemId
        {
            get { return _docRouteItemId; }
            set { _docRouteItemId = value; }
        }


        private bool _passed;
        public bool Passed
        {
            get { return _passed; }
            set { _passed = value; }
        }

        private bool _underCons;
        public bool UnderCons
        {
            get { return _underCons; }
            set { _underCons = value; }
        }

        public DocCardRouteItem(
            long? docRouteItemId,
            bool passed, bool underCons,
            RouteItem routeItem
            )
        {
            _routeItemType = routeItem.RouteItemType;
            _id = routeItem.Id;
            _user = routeItem.User;

            _docRouteItemId = docRouteItemId;
            _passed = passed;
            _underCons = underCons;
        }
    }

    public class DocCardConsiderationInfo
    {
        private long _docCardId;
        public long DocCardId
        {
            get { return _docCardId; }
            set { _docCardId = value; }
        }

        private int[] _employees;
        public int[] Employees
        {
            get { return _employees; }
            set { _employees = value; }
        }

        private int _routePointNum;
	    public int RoutePointNum
	    {
		    get { return _routePointNum;}
		    set { _routePointNum = value;}
	    }


        public DocCardConsiderationInfo(long docCardId, int routePointNum, int[] employees)
        {
            _docCardId = docCardId;
            _employees = employees;
            _routePointNum = routePointNum;
        }

        public DocCardConsiderationInfo()
        {
        }
    }
}
