//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;

namespace Evpo.DocCirc.Common
{
    /// <summary>
    /// 
    /// </summary>
    public class DocTypeItem
    {
        public class DcRoutePointsNotLoaded : ApplicationException
        {
        }

        public DocTypeItem()
        {
            _routePoints = new List<RoutePoint>();
        }

        public DocTypeItem(int id, int folder, string name, string prefix, string description, int nextNumber, int owner)
        {
            _id = id;
            _folder = folder;
            _name = name;
            _prefix = prefix;
            _description = description;
            _nextNumber = nextNumber;
            _owner = owner;
            _routePoints = new List<RoutePoint>();
        }

        public void SetRoutePointsLoaded(bool loaded)
        {
            _routePointsLoaded = loaded;
        }

        private bool _routePointsLoaded;
        public bool RoutePointsLoaded
        {
            get { return _routePointsLoaded; }
        }

        public void SetSavedState()
        {
            _isNew = false;
        }

        private int _id;
        public int Id
        {
            get
            {
                return _id;
            }
            set
            {
                _id = value;
            }
        }

        private int _folder;
        public int Folder
        {
            get
            {
                return _folder;
            }
            set
            {
                _folder = value;
            }
        }

        private string _name;
        public string Name
        {
            get
            {
                return _name;
            }
            set
            {
                _name = value;
            }
        }

        private string _description;
        public string Description
        {
            get
            {
                return _description;
            }
            set
            {
                _description = value;
            }
        }

        private string _prefix;
        public string Prefix
        {
            get
            {
                return _prefix;
            }
            set
            {
                _prefix = value;
            }
        }

        private int _nextNumber;
        public int NextNumber
        {
            get
            {
                return _nextNumber;
            }
            set
            {
                _nextNumber = value;
            }
        }

        private bool _isNew = false;
        public bool IsNew
        {
            get { return _isNew; }
            set { _isNew = value; }
        }

        private int _owner;

        public int Owner
        {
            get { return _owner; }
            set { _owner = value; }
        }


        private List<RoutePoint> _routePoints;
        public List<RoutePoint> RoutePoints
        {
            get
            {
                if (!_routePointsLoaded)
                {
                    throw new DcRoutePointsNotLoaded();
                }
                return _routePoints;
            }

            set
            {
                _routePoints = value;
                _routePointsLoaded = true;
            }
        }
    }


    public class RoutePoint
    {
        private int? _id;

        public int? Id
        {
            get { return _id; }
            set { _id = value; }
        }

        private int _orderNum;

        public int OrderNum
        {
            get { return _orderNum; }
            set { _orderNum = value; }
        }

        private string _expression;

        public string Expression
        {
            get { return _expression; }
            set { _expression = value; }
        }

        private List<RouteItem> _routeItems = new List<RouteItem>();

        public List<RouteItem> RouteItems
        {
            get { return _routeItems; }
            set { _routeItems = value; }
        }

        public RoutePoint():this(null, "", 1)
        { }

        public RoutePoint(int? id, string expression, int orderNum)
        {
            _id = id;
            _expression = expression;
            _orderNum = orderNum;
        }
    }

    /// <summary>
    /// 
    /// </summary>
    public class RouteItem
    {
        public RouteItem()
        {
            _id = null;
        }
        
        public RouteItem(int? id, int user, int routeItemType)
        {
            _id = id;
            _user = user;
            _routeItemType = routeItemType;
        }

        protected int? _id;
        public int? Id
        {
            get
            {
                return _id;
            }
            set
            {
                _id = value;
            }
        }

        protected int _user;
        public int User
        {
            get
            {
                return _user;
            }
            set
            {
                _user = value;
            }
        }

        //private int _routePoint;
        //public int RoutePoint
        //{
        //    get
        //    {
        //        return _routePoint;
        //    }
        //    set
        //    {
        //        _routePoint = value;
        //    }
        //}

        protected int _routeItemType;

        public int RouteItemType
        {
            get { return _routeItemType; }
            set { _routeItemType = value; }
        }

    }
}
