//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.IO;
using System.Text;
using System.Diagnostics;

namespace Evpo.DocCirc.Common
{
    public class LogHelper
    {
        private static LogHelper _current;

        public static LogHelper Current
        {
            get { return _current; }
            set { _current = value; }
        }

        protected FileStreamWrapper _fs;
        protected string _fileName;

        public LogHelper()
        {
        }

        public LogHelper(string fileName)
        {
            _fileName = fileName;
            if (!FileWrapper.Current.Exists(fileName))
            {
                _fs = FileWrapper.Current.Create(fileName);
            }
            else
            {
                _fs = FileWrapper.Current.Open(_fileName, FileMode.Append);
            }
        }

        public virtual string GetCurrentUserName()
        {
            return System.Security.Principal.WindowsIdentity.GetCurrent().Name;
        }

        public virtual void AddStr(string str)
        {
            AddStr(str, "");
        }

        public virtual void AddStr(string str, string userLogin)
        {
            string curUser = GetCurrentUserName();
            StringBuilder sb = new StringBuilder();
            if (userLogin != "")
            {
                sb.AppendFormat(
                    "{0} {1} {2} {3}{4}",
                    DateTime.Now.ToString(),
                    curUser,
                    userLogin,
                    str,
                    Environment.NewLine
                    );
            }
            else
            {
                sb.AppendFormat(
                    "{0} {1} {2}{3}",
                    DateTime.Now.ToString(),
                    curUser,
                    str,
                    Environment.NewLine
                    );
            }
            Encoding srcEnc = Encoding.Default;
            byte[] textBytes = srcEnc.GetBytes(sb.ToString());
            
            //FileStreamWrapper fs = FileWrapper.Current.Open(_fileName, FileMode.Append);
            _fs.Write(textBytes, 0, textBytes.Length);
            _fs.Flush();
            //fs.Close();
        }

        //public virtual void Close()
        //{
        //    //_fs.Close();
        //}

        public virtual void WriteException(Exception ex)
        {
            WriteException(ex, "");
        }

        public virtual void WriteException(Exception ex, string userLogin)
        {
            if (userLogin != "")
            {
                AddStr(
                    "Exception: " +
                    ex.Message +
                    ex.Source,
                    userLogin
                    );
            }
            else
            {
                AddStr(
                    "Exception: " +
                    ex.Message +
                    ex.Source
                    );
            }

            StackTrace st = new StackTrace(ex, true);
            StackFrame sf;
            for (int i = 0; i < st.FrameCount; i++)
            {
                sf = st.GetFrame(i);
                LogHelper.Current.AddStr(
                    "Method: " + sf.GetMethod() + Environment.NewLine +
                    "File: " + sf.GetFileName() + Environment.NewLine +
                    "Line number: " + sf.GetFileLineNumber() + Environment.NewLine +
                    "Column number: " + sf.GetFileColumnNumber() +
                    Environment.NewLine + Environment.NewLine
                    );
            }
        }

        public virtual void MakeDelimiter()
        {
            Encoding srcEnc = Encoding.Default;
            byte[] textBytes = srcEnc.GetBytes(
                Environment.NewLine + GlobalStringConstants.LogFileDelimiter + Environment.NewLine
                );
            //FileStreamWrapper fs = FileWrapper.Current.Open(_fileName, FileMode.Append);
            _fs.Write(textBytes, 0, textBytes.Length);
            _fs.Flush();
            //fs.Close();
        }

        public virtual void Close()
        {
            _fs.Close();
        }
    }

	
}
