//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;

namespace Evpo.DocCirc.Common
{
    public abstract class PermissionBase
    {
        protected int? _permissionId;

        public int? PermissionId
        {
            get { return _permissionId; }
            set { _permissionId = value; }
        }

        protected UserRole _role;

        public UserRole Role
        {
            get { return _role; }
            set { _role = value; }
        }

        protected SecurityItem _securityItem;

        public SecurityItem SecurityItem
        {
            get { return _securityItem; }
            set { _securityItem = value; }
        }
    }

    public class DocTypePermission : PermissionBase
    {
        protected int _docType;

        public int DocType
        {
            get { return _docType; }
            set { _docType = value; }
        }

        public DocTypePermission(int docType, UserRole role, SecurityItem securityItem)
            : this(docType, role, securityItem, null)
        {
        }

        public DocTypePermission(int docType, UserRole role, SecurityItem securityItem, int? permissionId)
        {
            _docType = docType;
            _securityItem = securityItem;
            _permissionId = permissionId;
            _role = role;
        }
    }

    public class FolderPermission : PermissionBase
    {
        protected int _folder;

        public int Folder
        {
            get { return _folder; }
            set { _folder = value; }
        }

        public FolderPermission(int folder, UserRole role, SecurityItem securityItem)
            : this(folder, role, securityItem, null)
        {
        }

        public FolderPermission(int folder, UserRole role, SecurityItem securityItem, int? permissionId)
        {
            _folder = folder;
            _securityItem = securityItem;
            _permissionId = permissionId;
            _role = role;
        }
    }
}
