//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Reflection;
using System.Resources;

namespace Evpo.DocCirc.Common
{
    /// <summary>
    /// Summary description for ResourceHelper
    /// </summary>
    public class ResourceHelper
    {
        public static StringResHelper StrForLog;
        public static StringResHelper StrForUI;
        public static StringResHelper StrForEx;
        public static ResourceHelper Current;

        protected AssemblyWrapper _assembly;

        /// <summary>
        /// for unit tests
        /// </summary>
        public ResourceHelper()
        {
        }

        public ResourceHelper(AssemblyWrapper assembly)
        {
            _assembly = assembly;
        }

        public ResourceManager GetResourceManagerForPage(string pageName)
        {
            return new ResourceManager(
                String.Format("{0}.{1}", GlobalStringConstants.ResourcePrefix, pageName),
                _assembly.Assembly
                );
        }
    }

    public class StringResHelper
    {

        public class WrongKeyException : ApplicationException
        {
            public WrongKeyException() { }
            public WrongKeyException(string message) : base(message) { }
        }

        protected ResourceManager _rm;

        protected virtual ResourceManager GetResourceManager(string baseName, AssemblyWrapper assembly)
        {
            return new ResourceManager(baseName, assembly.Assembly);
        }

        public StringResHelper()
        {
        }

        public StringResHelper(string baseName, AssemblyWrapper assembly)
        {
            _rm = GetResourceManager(baseName, assembly);
        }

        public virtual string this[string key]
        {
            get
            {
                string retVal = _rm.GetString(key);
                if(retVal == null)
                {
                    throw new StringResHelper.WrongKeyException(
                        String.Format("can't find a resource with the \"{0}\" key", key)
                        );
                }

                return retVal;
            }
        }

        public virtual void ReleaseAllResources()
        {
            _rm.ReleaseAllResources();
        }
    }

    /// <summary>
    /// keys to seek for a resource in the log string resources
    /// </summary>
    public struct LogResKeys
    {
        public const string ApplicationStart = "ApplicationStart";
        public const string ApplicationInitialized = "ApplicationInitialized";
        public const string ServiceUserInfo = "ServiceUserInfo";
        public const string DocCardSaved = "DocCardSaved";
        public const string DocCardDeleted = "DocCardDeleted";
        public const string DocCardCreated = "DocCardCreated";
        public const string DocTypeCreated = "DocTypeCreated";
        public const string DocTypeDeleted = "DocTypeDeleted";
        public const string DocTypeSaved = "DocTypeSaved";
        public const string UserSessionOpened = "UserSessionOpened";
        public const string UserSessionClosed = "UserSessionClosed";
        public const string ApplicationEnded = "ApplicationEnded";
        public const string UpdateExecutedWithText = "UpdateExecutedWithText";
    }

    /// <summary>
    /// see name
    /// </summary>
    public struct UIResKeys
    {
        public const string Greeting = "Greeting";
        public const string LoginASPXHeader = "LoginASPXHeader";
        public const string DocTypeFolderName = "DocTypeFolderName";
        public const string AccessDenied = "AccessDenied";

        // DocStatuses
        public const string NotSavedDocStatus = "NotSavedDocStatus";
        public const string DraftDocStatus = "DraftDocStatus";
        public const string UnderConsDocStatus = "UnderConsDocStatus";
        public const string FinalizedDocStatus = "FinalizedDocStatus";

        //Notification types
        public const string NextDecisionNotificationType = "NextDecisionNotificationType";
        public const string DeputyNextDecisionNotificationType = "DeputyNextDecisionNotificationType";
        public const string NextNotificationNotificationType = "NextNotificationNotificationType";
        public const string DecisionAfterReturnNotificationType = "DecisionAfterReturnNotificationType";
        public const string NotificationAfterReturnNotificationType = "NotificationAfterReturnNotificationType";
        public const string FinalizationNotificationType = "FinalizationNotificationType";
        public const string ReturnToDraftDueToDecisionNotificationType = "ReturnToDraftDueToDecisionNotificationType";
        public const string ReturnToDraftDueToRouteChangeNotificationType = "ReturnToDraftDueToRouteChangeNotificationType";
        public const string AcquaintanceRequestNotificationType = "AcquaintanceRequestNotificationType";
        
        // Decisions
        public const string PassDecision = "PassDecision";
        public const string PostponeDecision = "PostponeDecision";
        public const string RejectDecision = "RejectDecision";
    }

    public struct ExResKeys
    {
        public const string ConnectionStringIsNotSet = "ConnectionStringIsNotSet";
        public const string ServiceUserDoNotHaveEmail = "ServiceUserDoNotHaveEmail";
        public const string ConfigurationFileKeyIsNotFound = "ConfigurationFileKeyIsNotFound";
        public const string CannotConnectToDatabase = "CannotConnectToDatabase";

        // DocTypeChecker
        public const string DocTypeCheckerRouteIsEmpty = "DocTypeCheckerRouteIsEmpty";
        public const string DocTypeCheckerTextInroduction = "DocTypeCheckerTextInroduction";
        public const string DocTypeCheckerPrefixIsNotUnique = "DocTypeCheckerPrefixIsNotUnique";
        public const string DocTypeCheckerMustBeginFromFirst = "DocTypeCheckerMustBeginFromFirst";
        public const string DocTypeCheckerRoutePointIsMissed = "DocTypeCheckerRoutePointIsMissed";
        public const string DocTypeCheckerEmployeeMoreThanOnceOnRoutePoint = "DocTypeCheckerEmployeeMoreThanOnceOnRoutePoint";
    }
}