//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;

namespace Evpo.DocCirc.Common
{
    public class UserProperties
    {
        private int? _userId;
        public int? UserId
        {
            get { return _userId; }
            set { _userId = value; }
        }
        
        private string _login;
        public string Login
        {
            get { return _login; }
            set { _login = value; }
        }

        public string DisplayName
        {
            get
            {
                string retVal = FullName;
                if (String.IsNullOrEmpty(retVal))
                {
                    retVal = Login;
                }
                return retVal;
            }
        }

        private string _fullName;
        public string FullName
        {
            get { return _fullName; }
            set { _fullName = value; }
        }

        private bool _automaticPass;
        public bool AutomaticPass
        {
            get { return _automaticPass; }
            set { _automaticPass = value; }
        }

        private bool _instantMessaging;

        public bool InstantMessaging
        {
            get { return _instantMessaging; }
            set { _instantMessaging = value; }
        }

        private string _imAddress;

        public string ImAddress
        {
            get { return _imAddress; }
            set { _imAddress = value; }
        }

        private bool _allowDeputyNotification;
        public bool AllowDeputyNotification
        {
            get { return _allowDeputyNotification; }
            set { _allowDeputyNotification = value; }
        }

        private int _department;

        public int Department
        {
            get { return _department; }
            set { _department = value; }
        }

        private bool _changePwdNextTime;

        public bool ChangePwdNextTime
        {
            get { return _changePwdNextTime; }
            set { _changePwdNextTime = value; }
        }

        private bool _externalAuthorization;

        public bool ExternalAuthorization
        {
            get { return _externalAuthorization; }
            set { _externalAuthorization = value; }
        }

        private string _externalIdentity;

        public string ExternalIdentity
        {
            get { return _externalIdentity; }
            set { _externalIdentity = value; }
        }

        private int? _lastDocType;

        public int? LastDocType
        {
            get { return _lastDocType; }
            set { _lastDocType = value; }
        }


        public UserProperties()
        {
        }

        public UserProperties(
            string login, 
            string fullName, 
            bool automaticPass, 
            bool allowDeputyNotification, 
            int department, 
            bool instantMessaging, 
            string imAddress,
            bool changePwdNextTime,
            bool externalAuthorization,
            string externalIdentity,
            int? lastDocType
            )
            : this(
                login, 
                fullName, 
                automaticPass, 
                allowDeputyNotification, 
                department, 
                instantMessaging, 
                imAddress,
                changePwdNextTime,
                externalAuthorization,
                externalIdentity,
                lastDocType,
                null
            )
        {
        }

        public UserProperties(
            string login, 
            string fullName, 
            bool automaticPass, 
            bool allowDeputyNotification, 
            int department, 
            bool instantMessaging, 
            string imAddress,
            bool changePwdNextTime,
            bool externalAuthorization,
            string externalIdentity,
            int? lastDocType,
            int? userId
            )
        {
            _userId = userId;
            _login = login;
            _fullName = fullName;
            _automaticPass = automaticPass;
            _allowDeputyNotification = allowDeputyNotification;
            _department = department;
            _instantMessaging = instantMessaging;
            _imAddress = imAddress;
            _changePwdNextTime = changePwdNextTime;
            _externalAuthorization = externalAuthorization;
            _externalIdentity = externalIdentity;
            _lastDocType = lastDocType;
        }
    }

    public class UserToken
    {
        public UserToken(int userId, string login)
        {
            _userId = userId;
            _login = login;
        }

        private int _userId;
        public int UserId
        {
            get { return _userId; }
        }

        private string _login;
        public string Login
        {
            get { return _login; }
        }
    }
}
