//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Resources;
using System.Collections.Specialized;
using Evpo.DocCirc.Common;

namespace Evpo.DocCirc.Messaging
{
    public class EmailHtmlBodyComposer
    {

        #region Resources
        private const string _controlName = "EmailHtmlBodyComposer";
        public struct ResKeys
        {
            public const string InternalNumField = "InternalNumField";
            public const string DocTypeNameField = "DocTypeNameField";
            public const string DocCardSubjectField = "DocCardSubjectField";
            public const string DocCardStatusField = "DocCardStatusField";
            public const string RegistratorFullNameField = "RegistratorFullNameField";
            public const string OpenDocCardReference = "OpenDocCardReference";
            public const string NotificationTimeField = "NotificationTimeField";
            public const string LastUserWrote = "LastUserWrote";
            public const string DocCardInformation = "DocCardInformation";

        }
        protected ResourceManager _rm;
            
        #endregion

        //protected struct ClassConst
        //{
        //    public const string HtmlBodyTemplateFileName = "EmailHtmlBodyTemplate.htm";
        //}

        protected virtual ResourceManager GetResourceManager(string controlName)
        {
            return ResourceHelper.Current.GetResourceManagerForPage(_controlName);
        }

        protected SettingKeeper _sk;
        protected string _htmlBodyTemplate;

        public EmailHtmlBodyComposer(SettingKeeper sk)
        {
            _rm = GetResourceManager(_controlName);
            _sk = sk;
            //StreamReader reader = File.OpenText(
            //    Path.Combine(_sk.ApplicationFolder, ClassConst.HtmlBodyTemplateFileName)
            //    );
            //string htmlBodyTemplate = reader.ReadToEnd();
            //reader.Close();

            string htmlBodyTemplate = MessagingResources.EmailHtmlBodyTemplate;

            ReplacedItem[] resourceReplaceItems = new ReplacedItem[]
            {
                new ReplacedItem("^reskey:NotificationTimeField^", _rm.GetString(ResKeys.NotificationTimeField)),
                new ReplacedItem("^reskey:InternalNumField^", _rm.GetString(ResKeys.InternalNumField)),
                new ReplacedItem("^reskey:DocTypeNameField^", _rm.GetString(ResKeys.DocTypeNameField)),
                new ReplacedItem("^reskey:DocCardSubjectField^", _rm.GetString(ResKeys.DocCardSubjectField)),
                new ReplacedItem("^reskey:DocCardStatusField^", _rm.GetString(ResKeys.DocCardStatusField)),
                new ReplacedItem("^reskey:RegistratorFullNameField^", _rm.GetString(ResKeys.RegistratorFullNameField)),
                new ReplacedItem("^reskey:OpenDocCardReference^", _rm.GetString(ResKeys.OpenDocCardReference)),
                new ReplacedItem("^reskey:DocCardInformation^", _rm.GetString(ResKeys.DocCardInformation))
            };
            _htmlBodyTemplate = ComposerHelper.Current.ReplaceWithValues(htmlBodyTemplate, resourceReplaceItems);
        }

        public string CreateHtmlBody(NotificationMessage msg)
        {
            string displayUserNote = "none";
            string userNote = msg.UserNote;
            if (userNote != "")
            {
                displayUserNote = "inline";
                userNote = userNote.Replace(Environment.NewLine, "<br/>");
            }

            ReplacedItem[] replacedItems = new ReplacedItem[]
            {
                new ReplacedItem("^NotificationTypeText^", msg.NotificationTypeText),
                new ReplacedItem("^NotificationTime^", msg.NotificationTime.ToString()),
                new ReplacedItem("^InternalNum^", msg.InternalNum),
                new ReplacedItem("^DocTypeName^", msg.DocTypeName),
                new ReplacedItem("^DocCardSubject^", msg.DocCardSubject),
                new ReplacedItem("^DocCardStatus^", msg.DocStatusText),
                new ReplacedItem("^RegistratorFullName^",msg.RegistratorDisplayName),
                new ReplacedItem(
                    "^LastUserWrote^",
                    String.Format(_rm.GetString(ResKeys.LastUserWrote), msg.LastActionUserDisplayName)
                    ),
                new ReplacedItem("^LastUserNote^", userNote),
                new ReplacedItem("^DocCardUrl^", msg.DocCardUrl),
                new ReplacedItem("^control:LastUserDisplay^", displayUserNote)
            };

            return ComposerHelper.Current.ReplaceWithValues(_htmlBodyTemplate, replacedItems);
        }
    }
}
