//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using System.Net.Mail;
using Evpo.DocCirc.Common;
using System.Net;

namespace Evpo.DocCirc.Messaging
{
    public class EmailMessenger
    {
        protected SettingKeeper _sk;
        protected EmailHtmlBodyComposer _htmlBodyComposer;


        public EmailMessenger(SettingKeeper sk)
        {
            _sk = sk;
            _htmlBodyComposer = new EmailHtmlBodyComposer(sk);
        }

        public void SendMessage(NotificationMessage message)
        {
            // nothing
            MailMessage mailMsg = new MailMessage();
            mailMsg.From = new MailAddress(message.ServiceUserEmail, message.ServiceUserDisplayName);

            foreach(string recipient in message.Recipients)
            {
                mailMsg.To.Add(recipient);
            }

            mailMsg.Subject = String.Format("{0} : {1}", message.InternalNum, message.DocCardSubject);
            mailMsg.IsBodyHtml = true;
            mailMsg.Body = _htmlBodyComposer.CreateHtmlBody(message);

            SmtpClient smtpClient = new SmtpClient(_sk.EmailMessagingServer, _sk.EmailMessagingPort);
            if (!String.IsNullOrEmpty(_sk.EmailMessagingAccount) && !String.IsNullOrEmpty(_sk.EmailMessagingPassword))
            {
                smtpClient.Credentials = new NetworkCredential(_sk.EmailMessagingAccount, _sk.EmailMessagingPassword);
            }

            smtpClient.EnableSsl = _sk.EmailMessagingSSLEnabled;

            try
            {
                smtpClient.Send(mailMsg);
            }
            catch (Exception ex)
            {
                LogHelper.Current.AddStr("Can't send an email notification due to the following exception:");
                LogHelper.Current.WriteException(ex);
            }
        }
    }
}
