//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using MySql.Data;
using MySql.Data.MySqlClient;
using Evpo.DocCirc.Common;


namespace Evpo.DocCirc.MySqlDL
{
    public class DBConnection:IDisposable
    {
        protected MySqlTransaction _tran = null;

        protected bool _isOpen;
        public virtual bool IsOpen
        {
            get { return _isOpen; }
        }

        protected string _connString;
        protected MySqlConnection _prevCon = null;

        protected MySqlConnection _con;
        public MySqlConnection MySqlConnection
        {
            
            get {
               
                return _con; 
            }
        }

        public virtual void Open()
        {
            _con.Open();
            _isOpen = true;
        }

        protected virtual MySqlConnection GetMySqlConnection(string connString)
        {
            return new MySqlConnection(connString);
        }

        public DBConnection()
        {
        }

        public DBConnection(string connString)
        {
            _con = GetMySqlConnection(connString);
            _connString = connString;
        }

        public virtual void Close()
        {
            if (_tran != null)
            {
                _tran.Rollback();
                _tran = null;
            }
            Dispose();
        }

        protected void PushConnection()
        {
            _prevCon = _con;
            _con = GetMySqlConnection(_connString);
            _con.Open();
        }

        protected void PopConnection()
        {
            _con.Close();
            _con = _prevCon;
        }

        public virtual void BeginTransaction()
        {
            if (_tran != null)
            {
                throw new ApplicationException("Transaction is already open!");
            }
            PushConnection();
            _tran = _con.BeginTransaction();
        }

        public virtual void CommitTransaction()
        {
            if (_tran != null)
            {
                _tran.Commit();
                _tran.Dispose();
                _tran = null;
                PopConnection();
            }
            else
            {
                throw new ApplicationException("Can't commit! Transaction is not begun!");
            }
        }

        public virtual void RollbackTransaction()
        {
            if (_tran != null)
            {
                _tran.Rollback();
                _tran.Dispose();
                _tran = null;
                PopConnection();
            }
            else
            {
                throw new ApplicationException("Can't rollback! Transaction is not begun!");
            }
        }

        public virtual MySqlCommand CreateCommand()
        {
            _con.Ping();
            if (_con.State != System.Data.ConnectionState.Open && _isOpen)
            {
                _con.Open();
            }

            MySqlCommand retVal = _con.CreateCommand();
            if (_tran != null)
            {
                retVal.Transaction = _tran;
            }

            return retVal;
        }

        #region IDisposable Members

        public virtual void Dispose()
        {
            _con.Close();
            _isOpen = false;
        }

        #endregion
    }
}
