//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using MySql.Data.MySqlClient;
using Evpo.DocCirc.Common;

namespace Evpo.DocCirc.MySqlDL
{
    public class DBConstAccess
    {
        protected DBConnection _con;

        public virtual RootFolderDirectory GetRootFolderDirectory(FolderTypeDirectory ftDir)
        {
            int? document = null, docType = null;
            
            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                "select FolderId, FolderType from Folder where IsNull(ParentFolder)";

            MySqlDataReader reader = cmd.ExecuteReader();
            while (reader.Read())
            {
                if (reader.GetInt32("FolderType") == ftDir.Document)
                {
                    document = reader.GetInt32("FolderId");
                }
                else if (reader.GetInt32("FolderType") == ftDir.DocType)
                {
                    docType = reader.GetInt32("FolderId");
                }
            }
            reader.Close();

            if (document == null)
            {
                throw new DcObjectNotFoundException("Document root folder is not found!");
            }

            if (docType == null)
            {
                throw new DcObjectNotFoundException("DocType root folder is not found!");
            }

            return new RootFolderDirectory(document.Value, docType.Value);
        }

        public virtual FolderTypeDirectory GetFolderTypeDirectory()
        {
            if (!_con.IsOpen)
            {
                throw new DcConnectionClosedException();
            }

            int? document = null, docType = null;

            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                "select FolderTypeId, Name from FolderType";
            MySqlDataReader reader = cmd.ExecuteReader();
            while (reader.Read())
            {
                switch (reader.GetString("Name"))
                {
                    case "Document":
                        document = reader.GetInt32("FolderTypeId");
                        break;
                    case "DocType":
                        docType = reader.GetInt32("FolderTypeId");
                        break;
                }
            }
            reader.Close();
            if (document == null)
            {
                throw new DcObjectNotFoundException("Document folder type is not found in table FolderType!");
            }

            if (docType == null)
            {
                throw new DcObjectNotFoundException("DocType folder type is not found in table FolderType!");
            }

            return new FolderTypeDirectory(document.Value, docType.Value);
        }

        public virtual MainRoleDirectory GetMainRoleDirectory()
        {
            if (!_con.IsOpen)
            {
                throw new DcConnectionClosedException();
            }

            int? administratorsRoleId = null, usersRoleId = null;

            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                "select DcRoleId from DcRole where RoleName=?RoleName";
            cmd.Parameters.Add("?RoleName", MySqlDbType.String);
            cmd.Parameters["?RoleName"].Value = "Administrators";

            MySqlDataReader reader = cmd.ExecuteReader();
            if (reader.Read())
            {
                administratorsRoleId = reader.GetInt32("DcRoleId");
                reader.Close();
            }
            

            cmd.Parameters["?RoleName"].Value = "Users";
            reader = cmd.ExecuteReader();
            if (reader.Read())
            {
                usersRoleId = reader.GetInt32("DcRoleId");
                reader.Close();
            }

            return new MainRoleDirectory(administratorsRoleId.Value, usersRoleId.Value);
        }

        public virtual RouteItemTypeDirectory GetRouteItemTypeDirectory()
        {
            if (!_con.IsOpen)
            {
                throw new DcConnectionClosedException();
            }

            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                "select rit1.RouteItemTypeId as Decision, rit2.RouteItemTypeId as Notification " +
                "from RouteItemType as rit1 inner join RouteItemType as rit2 " +
                "on rit2.Name='Notification' where rit1.Name='Decision';";
            MySqlDataReader reader = cmd.ExecuteReader();
            bool result = reader.Read();
            if (!result)
            {
                throw new DcObjectNotFoundException("RouteItemType table is empty!");
            }

            RouteItemTypeDirectory retVal = new RouteItemTypeDirectory(
                reader.GetInt32("Decision"),
                reader.GetInt32("Notification")
                );
            reader.Close();
            return retVal;
        }

        public virtual DocStatusDirectory GetDocStatusDirectory()
        {
            if (!_con.IsOpen)
            {
                throw new DcConnectionClosedException();
            }

            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                "select notSaved.DocStatusId as NotSaved, draft.DocStatusId as Draft, " +
                "underCons.DocStatusId as UnderCons, finalized.DocStatusId as Finalized " +
                "from DocStatus as notSaved " +
                "inner join DocStatus as draft on draft.Name='DraftDocStatus' " +
                "inner join DocStatus as underCons on underCons.Name='UnderConsDocStatus' " +
                "inner join DocStatus as finalized on finalized.Name='FinalizedDocStatus' " +
                "where notSaved.Name='NotSavedDocStatus';";

            MySqlDataReader reader = cmd.ExecuteReader();
            bool result = reader.Read();
            if (!result)
            {
                throw new DcObjectNotFoundException("DocStatus table is empty!");
            }

            DocStatusDirectory retVal = new DocStatusDirectory(
                reader.GetInt32("NotSaved"),
                reader.GetInt32("Draft"),
                reader.GetInt32("UnderCons"),
                reader.GetInt32("Finalized")
                );

            reader.Close();
            return retVal;
        }

        public virtual NotificationTypeDirectory GetNotificationTypeDirectory()
        {
            // notification type directory
            NotificationTypeDirectory retVal = new NotificationTypeDirectory();
            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                "select NotificationTypeId, Name from NotificationType;";
            MySqlDataReader reader = cmd.ExecuteReader();
            while (reader.Read())
            {
                int id = reader.GetInt32("NotificationTypeId");
                switch (reader.GetString("Name"))
                {
                    case "NextDecision":
                        retVal.NextDecision = id;
                        break;
                    case "DeputyNextDecision":
                        retVal.DeputyNextDecision = id;
                        break;
                    case "NextNotification":
                        retVal.NextNotification = id;
                        break;
                    case "DecisionAfterReturn":
                        retVal.DecisionAfterReturn = id;
                        break;
                    case "NotificationAfterReturn":
                        retVal.NotificationAfterReturn = id;
                        break;
                    case "Finalization":
                        retVal.Finalization = id;
                        break;
                    case "ReturnToDraftDueToDecision":
                        retVal.ReturnToDraftDueToDecision = id;
                        break;
                    case "ReturnToDraftDueToRouteChange":
                        retVal.ReturnToDraftDueToRouteChange = id;
                        break;
                    case "AcquaintanceRequest":
                        retVal.AcquaintanceRequest = id;
                        break;
                    default:
                        throw new ApplicationException(
                            String.Format("{0} is unknown notification type", reader.GetString("Name"))
                            );
                }
            }
            reader.Close();

            return retVal;
        }

        public virtual EventTypeDirectory GetEventTypeDirectory()
        {
            EventTypeDirectory retVal = new EventTypeDirectory();
            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                "select EventTypeId, Name from EventType;";
            MySqlDataReader reader = cmd.ExecuteReader();
            while (reader.Read())
            {
                int id = reader.GetInt32("EventTypeId");
                switch (reader.GetString("Name"))
                {
                    case "PassDecision":
                        retVal.PassDecision = id;
                        break;
                    case "PassDecisionAutomatically":
                        retVal.PassDecisionAutomatically = id;
                        break;
                    case "PassDecisionByDeputy":
                        retVal.PassDecisionByDeputy = id;
                        break;
                    case "RejectDecision":
                        retVal.RejectDecision = id;
                        break;
                    case "RejectDecisionByDeputy":
                        retVal.RejectDecisionByDeputy = id;
                        break;
                    case "RejectDecisionByAdministrator":
                        retVal.RejectDecisionByAdministrator = id;
                        break;
                    case "PostponeDecision":
                        retVal.PostponeDecision = id;
                        break;
                    case "PostponeDecisionByDeputy":
                        retVal.PostponeDecisionByDeputy = id;
                        break;
                    case "PostponeDecisionByAdministrator":
                        retVal.PostponeDecisionByAdministrator = id;
                        break;
                    case "ReturnToDraftDueToRouteChange":
                        retVal.ReturnToDraftDueToRouteChange = id;
                        break;
                    case "DocCardRegistration":
                        retVal.DocCardRegistration = id;
                        break;
                    case "DocCardFinalization":
                        retVal.DocCardFinalization = id;
                        break;
                    case "ConsiderationStarted":
                        retVal.ConsiderationStarted = id;
                        break;
                    case "PassedToNextRoutePoint":
                        retVal.PassedToNextRoutePoint = id;
                        break;
                    default:
                        throw new ApplicationException(
                            String.Format("{0} is unknown event type", reader.GetString("Name"))
                            );
                }
            }
            reader.Close();

            return retVal;
        }

        public virtual int GetCommonDepartment()
        {
            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                "select DepartmentId from Department where Name='Common';";

            return (int)cmd.ExecuteScalar();
        }

        /// <summary>
        /// unit tests
        /// </summary>
        public DBConstAccess()
        {
        }

        public DBConstAccess(DBConnection con)
        {
            _con = con;
        }
    }
}
