//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using MySql.Data.MySqlClient;

namespace Evpo.DocCirc.MySqlDL
{
    public class DataColumn
    {
        private string _name;
        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }
        
        private MySqlDbType _dbType;
        public MySqlDbType DbType
        {
            get { return _dbType; }
            set { _dbType = value; }
        }

        private bool _nullable;

        public bool Nullable
        {
            get { return _nullable; }
            set { _nullable = value; }
        }


        private bool _autoInc;

        public bool AutoInc
        {
            get { return _autoInc; }
            set { _autoInc = value; }
        }

        public DataColumn(string name, MySqlDbType dbType)
            :this(name, dbType, false, false)
        {
        }

        public DataColumn(string name, MySqlDbType dbType, bool autoInc, bool nullable)
        {
            _dbType = dbType;
            _name = name;
            _autoInc = autoInc;
            _nullable = nullable;
        }

        public void AddParameter(MySqlCommand cmd)
        {
            if (!cmd.Parameters.Contains(Name))
            {
                cmd.Parameters.Add(Name, DbType);
            }
        }

        public void SetParameter(MySqlCommand cmd, object value)
        {
            if (!cmd.Parameters.Contains(Name))
            {
                cmd.Parameters.Add(Name, DbType);
            }
            cmd.Parameters[Name].Value = value;
        }

        public bool IsDBNull(MySqlDataReader reader)
        {
            return reader.IsDBNull(reader.GetOrdinal(Name));
        }

    }

    

    public class DataTbl
    {
        public enum SqlCmdType
        {
            SelectColumnPart,
            InsertColumnPart,
            InsertValuePart,
            UpdateSetPart
        }


        protected DataColumn[] _items;

        public DataColumn[] Items
        {
            get { return _items; }
            set { _items = value; }
        }

        public string GetColumnPart()
        {
            return GetColumnPart(DataTbl.SqlCmdType.SelectColumnPart);
        }

        public string GetColumnPart(DataTbl.SqlCmdType cmdType)
        {
            StringBuilder sb = new StringBuilder();

            foreach (DataColumn col in _items)
            {
                if (
                    (cmdType != DataTbl.SqlCmdType.InsertColumnPart &&
                        cmdType != DataTbl.SqlCmdType.InsertValuePart &&
                        cmdType != DataTbl.SqlCmdType.UpdateSetPart
                    )
                    ||
                    !col.AutoInc
                    )
                {

                    if (sb.Length > 0)
                    {
                        sb.Append(", ");
                    }

                    if (cmdType == DataTbl.SqlCmdType.InsertValuePart)
                    {
                        sb.AppendFormat("?{0}", col.Name);
                    }
                    else if (cmdType == DataTbl.SqlCmdType.UpdateSetPart)
                    {
                        sb.AppendFormat("{0}=?{0}", col.Name);
                    }
                    else
                    {
                        sb.Append(col.Name);
                    }
                }
            }

            return sb.ToString();
        }
    }

}
