//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using MySql.Data.MySqlClient;
using System.IO;
using System.Text.RegularExpressions;

namespace Evpo.DocCirc.MySqlDL
{
    public class DatabaseInstaller
    {
        public struct Const
        {
            public const string MachineName = "localhost";
            public const int Port = 3306;
            public const string Admin = "root";
            public const string ServiceUserName = "EvpoDCService";
            public const string UserName = "EvpoDCUser";
            public const string DatabaseName = "EvpoDCData";
            public const int ConnectionTimeOut = 100;
            public const string ConnectionString =
                "Database={0};Data Source={1};User Id={2};Password={3};Connect Timeout={4};Port={5}";
            public const string AttachmentDirectory = "edcfiles";
            public const string LogFileName = "EvpoDocCirc.log";
        }

        private string _databaseError = null;

        public string DatabaseError
        {
            get { return _databaseError; }
            set { _databaseError = value; }
        }

        public bool CreateAndCheckAttachmentDirectory(string fullPath)
        {
            bool result = true;
            try
            {
                if (!Directory.Exists(fullPath))
                {
                    Directory.CreateDirectory(fullPath);
                }

                string folder01Path = Path.Combine(fullPath, "Folder01");
                Directory.CreateDirectory(folder01Path);
                File.WriteAllText(Path.Combine(folder01Path, "file01.txt"), "file01 contents");

                File.Delete(Path.Combine(folder01Path, "file01.txt"));
                Directory.Delete(folder01Path);
            }
            catch (Exception ex)
            {
                result = false;
                _databaseError = ex.Message;
            }

            return result;
        }

        public bool CheckConnection(string connString)
        {
            MySqlConnection con = new MySqlConnection(connString);
                
            bool pingResult = false;
            try
            {
                con.Open();
                pingResult = con.Ping();
                con.Close();
            }
            catch (MySqlException ex)
            {
                _databaseError = ex.Message;
            }
            catch (Exception ex)
            {
                throw ex;
            }

            return pingResult;
        }

        public string GetConnectionString(
            string dataBase,
            string dataSource,
            string user,
            string password,
            int timeOut,
            string port
            )
        {
            return 
                String.Format(Const.ConnectionString, dataBase, dataSource, user, password, timeOut, port);
        }

        public bool CreateUsers(
            string connString,
            string databaseName,
            string applicationUser,
            string applicationUserPassword,
            string serviceUser,
            string serviceUserPassword,
            bool isLocalhost
            )
        {
            MySqlConnection con = new MySqlConnection(connString);
            MySqlCommand cmd = con.CreateCommand();
            cmd.CommandText = String.Format(
                AppResource.CreateUsers, 
                databaseName,
                applicationUser,
                applicationUserPassword,
                serviceUser,
                serviceUserPassword,
                isLocalhost ? "-- " : "");

            bool result = true;
            try
            {
                con.Open();
                cmd.ExecuteNonQuery();
                con.Close();
            }
            catch (MySqlException ex)
            {
                _databaseError = ex.Message;
                result = false;
            }
            catch (Exception ex)
            {
                throw ex;
            }

            return result;
        }

        public bool CreateDatabase(string databaseName, string connString)
        {
            MySqlConnection con = new MySqlConnection(connString);
            MySqlCommand cmd = con.CreateCommand();
            string commandText = String.Format(AppResource.CreateDatabase, databaseName);

            bool result = true;
            try
            {
                con.Open();
                Regex oRegex = new Regex("[^;]+;", RegexOptions.None);
                Match oMatch = oRegex.Match(commandText);
                while (oMatch.Success)
                {
                    cmd.CommandText = oMatch.Value;
                    cmd.ExecuteNonQuery();
                    oMatch = oMatch.NextMatch();
                }
                
                con.Close();
            }
            catch (MySqlException ex)
            {
                _databaseError = ex.Message;
                result = false;
            }
            catch (Exception ex)
            {
                throw ex;
            }

            return result;
        }

        
    }
}
