//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using MySql.Data;
using MySql.Data.MySqlClient;
using MySql.Data.Types;
using Evpo.DocCirc.Common;

namespace Evpo.DocCirc.MySqlDL
{
    public class DocCardAccess
    {

        protected class Tbl : DataTbl
        {
            public DataColumn DocCardId =
                new DataColumn("DocCardId", MySqlDbType.Int64, true, false);

            public DataColumn Folder =
                new DataColumn("Folder", MySqlDbType.Int32);

            public DataColumn DocType =
                new DataColumn("DocType", MySqlDbType.Int32, false, true);

            public DataColumn DocStatus =
                new DataColumn("DocStatus", MySqlDbType.Int32);

            public DataColumn RegisteringUser =
                new DataColumn("RegisteringUser", MySqlDbType.Int32);

            public DataColumn DcUser =
                new DataColumn("DcUser", MySqlDbType.Int32);

            public DataColumn LastMovement =
                new DataColumn("LastMovement", MySqlDbType.Datetime, false, true);

            public DataColumn Subject =
                new DataColumn("Subject", MySqlDbType.String);

            public DataColumn InternalNumber =
                new DataColumn("InternalNumber", MySqlDbType.String);

            public DataColumn RegistrationTime =
                new DataColumn("RegistrationTime", MySqlDbType.Datetime);

            public DataColumn FinalizingTime =
                new DataColumn("FinalizingTime", MySqlDbType.Datetime, false, true);

            public DataColumn AttachmentsAreChangeable =
                new DataColumn("AttachmentsAreChangeable", MySqlDbType.Bit);

            public DataColumn OriginalDate =
                new DataColumn("OriginalDate", MySqlDbType.Datetime, false, true);

            public DataColumn OriginalNumber =
                new DataColumn("OriginalNumber", MySqlDbType.String);

            public DataColumn SendingCompany =
                new DataColumn("SendingCompany", MySqlDbType.Int32, false, true);

            public DataColumn ReceivingCompany =
                new DataColumn("ReceivingCompany", MySqlDbType.Int32, false, true);

            public DataColumn CustomSendingCompany =
                new DataColumn("CustomSendingCompany", MySqlDbType.String);

            public DataColumn CustomReceivingCompany =
                new DataColumn("CustomReceivingCompany", MySqlDbType.String);

            public DataColumn Note =
                new DataColumn("Note", MySqlDbType.Text);

            public DataColumn RoutePointNum =
                new DataColumn("RoutePointNum", MySqlDbType.Int32);

            public DataColumn IsNew =
                new DataColumn("IsNew", MySqlDbType.Bit);

            public Tbl()
            {
                _items = new DataColumn[]
                {
                    DocCardId,
                    Folder,
                    DocType,
                    DocStatus,
                    RegisteringUser,
                    DcUser,
                    LastMovement,
                    Subject,
                    InternalNumber,
                    RegistrationTime,
                    FinalizingTime,
                    AttachmentsAreChangeable,
                    OriginalDate,
                    OriginalNumber,
                    SendingCompany,
                    ReceivingCompany,
                    CustomSendingCompany,
                    CustomReceivingCompany,
                    Note,
                    RoutePointNum,
                    IsNew
                };
            }
        }

        protected Tbl _tbl = new Tbl();
        
        protected DBConnection _con;
        protected int _userId;
        protected SettingKeeper _sk;

        public DocCardAccess(int userId, DBConnection con, SettingKeeper sk)
        {
            _userId = userId;
            _con = con;
            _sk = sk;
        }

        public DocCardItem CreateDocCard(int folder)
        {
            if (!_con.IsOpen)
            {
                throw new DcConnectionClosedException();
            }

            SecurityAccess sa = new SecurityAccess(_userId, _con);
            if (!sa.GetFolderSecurity(folder).Create)
            {
                throw new DcUnathorizedAccessException(DcUnathorizedAccessException.PolicyObjectType.Folder, "");
            }

            DocCardItem item = new DocCardItem();
            item.IsNew = true;
            item.Folder = folder;
            item.DocType = null;
            item.DocStatus = DBConstKeeper.Current.DocStatus.NotSaved;
            item.RegisteringUser = _userId;
            item.Owner = _userId;
            item.LastMovement = null;
            item.Subject = "";
            item.InternalNumber = "";
            item.RegistrationTime = DateTime.Now;
            item.FinalizingTime = null;
            item.AttachmentsAreChangeable = true;
            item.OriginalDate = null;
            item.OriginalNumber = "";
            item.SendingCompany = null;
            item.ReceivingCompany = null;
            item.CustomSendingCompany = "";
            item.CustomReceivingCompany = "";
            item.Note = "";
            item.RoutePointNum = 0;
            
            //item.SetRouteIsLoaded(true);

            StringBuilder fieldSb = new StringBuilder();
            StringBuilder valueSb = new StringBuilder();

            bool first = true;
            foreach (DataColumn col in _tbl.Items)
            {
                if (!col.AutoInc)
                {
                    if (first)
                    {
                        first = false;
                    }
                    else
                    {
                        fieldSb.Append(", ");
                        valueSb.Append(", ");
                    }

                    fieldSb.Append(col.Name);
                    valueSb.AppendFormat("?{0}", col.Name);
                }
            }

            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = String.Format(
                "insert into DocCard ({0}) Values ({1});",
                fieldSb.ToString(),
                valueSb.ToString()  
                );

            _tbl.Folder.SetParameter(cmd, item.Folder);
            _tbl.DocType.SetParameter(cmd, item.DocType);
            _tbl.DocStatus.SetParameter(cmd, item.DocStatus);
            _tbl.RegisteringUser.SetParameter(cmd, item.RegisteringUser);
            _tbl.DcUser.SetParameter(cmd, item.Owner);
            _tbl.LastMovement.SetParameter(cmd, item.LastMovement);
            _tbl.Subject.SetParameter(cmd, item.Subject);
            _tbl.InternalNumber.SetParameter(cmd, item.InternalNumber);
            _tbl.RegistrationTime.SetParameter(cmd, item.RegistrationTime);
            _tbl.FinalizingTime.SetParameter(cmd, item.FinalizingTime);
            _tbl.AttachmentsAreChangeable.SetParameter(cmd, item.AttachmentsAreChangeable);
            _tbl.OriginalDate.SetParameter(cmd, item.OriginalDate);
            _tbl.OriginalNumber.SetParameter(cmd, item.OriginalNumber);
            _tbl.SendingCompany.SetParameter(cmd, item.SendingCompany);
            _tbl.ReceivingCompany.SetParameter(cmd, item.ReceivingCompany);
            _tbl.CustomSendingCompany.SetParameter(cmd, item.CustomSendingCompany);
            _tbl.CustomReceivingCompany.SetParameter(cmd, item.CustomReceivingCompany);
            _tbl.Note.SetParameter(cmd, item.Note);
            _tbl.RoutePointNum.SetParameter(cmd, item.RoutePointNum);
            _tbl.IsNew.SetParameter(cmd, item.IsNew);
            
            cmd.ExecuteNonQuery();
            item.Id = cmd.LastInsertedId;

            return item;
        }

        public DocCardItem LoadDocCard(long id, bool loadRoute)
        {
            if (!_con.IsOpen)
            {
                throw new DcConnectionClosedException();
            }

            StringBuilder sb = new StringBuilder();
            sb.Append("select ");
            bool first = true;
            foreach (DataColumn col in _tbl.Items)
            {
                if (first)
                {
                    first = false;
                }
                else
                {
                    sb.Append(", ");
                }
                sb.Append(col.Name);
            }

            sb.AppendFormat(" from DocCard where {0}=?{0};", _tbl.DocCardId.Name);

            MySqlCommand cmd = _con.CreateCommand();
            _tbl.DocCardId.SetParameter(cmd, id);
            cmd.CommandText = sb.ToString();

            MySqlDataReader reader = cmd.ExecuteReader();
            bool readResult = reader.Read();
            if (!readResult)
            {
                reader.Close();
                throw new DcObjectNotFoundException();
            }

            DocCardItem item = new DocCardItem();
            item.Id = id;
            item.Folder = reader.GetInt32(_tbl.Folder.Name);

            if (!_tbl.DocType.IsDBNull(reader))
            {
                item.DocType = reader.GetInt32(_tbl.DocType.Name);
            }

            if (!_tbl.LastMovement.IsDBNull(reader))
            {
                item.LastMovement = reader.GetDateTime(_tbl.LastMovement.Name);
            }

            if (!_tbl.FinalizingTime.IsDBNull(reader))
            {
                item.FinalizingTime = reader.GetDateTime(_tbl.FinalizingTime.Name);
            }

            if (!_tbl.OriginalDate.IsDBNull(reader))
            {
                item.OriginalDate = reader.GetDateTime(_tbl.OriginalDate.Name);
            }

            if (!_tbl.SendingCompany.IsDBNull(reader))
            {
                item.SendingCompany = reader.GetInt32(_tbl.SendingCompany.Name);
            }

            if (!_tbl.ReceivingCompany.IsDBNull(reader))
            {
                item.ReceivingCompany = reader.GetInt32(_tbl.ReceivingCompany.Name);
            }

            item.DocStatus = reader.GetInt32(_tbl.DocStatus.Name);
            item.RegisteringUser = reader.GetInt32(_tbl.RegisteringUser.Name);
            item.Owner = reader.GetInt32(_tbl.DcUser.Name);
            
            item.Subject = reader.GetString(_tbl.Subject.Name);
            item.InternalNumber = reader.GetString(_tbl.InternalNumber.Name);
            item.RegistrationTime = reader.GetDateTime(_tbl.RegistrationTime.Name);
            
            item.AttachmentsAreChangeable = reader.GetBoolean(_tbl.AttachmentsAreChangeable.Name);
            
            item.OriginalNumber = reader.GetString(_tbl.OriginalNumber.Name);
            item.CustomSendingCompany = reader.GetString(_tbl.CustomSendingCompany.Name);
            item.CustomReceivingCompany = reader.GetString(_tbl.CustomReceivingCompany.Name);
            item.Note = reader.GetString(_tbl.Note.Name);
            item.RoutePointNum = reader.GetInt32(_tbl.RoutePointNum.Name);
            item.IsNew = reader.GetBoolean(_tbl.IsNew.Name);
            reader.Close();

            SecurityAccess sa = new SecurityAccess(_userId, _con);
            SecurityItem fldSI = sa.GetFolderSecurity(item.Folder);
            SecurityItem docTypeSI = null;
            if (item.DocType != null)
            {
                 docTypeSI = sa.GetDocTypeDocumentsSecurity(item.DocType.Value);
            }
            
            if(!fldSI.Read && !(fldSI.ReadOwn && item.Owner == _userId))
            {
                throw new DcUnathorizedAccessException(
                    DcUnathorizedAccessException.PolicyObjectType.Folder,
                    ""
                    );
            }


            if (item.DocType != null && !docTypeSI.Read && !(docTypeSI.ReadOwn && item.Owner == _userId))
            {
                throw new DcUnathorizedAccessException(
                    DcUnathorizedAccessException.PolicyObjectType.DocType,
                    ""
                    );
            }

            if (loadRoute)
            {
                LoadDocCardRoute(item);
            }

            return item;

        }

        private class DocCardRouteItemData
        {
            private long _id;
            public long Id
            {
                get { return _id; }
                set { _id = value; }
            }

            private bool _passed;
            public bool Passed
            {
                get { return _passed; }
                set { _passed = value; }
            }

            private bool _underCons;

            public bool UnderCons
            {
                get { return _underCons; }
                set { _underCons = value; }
            }


            public DocCardRouteItemData(long id, bool passed)
            {
                _id = id;
                _passed = passed;
                if (!_passed)
                {
                    _underCons = true;
                }

            }
        }

        public void LoadDocCardRoute(DocCardItem item)
        {
            if (item.DocType == null)
            {
                throw new ApplicationException("DocType is null!");
            }

            DocTypeAccess dta = new DocTypeAccess(_userId, _con);
            bool loadRoute = true;
            DocTypeItem docType = dta.LoadDocTypeItem(item.DocType.Value, loadRoute);

            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                "select DocCardRouteItemId, RouteItem, Passed from DocCardRouteItem " +
                "where DocCard=?DocCard;";
            cmd.Parameters.Add("?DocCard", MySqlDbType.Int64);
            cmd.Parameters["?DocCard"].Value = item.Id;



            SortedList<int, DocCardRouteItemData> docCardRouteItemsData = new SortedList<int, DocCardRouteItemData>();
            MySqlDataReader reader = cmd.ExecuteReader();
            while (reader.Read())
            {
                docCardRouteItemsData.Add(
                    reader.GetInt32("RouteItem"),
                    new DocCardRouteItemData(reader.GetInt64("DocCardRouteItemId"), reader.GetBoolean("Passed"))
                    );
            }
            reader.Close();

            RoutePoint[] routePoints = new RoutePoint[docType.RoutePoints.Count];
            for (int p = 0; p < routePoints.Length; p++)
            {
                RoutePoint rp = docType.RoutePoints[p];
                routePoints[p] = new RoutePoint(rp.Id, rp.Expression, rp.OrderNum);

                for (int i = 0; i < rp.RouteItems.Count; i++)
                {
                    DocCardRouteItem dcri = null;
                    if(docCardRouteItemsData.ContainsKey(rp.RouteItems[i].Id.Value))
                    {
                        DocCardRouteItemData riData = docCardRouteItemsData[rp.RouteItems[i].Id.Value];
                        dcri = new DocCardRouteItem(riData.Id, riData.Passed, riData.UnderCons, rp.RouteItems[i]);
                    }
                    else
                    {
                        dcri = new DocCardRouteItem(null, false, false, rp.RouteItems[i]);
                    }

                    routePoints[p].RouteItems.Add(dcri);
                }
            }

            item.RoutePoints = routePoints;

            item.SetRouteIsLoaded(true);
        }

        public void SaveDocCard(DocCardItem item)
        {
            if (!_con.IsOpen)
            {
                throw new DcConnectionClosedException();
            }

            if (item.DocType == null)
            {
                throw new ApplicationException("Doc type is not set befor saving!");
            }

            if (item.DocStatus == DBConstKeeper.Current.DocStatus.NotSaved)
            {
                throw new ApplicationException("DocStatus can't be 'not saved' while saving!");
            }

            SecurityAccess sa = new SecurityAccess(_userId, _con);
            SecurityItem fldSI = sa.GetFolderSecurity(item.Folder);
            SecurityItem docTypeSI = sa.GetDocTypeDocumentsSecurity(item.DocType.Value);

            if (!fldSI.Update && !(fldSI.UpdateOwn && item.Owner == _userId))
            {
                throw new DcUnathorizedAccessException(
                    DcUnathorizedAccessException.PolicyObjectType.Folder,
                    ""
                    );
            }

            if (!docTypeSI.Update && !(docTypeSI.UpdateOwn && item.Owner == _userId))
            {
                throw new DcUnathorizedAccessException(
                    DcUnathorizedAccessException.PolicyObjectType.DocType,
                    ""
                    );
            }

            if (item.IsNew && !docTypeSI.Create)
            {
                throw new DcUnathorizedAccessException(
                    DcUnathorizedAccessException.PolicyObjectType.DocType,
                    ""
                    );
            }

            if (item.IsNew)
            {
                item.IsNew = false;
            }

            StringBuilder sb = new StringBuilder();
            sb.Append("update DocCard set ");

            bool first = true;
            foreach (DataColumn col in _tbl.Items)
            {
                if (!col.AutoInc)
                {
                    if (first)
                    {
                        first = false;
                    }
                    else
                    {
                        sb.Append(", ");
                    }

                    sb.AppendFormat("{0}=?{0}", col.Name);
                }
            }

            sb.AppendFormat(" where {0}=?{0};", _tbl.DocCardId.Name);

            MySqlCommand cmd = _con.CreateCommand();

            _tbl.DocCardId.SetParameter(cmd, item.Id);
            _tbl.Folder.SetParameter(cmd, item.Folder);
            _tbl.DocType.SetParameter(cmd, item.DocType);
            _tbl.DocStatus.SetParameter(cmd, item.DocStatus);
            _tbl.RegisteringUser.SetParameter(cmd, item.RegisteringUser);
            _tbl.DcUser.SetParameter(cmd, item.Owner);
            _tbl.LastMovement.SetParameter(cmd, item.LastMovement);
            _tbl.Subject.SetParameter(cmd, item.Subject);
            _tbl.InternalNumber.SetParameter(cmd, item.InternalNumber);
            _tbl.RegistrationTime.SetParameter(cmd, item.RegistrationTime);
            _tbl.FinalizingTime.SetParameter(cmd, item.FinalizingTime);
            _tbl.AttachmentsAreChangeable.SetParameter(cmd, item.AttachmentsAreChangeable);
            _tbl.OriginalDate.SetParameter(cmd, item.OriginalDate);
            _tbl.OriginalNumber.SetParameter(cmd, item.OriginalNumber);
            _tbl.CustomSendingCompany.SetParameter(cmd, item.CustomSendingCompany);
            _tbl.CustomReceivingCompany.SetParameter(cmd, item.CustomReceivingCompany);
            _tbl.Note.SetParameter(cmd, item.Note);
            _tbl.RoutePointNum.SetParameter(cmd, item.RoutePointNum);
            _tbl.IsNew.SetParameter(cmd, item.IsNew);
            _tbl.SendingCompany.SetParameter(cmd, item.SendingCompany);
            _tbl.ReceivingCompany.SetParameter(cmd, item.ReceivingCompany);

            cmd.CommandText = sb.ToString();

            cmd.ExecuteNonQuery();

            if (item.RouteIsLoaded)
            {
                SaveDocCardRoute(item);
            }

            
        }

        protected void SaveDocCardRoute(DocCardItem item)
        {
            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                "create temporary table if not exists tmpDocCardRouteItem(DocCardRouteItem bigint(20));" +
                "delete from tmpDocCardRouteItem;";
            cmd.ExecuteNonQuery();

            MySqlCommand cmdTempTable = _con.CreateCommand();
            cmdTempTable.CommandText = 
                "insert into tmpDocCardRouteItem values(?DocCardRouteItem);";
            cmdTempTable.Parameters.Add("?DocCardRouteItem", MySqlDbType.Int64);


            cmd.Parameters.Add("?DocCardRouteItem", MySqlDbType.Int64);
            cmd.Parameters.Add("?DocCard", MySqlDbType.Int64);
            cmd.Parameters.Add("?RouteItem", MySqlDbType.Int32);
            cmd.Parameters.Add("?Passed", MySqlDbType.Bit);

            cmd.Parameters["?DocCard"].Value = item.Id;

            foreach (RoutePoint rp in item.RoutePoints)
            {
                foreach (RouteItem ri in rp.RouteItems)
                {
                    DocCardRouteItem dri = (DocCardRouteItem)ri;
                    if (dri.Passed || dri.UnderCons)
                    {
                        cmd.Parameters["?RouteItem"].Value = dri.Id;
                        cmd.Parameters["?Passed"].Value = dri.Passed;

                        if (dri.DocRouteItemId == null)
                        {
                            cmd.CommandText =
                                "insert into DocCardRouteItem (DocCard, RouteItem, Passed) " +
                                "Values (?DocCard, ?RouteItem, ?Passed);";
                            cmd.ExecuteNonQuery();
                            dri.DocRouteItemId = cmd.LastInsertedId;
                        }
                        else
                        {
                            cmd.Parameters["?DocCardRouteItem"].Value = dri.DocRouteItemId.Value;
                            cmd.CommandText =
                                "update DocCardRouteItem set DocCard=?DocCard, RouteItem=?RouteItem, " +
                                "Passed=?Passed where DocCardRouteItemId=?DocCardRouteItem;";
                            cmd.ExecuteNonQuery();
                        }

                        cmdTempTable.Parameters["?DocCardRouteItem"].Value = dri.DocRouteItemId;
                        cmdTempTable.ExecuteNonQuery();
                    }
                }

            }
            cmd.CommandText = "delete from DocCardRouteItem where DocCard=?DocCard and " +
                    "DocCardRouteItemId not in (select DocCardRouteItem from tmpDocCardRouteItem);";
            cmd.ExecuteNonQuery();
        }

        public void DeleteDocCard(long id)
        {
            if (!_con.IsOpen)
            {
                throw new DcConnectionClosedException();
            }

            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                "select Folder, DocType, DcUser, IsNew from DocCard where DocCardId=?DocCard;";

            cmd.Parameters.Add("?DocCard", MySqlDbType.Int64);
            cmd.Parameters["?DocCard"].Value = id;
            MySqlDataReader reader = cmd.ExecuteReader();
            int? docType = null;
            int folder = 0, owner = 0;
            bool isNew = false;
            if (reader.Read())
            {
                if (!reader.IsDBNull(1))
                {
                    docType = reader.GetInt32("DocType");
                }
                folder = reader.GetInt32("Folder");
                owner = reader.GetInt32("DcUser");
                isNew = reader.GetBoolean("IsNew");
            }
            reader.Close();

            SecurityAccess sa = new SecurityAccess(_userId, _con);
            SecurityItem fldSI = sa.GetFolderSecurity(folder);

            if (!fldSI.Delete && !(fldSI.DeleteOwn && owner == _userId))
            {
                throw new DcUnathorizedAccessException(
                    DcUnathorizedAccessException.PolicyObjectType.Folder,
                    ""
                    );
            }

            if (docType != null)
            {
                SecurityItem docTypeSI = sa.GetDocTypeDocumentsSecurity(docType.Value);
                if (!docTypeSI.Delete && !(docTypeSI.DeleteOwn && owner == _userId))
                {
                    throw new DcUnathorizedAccessException(
                        DcUnathorizedAccessException.PolicyObjectType.DocType,
                        ""
                        );
                }
            }

            if (!isNew)
            {
                // delete attachments
                AttachmentAccess aa = new AttachmentAccess(_userId, _con, _sk);
                AttachmentItem[] attItems = aa.GetDocCardAttachmentItems(id);
                foreach (AttachmentItem attItem in attItems)
                {
                    aa.DeleteAttachment(attItem);
                }

                // delete references
                DocCardReferenceAccess ra = new DocCardReferenceAccess(_userId, _con, _sk);
                ra.DeleteDocCardReferences(id);
            }


            cmd.CommandText =
                "delete from UserNotification where DocCard=?DocCard;" +
                "delete from Event where DocCard=?DocCard;" +
                "delete from DocCardRouteItem where DocCard=?DocCard;" +
                "delete from DocCard where DocCardId=?DocCard;";
            
            cmd.ExecuteNonQuery();

        }



        protected string GetConditionExpression(
            MySqlCommand cmd, 
            DocCardFilterItem[] filterItems, 
            SortedList<string, DocCardField> fieldDefList
            )
        {
            StringBuilder sb = new StringBuilder();
            foreach (DocCardFilterItem filterItem in filterItems)
            {
                if (sb.Length == 0)
                {
                    sb.Append("(");
                }
                else
                {
                    sb.Append(" AND ");
                }

                bool isPeriod = filterItem is PeriodDocCardFilterItem;
                DocCardField fieldDef = fieldDefList[filterItem.FieldName];
                sb.Append(GetLogicExpression(fieldDef, isPeriod, filterItem.SearchType));

                if (!isPeriod)
                {
                    string paramName = String.Format("?{0}", fieldDef.TableColumn);
                    cmd.Parameters.Add(paramName, fieldDef.DbType);
                    if (filterItem.SearchType == DocCardSearchType.Exact)
                    {
                        cmd.Parameters[paramName].Value = filterItem.FieldValue;
                    }
                    else if (filterItem.SearchType == DocCardSearchType.Begins)
                    {
                        cmd.Parameters[paramName].Value = String.Format("{0}%", filterItem.FieldValue);
                    }
                    else if (filterItem.SearchType == DocCardSearchType.Contains)
                    {
                        cmd.Parameters[paramName].Value = String.Format("%{0}%", filterItem.FieldValue);
                    }
                }
                else
                {
                    PeriodDocCardFilterItem period = (PeriodDocCardFilterItem)filterItem;
                    string startParamName = String.Format("?{0}Start", fieldDef.TableColumn);
                    string endParamName = String.Format("?{0}End", fieldDef.TableColumn);
                    cmd.Parameters.Add(startParamName, fieldDef.DbType);
                    cmd.Parameters.Add(endParamName, fieldDef.DbType);
                    cmd.Parameters[startParamName].Value = period.StartOfPeriod;
                    cmd.Parameters[endParamName].Value = period.EndOfPeriod;
                }
            }

            if (sb.Length > 0)
            {
                sb.Append(")");
            }
            return sb.ToString();
        }

        protected string GetUnderConsJoinPart(int[] consideringEmployees)
        {
            StringBuilder sb = new StringBuilder();
            foreach (int employee in consideringEmployees)
            {
                if (sb.Length > 0)
                {
                    sb.Append(",");
                }
                sb.Append(employee.ToString());
            }

            return String.Format(
                    "inner join DocCardRouteItem on DocCard.DocCardId=DocCardRouteItem.DocCard and " +
                    "NOT DocCardRouteItem.Passed " +
                    "inner join RouteItem on RouteItem.RouteItemId=DocCardRouteItem.RouteItem and " +
                    "RouteItem.DcUser in ({0})",
                    sb.ToString()
                    );
        }

        protected string GetSecurityJoinPart(MySqlCommand cmd, int userId)
        {
            string retVal =
               "inner join tmpFolderSecurity " +
               "on DocCard.Folder=tmpFolderSecurity.Folder and tmpFolderSecurity.DcUser=?SessionUser and " +
               "(tmpFolderSecurity.R or (tmpFolderSecurity.RO and DocCard.DcUser=?SessionUser)) " +
               "inner join tmpDocTypeSecurity " +
               "on DocCard.DocType=tmpDocTypeSecurity.DocType and tmpDocTypeSecurity.DcUser=?SessionUser and " +
               "(tmpDocTypeSecurity.R or (tmpDocTypeSecurity.RO and DocCard.DcUser=?SessionUser))";
            cmd.Parameters.Add("?SessionUser", MySqlDbType.Int32);
            cmd.Parameters["?SessionUser"].Value = userId;
            return retVal;
        }


        public DataTable GetDocumentTable(string[] fields, DocCardFilterItem[] filterItems)
        {
            return GetDocumentTable(fields, filterItems, new int[0]);
        }

        public DataTable GetDocumentTable(string[] fields, DocCardFilterItem[] filterItems, int[] consideringEmployees)
        {
            if (!_con.IsOpen)
            {
                throw new DcConnectionClosedException();
            }

            SortedList<string, DocCardField> fieldDefList = new SortedList<string, DocCardField>(
                DBFieldDefKeeper.Current.FieldDefs.Length
                );

            foreach (DocCardField fieldDef in DBFieldDefKeeper.Current.FieldDefs)
            {
                fieldDefList.Add(fieldDef.FieldName, fieldDef);
            }

            SecurityAccess sa = new SecurityAccess(_userId, _con);
            sa.CreateDocTypeSecurityTable();
            sa.CreateFolderSecurityTable();

            MySqlCommand cmd = _con.CreateCommand();

            // create fields part of query
            StringBuilder sb = new StringBuilder();
            foreach (string fld in fields)
            {
                if (sb.Length > 0)
                {
                    sb.Append(", ");
                }
                sb.AppendFormat("DocCard.{0}", fieldDefList[fld].TableColumn);
            }

            string fieldQueryPart = sb.ToString();

            //create the where part of query
            string conditionExpression = GetConditionExpression(cmd, filterItems, fieldDefList);
            //consideringEmployees filter
            string joinPartForUnderCons = null;
            if (consideringEmployees.Length > 0)
            {
                joinPartForUnderCons = GetUnderConsJoinPart(consideringEmployees);
            }

            string joinPartForSecurity = GetSecurityJoinPart(cmd, _userId);

            sb = new StringBuilder();
            sb.AppendFormat(
                "select {0} from DocCard {1} {2} where NOT IsNew",
                fieldQueryPart,
                joinPartForSecurity,
                joinPartForUnderCons
                );

            if (conditionExpression != "")
            {
                sb.AppendFormat(" AND {0}", conditionExpression);
            }

            cmd.CommandText = sb.ToString();

            //System.IO.File.WriteAllText(@"g:\projects\tmp\query.txt", cmd.CommandText); // for debugging

            MySqlDataReader reader = cmd.ExecuteReader();

            DataTable table = new DataTable();
            foreach (string field in fields)
            {
                DocCardField fieldDef = fieldDefList[field];
                table.Columns.Add(fieldDef.FieldName, GetUnderlyingType(fieldDef.DbType));
            }

            while(reader.Read())
            {
                DataRow row = table.NewRow();
                for (int i = 0; i < reader.FieldCount; i++)
                {
                    if (!reader.IsDBNull(i))
                    {
                        row[fields[i]] = reader.GetValue(i);
                    }
                    else
                    {
                        row[fields[i]] = DBNull.Value;
                    }
                }
                table.Rows.Add(row);
            }
            reader.Close();

            return table;

        }

        private Type GetUnderlyingType(MySqlDbType dbType)
        {
            Type retVal = null;
            switch(dbType)
            {
                case MySqlDbType.String:
                    retVal = typeof(string);
                    break;
                case MySqlDbType.Datetime:
                    retVal = typeof(DateTime);
                    break;
                case MySqlDbType.Int32:
                    retVal = typeof(int);
                    break;
                case MySqlDbType.Int64:
                    retVal = typeof(long);
                    break;
                case MySqlDbType.Bit:
                    retVal = typeof(bool);
                    break;
                case MySqlDbType.Text:
                    retVal = typeof(string);
                    break;
                default:
                    throw new ApplicationException(
                    String.Format("Unexpected MySqlDbType: {0}!", Enum.GetName(typeof(MySqlDbType), dbType))
                    );
            }

            return retVal;
        }

        private void AddColumnToTable(DataTable table, DocCardField fieldDef)
        {
            
        }

        private string GetLogicExpression(DocCardField fieldDef, bool isPeriod, DocCardSearchType searchType)
        {
            string retVal = null;
            if (isPeriod)
            {
                retVal = String.Format("DocCard.{0} BETWEEN ?{0}Start AND ?{0}End", fieldDef.TableColumn);
            }
            else if(searchType == DocCardSearchType.Exact)
            {
                retVal = String.Format("DocCard.{0}=?{0}", fieldDef.TableColumn);
            }
            else if (searchType == DocCardSearchType.Begins || searchType == DocCardSearchType.Contains)
            {
                retVal = String.Format("DocCard.{0} LIKE ?{0}", fieldDef.TableColumn);
            }
            return retVal;
        }


        public DocCardConsiderationInfo[] GetDocumentConsiderationInfo(DocCardFilterItem[] filterItems)
        {
            return GetDocumentConsiderationInfo(filterItems, new int[0]);
        }

        public DocCardConsiderationInfo[] GetDocumentConsiderationInfo(DocCardFilterItem[] filterItems, int[] consideringEmployees)
        {
            if (!_con.IsOpen)
            {
                throw new DcConnectionClosedException();
            }

            SortedList<string, DocCardField> fieldDefList = new SortedList<string, DocCardField>(
                DBFieldDefKeeper.Current.FieldDefs.Length
                );

            foreach (DocCardField fieldDef in DBFieldDefKeeper.Current.FieldDefs)
            {
                fieldDefList.Add(fieldDef.FieldName, fieldDef);
            }

            SecurityAccess sa = new SecurityAccess(_userId, _con);
            sa.CreateDocTypeSecurityTable();
            sa.CreateFolderSecurityTable();

            MySqlCommand cmd = _con.CreateCommand();

            string fieldQueryPart = "";

            //create the where part of query
            string conditionExpression = GetConditionExpression(cmd, filterItems, fieldDefList);


            ////consideringEmployees filter
            //string partForUnderCons = "";

            //StringBuilder sb = null;
            //if (consideringEmployees.Length > 0)
            //{
            //    sb = new StringBuilder();
            //    foreach (int employee in consideringEmployees)
            //    {
            //        if (sb.Length > 0)
            //        {
            //            sb.Append(",");
            //        }
            //        sb.Append(employee.ToString());
            //    }

            //    partForUnderCons = String.Format(
            //        "(RouteItem.DcUser in ({0}) )",
            //        sb.ToString()
            //        );
            //}

            string joinPartForSecurity = GetSecurityJoinPart(cmd, _userId);

            StringBuilder sb = new StringBuilder();
            sb.AppendFormat(
                "select DocCardRouteItem.DocCard, RouteItem.DcUser, DocCard.RoutePointNum " + 
                "from DocCardRouteItem " +
                "inner join DocCard on DocCardRouteItem.DocCard=DocCard.DocCardId " +
                "inner join RouteItem on DocCardRouteItem.RouteItem = RouteItem.RouteItemId " +
                "{0} " +
                "where NOT IsNew AND NOT DocCardRouteItem.Passed",
                joinPartForSecurity
                );

            //if (partForUnderCons != "")
            //{
            //    sb.AppendFormat(" AND {0}", partForUnderCons);
            //}

            if (conditionExpression != "")
            {
                sb.AppendFormat(" AND {0}", conditionExpression);
            }

            sb.Append(" order by DocCardRouteItem.DocCard, RouteItem.DcUser;");

            cmd.CommandText = sb.ToString();

            //System.IO.File.WriteAllText(@"g:\projects\tmp\query.txt", cmd.CommandText); // for debugging

            List<DocCardConsiderationInfo> consInfoList = new List<DocCardConsiderationInfo>();
            MySqlDataReader reader = cmd.ExecuteReader();
            bool read = reader.Read();
            while (read)
            {
                long docCardId = reader.GetInt64("DocCard");
                int routePointNum = reader.GetInt32("RoutePointNum");

                List<int> employeeList = new List<int>();

                bool inSetOfConsEmployees = false;
                do
                {
                    int user = reader.GetInt32("DcUser");
                    employeeList.Add(user);

                    if (!inSetOfConsEmployees && Array.IndexOf<int>(consideringEmployees, user) >= 0)
                    {
                        inSetOfConsEmployees = true;
                    }
                }
                while ((read = reader.Read()) && reader.GetInt64("DocCard") == docCardId);

                bool addItem = false;
                if (consideringEmployees.Length > 0 && inSetOfConsEmployees)
                {
                    addItem = true;
                }
                else if (consideringEmployees.Length == 0)
                {
                    addItem = true;
                }

                if (addItem)
                {
                    consInfoList.Add(
                           new DocCardConsiderationInfo(
                               docCardId,
                               routePointNum,
                               employeeList.ToArray()
                               )
                           );
                }
            }
            reader.Close();

            return consInfoList.ToArray();
        }
    }
}
