//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using MySql.Data.MySqlClient;
using Evpo.DocCirc.Common;

namespace Evpo.DocCirc.MySqlDL
{
    public class EventAccess
    {
        protected int _userId;
        protected DBConnection _con;

        public EventAccess(int userId, DBConnection con)
        {
            _userId = userId;
            _con = con;
        }

        public EventItem[] GetDocCardEventItems(long docCardId)
        {
            List<EventItem> eventItemList = new List<EventItem>();
            if (!_con.IsOpen)
            {
                throw new DcConnectionClosedException();
            }

            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                "select EventId, DocCard, EventType, DcUser, Deputy, RoutePointNum, EventTime, Note, Text " +
                "from event where DocCard=?DocCard;";

            cmd.Parameters.Add("?DocCard", MySqlDbType.Int64);
            cmd.Parameters["?DocCard"].Value = docCardId;

            MySqlDataReader reader = cmd.ExecuteReader();
            while (reader.Read())
            {
                int? deputy = null;
                if(!reader.IsDBNull(reader.GetOrdinal("Deputy")))
                {
                    deputy = reader.GetInt32("Deputy");
                }

                EventItem item = new EventItem(
                    reader.GetInt64("EventId"),
                    docCardId,
                    reader.GetInt32("EventType"),
                    reader.GetInt32("DcUser"),
                    deputy,
                    reader.GetInt32("RoutePointNum"),
                    reader.GetDateTime("EventTime"),
                    reader.GetString("Note"),
                    reader.GetString("Text")
                    );
                eventItemList.Add(item);
            }
            reader.Close();


            return eventItemList.ToArray();
        }

        public void AddEvent(EventItem item)
        {
            if (!_con.IsOpen)
            {
                throw new DcConnectionClosedException();
            }

            if (item.Id != null)
            {
                throw new ApplicationException("Can't add existent event");
            }

            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                "insert into event (DocCard, EventType, DcUser, Deputy, RoutePointNum, EventTime, Note, Text) " +
                "Values (?DocCard, ?EventType, ?DcUser, ?Deputy, ?RoutePointNum, ?EventTime, ?Note, ?Text);";
            cmd.Parameters.Add("?DocCard", MySqlDbType.Int64);
            cmd.Parameters.Add("?EventType", MySqlDbType.Int32);
            cmd.Parameters.Add("?DcUser", MySqlDbType.Int32);
            cmd.Parameters.Add("?Deputy", MySqlDbType.Int32);
            cmd.Parameters.Add("?RoutePointNum", MySqlDbType.Int32);
            cmd.Parameters.Add("?EventTime", MySqlDbType.Datetime);
            cmd.Parameters.Add("?Note", MySqlDbType.Text);
            cmd.Parameters.Add("?Text", MySqlDbType.Text);

            cmd.Parameters["?DocCard"].Value = item.DocCardId;
            cmd.Parameters["?EventType"].Value = item.EventType;
            cmd.Parameters["?DcUser"].Value = item.EventUser;
            cmd.Parameters["?Deputy"].Value = item.Deputy;
            cmd.Parameters["?RoutePointNum"].Value = item.RoutePointNum;
            cmd.Parameters["?EventTime"].Value = item.EventTime;
            cmd.Parameters["?Note"].Value = item.Note;
            cmd.Parameters["?Text"].Value = item.Text;

            cmd.ExecuteNonQuery();

            item.Id = cmd.LastInsertedId;
        }
    }
}
