//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using MySql.Data.MySqlClient;
using Evpo.DocCirc.Common;

namespace Evpo.DocCirc.MySqlDL
{
    public class FolderAccess
    {
        protected class Tbl : DataTbl
        {
            public DataColumn FolderId = new DataColumn("FolderId", MySqlDbType.Int32, true, false);
            public DataColumn ParentFolder = new DataColumn("ParentFolder", MySqlDbType.Int32);
            public DataColumn Name = new DataColumn("Name", MySqlDbType.String);
            public DataColumn FolderType = new DataColumn("FolderType", MySqlDbType.Int32);

            public Tbl()
            {
                Items = new DataColumn[]
                {
                    FolderId,
                    ParentFolder,
                    Name,
                    FolderType
                };
            }
        }

        protected Tbl _tbl = new Tbl();

        private DBConnection _con;
        private int _userId;

        public FolderAccess(int userId, DBConnection con)
        {
            _con = con;
            _userId = userId;
        }

        public FolderItem GetFolderItem(int folderId)
        {
            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = String.Format(
                "select {0} from Folder where {1}=?{1};",
                _tbl.GetColumnPart(DataTbl.SqlCmdType.SelectColumnPart),
                _tbl.FolderId.Name
                );
            _tbl.FolderId.SetParameter(cmd, folderId);

            FolderItem retVal = null;

            MySqlDataReader reader = cmd.ExecuteReader();
            if (reader.Read())
            {
                int? parentFolder = null;
                if (!_tbl.ParentFolder.IsDBNull(reader))
                {
                    parentFolder = reader.GetInt32(_tbl.ParentFolder.Name);
                }

                retVal = new FolderItem(
                    parentFolder,
                    reader.GetString(_tbl.Name.Name),
                    reader.GetInt32(_tbl.FolderType.Name),
                    reader.GetInt32(_tbl.FolderId.Name)
                    );
            }
            reader.Close();
            

            return retVal;
        }

        public FolderItem[] GetFolderItemsInFolder(int folderId)
        {
            MySqlCommand cmd = _con.CreateCommand();
            _tbl.ParentFolder.SetParameter(cmd, folderId);
            cmd.CommandText = String.Format(
                "select {0} from Folder where {1}=?{1};",
                _tbl.GetColumnPart(DataTbl.SqlCmdType.SelectColumnPart),
                _tbl.ParentFolder.Name
                );
            MySqlDataReader reader = cmd.ExecuteReader();

            List<FolderItem> items = new List<FolderItem>();
            while (reader.Read())
            {
                items.Add(
                    new FolderItem(
                        reader.GetInt32(_tbl.ParentFolder.Name),
                        reader.GetString(_tbl.Name.Name),
                        reader.GetInt32(_tbl.FolderType.Name),
                        reader.GetInt32(_tbl.FolderId.Name)
                        )
                    );
            }
            reader.Close();

            return items.ToArray();
        }

        public void SaveFolderItem(FolderItem item)
        {
            MySqlCommand cmd = _con.CreateCommand();
            _tbl.ParentFolder.SetParameter(cmd, item.Parent);
            _tbl.Name.SetParameter(cmd, item.Name);
            _tbl.FolderType.SetParameter(cmd, item.FolderType);

            if (item.FolderId == null)
            {
                cmd.CommandText = String.Format(
                    "insert into Folder ({0}) Values ({1});",
                    _tbl.GetColumnPart(DataTbl.SqlCmdType.InsertColumnPart),
                    _tbl.GetColumnPart(DataTbl.SqlCmdType.InsertValuePart)
                    );
                cmd.ExecuteNonQuery();
                item.FolderId = (int)cmd.LastInsertedId;
                cmd.CommandText = String.Format(
                    "insert into FolderPermission (Folder, DcRole, C, R, U, D, B, RO, UO, DO) " +
                    "select ?{0}, DcRole, C, R, U, D, B, RO, UO, DO from NewFolderPermission " +
                    "where {1}=?{1};",
                    _tbl.FolderId.Name,
                    _tbl.FolderType.Name
                    );
                _tbl.FolderId.SetParameter(cmd, item.FolderId);
                cmd.ExecuteNonQuery();
            }
            else
            {
                _tbl.FolderId.SetParameter(cmd, item.FolderId);
                cmd.CommandText = String.Format(
                    "update Folder set {0} where {1}=?{1};",
                    _tbl.GetColumnPart(DataTbl.SqlCmdType.UpdateSetPart),
                    _tbl.FolderId.Name
                    );
                cmd.ExecuteNonQuery();
            }
        }

        public void DeleteFolder(int folderId)
        {
            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = String.Format(
                "delete from FolderPermission where Folder=?{0};" +
                "delete from Folder where {0}=?{0};",
                _tbl.FolderId.Name
                );
            _tbl.FolderId.SetParameter(cmd, folderId);

            cmd.ExecuteNonQuery();
        }

        public DcFolder GetFolderHierarchy(int folderType)
        {
            if (!_con.IsOpen)
            {
                throw new DcConnectionClosedException();
            }

            if (DBConstKeeper.Current == null)
            {
                throw new DcNeededObjectNotInitializedException("DBConstKeeper has not been initialized!");
            }



            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                "select FolderId, Name, ParentFolder from folder where FolderType=?FolderType order by ParentFolder";
            cmd.Parameters.Add("?FolderType", MySqlDbType.Int32);
            cmd.Parameters["?FolderType"].Value = folderType;
            MySqlDataReader reader = cmd.ExecuteReader();

            reader.Read();


            DcFolderListElement rootElem = new DcFolderListElement(
                new DcFolder(
                    reader.GetInt32("FolderId"),
                    ((reader.IsDBNull(2)) ? null : (int?)reader.GetInt32("ParentFolder")),
                    reader.GetString("Name")
                    )
                );
            DcFolderListElement curElem = rootElem;
            while (reader.Read())
            {
                DcFolder fld = new DcFolder(
                    reader.GetInt32("FolderId"),
                    reader.GetInt32("ParentFolder"),
                    reader.GetString("Name")
                    );
                curElem.Next = new DcFolderListElement(fld);
                curElem = curElem.Next;
            }
            reader.Close();

            bool useTmpFolderSecurity = true;
            SecurityAccess sa = new SecurityAccess(_userId, _con);
            sa.CreateFolderSecurityTable();
            SetFoldersCollectionFromList(rootElem.DcFolder, ref rootElem.Next);



            return rootElem.DcFolder;
        }

        class DcFolderListElement
        {
            public DcFolderListElement(DcFolder folder)
            {
                DcFolder = folder;
            }

            public DcFolderListElement Next;
            public DcFolder DcFolder;
        }

        private void SetFoldersCollectionFromList(DcFolder parentFolder, ref DcFolderListElement rootElem)
        {
            List<DcFolder> childFolders = new List<DcFolder>();
            bool useTmpFolderSecurityTable = true;
            bool useTmpDocTypeSecurityTable = true;

            SecurityAccess sa = new SecurityAccess(
                _userId, _con,
                useTmpFolderSecurityTable,
                !useTmpDocTypeSecurityTable
                );

            DcFolderListElement curElem = rootElem, prevElem = null;
            while (curElem != null)
            {
                DcFolder fld = curElem.DcFolder;
                if (fld.Parent == parentFolder.Id)
                {
                    if (prevElem != null)
                    {
                        prevElem.Next = curElem.Next;
                    }
                    else
                    {
                        rootElem = curElem.Next;
                    }
                    curElem = curElem.Next;

                    if (sa.GetFolderSecurity(fld.Id).Browse)
                    {
                        SetFoldersCollectionFromList(fld, ref rootElem);
                        childFolders.Add(fld);
                    }
                }
                else
                {
                    prevElem = curElem;
                    curElem = curElem.Next;
                }
            }

            parentFolder.Folders = childFolders.ToArray();
        }

    }
}
