//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using MySql.Data.MySqlClient;
using Evpo.DocCirc.Common;

namespace Evpo.DocCirc.MySqlDL
{
    public class InstitutionAccess
    {
        protected class Tbl : DataTbl
        {
            public DataColumn InstitutionId = new DataColumn("InstitutionId", MySqlDbType.Int32, true, false);
            public DataColumn DcUser = new DataColumn("DcUser", MySqlDbType.Int32);
            public DataColumn Name = new DataColumn("Name", MySqlDbType.String);
            public DataColumn Phone1 = new DataColumn("Phone1", MySqlDbType.String);
            public DataColumn Phone2 = new DataColumn("Phone2", MySqlDbType.String);
            public DataColumn Address1 = new DataColumn("Address1", MySqlDbType.String);
            public DataColumn Address2 = new DataColumn("Address2", MySqlDbType.String);
            public DataColumn Email1 = new DataColumn("Email1", MySqlDbType.String);
            public DataColumn Email2 = new DataColumn("Email2", MySqlDbType.String);
            public DataColumn WebSite = new DataColumn("WebSite", MySqlDbType.String);
            public DataColumn Note = new DataColumn("Note", MySqlDbType.String);

            public Tbl()
            {
                _items = new DataColumn[]
                {
                    InstitutionId,
                    DcUser,
                    Name,
                    Phone1,
                    Phone2,
                    Address1,
                    Address2,
                    Email1,
                    Email2,
                    WebSite,
                    Note
                };
            }
        }

        protected Tbl _tbl = new Tbl();

        protected DBConnection _con;
        protected int _userId;

        public InstitutionAccess(int userId, DBConnection con)
        {
            _con = con;
            _userId = userId;
        }

        public void SaveInstitution(InstitutionItem item)
        {
            
            MySqlCommand cmd = _con.CreateCommand();
            
            _tbl.DcUser.SetParameter(cmd, item.Owner);
            _tbl.Name.SetParameter(cmd, item.Name);
            _tbl.Phone1.SetParameter(cmd, item.Phone1);
            _tbl.Phone2.SetParameter(cmd, item.Phone2);
            _tbl.Address1.SetParameter(cmd, item.Address1);
            _tbl.Address2.SetParameter(cmd, item.Address2);
            _tbl.Email1.SetParameter(cmd, item.Email1);
            _tbl.Email2.SetParameter(cmd, item.Email2);
            _tbl.WebSite.SetParameter(cmd, item.WebSite);
            _tbl.Note.SetParameter(cmd, item.Note);

            StringBuilder sb = new StringBuilder();
            if (item.InstitutionId != null)
            {
                _tbl.InstitutionId.SetParameter(cmd, item.InstitutionId.Value);
                sb.Append("update Institution set ");
                bool first = true;
                foreach (DataColumn col in _tbl.Items)
                {
                    if (!col.AutoInc)
                    {
                        if (first)
                        {
                            first = false;
                        }
                        else
                        {
                            sb.Append(", ");
                        }
                        sb.AppendFormat("{0}=?{0}", col.Name);
                    }
                }

                sb.AppendFormat(" where {0}=?{0};", _tbl.InstitutionId.Name);
            }
            else
            {
                StringBuilder fieldSb = new StringBuilder();
                StringBuilder valueSb = new StringBuilder();

                bool first = true;
                foreach (DataColumn col in _tbl.Items)
                {
                    if (!col.AutoInc)
                    {
                        if (first)
                        {
                            first = false;
                        }
                        else
                        {
                            fieldSb.Append(", ");
                            valueSb.Append(", ");
                        }
                        fieldSb.Append(col.Name);
                        valueSb.AppendFormat("?{0}", col.Name);
                    }
                }

                sb.AppendFormat(
                    "insert into Institution ({0}) Values ({1});",
                    fieldSb.ToString(),
                    valueSb.ToString()
                    );
            }

            cmd.CommandText = sb.ToString();
            cmd.ExecuteNonQuery();

            if (item.InstitutionId == null)
            {
                item.InstitutionId = (int)cmd.LastInsertedId;
            }

        }

        public InstitutionItem LoadInstitutionItem(int id)
        {
            InstitutionItem retVal;

            StringBuilder sb = new StringBuilder();
            sb.Append("select ");
            bool first = true;
            foreach (DataColumn col in _tbl.Items)
            {
                if (first)
                {
                    first = false;
                }
                else
                {
                    sb.Append(", ");
                }

                sb.Append(col.Name);
            }

            sb.AppendFormat(
                " from Institution where {0}=?{0} order by {1};", 
                _tbl.InstitutionId.Name,
                _tbl.Name.Name
                );

            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = sb.ToString();

            _tbl.InstitutionId.SetParameter(cmd, id);

            MySqlDataReader reader = cmd.ExecuteReader();

            if (reader.Read())
            {
                retVal = new InstitutionItem(
                    reader.GetInt32(_tbl.DcUser.Name),
                    reader.GetString(_tbl.Name.Name),
                    reader.GetString(_tbl.Phone1.Name),
                    reader.GetString(_tbl.Phone2.Name),
                    reader.GetString(_tbl.Address1.Name),
                    reader.GetString(_tbl.Address2.Name),
                    reader.GetString(_tbl.Email1.Name),
                    reader.GetString(_tbl.Email2.Name),
                    reader.GetString(_tbl.WebSite.Name),
                    reader.GetString(_tbl.Note.Name),
                    reader.GetInt32(_tbl.InstitutionId.Name)
                    );
            }
            else
            {
                throw new DcObjectNotFoundException();
            }

            reader.Close();

            return retVal;
        }

        public InstitutionToken[] GetAllInstitutionTokens()
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendFormat(
                "select {0}, {1} from Institution order by {1};",
                _tbl.InstitutionId.Name, 
                _tbl.Name.Name
                );

            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = sb.ToString();

            MySqlDataReader reader = cmd.ExecuteReader();

            List<InstitutionToken> tokenList = new List<InstitutionToken>();

            while(reader.Read())
            {
                tokenList.Add(
                    new InstitutionToken(
                        reader.GetInt32(_tbl.InstitutionId.Name),
                        reader.GetString(_tbl.Name.Name)
                        )
                    );
            }

            reader.Close();

            return tokenList.ToArray();
        }

        public void DeleteInstitution(int id)
        {
            MySqlCommand cmd = _con.CreateCommand();
            _tbl.InstitutionId.SetParameter(cmd, id);
            cmd.CommandText =
                String.Format("delete from Institution where {0}=?{0};", _tbl.InstitutionId.Name);
            cmd.ExecuteNonQuery();
        }
    }
}
