//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using MySql.Data.MySqlClient;
using Evpo.DocCirc.Common;

namespace Evpo.DocCirc.MySqlDL
{
    public class NotificationAccess
    {
        protected DBConnection _con;
        protected int _userId;
        protected SettingKeeper _sk;

        protected class Tbl : DataTbl
        {
            public DataColumn UserNotificationId =
                new DataColumn("UserNotificationId", MySqlDbType.Int64, true, false);

            public DataColumn DocCard =
                new DataColumn("DocCard", MySqlDbType.Int64, false, true);

            public DataColumn NotificationType =
                new DataColumn("NotificationType", MySqlDbType.Int32);

            public DataColumn NotificationTime =
                new DataColumn("NotificationTime", MySqlDbType.Datetime);

            public DataColumn Text =
                new DataColumn("Text", MySqlDbType.String);

            public DataColumn UserNote =
                new DataColumn("UserNote", MySqlDbType.String);

            public DataColumn LastActionUser =
                new DataColumn("LastActionUser", MySqlDbType.Int32, false, true);

            public DataColumn RoutePointNum =
                new DataColumn("RoutePointNum", MySqlDbType.Int32);

            public DataColumn DcUser =
                new DataColumn("DcUser", MySqlDbType.Int32);

            public Tbl()
            {
                _items = new DataColumn[]
                {
                    UserNotificationId,
                    DocCard,
                    NotificationType,
                    NotificationTime,
                    Text,
                    UserNote,
                    LastActionUser,
                    RoutePointNum,
                    DcUser
                };
            }

        }

        protected Tbl _tbl = new Tbl();

        public NotificationAccess(int userId, DBConnection con, SettingKeeper sk)
        {
            _userId = userId;
            _con = con;
            _sk = sk;
            
        }

        public NotificationItem[] GetUserNotifications(int user)
        {
            List<NotificationItem> itemList = new List<NotificationItem>();

            StringBuilder sb = new StringBuilder();
            sb.Append("select ");
            bool first = true;
            foreach (DataColumn col in _tbl.Items)
            {
                if (!first)
                {
                    sb.Append(",");
                }
                else
                {
                    first = false;
                }

                sb.Append(col.Name);
            }

            sb.Append(" from UserNotification where ");
            sb.AppendFormat("{0}=?{0}", _tbl.DcUser.Name);
            sb.Append(" order by NotificationTime desc");
            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = sb.ToString();

            _tbl.DcUser.SetParameter(cmd, user);

            MySqlDataReader reader = cmd.ExecuteReader();
            while (reader.Read())
            {

                long? docCard = null;
                if (!_tbl.DocCard.IsDBNull(reader))
                {
                    docCard = reader.GetInt64(_tbl.DocCard.Name);
                }

                int? lastActionUser = null;
                if (!_tbl.LastActionUser.IsDBNull(reader))
                {
                    lastActionUser = reader.GetInt32(_tbl.LastActionUser.Name);
                }

                NotificationItem item = new NotificationItem(
                    reader.GetInt64(_tbl.UserNotificationId.Name),
                    docCard,
                    reader.GetInt32(_tbl.NotificationType.Name),
                    reader.GetInt32(_tbl.DcUser.Name),
                    reader.GetDateTime(_tbl.NotificationTime.Name),
                    reader.GetString(_tbl.Text.Name),
                    reader.GetString(_tbl.UserNote.Name),
                    lastActionUser,
                    reader.GetInt32(_tbl.RoutePointNum.Name)
                    );

                itemList.Add(item);
            }
            reader.Close();

            return itemList.ToArray();
        }

        public void AddUserNotification(NotificationItem item)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("insert into UserNotification (");
            bool first = true;
            StringBuilder valueStr = new StringBuilder();
            foreach (DataColumn col in _tbl.Items)
            {
                if(!col.AutoInc)
                {
                    if(first)
                    {
                        first = false;
                    }
                    else
                    {
                        sb.Append(", ");
                        valueStr.Append(", ");
                    }

                    sb.Append(col.Name);
                    valueStr.AppendFormat("?{0}", col.Name);
                }
            }

            sb.AppendFormat(") Values ({0});", valueStr.ToString());

            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = sb.ToString();

            //ColumnKeeper.UserNotificationId.SetParameter(cmd, item.Id.Value);
            _tbl.DocCard.SetParameter(cmd, item.DocCardId.Value);
            _tbl.NotificationType.SetParameter(cmd, item.NotificationType);
            _tbl.DcUser.SetParameter(cmd, item.User);
            _tbl.NotificationTime.SetParameter(cmd, item.NotificationTime);
            _tbl.Text.SetParameter(cmd, item.Text);
            _tbl.UserNote.SetParameter(cmd, item.UserNote);
            _tbl.LastActionUser.SetParameter(cmd, item.LastActionUser);
            _tbl.RoutePointNum.SetParameter(cmd, item.RoutePointNum);

            cmd.ExecuteNonQuery();
            item.Id = cmd.LastInsertedId;
        }

    }
}
