//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using MySql.Data.MySqlClient;
using Evpo.DocCirc.Common;

namespace Evpo.DocCirc.MySqlDL
{
    /// <summary>
    /// 
    /// </summary>
    public class RoleAccess
    {
        protected class Tbl : DataTbl
        {
            public DataColumn DcRoleId = new DataColumn("DcRoleId", MySqlDbType.Int32, true, false);
            public DataColumn RoleName = new DataColumn("RoleName", MySqlDbType.String);
            public Tbl()
            {
                Items = new DataColumn[]
                {
                    DcRoleId,
                    RoleName
                };
            }
        }
        protected Tbl _tbl = new Tbl();

        DBConnection _con;

        public RoleAccess(DBConnection con)
        {
            _con = con;
        }

        public UserRole GetUserRole(int roleId)
        {
            if(!_con.IsOpen)
            {
                throw new ApplicationException("Can't get role name! Connection is not open.");
            }

            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                "select roleName from dcRole where dcRoleId=?dcRoleId";
            cmd.Parameters.Add(
                "?dcRoleId",
                MySqlDbType.Int32
                );
            cmd.Parameters["?dcRoleId"].Value = roleId;

            MySqlDataReader reader = cmd.ExecuteReader();
            
            bool result = reader.Read();
            if(!result)
            {
                throw new ApplicationException(String.Format("Can't find a role with id {0}!", roleId));
            }

            UserRole retVal = new UserRole(roleId, reader.GetString(0));
            reader.Close();

            return retVal;
        }

        public void SaveUserRole(UserRole role)
        {
            MySqlCommand cmd = _con.CreateCommand();
            _tbl.RoleName.SetParameter(cmd, role.RoleName);
            if (role.RoleId == null)
            {
                cmd.CommandText = String.Format(
                    "insert into DcRole ({0}) Values (?{0});",
                    _tbl.RoleName.Name
                    );
                cmd.ExecuteNonQuery();
                role.RoleId = (int)cmd.LastInsertedId;
            }
            else
            {
                _tbl.DcRoleId.SetParameter(cmd, role.RoleId);
                cmd.CommandText = String.Format(
                    "update DcRole set {0}=?{0} where {1}=?{1};",
                    _tbl.RoleName.Name,
                    _tbl.DcRoleId.Name
                    );
                cmd.ExecuteNonQuery();
            }
        }

        public void DeleteUserRole(int roleId)
        {
            MySqlCommand cmd = _con.CreateCommand();
            _tbl.DcRoleId.SetParameter(cmd, roleId);

            cmd.CommandText = String.Format(
                "delete from RoleToUser where DcRole=?{0};" +
                "delete from DcRole where {0}=?{0};",
                _tbl.DcRoleId.Name
                );
            //MySqlTransaction tran = _con.MySqlConnection.BeginTransaction();
            try
            {
                cmd.ExecuteNonQuery();
            }
            catch (MySqlException ex)
            {
                //tran.Rollback();
            }

            //tran.Commit();
        }

        public UserRole[] GetAllUserRoles()
        {
            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = "select DcRoleId, RoleName from DcRole;";
            MySqlDataReader reader = cmd.ExecuteReader();
            List<UserRole> roles = new List<UserRole>();
            while (reader.Read())
            {
                roles.Add(new UserRole(reader.GetInt32("DcRoleId"), reader.GetString("RoleName")));
            }
            reader.Close();
            return roles.ToArray();

        }

        public UserToken[] GetUserRoleUsers(int roleId)
        {
            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = String.Format(
                "select DcUser, DcUser.Login as Login from RoleToUser " +
                "inner join DcUser on RoleToUser.DcUser=DcUser.DcUserId " +
                "where RoleToUser.DcRole=?{0} order by DcUser.Login;",
                _tbl.DcRoleId.Name
                );
            _tbl.DcRoleId.SetParameter(cmd, roleId);

            MySqlDataReader reader = cmd.ExecuteReader();
            List<UserToken> users = new List<UserToken>();
            while (reader.Read())
            {
                users.Add(new UserToken(reader.GetInt32("DcUser"), reader.GetString("Login")));
            }
            reader.Close();

            return users.ToArray();
        }

        public void SetUserRoleUsers(int roleId, UserToken[] users)
        {
            List<int> userList = new List<int>(users.Length);
            foreach (UserToken user in users)
            {
                userList.Add(user.UserId);
            }

            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                String.Format("select DcUser from RoleToUser where DcRole=?{0}", _tbl.DcRoleId.Name);
            _tbl.DcRoleId.SetParameter(cmd, roleId);
            MySqlDataReader reader = cmd.ExecuteReader();
            List<int> usersBefore = new List<int>();

            while (reader.Read())
            {
                usersBefore.Add(reader.GetInt32("DcUser"));
            }
            reader.Close();

            cmd.CommandText = String.Format(
                "delete from RoleToUser where {0}=?{0} and DcRole=?{1}",
                "DcUser",
                _tbl.DcRoleId.Name
                );
            cmd.Parameters.Add("?DcUser", MySqlDbType.Int32);

            // delete unnecessary
            foreach (int user in usersBefore)
            {
                if (!userList.Contains(user))
                {
                    cmd.Parameters["?DcUser"].Value = user;
                    cmd.ExecuteNonQuery();
                }
            }

            // update
            cmd.CommandText = String.Format(
                "insert into RoleToUser (DcUser, DcRole, Position) Values (?DcUser, ?{0}, 1);",
                _tbl.DcRoleId.Name
                );

            foreach(int user in userList)
            {
                if (!usersBefore.Contains(user))
                {
                    cmd.Parameters["?DcUser"].Value = user;
                    cmd.ExecuteNonQuery();
                }
            }
        }
    }


}
