//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using MySql.Data.MySqlClient;
using Evpo.DocCirc.Common;

namespace Evpo.DocCirc.MySqlDL
{
    public class SecurityAccess
    {
        protected class DocTypeTbl : DataTbl
        {
            public DataColumn DocTypePermissionId = 
                new DataColumn("DocTypePermissionId", MySqlDbType.Int32, true, false);
            public DataColumn DcRole = new DataColumn("DcRole", MySqlDbType.Int32);
            public DataColumn DocType = new DataColumn("DocType", MySqlDbType.Int32);
            public DataColumn C = new DataColumn("C", MySqlDbType.Bit);
            public DataColumn R = new DataColumn("R", MySqlDbType.Bit);
            public DataColumn U = new DataColumn("U", MySqlDbType.Bit);
            public DataColumn D = new DataColumn("D", MySqlDbType.Bit);
            public DataColumn RO = new DataColumn("RO", MySqlDbType.Bit);
            public DataColumn UO = new DataColumn("UO", MySqlDbType.Bit);
            public DataColumn DO = new DataColumn("DO", MySqlDbType.Bit);

            public DocTypeTbl()
            {
                Items = new DataColumn[]
                {
                    DocTypePermissionId,
                    DcRole,
                    DocType,
                    C,
                    R,
                    U,
                    D,
                    RO,
                    UO,
                    DO
                };
            }
        }

        protected DocTypeTbl _docTypeTbl = new DocTypeTbl();

        protected class FolderTbl : DataTbl
        {
            public DataColumn FolderPermissionId =
                new DataColumn("FolderPermissionId", MySqlDbType.Int32, true, false);
            public DataColumn DcRole = new DataColumn("DcRole", MySqlDbType.Int32);
            public DataColumn Folder = new DataColumn("Folder", MySqlDbType.Int32);
            public DataColumn C = new DataColumn("C", MySqlDbType.Bit);
            public DataColumn R = new DataColumn("R", MySqlDbType.Bit);
            public DataColumn U = new DataColumn("U", MySqlDbType.Bit);
            public DataColumn D = new DataColumn("D", MySqlDbType.Bit);
            public DataColumn B = new DataColumn("B", MySqlDbType.Bit);
            public DataColumn RO = new DataColumn("RO", MySqlDbType.Bit);
            public DataColumn UO = new DataColumn("UO", MySqlDbType.Bit);
            public DataColumn DO = new DataColumn("DO", MySqlDbType.Bit);

            public FolderTbl()
            {
                Items = new DataColumn[]
                {
                    FolderPermissionId,
                    DcRole,
                    Folder,
                    C,
                    R,
                    U,
                    D,
                    B,
                    RO,
                    UO,
                    DO
                };
            }
        }

        protected FolderTbl _folderTbl = new FolderTbl();

        public enum SecurityAccessType
        {
            Create,
            Read,
            Update,
            Delete
        }


        protected DBConnection _con;
        protected int _userId;
        protected bool _useTmpFolderSecurityTable;
        protected bool _useTmpDocTypeSecurityTable;

        /// <summary>
        /// for unit testing
        /// </summary>
        public SecurityAccess() { }

        public SecurityAccess(int userId, DBConnection con):this(userId, con, false, false){}

        public SecurityAccess(
            int userId, 
            DBConnection con, 
            bool useTmpFolderSecurityTable, 
            bool useTmpDocTypeSecurityTable 
            )
        {
            _con = con;
            _userId = userId;
            _useTmpFolderSecurityTable = useTmpFolderSecurityTable;
            _useTmpDocTypeSecurityTable = useTmpDocTypeSecurityTable;
        }

        //public void CreateUserRoleTable()
        //{
        //    MySqlCommand cmd = new MySqlCommand(
        //        "create temporary table if not exists tmpUserRole(DcRole int, DcUser int);" +
        //        "delete from tmpUserRole;" +
        //        "insert into tmpUserRole (DcRole, DcUser) " +
        //        "   select DcRole, DcUser from RoleToUser where DcUser=?DcUser;"
        //        ,
        //        _con.MySqlConnection
        //        );
        //    cmd.Parameters.Add("?DcUser", MySqlDbType.Int32);
        //    cmd.Parameters["?DcUser"].Value = _userId;
        //    cmd.ExecuteNonQuery();
        //}

        /// <summary>
        /// creates temporary table tmpFolderSecurity(DcFolder int, DcUser int, CrudBRoUoDo)
        /// inserts into the table all folders that the user has any access
        /// </summary>
        /// <param name="accessType"></param>
        public void CreateFolderSecurityTable()
        {
            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                "create temporary table if not exists tmpFolderSecurity" +
                "(Folder int, DcUser int, C bool, R bool, U bool, D bool, B bool, RO bool, UO bool, DO bool);" +

                "delete from tmpFolderSecurity;" +

                "insert into tmpFolderSecurity " +
                "select Folder, ?DcUser as DcUser, " +
                "MAX(C) as C, MAX(R) as R, MAX(U) as U, MAX(D) as D, MAX(B) as B, " +
                "MAX(RO) as RO, MAX(UO) as UO, MAX(DO) as DO from FolderPermission " +
                "inner join RoleToUser " +
                "on FolderPermission.DcRole=RoleToUser.DcRole and RoleToUser.DcUser=?DcUser " +
                "where C or R or U or D or B or RO or UO or DO " +
                "group by 1, 2 ;";
            cmd.Parameters.Add("?DcUser", MySqlDbType.Int32);
            cmd.Parameters["?DcUser"].Value = _userId;
            cmd.ExecuteNonQuery();

        }

        public void CreateDocTypeSecurityTable()
        {
            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                 "create temporary table if not exists tmpDocTypeSecurity" +
                 "(DocType int, DcUser int, C bool, R bool, U bool, D bool, RO bool, UO bool, DO bool);" +

                 "delete from tmpDocTypeSecurity;" +

                 "insert into tmpDocTypeSecurity " +
                 "select DocType, ?DcUser as DcUser, " +
                 "MAX(C) as C, MAX(R) as R, MAX(U) as U, MAX(D) as D, " +
                 "MAX(RO) as RO, MAX(UO) as UO, MAX(DO) as DO from DocTypePermission " +
                 "inner join RoleToUser " +
                 "on DocTypePermission.DcRole=RoleToUser.DcRole and RoleToUser.DcUser=?DcUser " +
                 "where C or R or U or D or RO or UO or DO " +
                 "group by 1, 2 ;";
            cmd.Parameters.Add("?DcUser", MySqlDbType.Int32);
            cmd.Parameters["?DcUser"].Value = _userId;
            cmd.ExecuteNonQuery();

        }

        public virtual SecurityItem GetFolderSecurity(int folderId)
        {
            MySqlCommand cmd = _con.CreateCommand();
            if (_useTmpFolderSecurityTable)
            {
                cmd.CommandText = "select C, R, U, D, B, RO, UO, DO from tmpFolderSecurity " +
                    "where Folder=?Folder and DcUser=?DcUser;";
            }
            else
            {
                cmd.CommandText =
                    "select MAX(C) as C, MAX(R) as R, MAX(U) as U, MAX(D) as D, MAX(B) as B, " +
                    "MAX(RO) as RO, MAX(UO) as UO, MAX(DO) as DO " +
                    "from FolderPermission " +
                    "inner join RoleToUser " +
                    "on FolderPermission.DcRole=RoleToUser.DcRole and RoleToUser.DcUser=?DcUser " +
                    "where Folder=?Folder";
            }

            //MySqlCommand cmd = new MySqlCommand(
            //    "select MAX(C) as C, MAX(R) as R, MAX(U) as U, MAX(D) as D, MAX(B) as B, " +
            //    "MAX(RO) as RO, MAX(UO) as UO, MAX(DO) as DO " +
            //    "from FolderPermission " + 
            //    "inner join RoleToUser " +
            //    "on FolderPermission.DcRole=RoleToUser.DcRole and RoleToUser.DcUser=?DcUser " +
            //    "where Folder=?Folder",
            //    _con.MySqlConnection
            //    );
            cmd.Parameters.Add("?Folder", MySqlDbType.Int32);
            cmd.Parameters["?Folder"].Value = folderId;
            cmd.Parameters.Add("?DcUser", MySqlDbType.Int32);
            cmd.Parameters["?DcUser"].Value = _userId;
            MySqlDataReader reader = cmd.ExecuteReader();


            bool[] crudBRoUoDo = new bool[8];
            if(reader.Read())
            {
                for (int i = 0; i < reader.FieldCount; i++)
                {
                    if (!reader.IsDBNull(i))
                    {
                        crudBRoUoDo[i] = reader.GetBoolean(i);
                    }
                    else
                    {
                        crudBRoUoDo[i] = false;
                    }
                }
            }
            reader.Close();
            
            return new SecurityItem(crudBRoUoDo);
        }



        /// <summary>
        /// security for all documents of this doc type for the user
        /// </summary>
        /// <param name="docTypeId"></param>
        /// <returns></returns>
        public virtual SecurityItem GetDocTypeDocumentsSecurity(int docTypeId)
        {
            MySqlCommand cmd = _con.CreateCommand();
            if (_useTmpDocTypeSecurityTable)
            {
                cmd.CommandText = "select C, R, U, D, false as B, RO, UO, DO from tmpDocTypeSecurity " +
                    "where DocType=?DocType and DcUser=?DcUser;";
            }
            else
            {
                cmd.CommandText =
                    "select MAX(C) as C, MAX(R) as R, MAX(U) as U, MAX(D) as D, false as B, " +
                    "MAX(RO) as RO, MAX(UO) as UO, MAX(DO) as DO " +
                    "from DocTypePermission " +
                    "inner join RoleToUser " +
                    "on DocTypePermission.DcRole=RoleToUser.DcRole and RoleToUser.DcUser=?DcUser " +
                    "where DocType=?DocType";
            }

            //MySqlCommand cmd = new MySqlCommand(
            //    "select MAX(C) as C, MAX(R) as R, MAX(U) as U, MAX(D) as D, MAX(B) as B, " +
            //    "MAX(RO) as RO, MAX(UO) as UO, MAX(DO) as DO " +
            //    "from FolderPermission " + 
            //    "inner join RoleToUser " +
            //    "on FolderPermission.DcRole=RoleToUser.DcRole and RoleToUser.DcUser=?DcUser " +
            //    "where Folder=?Folder",
            //    _con.MySqlConnection
            //    );
            cmd.Parameters.Add("?DocType", MySqlDbType.Int32);
            cmd.Parameters["?DocType"].Value = docTypeId;
            cmd.Parameters.Add("?DcUser", MySqlDbType.Int32);
            cmd.Parameters["?DcUser"].Value = _userId;
            MySqlDataReader reader = cmd.ExecuteReader();


            bool[] crudBRoUoDo = new bool[8];
            if (reader.Read())
            {
                for (int i = 0; i < reader.FieldCount; i++)
                {
                    if (!reader.IsDBNull(i))
                    {
                        crudBRoUoDo[i] = reader.GetBoolean(i);
                    }
                    else
                    {
                        crudBRoUoDo[i] = false;
                    }
                }
            }
            reader.Close();

            return new SecurityItem(crudBRoUoDo);
        }

        public virtual SecurityItem GetDocumentCardSecurity(DocCardItem item)
        {
            SecurityItem retVal = new SecurityItem(false, false, false, false, false, false, false, false);

            SecurityItem docTypeSi = null;
            if (item.DocType != null)
            {
                docTypeSi = GetDocTypeDocumentsSecurity(item.DocType.Value);
            }
            SecurityItem folderSi = GetFolderSecurity(item.Folder);

            if (item.IsNew && docTypeSi == null && item.Owner == _userId)
            {
                retVal.Read = folderSi.Read || folderSi.ReadOwn;
                retVal.Update = folderSi.Create;
                retVal.Delete = folderSi.Delete || folderSi.DeleteOwn;
            }
            else if (item.IsNew && docTypeSi == null && item.Owner != _userId)
            {
                retVal.Read = false;
                retVal.Update = false;
                retVal.Delete = false;
            }
            else
            {
                SecurityItem resultSi = new SecurityItem();
                bool isOwn = (item.Owner == _userId);
                resultSi.Create = folderSi.Create && docTypeSi.Create;
                resultSi.Read = folderSi.Read && docTypeSi.Read;
                resultSi.Update = folderSi.Update && docTypeSi.Update;
                resultSi.Delete = folderSi.Delete && docTypeSi.Delete;
                resultSi.ReadOwn = folderSi.ReadOwn && docTypeSi.ReadOwn;
                resultSi.UpdateOwn = folderSi.UpdateOwn && docTypeSi.UpdateOwn;
                resultSi.DeleteOwn = folderSi.DeleteOwn && docTypeSi.DeleteOwn;

                retVal.Read = resultSi.Read || (resultSi.ReadOwn && isOwn);
                if (!item.IsNew)
                {
                    retVal.Update = resultSi.Update || (resultSi.UpdateOwn && isOwn);
                }
                else
                {
                    retVal.Update = resultSi.Create;
                }

                retVal.Delete = resultSi.Delete || (resultSi.DeleteOwn && isOwn);

            }

            return retVal;
        }

        public DocTypePermission[] GetDocTypePermissions(int docType)
        {
            MySqlCommand cmd = _con.CreateCommand();

            cmd.CommandText = 
                "select DocTypePermissionId, DocType, DcRole, C, R, U, D, RO, UO, DO, DcRole.RoleName as RoleName " +
                "from DocTypePermission inner join DcRole on DocTypePermission.DcRole=DcRole.DcRoleId " +
                "where DocType=?DocType;";
            cmd.Parameters.Add("?DocType", MySqlDbType.Int32);
            cmd.Parameters["?DocType"].Value = docType;

            List<DocTypePermission> perms = new List<DocTypePermission>();
            MySqlDataReader reader = cmd.ExecuteReader();
            while (reader.Read())
            {
                SecurityItem si = new SecurityItem(
                    reader.GetBoolean("C"),
                    reader.GetBoolean("R"),
                    reader.GetBoolean("U"),
                    reader.GetBoolean("D"),
                    false,
                    reader.GetBoolean("RO"),
                    reader.GetBoolean("UO"),
                    reader.GetBoolean("DO")
                    );
                UserRole role = new UserRole(reader.GetInt32("DcRole"), reader.GetString("RoleName"));
                perms.Add(new DocTypePermission(docType, role, si, reader.GetInt32("DocTypePermissionId")));
            }
            reader.Close();

            return perms.ToArray();
        }

        public void SaveDocTypePermissions(int docType, DocTypePermission[] permissions)
        {
            MySqlCommand cmd = _con.CreateCommand();
            // get previous permission id
            cmd.CommandText = String.Format(
                "select DocTypePermissionId from DocTypePermission where {0}=?{0};",
                _docTypeTbl.DocType.Name
                );
            _docTypeTbl.DocType.SetParameter(cmd, docType);
            List<int> permissionsBefore = new List<int>();
            MySqlDataReader reader = cmd.ExecuteReader();
            while (reader.Read())
            {
                permissionsBefore.Add(reader.GetInt32(0));
            }
            reader.Close();
            
            // update
            
            foreach (DocTypePermission perm in permissions)
            {
                _docTypeTbl.DcRole.SetParameter(cmd, perm.Role.RoleId);
                _docTypeTbl.C.SetParameter(cmd, perm.SecurityItem.Create);
                _docTypeTbl.R.SetParameter(cmd, perm.SecurityItem.Read);
                _docTypeTbl.U.SetParameter(cmd, perm.SecurityItem.Update);
                _docTypeTbl.D.SetParameter(cmd, perm.SecurityItem.Delete);
                _docTypeTbl.RO.SetParameter(cmd, perm.SecurityItem.ReadOwn);
                _docTypeTbl.UO.SetParameter(cmd, perm.SecurityItem.UpdateOwn);
                _docTypeTbl.DO.SetParameter(cmd, perm.SecurityItem.DeleteOwn);

                if (perm.PermissionId == null)
                {
                    cmd.CommandText = String.Format(
                        "insert into DocTypePermission ({0}) Values ({1});",
                        _docTypeTbl.GetColumnPart(DataTbl.SqlCmdType.InsertColumnPart),
                        _docTypeTbl.GetColumnPart(DataTbl.SqlCmdType.InsertValuePart)
                        );
                    cmd.ExecuteNonQuery();
                    perm.PermissionId = (int)cmd.LastInsertedId;
                }
                else
                {
                    _docTypeTbl.DocTypePermissionId.SetParameter(cmd, perm.PermissionId.Value);
                    cmd.CommandText = String.Format(
                        "update DocTypePermission set {0} where {1}=?{1};",
                        _docTypeTbl.GetColumnPart(DataTbl.SqlCmdType.UpdateSetPart),
                        _docTypeTbl.DocTypePermissionId.Name
                        );
                    cmd.ExecuteNonQuery();

                    if (permissionsBefore.Contains(perm.PermissionId.Value))
                    {
                        permissionsBefore.Remove(perm.PermissionId.Value);
                    }
                }
            }

            // delete unnecessary
            cmd.CommandText = String.Format(
                "delete from DocTypePermission where {0}=?{0};",
                _docTypeTbl.DocTypePermissionId.Name
                );
            foreach (int permToDel in permissionsBefore)
            {
                _docTypeTbl.DocTypePermissionId.SetParameter(cmd, permToDel);
                cmd.ExecuteNonQuery();
            }
        }

        public FolderPermission[] GetFolderPermissions(int folderId)
        {
            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = String.Format(
                "select {0}, DcRole.RoleName as RoleName from FolderPermission " +
                "inner join DcRole on FolderPermission.DcRole=DcRole.DcRoleId " +
                "where {1}=?{1};",
                _folderTbl.GetColumnPart(DataTbl.SqlCmdType.SelectColumnPart),
                _folderTbl.Folder.Name
                );
            _folderTbl.Folder.SetParameter(cmd, folderId);

            List<FolderPermission> perms = new List<FolderPermission>();
            MySqlDataReader reader = cmd.ExecuteReader();
            while (reader.Read())
            {
                SecurityItem si = new SecurityItem(
                    reader.GetBoolean(_folderTbl.C.Name),
                    reader.GetBoolean(_folderTbl.R.Name),
                    reader.GetBoolean(_folderTbl.U.Name),
                    reader.GetBoolean(_folderTbl.D.Name),
                    reader.GetBoolean(_folderTbl.B.Name),
                    reader.GetBoolean(_folderTbl.RO.Name),
                    reader.GetBoolean(_folderTbl.UO.Name),
                    reader.GetBoolean(_folderTbl.DO.Name)
                    );
                
                UserRole role = 
                    new UserRole(reader.GetInt32(_folderTbl.DcRole.Name), reader.GetString("RoleName"));

                perms.Add(
                    new FolderPermission(
                        folderId, 
                        role, 
                        si, 
                        reader.GetInt32(_folderTbl.FolderPermissionId.Name)
                        )
                    );
            }
            reader.Close();
            
            return perms.ToArray();
        }

        public void SaveFolderPermissions(int folderId, FolderPermission[] permissions)
        {
            MySqlCommand cmd = _con.CreateCommand();
            // get previous permission id
            cmd.CommandText = String.Format(
                "select FolderPermissionId from FolderPermission where {0}=?{0};",
                _folderTbl.Folder.Name
                );
            _folderTbl.Folder.SetParameter(cmd, folderId);
            List<int> permissionsBefore = new List<int>();
            MySqlDataReader reader = cmd.ExecuteReader();
            while (reader.Read())
            {
                permissionsBefore.Add(reader.GetInt32(0));
            }
            reader.Close();

            // update

            foreach (FolderPermission perm in permissions)
            {
                _folderTbl.DcRole.SetParameter(cmd, perm.Role.RoleId);
                _folderTbl.C.SetParameter(cmd, perm.SecurityItem.Create);
                _folderTbl.R.SetParameter(cmd, perm.SecurityItem.Read);
                _folderTbl.U.SetParameter(cmd, perm.SecurityItem.Update);
                _folderTbl.D.SetParameter(cmd, perm.SecurityItem.Delete);
                _folderTbl.B.SetParameter(cmd, perm.SecurityItem.Browse);
                _folderTbl.RO.SetParameter(cmd, perm.SecurityItem.ReadOwn);
                _folderTbl.UO.SetParameter(cmd, perm.SecurityItem.UpdateOwn);
                _folderTbl.DO.SetParameter(cmd, perm.SecurityItem.DeleteOwn);

                if (perm.PermissionId == null)
                {
                    cmd.CommandText = String.Format(
                        "insert into FolderPermission ({0}) Values ({1});",
                        _folderTbl.GetColumnPart(DataTbl.SqlCmdType.InsertColumnPart),
                        _folderTbl.GetColumnPart(DataTbl.SqlCmdType.InsertValuePart)
                        );
                    cmd.ExecuteNonQuery();
                    perm.PermissionId = (int)cmd.LastInsertedId;
                }
                else
                {
                    _folderTbl.FolderPermissionId.SetParameter(cmd, perm.PermissionId.Value);
                    cmd.CommandText = String.Format(
                        "update FolderPermission set {0} where {1}=?{1};",
                        _folderTbl.GetColumnPart(DataTbl.SqlCmdType.UpdateSetPart),
                        _folderTbl.FolderPermissionId.Name
                        );
                    cmd.ExecuteNonQuery();

                    if (permissionsBefore.Contains(perm.PermissionId.Value))
                    {
                        permissionsBefore.Remove(perm.PermissionId.Value);
                    }
                }
            }

            // delete unnecessary
            cmd.CommandText = String.Format(
                "delete from FolderPermission where {0}=?{0};",
                _folderTbl.FolderPermissionId.Name
                );
            foreach (int permToDel in permissionsBefore)
            {
                _folderTbl.FolderPermissionId.SetParameter(cmd, permToDel);
                cmd.ExecuteNonQuery();
            }
        }

        /// <summary>
        /// security for this doc type item
        /// </summary>
        /// <param name="docTypeId"></param>
        /// <returns></returns>
        //public SecurityItem GetDocTypeItemSecurity(int docTypeId)
        //{
        //    return null;
        //}
    }
}
