//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Collections;
using MySql.Data;
using MySql.Data.MySqlClient;
using Evpo.DocCirc.Common;



namespace Evpo.DocCirc.MySqlDL
{
    public class UserAccess
    {

        protected class Tbl : DataTbl
        {
            public DataColumn DcUserId = new DataColumn("DcUserId", MySqlDbType.Int32, true, false);
            public DataColumn Login = new DataColumn("Login", MySqlDbType.String);
            public DataColumn FullName = new DataColumn("FullName", MySqlDbType.String);
            public DataColumn AutomaticPass = new DataColumn("AutomaticPass", MySqlDbType.Bit);
            public DataColumn AllowDeputyNotification = new DataColumn("AllowDeputyNotification", MySqlDbType.Bit);
            public DataColumn Department = new DataColumn("Department", MySqlDbType.Int32);
            public DataColumn InstantMessaging = new DataColumn("InstantMessaging", MySqlDbType.Bit);
            public DataColumn ImAddress = new DataColumn("ImAddress", MySqlDbType.String);
            public DataColumn ChangePwdNextTime = new DataColumn("ChangePwdNextTime", MySqlDbType.Bit);
            public DataColumn ExternalAuthorization = new DataColumn("ExternalAuthorization", MySqlDbType.Bit);
            public DataColumn ExternalIdentity = new DataColumn("ExternalIdentity", MySqlDbType.String);
            public DataColumn LastDocType = new DataColumn("LastDocType", MySqlDbType.Int32, false, true);

            public Tbl()
            {
                _items = new DataColumn[]
                {
                    DcUserId, 
                    Login, 
                    FullName, 
                    AutomaticPass, 
                    AllowDeputyNotification,
                    Department,
                    InstantMessaging,
                    ImAddress,
                    ChangePwdNextTime,
                    ExternalAuthorization,
                    ExternalIdentity,
                    LastDocType
                };

            }
        }

        protected Tbl _tbl = new Tbl();

        protected DBConnection _con;

        /// <summary>
        /// for unit testing
        /// </summary>
        public UserAccess()
        {
        }

        public UserAccess(DBConnection con)
        {
            _con = con;
        }

        public int[] GetAllUsers()
        {
            if (!_con.IsOpen)
            {
                throw new ApplicationException("Can't receive all users. Connecection is not open.");
            }

            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                "select DcUserId from DcUser";
            MySqlDataReader reader = cmd.ExecuteReader();
            ArrayList arr = new ArrayList();
            while (reader.Read())
            {
                arr.Add(reader.GetInt32(_tbl.DcUserId.Name));
            }
            reader.Close();
            int[] retVal = new int[arr.Count];
            arr.CopyTo(retVal);
            
            return retVal;
        }
        
        

        protected UserProperties GetUserPropertiesFromReader(MySqlDataReader reader)
        {
            int? lastDocType = null;
            if(!_tbl.LastDocType.IsDBNull(reader))
            {
                lastDocType = reader.GetInt32(_tbl.LastDocType.Name);
            }

            return new UserProperties(
                reader.GetString(_tbl.Login.Name),
                reader.GetString(_tbl.FullName.Name),
                reader.GetBoolean(_tbl.AutomaticPass.Name),
                reader.GetBoolean(_tbl.AllowDeputyNotification.Name),
                reader.GetInt32(_tbl.Department.Name),
                reader.GetBoolean(_tbl.InstantMessaging.Name),
                reader.GetString(_tbl.ImAddress.Name),
                reader.GetBoolean(_tbl.ChangePwdNextTime.Name),
                reader.GetBoolean(_tbl.ExternalAuthorization.Name),
                reader.GetString(_tbl.ExternalIdentity.Name),
                lastDocType,
                reader.GetInt32(_tbl.DcUserId.Name)
                );

            
        }

        public virtual UserProperties[] GetAllUserProperties()
        {
            if(!_con.IsOpen)
            {
                throw new ApplicationException("Can't receive all users. Connecection is not open.");
            }
            
            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                String.Format("select {0} from DcUser;", _tbl.GetColumnPart());
            MySqlDataReader reader = cmd.ExecuteReader();
            List<UserProperties> list = new List<UserProperties>();
            while (reader.Read())
            {
                list.Add(GetUserPropertiesFromReader(reader));
            }
            reader.Close();
            
            return list.ToArray();
        }

        public void SetUserPassword(int userId, string password)
        {
            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = String.Format(
                "update DcUser set {0}=Password(?{0}) where {1}=?{1};",
                "Password",
                _tbl.DcUserId.Name
                );
            cmd.Parameters.Add("?Password", MySqlDbType.String);
            cmd.Parameters["?Password"].Value = password;
            _tbl.DcUserId.SetParameter(cmd, userId);

            cmd.ExecuteNonQuery();
        }

        public virtual UserProperties GetUserProperties(int userId)
        {
            if (!_con.IsOpen)
            {
                throw new ApplicationException("Can't receive user properties. Connecection is not open.");
            }

            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                String.Format("select {0} from DcUser where {1}=?{1};", _tbl.GetColumnPart(), _tbl.DcUserId.Name);
            _tbl.DcUserId.SetParameter(cmd, userId);

            MySqlDataReader reader = cmd.ExecuteReader();

            UserProperties userProps = null;
            bool dataIsRead = reader.Read();
            try
            {
                if (!dataIsRead)
                {
                    throw new ApplicationException(
                        String.Format("Can't receive user properties. User with id {0} is not found in DB", userId)
                        );
                }

                userProps = GetUserPropertiesFromReader(reader);

            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                reader.Close();
            }

            return userProps;
        }

        public virtual UserToken AuthorizeUser(string login, string password)
        {
            if (!_con.IsOpen)
            {
                throw new ApplicationException("Can't authorize user. Connecection is not open.");
            }

            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                String.Format(
                    "select {0}, {1} from DcUser where {1}=?{1} and Password=password(?Password)",
                    _tbl.DcUserId.Name, _tbl.Login.Name
                    );
            _tbl.Login.SetParameter(cmd, login);
            cmd.Parameters.Add("?Password", MySqlDbType.VarChar);
            cmd.Parameters["?Password"].Value = password;

            MySqlDataReader reader = cmd.ExecuteReader();

            UserToken retVal = null;
            if (reader.Read())
            {
                retVal = new UserToken(reader.GetInt32(_tbl.DcUserId.Name), reader.GetString(_tbl.Login.Name));
            }
            reader.Close();
            return retVal;
        }

        public virtual int GetUserIdByLogin(string login)
        {
            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = String.Format(
                "select {0} from DcUser where upper({1})=upper(?{1})", 
                _tbl.DcUserId.Name, 
                _tbl.Login.Name
                );
            _tbl.Login.SetParameter(cmd, login);

            MySqlDataReader reader = cmd.ExecuteReader();

            bool isRead = reader.Read();
            if (!isRead)
            {
                throw new DcObjectNotFoundException(
                    String.Format("UserAccess.GetUserIdByLogin: Login \"{0}\" is not found!", login)
                    );
            }
            int retVal = reader.GetInt32(_tbl.DcUserId.Name);
            reader.Close();
            return retVal;
        }

        public virtual int GetUserIdByExternalIdentity(string identity)
        {
            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText =
                String.Format(
                    "select {0} from DcUser where upper({1})=upper(?{1})", 
                    _tbl.DcUserId.Name, 
                    _tbl.ExternalIdentity.Name
                    );
            _tbl.ExternalIdentity.SetParameter(cmd, identity);

            MySqlDataReader reader = cmd.ExecuteReader();

            bool isRead = reader.Read();
            if (!isRead)
            {
                throw new DcObjectNotFoundException(
                    String.Format("UserAccess.GetUserIdByExternalIdentity: Identity \"{0}\" is not found!", identity)
                    );
            }
            int retVal = reader.GetInt32(_tbl.DcUserId.Name);
            reader.Close();
            return retVal;
        }

        public void SaveUserProperties(UserProperties userProps)
        {
            MySqlCommand cmd = _con.CreateCommand();

            _tbl.DcUserId.AddParameter(cmd);

            _tbl.Login.SetParameter(cmd, userProps.Login);
            _tbl.FullName.SetParameter(cmd, userProps.FullName);
            _tbl.AutomaticPass.SetParameter(cmd, userProps.AutomaticPass);
            _tbl.AllowDeputyNotification.SetParameter(cmd, userProps.AllowDeputyNotification);
            _tbl.Department.SetParameter(cmd, userProps.Department);
            _tbl.InstantMessaging.SetParameter(cmd, userProps.InstantMessaging);
            _tbl.ImAddress.SetParameter(cmd, userProps.ImAddress);
            _tbl.ChangePwdNextTime.SetParameter(cmd, userProps.ChangePwdNextTime);
            _tbl.ExternalAuthorization.SetParameter(cmd, userProps.ExternalAuthorization);
            _tbl.ExternalIdentity.SetParameter(cmd, userProps.ExternalIdentity);
            _tbl.LastDocType.SetParameter(cmd, userProps.LastDocType);

            if (userProps.UserId == null)
            {
                cmd.CommandText = String.Format(
                    "insert into DcUser ({0}, Password) Values ({1}, Password(''));",
                    _tbl.GetColumnPart(DataTbl.SqlCmdType.InsertColumnPart),
                    _tbl.GetColumnPart(DataTbl.SqlCmdType.InsertValuePart)
                    );
                cmd.ExecuteNonQuery();
                userProps.UserId = (int)cmd.LastInsertedId;

                cmd.Parameters.Add("?DcRole", MySqlDbType.Int32);
                cmd.Parameters.Add("?DcUser", MySqlDbType.Int32);
                cmd.Parameters.Add("?Position", MySqlDbType.Int32);
                cmd.Parameters["?DcRole"].Value = DBConstKeeper.Current.MainRole.Users;
                cmd.Parameters["?DcUser"].Value = userProps.UserId;
                cmd.Parameters["?Position"].Value = 1;
                cmd.CommandText = "insert into RoleToUser (DcRole, DcUser, Position) " +
                    "Values (?DcRole, ?DcUser, ?Position)";
                cmd.ExecuteNonQuery();
            }
            else
            {
                
                _tbl.DcUserId.SetParameter(cmd, userProps.UserId);
                cmd.CommandText = String.Format(
                    "update DcUser set {0} where {1}=?{1};",
                    _tbl.GetColumnPart(DataTbl.SqlCmdType.UpdateSetPart),
                    _tbl.DcUserId.Name
                    );
                cmd.ExecuteNonQuery();
            }
        }

        public void SaveUserDeputies(int userId, DeputyItem[] deputies)
        {
            MySqlCommand cmd = _con.CreateCommand();
            cmd.Parameters.Add("?DeputyId", MySqlDbType.Int32);
            cmd.Parameters.Add("?DcUser", MySqlDbType.Int32);
            cmd.Parameters.Add("?Deputy", MySqlDbType.Int32);
            cmd.Parameters.Add("?Notify", MySqlDbType.Bit);
            cmd.Parameters["?DcUser"].Value = userId;


            MySqlCommand cleanup = _con.CreateCommand();
            cleanup.CommandText = "create temporary table if not exists tmpUserDeputy (DeputyId int);" +
                "delete from tmpUserDeputy;";
            cleanup.ExecuteNonQuery();
            cleanup.CommandText = "insert into tmpUserDeputy (DeputyId) Values (?DeputyId);";
            cleanup.Parameters.Add("?DeputyId", MySqlDbType.Int32);
            cleanup.Parameters.Add("?DcUser", MySqlDbType.Int32);
            cleanup.Parameters["?DcUser"].Value = userId;
            

            foreach (DeputyItem item in deputies)
            {
                if (item.User != userId)
                {
                    throw new ApplicationException("Wrong user in DeputyItem!");
                }

                cmd.Parameters["?Deputy"].Value = item.Deputy;
                cmd.Parameters["?Notify"].Value = item.Notify;

                if (item.DeputyId == null)
                {
                    cmd.CommandText = "insert into Deputy (DcUser, Deputy, Notify) " +
                        "Values (?DcUser, ?Deputy, ?Notify);";
                    cmd.ExecuteNonQuery();
                    item.DeputyId = (int)cmd.LastInsertedId;
                }
                else
                {
                    cmd.Parameters["?DeputyId"].Value = item.DeputyId;
                    cmd.CommandText = "update Deputy set DcUser=?DcUser, Deputy=?Deputy, Notify=?Notify " +
                        "where DeputyId=?DeputyId;";
                    cmd.ExecuteNonQuery();
                }

                cleanup.Parameters["?DeputyId"].Value = item.DeputyId;
                cleanup.ExecuteNonQuery();
            }

            cleanup.CommandText = "delete from Deputy where DcUser=?DcUser and " +
                "DeputyId not in (select DeputyId from tmpUserDeputy);";
            cleanup.ExecuteNonQuery();
        }

        public virtual string[] GetUserEmails(int userId)
        {
            if (!_con.IsOpen)
            {
                throw new ApplicationException("Can't receive user emails. Connecection is not open.");
            }

            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                "select UserEmail from UserEmail where DcUser=?DcUser";
            cmd.Parameters.Add(
                "?DcUser",
                MySqlDbType.Int32
                );
            cmd.Parameters["?DcUser"].Value = userId;

            MySqlDataReader reader = cmd.ExecuteReader();
            ArrayList arrList = new ArrayList();
            while (reader.Read())
            {
                arrList.Add(reader.GetString("UserEmail"));
            }
            reader.Close();
            string[] retVal = new string[arrList.Count];
            arrList.CopyTo(retVal);
            return retVal;
        }

        public virtual UserRole[] GetUserRoles(int userId)
        {
            
            if (!_con.IsOpen)
            {
                throw new DcConnectionClosedException();
            }

            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                "select RoleToUser.DcRole, DcRole.RoleName, RoleToUser.Position from RoleToUser " + 
                "inner join DcRole on RoleToUser.DcRole=DcRole.DcRoleId " +
                "where RoleToUser.DcUser=?DcUser " +
                "order by RoleToUser.Position ";
            cmd.Parameters.Add(
                "?DcUser",
                MySqlDbType.Int32
                );
            cmd.Parameters["?DcUser"].Value = userId;

            MySqlDataReader reader = cmd.ExecuteReader();

            List<UserRole> arr = new List<UserRole>();
            while (reader.Read())
            {
                arr.Add(new UserRole(reader.GetInt32("DcRole"), reader.GetString("RoleName")));
            }
            reader.Close();
            return arr.ToArray();
        }

        public void DeleteUser(int userId)
        {
            MySqlCommand cmd = _con.CreateCommand();
            _tbl.DcUserId.SetParameter(cmd, userId);

            cmd.CommandText = String.Format(
                        "{0} Deputy where DcUser=?{1}; " +
                        "{0} UserEmail where DcUser=?{1}; " +
                        "{0} RoleToUser where DcUser=?{1}; " +
                        "{0} DcUser where {1}=?{1}; ",
                        "delete from",
                        _tbl.DcUserId.Name
                        );
            cmd.ExecuteNonQuery();
        }

        public void SaveUserEmails(int userId, string[] emails)
        {
            string keyCol = "UserEmailId";
            Dictionary<string, int> dict = new Dictionary<string, int>();
            MySqlCommand cmd = _con.CreateCommand();
            _tbl.DcUserId.SetParameter(cmd, userId);
            cmd.CommandText = String.Format(
                "select {0}, UserEmail from UserEmail where DcUser=?{1};", 
                keyCol,
                _tbl.DcUserId.Name
                );

            MySqlDataReader reader = cmd.ExecuteReader();
            while (reader.Read())
            {
                dict.Add(reader.GetString("UserEmail"), reader.GetInt32(keyCol));
            }
            reader.Close();

            List<string> emailsToAdd = new List<string>();
            foreach (string email in emails)
            {
                if (dict.ContainsKey(email))
                {
                    dict.Remove(email);
                }
                else
                {
                    emailsToAdd.Add(email);
                }
            }

            // delete unnecessary
            cmd.CommandText = String.Format(
               "delete from UserEmail where {0}=?{0};",
               keyCol
               );
            cmd.Parameters.Add(keyCol, MySqlDbType.Int32);
            

            foreach (int id in dict.Values)
            {
                cmd.Parameters[keyCol].Value = id;
                cmd.ExecuteNonQuery();
            }

            // add new
            cmd.CommandText = String.Format(
                "insert into UserEmail (DcUser, UserEmail) Values (?{0}, ?UserEmail);",
                _tbl.DcUserId.Name
                );
            _tbl.DcUserId.SetParameter(cmd, userId);
            cmd.Parameters.Add("?UserEmail", MySqlDbType.String);

            foreach(string email in emailsToAdd)
            {
                cmd.Parameters["?UserEmail"].Value = email;
                cmd.ExecuteNonQuery();
            }

        }

        public DeputyItem[] GetUserDeputies(int userId)
        {
            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                "select DeputyId, DcUser, Deputy, Notify from Deputy where DcUser=?DcUser;";
            cmd.Parameters.Add("?DcUser", MySqlDbType.Int32);
            cmd.Parameters["?DcUser"].Value = userId;

            MySqlDataReader reader = cmd.ExecuteReader();
            List<DeputyItem> items = new List<DeputyItem>();
            while (reader.Read())
            {
                items.Add(new DeputyItem(
                    reader.GetInt32("DeputyId"),
                    reader.GetInt32("DcUser"),
                    reader.GetInt32("Deputy"),
                    reader.GetBoolean("Notify")
                    )
                    );
            }
            reader.Close();
            return items.ToArray();
        }

        public DeputyItem[] GetWhomUserIsDeputyFor(int userId)
        {
            MySqlCommand cmd = _con.CreateCommand();
            cmd.CommandText = 
                "select DeputyId, DcUser, Deputy, Notify from Deputy where Deputy=?Deputy;";
            cmd.Parameters.Add("?Deputy", MySqlDbType.Int32);
            cmd.Parameters["?Deputy"].Value = userId;
            MySqlDataReader reader = cmd.ExecuteReader();
            List<DeputyItem> items = new List<DeputyItem>();
            while (reader.Read())
            {
                items.Add(new DeputyItem(
                    reader.GetInt32("DeputyId"),
                    reader.GetInt32("DcUser"),
                    reader.GetInt32("Deputy"),
                    reader.GetBoolean("Notify")
                    )
                    );
            }
            reader.Close();
            return items.ToArray();
        }
       
    }

    
}
