//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.Biz;

namespace Evpo.DocCirc.UI
{
    

    public class DocTypeChecker
    {
        protected const string _newLine = "<br/>";

        protected DataConnection _con;
        protected UserInfo _serviceUserInfo;

        /// <summary>
        /// for unit testing
        /// </summary>
        public DocTypeChecker()
        {
        }

        public DocTypeChecker(UserInfo serviceUserInfo, DataConnection con)
        {
            _con = con;
            _serviceUserInfo = serviceUserInfo;
        }

        protected virtual DocTypeData GetDocTypeData(UserInfo userInfo, DataConnection con)
        {
            return new DocTypeData(userInfo, con);
        }

        public CheckerAnswer CheckDocTypeBeforeSave(DocTypeItem item)
        {
            if(!item.RoutePointsLoaded)
            {
                throw new ApplicationException("RoutePoints are not loaded!");

            }

            DocTypeData dtd = GetDocTypeData(_serviceUserInfo, _con);

            bool success = true;
            StringBuilder sb = new StringBuilder(
                ResourceHelper.StrForEx[ExResKeys.DocTypeCheckerTextInroduction]
                );

            // check that prefix is unique
            int[] prefixDocTypes = dtd.PrefixExists(item.Prefix);
            bool prefixIsUnique = true;
            foreach (int prefDocType in prefixDocTypes)
            {
                if (prefDocType != item.Id)
                {
                    prefixIsUnique = false;
                    break;
                }
            }

            if (!prefixIsUnique)
            {
                success = false;
                sb.Append(_newLine);
                sb.AppendFormat(
                    ResourceHelper.StrForEx[ExResKeys.DocTypeCheckerPrefixIsNotUnique],
                    item.Prefix
                    );
            }

            bool routeIsEmpty = false;
            if (item.RoutePoints.Count == 0)
            {
                success = false;
                routeIsEmpty = true;;
                sb.Append(_newLine);
                sb.Append(ResourceHelper.StrForEx[ExResKeys.DocTypeCheckerRouteIsEmpty]);
            }

            if (!routeIsEmpty)
            {
                if (item.RoutePoints[0].OrderNum != 1)
                {
                    success = false;
                    sb.Append(_newLine);
                    sb.AppendFormat(
                        ResourceHelper.StrForEx[ExResKeys.DocTypeCheckerMustBeginFromFirst],
                        item.RoutePoints[0].OrderNum.ToString()
                        );
                }


                for (int i = 0; i < item.RoutePoints.Count; i++)
                {
                    RoutePoint rp = item.RoutePoints[i];
                    //check repeated users on the route point
                    List<int> users = new List<int>();
                    foreach (RouteItem ri in rp.RouteItems)
                    {
                        if (users.Contains(ri.User))
                        {
                            success = false;
                            sb.Append(_newLine);
                            sb.AppendFormat(
                                ResourceHelper.StrForEx[ExResKeys.DocTypeCheckerEmployeeMoreThanOnceOnRoutePoint],
                                rp.OrderNum.ToString()
                                );
                        }
                        else
                        {
                            users.Add(ri.User);
                        }
                    }

                    //check gap between this and prev route points
                    if (i > 0 && rp.OrderNum - item.RoutePoints[i - 1].OrderNum > 1)
                    {
                        success = false;
                        sb.Append(_newLine);
                        sb.AppendFormat(
                            ResourceHelper.StrForEx[ExResKeys.DocTypeCheckerRoutePointIsMissed],
                            rp.OrderNum - item.RoutePoints[i - 1].OrderNum - 1,
                            item.RoutePoints[i - 1].OrderNum,
                            rp.OrderNum
                            );
                    }

                }
            }
            string userText = "";
            if (!success)
            {
                userText = sb.ToString();
            }

            return new CheckerAnswer(success, userText);

        }
    }


}
