//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Resources;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.Biz;

namespace Evpo.DocCirc.UI
{
    public class EventHtmlComposer
    {
        //protected struct ClassConst
        //{
        //    public const string EventHtmlComposerTemplate = "EventHtmlComposer.htm";
        //}

        #region Resources
        private const string _controlName = "EventHtmlComposer";
        public struct ResKeys
        {
            public const string EventTimeField = "EventTimeField";
            public const string EventUserField = "EventUserField";
            public const string NoteField = "NoteField";
            public const string DeputyField = "DeputyField";
            public const string AdministratorField = "AdministratorField";
            public const string RoutePoint = "RoutePoint";
            public const string DraftState = "DraftState";

            //Event types
            public const string EventTypePassDecision = "EventTypePassDecision";
            public const string EventTypePassDecisionAutomatically = "EventTypePassDecisionAutomatically";
            public const string EventTypePassDecisionByDeputy = "EventTypePassDecisionByDeputy";
            public const string EventTypeRejectDecision = "EventTypeRejectDecision";
            public const string EventTypeRejectDecisionByDeputy = "EventTypeRejectDecisionByDeputy";
            public const string EventTypeRejectDecisionByAdministrator = "EventTypeRejectDecisionByAdministrator";
            public const string EventTypePostponeDecision = "EventTypePostponeDecision";
            public const string EventTypePostponeDecisionByDeputy = "EventTypePostponeDecisionByDeputy";
            public const string EventTypePostponeDecisionByAdministrator = "EventTypePostponeDecisionByAdministrator";
            public const string EventTypeReturnToDraftDueToRouteChange = "EventTypeReturnToDraftDueToRouteChange";
            public const string EventTypeDocCardRegistration = "EventTypeDocCardRegistration";
            public const string EventTypeDocCardFinalization = "EventTypeDocCardFinalization";
            public const string EventTypeConsiderationStarted = "EventTypeConsiderationStarted";
            public const string EventTypePassedToNextRoutePoint = "EventTypePassedToNextRoutePoint";
        }
        protected ResourceManager _rm;

        #endregion
        
        protected SettingKeeper _sk;
        protected string _template;
        protected DataConnection _con;
        protected UserInfo _serviceUserInfo;

        protected virtual ResourceManager GetResourceManager()
        {
            return ResourceHelper.Current.GetResourceManagerForPage(_controlName);
        }

        protected string GetEventTypeText(int eventType)
        {
            string retVal = "";

            if (eventType == DBConstKeeper.Current.EventType.ConsiderationStarted)
            {
                retVal = _rm.GetString(ResKeys.EventTypeConsiderationStarted);
            }
            else if (eventType == DBConstKeeper.Current.EventType.DocCardFinalization)
            {
                retVal = _rm.GetString(ResKeys.EventTypeDocCardFinalization);
            }
            else if (eventType == DBConstKeeper.Current.EventType.DocCardRegistration)
            {
                retVal = _rm.GetString(ResKeys.EventTypeDocCardRegistration);
            }
            else if (eventType == DBConstKeeper.Current.EventType.PassedToNextRoutePoint)
            {
                retVal = _rm.GetString(ResKeys.EventTypePassedToNextRoutePoint);
            }
            else if (eventType == DBConstKeeper.Current.EventType.PassDecision)
            {
                retVal = _rm.GetString(ResKeys.EventTypePassDecision);
            }
            else if (eventType == DBConstKeeper.Current.EventType.PassDecisionAutomatically)
            {
                retVal = _rm.GetString(ResKeys.EventTypePassDecisionAutomatically);
            }
            else if (eventType == DBConstKeeper.Current.EventType.PassDecisionByDeputy)
            {
                retVal = _rm.GetString(ResKeys.EventTypePassDecisionByDeputy);
            }
            else if (eventType == DBConstKeeper.Current.EventType.PostponeDecision)
            {
                retVal = _rm.GetString(ResKeys.EventTypePostponeDecision);
            }
            else if (eventType == DBConstKeeper.Current.EventType.PostponeDecisionByDeputy)
            {
                retVal = _rm.GetString(ResKeys.EventTypePostponeDecisionByDeputy);
            }
            else if (eventType == DBConstKeeper.Current.EventType.PostponeDecisionByAdministrator)
            {
                retVal = _rm.GetString(ResKeys.EventTypePostponeDecisionByAdministrator);
            }
            else if (eventType == DBConstKeeper.Current.EventType.RejectDecision)
            {
                retVal = _rm.GetString(ResKeys.EventTypeRejectDecision);
            }
            else if (eventType == DBConstKeeper.Current.EventType.RejectDecisionByAdministrator)
            {
                retVal = _rm.GetString(ResKeys.EventTypeRejectDecisionByAdministrator);
            }
            else if (eventType == DBConstKeeper.Current.EventType.RejectDecisionByDeputy)
            {
                retVal = _rm.GetString(ResKeys.EventTypeRejectDecisionByDeputy);
            }
            else if (eventType == DBConstKeeper.Current.EventType.ReturnToDraftDueToRouteChange)
            {
                retVal = _rm.GetString(ResKeys.EventTypeReturnToDraftDueToRouteChange);
            }
            else
            {
                throw new ApplicationException("Unknown event type!");
            }

            return retVal;
        }

        public EventHtmlComposer(SettingKeeper sk, DataConnection con, UserInfo serviceUserInfo)
        {
            _sk = sk;
            _con = con;
            _serviceUserInfo = serviceUserInfo;

            //TextReader reader = File.OpenText(
            //    Path.Combine(_sk.ApplicationFolder, ClassConst.EventHtmlComposerTemplate)
            //    );
            //string template = reader.ReadToEnd();
            //reader.Close();

            string template = UIResources.EventHtmlComposer;
            _rm = GetResourceManager();

            ReplacedItem[] resourceReplaceItems = new ReplacedItem[]
            {
                new ReplacedItem("^reskey:EventTimeField^", _rm.GetString(ResKeys.EventTimeField)),
                new ReplacedItem("^reskey:EventUserField^", _rm.GetString(ResKeys.EventUserField)),
                new ReplacedItem("^reskey:NoteField^", _rm.GetString(ResKeys.NoteField))
            };
            _template = ComposerHelper.Current.ReplaceWithValues(template, resourceReplaceItems);

        }

        public string ComposeHtml(EventItem item)
        {
            string userDisplayName, deputyDisplayName = null, eventTypeText;
            bool eventWithoutUser = false;
            if (
                 item.EventType == DBConstKeeper.Current.EventType.DocCardFinalization ||
                 item.EventType == DBConstKeeper.Current.EventType.PassedToNextRoutePoint ||
                 item.EventType == DBConstKeeper.Current.EventType.ReturnToDraftDueToRouteChange
                 )
            {
                eventWithoutUser = true;
            }


            userDisplayName = UserHelper.Current.GetUserProperties(item.EventUser, _con).DisplayName;
            if (item.Deputy != null)
            {
                deputyDisplayName = UserHelper.Current.GetUserProperties(item.Deputy.Value, _con).DisplayName;
            }

            eventTypeText = GetEventTypeText(item.EventType);

            if(
                item.EventType == DBConstKeeper.Current.EventType.PassedToNextRoutePoint ||
                item.EventType == DBConstKeeper.Current.EventType.RejectDecision ||
                item.EventType == DBConstKeeper.Current.EventType.RejectDecisionByAdministrator ||
                item.EventType == DBConstKeeper.Current.EventType.RejectDecisionByDeputy 
                )
            {
                string target = "";
                if (item.RoutePointNum == 0)
                {
                    target = _rm.GetString(ResKeys.DraftState);
                }
                else
                {
                    target = String.Format(_rm.GetString(ResKeys.RoutePoint), item.RoutePointNum);
                    
                }
                eventTypeText = String.Format(eventTypeText, target);
                
            }


            List<ReplacedItem> replacedItemList = new List<ReplacedItem>();
            replacedItemList.Add(new ReplacedItem("^EventTime^", item.EventTime.ToString()));
            replacedItemList.Add(new ReplacedItem("^UserFullName^", userDisplayName));
            replacedItemList.Add(new ReplacedItem("^EventTypeText^", eventTypeText));

            
            if (eventWithoutUser)
            {
                // events without action user
                replacedItemList.Add(new ReplacedItem("^control:UserRowDisplay^", "none"));
            }
            else
            {
                replacedItemList.Add(new ReplacedItem("^control:UserRowDisplay^", "inline"));
            }


            if (item.Note != "" && !eventWithoutUser)
            {
                replacedItemList.Add(new ReplacedItem("^control:NoteRowDisplay^", "inline"));
                replacedItemList.Add(
                    new ReplacedItem(
                        "^Note^", 
                        item.Note.Replace(Environment.NewLine, "<br/>")
                        )
                    );
            }
            else
            {
                replacedItemList.Add(new ReplacedItem("^control:NoteRowDisplay^", "none"));
            }

            if (item.Deputy != null && !eventWithoutUser)
            {
                replacedItemList.Add(new ReplacedItem("^control:DeputyRowDisplay^", "inline"));
                if(
                    item.EventType == DBConstKeeper.Current.EventType.PostponeDecisionByAdministrator ||
                    item.EventType == DBConstKeeper.Current.EventType.RejectDecisionByAdministrator
                    )
                {
                    replacedItemList.Add(new ReplacedItem("^reskey:DeputyField^", _rm.GetString(ResKeys.AdministratorField)));
                }
                else
                {
                    replacedItemList.Add(new ReplacedItem("^reskey:DeputyField^", _rm.GetString(ResKeys.DeputyField)));
                }
                replacedItemList.Add(new ReplacedItem("^DeputyFullName^", deputyDisplayName));
            }
            else
            {
                replacedItemList.Add(new ReplacedItem("^control:DeputyRowDisplay^", "none"));
            }

            return ComposerHelper.Current.ReplaceWithValues(_template, replacedItemList.ToArray());
        }

       

    }
}
