//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections;
using System.Collections.Specialized;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Xml;
using System.Xml.Schema;
using System.Xml.Serialization;


namespace Evpo.DocCirc.UI
{
    /// <summary>
    /// 
    /// </summary>
    public class MenuKeeper
    {
        private MenuData _menuData;
        public MenuData MenuData
        {
            get { return _menuData; }
        }


        public DcMenuItem GetMenuItem(string dotSeparatedId)
        {
            int[] nodes = DecriptDotSeparatedId(dotSeparatedId);
            DcMenuItem retVal = _menuData.MenuItems[nodes[0]];
            for (int i = 1; i < nodes.Length; i ++ )
            {
                retVal = retVal.MenuItems[nodes[i]];
            }

            return retVal;
        }

        public MenuKeeper(TextReader xmlTextReader, TextReader xsdTextReader)
        {
            //validate xml against schema
            XmlSchema schema = XmlSchema.Read(xsdTextReader, new ValidationEventHandler(SchemaError));
            XmlReaderSettings readerSettings = new XmlReaderSettings();
            readerSettings.ValidationType = ValidationType.Schema;
            readerSettings.Schemas.Add(schema);
            XmlReader xmlReader = XmlReader.Create(xmlTextReader, readerSettings);
            XmlSerializer ser = new XmlSerializer(typeof(MenuData));
            _menuData = (MenuData)ser.Deserialize(xmlReader);
            SetItemIds(_menuData.MenuItems, "");
            
        }

        protected void SchemaError(object sender, ValidationEventArgs args)
        {
            throw args.Exception;
        }


        protected int[] DecriptDotSeparatedId(string dotSeparatedId)
        {
            List<int> list = new List<int>();
            string[] nodes = dotSeparatedId.Split('.');
            for(int i = 1; i < nodes.Length; i ++) // begin from [1] because in ".0.1" [0] must be always empty
            {
                list.Add(int.Parse(nodes[i]));
            }
            return list.ToArray();
        }

        protected void SetItemIds(DcMenuItem[] items, string parentId)
        {
            for (int i = 0; i < items.Length; i ++ )
            {
                DcMenuItem item = items[i];
                item.Id = String.Format("{0}.{1}", parentId, i);
                if (item.MenuItems != null && item.MenuItems.Length > 0)
                {
                    SetItemIds(item.MenuItems, item.Id);
                }
            }
        }
    }

    [Serializable]
    [XmlRoot(ElementName = "Main", Namespace = "http://tempuri.org/Page01.xsd", IsNullable = false)]
    [XmlType(AnonymousType = true)]
    public class MenuData
    {
        private DcMenuItem[] _menuItems;
        [XmlArrayItem(ElementName = "MenuItem", IsNullable = false)]
        public DcMenuItem[] MenuItems
        {
            get { return _menuItems; }
            set { _menuItems = value; }
        }

        private string _defaultMenuId;
        [XmlAttribute(AttributeName="DefaultMenuId")]
        public string DefaultMenuId
        {
            get { return _defaultMenuId; }
            set { _defaultMenuId = value; }
        }
    }

    [Serializable]
    [XmlType(Namespace = "http://tempuri.org/Page01.xsd")]
    public class DcMenuItem
    {
        private MenuItemParameter[] _parameters;
        [XmlArrayItem(ElementName = "MenuItemParameter", IsNullable = false)]
        public MenuItemParameter[] Parameters
        {
            get { return _parameters; }
            set { _parameters = value; }
        }

        private DcMenuItem[] _menuItems;
        [XmlArrayItem(ElementName = "MenuItem", IsNullable = false)]
        public DcMenuItem[] MenuItems
        {
            get { return _menuItems; }
            set { _menuItems = value; }
        }

        private string _nameResKey;
        [XmlAttribute()]
        public string NameResKey
        {
            get { return _nameResKey; }
            set { _nameResKey = value; }
        }

        private string _controlFile;
        [XmlAttribute()]
        public string ControlFile
        {
            get { return _controlFile; }
            set { _controlFile = value; }
        }

        private string _id;
        public string Id
        {
            get { return _id; }
            set { _id = value; }
        }
    }

    [Serializable]
    [XmlType(Namespace = "http://tempuri.org/Page01.xsd")]
    public class MenuItemParameter
    {
        private string _name;
        [XmlAttribute()]
        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }

        private string _value;
        [XmlAttribute()]
        public string Value
        {
            get { return _value; }
            set { _value = value; }
        }
    }

    public interface IParameterReceiver
    {
        void SetParameters(MenuItemParameter[] parameters);
    }
}
