//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;
using System.IO;
using Evpo.DocCirc.Biz;
using Evpo.DocCirc.Common;

namespace Evpo.DocCirc.UI
{
    [Serializable]
    public class RoutePointForClient
    {
        private RoutePoint _routePoint;

       
        [XmlAttribute]
        public int OrderNum
        {
            get { return _routePoint.OrderNum; }
            set { _routePoint.OrderNum = value; }
        }

      
        [XmlElement(IsNullable=true)]
        public int? Id
        {
            get { return _routePoint.Id; }
            set { _routePoint.Id = value; }
        }

        [XmlAttribute]
        public string Expression
        {
            get { return _routePoint.Expression; }
            set { _routePoint.Expression = value; }
        }

        private RouteItemForClient[] _routeItems;
        [XmlArrayItem(ElementName="RouteItemForClient")]
        public RouteItemForClient[] RouteItems
        {
            get { return _routeItems; }
            set { _routeItems = value; }
        }

        public RoutePointForClient()
        {
            _routePoint = new RoutePoint();
        }

        public RoutePointForClient(RoutePoint routePoint)
        {
            _routePoint = routePoint;
        }
    }


    [Serializable]
    public class RouteItemForClient
    {
        private RouteItem _routeItem;

        [XmlElement(IsNullable=true)]
        public int? Id
        {
            get { return _routeItem.Id; }
            set { _routeItem.Id = value; }
        }

        [XmlAttribute]
        public int User
        {
            get { return _routeItem.User; }
            set { _routeItem.User = value; }
        }

        [XmlAttribute]
        public int RouteItemType
        {
            get { return _routeItem.RouteItemType; }
            set { _routeItem.RouteItemType = value; }
        }

        private string _userFullName;

        [XmlAttribute]
        public string UserFullName
        {
            get { return _userFullName; }
            set { _userFullName = value; }
        }

        private int _pointOrderNum;
        [XmlAttribute]
        public int PointOrderNum
        {
            get { return _pointOrderNum; }
            set { _pointOrderNum = value; }
        }


        /// <summary>
        /// for serialization
        /// </summary>
        public RouteItemForClient()
        {
            _routeItem = new RouteItem();
        }

        public RouteItemForClient(RouteItem routeItem, string userFullName, int pointOrderNum)
        {
            _routeItem = routeItem;
            _userFullName = userFullName;
            _pointOrderNum = pointOrderNum;
        }
    }

    public class RoutePointSerializer
    {
        protected DataConnection _con;
        public RoutePointSerializer(DataConnection con)
        {
            _con = con;
        }

        public string SerializeRoutePoints(List<RoutePoint> points)
        {
            RoutePointForClient[] pointsForClient = new RoutePointForClient[points.Count];
            for (int p = 0; p < points.Count; p ++)
            {
                RoutePoint rp = points[p];
                pointsForClient[p] = new RoutePointForClient(rp);
                pointsForClient[p].RouteItems = new RouteItemForClient[rp.RouteItems.Count];

                for (int i = 0; i < rp.RouteItems.Count; i ++ )
                {
                    string displayName = UserHelper.Current.GetUserProperties(rp.RouteItems[i].User, _con).DisplayName;
                    pointsForClient[p].RouteItems[i] = new RouteItemForClient(rp.RouteItems[i], displayName, rp.OrderNum);
                }
            }

            XmlSerializer ser = new XmlSerializer(typeof(RoutePointForClient[]), new XmlRootAttribute("Points"));

            TextWriter wri = new StringWriter();
            ser.Serialize(wri, pointsForClient);

            return wri.ToString();
        }

        public List<RoutePoint> DeserializeRoutePoints(string text)
        {
            XmlSerializer ser = new XmlSerializer(typeof(RoutePointForClient[]), new XmlRootAttribute("Points"));
            RoutePointForClient[] pointsForClient = (RoutePointForClient[])ser.Deserialize(new StringReader(text));

            List<RoutePoint> points = new List<RoutePoint>(pointsForClient.Length);
            foreach (RoutePointForClient rpClient in pointsForClient)
            {
                RoutePoint rp = new RoutePoint(rpClient.Id, rpClient.Expression, rpClient.OrderNum);
                points.Add(rp);
                foreach (RouteItemForClient riClient in rpClient.RouteItems)
                {
                    rp.RouteItems.Add(new RouteItem(riClient.Id, riClient.User, riClient.RouteItemType));
                }
            }

            return points;
        }
    }
}
