//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.Biz;

namespace Evpo.DocCirc.UI
{
    public class UserPropertiesOperator
    {
        protected enum SortedColumn
        {
            FullName,
            Login
        }
        
        public void SortUserPropertiesListByFullName(List<UserProperties> userProps)
        {
            QuickSort(userProps, 0, userProps.Count - 1, SortedColumn.FullName);
        }

        public void SortUserPropertiesListByLogin(List<UserProperties> userProps)
        {
            QuickSort(userProps, 0, userProps.Count - 1, SortedColumn.Login);
        }
        
        protected void QuickSort(List<UserProperties> userProps, int left, int right, SortedColumn sortedCol)
        {
            if(right > left)
            {
                int pivotIndex = Partition(userProps, left, right, sortedCol);
                QuickSort(userProps, left, pivotIndex - 1, sortedCol);
                QuickSort(userProps, pivotIndex + 1, right, sortedCol);
            }
        }

        protected int Partition(List<UserProperties> userProps, int left, int right, SortedColumn sortedCol)
        {
            string pivotValue = null;

            if (sortedCol == SortedColumn.FullName)
            {
                pivotValue = userProps[right].FullName;
            }
            else if (sortedCol == SortedColumn.Login)
            {
                pivotValue = userProps[right].Login;
            }
            
            int storeIndex = left;
            for (int i = left; i < right; i++)
            {
                bool lesserOrEqualThanPivot = false;
                if (sortedCol == SortedColumn.FullName)
                {
                    lesserOrEqualThanPivot = userProps[i].FullName.CompareTo(pivotValue) <= 0;
                }
                else if (sortedCol == SortedColumn.Login)
                {
                    lesserOrEqualThanPivot = userProps[i].Login.CompareTo(pivotValue) <= 0;
                }


                if (lesserOrEqualThanPivot)
                {
                    Swap(userProps, i, storeIndex);
                    storeIndex++;
                }
            }

            Swap(userProps, right, storeIndex);

            return storeIndex;
        }

        protected void Swap(List<UserProperties> userProps, int i1, int i2)
        {
            if (i1 != i2)
            {
                UserProperties up = userProps[i1];
                userProps[i1] = userProps[i2];
                userProps[i2] = up;
            }
        }

    }
}
