//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Collections.Specialized;
using System.Reflection;
using System.Globalization;
using System.Threading;
using NUnit.Framework;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.Biz;
using Evpo.DocCirc.MySqlDL;
using Evpo.DocCirc.UI;

namespace Evpo.DocCircUnitTest
{
    [TestFixture]
    public class BizGlobalTest
    {
        /// <summary>
        /// see name
        /// </summary>
        protected class MockLogHelper : LogHelper
        {
            private ArrayList _logStrings = new ArrayList();
            public ArrayList LogStrings
            {
                get { return _logStrings; }
            }

            public MockLogHelper(string fileName)
            {
                _fileName = fileName;
            }

            public override void AddStr(string str)
            {
                _logStrings.Add(str);
                Console.WriteLine(str);
            }

            public override void MakeDelimiter()
            {
                string str = "delimiter";
                _logStrings.Add(str);
                Console.WriteLine(str);
            }

            public override void WriteException(Exception ex)
            {
                Console.WriteLine(ex);
            }

            public void InitValidate()
            {

                if (_fileName != @"c:\TEST\EvpoDocCirc.log")
                {
                    throw new ApplicationException("Wrong file name!");
                }

                if (_logStrings[0].ToString() != "delimiter")
                {
                    throw new ApplicationException("\"delimiter\" string is not appended to log!");
                }

                if (_logStrings[1].ToString() != "Application started US")
                {
                    throw new ApplicationException("\"Application started US\" string is not appended to log!");
                }
                Assert.AreEqual("", _logStrings[2]);

                Assert.AreEqual(
                    "User DocCircService is OK. His email is doccircservice@test.org. US",
                    _logStrings[3].ToString()
                    );

                if (_logStrings[4].ToString() != "Application initialized US")
                {
                    throw new ApplicationException("\"Application initialized US\" string is not appended to log!");
                }

            }
        }

        /// <summary>
        /// see name
        /// </summary>
        protected class MockDCAppSettings : DCAppSettings
        {
            private System.Collections.Specialized.NameValueCollection _settings;
            public System.Collections.Specialized.NameValueCollection Settings
            {
                get { return _settings; }
            }

            public MockDCAppSettings()
            {
                _settings = new System.Collections.Specialized.NameValueCollection();
            }

            public override string this[string key]
            {
                get { return _settings[key]; }
            }

            public override void LoadSettings(string folder)
            {
                // nothing
            }

            public void SetValue(string key, string val)
            {
                _settings[key] = val;
            }
        }

        /// <summary>
        /// see name
        /// </summary>
        class MockAssemblyWrapper : AssemblyWrapper
        {
            private bool _satellite;
            public bool Satellite
            {
                get { return _satellite; }
            }

            private string _assemblyPath;
            public string AssemblyPath
            {
                get { return _assemblyPath; }
            }

            public MockAssemblyWrapper(bool satellite, string assemblyPath)
            {
                _satellite = satellite;
                _assemblyPath = assemblyPath;
            }
        }

        /// <summary>
        /// see name
        /// </summary>
        class MockStringResHelper : StringResHelper
        {
            private string _baseName;
            public string BaseName
            {
                get { return _baseName; }
            }

            private AssemblyWrapper _assembly;
            public AssemblyWrapper Assembly
            {
                get { return _assembly; }
            }

            private NameValueCollection _strings = new NameValueCollection();
            public NameValueCollection Strings
            {
                get { return _strings; }
            }

            public MockStringResHelper(string baseName, AssemblyWrapper assembly)
            {
                _baseName = baseName;
                _assembly = assembly;
            }

            public override string this[string key]
            {
                get
                {
                    return _strings[key];
                }
            }
        }

        class MockDBConstAccess : DBConstAccess
        {
            public override int GetCommonDepartment()
            {
                return 0;
            }

            public override DocStatusDirectory GetDocStatusDirectory()
            {
                return null;
            }

            public override EventTypeDirectory GetEventTypeDirectory()
            {
                return null;
            }

            public override FolderTypeDirectory GetFolderTypeDirectory()
            {
                return null;
            }

            public override MainRoleDirectory GetMainRoleDirectory()
            {
                return null;
            }

            public override NotificationTypeDirectory GetNotificationTypeDirectory()
            {
                return null;
            }

            public override RootFolderDirectory GetRootFolderDirectory(FolderTypeDirectory ftDir)
            {
                return null;
            }

            public override RouteItemTypeDirectory GetRouteItemTypeDirectory()
            {
                return null;
            }
        }

        class MockMySqlDLGlobal : MySqlDLGlobal
        {
            public override string UpdateDatabase(SettingKeeper sk)
            {
                return "";
            }
        }
        /// <summary>
        /// see name
        /// </summary>
        class TestBizGlobal : BizGlobal
        {
            public TestBizGlobal()
            {
            }

            protected override MySqlDLGlobal GetMySqlDLGlobal()
            {
                return new MockMySqlDLGlobal();
            }

            protected override LogHelper GetLogHelper(string logFile)
            {
                return new MockLogHelper(logFile);
            }

            protected override Notificator GetNotificator(UserInfo serviceUserInfo, SettingKeeper sk, DataConnection con)
            {
                return null;
            }

            protected override DBConstAccess GetDBConstAccess(DBConnection con)
            {
                return new MockDBConstAccess();
            }

            protected override DBFieldDefKeeper GetDBFieldDefKeeper(string applicationFolder)
            {
                return null;
            }
            //protected override string GetMachineName()
            //{
            //    return "server";
            //}

            //protected override string GetRequestApplicationPath()
            //{
            //    return "/DC";
            //}

            //protected override string GetAbsoluteUri()
            //{
            //    return "http://server.TEST.ORG/DC/Page01.aspx";
            //}

            //protected override string MapPath(string path)
            //{
            //    string retVal = "";
            //    if (path == "/DC")
            //    {
            //        retVal = @"c:\test";
            //    }
            //    return retVal;
            //}

            protected override AssemblyWrapper LoadAssembly(string assemblyFilePath)
            {
                bool satellite =
                    (assemblyFilePath != @"c:\test\bin\Evpo.DocCirc.Resources.dll");

                MockAssemblyWrapper assembly = new MockAssemblyWrapper(satellite, assemblyFilePath);
                return assembly;
            }

            protected override StringResHelper GetStringResHelper(
                string baseName, AssemblyWrapper assembly
                )
            {
                MockStringResHelper resHelper = new MockStringResHelper(
                    baseName,
                    assembly
                    );
                if (((MockAssemblyWrapper)assembly).AssemblyPath == @"c:\test\bin\en-US\Evpo.DocCirc.Resources.dll")
                {
                    if (baseName == GlobalStringConstants.StringsForLogResourcesId)
                    {
                        resHelper.Strings.Add(LogResKeys.ApplicationInitialized, "Application initialized US");
                        resHelper.Strings.Add(LogResKeys.ApplicationStart, "Application started US");
                        resHelper.Strings.Add(LogResKeys.ServiceUserInfo, "User {0} is OK. His email is {1}. US");
                        resHelper.Strings.Add(LogResKeys.UpdateExecutedWithText, "");
                    }
                    else if (baseName == GlobalStringConstants.StringsForUIResourcesId)
                    {
                        resHelper.Strings.Add(UIResKeys.Greeting, "Welcome US");
                    }
                    else if (baseName == GlobalStringConstants.StringsForExResourcesId)
                    {
                        resHelper.Strings.Add(
                            ExResKeys.ConnectionStringIsNotSet,
                            "ConnectionString is not set in web.config US"
                            );

                        resHelper.Strings.Add(
                            ExResKeys.ServiceUserDoNotHaveEmail,
                            "Service user doesn't have an email! US"
                            );
                    }
                }
                else if (((MockAssemblyWrapper)assembly).AssemblyPath == @"c:\test\bin\Evpo.DocCirc.Resources.dll")
                {
                    if (baseName == GlobalStringConstants.StringsForLogResourcesId)
                    {
                        resHelper.Strings.Add(LogResKeys.ApplicationInitialized, "Application initialized");
                        resHelper.Strings.Add(LogResKeys.ApplicationStart, "Application started");
                        resHelper.Strings.Add(LogResKeys.ServiceUserInfo, "User {0} is OK. His email is {1}.");
                        resHelper.Strings.Add(LogResKeys.UpdateExecutedWithText, "");
                    }
                    else if (baseName == GlobalStringConstants.StringsForUIResourcesId)
                    {
                        resHelper.Strings.Add(UIResKeys.Greeting, "Welcome");
                    }
                    else if (baseName == GlobalStringConstants.StringsForExResourcesId)
                    {
                        resHelper.Strings.Add(
                            ExResKeys.ConnectionStringIsNotSet,
                            "ConnectionString is not set in web.config"
                            );

                        resHelper.Strings.Add(
                            ExResKeys.ServiceUserDoNotHaveEmail,
                            "Service user doesn't have an email!"
                            );
                    }
                }
                else if (((MockAssemblyWrapper)assembly).AssemblyPath == @"c:\test\SpecialResources\Evpo.DocCirc.Resources.dll")
                {
                    if (baseName == GlobalStringConstants.StringsForLogResourcesId)
                    {
                        resHelper.Strings.Add(LogResKeys.ApplicationInitialized, "Application initialized");
                        resHelper.Strings.Add(LogResKeys.ApplicationStart, "Application started");
                        resHelper.Strings.Add(LogResKeys.ServiceUserInfo, "User {0} is OK. His email is {1}.");
                        resHelper.Strings.Add(LogResKeys.UpdateExecutedWithText, "");
                    }
                    else if (baseName == GlobalStringConstants.StringsForUIResourcesId)
                    {
                        resHelper.Strings.Add(UIResKeys.Greeting, "Welcome");
                    }
                    else if (baseName == GlobalStringConstants.StringsForExResourcesId)
                    {
                        resHelper.Strings.Add(
                            ExResKeys.ConnectionStringIsNotSet,
                            "ConnectionString is not set in web.config"
                            );

                        resHelper.Strings.Add(
                            ExResKeys.ServiceUserDoNotHaveEmail,
                            "Service user doesn't have an email!"
                            );
                    }
                }


                return resHelper;
            }

            protected override DataConnection GetDataConnection(string connString)
            {
                return new MockDataConnection(connString);
            }

            protected override UserInfo GetUserInfo(string userLogin, DataConnection con)
            {
                return new MockUserInfo(userLogin, con);
            }

            protected override ResourceHelper GetResourceHelper(AssemblyWrapper assembly)
            {
                return new MockResourceHelper(assembly);
            }
        }




        /// <summary>
        /// see name
        /// </summary>
        public class MockResourceHelper : ResourceHelper
        {
            public AssemblyWrapper GetAssembly()
            {
                return _assembly;
            }

            public MockResourceHelper(AssemblyWrapper assembly)
            {
                _assembly = assembly;
            }
        }

        /// <summary>
        /// see name
        /// </summary>
        class MockFileWrapper : FileWrapper
        {
            private ArrayList _files = new ArrayList();
            public ArrayList Files
            {
                get { return _files; }
            }

            public override bool Exists(string path)
            {
                return _files.Contains(path);
            }
        }

        public class MockDataConnection : DataConnection
        {
            public MockDataConnection(string connString)
            {
                if (connString != "Database=EvpoDC;Data Source=localhost;User Id=test;Password=test")
                {
                    throw new ApplicationException("Wrong connection string!");
                }
            }

            public override void Open()
            {
                // nothing
            }

            public override void Close()
            {
                //nothing
            }
        }

        public class MockUserInfo : UserInfo
        {
            public MockUserInfo(string userLogin, DataConnection con)
            {
                if (userLogin != "DocCircService")
                {
                    throw new ApplicationException("Wrong user login!");
                }

                _userProperties = new UserProperties();
                _userProperties.UserId = 1;
                _userProperties.Login = "DocCircService";
                _userProperties.FullName = "DocCirc Service";
                _userProperties.AutomaticPass = false;
                _userProperties.AllowDeputyNotification = true;
                _userProperties.Department = 0;
                _userProperties.InstantMessaging = false;
                _userProperties.ImAddress = "";
                
                _emails = new string[] { "doccircservice@test.org" };
            }
        }

        private MockDCAppSettings GetMockDCAppSettings()
        {
            MockDCAppSettings appSettings = new MockDCAppSettings();
            appSettings.Settings.Add(SettingFileKeys.Domain, "TEST.ORG");
            appSettings.Settings.Add(SettingFileKeys.LogFile, @"c:\TEST\EvpoDocCirc.log");
            appSettings.Settings.Add(SettingFileKeys.MSIMHost, "false");
            appSettings.Settings.Add(
                SettingFileKeys.ConnectionString,
                "Database=EvpoDC;Data Source=localhost;User Id=test;Password=test"
                );
            appSettings.Settings.Add(
                SettingFileKeys.ServiceConnectionString,
                "Database=EvpoDC;Data Source=localhost;User Id=ServiceTest;Password=test"
                );
            appSettings.Settings.Add(SettingFileKeys.ServiceUserLogin, "DocCircService");
            appSettings.Settings.Add(SettingFileKeys.AttachmentBufferSize, "131072");
            appSettings.Settings.Add(SettingFileKeys.MaxAttachmentSize, "2048");
            appSettings.Settings.Add(SettingFileKeys.MaxFilesInFolder, "10000");
            appSettings.Settings.Add(SettingFileKeys.AttachmentFolder, @"c:\TEST\Attachments1");

            appSettings.Settings.Add(SettingFileKeys.EmailMessagingServer, "server01");
            appSettings.Settings.Add(SettingFileKeys.EmailMessagingAccount, "doccircservice");
            appSettings.Settings.Add(SettingFileKeys.EmailMessagingPassword, "doccircservice***");
            appSettings.Settings.Add(SettingFileKeys.EmailMessagingPort, "25");
            appSettings.Settings.Add(SettingFileKeys.EmailMessagingSSLEnabled, "true");
            appSettings.Settings.Add(SettingFileKeys.IMWebService, "http://server01/IMMessagingService");
            appSettings.Settings.Add(SettingFileKeys.IMWebServiceEnabled, "false");
            return appSettings;
        }

        [SetUp]
        public void SetUp()
        {
            DCAppSettings.Current = GetMockDCAppSettings();
        }


        //protected override string GetMachineName()
        //{
        //    return "server";
        //}

        //protected override string GetRequestApplicationPath()
        //{
        //    return "/DC";
        //}

        //protected override string GetAbsoluteUri()
        //{
        //    return "http://server.TEST.ORG/DC/Page01.aspx";
        //}

        //protected override string MapPath(string path)
        //{
        //    string retVal = "";
        //    if (path == "/DC")
        //    {
        //        retVal = @"c:\test";
        //    }
        //    return retVal;
        //}

        protected struct FixtureConst
        {
            public const string MachineName = "server";
            public const string RequestApplicationPath = "/DC";
            public const string MappedRequestApplicationPath = @"c:\test";
            public const string AbsoluteUri = "http://server.TEST.ORG/DC/Page01.aspx";
        }

        [Test]
        public void Application_StartResourcesInitializationFallback()
        {
            MockFileWrapper fileInstance = new MockFileWrapper();
            fileInstance.Files.Add(@"c:\test\bin\Evpo.DocCirc.Resources.dll");
            FileWrapper.Current = fileInstance;



            TestBizGlobal global = new TestBizGlobal();
            Dictionary<string, object> appStateDict = global.Init(
                FixtureConst.MachineName,
                FixtureConst.RequestApplicationPath,
                FixtureConst.MappedRequestApplicationPath,
                FixtureConst.AbsoluteUri
                ); //tested method

            MockStringResHelper logResHlp = (MockStringResHelper)ResourceHelper.StrForLog;

            Assert.IsFalse(((MockAssemblyWrapper)logResHlp.Assembly).Satellite);

            Assert.AreEqual(
                @"c:\test\bin\Evpo.DocCirc.Resources.dll",
                ((MockAssemblyWrapper)logResHlp.Assembly).AssemblyPath
                );

            Assert.AreEqual(
                GlobalStringConstants.StringsForLogResourcesId,
                logResHlp.BaseName
                );

            MockResourceHelper resHlpInst = (MockResourceHelper)ResourceHelper.Current;
            Assert.AreEqual(
                @"c:\test\bin\Evpo.DocCirc.Resources.dll",
                ((MockAssemblyWrapper)resHlpInst.GetAssembly()).AssemblyPath
                );
                

            MockStringResHelper uiResHlp = (MockStringResHelper)ResourceHelper.StrForUI;
            
            Assert.IsFalse(((MockAssemblyWrapper)uiResHlp.Assembly).Satellite);

            Assert.AreEqual(
                @"c:\test\bin\Evpo.DocCirc.Resources.dll",
                ((MockAssemblyWrapper)uiResHlp.Assembly).AssemblyPath
                );

            Assert.AreEqual(
                GlobalStringConstants.StringsForUIResourcesId,
                uiResHlp.BaseName
                );

            MockStringResHelper exResHlp = (MockStringResHelper)ResourceHelper.StrForEx;

            Assert.IsFalse(((MockAssemblyWrapper)exResHlp.Assembly).Satellite);

            Assert.AreEqual(
                @"c:\test\bin\Evpo.DocCirc.Resources.dll",
                ((MockAssemblyWrapper)exResHlp.Assembly).AssemblyPath
                );

            Assert.AreEqual(
                GlobalStringConstants.StringsForExResourcesId,
                exResHlp.BaseName
                );

            MockLogHelper logHlp = (MockLogHelper)LogHelper.Current;
            
            Assert.AreEqual(5, logHlp.LogStrings.Count);

            Assert.AreEqual(
                "Application started",
                logHlp.LogStrings[1]
                );

            Assert.AreEqual("", logHlp.LogStrings[2]);

            Assert.AreEqual(
                "User DocCircService is OK. His email is doccircservice@test.org.",
                logHlp.LogStrings[3]
                );

            Assert.AreEqual(
                "Application initialized",
                logHlp.LogStrings[4]
                );

            Assert.AreEqual(
                "Welcome",
                ResourceHelper.StrForUI[UIResKeys.Greeting]
                );

            Assert.AreEqual(
                "ConnectionString is not set in web.config",
                ResourceHelper.StrForEx[ExResKeys.ConnectionStringIsNotSet]
                );
                
        }

        [Test]
        public void GetIsSetupNeeded()
        {
            DCAppSettings prevSettings = DCAppSettings.Current;

            TestBizGlobal global = new TestBizGlobal();
            MockDCAppSettings appSettings = GetMockDCAppSettings();
            DCAppSettings.Current = appSettings;

            string[] requiredKeys = new string[]
            {
                SettingFileKeys.LogFile,
                SettingFileKeys.ConnectionString,
                SettingFileKeys.ServiceConnectionString,
                SettingFileKeys.AttachmentFolder,
                SettingFileKeys.ServiceUserLogin
            };

            bool isSetupNeeded = global.GetIsSetupNeeded();
            Assert.IsFalse(isSetupNeeded);

            foreach (string key in requiredKeys)
            {
                string prevValue = appSettings[key];
                appSettings.SetValue(key, String.Empty);
                isSetupNeeded = global.GetIsSetupNeeded();
                appSettings.SetValue(key, prevValue);
                Assert.IsTrue(isSetupNeeded);
            }

            DCAppSettings.Current = prevSettings;
        }

        [Test]
        public void Init()
        {
            MockDCAppSettings appSettings = (MockDCAppSettings)DCAppSettings.Current;
            //appSettings.Settings.Add(SettingFileKeys.ApplicationCulture, "en-US");
            string prevCulture = Thread.CurrentThread.CurrentUICulture.Name;
            Thread.CurrentThread.CurrentUICulture = new CultureInfo("en-US");

            MockFileWrapper fileInstance = new MockFileWrapper();
            fileInstance.Files.Add(@"c:\test\bin\en-US\Evpo.DocCirc.Resources.dll");
            FileWrapper.Current = fileInstance;

            TestBizGlobal global = new TestBizGlobal();
            Dictionary<string, object> appStateDict = global.Init(
                FixtureConst.MachineName,
                FixtureConst.RequestApplicationPath,
                FixtureConst.MappedRequestApplicationPath,
                FixtureConst.AbsoluteUri
                ); // tested method

            Thread.CurrentThread.CurrentUICulture = new CultureInfo(prevCulture);

            Assert.IsTrue(appStateDict.ContainsKey(AppStateKeys.SettingKeeper));
            SettingKeeper sk = (SettingKeeper)appStateDict[AppStateKeys.SettingKeeper];

            Assert.AreEqual("TEST.ORG", sk.Domain);

            Assert.AreEqual("server", sk.ServerName);
            Assert.AreEqual("http://server/DC", sk.ApplicationUrl);

            //Assert.AreEqual("LDAP://server/DC=TEST,DC=ORG",sk.ADAccessPath);

            //Assert.AreEqual(@"c:\test\msgtempl.txt",sk.MessageTemplate);
            
            Assert.AreEqual(@"c:\test\bin",sk.ResourceFileFolder);

            Assert.AreEqual(@"c:\test", sk.ApplicationFolder);

            Assert.AreEqual("en-US", sk.ApplicationCulture);

            Assert.AreEqual(
                "Database=EvpoDC;Data Source=localhost;User Id=test;Password=test",
                sk.ConnectionString
                );

            Assert.AreEqual(131072, sk.AttachmentBufferSize);
            Assert.AreEqual(2048, sk.MaxAttachmentSize);
            Assert.AreEqual(10000, sk.MaxFilesInFolder);
            Assert.AreEqual(@"c:\TEST\Attachments1", sk.AttachmentFolder);

            Assert.AreEqual("server01", sk.EmailMessagingServer);
            Assert.AreEqual("doccircservice", sk.EmailMessagingAccount);
            Assert.AreEqual("doccircservice***", sk.EmailMessagingPassword);
            Assert.AreEqual(25, sk.EmailMessagingPort);
            Assert.IsTrue(sk.EmailMessagingSSLEnabled);
            Assert.AreEqual("http://server01/IMMessagingService", sk.IMWebService);
            Assert.AreEqual(false, sk.IMWebServiceEnabled);


            //check UI resources
            Assert.AreEqual(
                "Welcome US",
                ResourceHelper.StrForUI[UIResKeys.Greeting]
                );

            MockResourceHelper resHlpInst = (MockResourceHelper)ResourceHelper.Current;
            Assert.AreEqual(
                @"c:\test\bin\en-US\Evpo.DocCirc.Resources.dll",
                ((MockAssemblyWrapper)resHlpInst.GetAssembly()).AssemblyPath
                );
            
            //check UserInfo for service user
            Assert.IsTrue(appStateDict.ContainsKey(AppStateKeys.ServiceUserInfo));
            UserInfo ui = (UserInfo)appStateDict[AppStateKeys.ServiceUserInfo];
            
            Assert.AreEqual("DocCircService", ui.Login);

            Assert.AreEqual("DocCirc Service", ui.FullName);

            Assert.AreEqual(1, ui.UserId);

            Assert.AreEqual(1, ui.Emails.Length);

            Assert.AreEqual("doccircservice@test.org", ui.Emails[0]);

            ((MockLogHelper)LogHelper.Current).InitValidate();
        }

        [Test]
        public void Application_StartCfgResourcesPath()
        {
            MockDCAppSettings appSettings = (MockDCAppSettings)DCAppSettings.Current;
            appSettings.Settings.Add(SettingFileKeys.ResourceFileFolder, @"c:\test\SpecialResources");

            MockFileWrapper fileInst = new MockFileWrapper();
            fileInst.Files.Add(@"c:\test\SpecialResources\Evpo.DocCirc.Resources.dll");
            FileWrapper.Current = fileInst;

            TestBizGlobal global = new TestBizGlobal();
            Dictionary<string, object> appStateDict = global.Init(
                FixtureConst.MachineName,
                FixtureConst.RequestApplicationPath,
                FixtureConst.MappedRequestApplicationPath,
                FixtureConst.AbsoluteUri
                ); // tested method


            Assert.IsTrue(appStateDict.ContainsKey(AppStateKeys.SettingKeeper));
            SettingKeeper sk = (SettingKeeper)appStateDict[AppStateKeys.SettingKeeper];

            Assert.AreEqual(@"c:\test\SpecialResources", sk.ResourceFileFolder);

            MockResourceHelper resHlpInst = (MockResourceHelper)ResourceHelper.Current;
            Assert.AreEqual(
                @"c:\test\SpecialResources\Evpo.DocCirc.Resources.dll",
                ((MockAssemblyWrapper)resHlpInst.GetAssembly()).AssemblyPath
                );
            
        }

        [Test]
        public void Application_StartResourcesInitializationWrongName()
        {
            MockDCAppSettings appSettings = (MockDCAppSettings)DCAppSettings.Current;
            //appSettings.SetValue(SettingFileKeys.ApplicationCulture,"no-CULTURE");
            string prevCulture = Thread.CurrentThread.CurrentUICulture.Name;
            Thread.CurrentThread.CurrentUICulture = new CultureInfo("fr-FR");
            MockFileWrapper fileInstance = new MockFileWrapper();
            fileInstance.Files.Add(@"c:\test\bin\Evpo.DocCirc.Resources.dll");
            FileWrapper.Current = fileInstance;

            TestBizGlobal global = new TestBizGlobal();
            Dictionary<string, object> appStateDict = global.Init(
                FixtureConst.MachineName,
                FixtureConst.RequestApplicationPath,
                FixtureConst.MappedRequestApplicationPath,
                FixtureConst.AbsoluteUri
                ); // tested method
            Thread.CurrentThread.CurrentUICulture = new CultureInfo(prevCulture);

            MockStringResHelper logResHlp = (MockStringResHelper)ResourceHelper.StrForLog;
            
            Assert.IsFalse(((MockAssemblyWrapper)logResHlp.Assembly).Satellite);

            Assert.AreEqual(
                GlobalStringConstants.StringsForLogResourcesId,
                logResHlp.BaseName
                );

            MockStringResHelper uiResHlp = (MockStringResHelper)ResourceHelper.StrForUI;
            Assert.IsFalse(((MockAssemblyWrapper)uiResHlp.Assembly).Satellite);

            Assert.AreEqual(
                GlobalStringConstants.StringsForUIResourcesId,
                uiResHlp.BaseName
                );
            
            MockResourceHelper resHlpInst = (MockResourceHelper)ResourceHelper.Current;
            Assert.AreEqual(
                @"c:\test\bin\Evpo.DocCirc.Resources.dll",
                ((MockAssemblyWrapper)resHlpInst.GetAssembly()).AssemblyPath
                );
            
        }

        [Test]
        [
            ExpectedException(
            ExpectedException=typeof(ApplicationException),
            ExpectedMessage = "Can't open any resource file. Check existence of " +
            "c:\\test\\bin\\Evpo.DocCirc.Resources.dll"
            )
        ]
        public void Application_StartResourcesInializationNoResourceFile()
        {
            MockFileWrapper fileInst = new MockFileWrapper();
            FileWrapper.Current = fileInst;

            TestBizGlobal global = new TestBizGlobal();
            Dictionary<string, object> appStateDict = global.Init(
                FixtureConst.MachineName,
                FixtureConst.RequestApplicationPath,
                FixtureConst.MappedRequestApplicationPath,
                FixtureConst.AbsoluteUri
                ); // tested method
        }

        [TearDown]
        public void ClearAll()
        {
            DCAppSettings.Current = null;
            FileWrapper.Current = null;
            ResourceHelper.StrForLog = null;
            ResourceHelper.StrForUI = null;
            LogHelper.Current = null;
        }
        
    }
}
