//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
using Evpo.DocCirc.Biz;
using Evpo.DocCirc.Common;

namespace Evpo.DocCircUnitTest
{
    
    [TestFixture]
    public class DocCardMovementControllerTest
    {
        protected struct FixtureConst
        {
            public const int DocCard1 = 5, DocCard2 = 6;
            
            public const int DocType1 = 10;

            public const int DocCircUser1 = 1;
            public const int DocCircUser2 = 2;
            public const int DocCircUser3 = 3;
            public const int DocCircUser4 = 4;
            public const int DocCircUser5 = 5;
        }

        protected class FixtureData
        {
            public static DocCardItem DocCardItem1, DocCardItem2, DocCardItem3;
            public static UserProperties DocCircUser1Prop, DocCircUser2Prop,
                DocCircUser3Prop, DocCircUser4Prop, DocCircUser5Prop;
            public static DeputyItem[] DocCircUser1Deputies, DocCircUser2Deputies, DocCircUser3Deputies, 
                DocCircUser4Deputies, DocCircUser5Deputies;
        }

        protected class MockUserInfo : UserInfo
        {
            private bool _isAdministrator = false;
            public override bool UserIsAdministrator()
            {
                return _isAdministrator;
            }

            public MockUserInfo(int userId)
            {
                _userProperties = new UserProperties();
                _userProperties.UserId = userId;
            }

            public MockUserInfo(int userId, bool isAdministrator)
            {
                _userProperties = new UserProperties();
                _userProperties.UserId = userId;
                _isAdministrator = isAdministrator;
                
            }
        }

        protected class MockUserHelperInstance : UserHelperInstance
        {
            public override DeputyItem[] GetUserDeputies(int userId, DataConnection con)
            {
                DeputyItem[] retVal = null;
                if(userId == FixtureConst.DocCircUser1)
                {
                    retVal = FixtureData.DocCircUser1Deputies;
                }
                else if (userId == FixtureConst.DocCircUser2)
                {
                    retVal = FixtureData.DocCircUser2Deputies;
                }
                else if (userId == FixtureConst.DocCircUser3)
                {
                    retVal = FixtureData.DocCircUser3Deputies;
                }
                else if (userId == FixtureConst.DocCircUser4)
                {
                    retVal = FixtureData.DocCircUser4Deputies;
                }
                else if (userId == FixtureConst.DocCircUser5)
                {
                    retVal = FixtureData.DocCircUser5Deputies;
                }
                else
                {
                    throw new ApplicationException("Unknown id for this unit test!");
                }

                return retVal;
            }

            public override UserProperties GetUserProperties(int userId, DataConnection con)
            {
                UserProperties retVal = null;
                if (userId == FixtureConst.DocCircUser1)
                {
                    retVal = FixtureData.DocCircUser1Prop;
                }
                else if (userId == FixtureConst.DocCircUser2)
                {
                    retVal = FixtureData.DocCircUser2Prop;
                }
                else if (userId == FixtureConst.DocCircUser3)
                {
                    retVal = FixtureData.DocCircUser3Prop;
                }
                else if (userId == FixtureConst.DocCircUser4)
                {
                    retVal = FixtureData.DocCircUser4Prop;
                }
                else if (userId == FixtureConst.DocCircUser5)
                {
                    retVal = FixtureData.DocCircUser5Prop;
                }
                else
                {
                    throw new ApplicationException("Unknown id for this unit test!");
                }

                return retVal;
            }

        }

        protected class TestDocCardMovementController : DocCardMovementController
        {
            public TestDocCardMovementController(UserInfo userInfo, DataConnection con)
                :base(userInfo, con)
            {
            }

        }

        [TestFixtureSetUp]
        public void TFSetUp()
        {
            DBConstKeeper.Current = new DBConstKeeper();
            DBConstKeeper.Current.DocStatus = new DocStatusDirectory(0, 1, 2, 3);
            DBConstKeeper.Current.EventType = new EventTypeDirectory(0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13);
            DBConstKeeper.Current.NotificationType = new NotificationTypeDirectory(0, 1, 2, 3, 4, 5, 6, 7, 8);
            DBConstKeeper.Current.RouteItemType = new RouteItemTypeDirectory(0, 1);

            // DocCircUser1 has a deputy DocCircUser2, Notify=true
            bool notify = true;
            FixtureData.DocCircUser1Deputies = new DeputyItem[]
            {
                new DeputyItem(0, FixtureConst.DocCircUser1, FixtureConst.DocCircUser2, notify)
            };

            FixtureData.DocCircUser2Deputies = new DeputyItem[0];
            FixtureData.DocCircUser3Deputies = new DeputyItem[0];
            FixtureData.DocCircUser4Deputies = new DeputyItem[0];

            // DocCircUser5 has a deputy. He is DocCircUser3, Notify=false
            FixtureData.DocCircUser5Deputies = new DeputyItem[]
            {
                new DeputyItem(0, FixtureConst.DocCircUser5, FixtureConst.DocCircUser3, !notify)
            };


            //DocCircUser1
            //AllowDeputyNotif
            UserProperties prop = new UserProperties();
            prop.UserId = FixtureConst.DocCircUser1;
            prop.AllowDeputyNotification = true;
            prop.AutomaticPass = false;
            FixtureData.DocCircUser1Prop = prop;
            
            //DocCircUser2
            prop = new UserProperties();
            prop.UserId = FixtureConst.DocCircUser2;
            prop.AllowDeputyNotification = true;
            prop.AutomaticPass = false;
            FixtureData.DocCircUser2Prop = prop;

            //DocCircUser3
            prop = new UserProperties();
            prop.UserId = FixtureConst.DocCircUser3;
            prop.AllowDeputyNotification = true;
            prop.AutomaticPass = false;
            FixtureData.DocCircUser3Prop = prop;

            //DocCircUser4: AutomaticPass
            prop = new UserProperties();
            prop.UserId = FixtureConst.DocCircUser4;
            prop.AllowDeputyNotification = true;
            prop.AutomaticPass = true;
            FixtureData.DocCircUser4Prop = prop;

            //DocCircUser5
            prop = new UserProperties();
            prop.UserId = FixtureConst.DocCircUser5;
            prop.AllowDeputyNotification = true;
            prop.AutomaticPass = false;
            FixtureData.DocCircUser5Prop = prop;

            UserHelper.Current = new MockUserHelperInstance();

            /*  DocCardItem1
             * 
             * In the draft state
             * Registering user: DocCircUser1
             * */
            DocCardItem item = new DocCardItem();
            item.Id = FixtureConst.DocCard1;
            item.RegisteringUser = FixtureConst.DocCircUser1;
            item.RoutePointNum = 0;
            item.RoutePoints = new RoutePoint[0];
            item.RouteIsLoaded = true;
            FixtureData.DocCardItem1 = item;

            /*  DocCardItem2
             * 
             * Route:
             * 1 DocCircUser2(Decision) [Under cons now]
             * 
             * 2 DocCircUser3(Decision) AND DocCircUser4(Decision, AutomaticPass) AND 
             * DocCircUser1(Decision, Deputy:DocCircUser2 Notif, AllowDeputyNotif); DocCircUser5(Notification)
             * 
             * 3 DocCircUser5(Decision, Deputy:DocCircUser3 NOT Notif, AllowDeputyNotif) AND 
             * DocCircUser1(Decision)
             * 
             * 4 DocCircUser1(Notification)
             * */
            item = new DocCardItem(FixtureData.DocCardItem1);
            item.Id = FixtureConst.DocCard2;
            item.DocStatus = DBConstKeeper.Current.DocStatus.UnderCons;
            item.RoutePointNum = 1;
            item.RoutePoints = new RoutePoint[]
            {
                new RoutePoint(0, "{0}", 1),
                new RoutePoint(1, "{0}&&{1}&&{2}", 2),
                new RoutePoint(2, "{0}&&{1}", 3),
                new RoutePoint(3, "", 4)
            };

            // route point 1
            item.RoutePoints[0].RouteItems = new List<RouteItem>();
            RouteItem ri = new RouteItem(
                0,
                FixtureConst.DocCircUser2,
                DBConstKeeper.Current.RouteItemType.Decision
                );
            bool passed = true, underCons = true;
            item.RoutePoints[0].RouteItems.Add(new DocCardRouteItem(0, !passed, underCons, ri));

            // route point 2
            // DocCircUser3(Decision) AND DocCircUser4(Decision, AutomaticPass) AND 
            // DocCircUser1(Decision, Deputy:DocCircUser2 Notif, AllowDeputyNotif); DocCircUser5(Notification)
            ri = new RouteItem(
                1,
                FixtureConst.DocCircUser3,
                DBConstKeeper.Current.RouteItemType.Decision
                );
            item.RoutePoints[1].RouteItems.Add(new DocCardRouteItem(1, !passed, !underCons, ri));

            ri = new RouteItem(
                2,
                FixtureConst.DocCircUser4,
                DBConstKeeper.Current.RouteItemType.Decision
                );
            item.RoutePoints[1].RouteItems.Add(new DocCardRouteItem(2, !passed, !underCons, ri));

            ri = new RouteItem(
                3,
                FixtureConst.DocCircUser1,
                DBConstKeeper.Current.RouteItemType.Decision
                );
            item.RoutePoints[1].RouteItems.Add(new DocCardRouteItem(3, !passed, !underCons, ri));

            ri = new RouteItem(
                4,
                FixtureConst.DocCircUser5,
                DBConstKeeper.Current.RouteItemType.Notification
                );
            item.RoutePoints[1].RouteItems.Add(new DocCardRouteItem(4, !passed, !underCons, ri));
            
            FixtureData.DocCardItem2 = item;

            // route point 3
            // 3 DocCircUser5(Decision) AND DocCircUser2(Decision)
            ri = new RouteItem(
                5,
                FixtureConst.DocCircUser5,
                DBConstKeeper.Current.RouteItemType.Decision
                );
            item.RoutePoints[2].RouteItems.Add(new DocCardRouteItem(5, !passed, !underCons, ri));

            ri = new RouteItem(
                6,
                FixtureConst.DocCircUser1,
                DBConstKeeper.Current.RouteItemType.Decision
                );
            item.RoutePoints[2].RouteItems.Add(new DocCardRouteItem(5, !passed, !underCons, ri));

            // route point 4
            //DocCircUser1(Notification)
            ri = new RouteItem(
                7,
                FixtureConst.DocCircUser1,
                DBConstKeeper.Current.RouteItemType.Notification
                );
            item.RoutePoints[3].RouteItems.Add(new DocCardRouteItem(6, !passed, !underCons, ri));

        }

        [Test]
        public void StartConsideration()
        {
            /* Case 1
             * 
             * DocCard1 is in the Draft state
             * DocCircUser1 is the registering user
             * DocCircUser1 starts consideration by checking UnserCons check box and saving the card
             * 
             * User on the first route point must be notified. He did not set automatic pass
             * 
             * Route:
             * 1 DocCircUser2(Decision)
             * */
            DocCardItem item = new DocCardItem(FixtureData.DocCardItem1);
            item.RoutePoints = new RoutePoint[]
            {
                new RoutePoint(0, "{0}", 1)
            };

            item.RoutePoints[0].RouteItems = new List<RouteItem>();
            RouteItem ri = new RouteItem(
                0, 
                FixtureConst.DocCircUser2, 
                DBConstKeeper.Current.RouteItemType.Decision
                );
            item.RoutePoints[0].RouteItems.Add(new DocCardRouteItem(0, false, false, ri));

            TestDocCardMovementController cnt = new TestDocCardMovementController(
                new MockUserInfo(FixtureConst.DocCircUser1),
                null
                );

            cnt.StartConsideration(item, FixtureConst.DocCircUser1); //tested method

            Assert.AreEqual(
                DBConstKeeper.Current.DocStatus.UnderCons,
                item.DocStatus
                );

            Assert.IsNotNull(item.LastMovement);

            Assert.AreEqual(
                1,
                item.RoutePointNum
                );

            Assert.IsTrue(((DocCardRouteItem)item.RoutePoints[0].RouteItems[0]).UnderCons);
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[0].RouteItems[0]).Passed);

            // check events
            Assert.AreEqual(2, cnt.Events.Count);
            
            EventItem ei = cnt.Events[0];
            
            Assert.AreEqual(item.Id, ei.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.EventType.ConsiderationStarted, ei.EventType);
            Assert.AreEqual(FixtureConst.DocCircUser1, ei.EventUser);
            Assert.IsNull(ei.Deputy);
            Assert.AreEqual(0, ei.RoutePointNum);

            Assert.That(
                DateTime.Now.AddSeconds(-2) < ei.EventTime &&
                ei.EventTime < DateTime.Now.AddSeconds(2)
                );

            ei = cnt.Events[1];

            Assert.AreEqual(item.Id, ei.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.EventType.PassedToNextRoutePoint, ei.EventType);
            Assert.AreEqual(FixtureConst.DocCircUser1, ei.EventUser);
            Assert.IsNull(ei.Deputy);
            Assert.AreEqual(1, ei.RoutePointNum);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ei.EventTime &&
                ei.EventTime < DateTime.Now.AddSeconds(2)
                );

            // check notifications
            Assert.AreEqual(1, cnt.Notifications.Count);
            
            NotificationItem ni = cnt.Notifications[0];

            Assert.AreEqual(item.Id, ni.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.NotificationType.NextDecision, ni.NotificationType);
            Assert.AreEqual(FixtureConst.DocCircUser2, ni.User);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ni.NotificationTime &&
                ni.NotificationTime < DateTime.Now.AddSeconds(2)
                );
            Assert.IsEmpty(ni.UserNote);
            Assert.AreEqual(FixtureConst.DocCircUser1, ni.LastActionUser);
        }

        [Test]
        public void Pass()
        {
            /*
             *  Route:
             * 
             * 1 DocCircUser2(Decision) [Under cons now]
             * 2 DocCircUser3(Decision) AND DocCircUser4(Decision, AutomaticPass) AND 
             * DocCircUser1(Decision, Deputy:DocCircUser2 Notif, AllowDeputyNotif); DocCircUser5(Notification)
             * 3 DocCircUser5(Decision, Deputy:DocCircUser3 NOT Notif, AllowDeputyNotif)
             * AND DocCircUser1(Decision)
             * 4 DocCircUser1(Notification)
             * */


            /* Case 1
             * DocCircUser2 pass the doc card on the next route point
             * he adds note: "Hello guys!"
            */

            DocCardItem item = new DocCardItem(FixtureData.DocCardItem2);
            


            TestDocCardMovementController cnt = new TestDocCardMovementController(
                new MockUserInfo(FixtureConst.DocCircUser2),
                null
                );

            cnt.Pass(item, FixtureConst.DocCircUser2, "Hello guys!"); //tested method

            Assert.IsNotNull(item.LastMovement);

            Assert.AreEqual(
                DBConstKeeper.Current.DocStatus.UnderCons,
                item.DocStatus
                );

            Assert.AreEqual(
                2,
                item.RoutePointNum
                );

            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[0].RouteItems[0]).UnderCons);
            Assert.IsTrue(((DocCardRouteItem)item.RoutePoints[0].RouteItems[0]).Passed);

            Assert.IsTrue(((DocCardRouteItem)item.RoutePoints[1].RouteItems[0]).UnderCons); // DocCircUser3
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[0]).Passed);
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[1]).UnderCons); // DocCircUser4
            Assert.IsTrue(((DocCardRouteItem)item.RoutePoints[1].RouteItems[1]).Passed);
            Assert.IsTrue(((DocCardRouteItem)item.RoutePoints[1].RouteItems[2]).UnderCons); // DocCircUser1
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[2]).Passed);
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[3]).UnderCons); // DocCircUser5
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[3]).Passed);

            // check events
            Assert.AreEqual(3, cnt.Events.Count);

                //Pass decision of DocCircUser2
            EventItem ei = cnt.Events[0];

            Assert.AreEqual(item.Id, ei.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.EventType.PassDecision, ei.EventType);
            Assert.AreEqual(FixtureConst.DocCircUser2, ei.EventUser);
            Assert.IsNull(ei.Deputy);
            Assert.AreEqual(1, ei.RoutePointNum);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ei.EventTime &&
                ei.EventTime < DateTime.Now.AddSeconds(2)
                );
            Assert.AreEqual("Hello guys!", ei.Note);

                //Move to the second route point
            ei = cnt.Events[1];

            Assert.AreEqual(item.Id, ei.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.EventType.PassedToNextRoutePoint, ei.EventType);
            Assert.AreEqual(FixtureConst.DocCircUser2, ei.EventUser);
            Assert.IsNull(ei.Deputy);
            Assert.AreEqual(2, ei.RoutePointNum);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ei.EventTime &&
                ei.EventTime < DateTime.Now.AddSeconds(2)
                );

                // DocCircUser4 automatic pass
            ei = cnt.Events[2];
            Assert.AreEqual(item.Id, ei.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.EventType.PassDecisionAutomatically, ei.EventType); 
            Assert.AreEqual(FixtureConst.DocCircUser4, ei.EventUser);
            Assert.IsNull(ei.Deputy);
            Assert.AreEqual(2, ei.RoutePointNum);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ei.EventTime &&
                ei.EventTime < DateTime.Now.AddSeconds(2)
                );

            // check notifications
            
            Assert.AreEqual(4, cnt.Notifications.Count);

                //DocCircUser3 decision
            NotificationItem ni = cnt.Notifications[0];

            Assert.AreEqual(item.Id, ni.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.NotificationType.NextDecision, ni.NotificationType);
            Assert.AreEqual(FixtureConst.DocCircUser3, ni.User);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ni.NotificationTime &&
                ni.NotificationTime < DateTime.Now.AddSeconds(2)
                );
            Assert.AreEqual("Hello guys!", ni.UserNote);
            Assert.AreEqual(FixtureConst.DocCircUser2, ni.LastActionUser);

                //DocCircUser1 decision
            ni = cnt.Notifications[1];

            Assert.AreEqual(item.Id, ni.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.NotificationType.NextDecision, ni.NotificationType);
            Assert.AreEqual(FixtureConst.DocCircUser1, ni.User);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ni.NotificationTime &&
                ni.NotificationTime < DateTime.Now.AddSeconds(2)
                );
            Assert.AreEqual("Hello guys!", ni.UserNote);
            Assert.AreEqual(FixtureConst.DocCircUser2, ni.LastActionUser);
                
                //DocCircUser1 deputy decision
            ni = cnt.Notifications[2];

            Assert.AreEqual(FixtureConst.DocCircUser2, ni.User);
            Assert.AreEqual(DBConstKeeper.Current.NotificationType.DeputyNextDecision, ni.NotificationType);
            Assert.AreEqual(item.Id, ni.DocCardId);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ni.NotificationTime &&
                ni.NotificationTime < DateTime.Now.AddSeconds(2)
                );
            Assert.AreEqual("Hello guys!", ni.UserNote);
            Assert.AreEqual(FixtureConst.DocCircUser2, ni.LastActionUser);

                //DocCircUser5 decision
            ni = cnt.Notifications[3];

            Assert.AreEqual(item.Id, ni.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.NotificationType.NextNotification, ni.NotificationType);
            Assert.AreEqual(FixtureConst.DocCircUser5, ni.User);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ni.NotificationTime &&
                ni.NotificationTime < DateTime.Now.AddSeconds(2)
                );
            Assert.AreEqual("Hello guys!", ni.UserNote);
            Assert.AreEqual(FixtureConst.DocCircUser2, ni.LastActionUser);
            

             /* Case 2
             * DocCircUser2 passes the document card on behalf of DocCircUser1
             * 
             * */

             /*
             *  Route:
             * 
             * 1 DocCircUser2(Decision)
             * 
             * 2 DocCircUser3(Decision) AND DocCircUser4(Decision, AutomaticPass) AND 
             * DocCircUser1(Decision, Deputy:DocCircUser2 Notif, AllowDeputyNotif); DocCircUser5(Notification)
             * [UnderCons]
             * 
             * 3 DocCircUser5(Decision, Deputy:DocCircUser3 NOT Notif, AllowDeputyNotif) 
             * AND DocCircUser1(Decision)
             * 
             * 4 DocCircUser1(Notification)
             * */

           
            cnt = new TestDocCardMovementController(
                new MockUserInfo(FixtureConst.DocCircUser2),
                null
                );

            cnt.Pass(item, FixtureConst.DocCircUser1, "passed it"); //tested method

            Assert.AreEqual(DBConstKeeper.Current.DocStatus.UnderCons, item.DocStatus);

            Assert.AreEqual(2, item.RoutePointNum);

            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[0].RouteItems[0]).UnderCons);
            Assert.IsTrue(((DocCardRouteItem)item.RoutePoints[0].RouteItems[0]).Passed);

            Assert.IsTrue(((DocCardRouteItem)item.RoutePoints[1].RouteItems[0]).UnderCons); // DocCircUser3
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[0]).Passed);

            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[1]).UnderCons); // DocCircUser4
            Assert.IsTrue(((DocCardRouteItem)item.RoutePoints[1].RouteItems[1]).Passed);

            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[2]).UnderCons); // DocCircUser1
            Assert.IsTrue(((DocCardRouteItem)item.RoutePoints[1].RouteItems[2]).Passed);

            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[3]).UnderCons); // DocCircUser5
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[3]).Passed);

            // check events
            Assert.AreEqual(1, cnt.Events.Count);

                //Pass decision of DocCircUser2 (on behalf of DocCircUser1)
            ei = cnt.Events[0];

            Assert.AreEqual(item.Id, ei.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.EventType.PassDecisionByDeputy, ei.EventType);
            Assert.AreEqual(FixtureConst.DocCircUser1, ei.EventUser);
            Assert.AreEqual(FixtureConst.DocCircUser2, ei.Deputy);
            Assert.AreEqual(2, ei.RoutePointNum);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ei.EventTime &&
                ei.EventTime < DateTime.Now.AddSeconds(2)
                );
            Assert.AreEqual("passed it", ei.Note);

            // check notifications
            Assert.AreEqual(0, cnt.Notifications.Count);


            /* Case 3
             * DocCircUser3 passes the doc card
             * he adds note: "OK"
             * And DocCirc2 moves to the third route point
             * 
             * DocCircUser1 disables AllowDeputyNotification
            */

            /*
             *  Route:
             * 
             * 1 DocCircUser2(Decision)
             * 
             * 2 DocCircUser3(Decision) AND DocCircUser4(Decision, AutomaticPass) AND 
             * DocCircUser1(Decision, Deputy:DocCircUser2 Notif, AllowDeputyNotif); DocCircUser5(Notification)
             * [UnderCons]
             * 
             * 3 DocCircUser5(Decision, Deputy:DocCircUser3 NOT Notif, AllowDeputyNotif) 
             * AND DocCircUser1(Decision)
             * 
             * 4 DocCircUser1(Notification)
             * */

            FixtureData.DocCircUser1Prop.AllowDeputyNotification = false;

            cnt = new TestDocCardMovementController(
                new MockUserInfo(FixtureConst.DocCircUser3),
                null
                );

            cnt.Pass(item, FixtureConst.DocCircUser3, "OK"); //tested method

            Assert.AreEqual(
                DBConstKeeper.Current.DocStatus.UnderCons,
                item.DocStatus
                );

            Assert.AreEqual(
                3,
                item.RoutePointNum
                );

            // route point 2
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[0]).UnderCons); // DocCircUser3
            Assert.IsTrue(((DocCardRouteItem)item.RoutePoints[1].RouteItems[0]).Passed);

            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[1]).UnderCons); // DocCircUser4
            Assert.IsTrue(((DocCardRouteItem)item.RoutePoints[1].RouteItems[1]).Passed);

            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[2]).UnderCons); // DocCircUser1
            Assert.IsTrue(((DocCardRouteItem)item.RoutePoints[1].RouteItems[2]).Passed);

            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[3]).UnderCons); // DocCircUser5
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[3]).Passed);

            // route point 3
            Assert.IsTrue(((DocCardRouteItem)item.RoutePoints[2].RouteItems[0]).UnderCons); // DocCircUser5
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[2].RouteItems[0]).Passed);

            Assert.IsTrue(((DocCardRouteItem)item.RoutePoints[2].RouteItems[1]).UnderCons); // DocCircUser1
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[2].RouteItems[1]).Passed);

            // check events
            Assert.AreEqual(2, cnt.Events.Count);

                //Pass decision of DocCircUser3
            ei = cnt.Events[0];

            Assert.AreEqual(item.Id, ei.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.EventType.PassDecision, ei.EventType);
            Assert.AreEqual(FixtureConst.DocCircUser3, ei.EventUser);
            Assert.IsNull(ei.Deputy);
            Assert.AreEqual(2, ei.RoutePointNum);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ei.EventTime &&
                ei.EventTime < DateTime.Now.AddSeconds(2)
                );
            Assert.AreEqual("OK", ei.Note);

                //Move to the third route point
             ei = cnt.Events[1];

            Assert.AreEqual(item.Id, ei.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.EventType.PassedToNextRoutePoint, ei.EventType);
            Assert.AreEqual(FixtureConst.DocCircUser3, ei.EventUser);
            Assert.IsNull(ei.Deputy);
            Assert.AreEqual(3, ei.RoutePointNum);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ei.EventTime &&
                ei.EventTime < DateTime.Now.AddSeconds(2)
                );

            // check notifications
            Assert.AreEqual(2, cnt.Notifications.Count);

              //DocCircUser5 decision
            ni = cnt.Notifications[0];

            Assert.AreEqual(item.Id, ni.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.NotificationType.NextDecision, ni.NotificationType);
            Assert.AreEqual(FixtureConst.DocCircUser5, ni.User);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ni.NotificationTime &&
                ni.NotificationTime < DateTime.Now.AddSeconds(2)
                );
            Assert.AreEqual("OK", ni.UserNote);
            Assert.AreEqual(FixtureConst.DocCircUser3, ni.LastActionUser);

              //DocCircUser1 decision
            ni = cnt.Notifications[1];

            Assert.AreEqual(item.Id, ni.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.NotificationType.NextDecision, ni.NotificationType);
            Assert.AreEqual(FixtureConst.DocCircUser1, ni.User);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ni.NotificationTime &&
                ni.NotificationTime < DateTime.Now.AddSeconds(2)
                );
            Assert.AreEqual("OK", ni.UserNote);
            Assert.AreEqual(FixtureConst.DocCircUser3, ni.LastActionUser);

            FixtureData.DocCircUser1Prop.AllowDeputyNotification = true;

            /* Case 4
             * Finalization
             * 
             * DocCircUser1 is last to pass the document card and then it will be finalized
             * He passes the DocCard
             * */

             /*
             *  Route:
             * 
             * 1 DocCircUser2(Decision)
             * 
             * 2 DocCircUser3(Decision) AND DocCircUser4(Decision, AutomaticPass) AND 
             * DocCircUser1(Decision, Deputy:DocCircUser2 Notif, AllowDeputyNotif); DocCircUser5(Notification)
             * 
             * 
             * 3 DocCircUser5(Decision, Deputy:DocCircUser3 NOT Notif, AllowDeputyNotif) 
             * AND DocCircUser1(Decision)
             * [UnderCons]
             * 
             * 4 DocCircUser1(Notification)
             *
             * */
            
            ((DocCardRouteItem)item.RoutePoints[2].RouteItems[0]).Passed = true; // DocCircUser5 passed
            ((DocCardRouteItem)item.RoutePoints[2].RouteItems[0]).UnderCons = false;

            cnt = new TestDocCardMovementController(
                new MockUserInfo(FixtureConst.DocCircUser1),
                null
                );

            cnt.Pass(item, FixtureConst.DocCircUser1, "OK1"); //tested method

            Assert.AreEqual(
                DBConstKeeper.Current.DocStatus.Finalized,
                item.DocStatus
                );

            Assert.AreEqual(
                0,
                item.RoutePointNum
                );

            // route point 3
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[2].RouteItems[1]).UnderCons); // DocCircUser1
            Assert.IsTrue(((DocCardRouteItem)item.RoutePoints[2].RouteItems[1]).Passed);

            //route point 4
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[3].RouteItems[0]).UnderCons); // DocCircUser1
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[3].RouteItems[0]).Passed);

             // check events
            Assert.AreEqual(3, cnt.Events.Count);

                //Pass decision of DocCircUser1
            ei = cnt.Events[0];

            Assert.AreEqual(item.Id, ei.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.EventType.PassDecision, ei.EventType);
            Assert.AreEqual(FixtureConst.DocCircUser1, ei.EventUser);
            Assert.IsNull(ei.Deputy);
            Assert.AreEqual(3, ei.RoutePointNum);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ei.EventTime &&
                ei.EventTime < DateTime.Now.AddSeconds(2)
                );
            Assert.AreEqual("OK1", ei.Note);

                //Move to the fourth route point
            ei = cnt.Events[1];

            Assert.AreEqual(item.Id, ei.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.EventType.PassedToNextRoutePoint, ei.EventType);
            Assert.AreEqual(FixtureConst.DocCircUser1, ei.EventUser);
            Assert.IsNull(ei.Deputy);
            Assert.AreEqual(4, ei.RoutePointNum);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ei.EventTime &&
                ei.EventTime < DateTime.Now.AddSeconds(2)
                );

                //DocCard is finalized
            ei = cnt.Events[2];

            Assert.AreEqual(item.Id, ei.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.EventType.DocCardFinalization, ei.EventType);
            Assert.AreEqual(FixtureConst.DocCircUser1, ei.EventUser);
            Assert.IsNull(ei.Deputy);
            Assert.AreEqual(0, ei.RoutePointNum);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ei.EventTime &&
                ei.EventTime < DateTime.Now.AddSeconds(2)
                );

            // check notifications
            Assert.AreEqual(2, cnt.Notifications.Count);

              //DocCircUser1 just notification
            ni = cnt.Notifications[0];

            Assert.AreEqual(item.Id, ni.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.NotificationType.NextNotification, ni.NotificationType);
            Assert.AreEqual(FixtureConst.DocCircUser1, ni.User);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ni.NotificationTime &&
                ni.NotificationTime < DateTime.Now.AddSeconds(2)
                );
            Assert.AreEqual("OK1", ni.UserNote);
            Assert.AreEqual(FixtureConst.DocCircUser1, ni.LastActionUser);

                //Finalization. Registering user is DocCircUser1
            ni = cnt.Notifications[1];

            Assert.AreEqual(item.Id, ni.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.NotificationType.Finalization, ni.NotificationType);
            Assert.AreEqual(FixtureConst.DocCircUser1, ni.User);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ni.NotificationTime &&
                ni.NotificationTime < DateTime.Now.AddSeconds(2)
                );
            Assert.IsEmpty(ni.UserNote);
            Assert.IsNull(ni.LastActionUser);
        }

        [Test]
        public void Reject()
        {
            bool asAdmin = true;

            /*  Case 1
             * 
             * Return to one of the previous route points by DocCircUser5 himself
             * 
             * 
             * */

            /*
            *  Route:
            * 
            * 1 DocCircUser2(Decision)
            * 
            * 2 DocCircUser3(Decision) AND DocCircUser4(Decision, AutomaticPass) AND 
            * DocCircUser1(Decision, Deputy:DocCircUser2 Notif, AllowDeputyNotif); DocCircUser5(Notification)
            * 
            * 3 DocCircUser5(Decision, Deputy:DocCircUser3 NOT Notif, AllowDeputyNotif) 
            * AND DocCircUser1(Decision)
            * [UnderCons] 
            *
            * 4 DocCircUser1(Notification)
            *
            * */

            DocCardItem srcItem = new DocCardItem(FixtureData.DocCardItem2);
            srcItem.RoutePointNum = 3;

            // route point 1
            ((DocCardRouteItem)srcItem.RoutePoints[0].RouteItems[0]).Passed = true; // DocCircUser2
            ((DocCardRouteItem)srcItem.RoutePoints[0].RouteItems[0]).UnderCons = false;

            // route point 2
            ((DocCardRouteItem)srcItem.RoutePoints[1].RouteItems[0]).Passed = true; // DocCircUser3
            ((DocCardRouteItem)srcItem.RoutePoints[1].RouteItems[0]).UnderCons = false;
            ((DocCardRouteItem)srcItem.RoutePoints[1].RouteItems[1]).Passed = true; // DocCircUser4
            ((DocCardRouteItem)srcItem.RoutePoints[1].RouteItems[1]).UnderCons = false;
            ((DocCardRouteItem)srcItem.RoutePoints[1].RouteItems[2]).Passed = true; // DocCircUser1
            ((DocCardRouteItem)srcItem.RoutePoints[1].RouteItems[2]).UnderCons = false;
            ((DocCardRouteItem)srcItem.RoutePoints[1].RouteItems[3]).Passed = false; // DocCircUser5
            ((DocCardRouteItem)srcItem.RoutePoints[1].RouteItems[3]).UnderCons = false;

            // route point 3
            ((DocCardRouteItem)srcItem.RoutePoints[2].RouteItems[0]).Passed = false; // DocCircUser5
            ((DocCardRouteItem)srcItem.RoutePoints[2].RouteItems[0]).UnderCons = true;
            ((DocCardRouteItem)srcItem.RoutePoints[2].RouteItems[1]).Passed = false; // DocCircUser1
            ((DocCardRouteItem)srcItem.RoutePoints[2].RouteItems[1]).UnderCons = true;

            DocCardItem item = new DocCardItem(srcItem);

            TestDocCardMovementController cnt = new TestDocCardMovementController(
                new MockUserInfo(FixtureConst.DocCircUser5),
                null
                );
            int routePointToReturn = 1;
            cnt.Reject(item, FixtureConst.DocCircUser5, routePointToReturn, "Return to 1 by myself", !asAdmin); // tested method
            Assert.IsNotNull(item.LastMovement);

            Assert.AreEqual(
               DBConstKeeper.Current.DocStatus.UnderCons,
               item.DocStatus
               );

            Assert.AreEqual(
                1,
                item.RoutePointNum
                );

            // route point 1
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[0].RouteItems[0]).Passed); // DocCircUser2
            Assert.IsTrue(((DocCardRouteItem)item.RoutePoints[0].RouteItems[0]).UnderCons);
            
            // route point 2
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[0]).Passed); // DocCircUser3
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[0]).UnderCons);
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[1]).Passed); // DocCircUser4
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[1]).UnderCons);
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[2]).Passed); // DocCircUser1
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[2]).UnderCons);
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[3]).Passed); // DocCircUser5
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[3]).UnderCons);

            // route point 3
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[2].RouteItems[0]).Passed); // DocCircUser5
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[2].RouteItems[0]).UnderCons);
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[2].RouteItems[1]).Passed); // DocCircUser1
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[2].RouteItems[1]).UnderCons);

            // check events
            Assert.AreEqual(2, cnt.Events.Count);

            //Pass decision of DocCircUser1
                // return to route point 1 by DocCircUser5
            EventItem ei = cnt.Events[0];

            Assert.AreEqual(item.Id, ei.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.EventType.RejectDecision, ei.EventType);
            Assert.AreEqual(FixtureConst.DocCircUser5, ei.EventUser);
            Assert.IsNull(ei.Deputy);
            Assert.AreEqual(1, ei.RoutePointNum);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ei.EventTime &&
                ei.EventTime < DateTime.Now.AddSeconds(2)
                );
            Assert.AreEqual("Return to 1 by myself", ei.Note);

                // next route point is 1
            ei = cnt.Events[1];

            Assert.AreEqual(item.Id, ei.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.EventType.PassedToNextRoutePoint, ei.EventType);
            Assert.AreEqual(FixtureConst.DocCircUser5, ei.EventUser);
            Assert.IsNull(ei.Deputy);
            Assert.AreEqual(1, ei.RoutePointNum);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ei.EventTime &&
                ei.EventTime < DateTime.Now.AddSeconds(2)
                );
            Assert.IsEmpty(ei.Note);

            // check notifications
            Assert.AreEqual(1, cnt.Notifications.Count);

                //DocCircUser2
            NotificationItem ni = cnt.Notifications[0];

            Assert.AreEqual(item.Id, ni.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.NotificationType.DecisionAfterReturn, ni.NotificationType);
            Assert.AreEqual(FixtureConst.DocCircUser2, ni.User);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ni.NotificationTime &&
                ni.NotificationTime < DateTime.Now.AddSeconds(2)
                );
            Assert.AreEqual("Return to 1 by myself", ni.UserNote);
            Assert.AreEqual(FixtureConst.DocCircUser5, ni.LastActionUser);


            /*  Case 2
            * 
            * Return to the Draft state by DocCircUser5 himself
            * 
            * 
            * */

            /*
            *  Route:
            * 
            * 1 DocCircUser2(Decision)
            * 
            * 2 DocCircUser3(Decision) AND DocCircUser4(Decision, AutomaticPass) AND 
            * DocCircUser1(Decision, Deputy:DocCircUser2 Notif, AllowDeputyNotif); DocCircUser5(Notification)
            * 
            * 3 DocCircUser5(Decision, Deputy:DocCircUser3 NOT Notif, AllowDeputyNotif) 
            * AND DocCircUser1(Decision)
            * [UnderCons] 
            *
            * 4 DocCircUser1(Notification)
            *
            * */
            item = new DocCardItem(srcItem);

            cnt = new TestDocCardMovementController(
                new MockUserInfo(FixtureConst.DocCircUser5),
                null
                );

            routePointToReturn = 0; // to draft
            cnt.Reject(item, FixtureConst.DocCircUser5, routePointToReturn, "Return to draft by myself", !asAdmin); // tested method

            Assert.AreEqual(
               DBConstKeeper.Current.DocStatus.Draft,
               item.DocStatus
               );

            Assert.AreEqual(
                0,
                item.RoutePointNum
                );

            // route point 1
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[0].RouteItems[0]).Passed); // DocCircUser2
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[0].RouteItems[0]).UnderCons);

            // route point 2
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[0]).Passed); // DocCircUser3
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[0]).UnderCons);
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[1]).Passed); // DocCircUser4
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[1]).UnderCons);
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[2]).Passed); // DocCircUser1
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[2]).UnderCons);
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[3]).Passed); // DocCircUser5
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[1].RouteItems[3]).UnderCons);

            // route point 3
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[2].RouteItems[0]).Passed); // DocCircUser5
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[2].RouteItems[0]).UnderCons);
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[2].RouteItems[1]).Passed); // DocCircUser1
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[2].RouteItems[1]).UnderCons);

            // check events
            Assert.AreEqual(1, cnt.Events.Count);

            // return to Draft by DocCircUser5
            ei = cnt.Events[0];

            Assert.AreEqual(item.Id, ei.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.EventType.RejectDecision, ei.EventType);
            Assert.AreEqual(FixtureConst.DocCircUser5, ei.EventUser);
            Assert.IsNull(ei.Deputy);
            Assert.AreEqual(0, ei.RoutePointNum);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ei.EventTime &&
                ei.EventTime < DateTime.Now.AddSeconds(2)
                );
            Assert.AreEqual("Return to draft by myself", ei.Note);

            // check notifications
            Assert.AreEqual(1, cnt.Notifications.Count);

            //DocCircUser2
            ni = cnt.Notifications[0];

            Assert.AreEqual(item.Id, ni.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.NotificationType.ReturnToDraftDueToDecision, ni.NotificationType);
            Assert.AreEqual(FixtureConst.DocCircUser1, ni.User);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ni.NotificationTime &&
                ni.NotificationTime < DateTime.Now.AddSeconds(2)
                );
            Assert.AreEqual("Return to draft by myself", ni.UserNote);
            Assert.AreEqual(FixtureConst.DocCircUser5, ni.LastActionUser);


            /*  Case 2
           * 
           * Return to the Draft state by DocCircUser3 on behalf of DocCircUser5
           * 
           * 
           * */

            /*
            *  Route:
            * 
            * 1 DocCircUser2(Decision)
            * 
            * 2 DocCircUser3(Decision) AND DocCircUser4(Decision, AutomaticPass) AND 
            * DocCircUser1(Decision, Deputy:DocCircUser2 Notif, AllowDeputyNotif); DocCircUser5(Notification)
            * 
            * 3 DocCircUser5(Decision, Deputy:DocCircUser3 NOT Notif, AllowDeputyNotif) 
            * AND DocCircUser1(Decision)
            * [UnderCons] 
            *
            * 4 DocCircUser1(Notification)
            *
            * */
            item = new DocCardItem(srcItem);

            cnt = new TestDocCardMovementController(
                new MockUserInfo(FixtureConst.DocCircUser3),
                null
                );

            routePointToReturn = 0; // to draft
            cnt.Reject(item, FixtureConst.DocCircUser5, routePointToReturn, "Return to draft by deputy", !asAdmin); // tested method

            Assert.AreEqual(
              DBConstKeeper.Current.DocStatus.Draft,
              item.DocStatus
              );

            Assert.AreEqual(
                0,
                item.RoutePointNum
                );

            // check events
            Assert.AreEqual(1, cnt.Events.Count);

                // return to Draft by DocCircUser3
            ei = cnt.Events[0];

            Assert.AreEqual(item.Id, ei.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.EventType.RejectDecisionByDeputy, ei.EventType);
            Assert.AreEqual(FixtureConst.DocCircUser5, ei.EventUser);
            Assert.AreEqual(FixtureConst.DocCircUser3, ei.Deputy);
            Assert.AreEqual(0, ei.RoutePointNum);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ei.EventTime &&
                ei.EventTime < DateTime.Now.AddSeconds(2)
                );
            Assert.AreEqual("Return to draft by deputy", ei.Note);

            // check notifications
            Assert.AreEqual(1, cnt.Notifications.Count);

                //DocCircUser2
            ni = cnt.Notifications[0];

            Assert.AreEqual(item.Id, ni.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.NotificationType.ReturnToDraftDueToDecision, ni.NotificationType);
            Assert.AreEqual(FixtureConst.DocCircUser1, ni.User);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ni.NotificationTime &&
                ni.NotificationTime < DateTime.Now.AddSeconds(2)
                );
            Assert.AreEqual("Return to draft by deputy", ni.UserNote);
            Assert.AreEqual(FixtureConst.DocCircUser3, ni.LastActionUser);


            /*  Case 3
          * 
          * Return to the Draft state by Administrator (DocCircUser4) on behalf of DocCircUser5
          * 
          * 
          * */

            /*
            *  Route:
            * 
            * 1 DocCircUser2(Decision)
            * 
            * 2 DocCircUser3(Decision) AND DocCircUser4(Decision, AutomaticPass) AND 
            * DocCircUser1(Decision, Deputy:DocCircUser2 Notif, AllowDeputyNotif); DocCircUser5(Notification)
            * 
            * 3 DocCircUser5(Decision, Deputy:DocCircUser3 NOT Notif, AllowDeputyNotif) 
            * AND DocCircUser1(Decision)
            * [UnderCons] 
            *
            * 4 DocCircUser1(Notification)
            *
            * */
            item = new DocCardItem(srcItem);

            bool isAdministrator = true;
            cnt = new TestDocCardMovementController(
                new MockUserInfo(FixtureConst.DocCircUser4, isAdministrator),
                null
                );

            routePointToReturn = 0; // to draft
            cnt.Reject(item, FixtureConst.DocCircUser5, routePointToReturn, "Return to draft by admin", asAdmin); // tested method

            Assert.AreEqual(
              DBConstKeeper.Current.DocStatus.Draft,
              item.DocStatus
              );

            Assert.AreEqual(
                0,
                item.RoutePointNum
                );

            // check events
            Assert.AreEqual(1, cnt.Events.Count);

            // return to Draft by DocCircUser4
            ei = cnt.Events[0];

            Assert.AreEqual(item.Id, ei.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.EventType.RejectDecisionByAdministrator, ei.EventType);
            Assert.AreEqual(FixtureConst.DocCircUser5, ei.EventUser);
            Assert.AreEqual(FixtureConst.DocCircUser4, ei.Deputy);
            Assert.AreEqual(0, ei.RoutePointNum);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ei.EventTime &&
                ei.EventTime < DateTime.Now.AddSeconds(2)
                );
            Assert.AreEqual("Return to draft by admin", ei.Note);

            // check notifications
            Assert.AreEqual(1, cnt.Notifications.Count);

                //DocCircUser2
            ni = cnt.Notifications[0];

            Assert.AreEqual(item.Id, ni.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.NotificationType.ReturnToDraftDueToDecision, ni.NotificationType);
            Assert.AreEqual(FixtureConst.DocCircUser1, ni.User);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ni.NotificationTime &&
                ni.NotificationTime < DateTime.Now.AddSeconds(2)
                );
            Assert.AreEqual("Return to draft by admin", ni.UserNote);
            Assert.AreEqual(FixtureConst.DocCircUser4, ni.LastActionUser);

        }

        [Test]
        public void RejectDueToRouteChange()
        {
            /*  Case 1 reject to the draft state
             *  
             *  Item: DocCard2
             *  Route point number under cons = 1 
             * 
             * 
             * 
             * */
            DocCardItem item = new DocCardItem(FixtureData.DocCardItem2);

            TestDocCardMovementController cnt = new TestDocCardMovementController(
                new MockUserInfo(FixtureConst.DocCircUser2),
                null
                );
            bool asAdmin = true, routeChange = true;
            cnt.Reject(item, FixtureConst.DocCircUser2, 0, "", asAdmin, routeChange); // tested method

            Assert.AreEqual(DBConstKeeper.Current.DocStatus.Draft, item.DocStatus);
            Assert.AreEqual(0, item.RoutePointNum);
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[0].RouteItems[0]).UnderCons);
            Assert.IsFalse(((DocCardRouteItem)item.RoutePoints[0].RouteItems[0]).Passed);

            // check events
            Assert.AreEqual(1, cnt.Events.Count);
            EventItem ei = cnt.Events[0];
            Assert.AreEqual(DBConstKeeper.Current.EventType.ReturnToDraftDueToRouteChange, ei.EventType);
            Assert.IsNull(ei.Deputy);
            Assert.AreEqual(item.Id, ei.DocCardId);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ei.EventTime &&
                ei.EventTime < DateTime.Now.AddSeconds(2)
                );
            Assert.AreEqual(FixtureConst.DocCircUser2, ei.EventUser);
            Assert.IsEmpty(ei.Note);
            Assert.AreEqual(0, ei.RoutePointNum);
            Assert.IsEmpty(ei.Text);

            // check notifications
            Assert.AreEqual(1, cnt.Notifications.Count);
            NotificationItem ni = cnt.Notifications[0];

            Assert.AreEqual(item.Id, ni.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.NotificationType.ReturnToDraftDueToRouteChange, ni.NotificationType);
            Assert.AreEqual(FixtureConst.DocCircUser1, ni.User);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ni.NotificationTime &&
                ni.NotificationTime < DateTime.Now.AddSeconds(2)
                );
            Assert.IsEmpty(ni.UserNote);
            Assert.AreEqual(FixtureConst.DocCircUser2, ni.LastActionUser);

        }

        [Test]
        public void Postpone()
        {
            /* Case 1 postpone by DocCircUser1
             * */

            DocCardItem item = new DocCardItem(FixtureData.DocCardItem2);

            TestDocCardMovementController cnt = new TestDocCardMovementController(
                new MockUserInfo(FixtureConst.DocCircUser2),
                null
                );

            bool asAdmin = true;
            cnt.Postpone(item, FixtureConst.DocCircUser2, "wait", !asAdmin); // tested method
            Assert.IsNotNull(item.LastMovement);
            Assert.AreEqual(
              DBConstKeeper.Current.DocStatus.UnderCons,
              item.DocStatus
              );

            Assert.AreEqual(
                1,
                item.RoutePointNum
                );

            // check events
            Assert.AreEqual(1, cnt.Events.Count);

                // postpone by DocCircUser2
            EventItem ei = cnt.Events[0];

            Assert.AreEqual(item.Id, ei.DocCardId);
            Assert.AreEqual(DBConstKeeper.Current.EventType.PostponeDecision, ei.EventType);
            Assert.AreEqual(FixtureConst.DocCircUser2, ei.EventUser);
            Assert.IsNull(ei.Deputy);
            Assert.AreEqual(1, ei.RoutePointNum);
            Assert.That(
                DateTime.Now.AddSeconds(-2) < ei.EventTime &&
                ei.EventTime < DateTime.Now.AddSeconds(2)
                );
            Assert.AreEqual("wait", ei.Note);

            Assert.AreEqual(0, cnt.Notifications.Count);
        }

        [Test]
        public void CheckReturnDueToRouteChange()
        {
            TestDocCardMovementController cnt = new TestDocCardMovementController(
                new MockUserInfo(FixtureConst.DocCircUser2),
                null
                );

            DocCardItem item = new DocCardItem(FixtureData.DocCardItem2);


            /* Case 1
             * 
             * route before
             * 
             * Route:
             * 1 DocCircUser2(Decision) [Under cons now]
             * 
             * 2 DocCircUser3(Decision) AND DocCircUser4(Decision) AND 
             * DocCircUser1(Decision); DocCircUser5(Notification)
             * 
             * 3 DocCircUser5(Decision) AND DocCircUser1(Decision)
             * 
             * 4 DocCircUser1(Notification)
             * 
             * new route
             * is the same
             * */

            RoutePoint[] newRoute = new RoutePoint[4];
            
            RoutePoint rp = new RoutePoint(0, "{0}", 1);
            rp.RouteItems.Add(
                new RouteItem(0, FixtureConst.DocCircUser2, DBConstKeeper.Current.RouteItemType.Decision)
                );
            newRoute[0] = rp;

            rp = new RoutePoint(1, "{0}&&{1}&&{2}", 2);

            rp.RouteItems.Add(
                new RouteItem(1, FixtureConst.DocCircUser3, DBConstKeeper.Current.RouteItemType.Decision)
                );
            rp.RouteItems.Add(
                new RouteItem(2, FixtureConst.DocCircUser4, DBConstKeeper.Current.RouteItemType.Decision)
                );
            rp.RouteItems.Add(
                new RouteItem(3, FixtureConst.DocCircUser1, DBConstKeeper.Current.RouteItemType.Decision)
                );
            rp.RouteItems.Add(
                new RouteItem(4, FixtureConst.DocCircUser5, DBConstKeeper.Current.RouteItemType.Notification)
                );

            newRoute[1] = rp;

            rp = new RoutePoint(2, "{0}&&{1}", 3);

            rp.RouteItems.Add(
                new RouteItem(5, FixtureConst.DocCircUser5, DBConstKeeper.Current.RouteItemType.Decision)
                );
            rp.RouteItems.Add(
                new RouteItem(6, FixtureConst.DocCircUser1, DBConstKeeper.Current.RouteItemType.Decision)
                );
            
            newRoute[2] = rp;

            rp = new RoutePoint(3, "", 4);

            rp.RouteItems.Add(
                new RouteItem(7, FixtureConst.DocCircUser1, DBConstKeeper.Current.RouteItemType.Notification)
                );

            newRoute[3] = rp;

            
            bool result = cnt.CheckReturnDueToRouteChange(item, newRoute); // tested method

            Assert.IsFalse(result);


            /* Case 2
             * 
             * new route
             * 
             * Route:
             * 1 DocCircUser1(Decision)
             * 
             * 2 DocCircUser3(Decision) AND DocCircUser4(Decision) AND 
             * DocCircUser1(Decision); DocCircUser5(Notification)
             * 
             * 3 DocCircUser5(Decision) AND DocCircUser1(Decision)
             * 
             * 4 DocCircUser1(Notification)
             * */

            newRoute[0].RouteItems[0] = 
                new RouteItem(null, FixtureConst.DocCircUser1, DBConstKeeper.Current.RouteItemType.Decision);


            result = cnt.CheckReturnDueToRouteChange(item, newRoute); // tested method

            Assert.IsTrue(result);

            newRoute[0].RouteItems[0] =
                new RouteItem(0, FixtureConst.DocCircUser2, DBConstKeeper.Current.RouteItemType.Decision);

            /* Case 3
             * 
             * new route
             * 
             * Route:
             * 1 DocCircUser2(Decision) AND DocCircUser3(Decision)
             * 
             * 2 DocCircUser3(Decision) AND DocCircUser4(Decision) AND 
             * DocCircUser1(Decision); DocCircUser5(Notification)
             * 
             * 3 DocCircUser5(Decision) AND DocCircUser1(Decision)
             * 
             * 4 DocCircUser1(Notification)
             * */

            newRoute[0].RouteItems.Add(
                new RouteItem(null, FixtureConst.DocCircUser3, DBConstKeeper.Current.RouteItemType.Decision)
                );

            result = cnt.CheckReturnDueToRouteChange(item, newRoute); // tested method

            Assert.IsTrue(result);

            /* Case 4
             * 
             * new route
             * 
             * Route:
             * 1 DocCircUser2(Decision) AND DocCircUser3(Notification)
             * 
             * 2 DocCircUser3(Decision) AND DocCircUser4(Decision) AND 
             * DocCircUser1(Decision); DocCircUser5(Notification)
             * 
             * 3 DocCircUser5(Decision) AND DocCircUser1(Decision)
             * 
             * 4 DocCircUser1(Notification)
             * */

            newRoute[0].RouteItems[1].RouteItemType = DBConstKeeper.Current.RouteItemType.Notification;

            result = cnt.CheckReturnDueToRouteChange(item, newRoute); // tested method

            Assert.IsFalse(result);



        }

    }

    
}
