//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
using Evpo.DocCirc.Biz;
using Evpo.DocCirc.Common;

namespace Evpo.DocCircUnitTest
{
    [TestFixture]
    public class DocCardRelationToUserResolverTest
    {
        protected struct FixtureConst
        {
            public const int DocCircUser1 = 0;
            public const int DocCircUser2 = 1;
            public const int DocCircUser3 = 2;

            public const long DocCard1 = 0;
            public const long DocCard2 = 1;
            public const long DocCard3 = 2;
            public const long DocCard4 = 3;

            public const int DocType1 = 0;
            public const int DocType2 = 1;
        
            public const int DocCardFolder1 = 0;
            public const int DocCardFolder2 = 1;
        }

        protected class FixtureData
        {
            public static UserInfo DocCircUser1UserInfo;
            public static UserInfo DocCircUser2UserInfo;
            public static UserInfo DocCircUser3UserInfo;

            public static SecurityItem DocCircUser1DocCard1SI;
            public static SecurityItem DocCircUser2DocCard1SI;
            public static SecurityItem DocCircUser3DocCard1SI;

            //public static SecurityItem DocCircUser1DocCardFolder1SI;
            //public static SecurityItem DocCircUser2DocCardFolder1SI;
            //public static SecurityItem DocCircUser3DocCardFolder1SI;
            //public static SecurityItem DocCircUser1DocCardFolder2SI;
            //public static SecurityItem DocCircUser1DocType1SI;
            //public static SecurityItem DocCircUser2DocType1SI;
            //public static SecurityItem DocCircUser3DocType1SI;

        }

        protected class MockUserHelperInstance : UserHelperInstance
        {
            public MockUserHelperInstance()
            {
            }

            public override DeputyItem[] GetWhomUserIsDeputyFor(int userId, DataConnection con)
            {
                DeputyItem[] retVal = null;
                if (userId == FixtureConst.DocCircUser1)
                {
                    retVal = new DeputyItem[]
                    {
                        new DeputyItem(0, FixtureConst.DocCircUser2, FixtureConst.DocCircUser1, true)
                    };
                }
                else if (userId == FixtureConst.DocCircUser2)
                {
                    retVal = new DeputyItem[0];
                }
                else if (userId == FixtureConst.DocCircUser3)
                {
                    retVal = new DeputyItem[0];
                }
                else
                {
                    throw new ApplicationException(String.Format("Unknown id={0} for this unit test!", userId));
                }
                return retVal;
            }
        }

        protected class MockUserInfo : UserInfo
        {
            protected bool _userIsAdmin;

            public MockUserInfo(int userId, bool userIsAdmin)
            {
                _userProperties = new UserProperties();
                _userProperties.UserId = userId;
                _userIsAdmin = userIsAdmin;
            }

            public override bool UserIsAdministrator()
            {
                return _userIsAdmin;
            }
        }

        protected class MockSecurityData : SecurityData
        {


            public MockSecurityData(UserInfo userInfo)
            {
                _userInfo = userInfo;
            }

            public override SecurityItem GetDocumentCardSecurity(DocCardItem item)
            {
                SecurityItem retVal = null;
                
                if (_userInfo.UserId == FixtureConst.DocCircUser1 && item.Id == FixtureConst.DocCard1)
                {
                    retVal = FixtureData.DocCircUser1DocCard1SI;
                }
                else if (_userInfo.UserId == FixtureConst.DocCircUser2 && item.Id == FixtureConst.DocCard1)
                {
                    retVal = FixtureData.DocCircUser2DocCard1SI;
                }
                else if (_userInfo.UserId == FixtureConst.DocCircUser3 && item.Id == FixtureConst.DocCard1)
                {
                    retVal = FixtureData.DocCircUser3DocCard1SI;
                }
                else
                {
                    throw new ApplicationException("Unknown id for this unit test!");
                }

                return retVal;

            }

        }

        protected class MockDataConnection : DataConnection
        {
            public MockDataConnection()
            {
                _con = null;
            }
        }

        protected class TestDocCardRelationToUserResolver : DocCardRelationToUserResolver
        {

            public TestDocCardRelationToUserResolver(UserInfo userInfo, DataConnection con)
                : base(userInfo, con)
            {
            }

            protected override SecurityData GetSecurityData(UserInfo userInfo, DataConnection con)
            {
                return new MockSecurityData(userInfo);
            }

        }

        protected DataConnection _con;

        [TestFixtureSetUp]
        public void TFSetUp()
        {
            DBConstKeeper constKeeper = new DBConstKeeper();
            constKeeper.DocStatus = new DocStatusDirectory(0, 1, 2, 3);
            constKeeper.RouteItemType = new RouteItemTypeDirectory(0, 1);
            DBConstKeeper.Current = constKeeper;

            UserHelper.Current = new MockUserHelperInstance();

            bool userIsAdmin = true;
            FixtureData.DocCircUser1UserInfo = new MockUserInfo(FixtureConst.DocCircUser1, !userIsAdmin);
            FixtureData.DocCircUser2UserInfo = new MockUserInfo(FixtureConst.DocCircUser2, !userIsAdmin);
            FixtureData.DocCircUser3UserInfo = new MockUserInfo(FixtureConst.DocCircUser3, userIsAdmin);

            // DocCircUser1 for DocCard1
            SecurityItem si = new SecurityItem();
            si.Read = true;
            si.Update = true;
            si.Delete = true;
            
            FixtureData.DocCircUser1DocCard1SI = si;

            // DocCircUser2 for DocCard1
            si = new SecurityItem();

            si.Read = true;
            si.Update = false;
            si.Delete = false;

            FixtureData.DocCircUser2DocCard1SI = si;

            // DocCircUser3 for DocCard1
            si = new SecurityItem();

            si.Read = true;
            si.Update = true;
            si.Delete = false;

            FixtureData.DocCircUser3DocCard1SI = si;

            _con = new MockDataConnection();
        }



        [Test]
        public void GetDocCardRelationToUser()
        {
            /* case 1
             * DocCard1 
             * new document card in DocCardFolder1 card by DocCircUser1
             * */
            DocCardItem item = new DocCardItem();
            item.Id = FixtureConst.DocCard1;
            item.Owner = FixtureConst.DocCircUser1;
            item.DocType = null;
            item.Folder = FixtureConst.DocCardFolder1;
            item.DocStatus = DBConstKeeper.Current.DocStatus.NotSaved;
            item.IsNew = true;
            item.RouteIsLoaded = false;
            item.RoutePointNum = 0;
            item.AttachmentsAreChangeable = true;

            TestDocCardRelationToUserResolver stateResolver =
                new TestDocCardRelationToUserResolver(FixtureData.DocCircUser1UserInfo, _con);
            DocCardRelationToUser state =
                stateResolver.GetDocCardRelationToUser(item); // tested method

            Assert.IsNotNull(state);
            Assert.IsFalse(state.AttachmentIsReadOnly);
            Assert.IsFalse(state.DocumentTabIsReadOnly);
            Assert.IsFalse(state.RouteTabIsVisible);
            Assert.IsTrue(state.RouteTabIsReadOnly);
            Assert.IsTrue(state.CanBeSaved);
            Assert.IsFalse(state.RemovePassRouteDecisionOption);
            Assert.IsFalse(state.AttachmentReadOnlyCheckBoxIsReadOnly);

            
            /* case 2
             * 
             * DocCard1
             * 
             * DocStatus was changed to draft
             * DocCircUser1 can update the doc card
             * 
             * */
            item.DocStatus = DBConstKeeper.Current.DocStatus.Draft;
            item.IsNew = false;
            item.DocType = FixtureConst.DocType1;
            state = stateResolver.GetDocCardRelationToUser(item); // tested method

            Assert.IsNotNull(state);
            Assert.IsFalse(state.AttachmentIsReadOnly);
            Assert.IsFalse(state.DocumentTabIsReadOnly);
            Assert.IsFalse(state.RouteTabIsVisible);
            Assert.IsTrue(state.RouteTabIsReadOnly);
            Assert.IsTrue(state.CanBeSaved);
            Assert.IsFalse(state.RemovePassRouteDecisionOption);
            Assert.IsFalse(state.AttachmentReadOnlyCheckBoxIsReadOnly);
            Assert.IsNull(state.UserWhoseDeputyCurrentUserIs);

            /* case 3
             * DocCard1
             * 
             * DocCircUser2 doesn't have update permission in folder and doc type
             * */

            stateResolver =
                new TestDocCardRelationToUserResolver(FixtureData.DocCircUser2UserInfo, _con);
            state = stateResolver.GetDocCardRelationToUser(item); //tested method
             
            Assert.IsNotNull(state);
            Assert.IsTrue(state.AttachmentIsReadOnly);
            Assert.IsTrue(state.DocumentTabIsReadOnly);
            Assert.IsFalse(state.RouteTabIsVisible);
            Assert.IsTrue(state.RouteTabIsReadOnly);
            Assert.IsFalse(state.CanBeSaved);
            Assert.IsFalse(state.RemovePassRouteDecisionOption);
            Assert.IsTrue(state.AttachmentReadOnlyCheckBoxIsReadOnly);
            Assert.IsNull(state.UserWhoseDeputyCurrentUserIs);

            /* case 4
             * DocCard1
             * 
             * The card changed status to Under consideration
             * First route point: DocCircUser1 AND DocCircUser3
             * Second route point: DocCircUser2 (DocCircUser3 - Notification only)
             * 
             * DocCircUser2 can't access this doccard, because he doesn't have permissions
             * */
            item.DocStatus = DBConstKeeper.Current.DocStatus.UnderCons;
            item.RoutePointNum = 1;
            item.AttachmentsAreChangeable = true;
            item.RoutePoints = new RoutePoint[]
            {
                new RoutePoint(0, "{0}&&{1}", 0),
                new RoutePoint(1, "{0}", 1)
            };

            // route point 1
            item.RoutePoints[0].RouteItems.Add(
                new DocCardRouteItem(
                    0, 
                    false, 
                    true,
                    new RouteItem(0, FixtureConst.DocCircUser1, DBConstKeeper.Current.RouteItemType.Decision)
                    )
                );
            
            item.RoutePoints[0].RouteItems.Add(
                new DocCardRouteItem(
                    1, 
                    false, 
                    true,
                    new RouteItem(1, FixtureConst.DocCircUser3, DBConstKeeper.Current.RouteItemType.Decision)
                    )
                );

            // route point 2
            item.RoutePoints[1].RouteItems.Add(
                new DocCardRouteItem(
                    0, 
                    false, 
                    false,
                    new RouteItem(0, FixtureConst.DocCircUser2, DBConstKeeper.Current.RouteItemType.Decision)
                    )
                );
            item.RoutePoints[1].RouteItems.Add(
                new DocCardRouteItem(
                    1, 
                    false, 
                    false,
                    new RouteItem(1, FixtureConst.DocCircUser3, DBConstKeeper.Current.RouteItemType.Notification)
                    )
                );

            state = stateResolver.GetDocCardRelationToUser(item); //tested method

            Assert.IsNotNull(state);
            Assert.IsTrue(state.AttachmentIsReadOnly);
            Assert.IsTrue(state.DocumentTabIsReadOnly);
            Assert.IsTrue(state.RouteTabIsVisible);
            Assert.IsTrue(state.RouteTabIsReadOnly);
            Assert.IsFalse(state.CanBeSaved);
            Assert.IsFalse(state.RemovePassRouteDecisionOption);
            Assert.IsTrue(state.AttachmentReadOnlyCheckBoxIsReadOnly);
            Assert.IsNull(state.UserWhoseDeputyCurrentUserIs);

            /* case 5
             * DocCircUser1 can make a decision, 
             * because he is on under consideration point and haven't made a decision before
             * */
            stateResolver =
                new TestDocCardRelationToUserResolver(FixtureData.DocCircUser1UserInfo, _con);

            state = stateResolver.GetDocCardRelationToUser(item); //tested method

            Assert.IsNotNull(state);
            Assert.IsFalse(state.AttachmentIsReadOnly);
            Assert.IsTrue(state.DocumentTabIsReadOnly);
            Assert.IsTrue(state.RouteTabIsVisible);
            Assert.IsFalse(state.RouteTabIsReadOnly);
            Assert.IsTrue(state.CanBeSaved);
            Assert.IsFalse(state.RemovePassRouteDecisionOption);
            Assert.IsFalse(state.AttachmentReadOnlyCheckBoxIsReadOnly);
            Assert.IsNull(state.UserWhoseDeputyCurrentUserIs);

            /* case 6
             * DocCircUser1 made a decision and passed the document card
             * After previous action he opens the document card and can't change anything
             * */
            ((DocCardRouteItem)item.RoutePoints[0].RouteItems[0]).UnderCons = false;
            ((DocCardRouteItem)item.RoutePoints[0].RouteItems[0]).Passed = true; // DocCircUser1 made pass decision
            
            item.AttachmentsAreChangeable = false;

            state = stateResolver.GetDocCardRelationToUser(item); // tested method
            
            Assert.IsNotNull(state);
            Assert.IsTrue(state.AttachmentIsReadOnly);
            Assert.IsTrue(state.DocumentTabIsReadOnly);
            Assert.IsTrue(state.RouteTabIsVisible);
            Assert.IsTrue(state.RouteTabIsReadOnly);
            Assert.IsFalse(state.CanBeSaved);
            Assert.IsFalse(state.RemovePassRouteDecisionOption);
            Assert.IsTrue(state.AttachmentReadOnlyCheckBoxIsReadOnly);
            Assert.IsNull(state.UserWhoseDeputyCurrentUserIs);

            /* case 7
             * DocCircUser3 is on the second route item of the first route point 
             * and he haven't made a decsion yet
             * 
             * */
            stateResolver =
                new TestDocCardRelationToUserResolver(FixtureData.DocCircUser3UserInfo, _con);

            state = stateResolver.GetDocCardRelationToUser(item);

            Assert.IsNotNull(state);
            Assert.IsTrue(state.AttachmentIsReadOnly);
            Assert.IsTrue(state.DocumentTabIsReadOnly);
            Assert.IsTrue(state.RouteTabIsVisible);
            Assert.IsFalse(state.RouteTabIsReadOnly);
            Assert.IsTrue(state.CanBeSaved);
            Assert.IsFalse(state.RemovePassRouteDecisionOption);
            Assert.IsTrue(state.AttachmentReadOnlyCheckBoxIsReadOnly);
            Assert.IsNull(state.UserWhoseDeputyCurrentUserIs);

            /* case 8
             * DocCircUser3 made the pass decision and docCard moved to the next route point
             * 
             * DocCircUser3 is an administrator of the system and he can make a decision for 
             * DocCircUser2, but he can only return the document card back on the route.
             * */
            ((DocCardRouteItem)item.RoutePoints[0].RouteItems[1]).UnderCons = false;
            ((DocCardRouteItem)item.RoutePoints[0].RouteItems[1]).Passed = true; 
            item.RoutePointNum = 2;
            ((DocCardRouteItem)item.RoutePoints[1].RouteItems[0]).UnderCons = true;

            state = stateResolver.GetDocCardRelationToUser(item); // tested method

            Assert.IsNotNull(state);
            Assert.IsTrue(state.AttachmentIsReadOnly);
            Assert.IsTrue(state.DocumentTabIsReadOnly);
            Assert.IsTrue(state.RouteTabIsVisible);
            Assert.IsFalse(state.RouteTabIsReadOnly);
            Assert.IsTrue(state.CanBeSaved);
            Assert.IsTrue(state.RemovePassRouteDecisionOption);
            Assert.IsTrue(state.AttachmentReadOnlyCheckBoxIsReadOnly);
            Assert.AreEqual(FixtureConst.DocCircUser2, state.UserWhoseDeputyCurrentUserIs);

            /* case 9
             * DocCircUser2 is on the route and Under consideration, 
             * but he doesn't have update permission in docType1 and DocCircFolder1
             * */
            stateResolver =
                new TestDocCardRelationToUserResolver(FixtureData.DocCircUser2UserInfo, _con);
            state = stateResolver.GetDocCardRelationToUser(item);
            
            Assert.IsNotNull(state);
            Assert.IsTrue(state.AttachmentIsReadOnly);
            Assert.IsTrue(state.DocumentTabIsReadOnly);
            Assert.IsTrue(state.RouteTabIsVisible);
            Assert.IsTrue(state.RouteTabIsReadOnly);
            Assert.IsFalse(state.CanBeSaved);
            Assert.IsFalse(state.RemovePassRouteDecisionOption);
            Assert.IsTrue(state.AttachmentReadOnlyCheckBoxIsReadOnly);
            Assert.IsNull(state.UserWhoseDeputyCurrentUserIs);

            /* case 10
             * DocCircUser1 is a deputy for DocCircUser2 and can make a decision for him
             * */
            stateResolver =
               new TestDocCardRelationToUserResolver(FixtureData.DocCircUser1UserInfo, _con);
            state = stateResolver.GetDocCardRelationToUser(item);

            Assert.IsNotNull(state);
            Assert.IsTrue(state.AttachmentIsReadOnly);
            Assert.IsTrue(state.DocumentTabIsReadOnly);
            Assert.IsTrue(state.RouteTabIsVisible);
            Assert.IsFalse(state.RouteTabIsReadOnly);
            Assert.IsTrue(state.CanBeSaved);
            Assert.IsFalse(state.RemovePassRouteDecisionOption);
            Assert.IsTrue(state.AttachmentReadOnlyCheckBoxIsReadOnly);
            Assert.AreEqual(FixtureConst.DocCircUser2, state.UserWhoseDeputyCurrentUserIs);

            /* case 11
             * DocCard1 is finalized. Everybody can see it, but can't change anything on it
             * DocCircUser3 opens it
             * */
            item.RoutePointNum = 0;
            item.DocStatus = DBConstKeeper.Current.DocStatus.Finalized;
            ((DocCardRouteItem)item.RoutePoints[1].RouteItems[1]).UnderCons = false;
            ((DocCardRouteItem)item.RoutePoints[1].RouteItems[1]).Passed = true;

            stateResolver =
                new TestDocCardRelationToUserResolver(FixtureData.DocCircUser3UserInfo, _con);
            
            state = stateResolver.GetDocCardRelationToUser(item); // tested method

            Assert.IsNotNull(state);
            Assert.IsTrue(state.AttachmentIsReadOnly);
            Assert.IsTrue(state.DocumentTabIsReadOnly);
            Assert.IsTrue(state.RouteTabIsVisible);
            Assert.IsTrue(state.RouteTabIsReadOnly);
            Assert.IsFalse(state.CanBeSaved);
            Assert.IsFalse(state.RemovePassRouteDecisionOption);
            Assert.IsTrue(state.AttachmentReadOnlyCheckBoxIsReadOnly);
            Assert.IsNull(state.UserWhoseDeputyCurrentUserIs);

        }



    }
}
