//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
using Evpo.DocCirc.MySqlDL;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.Biz;

namespace Evpo.DocCircUnitTest
{
    [TestFixture]
    public class DocTypeDataTest
    {
        protected struct FixtureConst
        {
            public const int DocType1 = 0;
            public const int DocCircService = 10;
            public const string DocType1Prefix = "TEST";
            public const int DocType1InitialNextNumber = 1;
        }

        protected class FixtureData
        {

            public static DocTypeItem DocTypeItem1;
        }

        protected class MockDocTypeAccess : DocTypeAccess
        {
            public MockDocTypeAccess()
            {
            }

            private int _savedNextNumber;

            public int SavedNextNumber
            {
                get { return _savedNextNumber; }
                set { _savedNextNumber = value; }
            }


            public override DocTypeItem LoadDocTypeItem(int id, bool loadRoutePoints)
            {
                if (id != FixtureConst.DocType1)
                {
                    throw new ApplicationException("Unknown id for this unit test!");
                }

                return new DocTypeItem(
                    FixtureData.DocTypeItem1.Id,
                    FixtureData.DocTypeItem1.Folder,
                    FixtureData.DocTypeItem1.Name,
                    FixtureData.DocTypeItem1.Prefix,
                    FixtureData.DocTypeItem1.Description,
                    FixtureData.DocTypeItem1.NextNumber,
                    FixtureData.DocTypeItem1.Owner
                    );
            }

            public override void SaveDocTypeItem(DocTypeItem item)
            {
                FixtureData.DocTypeItem1 = new DocTypeItem(
                    item.Id,
                    item.Folder,
                    item.Name,
                    item.Prefix,
                    item.Description,
                    item.NextNumber,
                    item.Owner
                    );
            }
        }

        protected class MockUserInfo : UserInfo
        {
            public MockUserInfo(int userId)
            {
                _userProperties = new UserProperties();
                _userProperties.UserId = userId;
            }
        }

        protected class MockDataConnection : DataConnection
        {
            public MockDataConnection()
            {
                _con = null;
            }
        }

        protected class TestDocTypeData : DocTypeData
        {
            public TestDocTypeData()
            {
            }

            public void SetDocTypeAccess(DocTypeAccess dtAccess)
            {
                _dtAccess = dtAccess;
            }
            
        }

        [TestFixtureSetUp]
        public void TFSetUp()
        {
            FixtureData.DocTypeItem1 = new DocTypeItem(
                FixtureConst.DocType1,
                0,
                "",
                FixtureConst.DocType1Prefix,
                "",
                FixtureConst.DocType1InitialNextNumber,
                12
                );

        }

        [Test]
        public void GetNextInternalNumber()
        {
            DataConnection con = new MockDataConnection();
            UserInfo userInfo = new MockUserInfo(FixtureConst.DocCircService);

            TestDocTypeData dtd = new TestDocTypeData();
            dtd.SetDocTypeAccess(new MockDocTypeAccess());

            string nextNumber = dtd.GetNextInternalNumber(FixtureConst.DocType1); // tested method

            Assert.AreEqual(FixtureConst.DocType1Prefix + "000001", nextNumber);
            Assert.AreEqual(2, FixtureData.DocTypeItem1.NextNumber);

            nextNumber = dtd.GetNextInternalNumber(FixtureConst.DocType1); // tested method

            Assert.AreEqual(FixtureConst.DocType1Prefix + "000002", nextNumber);
            Assert.AreEqual(3, FixtureData.DocTypeItem1.NextNumber);

            nextNumber = dtd.GetNextInternalNumber(FixtureConst.DocType1); // tested method

            Assert.AreEqual(FixtureConst.DocType1Prefix + "000003", nextNumber);
            Assert.AreEqual(4, FixtureData.DocTypeItem1.NextNumber);
        }

    }
}
