//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
using Evpo.DocCirc.Biz;
using Evpo.DocCirc.MySqlDL;
using Evpo.DocCirc.Common;

namespace Evpo.DocCircUnitTest
{

    [TestFixture]
    public class UserHelperInstanceTest
    {
        /// <summary>
        /// see name
        /// </summary>
        class MockUserAccess : UserAccess
        {
            public MockUserAccess()
            {
            }

            public override UserToken AuthorizeUser(string login, string password)
            {
                UserToken retVal = null;
                if (login == "DocCircAdmin" && password == "doccirc")
                {
                    retVal = new UserToken(1, "DocCircAdmin");
                }

                return retVal;
            }
        }

        
        /// <summary>
        /// 
        /// </summary>
        class MockDBConnection : DBConnection
        {
            public override bool IsOpen
            {
                get
                {
                    return true;
                }
            }
        }

        /// <summary>
        /// see name
        /// </summary>
        class TestUserHelperInstance : UserHelperInstance
        {
            protected override UserAccess GetUserAccess(DBConnection con)
            {
                return new MockUserAccess();
            }

        }

        [Test]
        public void AuthorizeUser()
        {
            DataConnection con = new DataConnection(new MockDBConnection());
            TestUserHelperInstance userHlp = new TestUserHelperInstance();

            UserToken token = userHlp.AuthorizeUser("DocCircAdmin", "doccirc", con);

            Assert.IsNotNull(token);

            Assert.AreEqual(token.UserId, (int)1);

            Assert.AreEqual(token.Login, "DocCircAdmin");

            Assert.IsNull(userHlp.AuthorizeUser("DocCircAdmin", "wrong password", con));
            
            Assert.IsNull(userHlp.AuthorizeUser("wrong login", "doccirc", con));
        }

    }
}
