//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
using Evpo.DocCirc;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.MySqlDL;
using Evpo.DocCirc.Biz;

namespace Evpo.DocCircUnitTest
{
    [TestFixture]
    public class UserSessionTest
    {
        /// <summary>
        /// 
        /// </summary>
        class MockDataConnection : DataConnection
        {
            private string _connectionId;
            public string ConnectionId
            {
                get { return _connectionId; }
            }

            public MockDataConnection()
            {
            }

            public MockDataConnection(string connectionId)
            {
                _connectionId = connectionId;
            }

            private bool _isOpen = false;

            public override void Open()
            {
                _isOpen = true;
            }

            public override void Close()
            {
                _isOpen = false;
            }

            public override bool IsOpen
            {
                get { return _isOpen; }
            }

        }

        /// <summary>
        /// 
        /// </summary>
        class MockUserInfo : UserInfo
        {
            public MockUserInfo(int userId, DataConnection con)
            {
                if (userId == 1)
                {
                    _userProperties = new UserProperties();
                    _userProperties.UserId = 1;
                    _userProperties.Login = "DocCircUser";
                    _userProperties.FullName = "DocCirc User";
                    _userProperties.AutomaticPass = false;
                    _userProperties.AllowDeputyNotification = true;
                    _userProperties.Department = 0;
                    _userProperties.InstantMessaging = false;
                    _userProperties.ImAddress = "";
                    
                    _emails = new string[] { "dcuser1@test.org", "dcuser2@test.org" };
                }
            }
        }

        class MockSecurityData : SecurityData
        {
            public MockSecurityData(UserInfo userInfo, DataConnection con)
            {

            }

            public override void CreateFolderSecurityTable()
            {
            }
        }

        class MockLogHelper : LogHelper
        {
            public override void AddStr(string str, string userLogin)
            {
                //nothing
            }
        }

        class MockStringResHelper : StringResHelper
        {
            public override string this[string key]
            {
                get
                {
                    return "";
                }
            }
            
        }

        /// <summary>
        /// 
        /// </summary>
        class TestUserSession : UserSession
        {
            public TestUserSession(UserToken userToken, DataConnection con, string userConnString, UserInfo serviceUserInfo, SettingKeeper settingKeeper):
                base(userToken, con, userConnString, serviceUserInfo, settingKeeper)
            {
            }

            protected override DataConnection GetDataConnection(string connString)
            {
                return new MockDataConnection(connString);
            }

            protected override UserInfo GetUserInfo(int userId, DataConnection con)
            {
                return new MockUserInfo(userId, con);
            }

            protected override SecurityData GetSecurityData(UserInfo userInfo, DataConnection con)
            {
                return new MockSecurityData(userInfo, con);
            }
        }

        
        [SetUp]
        public void SetUp()
        {
            LogHelper.Current = new MockLogHelper();
            ResourceHelper.StrForLog = new MockStringResHelper();
        }

        [Test]
        public void UserSessionCtor()
        {
            UserToken token = new UserToken(1, "DocCircUser");
            MockDataConnection con = new MockDataConnection();
            string userConnString = "connection for DocCircUser";
            TestUserSession us = new TestUserSession(token, con, userConnString, new UserInfo(), null); // tested method

            Assert.AreEqual("DocCircUser", us.UserInfo.Login );
            Assert.AreEqual("DocCirc User", us.UserInfo.FullName );
            Assert.AreEqual((int)1, us.UserInfo.UserId );

            Assert.AreEqual(2, us.UserInfo.Emails.Length);
            Assert.AreEqual("dcuser1@test.org", us.UserInfo.Emails[0] );
            Assert.AreEqual("dcuser2@test.org", us.UserInfo.Emails[1] );

            Assert.AreEqual(token, us.UserToken );

            Assert.AreEqual("connection for DocCircUser", ((MockDataConnection)us.DataConnection).ConnectionId );
            
            Assert.IsTrue(us.DataConnection.IsOpen);

            us.DataConnection.Close(); // tested method

            Assert.IsFalse(us.DataConnection.IsOpen);

            Assert.IsNotNull(us.DocTypeDrafts);
            Assert.IsNotNull(us.DocumentDrafts);
        }
    }
}
