//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using System.Xml.Serialization;
using System.IO;
using NUnit.Framework;
using Evpo.DocCirc.Common;


namespace Evpo.DocCircUnitTest
{
    [TestFixture]
    public class AppSettingOperatorTest
    {
        public struct FixtureConst
        {
            public const string Folder = "Common";
            public const string SettingFile = "AppSettingOperatorTest.file1.xml";
            public const string SavedSettingFile = "AppSettingOperatorTest.file2.xml";
        }

        [Test]
        public void Load()
        {
            NameValueCollection pars = (new AppSettingOperator()).Load(
                Path.Combine(FixtureConst.Folder, FixtureConst.SettingFile)
                ); // tested method
            
            Assert.AreEqual(3, pars.Count);
            
            Assert.IsNotNull(pars["FirstKey"]);
            Assert.AreEqual("First", pars["FirstKey"]);
            
            Assert.IsNotNull(pars["SecondKey"]);
            Assert.AreEqual("Second", pars["SecondKey"]);
            
            Assert.IsNotNull(pars["ThirdKey"]);
            Assert.AreEqual("Third", pars["ThirdKey"]);

        }

        [Test]
        public void Save()
        {
            NameValueCollection pars = new NameValueCollection();
            pars.Add("FirstKey", "First");
            pars.Add("SecondKey", "Second");
            pars.Add("ThirdKey", "Third");
            
            (new AppSettingOperator()).Save(
                Path.Combine(FixtureConst.Folder, FixtureConst.SavedSettingFile),
                pars
                ); // tested method

            TextReader reader = File.OpenText(Path.Combine(FixtureConst.Folder, FixtureConst.SettingFile));
            string master = reader.ReadToEnd();
            reader.Close();
            
            reader = File.OpenText(Path.Combine(FixtureConst.Folder, FixtureConst.SavedSettingFile));
            string saved = reader.ReadToEnd();
            reader.Close();

            string[] masterLines = 
                master.Split(new string[] { Environment.NewLine }, StringSplitOptions.RemoveEmptyEntries);
            string[] savedLines = 
                master.Split(new string[] { Environment.NewLine }, StringSplitOptions.RemoveEmptyEntries);
            try
            {
                for (int i = 0; i < masterLines.Length; i++)
                {
                    Assert.AreEqual(masterLines[i], savedLines[i]);
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                File.Delete(Path.Combine(FixtureConst.Folder, FixtureConst.SavedSettingFile));
            }


        }

    }
}
