//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using NUnit.Framework;
using Evpo.DocCirc;
using Evpo.DocCirc.UI;
using Evpo.DocCirc.Common;

namespace Evpo.DocCircUnitTest
{
    

    class MockFileStreamWrapper : FileStreamWrapper
    {
        private string _text;
        public string Text
        {
            get { return _text; }
            set { _text = value; }
        }
        
        public MockFileStreamWrapper():base()
        {
            _text = "";
        }

        public override void Write(byte[] array, int offset, int count)
        {
            byte[] neededBytes = new byte[count];
            Array.Copy(array, offset, neededBytes, 0, count);
            string writtenText = Encoding.ASCII.GetString(neededBytes);
            Console.Write(writtenText);
            _text += writtenText;
        }

        public override void Flush()
        {
            // do nothing
        }
        public override void Close()
        {
            //do nothing
        }

        public static MockFileStreamWrapper MockFileStreamWrapperInst;
        
    }

    class TestLogHelperInstance : LogHelper
    {
        public TestLogHelperInstance(string fileName)
            : base(fileName)
        {
        }

        public override string GetCurrentUserName()
        {
            return @"TEST\testaccount";
        }
    }

    [TestFixture]
    public class LogHelperInstanceTest
    {
        class MockFileWrapperInstance : FileWrapper
        {
            private string _fileName;

            private bool _mockFileExists = false;

            public bool MockFileExists
            {
                get { return _mockFileExists; }
                set { _mockFileExists = value; }
            }

            public MockFileWrapperInstance()
            {
            }

            public override FileStreamWrapper Create(string path)
            {
                _fileName = path;
                if (_fileName == @"c:\test.log")
                {
                    _mockFileExists = true;
                    return MockFileStreamWrapper.MockFileStreamWrapperInst;
                }
                else
                {
                    return null;
                }
            }

            public override bool Exists(string path)
            {
                if (path == @"c:\test.log")
                {
                    return _mockFileExists;
                }
                else
                {
                    return false;
                }
            }

            public override FileStreamWrapper Open(string path, FileMode mode)
            {
                return MockFileStreamWrapper.MockFileStreamWrapperInst;
            }
        }

        [Test]
        public void AddStr()
        {
            FileWrapper.Current = new MockFileWrapperInstance();

            MockFileStreamWrapper mockFS = new MockFileStreamWrapper();
            MockFileStreamWrapper.MockFileStreamWrapperInst = mockFS;

            LogHelper.Current = new TestLogHelperInstance(@"c:\test.log");

            LogHelper.Current.AddStr("first log str");
            LogHelper.Current.AddStr("second log str");
            string text = mockFS.Text;
            string[] lines = text.Split(new string[] { Environment.NewLine }, StringSplitOptions.None);

            Assert.AreEqual(
                lines[0].Substring(DateTime.Now.ToString().Length),
                @" TEST\testaccount first log str"
                );

            Assert.AreEqual(
                lines[1].Substring(DateTime.Now.ToString().Length),
                @" TEST\testaccount second log str"
                );

            LogHelper.Current.AddStr("third log str", "user1");
            lines = mockFS.Text.Split(new string[]{ Environment.NewLine}, StringSplitOptions.None);
            Assert.AreEqual(
                @" TEST\testaccount user1 third log str",
                lines[2].Substring(DateTime.Now.ToString().Length)
                );

        }

        [Test]
        public void WriteException()
        {
            FileWrapper.Current = new MockFileWrapperInstance();

            MockFileStreamWrapper mockFS = new MockFileStreamWrapper();
            MockFileStreamWrapper.MockFileStreamWrapperInst = mockFS;

            LogHelper.Current = new TestLogHelperInstance(@"c:\test.log");

            ApplicationException ex = new ApplicationException("Unit test WriteException");
            LogHelper.Current.WriteException(ex);
            string text = MockFileStreamWrapper.MockFileStreamWrapperInst.Text;
            Assert.Greater(text.Length, 0);

            LogHelper.Current.WriteException(ex, "user1");
            text = MockFileStreamWrapper.MockFileStreamWrapperInst.Text;
            Assert.Greater(text.Length, 0);
        }

        [TearDown]
        public void Close()
        {
            LogHelper.Current = null;
            FileWrapper.Current = null;
        }

    }
}
