//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using NUnit.Framework;
using MySql.Data.MySqlClient;
using Evpo.DocCirc.MySqlDL;
using Evpo.DocCirc.Common;

namespace Evpo.DocCircUnitTest
{
    [TestFixture]
    public class AttachmentAccessTest
    {
        public struct FixtureFile
        {
            public const string Folder = "MySql";
            public const string CreatingDocCards = "AttachmentAccessTest.file4.sql";
            public const string CreatingAttachments = "AttachmentAccessTest.file5.sql";
            public const string Attachment1 = "AttachmentAccessTest.file1.txt";
            public const string Attachment2 = "AttachmentAccessTest.file2.txt";
            public const string Attachment3 = "AttachmentAccessTest.file3.txt";
            public const string Attachment6 = "AttachmentAccessTest.file6.gif";
        }

        public class FixtureConstant
        {
            public string AttachmentDir1, AttachmentDir2, AttachmentDir3;
            public long Attachment1, Attachment2, Attachment6;
            public long DocCard1, DocCard2, DocCard3;
            public int DocCircAdmin, DocCircUser1, DocCircUser2;
            public int TestDocumentFolder1;
        }

        private FixtureConstant _const = new FixtureConstant();
        
        private DBConnection _con;
        private MySqlTransaction _tran;
        private SettingKeeper _sk;

        private int _namingBase = 36;
        private int _nameLength = 13;

        private string ConvertToBase(long num, int nbase)
        {
            String chars = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";

            // check if we can convert to another base
            if (nbase < 2 || nbase > chars.Length)
                return "";

            int r;
            String newNumber = "";

            // in r we have the offset of the char that was converted to the new base
            while (num >= nbase)
            {
                r = (int)(num % nbase);
                newNumber = chars[r] + newNumber;
                num = num / nbase;
            }
            // the last number to convert
            newNumber = chars[(int)num] + newNumber;

            return newNumber;
        }

        private long ConvertFromBase(string num, int nbase)
        {
            String chars = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
            SortedList<char, int> charList = new SortedList<char, int>(chars.Length);
            for (int i = 0; i < chars.Length; i++)
            {
                charList.Add(chars[i], i);
            }

            long retVal = 0;
            for (int i = 0; i < num.Length; i++)
            {
                char ch = num[num.Length - i - 1];
                int val = charList[ch];
                retVal += (long)(Math.Pow(nbase, i) * val);
            }
            return retVal;
        }

        private string GetNameById(long id)
        {
            return ConvertToBase(id, _namingBase).PadLeft(_nameLength, '0');
        }

        [TestFixtureSetUp]
        public void TFSetUp()
        {
            _sk = new SettingKeeper();
            _sk.MaxFilesInFolder = 2;
            _sk.AttachmentBufferSize = 65536;
            _sk.AttachmentFolder = Path.Combine(
                Directory.GetCurrentDirectory(),
                "attachments"
                );

            if (Directory.Exists(_sk.AttachmentFolder))
            {
                DeleteFolder(_sk.AttachmentFolder);
            }
            
            Directory.CreateDirectory(_sk.AttachmentFolder);

            _const.AttachmentDir1 = Path.Combine(_sk.AttachmentFolder,"0000000000001");
            _const.AttachmentDir2 = Path.Combine(_sk.AttachmentFolder,"0000000000002");
            _const.AttachmentDir3 = Path.Combine(_sk.AttachmentFolder, "0000000000003");

            Directory.CreateDirectory(_const.AttachmentDir1);
            Directory.CreateDirectory(_const.AttachmentDir2);

            File.Copy(
                Path.Combine(FixtureFile.Folder, FixtureFile.Attachment1),
                Path.Combine(_const.AttachmentDir1, "0000000000001.txt")
                );

            File.Copy(
                Path.Combine(FixtureFile.Folder, FixtureFile.Attachment2),
                Path.Combine(_const.AttachmentDir1, "0000000000002.txt")
                );

            File.Copy(
                Path.Combine(FixtureFile.Folder, FixtureFile.Attachment6),
                Path.Combine(_const.AttachmentDir2, "0000000000006.gif")
                );


            _con = new DBConnection(
             "Database=EvpoDCTest;Data Source=localhost;User Id=doccirctest;Password=doccirctest"
             );
            _con.Open();
            _tran = _con.MySqlConnection.BeginTransaction();
            
            MySqlCommand cmd = new MySqlCommand();
            cmd.Connection = _con.MySqlConnection;
            //TextReader txtReader = File.OpenText(Path.Combine(FixtureFile.Folder, FixtureFile.CreatingDocCards));
            //txtReader.ReadToEnd();
            //txtReader.Close();
            cmd.CommandText = MySqlResources.AttachmentAccessTest_file4;
            cmd.ExecuteNonQuery();

            //txtReader = File.OpenText(Path.Combine(FixtureFile.Folder, FixtureFile.CreatingAttachments));
            //txtReader.ReadToEnd();
            //txtReader.Close();
            cmd.CommandText = MySqlResources.AttachmentAccessTest_file5;
            
            cmd.ExecuteNonQuery();

            // get consts
            cmd.CommandText =
                "select cast(@docCard1 as unsigned) as DocCard1, cast(@docCard2 as unsigned) as DocCard2, " +
                "cast(@docCard3 as unsigned) as DocCard3, " +
                "cast(@attachment1 as unsigned) as Attachment1, cast(@attachment2 as unsigned) as Attachment2, " +
                "cast(@attachment6 as unsigned) as Attachment6, " +
                "cast(@docCircAdmin as unsigned) as DocCircAdmin, " +
                "cast(@docCircUser1 as unsigned) as DocCircUser1, " +
                "cast(@docCircUser2 as unsigned) as DocCircUser2, " +
                "cast(@testDocumentFolder1 as unsigned) as TestDocumentFolder1;";

            MySqlDataReader reader = cmd.ExecuteReader();
            reader.Read();
            _const.DocCard1 = reader.GetInt64("DocCard1");
            _const.DocCard2 = reader.GetInt64("DocCard2");
            _const.DocCard3 = reader.GetInt64("DocCard3");
            _const.Attachment1 = reader.GetInt64("Attachment1");
            _const.Attachment2 = reader.GetInt64("Attachment2");
            _const.Attachment6 = reader.GetInt64("Attachment6");
            _const.DocCircAdmin = reader.GetInt32("DocCircAdmin");
            _const.DocCircUser1 = reader.GetInt32("DocCircUser1");
            _const.DocCircUser2 = reader.GetInt32("DocCircUser2");
            _const.TestDocumentFolder1 = reader.GetInt32("TestDocumentFolder1");
            
            reader.Close();

            DBConstAccess ca = new DBConstAccess(_con);
            FolderTypeDirectory ftd = ca.GetFolderTypeDirectory();
            DBConstKeeper.Current = new DBConstKeeper(
                ftd,
                ca.GetRootFolderDirectory(ftd),
                ca.GetMainRoleDirectory(),
                ca.GetRouteItemTypeDirectory(),
                ca.GetDocStatusDirectory(),
                ca.GetNotificationTypeDirectory(),
                ca.GetEventTypeDirectory(),
                ca.GetCommonDepartment()
                );
            DBFieldDefKeeper.Current = new DBFieldDefKeeper("");
        }

        [Test]
        public void GetAttachmentItem()
        {
            AttachmentAccess aa = new AttachmentAccess(_const.DocCircAdmin, _con, _sk);
            AttachmentItem item = aa.GetAttachmentItem(_const.Attachment1);

            Assert.IsNotNull(item);
            Assert.AreEqual(_const.Attachment1, item.Id);
            Assert.AreEqual(_const.DocCard1, item.DocCardId);
            Assert.AreEqual("AttachmentAccessTest.file1.txt", item.FileName);
            Assert.AreEqual(1, item.Folder);
            Assert.AreEqual(1, item.File);
            Assert.AreEqual("Content1", item.ContentType);
            Assert.AreEqual(1924, item.Size);
            Assert.AreEqual(
                new DateTime(2007, 06, 01, 10, 00, 00),
                item.UploadTime
                );

            item = aa.GetAttachmentItem(_const.Attachment6);

            Assert.IsNotNull(item);
            Assert.AreEqual(_const.Attachment6, item.Id);
            Assert.AreEqual(_const.DocCard2, item.DocCardId);
            Assert.AreEqual("AttachmentAccessTest.file6.gif", item.FileName);
            Assert.AreEqual(2, item.Folder);
            Assert.AreEqual(6, item.File);
            Assert.AreEqual("Content2", item.ContentType);
            Assert.AreEqual(879, item.Size);
            Assert.AreEqual(
                new DateTime(2007, 06, 01, 10, 04, 00),
                item.UploadTime
                );
        }

        [Test]
        public void GetDocCardAttachmentItems()
        {
            AttachmentAccess aa = new AttachmentAccess(_const.DocCircAdmin, _con, _sk);
            AttachmentItem[] items = aa.GetDocCardAttachmentItems(_const.DocCard2);

            Assert.AreEqual(2, items.Length);

            bool[] been = new bool[] { false, false };

            foreach (AttachmentItem item in items)
            {
                if(item.Id == _const.Attachment2 && !been[0])
                {
                    been[0] = true;
                    Assert.AreEqual(_const.DocCard2, item.DocCardId);
                    Assert.AreEqual("AttachmentAccessTest.file2.txt", item.FileName);
                    Assert.AreEqual(1, item.Folder);
                    Assert.AreEqual(2, item.File);
                    Assert.AreEqual("Content1", item.ContentType);
                    Assert.AreEqual(49, item.Size);
                    Assert.AreEqual(
                        new DateTime(2007, 06, 01, 10, 02, 00),
                        item.UploadTime
                        );
                }
                else if (item.Id == _const.Attachment6 && !been[1])
                {
                    been[1] = true;
                    Assert.AreEqual(_const.DocCard2, item.DocCardId);
                    Assert.AreEqual("AttachmentAccessTest.file6.gif", item.FileName);
                    Assert.AreEqual(2, item.Folder);
                    Assert.AreEqual(6, item.File);
                    Assert.AreEqual("Content2", item.ContentType);
                    Assert.AreEqual(879, item.Size);
                    Assert.AreEqual(
                        new DateTime(2007, 06, 01, 10, 04, 00),
                        item.UploadTime
                        );
                }
                else
                {
                    Assert.Fail(String.Format("Wrong or repeated attachment. Id={0}", item.Id));
                }
            }

            Assert.IsTrue(been[0]);
            Assert.IsTrue(been[1]);

            items = aa.GetDocCardAttachmentItems(_const.DocCard1);

            Assert.AreEqual(1, items.Length);
            Assert.AreEqual(_const.Attachment1, items[0].Id);
        }

        [Test]
        public void OpenAttachment()
        {
            AttachmentAccess aa = new AttachmentAccess(_const.DocCircAdmin, _con, _sk);
            AttachmentItem item1 = aa.GetAttachmentItem(_const.Attachment1),
                item2 = aa.GetAttachmentItem(_const.Attachment2),
                item6 = aa.GetAttachmentItem(_const.Attachment6);

            Stream masterFile = null, testedFile = null;
            int masterByte = 0;
            try
            {
                // attachment 1
                testedFile = aa.OpenAttachment(item1);
                Assert.IsNotNull(testedFile, "Attachment1");

                masterFile = File.OpenRead(Path.Combine(FixtureFile.Folder, FixtureFile.Attachment1));
                masterByte = 0;
                while ((masterByte = masterFile.ReadByte()) >= 0)
                {
                    Assert.AreEqual(masterByte, testedFile.ReadByte(), "Attachment1");
                }
                }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                masterFile.Close();
                testedFile.Close();
            }

            try
            {

                // attachment 2
                testedFile = aa.OpenAttachment(item2);
                Assert.IsNotNull(testedFile, "Attachment2");

                masterFile = File.OpenRead(Path.Combine(FixtureFile.Folder, FixtureFile.Attachment2));
                masterByte = 0;
                while ((masterByte = masterFile.ReadByte()) >= 0)
                {
                    Assert.AreEqual(masterByte, testedFile.ReadByte(), "Attachment2");
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                masterFile.Close();
                testedFile.Close();
            }

            try
            {

                // attachment 6
                testedFile = aa.OpenAttachment(item6);
                Assert.IsNotNull(testedFile, "Attachment6");

                masterFile = File.OpenRead(Path.Combine(FixtureFile.Folder, FixtureFile.Attachment6));
                masterByte = 0;
                while ((masterByte = masterFile.ReadByte()) >= 0)
                {
                    Assert.AreEqual(masterByte, testedFile.ReadByte(), "Attachment6");
                }
                
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                masterFile.Close();
                testedFile.Close();
            }
        }

        [Test]
        [ExpectedException(ExpectedException = typeof(DcUnathorizedAccessException))]
        public void OpenAttachmentWithoutAccessInFolder()
        {
            MySqlCommand cmd = new MySqlCommand(
               "update FolderPermission set R=?R where Folder=@testDocumentFolder1 and DcRole=@userRole;",
               _con.MySqlConnection
               );
            cmd.Parameters.Add("?R", MySqlDbType.Bit);
            cmd.Parameters["?R"].Value = false;
            cmd.ExecuteNonQuery();

            AttachmentAccess aa = new AttachmentAccess(_const.DocCircUser1, _con, _sk);

            try
            {
                AttachmentItem ai = aa.GetAttachmentItem(_const.Attachment2); //tested method
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                cmd.Parameters["?R"].Value = true;
                cmd.ExecuteNonQuery();
            }
        }

        [Test]
        [ExpectedException(ExpectedException = typeof(DcUnathorizedAccessException))]
        public void OpenAttachmentWithoutAccessInDocType()
        {
            MySqlCommand cmd = new MySqlCommand(
                "update DocTypePermission set R=?R where DocType=@docType2 and DcRole=@userRole;",
                _con.MySqlConnection
                );
            cmd.Parameters.Add("?R", MySqlDbType.Bit);
            cmd.Parameters["?R"].Value = false;
            cmd.ExecuteNonQuery();

            AttachmentAccess aa = new AttachmentAccess(_const.DocCircUser1, _con, _sk);

            try
            {
                AttachmentItem ai = aa.GetAttachmentItem(_const.Attachment2); // tested method
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                cmd.Parameters["?R"].Value = true;
                cmd.ExecuteNonQuery();
            }
        }

        [Test]
        public void OpenAttachmentWithReadOwnNotReadInDocType()
        {
            MySqlCommand cmd = new MySqlCommand(
                "update DocTypePermission set R=?R where DocType=@docType2 and DcRole=@userRole;",
                _con.MySqlConnection
                );
            cmd.Parameters.Add("?R", MySqlDbType.Bit);
            cmd.Parameters["?R"].Value = false;
            cmd.ExecuteNonQuery();

            AttachmentAccess aa = new AttachmentAccess(_const.DocCircUser2, _con, _sk);
            AttachmentItem ai = aa.GetAttachmentItem(_const.Attachment2); // tested method
            
            Stream readerStream = aa.OpenAttachment(ai); // tested method
            readerStream.Close();

            try
            {
                Assert.AreEqual(_const.Attachment2, ai.Id);
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                cmd.Parameters["?R"].Value = true;
                cmd.ExecuteNonQuery();
            }
        }

        [Test]
        public void OpenAttachmentWithReadOwnNotReadInFolder()
        {
            MySqlCommand cmd = new MySqlCommand(
               "update FolderPermission set R=?R where Folder=@testDocumentFolder1 and DcRole=@userRole;",
               _con.MySqlConnection
               );
            cmd.Parameters.Add("?R", MySqlDbType.Bit);
            cmd.Parameters["?R"].Value = false;
            cmd.ExecuteNonQuery();

            AttachmentAccess aa = new AttachmentAccess(_const.DocCircUser2, _con, _sk);
            AttachmentItem ai = aa.GetAttachmentItem(_const.Attachment2); // tested method

            Stream readerStream = aa.OpenAttachment(ai); // tested method
            readerStream.Close();

            try
            {
                Assert.AreEqual(_const.Attachment2, ai.Id);
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                cmd.Parameters["?R"].Value = true;
                cmd.ExecuteNonQuery();
            }
        }


        [Test]
        public void SaveAttachment()
        {
            List<string> filesToDelete = new List<string>();
            List<long> rowsInAttachmentToDelete = new List<long>();

            // add new attachment from file 3

            AttachmentAccess aa = new AttachmentAccess(_const.DocCircAdmin, _con, _sk);
            AttachmentItem item = new AttachmentItem(
                null,
                _const.DocCard3,
                "AttachmentAccessTest.file3.txt",
                null,
                null,
                "Content5",
                49,
                new DateTime(2007, 06, 01, 16, 50, 00)
                );

            Stream file3 = null;
            try
            {
                file3 = File.OpenRead(Path.Combine(FixtureFile.Folder, FixtureFile.Attachment3));
                aa.SaveAttachment(item, file3); // tested method
              
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                file3.Close();
            }

            Assert.IsNotNull(item.Id);
            Assert.IsNotNull(item.File);
            Assert.IsTrue(item.File > 0L);
            Assert.AreEqual(item.Id, item.File);
            //Assert.AreEqual(2, item.Folder);

            string folderPath =
                Path.Combine(_sk.AttachmentFolder, GetNameById(item.Folder.Value));
            string filePath = GetNameById(item.File.Value) + Path.GetExtension(FixtureFile.Attachment3);
            filePath = Path.Combine(folderPath, filePath);

            Assert.IsTrue(File.Exists(filePath));
            filesToDelete.Add(filePath);

            Stream masterFile = null, testedFile = null;
            int masterByte = 0;
            try
            {
                testedFile = File.OpenRead(filePath);

                masterFile = File.OpenRead(Path.Combine(FixtureFile.Folder, FixtureFile.Attachment3));
                masterByte = 0;
                while ((masterByte = masterFile.ReadByte()) >= 0)
                {
                    Assert.AreEqual(masterByte, testedFile.ReadByte(), "Attachment3");
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                masterFile.Close();
                testedFile.Close();
            }

            MySqlCommand cmd = new MySqlCommand(
                "select DocCard, Name, Folder, File, ContentType, Size, UploadTime " +
                "from Attachment where AttachmentId=?Attachment",
                _con.MySqlConnection
                );
            cmd.Parameters.Add("?Attachment", MySqlDbType.Int64);
            cmd.Parameters["?Attachment"].Value = item.Id;
            MySqlDataReader reader = cmd.ExecuteReader();
            try
            {
                Assert.IsTrue(reader.Read());
                Assert.AreEqual(item.DocCardId, reader.GetInt64("DocCard"));
                Assert.AreEqual(item.FileName, reader.GetString("Name"));
                Assert.AreEqual(item.Folder, reader.GetInt64("Folder"));
                Assert.AreEqual(item.File, reader.GetInt64("File"));
                Assert.AreEqual(item.ContentType, reader.GetString("ContentType"));
                Assert.AreEqual(item.Size, reader.GetInt32("Size"));
                Assert.AreEqual(item.UploadTime, reader.GetDateTime("UploadTime"));
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                reader.Close();
            }

            rowsInAttachmentToDelete.Add(item.Id.Value);

            // add new attachment from file 2

            item = new AttachmentItem(
                null,
                _const.DocCard3,
                "AttachmentAccessTest.file2.txt",
                null,
                null,
                "Content2",
                49,
                new DateTime(2007, 06, 01, 16, 50, 00)
                );

            Stream file2 = null;
            try
            {
                file2 = File.OpenRead(Path.Combine(FixtureFile.Folder, FixtureFile.Attachment2));
                aa.SaveAttachment(item, file2); // tested method
               
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                file2.Close();
            }

            Assert.IsNotNull(item.Id);
            Assert.IsNotNull(item.File);
            Assert.AreEqual(item.Id, item.File);

            string nextFolderPath =
                Path.Combine(_sk.AttachmentFolder, GetNameById(item.Folder.Value));
            
            Assert.AreNotEqual(nextFolderPath, folderPath);
            
            filePath = GetNameById(item.File.Value) + Path.GetExtension(FixtureFile.Attachment2);
            filePath = Path.Combine(nextFolderPath, filePath);
            Assert.IsTrue(File.Exists(filePath));

            filesToDelete.Add(filePath);

            masterFile = null;
            testedFile = null;
            masterByte = 0;
            try
            {
                testedFile = File.OpenRead(filePath);

                masterFile = File.OpenRead(Path.Combine(FixtureFile.Folder, FixtureFile.Attachment2));
                masterByte = 0;
                while ((masterByte = masterFile.ReadByte()) >= 0)
                {
                    Assert.AreEqual(masterByte, testedFile.ReadByte(), "Attachment2");
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                masterFile.Close();
                testedFile.Close();
            }

            //cmd = new MySqlCommand(
            //    "select DocCard, Name, Folder, File, ContentType, Size, UploadTime " +
            //    "from Attachment where AttachmentId=?Attachment",
            //    _con.MySqlConnection
            //    );
            //cmd.Parameters.Add("?Attachment", MySqlDbType.Int64);
            cmd.Parameters["?Attachment"].Value = item.Id;
            reader = cmd.ExecuteReader();
            try
            {
                Assert.IsTrue(reader.Read());
                Assert.AreEqual(item.DocCardId, reader.GetInt64("DocCard"));
                Assert.AreEqual(item.FileName, reader.GetString("Name"));
                Assert.AreEqual(item.Folder, reader.GetInt64("Folder"));
                Assert.AreEqual(item.File, reader.GetInt64("File"));
                Assert.AreEqual(item.ContentType, reader.GetString("ContentType"));
                Assert.AreEqual(item.Size, reader.GetInt32("Size"));
                Assert.AreEqual(item.UploadTime, reader.GetDateTime("UploadTime"));
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                reader.Close();
            }

            rowsInAttachmentToDelete.Add(item.Id.Value);

            // save last attachment again and replace it with content of attachment 1
            item.Size = 1924;
            item.UploadTime = new DateTime(2007, 06, 04, 9, 39, 00);
            long? idBeforeSaving = item.Id;
            Stream file1 = null;
            try
            {
                file1 = File.OpenRead(Path.Combine(FixtureFile.Folder, FixtureFile.Attachment1));
                aa.SaveAttachment(item, file1); // tested method
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                file1.Close();
            }
            Assert.AreEqual(idBeforeSaving, item.Id, "ID has been changed!");
            folderPath =
                Path.Combine(_sk.AttachmentFolder, GetNameById(item.Folder.Value));

            Assert.AreEqual(nextFolderPath, folderPath);

            filePath = GetNameById(item.File.Value) + Path.GetExtension(FixtureFile.Attachment2);
            filePath = Path.Combine(folderPath, filePath);
            Assert.IsTrue(File.Exists(filePath));

            masterFile = null;
            testedFile = null;
            masterByte = 0;
            try
            {
                testedFile = File.OpenRead(filePath);

                masterFile = File.OpenRead(Path.Combine(FixtureFile.Folder, FixtureFile.Attachment1));
                masterByte = 0;
                while ((masterByte = masterFile.ReadByte()) >= 0)
                {
                    Assert.AreEqual(masterByte, testedFile.ReadByte(), "Attachment1");
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                masterFile.Close();
                testedFile.Close();
            }

            //cmd = new MySqlCommand(
            //    "select DocCard, Name, Folder, File, ContentType, Size, UploadTime " +
            //    "from Attachment where AttachmentId=?Attachment",
            //    _con.MySqlConnection
            //    );
            //cmd.Parameters.Add("?Attachment", MySqlDbType.Int64);
            cmd.Parameters["?Attachment"].Value = item.Id;
            reader = cmd.ExecuteReader();
            try
            {
                Assert.IsTrue(reader.Read());
                Assert.AreEqual(item.DocCardId, reader.GetInt64("DocCard"));
                Assert.AreEqual(item.FileName, reader.GetString("Name"));
                Assert.AreEqual(item.Folder, reader.GetInt64("Folder"));
                Assert.AreEqual(item.File, reader.GetInt64("File"));
                Assert.AreEqual(item.ContentType, reader.GetString("ContentType"));
                Assert.AreEqual(1924, reader.GetInt32("Size"));
                Assert.AreEqual(new DateTime(2007, 06, 04, 9, 39, 00), reader.GetDateTime("UploadTime"));
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                reader.Close();
            }

            // clean up
            foreach (string fileToDelete in filesToDelete)
            {
                File.Delete(fileToDelete);
            }

            cmd.CommandText = "delete from attachment where AttachmentId=?Attachment;";
            foreach (long rowToDelete in rowsInAttachmentToDelete)
            {
                cmd.Parameters["?Attachment"].Value = rowToDelete;
                cmd.ExecuteNonQuery();
            }
        }

        [Test]
        [ExpectedException(ExpectedException = typeof(DcUnathorizedAccessException))]
        public void SaveAttachmentWithoutUpdateInFolder()
        {
            MySqlCommand cmd = new MySqlCommand(
               "update FolderPermission set U=?U where Folder=@testDocumentFolder1 and DcRole=@userRole;",
               _con.MySqlConnection
               );
            cmd.Parameters.Add("?U", MySqlDbType.Bit);
            cmd.Parameters["?U"].Value = false;
            cmd.ExecuteNonQuery();

            AttachmentAccess aa = new AttachmentAccess(_const.DocCircUser1, _con, _sk);
            AttachmentItem ai = aa.GetAttachmentItem(_const.Attachment2);
            
            Stream file2 = null;

            try
            {
                file2 = File.OpenRead(Path.Combine(FixtureFile.Folder, FixtureFile.Attachment2));
                aa.SaveAttachment(ai, file2);
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                file2.Close();
                cmd.Parameters["?U"].Value = true;
                cmd.ExecuteNonQuery();
            }
        }

        [Test]
        public void SaveAttachmentWithUpdateOwnNotUpdateInFolder()
        {
            MySqlCommand cmd = new MySqlCommand(
               "update FolderPermission set U=?U where Folder=@testDocumentFolder1 and DcRole=@userRole;",
               _con.MySqlConnection
               );
            cmd.Parameters.Add("?U", MySqlDbType.Bit);
            cmd.Parameters["?U"].Value = false;
            cmd.ExecuteNonQuery();

            AttachmentAccess aa = new AttachmentAccess(_const.DocCircUser2, _con, _sk);
            AttachmentItem ai = aa.GetAttachmentItem(_const.Attachment2);

            Stream file2 = null;

            try
            {
                file2 = File.OpenRead(Path.Combine(FixtureFile.Folder, FixtureFile.Attachment2));
                aa.SaveAttachment(ai, file2);
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                cmd.Parameters["?U"].Value = true;
                cmd.ExecuteNonQuery();
                file2.Close();
            }
        }

        [Test]
        [ExpectedException(ExpectedException = typeof(DcUnathorizedAccessException))]
        public void SaveAttachmentWithoutUpdateInDocType()
        {
            MySqlCommand cmd = new MySqlCommand(
                "update DocTypePermission set U=?U where DocType=@docType2 and DcRole=@userRole;",
                _con.MySqlConnection
                );
            cmd.Parameters.Add("?U", MySqlDbType.Bit);
            cmd.Parameters["?U"].Value = false;
            cmd.ExecuteNonQuery();

            AttachmentAccess aa = new AttachmentAccess(_const.DocCircUser1, _con, _sk);
            AttachmentItem ai = aa.GetAttachmentItem(_const.Attachment2);

            Stream file2 = null;

            try
            {
                file2 = File.OpenRead(Path.Combine(FixtureFile.Folder, FixtureFile.Attachment2));
                aa.SaveAttachment(ai, file2);
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                cmd.Parameters["?U"].Value = true;
                cmd.ExecuteNonQuery();
                file2.Close();
            }
        }

        [Test]
        public void SaveAttachmentWithUpdateOwnNotUpdateInDocType()
        {
            MySqlCommand cmd = new MySqlCommand(
                "update DocTypePermission set U=?U where DocType=@docType2 and DcRole=@userRole;",
                _con.MySqlConnection
                );
            cmd.Parameters.Add("?U", MySqlDbType.Bit);
            cmd.Parameters["?U"].Value = false;
            cmd.ExecuteNonQuery();

            AttachmentAccess aa = new AttachmentAccess(_const.DocCircUser2, _con, _sk);
            AttachmentItem ai = aa.GetAttachmentItem(_const.Attachment2);

            Stream file2 = null;

            try
            {
                file2 = File.OpenRead(Path.Combine(FixtureFile.Folder, FixtureFile.Attachment2));
                aa.SaveAttachment(ai, file2);
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                cmd.Parameters["?U"].Value = true;
                cmd.ExecuteNonQuery();
                file2.Close();
            }
        }

        [Test]
        public void DeleteAttachment()
        {
            AttachmentAccess aa = new AttachmentAccess(_const.DocCircAdmin, _con, _sk);
            AttachmentItem item = new AttachmentItem(
                _const.DocCard3,
                "AttachmentAccessTest.file3.txt",
                "Content3",
                49,
                new DateTime(2007, 06, 04, 14, 15, 00)
                );
            MySqlDataReader reader = null;
            Stream file3 = null;
            try
            {
                file3 = File.OpenRead(Path.Combine(FixtureFile.Folder, FixtureFile.Attachment3));
                aa.SaveAttachment(item, file3);
                aa.DeleteAttachment(item); // tested method
                string filePath = Path.Combine(_sk.AttachmentFolder, GetNameById(item.Folder.Value));
                filePath = Path.Combine(filePath, GetNameById(item.File.Value) + Path.GetExtension(item.FileName));

                Assert.IsFalse(File.Exists(filePath));

                MySqlCommand cmd = new MySqlCommand(
                    "select count(*) from attachment where AttachmentId=?Attachment;",
                    _con.MySqlConnection
                    );
                cmd.Parameters.Add("?Attachment", MySqlDbType.Int64);
                cmd.Parameters["?Attachment"].Value = item.Id;
                reader = cmd.ExecuteReader();
                if (reader.Read())
                {
                    Assert.AreEqual(0, reader.GetInt32(0));
                }
                reader.Close();

            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                file3.Close();
                if (!reader.IsClosed)
                {
                    reader.Close();
                }

                MySqlCommand cmd = new MySqlCommand(
                    "delete from attachment where AttachmentId=?Attachment",
                    _con.MySqlConnection
                    );
                cmd.Parameters.Add("?Attachment", MySqlDbType.Int64);
                cmd.Parameters["?Attachment"].Value = item.Id;
                cmd.ExecuteNonQuery();

                string filePath = Path.Combine(_sk.AttachmentFolder, GetNameById(item.Folder.Value));
                filePath = Path.Combine(filePath, GetNameById(item.File.Value) + Path.GetExtension(item.FileName));
                if (File.Exists(filePath))
                {
                    File.Delete(filePath);
                }

                if (Directory.GetFiles(Path.GetDirectoryName(filePath)).Length == 0)
                {
                    Directory.Delete(Path.GetDirectoryName(filePath));
                }
            }
        }

        [Test]
        [ExpectedException(ExpectedException = typeof(DcUnathorizedAccessException))]
        public void DeleteAttachmentWithoutUpdateInFolder()
        {
            MySqlCommand cmd = new MySqlCommand(
               "update FolderPermission set U=?U where Folder=@testDocumentFolder1 and DcRole=@userRole;",
               _con.MySqlConnection
               );
            cmd.Parameters.Add("?U", MySqlDbType.Bit);
            cmd.Parameters["?U"].Value = false;
            cmd.ExecuteNonQuery();

            AttachmentAccess aa = new AttachmentAccess(_const.DocCircAdmin, _con, _sk);
            AttachmentItem item = new AttachmentItem(
                _const.DocCard2,
                "AttachmentAccessTest.file3.txt",
                "Content3",
                49,
                new DateTime(2007, 06, 04, 14, 15, 00)
                );
            Stream file3 = null;
            try
            {
                file3 = File.OpenRead(Path.Combine(FixtureFile.Folder, FixtureFile.Attachment3));
                aa.SaveAttachment(item, file3);
                aa = new AttachmentAccess(_const.DocCircUser1, _con, _sk);
                aa.DeleteAttachment(item); // tested method
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                file3.Close();

                MySqlCommand cmdDel = new MySqlCommand(
                    "delete from attachment where AttachmentId=?Attachment",
                    _con.MySqlConnection
                    );
                cmdDel.Parameters.Add("?Attachment", MySqlDbType.Int64);
                cmdDel.Parameters["?Attachment"].Value = item.Id;
                cmdDel.ExecuteNonQuery();

                string filePath = Path.Combine(_sk.AttachmentFolder, GetNameById(item.Folder.Value));
                filePath = Path.Combine(filePath, GetNameById(item.File.Value) + Path.GetExtension(item.FileName));
                if (File.Exists(filePath))
                {
                    File.Delete(filePath);
                }

                if (Directory.GetFiles(Path.GetDirectoryName(filePath)).Length == 0)
                {
                    Directory.Delete(Path.GetDirectoryName(filePath));
                }

                cmd.Parameters["?U"].Value = true;
                cmd.ExecuteNonQuery();
            }
        }

        [Test]
        public void DeleteAttachmentWithUpdateOwnNotUpdateInFolder()
        {
            MySqlCommand cmd = new MySqlCommand(
               "update FolderPermission set U=?U where Folder=@testDocumentFolder1 and DcRole=@userRole;",
               _con.MySqlConnection
               );
            cmd.Parameters.Add("?U", MySqlDbType.Bit);
            cmd.Parameters["?U"].Value = false;
            cmd.ExecuteNonQuery();

            AttachmentAccess aa = new AttachmentAccess(_const.DocCircAdmin, _con, _sk);
            AttachmentItem item = new AttachmentItem(
                _const.DocCard2,
                "AttachmentAccessTest.file3.txt",
                "Content3",
                49,
                new DateTime(2007, 06, 04, 14, 15, 00)
                );
            Stream file3 = null;
            try
            {
                file3 = File.OpenRead(Path.Combine(FixtureFile.Folder, FixtureFile.Attachment3));
                aa.SaveAttachment(item, file3);
                aa = new AttachmentAccess(_const.DocCircUser2, _con, _sk);
                aa.DeleteAttachment(item); // tested method
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                file3.Close();

                MySqlCommand cmdDel = new MySqlCommand(
                    "delete from attachment where AttachmentId=?Attachment",
                    _con.MySqlConnection
                    );
                cmdDel.Parameters.Add("?Attachment", MySqlDbType.Int64);
                cmdDel.Parameters["?Attachment"].Value = item.Id;
                cmdDel.ExecuteNonQuery();

                string filePath = Path.Combine(_sk.AttachmentFolder, GetNameById(item.Folder.Value));
                filePath = Path.Combine(filePath, GetNameById(item.File.Value) + Path.GetExtension(item.FileName));
                if (File.Exists(filePath))
                {
                    File.Delete(filePath);
                }

                if (Directory.GetFiles(Path.GetDirectoryName(filePath)).Length == 0)
                {
                    Directory.Delete(Path.GetDirectoryName(filePath));
                }

                cmd.Parameters["?U"].Value = true;
                cmd.ExecuteNonQuery();
            }
        }

        [Test]
        [ExpectedException(ExpectedException = typeof(DcUnathorizedAccessException))]
        public void DeleteAttachmentWithoutUpdateInDocType()
        {
            MySqlCommand cmd = new MySqlCommand(
                "update DocTypePermission set U=?U where DocType=@docType2 and DcRole=@userRole;",
                _con.MySqlConnection
                );
            cmd.Parameters.Add("?U", MySqlDbType.Bit);
            cmd.Parameters["?U"].Value = false;
            cmd.ExecuteNonQuery();

            AttachmentAccess aa = new AttachmentAccess(_const.DocCircAdmin, _con, _sk);
            AttachmentItem item = new AttachmentItem(
                _const.DocCard2,
                "AttachmentAccessTest.file3.txt",
                "Content3",
                49,
                new DateTime(2007, 06, 04, 14, 15, 00)
                );
            Stream file3 = null;
            try
            {
                file3 = File.OpenRead(Path.Combine(FixtureFile.Folder, FixtureFile.Attachment3));
                aa.SaveAttachment(item, file3);
                aa = new AttachmentAccess(_const.DocCircUser1, _con, _sk);
                aa.DeleteAttachment(item); // tested method
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                file3.Close();

                MySqlCommand cmdDel = new MySqlCommand(
                    "delete from attachment where AttachmentId=?Attachment",
                    _con.MySqlConnection
                    );
                cmdDel.Parameters.Add("?Attachment", MySqlDbType.Int64);
                cmdDel.Parameters["?Attachment"].Value = item.Id;
                cmdDel.ExecuteNonQuery();

                string filePath = Path.Combine(_sk.AttachmentFolder, GetNameById(item.Folder.Value));
                filePath = Path.Combine(filePath, GetNameById(item.File.Value) + Path.GetExtension(item.FileName));
                if (File.Exists(filePath))
                {
                    File.Delete(filePath);
                }

                if (Directory.GetFiles(Path.GetDirectoryName(filePath)).Length == 0)
                {
                    Directory.Delete(Path.GetDirectoryName(filePath));
                }

                cmd.Parameters["?U"].Value = true;
                cmd.ExecuteNonQuery();
            }
        }

        [Test]
        public void DeleteAttachmentWithUpdateOwnNotUpdateInDocType()
        {
            MySqlCommand cmd = new MySqlCommand(
                "update DocTypePermission set U=?U where DocType=@docType2 and DcRole=@userRole;",
                _con.MySqlConnection
                );
            cmd.Parameters.Add("?U", MySqlDbType.Bit);
            cmd.Parameters["?U"].Value = false;
            cmd.ExecuteNonQuery();

            AttachmentAccess aa = new AttachmentAccess(_const.DocCircAdmin, _con, _sk);
            AttachmentItem item = new AttachmentItem(
                _const.DocCard2,
                "AttachmentAccessTest.file3.txt",
                "Content3",
                49,
                new DateTime(2007, 06, 04, 14, 15, 00)
                );
            Stream file3 = null;
            try
            {
                file3 = File.OpenRead(Path.Combine(FixtureFile.Folder, FixtureFile.Attachment3));
                aa.SaveAttachment(item, file3);
                aa = new AttachmentAccess(_const.DocCircUser2, _con, _sk);
                aa.DeleteAttachment(item); // tested method
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                file3.Close();

                MySqlCommand cmdDel = new MySqlCommand(
                    "delete from attachment where AttachmentId=?Attachment",
                    _con.MySqlConnection
                    );
                cmdDel.Parameters.Add("?Attachment", MySqlDbType.Int64);
                cmdDel.Parameters["?Attachment"].Value = item.Id;
                cmdDel.ExecuteNonQuery();

                string filePath = Path.Combine(_sk.AttachmentFolder, GetNameById(item.Folder.Value));
                filePath = Path.Combine(filePath, GetNameById(item.File.Value) + Path.GetExtension(item.FileName));
                if (File.Exists(filePath))
                {
                    File.Delete(filePath);
                }

                if (Directory.GetFiles(Path.GetDirectoryName(filePath)).Length == 0)
                {
                    Directory.Delete(Path.GetDirectoryName(filePath));
                }

                cmd.Parameters["?U"].Value = true;
                cmd.ExecuteNonQuery();
            }
        }

        [Test]
        public void BugGetDocCardAttachmentItemsIfNewDocCard()
        {
            DocCardAccess dca = new DocCardAccess(_const.DocCircAdmin, _con, _sk);
            DocCardItem dcItem = dca.CreateDocCard(_const.TestDocumentFolder1);
            AttachmentAccess aa = new AttachmentAccess(_const.DocCircAdmin, _con, _sk);

            try
            {
                AttachmentItem[] attItems = aa.GetDocCardAttachmentItems(dcItem.Id);
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                dca.DeleteDocCard(dcItem.Id);
            }
        }

        [TestFixtureTearDown]
        public void TFTearDown()
        {
            _tran.Rollback();
            _con.Close();

            DeleteFolder(_sk.AttachmentFolder);
        }

        private void DeleteFolder(string path)
        {
            foreach (string dir in Directory.GetDirectories(path))
            {
                DeleteFolder(dir);
            }

            foreach (string file in Directory.GetFiles(path))
            {
                File.Delete(file);
            }

            Directory.Delete(path);
        }

    }
}
