//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using MySql.Data.MySqlClient;
using NUnit.Framework;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.MySqlDL;

namespace Evpo.DocCircUnitTest
{
    [TestFixture]
    public class DBConstAccessTest
    {
        protected NotificationTypeDirectory _notificationTypeDir;
        protected EventTypeDirectory _eventTypeDir;

        private int _documentFolderType, _docTypeFolderType, _documentRootFolder, _docTypeRootFolder,
            _administratorsRoleId, _usersRoleId, _decisionRouteItemType, _notificationRouteItemType;
        private int _notSavedDocStatus, _draftDocStatus, _underConsDocStatus, _finalizedDocStatus;


        DBConnection _dbCon;
        [TestFixtureSetUp]
        public void FSetUp()
        {
            _dbCon = new DBConnection(
               "Database=EvpoDCTest;Data Source=localhost;User Id=doccirctest;Password=doccirctest"
               );
            _dbCon.Open();
            
            MySqlCommand cmd = new MySqlCommand(
                "select FolderTypeId, Name from FolderType",
                _dbCon.MySqlConnection
                );
            MySqlDataReader reader = cmd.ExecuteReader();
            while (reader.Read())
            {
                switch (reader.GetString("Name"))
                {
                    case "Document":
                        _documentFolderType = reader.GetInt32("FolderTypeId");
                        break;
                    case "DocType":
                        _docTypeFolderType = reader.GetInt32("FolderTypeId");
                        break;
                }
            }
            reader.Close();

            cmd.CommandText = "select FolderId, FolderType from Folder where IsNull(ParentFolder)";
            reader = cmd.ExecuteReader();
            while (reader.Read())
            {
                if (reader.GetInt32("FolderType") == _documentFolderType)
                {
                    _documentRootFolder = reader.GetInt32("FolderId");
                }
                else if (reader.GetInt32("FolderType") == _docTypeFolderType)
                {
                    _docTypeRootFolder = reader.GetInt32("FolderId");
                }
            }
            reader.Close();

            cmd.CommandText = "select DcRoleId from DcRole where RoleName=?RoleName";
            cmd.Parameters.Add("?RoleName", MySqlDbType.String);
            cmd.Parameters["?RoleName"].Value = "Administrators";

            reader = cmd.ExecuteReader();
            if (reader.Read())
            {
                _administratorsRoleId = reader.GetInt32("DcRoleId");
                reader.Close();
            }
            

            cmd.Parameters["?RoleName"].Value = "Users";
            reader = cmd.ExecuteReader();
            if (reader.Read())
            {
                _usersRoleId = reader.GetInt32("DcRoleId");
                reader.Close();
            }

            cmd.CommandText =
                "select @decisionRouteItemType:=RouteItemTypeId from RouteItemType where Name='Decision';" +
                "select @notificationRouteItemType:=RouteItemTypeId from RouteItemType where Name='Notification';";
            cmd.ExecuteNonQuery();
            cmd.CommandText = "select cast(@decisionRouteItemType as unsigned) as DecisionRouteItemType, " +
                "cast(@notificationRouteItemType as unsigned) as  NotificationRouteItemType";
            reader = cmd.ExecuteReader();
            if (reader.Read())
            {
                _decisionRouteItemType = reader.GetInt32("DecisionRouteItemType");
                _notificationRouteItemType = reader.GetInt32("NotificationRouteItemType");
            }
            reader.Close();

            cmd.CommandText =
                "select @notSavedDocStatus:=DocStatusId from DocStatus where Name='NotSavedDocStatus';" +
                "select @draftDocStatus:=DocStatusId from DocStatus where Name='DraftDocStatus';" +
                "select @underConsDocStatus:=DocStatusId from DocStatus where Name='UnderConsDocStatus';" +
                "select @finalizedDocStatus:=DocStatusId from DocStatus where Name='FinalizedDocStatus';";
            cmd.ExecuteNonQuery();

            cmd.CommandText =
                "select cast(@notSavedDocStatus as unsigned) as NotSavedDocStatus, " +
                "cast(@draftDocStatus as unsigned) as DraftDocStatus, " +
                "cast(@underConsDocStatus as unsigned) as UnderConsDocStatus, " +
                "cast(@finalizedDocStatus as unsigned) as FinalizedDocStatus; ";
            reader = cmd.ExecuteReader();
            if (reader.Read())
            {
                _notSavedDocStatus = reader.GetInt32("NotSavedDocStatus");
                _draftDocStatus = reader.GetInt32("DraftDocStatus");
                _underConsDocStatus = reader.GetInt32("UnderConsDocStatus");
                _finalizedDocStatus = reader.GetInt32("FinalizedDocStatus");
            }
            reader.Close();

            // notification type directory
            _notificationTypeDir = new NotificationTypeDirectory();
            cmd.CommandText = "select NotificationTypeId, Name from NotificationType;";
            reader = cmd.ExecuteReader();
            while (reader.Read())
            {
                int id = reader.GetInt32("NotificationTypeId");
                switch (reader.GetString("Name"))
                {
                    case "NextDecision":
                        _notificationTypeDir.NextDecision = id;
                        break;
                    case "DeputyNextDecision":
                        _notificationTypeDir.DeputyNextDecision = id;
                        break;
                    case "NextNotification":
                        _notificationTypeDir.NextNotification = id;
                        break;
                    case "DecisionAfterReturn":
                        _notificationTypeDir.DecisionAfterReturn = id;
                        break;
                    case "NotificationAfterReturn":
                        _notificationTypeDir.NotificationAfterReturn = id;
                        break;
                    case "Finalization":
                        _notificationTypeDir.Finalization = id;
                        break;
                    case "ReturnToDraftDueToDecision":
                        _notificationTypeDir.ReturnToDraftDueToDecision = id;
                        break;
                    case "ReturnToDraftDueToRouteChange":
                        _notificationTypeDir.ReturnToDraftDueToRouteChange = id;
                        break;
                    case "AcquaintanceRequest":
                        _notificationTypeDir.AcquaintanceRequest = id;
                        break;
                }
            }
            reader.Close();

            // event type directory
            _eventTypeDir = new EventTypeDirectory();
            cmd.CommandText = "select EventTypeId, Name from EventType;";
            reader = cmd.ExecuteReader();
            while (reader.Read())
            {
                int id = reader.GetInt32("EventTypeId");
                switch (reader.GetString("Name"))
                {
                    case "PassDecision":
                        _eventTypeDir.PassDecision = id;
                        break;
                    case "PassDecisionAutomatically":
                        _eventTypeDir.PassDecisionAutomatically = id;
                        break;
                    case "PassDecisionByDeputy":
                        _eventTypeDir.PassDecisionByDeputy = id;
                        break;
                    case "RejectDecision":
                        _eventTypeDir.RejectDecision = id;
                        break;
                    case "RejectDecisionByDeputy":
                        _eventTypeDir.RejectDecisionByDeputy = id;
                        break;
                    case "RejectDecisionByAdministrator":
                        _eventTypeDir.RejectDecisionByAdministrator = id;
                        break;
                    case "PostponeDecision":
                        _eventTypeDir.PostponeDecision = id;
                        break;
                    case "PostponeDecisionByDeputy":
                        _eventTypeDir.PostponeDecisionByDeputy = id;
                        break;
                    case "PostponeDecisionByAdministrator":
                        _eventTypeDir.PostponeDecisionByAdministrator = id;
                        break;
                    case "ReturnToDraftDueToRouteChange":
                        _eventTypeDir.ReturnToDraftDueToRouteChange = id;
                        break;
                    case "DocCardRegistration":
                        _eventTypeDir.DocCardRegistration = id;
                        break;
                    case "DocCardFinalization":
                        _eventTypeDir.DocCardFinalization = id;
                        break;
                    case "ConsiderationStarted":
                        _eventTypeDir.ConsiderationStarted = id;
                        break;
                    case "PassedToNextRoutePoint":
                        _eventTypeDir.PassedToNextRoutePoint = id;
                        break;
                    default:
                        throw new ApplicationException(
                            String.Format("{0} is unknown event type", reader.GetString("Name"))
                            );
                }
            }
            reader.Close();
        }

        [Test]
        public void GetFolderTypeDirectory()
        {
            DBConstAccess dba = new DBConstAccess(_dbCon);
            FolderTypeDirectory ftd = dba.GetFolderTypeDirectory();
            Assert.IsNotNull(ftd);
            Assert.AreEqual(_documentFolderType, ftd.Document);
            Assert.AreEqual(_docTypeFolderType, ftd.DocType);
        }

        [Test]
        public void GetRootFolderDirectory()
        {
            DBConstAccess dba = new DBConstAccess(_dbCon);
            RootFolderDirectory rfd = dba.GetRootFolderDirectory(
                dba.GetFolderTypeDirectory()
                );
            Assert.IsNotNull(rfd);
            Assert.AreEqual(_documentRootFolder, rfd.Document);
            Assert.AreEqual(_docTypeRootFolder, rfd.DocType);

        }

        [Test]
        public void GetMainRoleDirectory()
        {
            DBConstAccess dbca = new DBConstAccess(_dbCon);
            MainRoleDirectory mrd = dbca.GetMainRoleDirectory();

            Assert.IsNotNull(mrd);
            Assert.AreEqual(_usersRoleId, mrd.Users);
            Assert.AreEqual(_administratorsRoleId, mrd.Administrators);
            
        }

        [Test]
        public void GetRouteItemTypeDirectory()
        {
            DBConstAccess dbca = new DBConstAccess(_dbCon);
            RouteItemTypeDirectory ritd = dbca.GetRouteItemTypeDirectory();

            Assert.IsNotNull(ritd);
            Assert.AreEqual(_decisionRouteItemType, ritd.Decision);
            Assert.AreEqual(_notificationRouteItemType, ritd.Notification);
        }

        [Test]
        public void GetDocStatusDirectory()
        {
            DBConstAccess dbca = new DBConstAccess(_dbCon);
            DocStatusDirectory dir = dbca.GetDocStatusDirectory();

            Assert.IsNotNull(dir);

            Assert.AreEqual(_notSavedDocStatus, dir.NotSaved);
            Assert.AreEqual(_draftDocStatus, dir.Draft);
            Assert.AreEqual(_underConsDocStatus, dir.UnderCons);
            Assert.AreEqual(_finalizedDocStatus, dir.Finalized);
        }

        [Test]
        public void GetNotificationTypeDirectory()
        {
            DBConstAccess dbca = new DBConstAccess(_dbCon);
            NotificationTypeDirectory dir = dbca.GetNotificationTypeDirectory();

            Assert.IsNotNull(dir);

            Assert.AreEqual(_notificationTypeDir.DecisionAfterReturn, dir.DecisionAfterReturn);
            Assert.AreEqual(_notificationTypeDir.Finalization, dir.Finalization);
            Assert.AreEqual(_notificationTypeDir.NextDecision, dir.NextDecision);
            Assert.AreEqual(_notificationTypeDir.DeputyNextDecision, dir.DeputyNextDecision);
            Assert.AreEqual(_notificationTypeDir.NextNotification, dir.NextNotification);
            Assert.AreEqual(_notificationTypeDir.NotificationAfterReturn, dir.NotificationAfterReturn);
            Assert.AreEqual(_notificationTypeDir.ReturnToDraftDueToDecision, dir.ReturnToDraftDueToDecision);
            Assert.AreEqual(_notificationTypeDir.ReturnToDraftDueToRouteChange, dir.ReturnToDraftDueToRouteChange);
            Assert.AreEqual(_notificationTypeDir.AcquaintanceRequest, dir.AcquaintanceRequest);
        }

        [Test]
        public void GetEventTypeDirectory()
        {
            DBConstAccess dbca = new DBConstAccess(_dbCon);
            EventTypeDirectory dir = dbca.GetEventTypeDirectory();

            Assert.IsNotNull(dir);

            Assert.AreEqual(_eventTypeDir.PassDecision, dir.PassDecision);
            Assert.AreEqual(_eventTypeDir.PassDecisionAutomatically, dir.PassDecisionAutomatically);
            Assert.AreEqual(_eventTypeDir.PassDecisionByDeputy, dir.PassDecisionByDeputy);
            Assert.AreEqual(_eventTypeDir.RejectDecision, dir.RejectDecision);
            Assert.AreEqual(_eventTypeDir.RejectDecisionByDeputy, dir.RejectDecisionByDeputy);
            Assert.AreEqual(_eventTypeDir.RejectDecisionByAdministrator, dir.RejectDecisionByAdministrator);
            Assert.AreEqual(_eventTypeDir.ReturnToDraftDueToRouteChange, dir.ReturnToDraftDueToRouteChange);
            Assert.AreEqual(_eventTypeDir.DocCardRegistration, dir.DocCardRegistration);
            Assert.AreEqual(_eventTypeDir.DocCardFinalization, dir.DocCardFinalization);
            Assert.AreEqual(_eventTypeDir.ConsiderationStarted, dir.ConsiderationStarted);
            Assert.AreEqual(_eventTypeDir.PassedToNextRoutePoint, dir.PassedToNextRoutePoint);
        }

        [Test]
        public void GetCommonDepartment()
        {
            DBConstAccess dbca = new DBConstAccess(_dbCon);
            int commonDept = dbca.GetCommonDepartment();

            Assert.GreaterOrEqual(commonDept, 0);

            MySqlCommand cmd = new MySqlCommand(
                "select DepartmentId from Department where Name='Common';",
                _dbCon.MySqlConnection
                );

            Assert.AreEqual((int)cmd.ExecuteScalar(), commonDept);
        }

        [TestFixtureTearDown]
        public void FTearDown()
        {
            _dbCon.Close();
        }

    }
}
