//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Data;
using MySql.Data.MySqlClient;
using NUnit.Framework;
using Evpo.DocCirc.MySqlDL;
using Evpo.DocCirc.Common;

namespace Evpo.DocCircUnitTest
{
    [TestFixture]
    public class DocCardAccessTest
    {
        public struct FixtureFile
        {
            public const string Folder = "MySql";
            public const string Attachment2 = "DocCardAccessTest.file2.txt";
            public const string SqlFile1 = "DocCardAccessTest.file1.sql";
        }
        
        
        private void DeleteFolder(string path)
        {
            foreach (string dir in Directory.GetDirectories(path))
            {
                DeleteFolder(dir);
            }

            foreach (string file in Directory.GetFiles(path))
            {
                File.Delete(file);
            }

            Directory.Delete(path);
        }

        private int _namingBase = 36;
        private int _nameLength = 13;

        private string ConvertToBase(long num, int nbase)
        {
            String chars = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";

            // check if we can convert to another base
            if (nbase < 2 || nbase > chars.Length)
                return "";

            int r;
            String newNumber = "";

            // in r we have the offset of the char that was converted to the new base
            while (num >= nbase)
            {
                r = (int)(num % nbase);
                newNumber = chars[r] + newNumber;
                num = num / nbase;
            }
            // the last number to convert
            newNumber = chars[(int)num] + newNumber;

            return newNumber;
        }

        private long ConvertFromBase(string num, int nbase)
        {
            String chars = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
            SortedList<char, int> charList = new SortedList<char, int>(chars.Length);
            for (int i = 0; i < chars.Length; i++)
            {
                charList.Add(chars[i], i);
            }

            long retVal = 0;
            for (int i = 0; i < num.Length; i++)
            {
                char ch = num[num.Length - i - 1];
                int val = charList[ch];
                retVal += (long)(Math.Pow(nbase, i) * val);
            }
            return retVal;
        }

        private string GetNameById(long id)
        {
            return ConvertToBase(id, _namingBase).PadLeft(_nameLength, '0');
        }


        public class Const
        {
            public int DocumentFolderType, RootDocumentFolder, TestDocumentFolder1;
            public int DocCircAdmin, DocCircUser1, DocCircUser2, DocCircUser3;
            public int DocType1, DocType2;
            public long DocCard1, DocCard2, DocCard3, DocCard4, DocCard5;
            public int Institution1, Institution2;
        }
        private Const _const = new Const();

        //private const string _sqlFile1Path = @"Evpo.DocCirc.MySql\";
        private DBConnection _con;
        private MySqlTransaction _tran;
        private SettingKeeper _sk;

        [TestFixtureSetUp]
        public void TFSetUp()
        {
            //TextReader tr = File.OpenText(Path.Combine(FixtureFile.Folder, FixtureFile.SqlFile1));
            //string scriptText = tr.ReadToEnd();
            //tr.Close();
            string scriptText = MySqlResources.DocCardAccessTest_file1;
            _con = new DBConnection(
              "Database=EvpoDCTest;Data Source=localhost;User Id=doccirctest;Password=doccirctest"
              );
            _con.Open();
            _tran = _con.MySqlConnection.BeginTransaction();

            MySqlCommand cmd = new MySqlCommand(
                scriptText,
                _con.MySqlConnection
                );
            cmd.ExecuteNonQuery();

            cmd.CommandText =
                "select cast(@documentFolderType as unsigned) as DocumentFolderType, " +
                "cast(@rootDocumentFolder as unsigned) as RootDocumentFolder, " +
                "cast(@docCircAdmin as unsigned) as DocCircAdmin, " +
                "cast(@docCircUser1 as unsigned) as DocCircUser1, " +
                "cast(@docCircUser2 as unsigned) as DocCircUser2, " +
                "cast(@docCircUser3 as unsigned) as DocCircUser3, " +
                "cast(@docType1 as unsigned) as DocType1, " +
                "cast(@docType2 as unsigned) as DocType2, " +
                "cast(@docCard1 as unsigned) as DocCard1, " +
                "cast(@docCard2 as unsigned) as DocCard2, " +
                "cast(@docCard3 as unsigned) as DocCard3, " +
                "cast(@docCard4 as unsigned) as DocCard4, " +
                "cast(@docCard5 as unsigned) as DocCard5, " +
                "cast(@testDocumentFolder1 as unsigned) as TestDocumentFolder1, " +
                "cast(@institution1 as unsigned) as Institution1, " +
                "cast(@institution2 as unsigned) as institution2; ";


            MySqlDataReader reader = cmd.ExecuteReader();
            if (reader.Read())
            {
                _const.DocumentFolderType = reader.GetInt32("DocumentFolderType");
                _const.RootDocumentFolder = reader.GetInt32("RootDocumentFolder");
                _const.TestDocumentFolder1 = reader.GetInt32("TestDocumentFolder1");
                
                _const.DocCircAdmin = reader.GetInt32("DocCircAdmin");
                _const.DocCircUser1 = reader.GetInt32("DocCircUser1");
                _const.DocCircUser2 = reader.GetInt32("DocCircUser2");
                _const.DocCircUser3 = reader.GetInt32("DocCircUser3");

                _const.DocType1 = reader.GetInt32("DocType1");
                _const.DocType2 = reader.GetInt32("DocType2");
                _const.DocCard1 = reader.GetInt64("DocCard1");
                _const.DocCard2 = reader.GetInt64("DocCard2");
                _const.DocCard3 = reader.GetInt64("DocCard3");
                _const.DocCard4 = reader.GetInt64("DocCard4");
                _const.DocCard5 = reader.GetInt64("DocCard5");

                _const.Institution1 = reader.GetInt32("Institution1");
                _const.Institution2 = reader.GetInt32("Institution2");
            }
            reader.Close();

            DBConstAccess ca = new DBConstAccess(_con);
            FolderTypeDirectory ftd = ca.GetFolderTypeDirectory();
            DBConstKeeper.Current = new DBConstKeeper(
                ftd,
                ca.GetRootFolderDirectory(ftd),
                ca.GetMainRoleDirectory(),
                ca.GetRouteItemTypeDirectory(),
                ca.GetDocStatusDirectory(),
                ca.GetNotificationTypeDirectory(),
                ca.GetEventTypeDirectory(),
                ca.GetCommonDepartment()
                );
            DBFieldDefKeeper.Current = new DBFieldDefKeeper("");

            _sk = new SettingKeeper();

            _sk.MaxFilesInFolder = 2;
            _sk.AttachmentBufferSize = 65536;
            _sk.AttachmentFolder = Path.Combine(
                Directory.GetCurrentDirectory(),
                "attachments"
                );

            if (Directory.Exists(_sk.AttachmentFolder))
            {
                DeleteFolder(_sk.AttachmentFolder);
            }

            Directory.CreateDirectory(_sk.AttachmentFolder);
        }

        [Test]
        [ExpectedException(ExpectedException=typeof(DcUnathorizedAccessException))]
        public void CreateDocCardWithoutAccessInFolder()
        {
            MySqlCommand cmd = new MySqlCommand(
                "update FolderPermission set C=?C where Folder=@testDocumentFolder1 and DcRole=@userRole;",
                _con.MySqlConnection
                );
            cmd.Parameters.Add("?C", MySqlDbType.Bit);
            cmd.Parameters["?C"].Value = false;
            cmd.ExecuteNonQuery();

            DocCardAccess dca = new DocCardAccess(_const.DocCircUser1, _con, _sk);

            try
            {
                DocCardItem item = dca.CreateDocCard(_const.TestDocumentFolder1); // tested method
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                cmd.Parameters["?C"].Value = true;
                cmd.ExecuteNonQuery();
            }
        }

        [Test]
        [ExpectedException(ExpectedException = typeof(DcUnathorizedAccessException))]
        public void CreateDocCardWithoutAccessInDocType()
        {
            MySqlCommand cmd = new MySqlCommand(
                "update DocTypePermission set C=?C where DocType=@docType2 and DcRole=@userRole;",
                _con.MySqlConnection
                );
            cmd.Parameters.Add("?C", MySqlDbType.Bit);
            cmd.Parameters["?C"].Value = false;
            cmd.ExecuteNonQuery();

            DocCardAccess dca = new DocCardAccess(_const.DocCircUser1, _con, _sk);
            DocCardItem item = dca.CreateDocCard(_const.TestDocumentFolder1);
            item.DocType = _const.DocType2;
            item.DocStatus = DBConstKeeper.Current.DocStatus.Draft;

            try
            {
                dca.SaveDocCard(item); // tested method
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                cmd.Parameters["?C"].Value = true;
                cmd.ExecuteNonQuery();
                dca = new DocCardAccess(_const.DocCircAdmin, _con, _sk);
                dca.DeleteDocCard(item.Id);
            }
        }

        [Test]
        public void CreateDocCard()
        {
            DocCardAccess dca = new DocCardAccess(_const.DocCircAdmin, _con, _sk);
            DocCardItem item = dca.CreateDocCard(_const.RootDocumentFolder); // tested method

            Assert.IsNotNull(item);

            MySqlCommand cmd = new MySqlCommand(
                "select DocCardId, Folder, DocType, DocStatus, RegisteringUser, " +
                "DcUser, RegistrationTime, AttachmentsAreChangeable, RoutePointNum " +
                "from DocCard where DocCardId=?DocCard;",
                _con.MySqlConnection
                );
            cmd.Parameters.Add("?DocCard", MySqlDbType.Int32);
            cmd.Parameters["?DocCard"].Value = item.Id;

            MySqlDataReader reader = cmd.ExecuteReader();

            try
            {
                Assert.IsTrue(reader.Read());
                Assert.AreEqual(item.Id, reader.GetInt32("DocCardId"), "DocCardId");
                Assert.AreEqual(_const.RootDocumentFolder, reader.GetInt32("Folder"), "Folder");
                Assert.IsTrue(reader.IsDBNull(2), "DocType");
                Assert.AreEqual(DBConstKeeper.Current.DocStatus.NotSaved, reader.GetInt32("DocStatus"), "DocStatus");
                Assert.AreEqual(_const.DocCircAdmin, reader.GetInt32("RegisteringUser"), "RegisteringUser");
                Assert.AreEqual(_const.DocCircAdmin, reader.GetInt32("DcUser"), "DcUser");
                Assert.AreEqual(DateTime.Today, reader.GetDateTime("RegistrationTime").Date, "RegistrationTime");
                Assert.IsTrue(reader.GetBoolean("AttachmentsAreChangeable"), "AttachmentsAreChengeable");
                Assert.AreEqual(0, reader.GetInt32("RoutePointNum"), "RoutePointNum");
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }
            }

        }

        [Test]
        [ExpectedException(ExpectedException = typeof(DcUnathorizedAccessException))]
        public void LoadDocCardWithoutAccessInFolder()
        {
            MySqlCommand cmd = new MySqlCommand(
               "update FolderPermission set R=?R where Folder=@testDocumentFolder1 and DcRole=@userRole;",
               _con.MySqlConnection
               );
            cmd.Parameters.Add("?R", MySqlDbType.Bit);
            cmd.Parameters["?R"].Value = false;
            cmd.ExecuteNonQuery();

            DocCardAccess dca = new DocCardAccess(_const.DocCircUser1, _con, _sk);

            try
            {
                bool loadRoute = true;
                DocCardItem item = dca.LoadDocCard(_const.DocCard2, loadRoute); // tested method
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                cmd.Parameters["?R"].Value = true;
                cmd.ExecuteNonQuery();
            }
        }

        [Test]
        [ExpectedException(ExpectedException = typeof(DcUnathorizedAccessException))]
        public void LoadDocCardWithoutAccessInDocType()
        {
            MySqlCommand cmd = new MySqlCommand(
                "update DocTypePermission set R=?R where DocType=@docType2 and DcRole=@userRole;",
                _con.MySqlConnection
                );
            cmd.Parameters.Add("?R", MySqlDbType.Bit);
            cmd.Parameters["?R"].Value = false;
            cmd.ExecuteNonQuery();

            DocCardAccess dca = new DocCardAccess(_const.DocCircUser1, _con, _sk);

            try
            {
                bool loadRoute = true;
                DocCardItem item = dca.LoadDocCard(_const.DocCard2, loadRoute); // tested method
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                cmd.Parameters["?R"].Value = true;
                cmd.ExecuteNonQuery();
            }
        }

        [Test]
        public void LoadDocCardWithReadOwnNotReadInDocType()
        {
            MySqlCommand cmd = new MySqlCommand(
                "update DocTypePermission set R=?R where DocType=@docType2 and DcRole=@userRole;",
                _con.MySqlConnection
                );
            cmd.Parameters.Add("?R", MySqlDbType.Bit);
            cmd.Parameters["?R"].Value = false;
            cmd.ExecuteNonQuery();

            DocCardAccess dca = new DocCardAccess(_const.DocCircUser2, _con, _sk);
            
            bool loadRoute = true;
            DocCardItem item = dca.LoadDocCard(_const.DocCard2, loadRoute); // tested method

            try
            {
                Assert.AreEqual(_const.DocCard2, item.Id);    
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                cmd.Parameters["?R"].Value = true;
                cmd.ExecuteNonQuery();
            }
        }

        [Test]
        public void LoadDocCardWithReadOwnNotReadInFolder()
        {
            MySqlCommand cmd = new MySqlCommand(
               "update FolderPermission set R=?R where Folder=@testDocumentFolder1 and DcRole=@userRole;",
               _con.MySqlConnection
               );
            cmd.Parameters.Add("?R", MySqlDbType.Bit);
            cmd.Parameters["?R"].Value = false;
            cmd.ExecuteNonQuery();

            DocCardAccess dca = new DocCardAccess(_const.DocCircUser2, _con, _sk);
            bool loadRoute = true;
            DocCardItem item = dca.LoadDocCard(_const.DocCard2, loadRoute); // tested method
            try
            {
                Assert.AreEqual(_const.DocCard2, item.Id);
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                cmd.Parameters["?R"].Value = true;
                cmd.ExecuteNonQuery();
            }
        }

        [Test]
        public void LoadDocCard()
        {
            DocCardAccess dca = new DocCardAccess(_const.DocCircAdmin, _con, _sk);
            bool loadRoute = true;
            DocCardItem item = dca.LoadDocCard(_const.DocCard1, loadRoute); // tested method

            Assert.IsNotNull(item);

            Assert.AreEqual(_const.DocCard1, item.Id);
            Assert.AreEqual(_const.DocType1, item.DocType);
            Assert.AreEqual(_const.RootDocumentFolder, item.Folder);
            Assert.AreEqual(DBConstKeeper.Current.DocStatus.UnderCons, item.DocStatus);
            Assert.AreEqual(_const.DocCircAdmin, item.RegisteringUser);
            Assert.AreEqual(_const.DocCircAdmin, item.Owner);
            Assert.AreEqual(new DateTime(2007, 5, 23, 12, 0, 0), item.LastMovement);
            Assert.AreEqual("Test DocCard1", item.Subject);
            Assert.AreEqual("TTF000001", item.InternalNumber);
            Assert.AreEqual(new DateTime(2007, 5, 24, 11, 35, 00), item.RegistrationTime);
            Assert.IsNull(item.FinalizingTime);
            Assert.IsFalse(item.AttachmentsAreChangeable);
            Assert.AreEqual(new DateTime(2007, 5, 1), item.OriginalDate);
            Assert.AreEqual("T21", item.OriginalNumber);
            Assert.AreEqual("MS", item.CustomSendingCompany);
            Assert.AreEqual("Evpo", item.CustomReceivingCompany);
            Assert.AreEqual(_const.Institution1, item.SendingCompany);
            Assert.IsNull(item.ReceivingCompany);
            Assert.AreEqual("this is the note of DocCard1", item.Note);
            Assert.AreEqual(2, item.RoutePointNum);
            Assert.IsFalse(item.IsNew);
            Assert.IsTrue(item.RouteIsLoaded);

            // route points
            Assert.AreEqual(3, item.RoutePoints.Length);

            Assert.AreEqual(1, item.RoutePoints[0].OrderNum);
            Assert.AreEqual(1, item.RoutePoints[0].RouteItems.Count);
            DocCardRouteItem ri = (DocCardRouteItem)item.RoutePoints[0].RouteItems[0];
            Assert.IsNotNull(ri.DocRouteItemId);
            Assert.AreEqual(_const.DocCircUser1, ri.User);
            Assert.IsTrue(ri.Passed);
            Assert.IsFalse(ri.UnderCons);

            Assert.AreEqual(2, item.RoutePoints[1].OrderNum);
            Assert.AreEqual(2, item.RoutePoints[1].RouteItems.Count);
            
            ri = (DocCardRouteItem)item.RoutePoints[1].RouteItems[0];
            Assert.IsNotNull(ri.DocRouteItemId);
            Assert.AreEqual(_const.DocCircUser2, ri.User);
            Assert.IsTrue(ri.Passed);
            Assert.IsFalse(ri.UnderCons);

            ri = (DocCardRouteItem)item.RoutePoints[1].RouteItems[1];
            Assert.IsNotNull(ri.DocRouteItemId);
            Assert.AreEqual(_const.DocCircUser3, ri.User);
            Assert.IsFalse(ri.Passed);
            Assert.IsTrue(ri.UnderCons);

            Assert.AreEqual(3, item.RoutePoints[2].OrderNum);
            ri = (DocCardRouteItem)item.RoutePoints[2].RouteItems[0];
            Assert.IsNull(ri.DocRouteItemId);
            Assert.AreEqual(_const.DocCircAdmin, ri.User);
            Assert.IsFalse(ri.Passed);
            Assert.IsFalse(ri.UnderCons);
        }

        [Test]
        [ExpectedException(ExpectedException = typeof(DcUnathorizedAccessException))]
        public void SaveDocCardWithoutUpdateInFolder()
        {
            MySqlCommand cmd = new MySqlCommand(
               "update FolderPermission set U=?U where Folder=@testDocumentFolder1 and DcRole=@userRole;",
               _con.MySqlConnection
               );
            cmd.Parameters.Add("?U", MySqlDbType.Bit);
            cmd.Parameters["?U"].Value = false;
            cmd.ExecuteNonQuery();

            DocCardAccess dca = new DocCardAccess(_const.DocCircUser1, _con, _sk);
            
            bool loadRoute = true;
            DocCardItem item = dca.LoadDocCard(_const.DocCard2, loadRoute);

            try
            {
                dca.SaveDocCard(item); // tested method
                
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                cmd.Parameters["?U"].Value = true;
                cmd.ExecuteNonQuery();
            }
        }

        [Test]
        public void SaveDocCardWithUpdateOwnNotUpdateInFolder()
        {
            MySqlCommand cmd = new MySqlCommand(
               "update FolderPermission set U=?U where Folder=@testDocumentFolder1 and DcRole=@userRole;",
               _con.MySqlConnection
               );
            cmd.Parameters.Add("?U", MySqlDbType.Bit);
            cmd.Parameters["?U"].Value = false;
            cmd.ExecuteNonQuery();

            DocCardAccess dca = new DocCardAccess(_const.DocCircUser2, _con, _sk);

            bool loadRoute = true;
            DocCardItem item = dca.LoadDocCard(_const.DocCard2, loadRoute);

            try
            {
                dca.SaveDocCard(item); // tested method
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                cmd.Parameters["?U"].Value = true;
                cmd.ExecuteNonQuery();
            }
        }

        [Test]
        [ExpectedException(ExpectedException = typeof(DcUnathorizedAccessException))]
        public void SaveDocCardWithoutUpdateInDocType()
        {
            MySqlCommand cmd = new MySqlCommand(
                "update DocTypePermission set U=?U where DocType=@docType2 and DcRole=@userRole;",
                _con.MySqlConnection
                );
            cmd.Parameters.Add("?U", MySqlDbType.Bit);
            cmd.Parameters["?U"].Value = false;
            cmd.ExecuteNonQuery();

            DocCardAccess dca = new DocCardAccess(_const.DocCircUser1, _con, _sk);

            bool loadRoute = true;
            DocCardItem item = dca.LoadDocCard(_const.DocCard2, loadRoute);

            try
            {
                dca.SaveDocCard(item); // tested method
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                cmd.Parameters["?U"].Value = true;
                cmd.ExecuteNonQuery();
            }
        }

        [Test]
        public void SaveDocCardWithUpdateOwnNotUpdateInDocType()
        {
            MySqlCommand cmd = new MySqlCommand(
                "update DocTypePermission set U=?U where DocType=@docType2 and DcRole=@userRole;",
                _con.MySqlConnection
                );
            cmd.Parameters.Add("?U", MySqlDbType.Bit);
            cmd.Parameters["?U"].Value = false;
            cmd.ExecuteNonQuery();

            DocCardAccess dca = new DocCardAccess(_const.DocCircUser2, _con, _sk);

            bool loadRoute = true;
            DocCardItem item = dca.LoadDocCard(_const.DocCard2, loadRoute);

            try
            {
                dca.SaveDocCard(item); // tested method
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                cmd.Parameters["?U"].Value = true;
                cmd.ExecuteNonQuery();
            }
        }

        [Test]
        public void SaveDocCard()
        {
            DocCardAccess dca = new DocCardAccess(_const.DocCircAdmin, _con, _sk);

            DocCardItem item = dca.CreateDocCard(_const.TestDocumentFolder1);
            item.DocType = _const.DocType2;
            item.DocStatus = DBConstKeeper.Current.DocStatus.UnderCons;
            item.RegisteringUser = _const.DocCircUser3;
            item.Owner = _const.DocCircAdmin;
            item.LastMovement = new DateTime(2007, 07, 11, 13, 00, 00);
            item.Subject = "SaveDocCard testing";
            item.InternalNumber = "TTS000003";
            item.RegistrationTime = new DateTime(2007, 07, 11, 12, 00, 00);
            item.FinalizingTime = null;
            item.AttachmentsAreChangeable = true;
            item.OriginalDate = null;
            item.OriginalNumber = "T24";
            item.CustomSendingCompany = "HOUSE1";
            item.CustomReceivingCompany = "HOUSE2";
            item.SendingCompany = _const.Institution1;
            item.ReceivingCompany = _const.Institution2;
            item.Note = "this is the note of a new saved doc card";
            item.RoutePointNum = 2;
            
            item.IsNew = false;
            

            MySqlCommand cmd = new MySqlCommand(
                "select rp.RoutePointId, rp.OrderNum, rp.Expression, " +
                "ri.RouteItemId, ri.DcUser, ri.RouteItemType, ri.ItemIndex " +
                "from RoutePoint as rp inner join RouteItem as ri " +
                "on ri.RoutePoint=rp.RoutePointId and rp.DocType=@docType2 " +
                "order by rp.OrderNum, ri.ItemIndex;",
                _con.MySqlConnection
                );

            MySqlDataReader reader = cmd.ExecuteReader();
            RoutePoint[] routePoints = new RoutePoint[3];
            //route point 1
            reader.Read();
            routePoints[0] = new RoutePoint(
                reader.GetInt32("RoutePointId"),
                reader.GetString("Expression"),
                reader.GetInt32("OrderNum")
                );
            RouteItem ri = new RouteItem(
                reader.GetInt32("RouteItemId"),
                reader.GetInt32("DcUser"),
                reader.GetInt32("RouteItemType")
                );
            routePoints[0].RouteItems.Add(
                new DocCardRouteItem(null, true, false, ri)
                );
            
            //route point 2
            reader.Read();
            routePoints[1] = new RoutePoint(
                reader.GetInt32("RoutePointId"),
                reader.GetString("Expression"),
                reader.GetInt32("OrderNum")
                );
            ri = new RouteItem(
               reader.GetInt32("RouteItemId"),
               reader.GetInt32("DcUser"),
               reader.GetInt32("RouteItemType")
               );
            routePoints[1].RouteItems.Add(
                new DocCardRouteItem(null, false, true, ri)
                );

            //route point 3
            reader.Read();
            routePoints[2] = new RoutePoint(
                reader.GetInt32("RoutePointId"),
                reader.GetString("Expression"),
                reader.GetInt32("OrderNum")
                );
            ri = new RouteItem(
               reader.GetInt32("RouteItemId"),
               reader.GetInt32("DcUser"),
               reader.GetInt32("RouteItemType")
               );
            routePoints[2].RouteItems.Add(
                new DocCardRouteItem(null, false, false, ri)
                );
            reader.Close();
            item.RoutePoints = routePoints;
            item.SetRouteIsLoaded(true);

            dca.SaveDocCard(item); // tested method

            cmd.CommandText =
                "select Folder, DocType, DocStatus, RegisteringUser, DcUser, LastMovement, Subject, " +
                "InternalNumber, RegistrationTime, FinalizingTime, AttachmentsAreChangeable, " +
                "OriginalDate, OriginalNumber, CustomSendingCompany, CustomReceivingCompany, Note, " +
                "RoutePointNum, IsNew, SendingCompany, ReceivingCompany from DocCard where DocCardId=?DocCard;";
            cmd.Parameters.Add("?DocCard", MySqlDbType.Int64);
            cmd.Parameters["?DocCard"].Value = item.Id;
            
            reader = cmd.ExecuteReader();

            try
            {
                Assert.IsTrue(reader.Read(), "DocCard reader doesn't read!!");
                Assert.IsFalse(reader.GetBoolean("IsNew"), "IsNew");
                Assert.AreEqual(_const.TestDocumentFolder1, reader.GetInt32("Folder"), "Folder");
                Assert.AreEqual(_const.DocType2, reader.GetInt32("DocType"), "DocType");
                Assert.AreEqual(
                    DBConstKeeper.Current.DocStatus.UnderCons,
                    reader.GetInt32("DocStatus")
                    );
                Assert.AreEqual(_const.DocCircUser3, reader.GetInt32("RegisteringUser"), "RegisteringUser");
                Assert.AreEqual(_const.DocCircAdmin, reader.GetInt32("DcUser"), "DcUser");
                Assert.AreEqual(
                    new DateTime(2007, 07, 11, 13, 00, 00), 
                    reader.GetDateTime("LastMovement"), 
                    "LastMovement"
                    );
                Assert.AreEqual("SaveDocCard testing", reader.GetString("Subject"), "Subject");
                Assert.AreEqual("TTS000003", reader.GetString("InternalNumber"), "InternalNumber");
                Assert.AreEqual(
                    new DateTime(2007, 07, 11, 12, 00, 00),
                    reader.GetDateTime("RegistrationTime"), 
                    "RegistrationTime"
                    );
                Assert.IsTrue(reader.IsDBNull(9), "FinalizingTime");
                Assert.IsTrue(reader.GetBoolean("AttachmentsAreChangeable"), "AttachmentsAreChangeable");
                Assert.IsTrue(reader.IsDBNull(11), "OriginalDate");
                Assert.AreEqual("T24", reader.GetString("OriginalNumber"), "OriginalNumber");
                Assert.AreEqual("HOUSE1", reader.GetString("CustomSendingCompany"), "CustomSendingCompany");
                Assert.AreEqual("HOUSE2", reader.GetString("CustomReceivingCompany"), "CustomReceivingCompany");
                Assert.AreEqual(_const.Institution1, reader.GetInt32("SendingCompany"), "SendingCompany");
                Assert.AreEqual(_const.Institution2, reader.GetInt32("ReceivingCompany"), "ReceivingCompany");
                Assert.AreEqual("this is the note of a new saved doc card", reader.GetString("Note"), "Note");
                Assert.AreEqual(2, reader.GetInt32("RoutePointNum"), "RoutePointNum");
                reader.Close();

                // check doc card route items
                cmd.CommandText =
                    "select DocCardRouteItemId, RouteItem, Passed from DocCardRouteItem " +
                    "where DocCard=?DocCard;";
                reader = cmd.ExecuteReader();
                Assert.IsTrue(reader.Read(), "DocCardRouteItem reader doesn't read!");
                bool[] been = new bool[] { false, false };
                do
                {
                    if (reader.GetInt32("RouteItem") == item.RoutePoints[0].RouteItems[0].Id && !been[0])
                    {
                        been[0] = true;
                        Assert.IsTrue(reader.GetBoolean("Passed"), "RouteItem on rp 1 passed is wrong!");
                        Assert.AreEqual(
                            ((DocCardRouteItem)item.RoutePoints[0].RouteItems[0]).DocRouteItemId,
                            reader.GetInt64("DocCardRouteItemId"),
                            "DocCardRouteItemId of route item on route point 1 is wrong!"
                            );
                    }
                    else if (reader.GetInt32("RouteItem") == item.RoutePoints[1].RouteItems[0].Id && !been[1])
                    {
                        been[1] = true;
                        Assert.IsFalse(reader.GetBoolean("Passed"), "RouteItem on rp 2 passed is wrong!");
                        Assert.AreEqual(
                            ((DocCardRouteItem)item.RoutePoints[1].RouteItems[0]).DocRouteItemId,
                            reader.GetInt64("DocCardRouteItemId"),
                            "DocCardRouteItemId of route item on route point 2 is wrong!"
                            );
                    }
                    else
                    {
                        Assert.Fail(
                            String.Format(
                                "Unknown or repeated doc card route item! Id={0}", 
                                reader.GetInt32("RouteItem")
                                )
                            );
                    }

                }
                while (reader.Read());
                reader.Close();
                Assert.IsTrue(been[0], "RoutePoint 1 routeItem not found!");
                Assert.IsTrue(been[1], "RoutePoint 2 routeItem not found!");
                
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }
            }

            // changes in existent DocCard
            bool loadRoute = true;
            item = dca.LoadDocCard(item.Id, loadRoute);

            item.RoutePointNum = 0;
            item.FinalizingTime = new DateTime(2007, 7, 30, 12, 00, 00);
            item.DocStatus = DBConstKeeper.Current.DocStatus.Finalized;
            item.SendingCompany = _const.Institution2;
            item.ReceivingCompany = _const.Institution1;

            ((DocCardRouteItem)item.RoutePoints[1].RouteItems[0]).UnderCons = false;
            ((DocCardRouteItem)item.RoutePoints[1].RouteItems[0]).Passed = true;
            ((DocCardRouteItem)item.RoutePoints[2].RouteItems[0]).UnderCons = false;
            ((DocCardRouteItem)item.RoutePoints[2].RouteItems[0]).Passed = true;

            dca.SaveDocCard(item); // tested method

            cmd.CommandText =
                "select Folder, DocType, DocStatus, RegisteringUser, DcUser, LastMovement, Subject, " +
                "InternalNumber, RegistrationTime, FinalizingTime, AttachmentsAreChangeable, " +
                "OriginalDate, OriginalNumber, CustomSendingCompany, CustomReceivingCompany, " +
                "RoutePointNum, IsNew, SendingCompany, ReceivingCompany from DocCard where DocCardId=?DocCard;";
            
            reader = cmd.ExecuteReader();

            try
            {
                Assert.IsTrue(reader.Read(), "DocCard reader doesn't read!!");
                
                Assert.AreEqual(0, reader.GetInt32("RoutePointNum"), "RoutePointNum");
                Assert.AreEqual(
                    new DateTime(2007, 7, 30, 12, 00, 00),
                    reader.GetDateTime("FinalizingTime"),
                    "FinalizingTime"
                    );
                Assert.AreEqual(
                    DBConstKeeper.Current.DocStatus.Finalized,
                    reader.GetInt32("DocStatus"),
                    "DocStatus"
                    );
                Assert.AreEqual(_const.Institution2, reader.GetInt32("SendingCompany"));
                Assert.AreEqual(_const.Institution1, reader.GetInt32("ReceivingCompany"));
                reader.Close();
                // check doc card route items
                cmd.CommandText =
                    "select DocCardRouteItemId, RouteItem, Passed from DocCardRouteItem " +
                    "where DocCard=?DocCard;";
                reader = cmd.ExecuteReader();
                Assert.IsTrue(reader.Read(), "DocCardRouteItem reader doesn't read!");
                bool[] been = new bool[] { false, false, false };
                do
                {
                    if (reader.GetInt32("RouteItem") == item.RoutePoints[0].RouteItems[0].Id && !been[0])
                    {
                        been[0] = true;
                        Assert.IsTrue(reader.GetBoolean("Passed"), "RouteItem on rp 1 passed is wrong!");
                        Assert.AreEqual(
                            ((DocCardRouteItem)item.RoutePoints[0].RouteItems[0]).DocRouteItemId,
                            reader.GetInt64("DocCardRouteItemId"),
                            "DocCardRouteItemId of route item on route point 1 is wrong!"
                            );
                    }
                    else if (reader.GetInt32("RouteItem") == item.RoutePoints[1].RouteItems[0].Id && !been[1])
                    {
                        been[1] = true;
                        Assert.IsTrue(reader.GetBoolean("Passed"), "RouteItem on rp 2 passed is wrong!");
                        Assert.AreEqual(
                            ((DocCardRouteItem)item.RoutePoints[1].RouteItems[0]).DocRouteItemId,
                            reader.GetInt64("DocCardRouteItemId"),
                            "DocCardRouteItemId of route item on route point 2 is wrong!"
                            );
                    }
                    else if (reader.GetInt32("RouteItem") == item.RoutePoints[2].RouteItems[0].Id && !been[2])
                    {
                        been[2] = true;
                        Assert.IsTrue(reader.GetBoolean("Passed"), "RouteItem on rp 3 passed is wrong!");
                        Assert.AreEqual(
                            ((DocCardRouteItem)item.RoutePoints[2].RouteItems[0]).DocRouteItemId,
                            reader.GetInt64("DocCardRouteItemId"),
                            "DocCardRouteItemId of route item on route point 3 is wrong!"
                            );
                    }
                    else
                    {
                        Assert.Fail(
                            String.Format(
                                "Unknown or repeated doc card route item! Id={0}",
                                reader.GetInt32("RouteItem")
                                )
                            );
                    }

                }
                while (reader.Read());
                reader.Close();
                Assert.IsTrue(been[0], "RoutePoint 1 routeItem not found!");
                Assert.IsTrue(been[1], "RoutePoint 2 routeItem not found!");
                Assert.IsTrue(been[2], "RoutePoint 3 routeItem not found!");

            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }
            }

            // clean up
            cmd = new MySqlCommand(
                "delete from DocCardRouteItem where DocCard=?DocCard;" +
                "delete from DocCard where DocCardId=?DocCard;",
                _con.MySqlConnection
                );
            cmd.Parameters.Add("?DocCard", MySqlDbType.Int64);
            cmd.Parameters["?DocCard"].Value = item.Id;
            cmd.ExecuteNonQuery();
        }

        [Test]
        [ExpectedException(ExpectedException=typeof(DcUnathorizedAccessException))]
        public void DeleteDocCardWithoutDeleteInFolder()
        {
            MySqlCommand cmd = new MySqlCommand(
                   "update FolderPermission set D=?D where Folder=@testDocumentFolder1 and DcRole=@userRole;",
                   _con.MySqlConnection
                   );
            cmd.Parameters.Add("?D", MySqlDbType.Bit);
            cmd.Parameters["?D"].Value = false;
            cmd.ExecuteNonQuery();

            DocCardAccess dca = new DocCardAccess(_const.DocCircUser2, _con, _sk);
            DocCardItem item = dca.CreateDocCard(_const.TestDocumentFolder1);
            item.DocType = _const.DocType2;
            item.DocStatus = DBConstKeeper.Current.DocStatus.Draft;
            dca.SaveDocCard(item);

            try
            {
                dca.DeleteDocCard(item.Id); // tested method

            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                cmd.Parameters["?D"].Value = true;
                cmd.ExecuteNonQuery();
                dca = new DocCardAccess(_const.DocCircAdmin, _con, _sk);
                dca.DeleteDocCard(item.Id);
            }
        }

        [Test]
        [ExpectedException(ExpectedException = typeof(DcUnathorizedAccessException))]
        public void DeleteDocCardWithoutDeleteInDocType()
        {
            MySqlCommand cmd = new MySqlCommand(
               "update DocTypePermission set D=?D where DocType=@docType2 and DcRole=@userRole;",
               _con.MySqlConnection
               );
            cmd.Parameters.Add("?D", MySqlDbType.Bit);
            cmd.Parameters["?D"].Value = false;
            cmd.ExecuteNonQuery();

            DocCardAccess dca = new DocCardAccess(_const.DocCircUser2, _con, _sk);
            DocCardItem item = dca.CreateDocCard(_const.TestDocumentFolder1);
            item.DocType = _const.DocType2;
            item.DocStatus = DBConstKeeper.Current.DocStatus.Draft;
            dca.SaveDocCard(item);

            try
            {
                dca.DeleteDocCard(item.Id); // tested method
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                cmd.Parameters["?D"].Value = true;
                cmd.ExecuteNonQuery();
                dca = new DocCardAccess(_const.DocCircAdmin, _con, _sk);
                dca.DeleteDocCard(item.Id);
            }
        }

        [Test]
        public void DeleteDocCardWithDeleteOwnNotDeleteInFolder()
        {
            MySqlCommand cmd = new MySqlCommand(
               "update FolderPermission set D=?D where Folder=@testDocumentFolder1 and DcRole=@userRole;",
               _con.MySqlConnection
               );
            cmd.Parameters.Add("?D", MySqlDbType.Bit);
            cmd.Parameters["?D"].Value = false;
            cmd.ExecuteNonQuery();

            DocCardAccess dca = new DocCardAccess(_const.DocCircUser2, _con, _sk);
            DocCardItem item = dca.CreateDocCard(_const.TestDocumentFolder1);
            item.DocType = _const.DocType2;
            item.DocStatus = DBConstKeeper.Current.DocStatus.Draft;
            dca.SaveDocCard(item);

            try
            {
                dca.DeleteDocCard(item.Id); // tested method
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                cmd.Parameters["?D"].Value = true;
                cmd.ExecuteNonQuery();
            }
        }

        [Test]
        public void DeleteDocCard()
        {
            DocCardAccess dca = new DocCardAccess(_const.DocCircAdmin, _con, _sk);
            DocCardItem item = dca.CreateDocCard(_const.TestDocumentFolder1);
            item.DocType = _const.DocType1;
            dca.LoadDocCardRoute(item);
            item.DocStatus = DBConstKeeper.Current.DocStatus.UnderCons;

            ((DocCardRouteItem)item.RoutePoints[0].RouteItems[0]).Passed = true;
            ((DocCardRouteItem)item.RoutePoints[1].RouteItems[0]).UnderCons = true;

            dca.SaveDocCard(item);
            AttachmentItem att = new AttachmentItem(
                item.Id,
                "DocCardAccessTest.file2.txt",
                "Content2",
                6,
                new DateTime(2007, 06, 04)
                );


            Stream att2Stream = File.OpenRead(Path.Combine(FixtureFile.Folder, FixtureFile.Attachment2));
            AttachmentAccess aa = new AttachmentAccess(_const.DocCircAdmin, _con, _sk);
            aa.SaveAttachment(att, att2Stream);
            att2Stream.Close();

            dca.DeleteDocCard(item.Id); // tested method

            MySqlCommand cmd = new MySqlCommand(
                "select DocCardId from DocCard where DocCardId=?DocCard",
                _con.MySqlConnection
                );
            cmd.Parameters.Add("?DocCard", MySqlDbType.Int64);
            cmd.Parameters["?DocCard"].Value = item.Id;
            MySqlDataReader reader = cmd.ExecuteReader();
            try
            {
                Assert.IsFalse(reader.Read());
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }
            }

            cmd.CommandText =
                "select DocCardRouteItemId from DocCardRouteItem where DocCard=?DocCard;";
            reader = cmd.ExecuteReader();
            try
            {
                Assert.IsFalse(reader.Read());
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }
            }

            cmd.CommandText =
                "select AttachmentId from attachment where DocCard=?DocCard;";
            reader = cmd.ExecuteReader();
            try
            {
                Assert.IsFalse(reader.Read());
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }
            }

            string filePath = Path.Combine(_sk.AttachmentFolder, GetNameById(att.Folder.Value));
            filePath = Path.Combine(filePath, GetNameById(att.File.Value) + Path.GetExtension(att.FileName));
            Assert.IsFalse(File.Exists(filePath));

        }

        [Test]
        public void GetDocumentConsiderationInfo()
        {

            DocCardAccess dca = new DocCardAccess(_const.DocCircAdmin, _con, _sk);


            // get all documents in Root document folder

            DocCardFilterItem[] filterItems = new DocCardFilterItem[]
            {
                new DocCardFilterItem(DocCardFieldName.Folder, _const.RootDocumentFolder)
            };

            DateTime startPoint = DateTime.Now;
            DocCardConsiderationInfo[] consInfoArr = dca.GetDocumentConsiderationInfo(filterItems); // tested method
            Console.WriteLine("GetDocumentConsiderationInfo 1 running time:" + (DateTime.Now - startPoint));

            Assert.IsNotNull(consInfoArr);
            Assert.AreEqual(3, consInfoArr.Length);
            bool[] been = new bool[] { false, false, false };
            foreach (DocCardConsiderationInfo consInfo in consInfoArr)
            {
                if (consInfo.DocCardId == _const.DocCard1 && !been[0])
                {
                    been[0] = true;
                    Assert.AreEqual(1, consInfo.Employees.Length);
                    Assert.Contains(_const.DocCircUser3, consInfo.Employees);
                    Assert.AreEqual(2, consInfo.RoutePointNum);
                }
                else if (consInfo.DocCardId == _const.DocCard3 && !been[1])
                {
                    been[1] = true;
                    Assert.AreEqual(1, consInfo.RoutePointNum);
                    Assert.AreEqual(1, consInfo.Employees.Length);
                    Assert.Contains(_const.DocCircUser1, consInfo.Employees);
                }
                else if (consInfo.DocCardId == _const.DocCard5 && !been[2])
                {
                    been[2] = true;
                    Assert.AreEqual(2, consInfo.RoutePointNum);
                    Assert.AreEqual(2, consInfo.Employees.Length);
                    Assert.Contains(_const.DocCircUser2, consInfo.Employees);
                    Assert.Contains(_const.DocCircUser3, consInfo.Employees);
                }
                else
                {
                    Assert.Fail("Unknown or repeated DocCard!");
                }
            }
            Assert.IsTrue(been[0]);
            Assert.IsTrue(been[1]);
            Assert.IsTrue(been[2]);

            // get all documents under cons at DocCircUser3's 

            filterItems = new DocCardFilterItem[0];

            startPoint = DateTime.Now;
            consInfoArr = dca.GetDocumentConsiderationInfo(filterItems, new int[] { _const.DocCircUser3 }); // tested method
            Console.WriteLine("GetDocumentConsiderationInfo 2 running time:" + (DateTime.Now - startPoint));

            Assert.IsNotNull(consInfoArr);
            Assert.AreEqual(2, consInfoArr.Length);

            been = new bool[] { false, false };
            foreach (DocCardConsiderationInfo consInfo in consInfoArr)
            {
                if (consInfo.DocCardId == _const.DocCard1 && !been[0])
                {
                    been[0] = true;
                    Assert.AreEqual(1, consInfo.Employees.Length);
                    Assert.Contains(_const.DocCircUser3, consInfo.Employees);
                    Assert.AreEqual(2, consInfo.RoutePointNum);
                }
                else if (consInfo.DocCardId == _const.DocCard5 && !been[1])
                {
                    been[1] = true;
                    Assert.AreEqual(2, consInfo.RoutePointNum);
                    Assert.AreEqual(2, consInfo.Employees.Length);
                    Assert.Contains(_const.DocCircUser2, consInfo.Employees);
                    Assert.Contains(_const.DocCircUser3, consInfo.Employees);
                }
                else
                {
                    Assert.Fail("Unknown or repeated DocCard!");
                }
            }
            Assert.IsTrue(been[0]);
            Assert.IsTrue(been[1]);

            // get all documents under cons and registered by DocCircUser2
            filterItems = new DocCardFilterItem[]
            {
                new DocCardFilterItem(DocCardFieldName.DocStatus, DBConstKeeper.Current.DocStatus.UnderCons),
                new DocCardFilterItem(DocCardFieldName.RegisteringUser, _const.DocCircUser2)
            };

            startPoint = DateTime.Now;
            consInfoArr = dca.GetDocumentConsiderationInfo(filterItems); // tested method
            Console.WriteLine("GetDocumentConsiderationInfo 3 running time:" + (DateTime.Now - startPoint));

            Assert.IsNotNull(consInfoArr);
            Assert.AreEqual(2, consInfoArr.Length);

            been = new bool[] { false, false };
            foreach (DocCardConsiderationInfo consInfo in consInfoArr)
            {
                if (consInfo.DocCardId == _const.DocCard3 && !been[0])
                {
                    been[0] = true;
                    Assert.AreEqual(1, consInfo.RoutePointNum);
                    Assert.AreEqual(1, consInfo.Employees.Length);
                    Assert.Contains(_const.DocCircUser1, consInfo.Employees);
                }
                else if (consInfo.DocCardId == _const.DocCard4 && !been[1])
                {
                    been[1] = true;
                    Assert.AreEqual(3, consInfo.RoutePointNum);
                    Assert.AreEqual(1, consInfo.Employees.Length);
                    Assert.Contains(_const.DocCircAdmin, consInfo.Employees);
                }
                else
                {
                    Assert.Fail("Unknown or repeated DocCard!");
                }
            }
            Assert.IsTrue(been[0]);
            Assert.IsTrue(been[1]);

            // get documents where LastMovement between 01.03.07 and 30.04.07
            filterItems = new DocCardFilterItem[]
            {
                new PeriodDocCardFilterItem(
                    DocCardFieldName.LastMovement,
                    new DateTime(2007, 03, 01, 00, 00, 00),
                    new DateTime(2007, 04, 30, 23, 59, 59)
                    )
            };

            startPoint = DateTime.Now;
            consInfoArr = dca.GetDocumentConsiderationInfo(filterItems); // tested method
            Console.WriteLine("GetDocumentConsiderationInfo 5 running time:" + (DateTime.Now - startPoint));

            Assert.IsNotNull(consInfoArr);
            Assert.AreEqual(2, consInfoArr.Length);

            been = new bool[] { false, false };
            foreach (DocCardConsiderationInfo consInfo in consInfoArr)
            {
                if (consInfo.DocCardId == _const.DocCard3 && !been[0])
                {
                    been[0] = true;
                }
                else if (consInfo.DocCardId == _const.DocCard4 && !been[1])
                {
                    been[1] = true;
                }
                else
                {
                    Assert.Fail("Unknown or repeated DocCard!");
                }
            }
            Assert.IsTrue(been[0]);
            Assert.IsTrue(been[1]);

            //get DocCard5 by maximum definition
            filterItems = new DocCardFilterItem[]
            {
                new DocCardFilterItem(DocCardFieldName.Folder, _const.RootDocumentFolder),
                new DocCardFilterItem(DocCardFieldName.DocStatus, DBConstKeeper.Current.DocStatus.UnderCons),
                new DocCardFilterItem(DocCardFieldName.RegisteringUser, _const.DocCircUser3),
                new DocCardFilterItem(DocCardFieldName.Owner, _const.DocCircAdmin),
                new DocCardFilterItem(DocCardFieldName.Subject, "Test DocCard5"),
                new DocCardFilterItem(DocCardFieldName.InternalNumber, "TTF000005"),
                new DocCardFilterItem(
                    DocCardFieldName.RegistrationTime, 
                    new DateTime(2007, 05, 24, 11, 35, 00)
                    ),
                new DocCardFilterItem(DocCardFieldName.AttachmentsAreChangeable, false),
                new DocCardFilterItem(DocCardFieldName.OriginalNumber, "T215"),
                new DocCardFilterItem(DocCardFieldName.CustomSendingCompany, "MS5"),
                new DocCardFilterItem(DocCardFieldName.CustomReceivingCompany, "Evpo5"),
                new DocCardFilterItem(DocCardFieldName.RoutePointNum, 2),
                new DocCardFilterItem(DocCardFieldName.DocType, _const.DocType1),
                new DocCardFilterItem(
                    DocCardFieldName.LastMovement,
                    new DateTime(2007, 05, 23, 12, 00, 00)
                    ),
                new PeriodDocCardFilterItem(
                    DocCardFieldName.OriginalDate,
                    new DateTime(2007, 05, 01, 00, 00, 00),
                    new DateTime(2007, 05, 01, 23, 59, 59)
                    )
            };

            int[] consideringEmployees = new int[]
            {
                _const.DocCircUser2
            };

            startPoint = DateTime.Now;
            consInfoArr = dca.GetDocumentConsiderationInfo(filterItems, consideringEmployees); // tested method
            Console.WriteLine("GetDocumentConsiderationInfo 5 running time:" + (DateTime.Now - startPoint));

            Assert.IsNotNull(consInfoArr);
            Assert.AreEqual(1, consInfoArr.Length);
            Assert.AreEqual(_const.DocCard5, consInfoArr[0].DocCardId);
        }


        [Test]
        public void GetDocumentTableWithoutReadInFolderAndDocType()
        {
            MySqlCommand cmd = new MySqlCommand(
              "update FolderPermission set R=?R, RO=?R where Folder=@rootDocumentFolder and DcRole=@userRole;" +
              "update DocTypePermission set R=?R, RO=?R where DocType=@docType1 and DcRole=@userRole;",
              _con.MySqlConnection
              );
            cmd.Parameters.Add("?R", MySqlDbType.Bit);
            cmd.Parameters["?R"].Value = false;

            cmd.ExecuteNonQuery();

            DocCardAccess dca = new DocCardAccess(_const.DocCircUser1, _con, _sk);

            DocCardFilterItem[] filterItems = new DocCardFilterItem[]
            {
                new DocCardFilterItem(DocCardFieldName.Folder, _const.RootDocumentFolder)
            };

            DataTable table = dca.GetDocumentTable( // tested method
                new string[] { "Id" },
                filterItems
                );

            Assert.IsNotNull(table);
            Assert.AreEqual(0, table.Rows.Count);

            cmd.Parameters["?R"].Value = true;
            cmd.ExecuteNonQuery();
        }

        [Test]
        public void GetDocumentTableWithReadOwnNotReadInDocType()
        {
            MySqlCommand cmd = new MySqlCommand(
              "update DocTypePermission set R=?R where DocType=@docType1 and DcRole=@userRole;" +
              "update DocTypePermission set R=?R where DocType=@docType2 and DcRole=@userRole;",
              _con.MySqlConnection
              );
            cmd.Parameters.Add("?R", MySqlDbType.Bit);
            cmd.Parameters["?R"].Value = false;

            cmd.ExecuteNonQuery();

            DocCardAccess dca = new DocCardAccess(_const.DocCircUser3, _con, _sk);

            DataTable table = dca.GetDocumentTable( // tested method
                new string[] { "Id" },
                new DocCardFilterItem[0]
                );

            Assert.IsNotNull(table);
            Assert.AreEqual(1, table.Rows.Count);

            table.DefaultView.Sort = "Id ASC";

            Assert.AreEqual(_const.DocCard4, (long)table.Rows[0]["Id"]);

            cmd.Parameters["?R"].Value = true;
            cmd.ExecuteNonQuery();
        }

        [Test]
        public void GetDocumentTableWithReadOwnNotReadInFolder()
        {
            MySqlCommand cmd = new MySqlCommand(
              "update FolderPermission set R=?R where Folder=@testDocumentFolder1 and DcRole=@userRole;" +
              "update FolderPermission set R=?R where Folder=@rootDocumentFolder and DcRole=@userRole;",
              _con.MySqlConnection
              );
            cmd.Parameters.Add("?R", MySqlDbType.Bit);
            cmd.Parameters["?R"].Value = false;
           
            cmd.ExecuteNonQuery();

            DocCardAccess dca = new DocCardAccess(_const.DocCircUser3, _con, _sk);

            DataTable table = dca.GetDocumentTable( // tested method
                new string[] { "Id" },
                new DocCardFilterItem[0]
                );

            Assert.IsNotNull(table);
            Assert.AreEqual(1, table.Rows.Count);

            table.DefaultView.Sort = "Id ASC";

            Assert.AreEqual(_const.DocCard4, (long)table.Rows[0]["Id"]);

            cmd.Parameters["?R"].Value = true;
            cmd.ExecuteNonQuery();
        }


        [Test]
        public void GetDocumentTableWithoutReadInFolder()
        {
            MySqlCommand cmd = new MySqlCommand(
              "update FolderPermission set R=?R, RO=?RO where Folder=@testDocumentFolder1 and DcRole=@userRole;",
              _con.MySqlConnection
              );
            cmd.Parameters.Add("?R", MySqlDbType.Bit);
            cmd.Parameters["?R"].Value = false;
            cmd.Parameters.Add("?RO", MySqlDbType.Bit);
            cmd.Parameters["?RO"].Value = false;
            cmd.ExecuteNonQuery();

            DocCardAccess dca = new DocCardAccess(_const.DocCircUser1, _con, _sk);

            DataTable table = dca.GetDocumentTable( // tested method
                new string[] { "Id" },
                new DocCardFilterItem[0]
                );

            Assert.IsNotNull(table);
            Assert.AreEqual(3, table.Rows.Count);
            
            table.DefaultView.Sort = "Id ASC";

            Assert.AreEqual(_const.DocCard1, (long)table.Rows[0]["Id"]);
            Assert.AreEqual(_const.DocCard3, (long)table.Rows[1]["Id"]);
            Assert.AreEqual(_const.DocCard5, (long)table.Rows[2]["Id"]);

            cmd.Parameters["?R"].Value = true;
            cmd.Parameters["?RO"].Value = true;
            cmd.ExecuteNonQuery();
        }

        [Test]
        public void GetDocumentTableWithoutReadInDocType()
        {
            MySqlCommand cmd = new MySqlCommand(
              "update DocTypePermission set R=?R, RO=?RO where DocType=@docType1 and DcRole=@userRole;",
              _con.MySqlConnection
              );
            cmd.Parameters.Add("?R", MySqlDbType.Bit);
            cmd.Parameters["?R"].Value = false;
            cmd.Parameters.Add("?RO", MySqlDbType.Bit);
            cmd.Parameters["?RO"].Value = false;
            cmd.ExecuteNonQuery();

            DocCardAccess dca = new DocCardAccess(_const.DocCircUser1, _con, _sk);

            DataTable table = dca.GetDocumentTable( // tested method
                new string[] { "Id" },
                new DocCardFilterItem[0]
                );

            Assert.IsNotNull(table);
            Assert.AreEqual(2, table.Rows.Count);

            table.DefaultView.Sort = "Id ASC";

            Assert.AreEqual(_const.DocCard2, (long)table.Rows[0]["Id"]);
            Assert.AreEqual(_const.DocCard4, (long)table.Rows[1]["Id"]);

            cmd.Parameters["?R"].Value = true;
            cmd.Parameters["?RO"].Value = true;
            cmd.ExecuteNonQuery();
        }

        [Test]
        public void GetDocumentTableWithNotExactSearchType()
        {
            DocCardAccess dca = new DocCardAccess(_const.DocCircAdmin, _con, _sk);


            // get all documents in Root document folder
            string[] fields = new string[]
            {
                DocCardFieldName.Id, 
                DocCardFieldName.OriginalNumber
            };

            // begins
            DocCardFilterItem[] filterItems = new DocCardFilterItem[]
            {
                new DocCardFilterItem(DocCardFieldName.OriginalNumber, "T21", DocCardSearchType.Begins)
            };

            DataTable docCardTable = dca.GetDocumentTable(fields, filterItems); // tested method

            Assert.AreEqual(3, docCardTable.Rows.Count);

            bool[] been = new bool[] { false, false, false };
            foreach (DataRow row in docCardTable.Rows)
            {
                if (((long)row[DocCardFieldName.Id]) == _const.DocCard1 && !been[0])
                {
                    been[0] = true;
                    Assert.AreEqual("T21", row[DocCardFieldName.OriginalNumber].ToString());
                }
                else if (((long)row[DocCardFieldName.Id]) == _const.DocCard3 && !been[1])
                {
                    been[1] = true;
                    Assert.AreEqual("T213", row[DocCardFieldName.OriginalNumber].ToString());
                }
                else if (((long)row[DocCardFieldName.Id]) == _const.DocCard5 && !been[2])
                {
                    been[2] = true;
                    Assert.AreEqual("T215", row[DocCardFieldName.OriginalNumber].ToString());
                }
                else
                {
                    Assert.Fail("Unknown or repeated id!");
                }
            }

            // contains
            filterItems = new DocCardFilterItem[]
            {
                new DocCardFilterItem(DocCardFieldName.OriginalNumber, "3", DocCardSearchType.Contains)
            };

            docCardTable = dca.GetDocumentTable(fields, filterItems); // tested method
            
            been = new bool[] { false, false, false };
            foreach (DataRow row in docCardTable.Rows)
            {
                if (((long)row[DocCardFieldName.Id]) == _const.DocCard2 && !been[0])
                {
                    been[0] = true;
                    Assert.AreEqual("T232", row[DocCardFieldName.OriginalNumber].ToString());
                }
                else if (((long)row[DocCardFieldName.Id]) == _const.DocCard3 && !been[1])
                {
                    been[1] = true;
                    Assert.AreEqual("T213", row[DocCardFieldName.OriginalNumber].ToString());
                }
                else if (((long)row[DocCardFieldName.Id]) == _const.DocCard4 && !been[2])
                {
                    been[2] = true;
                    Assert.AreEqual("T234", row[DocCardFieldName.OriginalNumber].ToString());
                }
                else
                {
                    Assert.Fail("Unknown or repeated id!");
                }
            }

        }

        [Test]
        public void GetDocumentTable()
        {
            DocCardAccess dca = new DocCardAccess(_const.DocCircAdmin, _con, _sk);
            
            
            // get all documents in Root document folder
            string[] fields = new string[]
            {
                DocCardFieldName.Id, 
                DocCardFieldName.InternalNumber, 
                DocCardFieldName.Subject,
                DocCardFieldName.RegisteringUser,
                DocCardFieldName.RegistrationTime,
                DocCardFieldName.LastMovement,
                DocCardFieldName.Note
            };

            DocCardFilterItem[] filterItems = new DocCardFilterItem[]
            {
                new DocCardFilterItem(DocCardFieldName.Folder, _const.RootDocumentFolder)
            };

            DataTable docCardTable = dca.GetDocumentTable(fields, filterItems); // tested method
            
            Assert.IsNotNull(docCardTable);
            Assert.AreEqual(3, docCardTable.Rows.Count);
            docCardTable.DefaultView.Sort = "Id ASC";

            // DocCard1
            DataRow docCardRow = docCardTable.Rows[0];

            Assert.AreEqual(_const.DocCard1, (long)docCardRow[DocCardFieldName.Id]);
            Assert.AreEqual("TTF000001", (string)docCardRow[DocCardFieldName.InternalNumber]);
            Assert.AreEqual("Test DocCard1", (string)docCardRow[DocCardFieldName.Subject]);
            Assert.AreEqual(_const.DocCircAdmin, (int)docCardRow[DocCardFieldName.RegisteringUser]);
            Assert.AreEqual(
                new DateTime(2007, 5, 24, 11, 35, 00), 
                (DateTime)docCardRow[DocCardFieldName.RegistrationTime]
                );
            Assert.AreEqual(
                new DateTime(2007, 5, 23, 12, 00, 00),
                (DateTime)docCardRow[DocCardFieldName.LastMovement]
                );
            Assert.AreEqual("this is the note of DocCard1", (string)docCardRow[DocCardFieldName.Note]);

            // DocCard3
            docCardRow = docCardTable.Rows[1];

            Assert.AreEqual(_const.DocCard3, (long)docCardRow[DocCardFieldName.Id]);
            Assert.AreEqual("TTF000003", (string)docCardRow[DocCardFieldName.InternalNumber]);
            Assert.AreEqual("Test DocCard3", (string)docCardRow[DocCardFieldName.Subject]);
            Assert.AreEqual(_const.DocCircUser2, (int)docCardRow[DocCardFieldName.RegisteringUser]);
            Assert.AreEqual(
                new DateTime(2007, 3, 24, 11, 35, 00),
                (DateTime)docCardRow[DocCardFieldName.RegistrationTime]
                );
            Assert.AreEqual(
                new DateTime(2007, 3, 23, 12, 00, 00),
                (DateTime)docCardRow[DocCardFieldName.LastMovement]
                );
            Assert.AreEqual("this is the note of DocCard3", (string)docCardRow[DocCardFieldName.Note]);

            // DocCard5
            docCardRow = docCardTable.Rows[2];

            Assert.AreEqual(_const.DocCard5, (long)docCardRow[DocCardFieldName.Id]);
            Assert.AreEqual("TTF000005", (string)docCardRow[DocCardFieldName.InternalNumber]);
            Assert.AreEqual("Test DocCard5", (string)docCardRow[DocCardFieldName.Subject]);
            Assert.AreEqual(_const.DocCircUser3, (int)docCardRow[DocCardFieldName.RegisteringUser]);
            Assert.AreEqual(
                new DateTime(2007, 5, 24, 11, 35, 00),
                (DateTime)docCardRow[DocCardFieldName.RegistrationTime]
                );
            Assert.AreEqual(
                new DateTime(2007, 5, 23, 12, 00, 00),
                (DateTime)docCardRow[DocCardFieldName.LastMovement]
                );
            Assert.AreEqual("this is the note of DocCard5", (string)docCardRow[DocCardFieldName.Note]);


            // get all documents under cons at DocCircUser3's 

            filterItems = new DocCardFilterItem[0];

            docCardTable = dca.GetDocumentTable(fields, filterItems, new int[] { _const.DocCircUser3}); // tested method

            Assert.IsNotNull(docCardTable);
            Assert.AreEqual(2, docCardTable.Rows.Count);
            docCardTable.DefaultView.Sort = "Id ASC";

            Assert.AreEqual(_const.DocCard1, (long)docCardTable.Rows[0]["Id"]);
            Assert.AreEqual(_const.DocCard5, (long)docCardTable.Rows[1]["Id"]);

            // get all documents under cons and registered by DocCircUser2
            filterItems = new DocCardFilterItem[]
            {
                new DocCardFilterItem(DocCardFieldName.DocStatus, DBConstKeeper.Current.DocStatus.UnderCons),
                new DocCardFilterItem(DocCardFieldName.RegisteringUser, _const.DocCircUser2)
            };

            docCardTable = dca.GetDocumentTable(fields, filterItems); // tested method

            Assert.IsNotNull(docCardTable);
            Assert.AreEqual(2, docCardTable.Rows.Count);
            docCardTable.DefaultView.Sort = "Id ASC";

            Assert.AreEqual(_const.DocCard3, (long)docCardTable.Rows[0]["Id"]);
            Assert.AreEqual(_const.DocCard4, (long)docCardTable.Rows[1]["Id"]);
            
            // get documents where LastMovement between 01.03.07 and 30.04.07
            filterItems = new DocCardFilterItem[]
            {
                new PeriodDocCardFilterItem(
                    DocCardFieldName.LastMovement,
                    new DateTime(2007, 03, 01, 00, 00, 00),
                    new DateTime(2007, 04, 30, 23, 59, 59)
                    )
            };
            
            docCardTable = dca.GetDocumentTable(fields, filterItems); // tested method

            Assert.IsNotNull(docCardTable);
            Assert.AreEqual(2, docCardTable.Rows.Count);
            docCardTable.DefaultView.Sort = "Id ASC";

            Assert.AreEqual(_const.DocCard3, (long)docCardTable.Rows[0]["Id"]);
            Assert.AreEqual(_const.DocCard4, (long)docCardTable.Rows[1]["Id"]);

            //get DocCard5 by maximum definition
            filterItems = new DocCardFilterItem[]
            {
                new DocCardFilterItem(DocCardFieldName.Folder, _const.RootDocumentFolder),
                new DocCardFilterItem(DocCardFieldName.DocStatus, DBConstKeeper.Current.DocStatus.UnderCons),
                new DocCardFilterItem(DocCardFieldName.RegisteringUser, _const.DocCircUser3),
                new DocCardFilterItem(DocCardFieldName.Owner, _const.DocCircAdmin),
                new DocCardFilterItem(DocCardFieldName.Subject, "Test DocCard5"),
                new DocCardFilterItem(DocCardFieldName.InternalNumber, "TTF000005"),
                new DocCardFilterItem(
                    DocCardFieldName.RegistrationTime, 
                    new DateTime(2007, 05, 24, 11, 35, 00)
                    ),
                new DocCardFilterItem(DocCardFieldName.AttachmentsAreChangeable, false),
                new DocCardFilterItem(DocCardFieldName.OriginalNumber, "T215"),
                new DocCardFilterItem(DocCardFieldName.CustomSendingCompany, "MS5"),
                new DocCardFilterItem(DocCardFieldName.CustomReceivingCompany, "Evpo5"),
                new DocCardFilterItem(DocCardFieldName.Note, "this is the note of DocCard5"),
                new DocCardFilterItem(DocCardFieldName.RoutePointNum, 2),
                new DocCardFilterItem(DocCardFieldName.DocType, _const.DocType1),
                new DocCardFilterItem(
                    DocCardFieldName.LastMovement,
                    new DateTime(2007, 05, 23, 12, 00, 00)
                    ),
                new PeriodDocCardFilterItem(
                    DocCardFieldName.OriginalDate,
                    new DateTime(2007, 05, 01, 00, 00, 00),
                    new DateTime(2007, 05, 01, 23, 59, 59)
                    )
            };

            int[] consideringEmployees = new int[]
            {
                _const.DocCircUser2
            };

            docCardTable = dca.GetDocumentTable(fields, filterItems, consideringEmployees); // tested method

            Assert.IsNotNull(docCardTable);
            Assert.AreEqual(1, docCardTable.Rows.Count);
            Assert.AreEqual(_const.DocCard5, (long)docCardTable.Rows[0]["Id"]);
        }

        [Test]
        public void BugReceivingCompanyNotLoaded()
        {
            DocCardAccess dca = new DocCardAccess(_const.DocCircAdmin, _con, _sk);
            bool loadRoute = false;
            DocCardItem item = dca.LoadDocCard(_const.DocCard3, !loadRoute);

            Assert.AreEqual(_const.Institution2, item.SendingCompany);
            Assert.AreEqual(_const.Institution1, item.ReceivingCompany);


        }

        [Test]
        [ExpectedException(ExpectedException = typeof(DcObjectNotFoundException))]
        public void LoadDocCard_NotExistentId()
        {
            DocCardAccess dca = new DocCardAccess(_const.DocCircAdmin, _con, _sk);
            bool loadRoute = false;
            long wrongId = -1;
            DocCardItem item = dca.LoadDocCard(wrongId, !loadRoute); // tested method
        }

        [TestFixtureTearDown]
        public void TFTearDown()
        {
            _tran.Rollback();
            _con.Close();
            DeleteFolder(_sk.AttachmentFolder);
        }

    }
}
