//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using NUnit.Framework;
using MySql.Data.MySqlClient;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.MySqlDL;

namespace Evpo.DocCircUnitTest
{
    [TestFixture]
    public class DocCardReferenceAccessTest
    {
        protected struct FixtureFile
        {
            public const string Folder = "MySql";
            public const string DocCardAndReferenceCreation = "DocCardReferenceAccessTest.file1.sql";
        }

        protected class FixtureData
        {
            public static int RootDocumentFolder;
            public static int UserRole;
            public static int DocCircAdmin, DocCircUser1, DocCircUser2, DocCircUser3;
            public static int DraftDS;
            public static long DocCard1, DocCard2, DocCard3, DocCard4, DocCard5;
            public static int DocType1, DocType2;
        }

        private DBConnection _con;
        private MySqlTransaction _tran;
        private SettingKeeper _sk;

        [TestFixtureSetUp]
        public void TFSetUp()
        {
            _con = new DBConnection(
              "Database=EvpoDCTest;Data Source=localhost;User Id=doccirctest;Password=doccirctest"
              );
            _con.Open();
            _tran = _con.MySqlConnection.BeginTransaction();

            MySqlCommand cmd = _con.MySqlConnection.CreateCommand();

            //TextReader txtReader = File.OpenText(
            //    Path.Combine(FixtureFile.Folder, FixtureFile.DocCardAndReferenceCreation)
            //    );
            //cmd.CommandText = txtReader.ReadToEnd();
            //txtReader.Close();

            cmd.CommandText = MySqlResources.DocCardReferenceAccessTest_file1;
            cmd.ExecuteNonQuery();

            string constTempl = "cast(@{0} as unsigned) as {0}";
            StringBuilder sb = new StringBuilder();
            sb.Append("select ");
            sb.AppendFormat(constTempl, "rootDocumentFolder");
            sb.Append(", ");
            sb.AppendFormat(constTempl, "userRole");
            sb.Append(", ");
            sb.AppendFormat(constTempl, "docCircAdmin");
            sb.Append(", ");
            sb.AppendFormat(constTempl, "docCircUser1");
            sb.Append(", ");
            sb.AppendFormat(constTempl, "docCircUser2");
            sb.Append(", ");
            sb.AppendFormat(constTempl, "docCircUser3");
            sb.Append(", ");
            sb.AppendFormat(constTempl, "draftDS");
            sb.Append(", ");
            sb.AppendFormat(constTempl, "docCard1");
            sb.Append(", ");
            sb.AppendFormat(constTempl, "docCard2");
            sb.Append(", ");
            sb.AppendFormat(constTempl, "docCard3");
            sb.Append(", ");
            sb.AppendFormat(constTempl, "docCard4");
            sb.Append(", ");
            sb.AppendFormat(constTempl, "docCard5");
            sb.Append(", ");
            sb.AppendFormat(constTempl, "docType1");
            sb.Append(", ");
            sb.AppendFormat(constTempl, "docType2");
            sb.Append(";");

            cmd.CommandText = sb.ToString();
            MySqlDataReader reader = cmd.ExecuteReader();

            if (reader.Read())
            {
                FixtureData.RootDocumentFolder = reader.GetInt32("rootDocumentFolder");
                FixtureData.UserRole = reader.GetInt32("userRole");
                FixtureData.DocCircAdmin = reader.GetInt32("docCircAdmin");
                FixtureData.DocCircUser1 = reader.GetInt32("docCircUser1");
                FixtureData.DocCircUser2 = reader.GetInt32("docCircUser2");
                FixtureData.DocCircUser3 = reader.GetInt32("docCircUser3");
                FixtureData.DraftDS = reader.GetInt32("draftDS");
                FixtureData.DocType1 = reader.GetInt32("docType1");
                FixtureData.DocType2 = reader.GetInt32("docType2");
                FixtureData.DocCard1 = reader.GetInt64("docCard1");
                FixtureData.DocCard2 = reader.GetInt64("docCard2");
                FixtureData.DocCard3 = reader.GetInt64("docCard3");
                FixtureData.DocCard4 = reader.GetInt64("docCard4");
                FixtureData.DocCard5 = reader.GetInt64("docCard5");
            }
            reader.Close();

            _sk = new SettingKeeper();
        }

        [Test]
        public void GetDocCardReferences()
        {
            DocCardReferenceAccess access = new DocCardReferenceAccess(FixtureData.DocCircAdmin, _con, _sk);

            //DocCard 1 references
            DocCardReferenceItem[] items = access.GetDocCardReferences(FixtureData.DocCard1); // tested method

            Assert.AreEqual(2, items.Length);

            bool[] been = new bool[] { false, false };
            foreach (DocCardReferenceItem item in items)
            {
                Assert.AreEqual(FixtureData.DocCard1, item.Master);
                if (item.Reference == FixtureData.DocCard2 && !been[0])
                {
                    been[0] = true;
                }
                else if (item.Reference == FixtureData.DocCard3 && !been[1])
                {
                    been[1] = true;
                }
                else
                {
                    Assert.Fail("Unknown or repeated id!");
                }
            }
            
            //DocCard 5 references
            items = access.GetDocCardReferences(FixtureData.DocCard5); // tested method

            Assert.AreEqual(4, items.Length);

            been = new bool[] { false, false, false, false };
            foreach (DocCardReferenceItem item in items)
            {
                Assert.AreEqual(FixtureData.DocCard5, item.Master);
                Assert.IsNotNull(item.Id);

                if (item.Reference == FixtureData.DocCard4 && !been[0])
                {
                    been[0] = true;
                }
                else if (item.Reference == FixtureData.DocCard1 && !been[1])
                {
                    been[1] = true;
                }
                else if (item.Reference == FixtureData.DocCard2 && !been[2])
                {
                    been[2] = true;
                }
                else if (item.Reference == FixtureData.DocCard3 && !been[3])
                {
                    been[3] = true;
                }
                else
                {
                    Assert.Fail("Unknown or repeated id!");
                }
            }
        }

        [Test]
        [ExpectedException(ExpectedException=typeof(DcUnathorizedAccessException))]
        public void GetDocCardReferencesWithoutReadAccess()
        {
            CloseAccessToDocTypeForUsers(FixtureData.DocType1, AccessType.Read);

            DocCardReferenceAccess access = new DocCardReferenceAccess(FixtureData.DocCircUser1, _con, _sk);

            try
            {
                DocCardReferenceItem[] items = access.GetDocCardReferences(FixtureData.DocCard1); // tested method
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                RestoreAccesToDocTypeForUsers(FixtureData.DocType1);
            }
        }

        //[Test]
        //public void GetDocCardReferencesWithoutReadAccessOfReferences()
        //{
        //    CloseAccessToDocTypeForUsers(FixtureData.DocType2, AccessType.Read);

        //    DocCardReferenceAccess access = new DocCardReferenceAccess(FixtureData.DocCircUser1, _con);
        //    DocCardReferenceItem[] items = access.GetDocCardReferences(FixtureData.DocCard5); // tested method

        //    try
        //    {
        //        Assert.AreEqual(4, items.Length);

        //        int nullItemCount = 0;
        //        bool[] been = new bool[] { false, false };
        //        foreach (DocCardReferenceItem refItem in items)
        //        {
        //            if (refItem == null)
        //            {
        //                if (refItem.Reference == FixtureData.DocCard1 && !been[0])
        //                {
        //                    been[0] = true;
        //                }
        //                else if (refItem.Reference == FixtureData.DocCard3 && !been[1])
        //                {
        //                    been[1] = true;
        //                }
        //                else
        //                {
        //                    Assert.Fail("Unknown or repeated id!");
        //                }
        //            }
        //            else
        //            {
        //                nullItemCount++;
        //            }
        //        }

        //        Assert.AreEqual(2, nullItemCount);
        //    }
        //    catch (Exception ex)
        //    {
        //        throw ex;
        //    }
        //    finally
        //    {
        //        RestoreAccesToDocTypeForUsers(FixtureData.DocType2);
        //    }
        //}

        [Test]
        [ExpectedException(ExpectedException = typeof(DcUnathorizedAccessException))]
        public void SaveDocCardReferencesWithoutWriteAccess()
        {
            CloseAccessToDocTypeForUsers(FixtureData.DocType2, AccessType.Write);

            DocCardReferenceAccess access = new DocCardReferenceAccess(FixtureData.DocCircUser1, _con, _sk);

            DocCardReferenceItem[] items = new DocCardReferenceItem[]
            {
                new DocCardReferenceItem(FixtureData.DocCard2, FixtureData.DocCard3),
                new DocCardReferenceItem(FixtureData.DocCard2, FixtureData.DocCard4)
            };

            try
            {
                access.SaveDocCardReferences(FixtureData.DocCard2, items); // tested method
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                RestoreAccesToDocTypeForUsers(FixtureData.DocType2);
            }

        }

        [Test]
        [ExpectedException(ExpectedException = typeof(DcUnathorizedAccessException))]
        public void DeleteDocCardReferencesWithoutWriteAccess()
        {
            CloseAccessToDocTypeForUsers(FixtureData.DocType2, AccessType.Write);

            DocCardReferenceAccess access = new DocCardReferenceAccess(FixtureData.DocCircUser1, _con, _sk);

            try
            {
                access.DeleteDocCardReferences(FixtureData.DocCard2); // tested method
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                RestoreAccesToDocTypeForUsers(FixtureData.DocType2);
            }
        }

        protected enum AccessType
        {
            Read,
            Write
        }

        protected void CloseAccessToDocTypeForUsers(int docType, AccessType accessType)
        {
            MySqlCommand cmd = _con.MySqlConnection.CreateCommand();
            string access = "";
            if(accessType == AccessType.Read)
            {
                access = "R";
            }
            else if(accessType == AccessType.Write)
            {
                access = "U";
            }

            cmd.CommandText = String.Format(
                "update DocTypePermission set {0}=false, {0}O=false " +
                "where DocType=?DocType and DcRole=@userRole;",
                access
                );

            AddParameter(cmd, "DocType", MySqlDbType.Int32, docType);
            cmd.ExecuteNonQuery();
        }

        protected void RestoreAccesToDocTypeForUsers(int docType)
        {
            MySqlCommand cmd = _con.MySqlConnection.CreateCommand();
            cmd.CommandText =
                "delete from DocTypePermission where DocType=?DocType and DcRole=@userRole;" +
                
                "insert into DocTypePermission (DocType, DcRole, C, R, U, D, RO, UO, DO) " +
                "select ?DocType, DcRole, C, R, U, D, RO, UO, DO from NewDocTypePermission " +
                "where DcRole=@userRole;";

            AddParameter(cmd, "DocType", MySqlDbType.Int32, docType);

            cmd.ExecuteNonQuery();
        }

        protected void AddParameter(MySqlCommand cmd, string paramName, MySqlDbType paramType, object paramValue)
        {
            string param = String.Format("?{0}", paramName);
            cmd.Parameters.Add(param, paramType);
            cmd.Parameters[param].Value = paramValue;
        }

        [Test]
        public void SaveDocCardReferencesAfterAllReferencesRemoved()
        {
            DocCardReferenceAccess access = new DocCardReferenceAccess(FixtureData.DocCircAdmin, _con, _sk);

            DocCardReferenceItem[] items = new DocCardReferenceItem[]
            {
                new DocCardReferenceItem(FixtureData.DocCard2, FixtureData.DocCard3),
                new DocCardReferenceItem(FixtureData.DocCard2, FixtureData.DocCard4)
            };

            access.SaveDocCardReferences(FixtureData.DocCard2, items);

            items = new DocCardReferenceItem[0];

            access.SaveDocCardReferences(FixtureData.DocCard2, items); // tested method

            MySqlCommand cmd = _con.MySqlConnection.CreateCommand();
            AddParameter(cmd, "MasterDocCard", MySqlDbType.Int64, FixtureData.DocCard2);
            cmd.CommandText =
                "select count(*) from DocCardReference where MasterDocCard=?MasterDocCard;";

            MySqlDataReader reader = cmd.ExecuteReader();
            reader.Read();
            int count = reader.GetInt32(0);
            reader.Close();

            if (count > 0)
            {
                cmd.CommandText = "delete from DocCardReference where MasterDocCard=?MasterDocCard;";
                cmd.ExecuteNonQuery();
            }

            Assert.AreEqual(0, count);
        }

        [Test]
        public void SaveDocCardReferences()
        {
            DocCardReferenceAccess access = new DocCardReferenceAccess(FixtureData.DocCircAdmin, _con, _sk);

            DocCardReferenceItem[] items = new DocCardReferenceItem[]
            {
                new DocCardReferenceItem(FixtureData.DocCard2, FixtureData.DocCard3),
                new DocCardReferenceItem(FixtureData.DocCard2, FixtureData.DocCard4)
            };

            access.SaveDocCardReferences(FixtureData.DocCard2, items); // tested method

            Assert.IsNotNull(items[0].Id);
            Assert.IsNotNull(items[1].Id);

            MySqlCommand cmd = _con.MySqlConnection.CreateCommand();
            cmd.CommandText = 
                "select MasterDocCard, SlaveDocCard from DocCardReference " +
                "where DocCardReferenceId=?DocCardReferenceId;";
            AddParameter(cmd, "DocCardReferenceId", MySqlDbType.Int64, items[0].Id);
            MySqlDataReader reader = cmd.ExecuteReader();

            bool isException = false;

            try
            {
                Assert.IsTrue(reader.Read(), "read 1");
                Assert.AreEqual(FixtureData.DocCard2, reader.GetInt64("MasterDocCard"), "master 1");
                Assert.AreEqual(FixtureData.DocCard3, reader.GetInt64("SlaveDocCard"), "slave 1");
                reader.Close();
                cmd.Parameters["?DocCardReferenceId"].Value = items[1].Id;
                reader = cmd.ExecuteReader();

                Assert.IsTrue(reader.Read(), "read 2");
                Assert.AreEqual(FixtureData.DocCard2, reader.GetInt64("MasterDocCard"), "master 2");
                Assert.AreEqual(FixtureData.DocCard4, reader.GetInt64("SlaveDocCard"), "slave 2");

            }
            catch (Exception ex)
            {
                isException = true;
                throw ex;
            }
            finally
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }

                if (isException)
                {
                    cmd.CommandText = "delete from DocCardReference where MasterDocCard=?MasterDocCard";
                    AddParameter(cmd, "MasterDocCard", MySqlDbType.Int64, FixtureData.DocCard2);
                    cmd.ExecuteNonQuery();
                }
            }

            // save additional references and remove old one at index 1
            
            DocCardReferenceItem[] oldItems = items;
            items = new DocCardReferenceItem[3];
            oldItems.CopyTo(items, 0);
            items[0] = oldItems[0];
            items[1] = new DocCardReferenceItem(FixtureData.DocCard2, FixtureData.DocCard5);
            items[2] = new DocCardReferenceItem(FixtureData.DocCard2, FixtureData.DocCard1);

            long[] itemIdsBefore = new long[]
            {
                oldItems[0].Id.Value,
                oldItems[1].Id.Value
            };

            access.SaveDocCardReferences(FixtureData.DocCard2, items); // tested method

            Assert.IsNotNull(items[1].Id);
            Assert.IsNotNull(items[2].Id);

            try
            {
                // DocCard2 -> DocCard3
                Assert.AreEqual(itemIdsBefore[0], items[0].Id, "check persistent id 1");

                cmd.Parameters["?DocCardReferenceId"].Value = items[0].Id;
                reader = cmd.ExecuteReader();
                Assert.IsTrue(reader.Read(), "read 1");

                Assert.AreEqual(FixtureData.DocCard2, reader.GetInt64("MasterDocCard"), "master 1");
                Assert.AreEqual(FixtureData.DocCard3, reader.GetInt64("SlaveDocCard"), "slave 1");
                reader.Close();
                // DocCard2 -> DocCard5
                cmd.Parameters["?DocCardReferenceId"].Value = items[1].Id;
                reader = cmd.ExecuteReader();
                Assert.IsTrue(reader.Read(), "read 2");

                Assert.AreEqual(FixtureData.DocCard2, reader.GetInt64("MasterDocCard"), "master 2");
                Assert.AreEqual(FixtureData.DocCard5, reader.GetInt64("SlaveDocCard"), "slave 2");
                reader.Close();
                // DocCard2 -> DocCard1
                cmd.Parameters["?DocCardReferenceId"].Value = items[2].Id;
                reader = cmd.ExecuteReader();
                Assert.IsTrue(reader.Read(), "read 3");

                Assert.AreEqual(FixtureData.DocCard2, reader.GetInt64("MasterDocCard"), "master 3");
                Assert.AreEqual(FixtureData.DocCard1, reader.GetInt64("SlaveDocCard"), "slave 3");
                reader.Close();

                // check that the item at index 1 was removed
                cmd.Parameters["?DocCardReferenceId"].Value = oldItems[1].Id.Value;
                reader = cmd.ExecuteReader();
                Assert.IsFalse(reader.Read(), "read 4");
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }

                cmd.CommandText = "delete from DocCardReference where MasterDocCard=?MasterDocCard";
                AddParameter(cmd, "MasterDocCard", MySqlDbType.Int64, FixtureData.DocCard2);
                cmd.ExecuteNonQuery();
            }
        }

        [Test]
        public void DeleteDocCardReferences()
        {
            //Add two references DocCard3 -> DocCard1, DocCard2
            long[] referenceIds = new long[2];
            MySqlCommand cmd = _con.MySqlConnection.CreateCommand();
            cmd.CommandText = "insert into DocCardReference (MasterDocCard, SlaveDocCard) Value (?Master, ?Slave);";
            AddParameter(cmd, "Master", MySqlDbType.Int64, FixtureData.DocCard3);
            AddParameter(cmd, "Slave", MySqlDbType.Int64, FixtureData.DocCard1);
            cmd.ExecuteNonQuery();
            referenceIds[0] = cmd.LastInsertedId;

            cmd.Parameters["?Master"].Value = FixtureData.DocCard3;
            cmd.Parameters["?Slave"].Value = FixtureData.DocCard2;
            cmd.ExecuteNonQuery();
            referenceIds[1] = cmd.LastInsertedId;

            DocCardReferenceAccess access = new DocCardReferenceAccess(FixtureData.DocCircAdmin, _con, _sk);
            access.DeleteDocCardReferences(FixtureData.DocCard3); // tested method

            cmd.CommandText = 
                "select DocCardReferenceId as RowCount from DocCardReference " +
                "where DocCardReferenceId in (?Id1, ?Id2);";
            AddParameter(cmd, "Id1", MySqlDbType.Int64, referenceIds[0]);
            AddParameter(cmd, "Id2", MySqlDbType.Int64, referenceIds[1]);

            MySqlDataReader reader = cmd.ExecuteReader();

            try
            {
                Assert.IsFalse(reader.Read());
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }
            }
        }


        [TestFixtureTearDown]
        public void TFTearDown()
        {
            _tran.Rollback();
            _con.Close();
        }

    }
}
