//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using NUnit.Framework;
using MySql.Data.MySqlClient;
using Evpo.DocCirc.Biz;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.MySqlDL;


namespace Evpo.DocCircUnitTest
{
    [TestFixture]
    public class DocTypeAccessTest
    {
        private const string _sqlFile1Path = @"MySql\DocTypeAccessTest.file1.sql";

        DBConnection _dbCon;
        MySqlTransaction _tran;
        int _usersRoleId, _docCircAdmin, _docCircUser, _user1, _user2, _user3, 
            _docType1, _docType2, _testDocTypeFolder, _rootDocTypeFolder;

        [TestFixtureSetUp]
        public void FSetUp()
        {
            string cmdStr = null;
            using (TextReader tr = File.OpenText(_sqlFile1Path))
            {
                cmdStr = tr.ReadToEnd();
                tr.Close();
            }

            _dbCon = new DBConnection(
               "Database=EvpoDCTest;Data Source=localhost;User Id=doccirctest;Password=doccirctest"
               );
            _dbCon.Open();

            _tran = _dbCon.MySqlConnection.BeginTransaction();

            MySqlCommand cmd = new MySqlCommand(cmdStr, _dbCon.MySqlConnection);
            cmd.ExecuteNonQuery();
            cmd.CommandText =
                "select " +
                "cast(@user1 as unsigned), cast(@user2 as unsigned), cast(@user3 as unsigned)," +
                "cast(@DocType1 as unsigned), cast(@DocType2 as unsigned), cast(@testDocTypeFolder1 as unsigned)," +
                "cast(@docTypeFolder as unsigned), cast(@usersRoleId as unsigned)," +
                "cast(@docCircAdminId as unsigned), cast(@docCircUserId as unsigned);";
            MySqlDataReader reader = cmd.ExecuteReader();
            reader.Read();
            _user1 = reader.GetInt32(0);
            _user2 = reader.GetInt32(1);
            _user3 = reader.GetInt32(2);
            _docType1 = reader.GetInt32(3);
            _docType2 = reader.GetInt32(4);
            _testDocTypeFolder = reader.GetInt32(5);
            _rootDocTypeFolder = reader.GetInt32(6);
            _usersRoleId = reader.GetInt32(7);
            _docCircAdmin = reader.GetInt32(8);
            _docCircUser = reader.GetInt32(9);
            reader.Close();

            DBConstAccess dca = new DBConstAccess(_dbCon);
            FolderTypeDirectory ftd = dca.GetFolderTypeDirectory();
            DBConstKeeper.Current = new DBConstKeeper(
                ftd,
                dca.GetRootFolderDirectory(ftd),
                dca.GetMainRoleDirectory(),
                dca.GetRouteItemTypeDirectory(),
                dca.GetDocStatusDirectory(),
                dca.GetNotificationTypeDirectory(),
                dca.GetEventTypeDirectory(),
                dca.GetCommonDepartment()
                );


            // create folder hierarchy
            MySqlCommand permCmd = new MySqlCommand(
                "insert into FolderPermission (DcRole, Folder, C, R, U, D, B, RO, UO, DO) " +
                "select DcRoleId, ?Folder,true,true,true,true,true,true,true,true " +
                "from DcRole where RoleName='Administrators';" +
                "insert into FolderPermission (DcRole, Folder, C, R, U, D, B, RO, UO, DO) " +
                "select DcRoleId, ?Folder,false,true,false,false,?B,true,false,false " +
                "from DcRole where RoleName='Users';"
                ,
                _dbCon.MySqlConnection
                );
            permCmd.Parameters.Add("?Folder", MySqlDbType.Int32);
            permCmd.Parameters.Add("?B", MySqlDbType.Bit);

            cmd = new MySqlCommand(
                "insert into folder (FolderId, ParentFolder, FolderType, Name)" +
                " Values (default, ?ParentFolder, ?FolderType, ?FolderName);"
                ,
                _dbCon.MySqlConnection
                );
            cmd.Parameters.Add("?ParentFolder", MySqlDbType.Int32);
            cmd.Parameters.Add("?FolderType", MySqlDbType.Int32);
            cmd.Parameters.Add("?FolderName", MySqlDbType.String);
            cmd.Parameters["?FolderType"].Value = DBConstKeeper.Current.FolderType.DocType;

            /***************************
             * Test DocType folder1
             * fld1
             *   fld11 // for Users B=false
             * fld2
             *   fld21
             *   fld22 // for Users B=false
             *     fld221
             * fld3 // for Users B=false
             * *************************/

            cmd.Parameters["?FolderName"].Value = "fld1";
            cmd.Parameters["?ParentFolder"].Value = DBConstKeeper.Current.RootFolder.DocType;
            cmd.ExecuteNonQuery();
            long curParent = cmd.LastInsertedId;
            permCmd.Parameters["?Folder"].Value = cmd.LastInsertedId;
            permCmd.Parameters["?B"].Value = true;
            permCmd.ExecuteNonQuery();

            cmd.Parameters["?FolderName"].Value = "fld11";
            cmd.Parameters["?ParentFolder"].Value = curParent;
            cmd.ExecuteNonQuery();
            permCmd.Parameters["?Folder"].Value = cmd.LastInsertedId;
            permCmd.Parameters["?B"].Value = false;
            permCmd.ExecuteNonQuery();

            cmd.Parameters["?FolderName"].Value = "fld2";
            cmd.Parameters["?ParentFolder"].Value = DBConstKeeper.Current.RootFolder.DocType;
            cmd.ExecuteNonQuery();
            curParent = cmd.LastInsertedId;
            permCmd.Parameters["?Folder"].Value = cmd.LastInsertedId;
            permCmd.Parameters["?B"].Value = true;
            permCmd.ExecuteNonQuery();

            cmd.Parameters["?FolderName"].Value = "fld21";
            cmd.Parameters["?ParentFolder"].Value = curParent;
            cmd.ExecuteNonQuery();
            permCmd.Parameters["?Folder"].Value = cmd.LastInsertedId;
            permCmd.Parameters["?B"].Value = true;
            permCmd.ExecuteNonQuery();

            cmd.Parameters["?FolderName"].Value = "fld22";
            cmd.Parameters["?ParentFolder"].Value = curParent;
            cmd.ExecuteNonQuery();
            curParent = cmd.LastInsertedId;
            permCmd.Parameters["?Folder"].Value = cmd.LastInsertedId;
            permCmd.Parameters["?B"].Value = false;
            permCmd.ExecuteNonQuery();
            
            cmd.Parameters["?FolderName"].Value = "fld221";
            cmd.Parameters["?ParentFolder"].Value = curParent;
            cmd.ExecuteNonQuery();
            permCmd.Parameters["?Folder"].Value = cmd.LastInsertedId;
            permCmd.ExecuteNonQuery();

            cmd.Parameters["?FolderName"].Value = "fld3";
            cmd.Parameters["?ParentFolder"].Value = DBConstKeeper.Current.RootFolder.DocType;
            cmd.ExecuteNonQuery();
            permCmd.Parameters["?Folder"].Value = cmd.LastInsertedId;
            permCmd.Parameters["?B"].Value = false;
            permCmd.ExecuteNonQuery();
        }

        //[SetUp]
        //public void TestSetUp()
        //{
        //    SecurityAccess sa = new SecurityAccess(_docCircAdmin, _dbCon);
        //    sa.CreateFolderSecurityTable();
        //}

        [Test]
        public void LoadDocTypeItem()
        {
            DocTypeAccess dta = new DocTypeAccess(_docCircAdmin, _dbCon);
            bool loadRouteItems = true;
            DocTypeItem docTypeItem = dta.LoadDocTypeItem(_docType2, loadRouteItems); // tested method

            Assert.AreEqual(_docType2, docTypeItem.Id);

            Assert.AreEqual("Test type second", docTypeItem.Name);
            Assert.AreEqual("second type", docTypeItem.Description);
            Assert.AreEqual("TTS", docTypeItem.Prefix);
            Assert.AreEqual(1, docTypeItem.NextNumber);
            Assert.AreEqual(false, docTypeItem.IsNew);
            Assert.AreEqual(_user2, docTypeItem.Owner);

            Assert.AreEqual(2, docTypeItem.RoutePoints.Count);

            RoutePoint rp = docTypeItem.RoutePoints[0];
            Assert.AreEqual(1, rp.OrderNum);
            Assert.AreEqual(1, rp.RouteItems.Count);
            Assert.AreEqual(_user3, rp.RouteItems[0].User);
            Assert.AreEqual(
                DBConstKeeper.Current.RouteItemType.Decision, 
                rp.RouteItems[0].RouteItemType
                );
            Assert.AreEqual("[0]", rp.Expression);

            rp = docTypeItem.RoutePoints[1];
            Assert.AreEqual(2, rp.OrderNum);
            Assert.AreEqual("[0]&&[1]", rp.Expression);
            Assert.AreEqual(2, rp.RouteItems.Count);
            Assert.AreEqual(_user1, rp.RouteItems[0].User);
            Assert.AreEqual(
                DBConstKeeper.Current.RouteItemType.Decision,
                rp.RouteItems[0].RouteItemType
                );
            Assert.AreEqual(_user2, rp.RouteItems[1].User);
            Assert.AreEqual(
                DBConstKeeper.Current.RouteItemType.Decision,
                rp.RouteItems[1].RouteItemType
                );

            docTypeItem = dta.LoadDocTypeItem(_docType1, loadRouteItems); // tested method
            Assert.AreEqual("Test type first", docTypeItem.Name);
            Assert.AreEqual(_docCircAdmin, docTypeItem.Owner);
            Assert.AreEqual(3, docTypeItem.RoutePoints.Count);
        }

        [Test]
        [ExpectedException(ExpectedException = typeof(DcObjectNotFoundException))]
        public void LoadDocTypeItemWithWrongId()
        {
            DocTypeAccess dta = new DocTypeAccess(_docCircAdmin, _dbCon);
            int wrongId = -100;
            bool loadRouteItems = true;
            DocTypeItem docTypeItem = dta.LoadDocTypeItem(wrongId, !loadRouteItems);
        }

        [Test]
        public void LoadDocTypeItemSecurityRoNotR()
        {
            // disable read permission to rootDocTypeFolder for role Users 
            MySqlCommand cmd = new MySqlCommand(
                "update FolderPermission set R=?R where DcRole=@usersRoleId and Folder=@docTypeFolder;",
                _dbCon.MySqlConnection
                );
            cmd.Parameters.Add("?R", MySqlDbType.Bit);
            cmd.Parameters["?R"].Value = false;
            cmd.ExecuteNonQuery();

            try
            {
                //SecurityAccess sa = new SecurityAccess(_user2, _dbCon);
                //sa.CreateFolderSecurityTable();
                DocTypeAccess dta = new DocTypeAccess(_user2, _dbCon);
                bool loadRouteItems = true;
                DocTypeItem item = dta.LoadDocTypeItem(_docType2, loadRouteItems); //tested method
                Assert.AreEqual(_docType2, item.Id);
                Assert.AreEqual(2, item.RoutePoints.Count);
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                // enable read permission to RootDocTypeFolder for role Users 
                cmd.Parameters["?R"].Value = true;
                cmd.ExecuteNonQuery();
            }
        }

        [Test]
        public void SaveDocTypeItemSecurityUoNotU()
        {
            // disable read permission to rootDocTypeFolder for role Users 
            MySqlCommand cmd = new MySqlCommand(
                "update FolderPermission set UO=?UO where DcRole=@usersRoleId and Folder=@docTypeFolder;",
                _dbCon.MySqlConnection
                );
            cmd.Parameters.Add("?UO", MySqlDbType.Bit);
            cmd.Parameters["?UO"].Value = true;
            cmd.ExecuteNonQuery();

            try
            {
                //SecurityAccess sa = new SecurityAccess(_user2, _dbCon);
                //sa.CreateFolderSecurityTable();
                DocTypeAccess dta = new DocTypeAccess(_user2, _dbCon);
                bool loadRouteItems = true;
                DocTypeItem item = dta.LoadDocTypeItem(_docType2, loadRouteItems);
                dta.SaveDocTypeItem(item); //tested method
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                // enable read permission to RootDocTypeFolder for role Users 
                cmd.Parameters["?UO"].Value = false;
                cmd.ExecuteNonQuery();
            }
        }

        [Test]
        public void GetDocTypes()
        {
            DocTypeAccess dta = new DocTypeAccess(_docCircAdmin, _dbCon);
            DocTypeItem[] dtItems = dta.GetDocTypes(); // tested method
            
            Assert.IsNotNull(dtItems);

            Assert.AreEqual(2, dtItems.Length);
            foreach (DocTypeItem item in dtItems)
            {
                if (item.Id == _docType1)
                {
                    Assert.AreEqual(_testDocTypeFolder, item.Folder);
                    Assert.AreEqual("Test type first", item.Name);
                    Assert.AreEqual("first type", item.Description);
                    Assert.AreEqual("TTF", item.Prefix);
                    Assert.AreEqual(1, item.NextNumber);
                    Assert.AreEqual(false, item.IsNew);
                    Assert.AreEqual(_docCircAdmin, item.Owner);
                }
                else if (item.Id == _docType2)
                {
                    Assert.AreEqual(_rootDocTypeFolder, item.Folder);
                    Assert.AreEqual("Test type second", item.Name);
                    Assert.AreEqual("second type", item.Description);
                    Assert.AreEqual("TTS", item.Prefix);
                    Assert.AreEqual(1, item.NextNumber);
                    Assert.AreEqual(false, item.IsNew);
                    Assert.AreEqual(_user2, item.Owner);
                }
                else
                {
                    Assert.Fail("Unknown DocType id!");
                }
            }

            dtItems = dta.GetDocTypes(_rootDocTypeFolder);
            Assert.IsNotNull(dtItems);
            Assert.AreEqual(1, dtItems.Length);
            Assert.AreEqual(_docType2, dtItems[0].Id);
        }

        [Test]
        public void CreateDocType()
        {
            DocTypeAccess dta = new DocTypeAccess(_docCircAdmin, _dbCon);

            DocTypeItem docTypeItem = dta.CreateDocType(_testDocTypeFolder); //Tested method

            Assert.Greater(docTypeItem.Id, 0);
            Assert.IsTrue(docTypeItem.IsNew);
            Assert.AreEqual(_testDocTypeFolder, docTypeItem.Folder);

            MySqlCommand cmd = new MySqlCommand(
                "select Folder, IsNew, DcUser from DocType where DocTypeId=?DocTypeId",
                _dbCon.MySqlConnection
                );
            cmd.Parameters.Add("?DocTypeId", MySqlDbType.Int32);
            cmd.Parameters["?DocTypeId"].Value = docTypeItem.Id;
            MySqlDataReader reader = cmd.ExecuteReader();

            Assert.IsTrue(reader.Read());
            Assert.IsTrue(reader.GetBoolean("IsNew"));
            Assert.AreEqual(_testDocTypeFolder, reader.GetInt32("Folder"));
            Assert.AreEqual(_docCircAdmin, reader.GetInt32("DcUser"));

            reader.Close();

            cmd.CommandText =
                "select dtp.DcRole as DcRole from DocTypePermission as dtp " +
                "right join NewDocTypePermission as newDtp " +
                "on dtp.DcRole=newDtp.DcRole and dtp.DocType=?DocTypeId " +
                "and dtp.C=newDtp.C and dtp.R=newDtp.R and dtp.U=newDtp.U and dtp.D=newDtp.D and " +
                "dtp.RO=newDtp.RO and dtp.UO=newDtp.UO and dtp.DO=newDtp.DO;";
            reader = cmd.ExecuteReader();
            try
            {
                Assert.IsTrue(reader.Read());
                do
                {
                    Assert.IsFalse(reader.IsDBNull(0));
                }
                while (reader.Read());
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }
            }


        }

        [Test]
        public void SaveDocTypeItem()
        {

            DocTypeAccess dta = new DocTypeAccess(_docCircAdmin, _dbCon);

            DocTypeItem dtItem = dta.CreateDocType(_rootDocTypeFolder);
                        
            dtItem.Name = "Saved test DocType1";
            dtItem.NextNumber = 1;
            dtItem.Prefix = "STDT1";
            dtItem.Description = "Description about DocType";

            RoutePoint rp = new RoutePoint(null, "[0]&&[1]", 1);
            
            RouteItem routeItem = new RouteItem();
            routeItem.User = _user1;
            routeItem.RouteItemType = DBConstKeeper.Current.RouteItemType.Decision;
            rp.RouteItems.Add(routeItem);

            routeItem = new RouteItem();
            routeItem.User = _user3;
            routeItem.RouteItemType = DBConstKeeper.Current.RouteItemType.Decision;
            rp.RouteItems.Add(routeItem);
            dtItem.RoutePoints.Add(rp);


            rp = new RoutePoint(null, "[0]", 2);
            
            routeItem = new RouteItem();
            routeItem.User = _user2;
            routeItem.RouteItemType = DBConstKeeper.Current.RouteItemType.Decision;
            rp.RouteItems.Add(routeItem);
            dtItem.RoutePoints.Add(rp);

            dta.SaveDocTypeItem(dtItem); //Tested method
            

            // chech DocType table
            MySqlCommand cmd = new MySqlCommand(
                "select Name, NextNumber, Prefix, Description, IsNew, Folder, DcUser from DocType" +
                " where DocTypeId=?DocTypeId;",
                _dbCon.MySqlConnection
                );
            cmd.Parameters.Add("?DocTypeId", MySqlDbType.Int32);
            cmd.Parameters["?DocTypeId"].Value = dtItem.Id;
            MySqlDataReader reader = cmd.ExecuteReader();

            try
            {
                Assert.IsTrue(reader.Read());
                Assert.IsFalse(reader.GetBoolean("IsNew"));
                Assert.AreEqual(_rootDocTypeFolder, reader.GetInt32("Folder"));
                Assert.AreEqual("Saved test DocType1", reader.GetString("Name"));
                Assert.AreEqual(1, reader.GetInt32("NextNumber"));
                Assert.AreEqual("STDT1", reader.GetString("Prefix"));
                Assert.AreEqual("Description about DocType", reader.GetString("Description"));
                Assert.AreEqual(_docCircAdmin, reader.GetInt32("DcUser"));
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }
            }
            
            //chech RoutePoint table
            cmd.CommandText = "select RoutePointId, DocType, OrderNum, Expression from RoutePoint " +
                "where DocType=?DocTypeId order by OrderNum;";
            reader = cmd.ExecuteReader();
            int[] routePointIds = new int[2];

            try
            {
                Assert.IsTrue(reader.Read(), "first route point not found");
                routePointIds[0] = reader.GetInt32("RoutePointId");
                Assert.AreEqual(dtItem.Id, reader.GetInt32("DocType"), "wrong Id");
                Assert.AreEqual(1, reader.GetInt32("OrderNum"), "wrong order num");
                Assert.AreEqual("[0]&&[1]", reader.GetString("Expression"), "wrong expression");

                Assert.IsTrue(reader.Read(), "second route point not found");
                routePointIds[1] = reader.GetInt32("RoutePointId");
                Assert.AreEqual(dtItem.Id, reader.GetInt32("DocType"));
                Assert.AreEqual(2, reader.GetInt32("OrderNum"));
                Assert.AreEqual("[0]", reader.GetString("Expression"));
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }
            }

            // check RouteItem table
            cmd.CommandText = "select DcUser, RoutePoint, RouteItemType, ItemIndex from RouteItem " +
                "where DocType=?DocTypeId order by RoutePoint, ItemIndex;";
            reader = cmd.ExecuteReader();
            try
            {
                Assert.IsTrue(reader.Read());
                Assert.AreEqual(_user1, reader.GetInt32("DcUser"));
                Assert.AreEqual(routePointIds[0], reader.GetInt32("RoutePoint"));
                Assert.AreEqual(
                    DBConstKeeper.Current.RouteItemType.Decision,
                    reader.GetInt32("RouteItemType")
                    );
                Assert.AreEqual(0, reader.GetInt32("ItemIndex"));

                Assert.IsTrue(reader.Read());
                Assert.AreEqual(_user3, reader.GetInt32("DcUser"));
                Assert.AreEqual(routePointIds[0], reader.GetInt32("RoutePoint"));
                Assert.AreEqual(
                    DBConstKeeper.Current.RouteItemType.Decision,
                    reader.GetInt32("RouteItemType")
                    );
                Assert.AreEqual(1, reader.GetInt32("ItemIndex"));

                Assert.IsTrue(reader.Read());
                Assert.AreEqual(_user2, reader.GetInt32("DcUser"));
                Assert.AreEqual(routePointIds[1], reader.GetInt32("RoutePoint"));
                Assert.AreEqual(
                    DBConstKeeper.Current.RouteItemType.Decision,
                    reader.GetInt32("RouteItemType")
                    );
                Assert.AreEqual(0, reader.GetInt32("ItemIndex"));

                Assert.IsFalse(reader.Read());
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }
            }

            //delete RoutePoint with OrderNum=2 in DocType, RoutePoint with OrderNum=1 must leave
            bool loadRoutePoints = true;
            dtItem = dta.LoadDocTypeItem(dtItem.Id, loadRoutePoints);
            dtItem.RoutePoints.Remove(dtItem.RoutePoints[1]);
            
            dta.SaveDocTypeItem(dtItem);

            reader = cmd.ExecuteReader();
            try
            {
                Assert.IsTrue(reader.Read());
                Assert.AreEqual(_user1, reader.GetInt32("DcUser"));
                Assert.AreEqual(routePointIds[0], reader.GetInt32("RoutePoint"));
                Assert.AreEqual(
                    DBConstKeeper.Current.RouteItemType.Decision,
                    reader.GetInt32("RouteItemType")
                    );
                Assert.AreEqual(0, reader.GetInt32("ItemIndex"));

                Assert.IsTrue(reader.Read());
                Assert.AreEqual(_user3, reader.GetInt32("DcUser"));
                Assert.AreEqual(routePointIds[0], reader.GetInt32("RoutePoint"));
                Assert.AreEqual(
                    DBConstKeeper.Current.RouteItemType.Decision,
                    reader.GetInt32("RouteItemType")
                    );
                Assert.AreEqual(1, reader.GetInt32("ItemIndex"));

                Assert.IsFalse(reader.Read());
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }
            }
            
        }

        [Test]
        public void DeleteDocType()
        {
            DocTypeAccess dta = new DocTypeAccess(_docCircAdmin, _dbCon);
            DocTypeItem item = dta.CreateDocType(_rootDocTypeFolder);
            item.RoutePoints.Add(new RoutePoint(null, "[0]", 1));
            item.RoutePoints[0].RouteItems.Add(
                new RouteItem(null, _user1, DBConstKeeper.Current.RouteItemType.Decision)
                );
            item.RoutePoints.Add(new RoutePoint(null, "[1]", 2));
            item.RoutePoints[0].RouteItems.Add(
                new RouteItem(null, _user2, DBConstKeeper.Current.RouteItemType.Decision)
                );

            dta.SaveDocTypeItem(item);

            dta.DeleteDocType(item.Id);

            MySqlCommand cmd = new MySqlCommand(
                "select 1 from DocType where DocTypeId=?DocTypeId union " +
                "select 1 from RoutePoint where DocType=?DocTypeId union " +
                "select 1 from RouteItem where DocType=?DocTypeId;",
                _dbCon.MySqlConnection
                );

            cmd.Parameters.Add("?DocTypeId", MySqlDbType.Int32);
            cmd.Parameters["?DocTypeId"].Value = item.Id;

            MySqlDataReader reader = cmd.ExecuteReader();
            bool exists = reader.Read();
            if (!reader.IsClosed)
            {
                reader.Close();
            }

            Assert.IsFalse(exists);
        }

        [Test]
        [ExpectedException(ExpectedException=typeof(DcUnathorizedAccessException))]
        public void DeleteDocTypeWithUnauthorizedAccess()
        {
            //SecurityAccess sa = new SecurityAccess(_user1, _dbCon);
            //sa.CreateFolderSecurityTable();
            DocTypeAccess dta = new DocTypeAccess(_user1, _dbCon);
            dta.DeleteDocType(_docType1);
        }

        [Test]
        public void DeleteDocTypeWithDoNotD()
        {
            MySqlCommand cmd = new MySqlCommand(
                "update FolderPermission set DO=?DO, C=?C, U=?U where Folder=@testDocTypeFolder1 and DcRole=@usersRoleId;",
                _dbCon.MySqlConnection
                );
            cmd.Parameters.Add("?DO", MySqlDbType.Int32);
            cmd.Parameters.Add("?C", MySqlDbType.Int32);
            cmd.Parameters.Add("?U", MySqlDbType.Int32);

            cmd.Parameters["?DO"].Value = true;
            cmd.Parameters["?C"].Value = true;
            cmd.Parameters["?U"].Value = true;
            cmd.ExecuteNonQuery();

            //SecurityAccess sa = new SecurityAccess(_user2, _dbCon);
            //sa.CreateFolderSecurityTable();
            DocTypeAccess dta = new DocTypeAccess(_user2, _dbCon);
            DocTypeItem itemForDeletion = dta.CreateDocType(_testDocTypeFolder);
            dta.SaveDocTypeItem(itemForDeletion);
            bool isException = false;

            try
            {
                dta.DeleteDocType(itemForDeletion.Id); //tested method
            }
            catch (Exception ex)
            {
                isException = true;
                throw ex;
            }
            finally
            {
                if (isException)
                {
                    //sa = new SecurityAccess(_docCircAdmin, _dbCon);
                    //sa.CreateFolderSecurityTable();
                    dta = new DocTypeAccess(_docCircAdmin, _dbCon);
                    dta.DeleteDocType(itemForDeletion.Id);
                }
                
                cmd.Parameters["?C"].Value = false;
                cmd.Parameters["?DO"].Value = false;
                cmd.Parameters["?U"].Value = false;
                cmd.ExecuteNonQuery();
            }
        }

        [Test]
        [ExpectedException(ExpectedException=typeof(DcObjectNotFoundException))]
        public void DeleteDocTypeWithWrongId()
        {
            DocTypeAccess dta = new DocTypeAccess(_docCircAdmin, _dbCon);
            int wrongId = -999;
            dta.DeleteDocType(wrongId);
        }
        
        /// <summary>
        /// all route items in RouteItem table is updated with the same values when saving existed before route items
        /// </summary>
        [Test]
        public void BugFixUpdateAllRouteItems()
        {
            DocTypeAccess dta = new DocTypeAccess(_docCircAdmin, _dbCon);
            DocTypeItem item = dta.CreateDocType(_rootDocTypeFolder);
            item.RoutePoints.Add(new RoutePoint(null, "[0]", 1));
            item.RoutePoints[0].RouteItems.Add(
                new RouteItem(null, _user1, DBConstKeeper.Current.RouteItemType.Decision)
                );
            item.RoutePoints.Add(new RoutePoint(null, "[1]", 2));
            item.RoutePoints[1].RouteItems.Add(
                new RouteItem(null, _user2, DBConstKeeper.Current.RouteItemType.Decision)
                );
            dta.SaveDocTypeItem(item);

            bool loadRoutePoints = true;
            item = dta.LoadDocTypeItem(item.Id, loadRoutePoints);
            dta.SaveDocTypeItem(item);

            item = dta.LoadDocTypeItem(item.Id, loadRoutePoints);

            Assert.AreEqual(2, item.RoutePoints.Count);
            Assert.AreEqual(1, item.RoutePoints[0].RouteItems.Count);
            Assert.AreEqual(1, item.RoutePoints[1].RouteItems.Count);

            Assert.AreNotEqual(item.RoutePoints[0].OrderNum, item.RoutePoints[1].OrderNum);
            Assert.AreNotEqual(item.RoutePoints[0].RouteItems[0].User, item.RoutePoints[1].RouteItems[0].User);


            Assert.AreEqual(1, item.RoutePoints[0].OrderNum);
            Assert.AreEqual(_user1, item.RoutePoints[0].RouteItems[0].User);

            Assert.AreEqual(2, item.RoutePoints[1].OrderNum);
            Assert.AreEqual(_user2, item.RoutePoints[1].RouteItems[0].User);
            

            dta.DeleteDocType(item.Id);
        }


        /// <summary>
        /// When loading routePoint (RP) OrderNum=1 , RP OrderNum=3, an index out of range exception is thrown
        /// There must not be such docTypes to load
        /// </summary>
        //[Test]
        //[Ignore]
        //public void BugFixLoadWithGapInRoutePoints()
        //{
        //    DocTypeAccess dta = new DocTypeAccess(_docCircAdmin, _dbCon);
        //    DocTypeItem item = dta.CreateDocType(_testDocTypeFolder);
        //    item.Name = "BugFixSaveWithGapInRoutePoints";
            
        //    item.RoutePoints.Add(new RoutePoint(null, "[0]", 1));
        //    item.RoutePoints[0].RouteItems.Add(
        //        new RouteItem(null, _user1, DBConstKeeper.Current.RouteItemTypeDirectory.Decision)
        //        );

        //    item.RoutePoints.Add(new RoutePoint(null, "[0]", 3));
        //    item.RoutePoints[1].RouteItems.Add(
        //        new RouteItem(null, _user1, DBConstKeeper.Current.RouteItemTypeDirectory.Decision)
        //        );

        //    dta.SaveDocTypeItem(item);

        //    //try
        //    //{
        //    item = dta.LoadDocTypeItem(item.Id, true);
        //    //}
        //    //catch (Exception ex)
        //    //{
        //    //    throw ex;
        //    //}
        //    //finally
        //    //{
        //    dta.DeleteDocType(item.Id);
        //    //}
        //}

        [Test]
        [ExpectedException(ExpectedException=typeof(DcUnathorizedAccessException))]
        public void CreateDocTypeWithUnauthorizeAccess()
        {
            //SecurityAccess sa = new SecurityAccess(_user1, _dbCon);
            //sa.CreateFolderSecurityTable();
            DocTypeAccess dta = new DocTypeAccess(_user1, _dbCon);
            DocTypeItem item = dta.CreateDocType(_rootDocTypeFolder);
        }

        [Test]
        [ExpectedException(ExpectedException = typeof(DcUnathorizedAccessException))]
        public void LoadDocTypeItemWithUnauthorizedAccess()
        {
            // disable read permission to TestDocTypeFolder1 for role Users 
            MySqlCommand cmd = new MySqlCommand(
                "update FolderPermission set R=?R where DcRole=@usersRoleId and Folder=@testDocTypeFolder1;",
                _dbCon.MySqlConnection
                );
            cmd.Parameters.Add("?R", MySqlDbType.Bit);
            cmd.Parameters["?R"].Value = false;
            cmd.ExecuteNonQuery();

            //SecurityAccess sa = new SecurityAccess(_user1, _dbCon);
            //sa.CreateFolderSecurityTable();
            DocTypeAccess dta = new DocTypeAccess(_user1, _dbCon);
            bool loadRouteItems = true;
            try
            {
                DocTypeItem item = dta.LoadDocTypeItem(_docType1, loadRouteItems); //tested method
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                // enable read permission to TestDocTypeFolder1 for role Users 
                cmd.Parameters["?R"].Value = true;
                cmd.ExecuteNonQuery();
            }
        }

        [Test]
        [ExpectedException(ExpectedException = typeof(DcUnathorizedAccessException))]
        public void SaveDocTypeItemWithUnauthorizedAccess()
        {
            //SecurityAccess sa = new SecurityAccess(_user1, _dbCon);
            //sa.CreateFolderSecurityTable();
            DocTypeAccess dta = new DocTypeAccess(_user1, _dbCon);
            bool loadRouteItems = true;
            DocTypeItem item = dta.LoadDocTypeItem(_docType2, loadRouteItems);
            dta.SaveDocTypeItem(item); // tested method
        }

        [Test]
        public void GetDocTypesWithUnauthorizedAccess()
        {
            DocTypeAccess dta = new DocTypeAccess(_docCircAdmin, _dbCon);
            DocTypeItem newItemInRoot = dta.CreateDocType(_testDocTypeFolder);
            newItemInRoot.Name = "ForGetDocTypesWithUnauthorizedAccessItem";
            dta.SaveDocTypeItem(newItemInRoot);

            // disable read permission to RootDocTypeFolder for role Users 
            MySqlCommand cmd = new MySqlCommand(
                "update FolderPermission set R=?R, RO=?RO where DcRole=@usersRoleId and Folder=@docTypeFolder;",
                _dbCon.MySqlConnection
                );
            cmd.Parameters.Add("?R", MySqlDbType.Bit);
            cmd.Parameters["?R"].Value = false;
            cmd.Parameters.Add("?RO", MySqlDbType.Bit);
            cmd.Parameters["?RO"].Value = false;
            cmd.ExecuteNonQuery();

            //SecurityAccess sa = new SecurityAccess(_user2, _dbCon);
            //sa.CreateFolderSecurityTable();

            dta = new DocTypeAccess(_user2, _dbCon);
            try
            {
                DocTypeItem[] items = dta.GetDocTypes(); //tested method
                Assert.AreEqual(2, items.Length);
                bool[] been = new bool[2] { false, false };
                for (int i = 0; i < 2; i++)
                {
                    DocTypeItem item = items[i];
                    if (item.Id == _docType1 && !been[0])
                    {
                        been[0] = true;
                        Assert.AreEqual("Test type first", item.Name);
                    }
                    else if (item.Id == newItemInRoot.Id && !been[1])
                    {
                        been[1] = true;
                        Assert.AreEqual("ForGetDocTypesWithUnauthorizedAccessItem", item.Name);
                    }
                    else
                    {
                        Assert.Fail(
                            String.Format(
                                "Unexpected or repeated doctype! Id={0} Name='{1}'", 
                                item.Id,
                                item.Name
                                )
                                );
                    }
                }

                items = dta.GetDocTypes(_rootDocTypeFolder);
                Assert.AreEqual(0, items.Length);
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                // enable read permission to TestDocTypeFolder1 for role Users 
                cmd.Parameters["?R"].Value = true;
                cmd.Parameters["?RO"].Value = true;
                cmd.ExecuteNonQuery();
                //sa = new SecurityAccess(_docCircAdmin, _dbCon);
                //sa.CreateFolderSecurityTable();
                dta = new DocTypeAccess(_docCircAdmin, _dbCon);
                dta.DeleteDocType(newItemInRoot.Id);
            }
        }

        [Test]
        public void GetDocTypeWithCreatePermission()
        {
            MySqlCommand cmd = new MySqlCommand(
                "update DocTypePermission set C=?C where DocType=@docType1 and DcRole=@usersRoleId;",
                _dbCon.MySqlConnection
                );

            cmd.Parameters.Add("?C", MySqlDbType.Bit);
            cmd.Parameters["?C"].Value = false;

            cmd.ExecuteNonQuery();

            
            try
            {
                bool useTmpDocTypeSecurityTable = false;
                
                DocTypeAccess dta = new DocTypeAccess(_user1, _dbCon); // user 1
                DocTypeItem[] items = dta.GetDocTypesWithCreatePermission(useTmpDocTypeSecurityTable); // tested method
                Assert.IsNotNull(items);
                Assert.AreEqual(1, items.Length);

                Assert.AreEqual(_docType2, items[0].Id);
                Assert.AreEqual("Test type second", items[0].Name);
                Assert.AreEqual("second type", items[0].Description);
                Assert.AreEqual("TTS", items[0].Prefix);

                dta = new DocTypeAccess(_docCircAdmin, _dbCon);
                items = dta.GetDocTypesWithCreatePermission(useTmpDocTypeSecurityTable); // tested method
                Assert.IsNotNull(items);
                Assert.AreEqual(2, items.Length);
                
                bool[] been = new bool[] { false, false };
                foreach (DocTypeItem item in items)
                {
                    if (item.Id == _docType1 && !been[0])
                    {
                        been[0] = true;
                    }
                    else if (item.Id == _docType2 && !been[1])
                    {
                        been[1] = true;
                    }
                }

                Assert.IsTrue(been[0]);
                Assert.IsTrue(been[1]);

            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                cmd.Parameters["?C"].Value = true;
                cmd.ExecuteNonQuery();
            }

        }


        [Test]
        public void PrefixExists()
        {
            DocTypeAccess dta = new DocTypeAccess(_docCircAdmin, _dbCon);
            int[] ttfDocTypes = dta.PrefixExists("TTF");
            Assert.AreEqual(1, ttfDocTypes.Length);
            Assert.AreEqual(_docType1, ttfDocTypes[0]);

            int[] ttsDocTypes = dta.PrefixExists("TTS");
            Assert.AreEqual(1, ttsDocTypes.Length);
            Assert.AreEqual(_docType2, ttsDocTypes[0]);

            int[] unknownDocTypes = dta.PrefixExists("UNKNOWNPREFIX");
            Assert.AreEqual(0, unknownDocTypes.Length);
        }


        [TestFixtureTearDown]
        public void TFTearDown()
        {
            _tran.Rollback();
            _dbCon.Close();

            DBConstKeeper.Current = null;
        }

    }
}
