//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using MySql.Data.MySqlClient;
using NUnit.Framework;

using Evpo.DocCirc.Common;
using Evpo.DocCirc.MySqlDL;

namespace Evpo.DocCircUnitTest
{
    [TestFixture]
    public class EventAccessTest
    {
        public struct FixtureFile
        {
            public const string Folder = "MySql";
            public const string CreatingDocCards = "EventAccessTest.file1.sql";
            public const string CreatingEvents = "EventAccessTest.file2.sql";
        }

        public class FixtureConstant
        {
            public long DocCard1, DocCard2;
            public int DocCircAdmin, DocCircUser1, DocCircUser2;
            public int TestDocumentFolder1;
            public long DocCard1Event1, DocCard1Event2, DocCard1Event3, DocCard2Event1;
        }

        private FixtureConstant _const = new FixtureConstant();

        private DBConnection _con;
        private MySqlTransaction _tran;
        private SettingKeeper _sk;

        [TestFixtureSetUp]
        public void TFSetUp()
        {
            _sk = new SettingKeeper();

            _con = new DBConnection(
            "Database=EvpoDCTest;Data Source=localhost;User Id=doccirctest;Password=doccirctest"
            );
            _con.Open();
            _tran = _con.MySqlConnection.BeginTransaction();


            MySqlCommand cmd = new MySqlCommand();
            cmd.Connection = _con.MySqlConnection;
            TextReader txtReader = File.OpenText(Path.Combine(FixtureFile.Folder, FixtureFile.CreatingDocCards));
            cmd.CommandText = txtReader.ReadToEnd();
            txtReader.Close();
            cmd.ExecuteNonQuery();

            txtReader = File.OpenText(Path.Combine(FixtureFile.Folder, FixtureFile.CreatingEvents));
            cmd.CommandText = txtReader.ReadToEnd();
            txtReader.Close();
            cmd.ExecuteNonQuery();

            // get consts
            cmd.CommandText =
                "select cast(@docCard1 as unsigned) as DocCard1, cast(@docCard2 as unsigned) as DocCard2, " +
                "cast(@docCircAdmin as unsigned) as DocCircAdmin, " +
                "cast(@docCircUser1 as unsigned) as DocCircUser1, " +
                "cast(@docCircUser2 as unsigned) as DocCircUser2, " +
                "cast(@testDocumentFolder1 as unsigned) as TestDocumentFolder1, " +
                "cast(@docCard1Event1 as unsigned) as DocCard1Event1, " +
                "cast(@docCard1Event2 as unsigned) as DocCard1Event2, " +
                "cast(@docCard1Event3 as unsigned) as DocCard1Event3, " +
                "cast(@docCard2Event1 as unsigned) as DocCard2Event1;";
            MySqlDataReader reader = cmd.ExecuteReader();
            reader.Read();
            _const.DocCard1 = reader.GetInt64("DocCard1");
            _const.DocCard2 = reader.GetInt64("DocCard2");
            _const.DocCircAdmin = reader.GetInt32("DocCircAdmin");
            _const.DocCircUser1 = reader.GetInt32("DocCircUser1");
            _const.DocCircUser2 = reader.GetInt32("DocCircUser2");
            _const.TestDocumentFolder1 = reader.GetInt32("TestDocumentFolder1");
            _const.DocCard1Event1 = reader.GetInt64("DocCard1Event1");
            _const.DocCard1Event2 = reader.GetInt64("DocCard1Event2");
            _const.DocCard1Event3 = reader.GetInt64("DocCard1Event3");
            _const.DocCard2Event1 = reader.GetInt64("DocCard2Event1");
            reader.Close();

            DBConstAccess ca = new DBConstAccess(_con);
            FolderTypeDirectory ftd = ca.GetFolderTypeDirectory();
            DBConstKeeper.Current = new DBConstKeeper(
                ftd,
                ca.GetRootFolderDirectory(ftd),
                ca.GetMainRoleDirectory(),
                ca.GetRouteItemTypeDirectory(),
                ca.GetDocStatusDirectory(),
                ca.GetNotificationTypeDirectory(),
                ca.GetEventTypeDirectory(),
                ca.GetCommonDepartment()
                );
        }

        [Test]
        public void GetDocCardEvents()
        {
            EventAccess ea = new EventAccess(_const.DocCircAdmin, _con);
            EventItem[] items = ea.GetDocCardEventItems(_const.DocCard1); // tested method

            Assert.AreEqual(3, items.Length);
            bool[] been = new bool[] { false, false, false };
            foreach (EventItem item in items)
            {
                if (item.Id == _const.DocCard1Event1 && !been[0])
                {
                    been[0] = true;
                    Assert.AreEqual(_const.DocCard1, item.DocCardId);
                    Assert.AreEqual(
                        DBConstKeeper.Current.EventType.DocCardRegistration,
                        item.EventType
                        );
                    Assert.AreEqual(_const.DocCircUser1, item.EventUser);
                    Assert.IsNull(item.Deputy);
                    Assert.AreEqual(0, item.RoutePointNum);
                    Assert.AreEqual(
                        new DateTime(2007, 06, 06, 10, 00, 00),
                        item.EventTime
                        );
                    Assert.AreEqual(
                        "This is a new document card 1",
                        item.Note
                        );
                    Assert.AreEqual("Text1", item.Text);

                }
                else if(item.Id == _const.DocCard1Event2 && !been[1])
                {
                    been[1] = true;
                    Assert.AreEqual(_const.DocCard1, item.DocCardId);
                    Assert.AreEqual(
                       DBConstKeeper.Current.EventType.ConsiderationStarted,
                       item.EventType
                       );
                    Assert.AreEqual(_const.DocCircUser1, item.EventUser);
                    Assert.IsNull(item.Deputy);
                    Assert.AreEqual(0, item.RoutePointNum);
                    Assert.AreEqual(
                        new DateTime(2007, 06, 06, 10, 00, 00),
                        item.EventTime
                        );
                    Assert.AreEqual(
                        "Consideration is started",
                        item.Note
                        );
                    Assert.AreEqual("Text2", item.Text);
                }
                else if (item.Id == _const.DocCard1Event3 && !been[2])
                {
                    been[2] = true;
                    Assert.AreEqual(_const.DocCard1, item.DocCardId);
                    Assert.AreEqual(
                       DBConstKeeper.Current.EventType.PassedToNextRoutePoint,
                       item.EventType
                       );
                    Assert.AreEqual(_const.DocCircUser1, item.EventUser);
                    Assert.IsNull(item.Deputy);
                    Assert.AreEqual(1, item.RoutePointNum);
                    Assert.AreEqual(
                        new DateTime(2007, 06, 07, 10, 00, 00),
                        item.EventTime
                        );
                    Assert.AreEqual(
                        "The first route point consideration has begun",
                        item.Note
                        );
                    Assert.AreEqual("Text3", item.Text);

                }
                else
                {
                    Assert.Fail("Unknown or repeated id!");
                }
            }

            items = ea.GetDocCardEventItems(_const.DocCard2); // tested method
            Assert.AreEqual(1, items.Length);
            
            Assert.AreEqual(_const.DocCard2, items[0].DocCardId);
            Assert.AreEqual(
               DBConstKeeper.Current.EventType.DocCardRegistration,
               items[0].EventType
               );
            Assert.AreEqual(_const.DocCircUser2, items[0].EventUser);
            Assert.AreEqual(_const.DocCircUser1, items[0].Deputy);
            Assert.AreEqual(0, items[0].RoutePointNum);
            Assert.AreEqual(
                new DateTime(2007, 06, 10, 11, 00, 00),
                items[0].EventTime
                );
            Assert.AreEqual(
                "This is a new document card 2",
                items[0].Note
                );
            Assert.AreEqual("Text4", items[0].Text);
        }

        [Test]
        public void AddEvent()
        {
            EventAccess ea = new EventAccess(_const.DocCircAdmin, _con);

            EventItem item = new EventItem(
                _const.DocCard2, DBConstKeeper.Current.EventType.PassedToNextRoutePoint, _const.DocCircAdmin,
                _const.DocCircUser1, 1, new DateTime(2007, 06, 06, 16, 42, 00), "this is added event", "new text"
                );

            ea.AddEvent(item); //tested method

            Assert.IsNotNull(item.Id);

            MySqlCommand cmd = new MySqlCommand(
                "select DocCard, EventType, DcUser, Deputy, RoutePointNum, EventTime, Note, Text from " +
                "Event where EventId=?Event;",
                _con.MySqlConnection
                );

            cmd.Parameters.Add("?Event", MySqlDbType.Int64);
            cmd.Parameters["?Event"].Value = item.Id;

            MySqlDataReader reader = cmd.ExecuteReader();

            try
            {
                Assert.IsTrue(reader.Read());
                Assert.AreEqual(_const.DocCard2, reader.GetInt64("DocCard"));
                Assert.AreEqual(
                    DBConstKeeper.Current.EventType.PassedToNextRoutePoint,
                    reader.GetInt32("EventType")
                    );
                Assert.AreEqual(_const.DocCircAdmin, reader.GetInt32("DcUser"));
                Assert.AreEqual(_const.DocCircUser1, reader.GetInt32("Deputy"));
                Assert.AreEqual(1, reader.GetInt32("RoutePointNum"));
                Assert.AreEqual(new DateTime(2007, 06, 06, 16, 42, 00), reader.GetDateTime("EventTime"));
                Assert.AreEqual("this is added event", reader.GetString("Note"));
                Assert.AreEqual("new text", reader.GetString("Text"));
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }
                cmd.CommandText = "delete from event where EventId=?Event;";
                cmd.ExecuteNonQuery();
            }

        }


        [TestFixtureTearDown]
        public void TFTearDown()
        {
            _tran.Rollback();
            _con.Close();
        }

    }
}
