//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using MySql.Data.MySqlClient;
using NUnit.Framework;
using Evpo.DocCirc.MySqlDL;
using Evpo.DocCirc.Common;

namespace Evpo.DocCircUnitTest
{
    [TestFixture]
    public class FolderAccessTest
    {
        protected struct FixtureConst
        {
            public const string Folder = "MySql";
            public const string SqlFile1 = "FolderAccessTest.file1.sql";
        }

        protected class FixtureData
        {
            public static int UserRole, DocCircAdmin, DocCircUser, 
                User1, User2, User3, TestDocTypeFolder, RootDocTypeFolder;
            public static int Fld1, Fld2, Fld3;
        }

        DBConnection _con;
        MySqlTransaction _tran;

        [TestFixtureSetUp]
        public void FSetUp()
        {
            string cmdStr = null;
            using (
                TextReader tr = File.OpenText(
                    Path.Combine(FixtureConst.Folder, FixtureConst.SqlFile1)
                    )
                )
            {
                cmdStr = tr.ReadToEnd();
                tr.Close();
            }

            _con = new DBConnection(
               "Database=EvpoDCTest;Data Source=localhost;User Id=doccirctest;Password=doccirctest"
               );
            _con.Open();

            _tran = _con.MySqlConnection.BeginTransaction();

            MySqlCommand cmd = new MySqlCommand(cmdStr, _con.MySqlConnection);
            cmd.ExecuteNonQuery();
            cmd.CommandText =
                "select " +
                "cast(@user1 as unsigned), cast(@user2 as unsigned), cast(@user3 as unsigned)," +
                "cast(@testDocTypeFolder1 as unsigned), " +
                "cast(@docTypeFolder as unsigned), cast(@usersRoleId as unsigned), " +
                "cast(@docCircAdminId as unsigned), cast(@docCircUserId as unsigned);";
            MySqlDataReader reader = cmd.ExecuteReader();
            reader.Read();
            FixtureData.User1 = reader.GetInt32(0);
            FixtureData.User2 = reader.GetInt32(1);
            FixtureData.User3 = reader.GetInt32(2);
            FixtureData.TestDocTypeFolder = reader.GetInt32(3);
            FixtureData.RootDocTypeFolder = reader.GetInt32(4);
            FixtureData.UserRole = reader.GetInt32(5);
            FixtureData.DocCircAdmin = reader.GetInt32(6);
            FixtureData.DocCircUser = reader.GetInt32(7);
            reader.Close();

            DBConstAccess dca = new DBConstAccess(_con);
            FolderTypeDirectory ftd = dca.GetFolderTypeDirectory();
            DBConstKeeper.Current = new DBConstKeeper(
                ftd,
                dca.GetRootFolderDirectory(ftd),
                dca.GetMainRoleDirectory(),
                dca.GetRouteItemTypeDirectory(),
                dca.GetDocStatusDirectory(),
                dca.GetNotificationTypeDirectory(),
                dca.GetEventTypeDirectory(),
                dca.GetCommonDepartment()
                );


            // create folder hierarchy
            MySqlCommand permCmd = new MySqlCommand(
                "insert into FolderPermission (DcRole, Folder, C, R, U, D, B, RO, UO, DO) " +
                "select DcRoleId, ?Folder,true,true,true,true,true,true,true,true " +
                "from DcRole where RoleName='Administrators';" +
                "insert into FolderPermission (DcRole, Folder, C, R, U, D, B, RO, UO, DO) " +
                "select DcRoleId, ?Folder,false,true,false,false,?B,true,false,false " +
                "from DcRole where RoleName='Users';"
                ,
                _con.MySqlConnection
                );
            permCmd.Parameters.Add("?Folder", MySqlDbType.Int32);
            permCmd.Parameters.Add("?B", MySqlDbType.Bit);

            cmd = new MySqlCommand(
                "insert into folder (FolderId, ParentFolder, FolderType, Name)" +
                " Values (default, ?ParentFolder, ?FolderType, ?FolderName);"
                ,
                _con.MySqlConnection
                );
            cmd.Parameters.Add("?ParentFolder", MySqlDbType.Int32);
            cmd.Parameters.Add("?FolderType", MySqlDbType.Int32);
            cmd.Parameters.Add("?FolderName", MySqlDbType.String);
            cmd.Parameters["?FolderType"].Value = DBConstKeeper.Current.FolderType.DocType;

            /***************************
             * Test DocType folder1
             * fld1
             *   fld11 // for Users B=false
             * fld2
             *   fld21
             *   fld22 // for Users B=false
             *     fld221
             * fld3 // for Users B=false
             * *************************/

            cmd.Parameters["?FolderName"].Value = "fld1";
            cmd.Parameters["?ParentFolder"].Value = DBConstKeeper.Current.RootFolder.DocType;
            cmd.ExecuteNonQuery();
            long curParent = cmd.LastInsertedId;
            FixtureData.Fld1 = (int)cmd.LastInsertedId;
            permCmd.Parameters["?Folder"].Value = cmd.LastInsertedId;
            permCmd.Parameters["?B"].Value = true;
            permCmd.ExecuteNonQuery();

            cmd.Parameters["?FolderName"].Value = "fld11";
            cmd.Parameters["?ParentFolder"].Value = curParent;
            cmd.ExecuteNonQuery();
            permCmd.Parameters["?Folder"].Value = cmd.LastInsertedId;
            permCmd.Parameters["?B"].Value = false;
            permCmd.ExecuteNonQuery();

            cmd.Parameters["?FolderName"].Value = "fld2";
            cmd.Parameters["?ParentFolder"].Value = DBConstKeeper.Current.RootFolder.DocType;
            cmd.ExecuteNonQuery();
            curParent = cmd.LastInsertedId;
            FixtureData.Fld2 = (int)cmd.LastInsertedId;
            permCmd.Parameters["?Folder"].Value = cmd.LastInsertedId;
            permCmd.Parameters["?B"].Value = true;
            permCmd.ExecuteNonQuery();

            cmd.Parameters["?FolderName"].Value = "fld21";
            cmd.Parameters["?ParentFolder"].Value = curParent;
            cmd.ExecuteNonQuery();
            permCmd.Parameters["?Folder"].Value = cmd.LastInsertedId;
            permCmd.Parameters["?B"].Value = true;
            permCmd.ExecuteNonQuery();

            cmd.Parameters["?FolderName"].Value = "fld22";
            cmd.Parameters["?ParentFolder"].Value = curParent;
            cmd.ExecuteNonQuery();
            curParent = cmd.LastInsertedId;
            permCmd.Parameters["?Folder"].Value = cmd.LastInsertedId;
            permCmd.Parameters["?B"].Value = false;
            permCmd.ExecuteNonQuery();
            
            cmd.Parameters["?FolderName"].Value = "fld221";
            cmd.Parameters["?ParentFolder"].Value = curParent;
            cmd.ExecuteNonQuery();
            permCmd.Parameters["?Folder"].Value = cmd.LastInsertedId;
            permCmd.ExecuteNonQuery();

            cmd.Parameters["?FolderName"].Value = "fld3";
            cmd.Parameters["?ParentFolder"].Value = DBConstKeeper.Current.RootFolder.DocType;
            cmd.ExecuteNonQuery();
            FixtureData.Fld3 = (int)cmd.LastInsertedId;
            permCmd.Parameters["?Folder"].Value = cmd.LastInsertedId;
            permCmd.Parameters["?B"].Value = false;
            permCmd.ExecuteNonQuery();
        }

        [Test]
        public void GetFolderHierarchy()
        {

            FolderAccess fa = new FolderAccess(FixtureData.DocCircAdmin, _con);
            DcFolder folder = fa.GetFolderHierarchy(DBConstKeeper.Current.FolderType.DocType); //Tested method

            Assert.IsNotNull(folder);
            Assert.IsNull(folder.Parent);
            Assert.AreEqual(4, folder.Folders.Length);
            Assert.AreEqual("^DocTypeFolderName^", folder.Name);

            DcFolder testDTFolder = folder.Folders[0];
            Assert.AreEqual(folder.Id, testDTFolder.Parent);
            Assert.AreEqual(0, testDTFolder.Folders.Length);
            Assert.AreEqual("Test DocType folder1", testDTFolder.Name);

            DcFolder fld1 = folder.Folders[1];
            Assert.AreEqual("fld1", fld1.Name);
            Assert.AreEqual(1, fld1.Folders.Length);
            Assert.AreEqual("fld11", fld1.Folders[0].Name);

            DcFolder fld2 = folder.Folders[2];
            Assert.AreEqual("fld2", fld2.Name);
            Assert.AreEqual(2, fld2.Folders.Length);
            Assert.AreEqual("fld21", fld2.Folders[0].Name);
            Assert.AreEqual("fld22", fld2.Folders[1].Name);
            Assert.AreEqual(1, fld2.Folders[1].Folders.Length);
            Assert.AreEqual("fld221", fld2.Folders[1].Folders[0].Name);

            DcFolder fld3 = folder.Folders[3];
            Assert.AreEqual("fld3", fld3.Name);
            Assert.AreEqual(0, fld3.Folders.Length);

            MySqlCommand permCmd = new MySqlCommand(
               "insert into FolderPermission (DcRole, Folder, C, R, U, D, B, RO, UO, DO) " +
               "select DcRoleId, ?Folder,true,true,true,true,true,true,true,true " +
               "from DcRole where RoleName='Administrators';",
               _con.MySqlConnection
               );
            permCmd.Parameters.Add("?Folder", MySqlDbType.Int32);

            MySqlCommand cmd = new MySqlCommand(
                "insert into folder (FolderId, ParentFolder, FolderType, Name)" +
                " Values (default, ?ParentFolder, ?FolderType, ?FolderName);"
                ,
                _con.MySqlConnection
                );
            cmd.Parameters.Add("?ParentFolder", MySqlDbType.Int32);
            cmd.Parameters.Add("?FolderType", MySqlDbType.Int32);
            cmd.Parameters.Add("?FolderName", MySqlDbType.String);
            cmd.Parameters["?FolderType"].Value = DBConstKeeper.Current.FolderType.DocType;
            /***************************
             * Test DocType folder1
             * fld1
             *   fld11
             * fld2
             *   fld21
             *     fld211 // no access for Users
             *     ^^^ - added 
             *   fld22
             *     fld221
             *   fld23 // no access for Users
             *   ^^^ - added
             * fld3
             * *************************/

            cmd.Parameters["?FolderName"].Value = "fld211";
            cmd.Parameters["?ParentFolder"].Value = fld2.Folders[0].Id;
            cmd.ExecuteNonQuery();
            permCmd.Parameters["?Folder"].Value = cmd.LastInsertedId;
            permCmd.ExecuteNonQuery();

            cmd.Parameters["?FolderName"].Value = "fld23";
            cmd.Parameters["?ParentFolder"].Value = fld2.Id;
            cmd.ExecuteNonQuery();
            permCmd.Parameters["?Folder"].Value = cmd.LastInsertedId;
            permCmd.ExecuteNonQuery();

            //SecurityAccess sa = new SecurityAccess(FixtureData.DocCircAdmin, _dbCon);
            //sa.CreateFolderSecurityTable();

            folder = fa.GetFolderHierarchy(DBConstKeeper.Current.FolderType.DocType);

            fld2 = folder.Folders[2];
            Assert.AreEqual(3, fld2.Folders.Length);
            Assert.AreEqual("fld23", fld2.Folders[2].Name);

            bool found = false;
            foreach (DcFolder fld in fld2.Folders)
            {
                if (fld.Name == "fld21")
                {
                    found = true;
                    Assert.AreEqual(1, fld.Folders.Length);
                    Assert.AreEqual("fld211", fld.Folders[0].Name);
                    break;
                }
            }

            Assert.IsTrue(found);
        }

        [Test]
        public void GetFolderItemsInFolder()
        {
            FolderAccess fa = new FolderAccess(FixtureData.DocCircAdmin, _con);

            FolderItem[] items = fa.GetFolderItemsInFolder(FixtureData.RootDocTypeFolder);
            
            Assert.AreEqual(4, items.Length);

            bool[] been = new bool[] { false, false, false, false };
            foreach (FolderItem item in items)
            {
                if (item.FolderId == FixtureData.Fld1 && !been[0])
                {
                    been[0] = true;
                    Assert.AreEqual("fld1", item.Name);
                    Assert.AreEqual(DBConstKeeper.Current.FolderType.DocType, item.FolderType);
                    Assert.AreEqual(FixtureData.RootDocTypeFolder, item.Parent);
                }
                else if (item.FolderId == FixtureData.Fld2 && !been[1])
                {
                    been[1] = true;
                    Assert.AreEqual("fld2", item.Name);
                    Assert.AreEqual(DBConstKeeper.Current.FolderType.DocType, item.FolderType);
                    Assert.AreEqual(FixtureData.RootDocTypeFolder, item.Parent);
                }
                else if (item.FolderId == FixtureData.Fld3 && !been[2])
                {
                    been[2] = true;
                    Assert.AreEqual("fld3", item.Name);
                    Assert.AreEqual(DBConstKeeper.Current.FolderType.DocType, item.FolderType);
                    Assert.AreEqual(FixtureData.RootDocTypeFolder, item.Parent);
                }
                else if (item.FolderId == FixtureData.TestDocTypeFolder && !been[3])
                {
                    been[3] = true;
                    Assert.AreEqual("Test DocType folder1", item.Name);
                    Assert.AreEqual(DBConstKeeper.Current.FolderType.DocType, item.FolderType);
                    Assert.AreEqual(FixtureData.RootDocTypeFolder, item.Parent);
                }
                else
                {
                    Assert.Fail("Unknown or repeated id!");
                }
            }
        }

        [Test]
        public void GetFolderItem()
        {
            FolderAccess fa = new FolderAccess(FixtureData.DocCircAdmin, _con);
            FolderItem item = fa.GetFolderItem(FixtureData.Fld2);

            Assert.IsNotNull(item);
            Assert.AreEqual("fld2", item.Name);
            Assert.AreEqual(DBConstKeeper.Current.FolderType.DocType, item.FolderType);
            Assert.AreEqual(FixtureData.RootDocTypeFolder, item.Parent);

        }

        [Test]
        public void GetFolderItem_RootFolder()
        {
            FolderAccess fa = new FolderAccess(FixtureData.DocCircAdmin, _con);
            
            FolderItem item = fa.GetFolderItem(FixtureData.RootDocTypeFolder);

            Assert.IsNotNull(item);
            Assert.AreEqual(DBConstKeeper.Current.RootFolder.DocType, item.FolderId);
            Assert.IsNull(item.Parent);
            Assert.AreEqual(DBConstKeeper.Current.FolderType.DocType, item.FolderType);
        }

        [Test]
        public void SaveFolderItem()
        {
            // case 1
            // add a new folder

            FolderItem item = new FolderItem(
                FixtureData.TestDocTypeFolder,
                "Folder 90",
                DBConstKeeper.Current.FolderType.DocType
                );
            
            FolderAccess fa = new FolderAccess(FixtureData.DocCircAdmin, _con);

            fa.SaveFolderItem(item); // tested method

            Assert.IsNotNull(item.FolderId);

            MySqlCommand cmd = _con.MySqlConnection.CreateCommand();
            cmd.Parameters.Add("?FolderId", MySqlDbType.Int32);
            cmd.Parameters["?FolderId"].Value = item.FolderId;
            string selectCmd = "select ParentFolder, Name, FolderType from Folder where FolderId=?FolderId";
            cmd.CommandText = selectCmd;
            
            MySqlDataReader reader = cmd.ExecuteReader();

            try
            {
                Assert.IsTrue(reader.Read());
                Assert.AreEqual(item.Parent, reader.GetInt32("ParentFolder"));
                Assert.AreEqual(item.Name, reader.GetString("Name"));
                Assert.AreEqual(item.FolderType, reader.GetInt32("FolderType"));
                Assert.IsFalse(reader.Read());
                reader.Close();

                cmd.CommandText = "select count(*) from NewFolderPermission where FolderType=@docTypeFolderType";
                reader = cmd.ExecuteReader();
                reader.Read();
                int newPermCount = reader.GetInt32(0);
                reader.Close();

                cmd.CommandText = "select count(*) from FolderPermission where Folder=?FolderId;";
                reader = cmd.ExecuteReader();
                reader.Read();
                int count = reader.GetInt32(0);
                reader.Close();

                Assert.AreEqual(newPermCount, count, "Added permissions count");

                // case 2
                // change name of the folder
                
                item.Name = "Folder 90 changed";
                fa.SaveFolderItem(item); // tested method
                
                cmd.CommandText = selectCmd;
                reader = cmd.ExecuteReader();

                Assert.IsTrue(reader.Read());
                Assert.AreEqual(item.Parent, reader.GetInt32("ParentFolder"));
                Assert.AreEqual(item.Name, reader.GetString("Name"));
                Assert.AreEqual(item.FolderType, reader.GetInt32("FolderType"));
                Assert.IsFalse(reader.Read());
                reader.Close();
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }
                cmd.CommandText =
                    "delete from FolderPermission where Folder=?FolderId;" +
                    "delete from Folder where FolderId=?FolderId;";
                    
                cmd.ExecuteNonQuery();
            }

        }

        [Test]
        public void DeleteFolder()
        {
            MySqlCommand cmd = _con.MySqlConnection.CreateCommand();
            cmd.CommandText = "insert into Folder (ParentFolder, Name, FolderType) " +
                "Values (?ParentFolder, ?Name, ?FolderType);";
            cmd.Parameters.Add("?ParentFolder", MySqlDbType.Int32);
            cmd.Parameters.Add("?Name", MySqlDbType.String);
            cmd.Parameters.Add("?FolderType", MySqlDbType.Int32);
            cmd.Parameters["?ParentFolder"].Value = FixtureData.RootDocTypeFolder;
            cmd.Parameters["?FolderType"].Value = DBConstKeeper.Current.FolderType.DocType;
            cmd.Parameters["?Name"].Value = "folder 90";

            cmd.ExecuteNonQuery();

            int folder90 = (int)cmd.LastInsertedId;

            FolderAccess fa = new FolderAccess(FixtureData.DocCircAdmin, _con);
            fa.DeleteFolder(folder90); // tested method

            cmd.Parameters.Add("?FolderId", MySqlDbType.Int32);
            cmd.Parameters["?FolderId"].Value = folder90;
            cmd.CommandText = "select count(*) from Folder where FolderId=?FolderId;";
            MySqlDataReader reader = cmd.ExecuteReader();
            reader.Read();
            int count = reader.GetInt32(0);
            reader.Close();

            try
            {
                Assert.AreEqual(0, count);
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                cmd.CommandText = "delete from Folder where FolderId=?FolderId;";
                cmd.ExecuteNonQuery();
            }


        }

        [Test]
        public void GetFolderHierarchyWithUnathorizedAccess()
        {
            //SecurityAccess sa = new SecurityAccess(FixtureData.DocCircUser, _dbCon);
            //sa.CreateFolderSecurityTable();
            FolderAccess fa = new FolderAccess(FixtureData.DocCircUser, _con);
            DcFolder rootFld = fa.GetFolderHierarchy(DBConstKeeper.Current.FolderType.DocType);

            Assert.AreEqual(FixtureData.RootDocTypeFolder, rootFld.Id);
            Assert.AreEqual(3, rootFld.Folders.Length);

            Assert.AreEqual("Test DocType folder1", rootFld.Folders[0].Name);

            DcFolder fld1 = rootFld.Folders[1];
            Assert.AreEqual("fld1", fld1.Name);
            Assert.AreEqual(0, fld1.Folders.Length);

            DcFolder fld2 = rootFld.Folders[2];
            Assert.AreEqual("fld2", fld2.Name);
            Assert.AreEqual(1, fld2.Folders.Length);
            Assert.AreEqual("fld21", fld2.Folders[0].Name);
        }

        [TestFixtureTearDown]
        public void TFTearDown()
        {
            _tran.Rollback();
            _con.Close();
        }
    }
}
