//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using NUnit.Framework;
using MySql.Data.MySqlClient;
using Evpo.DocCirc.MySqlDL;
using Evpo.DocCirc.Common;

namespace Evpo.DocCircUnitTest
{
    [TestFixture]
    public class NotificationAccessTest
    {
        public struct FixtureFile
        {
            public const string Folder = "MySql";
            public const string CreatingDocCards = "NotificationAccessTest.file1.sql";
            public const string CreatingNotifications = "NotificationAccessTest.file2.sql";
        }

        public class FixtureConstant
        {
            public long DocCard1, DocCard2;
            public int DocCircAdmin, DocCircUser1, DocCircUser2;
            public int TestDocumentFolder1;
            public long User1Notif1, User1Notif2, User1Notif3, User2Notif1;
        }

        private FixtureConstant _const = new FixtureConstant();

        private DBConnection _con;
        private MySqlTransaction _tran;
        private SettingKeeper _sk;

        [TestFixtureSetUp]
        public void TFSetUp()
        {
            _sk = new SettingKeeper();

            _con = new DBConnection(
            "Database=EvpoDCTest;Data Source=localhost;User Id=doccirctest;Password=doccirctest"
            );
            _con.Open();
            _tran = _con.MySqlConnection.BeginTransaction();


            MySqlCommand cmd = new MySqlCommand();
            cmd.Connection = _con.MySqlConnection;
            TextReader txtReader = File.OpenText(Path.Combine(FixtureFile.Folder, FixtureFile.CreatingDocCards));
            cmd.CommandText = txtReader.ReadToEnd();
            txtReader.Close();
            cmd.ExecuteNonQuery();

            txtReader = File.OpenText(Path.Combine(FixtureFile.Folder, FixtureFile.CreatingNotifications));
            cmd.CommandText = txtReader.ReadToEnd();
            txtReader.Close();
            cmd.ExecuteNonQuery();

            // get consts
            cmd.CommandText =
                "select cast(@docCard1 as unsigned) as DocCard1, cast(@docCard2 as unsigned) as DocCard2, " +
                "cast(@docCircAdmin as unsigned) as DocCircAdmin, " +
                "cast(@docCircUser1 as unsigned) as DocCircUser1, " +
                "cast(@docCircUser2 as unsigned) as DocCircUser2, " +
                "cast(@testDocumentFolder1 as unsigned) as TestDocumentFolder1, " +
                "cast(@user1Notif1 as unsigned) as User1Notif1, " +
                "cast(@user1Notif2 as unsigned) as User1Notif2, " +
                "cast(@user1Notif3 as unsigned) as User1Notif3, " +
                "cast(@user2Notif1 as unsigned) as User2Notif1;";

            MySqlDataReader reader = cmd.ExecuteReader();
            reader.Read();
            _const.DocCard1 = reader.GetInt64("DocCard1");
            _const.DocCard2 = reader.GetInt64("DocCard2");
            _const.DocCircAdmin = reader.GetInt32("DocCircAdmin");
            _const.DocCircUser1 = reader.GetInt32("DocCircUser1");
            _const.DocCircUser2 = reader.GetInt32("DocCircUser2");
            _const.TestDocumentFolder1 = reader.GetInt32("TestDocumentFolder1");
            _const.User1Notif1 = reader.GetInt64("User1Notif1");
            _const.User1Notif2 = reader.GetInt64("User1Notif2");
            _const.User1Notif3 = reader.GetInt64("User1Notif3");
            _const.User2Notif1 = reader.GetInt64("User2Notif1");
            
            reader.Close();

            DBConstAccess ca = new DBConstAccess(_con);
            FolderTypeDirectory ftd = ca.GetFolderTypeDirectory();
            DBConstKeeper.Current = new DBConstKeeper(
                ftd,
                ca.GetRootFolderDirectory(ftd),
                ca.GetMainRoleDirectory(),
                ca.GetRouteItemTypeDirectory(),
                ca.GetDocStatusDirectory(),
                ca.GetNotificationTypeDirectory(),
                ca.GetEventTypeDirectory(),
                ca.GetCommonDepartment()
                );
        }

        [Test]
        public void GetUserNotifications()
        {
            NotificationAccess na = new NotificationAccess(_const.DocCircUser1, _con, _sk);
            NotificationItem[] items = na.GetUserNotifications(_const.DocCircUser1); // tested method

            Assert.AreEqual(3, items.Length);

            bool[] been = new bool[] { false, false, false };
            foreach (NotificationItem item in items)
            {
                if (item.Id == _const.User1Notif1 && !been[0])
                {
                    been[0] = true;
                    Assert.AreEqual(_const.DocCard1, item.DocCardId);
                    Assert.AreEqual(
                        new DateTime(2007, 06, 07, 9, 55, 00),
                        item.NotificationTime
                        );
                    Assert.AreEqual(
                        DBConstKeeper.Current.NotificationType.NextDecision,
                        item.NotificationType
                        );
                    Assert.AreEqual("pass docCard1", item.Text);
                    Assert.AreEqual("Hi1", item.UserNote);
                    Assert.AreEqual(_const.DocCircUser1, item.User);
                    Assert.AreEqual(_const.DocCircAdmin, item.LastActionUser);
                    Assert.AreEqual(1, item.RoutePointNum);

                }
                else if (item.Id == _const.User1Notif2 && !been[1])
                {
                    been[1] = true;
                    Assert.AreEqual(_const.DocCard1, item.DocCardId);
                    Assert.AreEqual(
                        new DateTime(2007, 06, 07, 9, 56, 00),
                        item.NotificationTime
                        );
                    Assert.AreEqual(
                        DBConstKeeper.Current.NotificationType.Finalization,
                        item.NotificationType
                        );
                    Assert.AreEqual("docCard1 finalized", item.Text);
                    Assert.AreEqual("Hi2", item.UserNote);
                    Assert.AreEqual(_const.DocCircUser1, item.User);
                    Assert.AreEqual(_const.DocCircAdmin, item.LastActionUser);
                    Assert.AreEqual(2, item.RoutePointNum);
                }
                else if (item.Id == _const.User1Notif3 && !been[2])
                {
                    been[2] = true;
                    Assert.AreEqual(_const.DocCard2, item.DocCardId);
                    Assert.AreEqual(
                        new DateTime(2007, 06, 07, 9, 58, 00),
                        item.NotificationTime
                        );
                    Assert.AreEqual(
                        DBConstKeeper.Current.NotificationType.AcquaintanceRequest,
                        item.NotificationType
                        );
                    Assert.AreEqual("check docCard1", item.Text);
                    Assert.AreEqual("Hi3", item.UserNote);
                    Assert.AreEqual(_const.DocCircUser1, item.User);
                    Assert.IsNull(item.LastActionUser);
                    Assert.AreEqual(3, item.RoutePointNum);
                }
                else
                {
                    Assert.Fail("Unknown or repeated item!");
                }
            }

            items = na.GetUserNotifications(_const.DocCircUser2); // tested method
            
            Assert.AreEqual(1, items.Length);

            Assert.AreEqual(_const.User2Notif1, items[0].Id);
            Assert.AreEqual(_const.DocCard2, items[0].DocCardId);
            Assert.AreEqual(
                new DateTime(2007, 06, 07, 9, 57, 00),
                items[0].NotificationTime
                );
            Assert.AreEqual(
                DBConstKeeper.Current.NotificationType.NextNotification,
                items[0].NotificationType
                );
            Assert.AreEqual("see docCard2", items[0].Text);
            Assert.AreEqual("Hi4", items[0].UserNote);
            Assert.AreEqual(_const.DocCircUser2, items[0].User);
            Assert.AreEqual(_const.DocCircUser1, items[0].LastActionUser);
            Assert.AreEqual(4, items[0].RoutePointNum);


        }

        [Test]
        public void AddUserNotification()
        {
            NotificationAccess na = new NotificationAccess(_const.DocCircAdmin, _con, _sk);
            NotificationItem item = new NotificationItem(
                _const.DocCard2,
                DBConstKeeper.Current.NotificationType.ReturnToDraftDueToDecision,
                _const.DocCircUser2,
                new DateTime(2007, 06, 07, 10, 53, 00),
                "new added notification",
                "Hello there!",
                _const.DocCircAdmin,
                5
                );

            na.AddUserNotification(item); // tested method

            Assert.IsNotNull(item.Id);

            MySqlCommand cmd = new MySqlCommand(
                "select DocCard, NotificationType, DcUser, NotificationTime, Text, UserNote, LastActionUser, RoutePointNum " +
                "from UserNotification where UserNotificationId=?UserNotification;",
                _con.MySqlConnection
                );
            cmd.Parameters.Add("?UserNotification", MySqlDbType.Int64);
            cmd.Parameters["?UserNotification"].Value = item.Id;

            MySqlDataReader reader = cmd.ExecuteReader();
            try
            {
                Assert.IsTrue(reader.Read());
                Assert.AreEqual(_const.DocCard2,reader.GetInt64("DocCard"));

                Assert.AreEqual(
                    DBConstKeeper.Current.NotificationType.ReturnToDraftDueToDecision,
                    reader.GetInt32("NotificationType")
                    );
                Assert.AreEqual(_const.DocCircUser2, reader.GetInt32("DcUser"));
                Assert.AreEqual(
                    new DateTime(2007, 06, 07, 10, 53, 00),
                    reader.GetDateTime("NotificationTime")
                    );
                Assert.AreEqual("new added notification", reader.GetString("Text"));
                Assert.AreEqual("Hello there!", reader.GetString("UserNote"));
                Assert.AreEqual(_const.DocCircAdmin, reader.GetInt32("LastActionUser"));
                Assert.AreEqual(5, reader.GetInt32("RoutePointNum"));
                
            }
            catch(Exception ex)
            {
                throw ex;
            }
            finally
            {
                if(!reader.IsClosed)
                {
                    reader.Close();
                }
                cmd.CommandText = "delete from UserNotification where UserNotificationId=?UserNotification;";
                cmd.ExecuteNonQuery();
            }

        }



        [TestFixtureTearDown]
        public void TFTearDown()
        {
            _tran.Rollback();
            _con.Close();
        }
    }
}
