//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using MySql.Data.MySqlClient;
using NUnit.Framework;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.MySqlDL;

namespace Evpo.DocCircUnitTest
{
    [TestFixture]
    public class SecurityAccessTest
    {
        protected struct FixtureConst
        {
            public const string Folder = "MySql";
            public const string SqlFile1 = "SecurityAccessTest.file1.sql";
            public const string SqlFile2 = "SecurityAccessTest.file2.sql";
            public const string SqlFile2ClearUp = "SecurityAccessTest.file3.sql";
        }
        protected class FixtureData
        {
            public static int DocCircAdmin, DocCircUser1, DocCircUser,
                AdminRole, UserRole, SaTestRole1, SaTestRole2,
                SaTestFolder1, SaTestFolder2, RootDocTypeFolder, TestDocTypeFolder1, RootDocumentFolder,
                DocType1, DocType2;
        }

        DBConnection _con;
        MySqlTransaction _tran;

        [TestFixtureSetUp]
        public void TFSetUp()
        {
            _con = new DBConnection(
               "Database=EvpoDCTest;Data Source=localhost;User Id=doccirctest;Password=doccirctest"
               );
            _con.Open();
            _tran = _con.MySqlConnection.BeginTransaction();

            TextReader tr = File.OpenText(
                Path.Combine(FixtureConst.Folder, FixtureConst.SqlFile1)
                );
            
            MySqlCommand cmd = new MySqlCommand(
                tr.ReadToEnd(),
                _con.MySqlConnection
                );
            tr.Close();
            cmd.ExecuteNonQuery();

            cmd.CommandText =
                "select cast(@docCircUser as unsigned) as DocCircUser, " +
                "cast(@docCircUser1 as unsigned) as DocCircUser1, " +
                "cast(@docCircAdmin as unsigned) as DocCircAdmin, " +

                "cast(@adminRole as unsigned) as AdminRole, cast(@userRole as unsigned) as UserRole, " +
                "cast(@saTestRole1 as unsigned) as SaTestRole1, cast(@saTestRole2 as unsigned) as SaTestRole2, " +

                "cast(@saTestFolder1 as unsigned) as SaTestFolder1, " +
                "cast(@saTestFolder2 as unsigned) as SaTestFolder2, " +
                "cast(@rootDocTypeFolder as unsigned) as RootDocTypeFolder, " +
                "cast(@testDocTypeFolder1 as unsigned) as TestDocTypeFolder1, " +
                "cast(@rootDocumentFolder as unsigned) as RootDocumentFolder, " +
                "cast(@docType1 as unsigned) as DocType1, cast(@docType2 as unsigned) as DocType2;";


            MySqlDataReader reader = cmd.ExecuteReader();
            reader.Read();
            FixtureData.DocCircAdmin = reader.GetInt32("DocCircAdmin");
            FixtureData.DocCircUser = reader.GetInt32("DocCircUser");
            FixtureData.DocCircUser1 = reader.GetInt32("DocCircUser1");

            FixtureData.AdminRole = reader.GetInt32("AdminRole");
            FixtureData.UserRole = reader.GetInt32("UserRole");
            FixtureData.SaTestRole1 = reader.GetInt32("SaTestRole1");
            FixtureData.SaTestRole2 = reader.GetInt32("SaTestRole2");

            FixtureData.SaTestFolder1 = reader.GetInt32("SaTestFolder1");
            FixtureData.SaTestFolder2 = reader.GetInt32("SaTestFolder2");
            FixtureData.RootDocumentFolder = reader.GetInt32("RootDocumentFolder");
            FixtureData.RootDocTypeFolder = reader.GetInt32("RootDocTypeFolder");
            FixtureData.TestDocTypeFolder1 = reader.GetInt32("TestDocTypeFolder1");

            FixtureData.DocType1 = reader.GetInt32("DocType1");
            FixtureData.DocType2 = reader.GetInt32("DocType2");
            reader.Close();

            
        }

        //[Test]
        //public void CreateUserRoleTable()
        //{
        //    //for user DocCircUser
        //    SecurityAccess sa = new SecurityAccess(FixtureData.DocCircUser, _dbCon);
        //    sa.CreateUserRoleTable(); //tested method

            
        //    MySqlCommand cmd = new MySqlCommand(
        //        "select DcRole, DcUser from tmpUserRole;",
        //        _dbCon.MySqlConnection
        //        );

        //    MySqlDataReader reader = null;
        //    try
        //    {
        //        reader = cmd.ExecuteReader();
        //    }
        //    catch (MySqlException ex)
        //    {
        //        Assert.Fail(ex.Message);
        //    }

        //    List<int> roles = new List<int>();
        //    while (reader.Read())
        //    {
        //        Assert.AreEqual(FixtureData.DocCircUser, reader.GetInt32("DcUser"));
        //        roles.Add(reader.GetInt32("DcRole"));
        //    }
        //    reader.Close();

        //    Assert.AreEqual(1, roles.Count);
        //    Assert.AreEqual(FixtureData.UserRole, roles[0]);

        //    // for user DocCircUser1
        //    sa = new SecurityAccess(FixtureData.DocCircUser1, _dbCon);
        //    sa.CreateUserRoleTable(); // tested method
        //    reader = cmd.ExecuteReader();

        //    roles.Clear();
        //    while (reader.Read())
        //    {
        //        roles.Add(reader.GetInt32("DcRole"));
        //    }
        //    reader.Close();

        //    Assert.AreEqual(3, roles.Count);
        //    Assert.Contains(FixtureData.UserRole, roles);
        //    Assert.Contains(FixtureData.SaTestRole1, roles);
        //    Assert.Contains(FixtureData.SaTestRole2, roles);
        //}

        [Test]
        public void CreateFolderSecurityTable()
        {
            
            MySqlCommand cmd = new MySqlCommand(
                "select Folder, DcUser, C, R, U, D, B, RO, UO, DO from tmpFolderSecurity",
                _con.MySqlConnection
                );

            SecurityAccess sa = new SecurityAccess(FixtureData.DocCircUser1, _con);
            sa.CreateFolderSecurityTable();

            MySqlDataReader reader = cmd.ExecuteReader();
            try // comment to know code line
            {
                bool[] been = new bool[] { false, false, false, false, false };
                int i = 0;
                while (reader.Read())
                {
                    i++;
                    if (reader.GetInt32("Folder") == FixtureData.RootDocumentFolder && !been[0])
                    {
                        been[0] = true;
                        Assert.AreEqual(FixtureData.DocCircUser1, reader.GetInt32("DcUser"));
                        Assert.IsTrue(reader.GetBoolean("C"));
                        Assert.IsTrue(reader.GetBoolean("R"));
                        Assert.IsTrue(reader.GetBoolean("U"));
                        Assert.IsTrue(reader.GetBoolean("D"));
                        Assert.IsTrue(reader.GetBoolean("B"));
                        Assert.IsTrue(reader.GetBoolean("RO"));
                        Assert.IsTrue(reader.GetBoolean("UO"));
                        Assert.IsTrue(reader.GetBoolean("DO"));
                    }
                    else if (reader.GetInt32("Folder") == FixtureData.RootDocTypeFolder && !been[1])
                    {
                        been[1] = true;
                        Assert.AreEqual(FixtureData.DocCircUser1, reader.GetInt32("DcUser"));
                        Assert.IsFalse(reader.GetBoolean("C"));
                        Assert.IsTrue(reader.GetBoolean("R"));
                        Assert.IsFalse(reader.GetBoolean("U"));
                        Assert.IsFalse(reader.GetBoolean("D"));
                        Assert.IsTrue(reader.GetBoolean("B"));
                        Assert.IsTrue(reader.GetBoolean("RO"));
                        Assert.IsFalse(reader.GetBoolean("UO"));
                        Assert.IsFalse(reader.GetBoolean("DO"));
                    }
                    else if (reader.GetInt32("Folder") == FixtureData.TestDocTypeFolder1 && !been[2])
                    {
                        been[2] = true;
                        Assert.AreEqual(FixtureData.DocCircUser1, reader.GetInt32("DcUser"));
                        Assert.IsFalse(reader.GetBoolean("C"));
                        Assert.IsTrue(reader.GetBoolean("R"));
                        Assert.IsFalse(reader.GetBoolean("U"));
                        Assert.IsFalse(reader.GetBoolean("D"));
                        Assert.IsTrue(reader.GetBoolean("B"));
                        Assert.IsTrue(reader.GetBoolean("RO"));
                        Assert.IsFalse(reader.GetBoolean("UO"));
                        Assert.IsFalse(reader.GetBoolean("DO"));
                    }
                    else if (reader.GetInt32("Folder") == FixtureData.SaTestFolder1 && !been[3])
                    {
                        been[3] = true;
                        Assert.AreEqual(FixtureData.DocCircUser1, reader.GetInt32("DcUser"));
                        
                        Assert.IsTrue(reader.GetBoolean("C"));
                        Assert.IsTrue(reader.GetBoolean("R"));
                        Assert.IsFalse(reader.GetBoolean("U"));
                        Assert.IsFalse(reader.GetBoolean("D"));
                        Assert.IsTrue(reader.GetBoolean("B"));
                        Assert.IsTrue(reader.GetBoolean("RO"));
                        Assert.IsFalse(reader.GetBoolean("UO"));
                        Assert.IsTrue(reader.GetBoolean("DO"));
                    }
                    else if (reader.GetInt32("Folder") == FixtureData.SaTestFolder2 && !been[4])
                    {
                        been[4] = true;
                        Assert.AreEqual(FixtureData.DocCircUser1, reader.GetInt32("DcUser"));

                        Assert.IsFalse(reader.GetBoolean("C"));
                        Assert.IsTrue(reader.GetBoolean("R"));
                        Assert.IsTrue(reader.GetBoolean("U"));
                        Assert.IsTrue(reader.GetBoolean("D"));
                        Assert.IsTrue(reader.GetBoolean("B"));
                        Assert.IsTrue(reader.GetBoolean("RO"));
                        Assert.IsTrue(reader.GetBoolean("UO"));
                        Assert.IsTrue(reader.GetBoolean("DO"));
                    }
                    else
                    {
                        Assert.Fail(String.Format("Unexpected folder! FolderId={0}", reader.GetInt32("Folder")));
                    }

                }
                reader.Close();

                Assert.AreEqual(5, i);
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }
            }

        }

        [Test]
        public void CreateDocTypeSecurityTable()
        {
            SecurityAccess sa = new SecurityAccess(FixtureData.DocCircUser1, _con);
            sa.CreateDocTypeSecurityTable(); // tested method

            MySqlCommand cmd = new MySqlCommand(
                "select DocType, DcUser, C, R, U, D, RO, UO, DO from tmpDocTypeSecurity;",
                _con.MySqlConnection
                );

            
            MySqlDataReader reader = cmd.ExecuteReader();
            try
            {
                
                bool[] been = new bool[] { false, false };
                while(reader.Read())
                {
                    if (reader.GetInt32("DocType") == FixtureData.DocType1 && !been[0])
                    {
                        been[0] = true;
                        Assert.IsTrue(reader.GetBoolean("C"));
                        Assert.IsTrue(reader.GetBoolean("R"));
                        Assert.IsFalse(reader.GetBoolean("U"));
                        Assert.IsFalse(reader.GetBoolean("D"));
                        Assert.IsTrue(reader.GetBoolean("RO"));
                        Assert.IsFalse(reader.GetBoolean("UO"));
                        Assert.IsTrue(reader.GetBoolean("DO"));
                    }
                    else if (reader.GetInt32("DocType") == FixtureData.DocType2 && !been[1])
                    {
                        been[1] = true;
                        Assert.IsTrue(reader.GetBoolean("C"));
                        Assert.IsTrue(reader.GetBoolean("R"));
                        Assert.IsFalse(reader.GetBoolean("U"));
                        Assert.IsFalse(reader.GetBoolean("D"));
                        Assert.IsTrue(reader.GetBoolean("RO"));
                        Assert.IsTrue(reader.GetBoolean("UO"));
                        Assert.IsFalse(reader.GetBoolean("DO"));

                    }
                    else
                    {
                        Assert.Fail(
                            String.Format(
                                "Unknown or repeated Doc type. Id={0}", 
                                reader.GetInt32("DocType")
                                )
                            );
                    }

                }

                Assert.IsTrue(been[0], "DocType1 not found!");
                Assert.IsTrue(been[1], "DocType2 not found!");

            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }
            }

        }

        [Test]
        public void GetFolderSecurity()
        {

            SecurityAccess sa = new SecurityAccess(FixtureData.DocCircUser1, _con);
            SecurityItem item = sa.GetFolderSecurity(FixtureData.SaTestFolder1);

            Assert.IsNotNull(item);

            Assert.IsTrue(item.Create);
            Assert.IsTrue(item.Read);
            Assert.IsFalse(item.Update);
            Assert.IsFalse(item.Delete);
            Assert.IsTrue(item.Browse);
            Assert.IsTrue(item.ReadOwn);
            Assert.IsFalse(item.UpdateOwn);
            Assert.IsTrue(item.DeleteOwn);

            item = sa.GetFolderSecurity(FixtureData.SaTestFolder2);

            Assert.IsFalse(item.Create);
            Assert.IsTrue(item.Read);
            Assert.IsTrue(item.Update);
            Assert.IsTrue(item.Delete);
            Assert.IsTrue(item.Browse);
            Assert.IsTrue(item.ReadOwn);
            Assert.IsTrue(item.UpdateOwn);
            Assert.IsTrue(item.DeleteOwn);

            // use tmpFolderSecurityTable
            bool useTmpFolderSecurityTable = true;
            bool useTmpDocTypeSecurityTable = true;
            sa = new SecurityAccess(
                FixtureData.DocCircUser1, _con, 
                useTmpFolderSecurityTable,
                useTmpDocTypeSecurityTable
                );
            sa.CreateFolderSecurityTable();
            
            item = sa.GetFolderSecurity(FixtureData.SaTestFolder1);

            Assert.IsNotNull(item);

            Assert.IsTrue(item.Create);
            Assert.IsTrue(item.Read);
            Assert.IsFalse(item.Update);
            Assert.IsFalse(item.Delete);
            Assert.IsTrue(item.Browse);
            Assert.IsTrue(item.ReadOwn);
            Assert.IsFalse(item.UpdateOwn);
            Assert.IsTrue(item.DeleteOwn);

            item = sa.GetFolderSecurity(FixtureData.SaTestFolder2);

            Assert.IsFalse(item.Create);
            Assert.IsTrue(item.Read);
            Assert.IsTrue(item.Update);
            Assert.IsTrue(item.Delete);
            Assert.IsTrue(item.Browse);
            Assert.IsTrue(item.ReadOwn);
            Assert.IsTrue(item.UpdateOwn);
            Assert.IsTrue(item.DeleteOwn);
        }

        [Test]
        public void GetDocTypeDocumentsSecurity()
        {
            SecurityAccess sa = new SecurityAccess(FixtureData.DocCircUser1, _con);
            SecurityItem item = sa.GetDocTypeDocumentsSecurity(FixtureData.DocType1);

            Assert.IsNotNull(item);
            Assert.IsTrue(item.Create);
            Assert.IsTrue(item.Read);
            Assert.IsFalse(item.Update);
            Assert.IsFalse(item.Delete);
            Assert.IsTrue(item.ReadOwn);
            Assert.IsFalse(item.UpdateOwn);
            Assert.IsTrue(item.DeleteOwn);

            item = sa.GetDocTypeDocumentsSecurity(FixtureData.DocType2);

            Assert.IsNotNull(item);
            Assert.IsTrue(item.Create);
            Assert.IsTrue(item.Read);
            Assert.IsFalse(item.Update);
            Assert.IsFalse(item.Delete);
            Assert.IsTrue(item.ReadOwn);
            Assert.IsTrue(item.UpdateOwn);
            Assert.IsFalse(item.DeleteOwn);

            sa = new SecurityAccess(FixtureData.DocCircAdmin, _con);
            item = sa.GetDocTypeDocumentsSecurity(FixtureData.DocType1);

            Assert.IsNotNull(item);
            Assert.IsTrue(item.Create);
            Assert.IsTrue(item.Read);
            Assert.IsTrue(item.Update);
            Assert.IsTrue(item.Delete);
            Assert.IsTrue(item.ReadOwn);
            Assert.IsTrue(item.UpdateOwn);
            Assert.IsTrue(item.DeleteOwn);

        }

        [Test]
        public void GetDocTypeDocumentsSecurityWithSecurityTable()
        {
            bool useTmpFolderSecurityTable = true, useTmpDocTypeSecurityTable = true;
            SecurityAccess sa = new SecurityAccess(
                FixtureData.DocCircUser1, _con,
                !useTmpFolderSecurityTable,
                useTmpDocTypeSecurityTable
                );

            sa.CreateDocTypeSecurityTable();

            SecurityItem item = sa.GetDocTypeDocumentsSecurity(FixtureData.DocType1);

            Assert.IsNotNull(item);
            Assert.IsTrue(item.Create);
            Assert.IsTrue(item.Read);
            Assert.IsFalse(item.Update);
            Assert.IsFalse(item.Delete);
            Assert.IsTrue(item.ReadOwn);
            Assert.IsFalse(item.UpdateOwn);
            Assert.IsTrue(item.DeleteOwn);

            item = sa.GetDocTypeDocumentsSecurity(FixtureData.DocType2);

            Assert.IsNotNull(item);
            Assert.IsTrue(item.Create);
            Assert.IsTrue(item.Read);
            Assert.IsFalse(item.Update);
            Assert.IsFalse(item.Delete);
            Assert.IsTrue(item.ReadOwn);
            Assert.IsTrue(item.UpdateOwn);
            Assert.IsFalse(item.DeleteOwn);

            sa = new SecurityAccess(
                FixtureData.DocCircAdmin, _con,
                !useTmpFolderSecurityTable,
                useTmpDocTypeSecurityTable
                );

            sa.CreateDocTypeSecurityTable();

            item = sa.GetDocTypeDocumentsSecurity(FixtureData.DocType1);

            Assert.IsNotNull(item);
            Assert.IsTrue(item.Create);
            Assert.IsTrue(item.Read);
            Assert.IsTrue(item.Update);
            Assert.IsTrue(item.Delete);
            Assert.IsTrue(item.ReadOwn);
            Assert.IsTrue(item.UpdateOwn);
            Assert.IsTrue(item.DeleteOwn);
        }

        [Test]
        public void GetFolderPermissions()
        {
            SecurityAccess sa = new SecurityAccess(FixtureData.DocCircAdmin, _con);
            FolderPermission[] perms = sa.GetFolderPermissions(FixtureData.SaTestFolder1); // tested method
            
            Assert.AreEqual(4, perms.Length);

            bool[] been = new bool[] { false, false, false, false };
            foreach (FolderPermission perm in perms)
            {
                if (perm.Role.RoleId == FixtureData.AdminRole && !been[0])
                {
                    been[0] = true;
                    Assert.IsNotNull(perm.PermissionId);
                    Assert.AreEqual(FixtureData.SaTestFolder1, perm.Folder);
                    SecurityItem si = perm.SecurityItem;
                    Assert.IsTrue(si.Create);
                    Assert.IsTrue(si.Read);
                    Assert.IsTrue(si.Update);
                    Assert.IsTrue(si.Delete);
                    Assert.IsTrue(si.Browse);
                    Assert.IsTrue(si.ReadOwn);
                    Assert.IsTrue(si.UpdateOwn);
                    Assert.IsTrue(si.DeleteOwn);
                }
                else if (perm.Role.RoleId == FixtureData.UserRole && !been[1])
                {
                    been[1] = true;
                    Assert.IsNotNull(perm.PermissionId);
                    Assert.AreEqual(FixtureData.SaTestFolder1, perm.Folder);
                    SecurityItem si = perm.SecurityItem;
                    Assert.IsFalse(si.Create);
                    Assert.IsTrue(si.Read);
                    Assert.IsFalse(si.Update);
                    Assert.IsFalse(si.Delete);
                    Assert.IsTrue(si.Browse);
                    Assert.IsTrue(si.ReadOwn);
                    Assert.IsFalse(si.UpdateOwn);
                    Assert.IsFalse(si.DeleteOwn);
                }
                else if (perm.Role.RoleId == FixtureData.SaTestRole1 && !been[2])
                {
                    been[2] = true;
                    Assert.IsNotNull(perm.PermissionId);
                    Assert.AreEqual(FixtureData.SaTestFolder1, perm.Folder);
                    SecurityItem si = perm.SecurityItem;
                    Assert.IsTrue(si.Create);
                    Assert.IsTrue(si.Read);
                    Assert.IsFalse(si.Update);
                    Assert.IsFalse(si.Delete);
                    Assert.IsTrue(si.Browse);
                    Assert.IsTrue(si.ReadOwn);
                    Assert.IsFalse(si.UpdateOwn);
                    Assert.IsFalse(si.DeleteOwn);
                }
                else if (perm.Role.RoleId == FixtureData.SaTestRole2 && !been[3])
                {
                    been[3] = true;
                    Assert.IsNotNull(perm.PermissionId);
                    Assert.AreEqual(FixtureData.SaTestFolder1, perm.Folder);
                    SecurityItem si = perm.SecurityItem;
                    Assert.IsFalse(si.Create);
                    Assert.IsTrue(si.Read);
                    Assert.IsFalse(si.Update);
                    Assert.IsFalse(si.Delete);
                    Assert.IsTrue(si.Browse);
                    Assert.IsTrue(si.ReadOwn);
                    Assert.IsFalse(si.UpdateOwn);
                    Assert.IsTrue(si.DeleteOwn);
                }
                else
                {
                    Assert.Fail("Unknown or repeated id!");
                }

            }



        }

        [Test]
        public void SaveFolderPermissions()
        {
            MySqlCommand cmd = _con.MySqlConnection.CreateCommand();
            TextReader txtReader = File.OpenText(
                Path.Combine(FixtureConst.Folder, FixtureConst.SqlFile2)
                );
            cmd.CommandText = txtReader.ReadToEnd();
            txtReader.Close();
            txtReader = File.OpenText(
                Path.Combine(FixtureConst.Folder, FixtureConst.SqlFile2ClearUp)
                );
            string clearUp = txtReader.ReadToEnd();
            txtReader.Close();

            cmd.ExecuteNonQuery();

            cmd.CommandText = "select cast(@docTypeFolder90 as unsigned);";
            MySqlDataReader reader = cmd.ExecuteReader();
            reader.Read();
            int docTypeFolder90 = reader.GetInt32(0);
            reader.Close();

            SecurityAccess sa = new SecurityAccess(FixtureData.DocCircAdmin, _con);

            //case 1
            // clear all permissions

            FolderPermission[] perms = new FolderPermission[0];
            sa.SaveFolderPermissions(docTypeFolder90, perms); // tested method

            cmd.CommandText = "select count(*) from FolderPermission where Folder=@docTypeFolder90;";
            reader = cmd.ExecuteReader();
            reader.Read();
            int count = reader.GetInt32(0);
            reader.Close();

            Assert.AreEqual(0, count);

            //case 2
            // add two permissions
            SecurityItem si = new SecurityItem(true, true, true, true, false, true, true, true);
            UserRole role = new UserRole(FixtureData.AdminRole, "Administrators");
            FolderPermission perm1 = new FolderPermission(docTypeFolder90, role, si);

            si = new SecurityItem(true, false, true, false, false, true, false, true);
            role = new UserRole(FixtureData.UserRole, "Users");
            FolderPermission perm2 = new FolderPermission(docTypeFolder90, role, si);

            perms = new FolderPermission[] { perm1, perm2 };

            sa.SaveFolderPermissions(docTypeFolder90, perms); // tested method


            Assert.IsNotNull(perms[0].PermissionId);
            Assert.IsNotNull(perms[1].PermissionId);

            cmd.CommandText =
                "select FolderPermissionId, Folder, DcRole, C, R, U, D, B, RO, UO, DO " +
                "from FolderPermission where Folder=@docTypeFolder90;";

            reader = cmd.ExecuteReader();
            //bool violation = false;

            bool[] been = new bool[] { false, false };
            int rowCount = 0;
            while (reader.Read())
            {
                rowCount++;
                if (reader.GetInt32("FolderPermissionId") == perms[0].PermissionId && !been[0])
                {
                    been[0] = true;
                    Assert.AreEqual(perms[0].Role.RoleId, reader.GetInt32("DcRole"));
                    Assert.AreEqual(perms[0].SecurityItem.Create, reader.GetBoolean("C"));
                    Assert.AreEqual(perms[0].SecurityItem.Read, reader.GetBoolean("R"));
                    Assert.AreEqual(perms[0].SecurityItem.Update, reader.GetBoolean("U"));
                    Assert.AreEqual(perms[0].SecurityItem.Delete, reader.GetBoolean("D"));
                    Assert.AreEqual(perms[0].SecurityItem.Browse, reader.GetBoolean("B"));
                    Assert.AreEqual(perms[0].SecurityItem.ReadOwn, reader.GetBoolean("RO"));
                    Assert.AreEqual(perms[0].SecurityItem.UpdateOwn, reader.GetBoolean("UO"));
                    Assert.AreEqual(perms[0].SecurityItem.DeleteOwn, reader.GetBoolean("DO"));
                }
                else if (reader.GetInt32("FolderPermissionId") == perms[1].PermissionId && !been[1])
                {
                    been[1] = true;
                    Assert.AreEqual(perms[1].Role.RoleId, reader.GetInt32("DcRole"));
                    Assert.AreEqual(perms[1].SecurityItem.Create, reader.GetBoolean("C"));
                    Assert.AreEqual(perms[1].SecurityItem.Read, reader.GetBoolean("R"));
                    Assert.AreEqual(perms[1].SecurityItem.Update, reader.GetBoolean("U"));
                    Assert.AreEqual(perms[1].SecurityItem.Delete, reader.GetBoolean("D"));
                    Assert.AreEqual(perms[1].SecurityItem.Browse, reader.GetBoolean("B"));
                    Assert.AreEqual(perms[1].SecurityItem.ReadOwn, reader.GetBoolean("RO"));
                    Assert.AreEqual(perms[1].SecurityItem.UpdateOwn, reader.GetBoolean("UO"));
                    Assert.AreEqual(perms[1].SecurityItem.DeleteOwn, reader.GetBoolean("DO"));
                }
                else
                {
                    Assert.Fail("Unknown or repeated id!");
                }
            }
            reader.Close();

            Assert.AreEqual(2, rowCount);

            // case 3
            // add a permission to existent ones
            // change perm2 accesses
            si = new SecurityItem(false, false, false, false, false, false, false, false);
            role = new UserRole(FixtureData.SaTestRole1, "TestRole1");
            FolderPermission perm3 = new FolderPermission(docTypeFolder90, role, si);
            perms = new FolderPermission[] { perms[0], perms[1], perm3 };
            perms[1].SecurityItem = new SecurityItem(false, true, false, true, false, false, true, false);
            int perm1IdBefore = perms[0].PermissionId.Value, perm2IdBefore = perms[1].PermissionId.Value;

            sa.SaveFolderPermissions(docTypeFolder90, perms); // tested method

            Assert.AreEqual(perm1IdBefore, perms[0].PermissionId);
            Assert.AreEqual(perm2IdBefore, perms[1].PermissionId);
            Assert.IsNotNull(perm3.PermissionId);

            reader = cmd.ExecuteReader();

            been = new bool[] { false, false, false };
            rowCount = 0;
            while (reader.Read())
            {
                rowCount++;
                if (reader.GetInt32("FolderPermissionId") == perms[0].PermissionId && !been[0])
                {
                    been[0] = true;

                    Assert.AreEqual(perms[0].Role.RoleId, reader.GetInt32("DcRole"));
                    Assert.AreEqual(perms[0].SecurityItem.Create, reader.GetBoolean("C"));
                    Assert.AreEqual(perms[0].SecurityItem.Read, reader.GetBoolean("R"));
                    Assert.AreEqual(perms[0].SecurityItem.Update, reader.GetBoolean("U"));
                    Assert.AreEqual(perms[0].SecurityItem.Delete, reader.GetBoolean("D"));
                    Assert.AreEqual(perms[0].SecurityItem.Browse, reader.GetBoolean("B"));
                    Assert.AreEqual(perms[0].SecurityItem.ReadOwn, reader.GetBoolean("RO"));
                    Assert.AreEqual(perms[0].SecurityItem.UpdateOwn, reader.GetBoolean("UO"));
                    Assert.AreEqual(perms[0].SecurityItem.DeleteOwn, reader.GetBoolean("DO"));
                }
                else if (reader.GetInt32("FolderPermissionId") == perms[1].PermissionId && !been[1])
                {
                    been[1] = true;
                    Assert.AreEqual(perms[1].Role.RoleId, reader.GetInt32("DcRole"));
                    Assert.AreEqual(perms[1].SecurityItem.Create, reader.GetBoolean("C"));
                    Assert.AreEqual(perms[1].SecurityItem.Read, reader.GetBoolean("R"));
                    Assert.AreEqual(perms[1].SecurityItem.Update, reader.GetBoolean("U"));
                    Assert.AreEqual(perms[1].SecurityItem.Delete, reader.GetBoolean("D"));
                    Assert.AreEqual(perms[1].SecurityItem.Browse, reader.GetBoolean("B"));
                    Assert.AreEqual(perms[1].SecurityItem.ReadOwn, reader.GetBoolean("RO"));
                    Assert.AreEqual(perms[1].SecurityItem.UpdateOwn, reader.GetBoolean("UO"));
                    Assert.AreEqual(perms[1].SecurityItem.DeleteOwn, reader.GetBoolean("DO"));
                }
                else if (reader.GetInt32("FolderPermissionId") == perms[2].PermissionId && !been[2])
                {
                    been[2] = true;
                    Assert.AreEqual(perms[2].Role.RoleId, reader.GetInt32("DcRole"));
                    Assert.AreEqual(perms[2].SecurityItem.Create, reader.GetBoolean("C"));
                    Assert.AreEqual(perms[2].SecurityItem.Read, reader.GetBoolean("R"));
                    Assert.AreEqual(perms[2].SecurityItem.Update, reader.GetBoolean("U"));
                    Assert.AreEqual(perms[2].SecurityItem.Delete, reader.GetBoolean("D"));
                    Assert.AreEqual(perms[2].SecurityItem.Browse, reader.GetBoolean("B"));
                    Assert.AreEqual(perms[2].SecurityItem.ReadOwn, reader.GetBoolean("RO"));
                    Assert.AreEqual(perms[2].SecurityItem.UpdateOwn, reader.GetBoolean("UO"));
                    Assert.AreEqual(perms[2].SecurityItem.DeleteOwn, reader.GetBoolean("DO"));
                }
                else
                {
                    Assert.Fail("Unknown or repeated id!");
                }
            }
            reader.Close();

            Assert.AreEqual(3, rowCount);
            try
            {
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (!reader.IsClosed)
                {
                    reader.Close();
                }

                cmd.CommandText = clearUp;
                cmd.ExecuteNonQuery();
            }
        }

        [Test]
        public void GetDocTypePermissions()
        {
            SecurityAccess sa = new SecurityAccess(FixtureData.DocCircAdmin, _con);
            DocTypePermission[] perms = sa.GetDocTypePermissions(FixtureData.DocType1); // tested method
            Assert.AreEqual(4, perms.Length);

            bool[] been = new bool[] { false, false, false, false };
            foreach (DocTypePermission perm in perms)
            {
                if (perm.Role.RoleId == FixtureData.AdminRole && !been[0])
                {
                    been[0] = true;
                    Assert.IsNotNull(perm.PermissionId);
                    Assert.AreEqual(FixtureData.DocType1, perm.DocType);
                    SecurityItem si = perm.SecurityItem;
                    Assert.IsTrue(si.Create);
                    Assert.IsTrue(si.Read);
                    Assert.IsTrue(si.Update);
                    Assert.IsTrue(si.Delete);
                    Assert.IsTrue(si.ReadOwn);
                    Assert.IsTrue(si.UpdateOwn);
                    Assert.IsTrue(si.DeleteOwn);
                }
                else if (perm.Role.RoleId == FixtureData.UserRole && !been[1])
                {
                    been[1] = true;
                    Assert.IsNotNull(perm.PermissionId);
                    Assert.AreEqual(FixtureData.DocType1, perm.DocType);
                    SecurityItem si = perm.SecurityItem;
                    Assert.IsFalse(si.Create);
                    Assert.IsFalse(si.Read);
                    Assert.IsFalse(si.Update);
                    Assert.IsFalse(si.Delete);
                    Assert.IsFalse(si.ReadOwn);
                    Assert.IsFalse(si.UpdateOwn);
                    Assert.IsFalse(si.DeleteOwn);
                }
                else if (perm.Role.RoleId == FixtureData.SaTestRole1 && !been[2])
                {
                    been[2] = true;
                    Assert.IsNotNull(perm.PermissionId);
                    Assert.AreEqual(FixtureData.DocType1, perm.DocType);
                    SecurityItem si = perm.SecurityItem;
                    Assert.IsTrue(si.Create);
                    Assert.IsTrue(si.Read);
                    Assert.IsFalse(si.Update);
                    Assert.IsFalse(si.Delete);
                    Assert.IsTrue(si.ReadOwn);
                    Assert.IsFalse(si.UpdateOwn);
                    Assert.IsFalse(si.DeleteOwn);
                }
                else if (perm.Role.RoleId == FixtureData.SaTestRole2 && !been[3])
                {
                    been[3] = true;
                    Assert.IsNotNull(perm.PermissionId);
                    Assert.AreEqual(FixtureData.DocType1, perm.DocType);
                    SecurityItem si = perm.SecurityItem;
                    Assert.IsFalse(si.Create);
                    Assert.IsTrue(si.Read);
                    Assert.IsFalse(si.Update);
                    Assert.IsFalse(si.Delete);
                    Assert.IsTrue(si.ReadOwn);
                    Assert.IsFalse(si.UpdateOwn);
                    Assert.IsTrue(si.DeleteOwn);
                }
                else
                {
                    Assert.Fail("Unknown or repeated id!");
                }

            }


        }

        [Test]
        public void SaveDocTypePermissions()
        {
            MySqlCommand cmd = _con.MySqlConnection.CreateCommand();
            TextReader txtReader = File.OpenText(
                Path.Combine(FixtureConst.Folder, FixtureConst.SqlFile2)
                );
            cmd.CommandText = txtReader.ReadToEnd();
            txtReader.Close();
            txtReader = File.OpenText(
                Path.Combine(FixtureConst.Folder, FixtureConst.SqlFile2ClearUp)
                );
            string clearUp = txtReader.ReadToEnd();
            txtReader.Close();

            cmd.ExecuteNonQuery();

            cmd.CommandText = "select cast(@docType90 as unsigned);";
            MySqlDataReader reader = cmd.ExecuteReader();
            reader.Read();
            int docType90 = reader.GetInt32(0);
            reader.Close();
            
            SecurityAccess sa = new SecurityAccess(FixtureData.DocCircAdmin, _con);
            
            //case 1
            // clear all permissions

            DocTypePermission[] perms = new DocTypePermission[0];
            sa.SaveDocTypePermissions(docType90, perms); // tested method

            cmd.CommandText = "select count(*) from DocTypePermission where DocType=@docType90;";
            reader = cmd.ExecuteReader();
            reader.Read();
            int count = reader.GetInt32(0);
            reader.Close();
          
                Assert.AreEqual(0, count);

                //case 2
                // add two permissions
                SecurityItem si = new SecurityItem(true, true, true, true, false, true, true, true);
                UserRole role = new UserRole(FixtureData.AdminRole, "Administrators");
                DocTypePermission perm1 = new DocTypePermission(docType90, role, si);
                
                si = new SecurityItem(true, false, true, false, false, true, false, true);
                role = new UserRole(FixtureData.UserRole, "Users");
                DocTypePermission perm2 = new DocTypePermission(docType90, role, si);

                perms = new DocTypePermission[] { perm1, perm2 };

                sa.SaveDocTypePermissions(docType90, perms); // tested method
            
            
                Assert.IsNotNull(perms[0].PermissionId);
                Assert.IsNotNull(perms[1].PermissionId);

                cmd.CommandText =
                    "select DocTypePermissionId, DocType, DcRole, C, R, U, D, RO, UO, DO " +
                    "from DocTypePermission where DocType=@docType90;";
                
                reader = cmd.ExecuteReader();
                //bool violation = false;
          
                bool[] been = new bool[] { false, false };
                int rowCount = 0;
                while (reader.Read())
                {
                    rowCount++;
                    if (reader.GetInt32("DocTypePermissionId") == perms[0].PermissionId && !been[0])
                    {
                        been[0] = true;
                        Assert.AreEqual(perms[0].Role.RoleId, reader.GetInt32("DcRole"));
                        Assert.AreEqual(perms[0].SecurityItem.Create, reader.GetBoolean("C"));
                        Assert.AreEqual(perms[0].SecurityItem.Read, reader.GetBoolean("R"));
                        Assert.AreEqual(perms[0].SecurityItem.Update, reader.GetBoolean("U"));
                        Assert.AreEqual(perms[0].SecurityItem.Delete, reader.GetBoolean("D"));
                        Assert.AreEqual(perms[0].SecurityItem.ReadOwn, reader.GetBoolean("RO"));
                        Assert.AreEqual(perms[0].SecurityItem.UpdateOwn, reader.GetBoolean("UO"));
                        Assert.AreEqual(perms[0].SecurityItem.DeleteOwn, reader.GetBoolean("DO"));
                    }
                    else if (reader.GetInt32("DocTypePermissionId") == perms[1].PermissionId && !been[1])
                    {
                        been[1] = true;
                        Assert.AreEqual(perms[1].Role.RoleId, reader.GetInt32("DcRole"));
                        Assert.AreEqual(perms[1].SecurityItem.Create, reader.GetBoolean("C"));
                        Assert.AreEqual(perms[1].SecurityItem.Read, reader.GetBoolean("R"));
                        Assert.AreEqual(perms[1].SecurityItem.Update, reader.GetBoolean("U"));
                        Assert.AreEqual(perms[1].SecurityItem.Delete, reader.GetBoolean("D"));
                        Assert.AreEqual(perms[1].SecurityItem.ReadOwn, reader.GetBoolean("RO"));
                        Assert.AreEqual(perms[1].SecurityItem.UpdateOwn, reader.GetBoolean("UO"));
                        Assert.AreEqual(perms[1].SecurityItem.DeleteOwn, reader.GetBoolean("DO"));
                    }
                    else
                    {
                        Assert.Fail("Unknown or repeated id!");
                    }
                }
                reader.Close();

                Assert.AreEqual(2, rowCount);
                
                // case 3
                // add a permission to existent ones
                // change perm2 accesses
                si = new SecurityItem(false, false, false, false, false, false, false, false);
                role = new UserRole(FixtureData.SaTestRole1, "TestRole1");
                DocTypePermission perm3 = new DocTypePermission(docType90, role, si);
                perms = new DocTypePermission[] { perms[0], perms[1], perm3 };
                perms[1].SecurityItem = new SecurityItem(false, true, false, true, false, false, true, false);
                int perm1IdBefore = perms[0].PermissionId.Value, perm2IdBefore = perms[1].PermissionId.Value;

                sa.SaveDocTypePermissions(docType90, perms); // tested method
                
                Assert.AreEqual(perm1IdBefore, perms[0].PermissionId);
                Assert.AreEqual(perm2IdBefore, perms[1].PermissionId);
                Assert.IsNotNull(perm3.PermissionId);

                reader = cmd.ExecuteReader();

                been = new bool[] { false, false, false };
                rowCount = 0;
                while (reader.Read())
                {
                    rowCount++;
                    if (reader.GetInt32("DocTypePermissionId") == perms[0].PermissionId && !been[0])
                    {
                        been[0] = true;
                        
                        Assert.AreEqual(perms[0].Role.RoleId, reader.GetInt32("DcRole"));
                        Assert.AreEqual(perms[0].SecurityItem.Create, reader.GetBoolean("C"));
                        Assert.AreEqual(perms[0].SecurityItem.Read, reader.GetBoolean("R"));
                        Assert.AreEqual(perms[0].SecurityItem.Update, reader.GetBoolean("U"));
                        Assert.AreEqual(perms[0].SecurityItem.Delete, reader.GetBoolean("D"));
                        Assert.AreEqual(perms[0].SecurityItem.ReadOwn, reader.GetBoolean("RO"));
                        Assert.AreEqual(perms[0].SecurityItem.UpdateOwn, reader.GetBoolean("UO"));
                        Assert.AreEqual(perms[0].SecurityItem.DeleteOwn, reader.GetBoolean("DO"));
                    }
                    else if (reader.GetInt32("DocTypePermissionId") == perms[1].PermissionId && !been[1])
                    {
                        been[1] = true;
                        Assert.AreEqual(perms[1].Role.RoleId, reader.GetInt32("DcRole"));
                        Assert.AreEqual(perms[1].SecurityItem.Create, reader.GetBoolean("C"));
                        Assert.AreEqual(perms[1].SecurityItem.Read, reader.GetBoolean("R"));
                        Assert.AreEqual(perms[1].SecurityItem.Update, reader.GetBoolean("U"));
                        Assert.AreEqual(perms[1].SecurityItem.Delete, reader.GetBoolean("D"));
                        Assert.AreEqual(perms[1].SecurityItem.ReadOwn, reader.GetBoolean("RO"));
                        Assert.AreEqual(perms[1].SecurityItem.UpdateOwn, reader.GetBoolean("UO"));
                        Assert.AreEqual(perms[1].SecurityItem.DeleteOwn, reader.GetBoolean("DO"));
                    }
                    else if (reader.GetInt32("DocTypePermissionId") == perms[2].PermissionId && !been[2])
                    {
                        been[2] = true;
                        Assert.AreEqual(perms[2].Role.RoleId, reader.GetInt32("DcRole"));
                        Assert.AreEqual(perms[2].SecurityItem.Create, reader.GetBoolean("C"));
                        Assert.AreEqual(perms[2].SecurityItem.Read, reader.GetBoolean("R"));
                        Assert.AreEqual(perms[2].SecurityItem.Update, reader.GetBoolean("U"));
                        Assert.AreEqual(perms[2].SecurityItem.Delete, reader.GetBoolean("D"));
                        Assert.AreEqual(perms[2].SecurityItem.ReadOwn, reader.GetBoolean("RO"));
                        Assert.AreEqual(perms[2].SecurityItem.UpdateOwn, reader.GetBoolean("UO"));
                        Assert.AreEqual(perms[2].SecurityItem.DeleteOwn, reader.GetBoolean("DO"));
                    }
                    else
                    {
                        Assert.Fail("Unknown or repeated id!");
                    }
                }
                reader.Close();    

                Assert.AreEqual(3, rowCount);
                try
                {
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if(!reader.IsClosed)
                {
                    reader.Close();
                }

                cmd.CommandText = clearUp;
                cmd.ExecuteNonQuery();
            }
        }

        [TestFixtureTearDown]
        public void TFTearDown()
        {
            _tran.Rollback();
            _con.Close();
        }

    }

    [TestFixture]
    public class SecurityAccessGetDocCardSecurityTest
    {
        protected struct FixtureConst
        {
            public const int DocCircUser1 = 0;
            public const int DocCircUser2 = 1;
            public const int DocCircUser3 = 2;

            public const long DocCard1 = 0;
            public const long DocCard2 = 1;
            public const long DocCard3 = 2;

            public const int DocType1 = 0;
            public const int DocType2 = 1;

            public const int DocCardFolder1 = 0;
            public const int DocCardFolder2 = 1;
        }

        protected class FixtureData
        {
            public static SecurityItem DocType1User1SI, DocType1User2SI;
            public static SecurityItem Folder1User1SI, Folder1User2SI;
        }

        protected class TestSecurityAccess : SecurityAccess
        {
            public TestSecurityAccess(int userId)
            {
                _userId = userId;
            }

            public override SecurityItem GetDocTypeDocumentsSecurity(int docTypeId)
            {
                SecurityItem retVal = null;
                if (_userId == FixtureConst.DocCircUser1 && docTypeId == FixtureConst.DocType1)
                {
                    retVal = FixtureData.DocType1User1SI;
                }
                else
                {
                    throw new ApplicationException("Unknown id for this unit test!");
                }

                return retVal;
            }

            public override SecurityItem GetFolderSecurity(int folderId)
            {
                SecurityItem retVal = null;
                if (_userId == FixtureConst.DocCircUser1 && folderId == FixtureConst.DocCardFolder1)
                {
                    retVal = FixtureData.Folder1User1SI;
                }
                else
                {
                    throw new ApplicationException("Unknown id for this unit test!");
                }

                return retVal;
            }

        }

        [Test]
        public void GetDocumentCardSecurity()
        {
            /*
             *      C   R   U   D   B   RO  UO  DO
             * 1
             * 2    *                   *   *                                             
             * 3    *                   *   *   *                        
             * 4        *               *
             * 5    *   *               *   *
             * 6        *
             * 7        *   *
             * 8        *   *   *
             * 9    *   *   *   *       *   *   *
             * */

            // create permission test

            /* case 1
             * DocCircUser1 has 1 permission case in folder, and 9 permission case in DocType
             * 
             * */
            FixtureData.Folder1User1SI = new SecurityItem(false, false, false, false, false, false, false, false); //1
            FixtureData.DocType1User1SI = new SecurityItem(true, true, true, true, false, true, true, true); //9

            DocCardItem item = new DocCardItem();
            item.Id = FixtureConst.DocCard1;
            item.DocType = FixtureConst.DocType1;
            item.IsNew = false;
            item.Owner = FixtureConst.DocCircUser2;
            TestSecurityAccess sa = new TestSecurityAccess(FixtureConst.DocCircUser1);
            SecurityItem si = sa.GetDocumentCardSecurity(item); //tested method

            Assert.IsNotNull(si);
            Assert.IsFalse(si.Create);
            Assert.IsFalse(si.Read);
            Assert.IsFalse(si.Update);
            Assert.IsFalse(si.Delete);
            Assert.IsFalse(si.ReadOwn);
            Assert.IsFalse(si.UpdateOwn);
            Assert.IsFalse(si.DeleteOwn);

            /* case 2
             * Folder: 2
             * DocType: 9
             * DocCircUser1 doesn't have permissions because Owner is DocCircUser2.
             * 
             * */
            FixtureData.Folder1User1SI = new SecurityItem(true, false, false, false, false, true, true, false); //2

            si = sa.GetDocumentCardSecurity(item); //tested method

            Assert.IsFalse(si.Read);
            Assert.IsFalse(si.Update);
            Assert.IsFalse(si.Delete);

            /* case 3
             * Folder: 9
             * DocType: 8
             * 
             * DocCard1 is new
             * DocCircUser1 can't create documents of this type, but can create in the folder
             * */

            item.IsNew = true;
            FixtureData.Folder1User1SI = FixtureData.DocType1User1SI;
            FixtureData.DocType1User1SI = new SecurityItem(false, true, true, true, false, false, false, false); //8

            si = sa.GetDocumentCardSecurity(item); //tested method

            Assert.IsTrue(si.Read);
            Assert.IsFalse(si.Update);
            Assert.IsTrue(si.Delete);

            /* case 4
             * Folder: 9
             * DocType: 3
             * 
             * DocCircUser1 has permissions because he is the owner of the document card
             * */
            item.IsNew = false;
            item.Owner = FixtureConst.DocCircUser1;

            FixtureData.DocType1User1SI = new SecurityItem(true, false, false, false, false, true, true, true); // 3

            si = sa.GetDocumentCardSecurity(item); //tested method

            Assert.IsTrue(si.Read);
            Assert.IsTrue(si.Update);
            Assert.IsTrue(si.Delete);

            /* case 4
             * Folder: 9
             * DocType: 3
             * 
             * DocCircUser1 doesn't have permissions because he is not the owner of the document card
             * */

            item.Owner = FixtureConst.DocCircUser2;

            si = sa.GetDocumentCardSecurity(item); //tested method

            Assert.IsFalse(si.Read);
            Assert.IsFalse(si.Update);
            Assert.IsFalse(si.Delete);

            /* case 5
             * Folder: 9
             * DocType: 9
             * 
             * DocCircUser1 have all permissions
             * */
            FixtureData.DocType1User1SI = FixtureData.Folder1User1SI;
            si = sa.GetDocumentCardSecurity(item); //tested method

            Assert.IsTrue(si.Read);
            Assert.IsTrue(si.Update);
            Assert.IsTrue(si.Delete);


            /* case 6
             * Folder: 9
             * DocType: not set
             * 
             * New document
             * */
            item.IsNew = true;
            item.DocType = null;
            item.Owner = FixtureConst.DocCircUser1;

            si = sa.GetDocumentCardSecurity(item); //tested method

            Assert.IsTrue(si.Read);
            Assert.IsTrue(si.Update);
            Assert.IsTrue(si.Delete);
        }


    }

}
