//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using System.Resources;
using System.IO;
using NUnit.Framework;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.UI;
using Evpo.DocCirc.Biz;

namespace Evpo.DocCircUnitTest
{
    [TestFixture]
    public class EventHtmlComposerTest
    {
        public struct FixtureFile
        {
            public const string ApplicationFolder = "UI";
            public const string Template = "EventHtmlComposer.htm";
        }

        protected struct FixtureConst
        {
            public const int DocCircUser1 = 10;
            public const int DocCircUser2 = 11;
            public const int ServiceUserId = 12;
            public const string ServiceUserFullName = "DocCirc Service";
            public const string ServiceUserEmail = "doccircservice@test.org";
            public const string DocCircUser1FullName = "DocCirc user1";
            public const string DocCircUser2FullName = "DocCirc user2";
            public const long DocCard1 = 21, DocCard2 = 22;
            public const int DocType1 = 23;
            public const string DocType1Name = "Doc type 1";
            public const long DocCardEvent1 = 101;
        }

        protected struct ResourceString
        {
            public const string EventTimeField = "Event time:";
            public const string EventUserField = "User:";
            public const string NoteField = "User note:";
            public const string DeputyField = "The action was done by the following deputy of the user:";
            public const string EventTypePassDecision = "The user passed the document card";
        }


        protected struct DocCard1Property
        {
            public const string Subject = "doc card 1 subject";
            public const string InternalNum = "doc card 1 internal num";
            public const long Id = FixtureConst.DocCard1;
            public const int RegisteringUser = FixtureConst.DocCircUser1;
        }

        protected class MockResourceManager : ResourceManager
        {
            public override string GetString(string key)
            {
                string retVal = "";
                if (key == EventHtmlComposer.ResKeys.EventTimeField)
                {
                    retVal = ResourceString.EventTimeField;
                }
                else if (key == EventHtmlComposer.ResKeys.EventUserField)
                {
                    retVal = ResourceString.EventUserField;
                }
                else if (key == EventHtmlComposer.ResKeys.NoteField)
                {
                    retVal = ResourceString.NoteField;
                }
                else if (key == EventHtmlComposer.ResKeys.DeputyField)
                {
                    retVal = ResourceString.DeputyField;
                }
                else if (key == EventHtmlComposer.ResKeys.EventTypePassDecision)
                {
                    retVal = ResourceString.EventTypePassDecision;
                }
                else
                {
                    throw new ApplicationException("Unknown resource for this unit test!");
                }

                return retVal;
            }
        }

        protected class MockStringResHelperInstance : StringResHelper
        {
            public override string this[string key]
            {
                get
                {
                    string retVal = null;
                    throw new ApplicationException("Resource is unknown in this test!");
                    return retVal;
                }
            }
        }

        protected class MockUserHelperInstance : UserHelperInstance
        {
            //public override string[] GetUserEmails(int userId, DataConnection con)
            //{
            //    string[] retVal = null;
            //    if (userId == FixtureConst.DocCircUser2)
            //    {
            //        retVal = new string[] { FixtureConst.DocCircUser2Email };
            //    }
            //    else
            //    {
            //        throw new ApplicationException("Unknown userId for this unit test!");
            //    }
            //    return retVal;
            //}

            public override UserProperties GetUserProperties(int userId, DataConnection con)
            {
                UserProperties props = new UserProperties();
                if (userId == FixtureConst.DocCircUser1)
                {
                    props.FullName = FixtureConst.DocCircUser1FullName;
                }
                else if (userId == FixtureConst.DocCircUser2)
                {
                    props.FullName = FixtureConst.DocCircUser2FullName;
                }
                else
                {
                    throw new ApplicationException("Unknown doc circ user for this unit test!");
                }
                return props;
            }

        }

        protected class MockUserInfo : UserInfo
        {
            public void SetEmails(string[] emails)
            {
                _emails = emails;
            }

            public void SetUserProperties(UserProperties props)
            {
                _userProperties = props;
            }
        }

        protected class TestEventHtmlComposer : EventHtmlComposer
        {
            protected override ResourceManager GetResourceManager()
            {
                return new MockResourceManager();
            }

            public TestEventHtmlComposer(SettingKeeper sk, DataConnection con, UserInfo serviceUserInfo) : base(sk, con, serviceUserInfo) { }
        }

        protected NameValueCollection _replaceResourceValues;
        protected SettingKeeper _sk;
        protected string _template;
        protected MockUserInfo _serviceUser;


        [TestFixtureSetUp]
        public void TFSetUp()
        {
            _sk = new SettingKeeper();
            _sk.ApplicationFolder = FixtureFile.ApplicationFolder;

            _serviceUser = new MockUserInfo();
            _serviceUser.SetEmails(new string[] { FixtureConst.ServiceUserEmail });
            UserProperties userProp = new UserProperties();
            userProp.UserId = FixtureConst.ServiceUserId;
            userProp.Login = "doccircservice";
            userProp.FullName = FixtureConst.ServiceUserFullName;
            userProp.AutomaticPass = false;
            userProp.AllowDeputyNotification = true;
            userProp.Department = 0;
            userProp.InstantMessaging = false;
            userProp.ImAddress = "";

            _serviceUser.SetUserProperties(userProp);

            UserHelper.Current = new MockUserHelperInstance();

            DBConstKeeper.Current = new DBConstKeeper();
            DBConstKeeper.Current.EventType = new EventTypeDirectory(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14);

            _replaceResourceValues = new NameValueCollection();

            _replaceResourceValues.Add("^reskey:EventTimeField^", ResourceString.EventTimeField);
            _replaceResourceValues.Add("^reskey:EventUserField^", ResourceString.EventUserField);
            _replaceResourceValues.Add("^reskey:NoteField^", ResourceString.NoteField);

            //TextReader reader = File.OpenText(
            //    Path.Combine(FixtureFile.ApplicationFolder, FixtureFile.Template)
            //    );
            //_template = reader.ReadToEnd();
            //reader.Close();

            _template = UIResources.EventHtmlComposer;
            ResourceHelper.StrForUI = new MockStringResHelperInstance();
        }

        [Test]
        public void ComposeHtml()
        {
            EventItem item = new EventItem(
                FixtureConst.DocCardEvent1,
                FixtureConst.DocCard1,
                DBConstKeeper.Current.EventType.PassDecision,
                FixtureConst.DocCircUser1,
                FixtureConst.DocCircUser2,
                2,
                new DateTime(2007, 06, 10, 12, 10, 00),
                "document is ok!",
                "text1"
                );

            

            string masterText = ReplaceValues(_template, _replaceResourceValues);
            NameValueCollection replacedValues = new NameValueCollection();
            replacedValues.Add("^reskey:DeputyField^", ResourceString.DeputyField);
            
            replacedValues.Add("^EventTime^", item.EventTime.ToString());
            replacedValues.Add("^UserFullName^", FixtureConst.DocCircUser1FullName);
            replacedValues.Add("^DeputyFullName^", FixtureConst.DocCircUser2FullName);
            replacedValues.Add("^Text^", item.Text);
            replacedValues.Add("^EventTypeText^", ResourceString.EventTypePassDecision);
            replacedValues.Add("^Note^", item.Note);

            replacedValues.Add("^control:UserRowDisplay^", "inline");
            replacedValues.Add("^control:DeputyRowDisplay^", "inline");
            replacedValues.Add("^control:NoteRowDisplay^", "inline");



            masterText = ReplaceValues(masterText, replacedValues);
            Console.Write(masterText);
            TestEventHtmlComposer composer = new TestEventHtmlComposer(_sk, null, _serviceUser);
            
            string resultText = composer.ComposeHtml(item); // tested method
            
            Assert.IsNotNull(resultText);
            StringReader masterTextReader = new StringReader(masterText);
            StringReader resultTextReader = new StringReader(resultText);

            while (masterTextReader.Peek() >= 0)
            {
                Assert.GreaterOrEqual(resultTextReader.Peek(), 0);
                Assert.AreEqual(masterTextReader.ReadLine(), resultTextReader.ReadLine());
            }
            
        }

        protected string ReplaceValues(string template, NameValueCollection replaceValues)
        {
            foreach (string key in replaceValues.AllKeys)
            {
                template = template.Replace(key, replaceValues[key]);
            }
            return template;
        }
    }
}
