//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using NUnit.Framework;
using Evpo.DocCirc.UI;

namespace Evpo.DocCircUnitTest
{
    [TestFixture]
    public class MenuKeeperTest
    {
        protected string _xmlText;
        private const string _xsdFile1Path = @"UI\Page01.MainMenu.xsd";

        [TestFixtureSetUp]
        public void FixtureSetUp()
        {
            string r = Environment.NewLine;
            _xmlText = "" +
                "<?xml version='1.0' encoding='utf-8' standalone='no'?>                                         " + r +
                "<Main xmlns='http://tempuri.org/Page01.xsd' DefaultMenuId='.0'>                                " + r +
                "       <MenuItems>                                                                             " + r +
                "           <MenuItem NameResKey='FileMenu' ControlFile='FileSubMenu.aspx'>                     " + r +
                "               <Parameters>                                                                    " + r +
                "                   <MenuItemParameter Name='Theme' Value='classic' />                                  " + r +
                "               </Parameters>                                                                   " + r +
                "               <MenuItems>                                                                     " + r +
                "                   <MenuItem NameResKey='NewFile' ControlFile='CreateNewFile.aspx'>            " + r +
                "                       <Parameters>                                                            " + r +
                "                           <MenuItemParameter Name='FileType' Value='c# code'/>                        " + r +
                "                           <MenuItemParameter Name='FilePath' Value='temp folder' />                   " + r +
                "                       </Parameters>                                                           " + r +
                "                       <MenuItems/>                                                            " + r +
                "                   </MenuItem>                                                                 " + r +
                "                   <MenuItem NameResKey='CreateProject' ControlFile='CreateNewProject.aspx'>   " + r +
                "                       <Parameters>                                                            " + r +
                "                           <MenuItemParameter Name='FileType' Value='window project'/>                 " + r +
                "                           <MenuItemParameter Name='FilePath' Value='temp folder' />                   " + r +
                "                       </Parameters>                                                           " + r +
                "                       <MenuItems/>                                                            " + r +
                "                   </MenuItem>                                                                 " + r +
                "               </MenuItems>                                                                    " + r +
                "           </MenuItem>                                                                         " + r +
                "           <MenuItem NameResKey='exit' ControlFile='Exit.aspx'>                                " + r +
                "               <Parameters/>                                                                   " + r +
                "               <MenuItems/>                                                                    " + r +
                "           </MenuItem>                                                                         " + r +
                "       </MenuItems>                                                                            " + r +
                "   </Main>                                                                                     ";
            

        }

        [Test]
        public void CtorTest()
        {
            StreamReader schema = File.OpenText(_xsdFile1Path);
            MenuKeeper mk = new MenuKeeper(new StringReader(_xmlText), schema); //Tested method
            schema.Close();

            Assert.AreEqual(".0", mk.MenuData.DefaultMenuId);
            Assert.AreEqual(2, mk.MenuData.MenuItems.Length);

            DcMenuItem fileMenu = mk.MenuData.MenuItems[0];
            Assert.AreEqual(".0", fileMenu.Id);
            Assert.AreEqual("FileMenu", fileMenu.NameResKey);
            Assert.AreEqual("FileSubMenu.aspx", fileMenu.ControlFile);
            Assert.AreEqual(2, fileMenu.MenuItems.Length);

            DcMenuItem fileMenuNewFile = fileMenu.MenuItems[0];
            Assert.AreEqual(".0.0", fileMenuNewFile.Id);
            Assert.AreEqual("NewFile", fileMenuNewFile.NameResKey);
            Assert.AreEqual("CreateNewFile.aspx", fileMenuNewFile.ControlFile);
            Assert.AreEqual(2, fileMenuNewFile.Parameters.Length);
            Assert.AreEqual("FilePath", fileMenuNewFile.Parameters[1].Name);
            Assert.AreEqual("temp folder", fileMenuNewFile.Parameters[1].Value);

            DcMenuItem exit = mk.MenuData.MenuItems[1];
            Assert.AreEqual(".1", exit.Id);
            Assert.AreEqual("exit", exit.NameResKey);
            Assert.AreEqual("Exit.aspx", exit.ControlFile);
            Assert.IsEmpty(exit.MenuItems);
            Assert.IsEmpty(exit.Parameters);
        }

        [Test]
        public void GetMenuItem()
        {
            StreamReader schema = File.OpenText(_xsdFile1Path);
            MenuKeeper mk = new MenuKeeper(new StringReader(_xmlText), schema);
            schema.Close();

            DcMenuItem createProject = mk.GetMenuItem(".0.1");
            Assert.AreEqual("CreateProject", createProject.NameResKey);
            Assert.AreEqual("CreateNewProject.aspx", createProject.ControlFile);

            DcMenuItem newFile = mk.GetMenuItem(".0.0");
            Assert.AreEqual("NewFile", newFile.NameResKey);
            Assert.AreEqual("CreateNewFile.aspx", newFile.ControlFile);

            DcMenuItem item;
            
            item = mk.GetMenuItem(".0");
            Assert.AreEqual("FileMenu", item.NameResKey);
            Assert.AreEqual("FileSubMenu.aspx", item.ControlFile);

            item = mk.GetMenuItem(".1");
            Assert.AreEqual("exit", item.NameResKey);
            Assert.AreEqual("Exit.aspx", item.ControlFile);

        }


    }
}
