//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using System.Resources;
using System.Collections.Specialized;
using Evpo.DocCirc.UI;
using Evpo.DocCirc;
using Evpo.DocCirc.Common;
using NUnit.Framework;


namespace Evpo.DocCircUnitTest
{
    /// <summary>
    /// see name
    /// </summary>
    class MockResourceManager : ResourceManager
    {
        private bool _resourcesReleased = false;
        public bool ResourcesReleased
        {
            get { return _resourcesReleased; }
        }


        private NameValueCollection _strings = new NameValueCollection();
	    public NameValueCollection Strings
	    {
	      get { return _strings;}
	    }

        public MockResourceManager()
        {
        }

        public override string GetString(string name)
        {
 	         return _strings[name];
        }

        public override void ReleaseAllResources()
        {
            _resourcesReleased = true;
        }
    }

    /// <summary>
    /// see name
    /// </summary>
    class TestStringResHelperInstance : StringResHelper
    {
        public TestStringResHelperInstance(string baseName, AssemblyWrapper assembly)
        {
        }

        public ResourceManager RM
        {
            get { return _rm; }
            set { _rm = value; }
        }

        
    }

    [TestFixture]
    public class ResourceHelperInstanceTest
    {
        [TestFixtureSetUp]
        public void Prepare()
        {
            MockResourceManager logRM = new MockResourceManager();
            logRM.Strings.Add("Greeting", "Hello user!");
            logRM.Strings.Add("First", "first string");
            logRM.Strings.Add("Second", "second string");

            MockResourceManager uiRM = new MockResourceManager();
            uiRM.Strings.Add("Greeting", "Hello user in UI!");
            uiRM.Strings.Add("First", "first string in UI");
            uiRM.Strings.Add("Second", "second string in UI");

            TestStringResHelperInstance logResHlp =
                new TestStringResHelperInstance(GlobalStringConstants.StringsForLogResourcesId, null);
            logResHlp.RM = logRM;

            TestStringResHelperInstance uiResHlp =
                new TestStringResHelperInstance(GlobalStringConstants.StringsForUIResourcesId, null);
            uiResHlp.RM = uiRM;

            ResourceHelper.StrForLog = logResHlp;
            ResourceHelper.StrForUI = uiResHlp;
        }

        [Test]
        public void Indexer()
        {
            Assert.AreEqual("Hello user!", ResourceHelper.StrForLog["Greeting"]);
            Assert.AreEqual("second string", ResourceHelper.StrForLog["Second"]);

            Assert.AreEqual("first string in UI", ResourceHelper.StrForUI["First"]);
            Assert.AreEqual("second string in UI", ResourceHelper.StrForUI["Second"]);
        }

        [Test]
        [ExpectedException(ExpectedException=typeof(StringResHelper.WrongKeyException), 
            ExpectedMessage="can't find a resource with the \"wrong key\" key")]
        public void WrongKeyInIndexer()
        {
            string str = ResourceHelper.StrForLog["wrong key"];
        }

        [Test]
        public void ReleaseAllResources()
        {
            ResourceHelper.StrForLog.ReleaseAllResources();
            TestStringResHelperInstance strResHelper= (TestStringResHelperInstance)ResourceHelper.StrForLog;
            Assert.IsTrue(((MockResourceManager)strResHelper.RM).ResourcesReleased);
        }
    }
}
