//**********************************************************************************
//EVPO Document Circulation Copyright  2007 Evgeny Pokhilko 
//<evpomail@gmail.com> <http://www.evpo.net>
//
//This file is part of EVPO Document Circulation.
//
//EVPO Document Circulation is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 2 of the License, or
//(at your option) any later version.
//
//EVPO Document Circulation is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with EVPO Document Circulation.  If not, see <http://www.gnu.org/licenses/>.
//**********************************************************************************

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.IO;
using NUnit.Framework;
using Evpo.DocCirc.Biz;
using Evpo.DocCirc.Common;
using Evpo.DocCirc.UI;

namespace Evpo.DocCircUnitTest
{
    public class MockUserHelperInstance : UserHelperInstance
    {
        public override UserProperties GetUserProperties(int userId, DataConnection con)
        {
            UserProperties retVal = null;
            retVal = new UserProperties();
            switch (userId)
            {
                case 0:
                    retVal.UserId = 0;
                    retVal.Login = "first";
                    retVal.FullName = "First User";
                    retVal.AutomaticPass = false;
                    retVal.AllowDeputyNotification = true;
                    retVal.Department = 0;
                    retVal.InstantMessaging = false;

                    break;
                case 1:
                    retVal.UserId = 1;
                    retVal.Login = "second";
                    retVal.FullName = "Second User";
                    retVal.AutomaticPass = false;
                    retVal.AllowDeputyNotification = true;
                    retVal.Department = 0;
                    retVal.InstantMessaging = false;
                    break;
            }
            return retVal;
        } 
    }

    [TestFixture]
    public class RoutePointSerializerTest
    {
        private const string _xmlFile1Path = @"UI\RoutePointSerializerTest.file1.xml";

        private UserHelperInstance _storedUserHelper;
        private int _decisionType;
        private int _notificationType;

        [TestFixtureSetUp]
        public void TFSetUp()
        {
           

            _storedUserHelper = UserHelper.Current;
            UserHelper.Current = new MockUserHelperInstance();

            _decisionType = 10;
            _notificationType = 11;
            int noMatter = 0;
            DBConstKeeper.Current = new DBConstKeeper();
            DBConstKeeper.Current.RouteItemType = new RouteItemTypeDirectory(_decisionType, _notificationType);
            //new FolderTypeDirectory(noMatter, noMatter),
            //new RootFolderDirectory(noMatter, noMatter),
            //new MainRoleDirectory(noMatter, noMatter),
            //new RouteItemTypeDirectory(_decisionType, _notificationType),
            //new DocStatusDirectory(noMatter, noMatter, noMatter, noMatter)
            //);
        }

        [Test]
        public void SerializeRoutePoints()
        {
            // see file RouteItemSerializer.file1.xml
            // see MockUserHelperInstance.GetUserProperties
            List<RoutePoint> points = new List<RoutePoint>(2);
            RoutePoint rp = new RoutePoint(0, "[0]&&[0]", 1);
            points.Add(rp);
            rp.RouteItems.Add(new RouteItem(0, 0, _decisionType));
            rp.RouteItems.Add(new RouteItem(null, 1, _decisionType));

            rp = new RoutePoint(null, "[0]", 2);
            points.Add(rp);
            rp.RouteItems.Add(new RouteItem(null, 1, _decisionType));

            RoutePointSerializer ser = new RoutePointSerializer(new DataConnection());
            string resultString = ser.SerializeRoutePoints(points); // tested method
            //Console.Write(resultString);
            Assert.IsNotNull(resultString);
            
            TextReader testResultXml = new StringReader(resultString);
            XmlReader testResult = XmlReader.Create(testResultXml);
            TextReader masterXml = File.OpenText(_xmlFile1Path);
            XmlReader master = XmlReader.Create(masterXml);

            master.ReadStartElement("Points");
            testResult.ReadStartElement("Points");
            
            while (master.Read() && master.Name == "RoutePointForClient" && master.NodeType == XmlNodeType.Element)
            {
                Assert.IsTrue(
                    testResult.Read() && 
                    testResult.Name == "RoutePointForClient" && 
                    testResult.NodeType == XmlNodeType.Element
                    );
                
                Assert.AreEqual(master.AttributeCount, testResult.AttributeCount);
                Assert.IsNotNull(testResult.GetAttribute("OrderNum"));
                Assert.AreEqual(master.GetAttribute("OrderNum"), testResult.GetAttribute("OrderNum"));
                Assert.IsNotNull(testResult.GetAttribute("Expression"));
                Assert.AreEqual(master.GetAttribute("Expression"), testResult.GetAttribute("Expression"));

                master.ReadStartElement("RoutePointForClient");
                testResult.ReadStartElement("RoutePointForClient");
                master.Read();
                testResult.Read();
                if (master.GetAttribute("xsi:nil") != null && master.GetAttribute("xsi:nil") == "true")
                {
                    Assert.AreEqual("true", testResult.GetAttribute("xsi:nil"));
                    bool masterIdIsEmpty = master.IsEmptyElement, testResultIdIsEmpty = testResult.IsEmptyElement;
                    master.ReadStartElement("Id");
                    testResult.ReadStartElement("Id");

                    if (!masterIdIsEmpty)
                    {
                        master.ReadEndElement(); // of Id
                    }

                    if (!testResultIdIsEmpty)
                    {
                        testResult.ReadEndElement(); // of Id
                    }
                }
                else
                {
                    Assert.AreEqual(master.ReadString(), testResult.ReadString());
                    master.ReadEndElement(); // of Id
                    testResult.ReadEndElement(); // of Id
                }
                master.Read();
                testResult.Read();
                master.ReadStartElement("RouteItems");
                testResult.ReadStartElement("RouteItems");

                while (master.Read() && master.Name == "RouteItemForClient" && master.NodeType == XmlNodeType.Element)
                {
                    Assert.IsTrue(
                        testResult.Read() &&
                        testResult.Name == "RouteItemForClient" &&
                        testResult.NodeType == XmlNodeType.Element
                        );

                    Assert.IsNotNull(testResult.GetAttribute("RouteItemType"));
                    Assert.AreEqual(master.GetAttribute("RouteItemType"), testResult.GetAttribute("RouteItemType"));
                    Assert.IsNotNull(testResult.GetAttribute("UserFullName"));
                    Assert.AreEqual(master.GetAttribute("UserFullName"), testResult.GetAttribute("UserFullName"));
                    Assert.IsNotNull(testResult.GetAttribute("User"));
                    Assert.AreEqual(master.GetAttribute("User"), testResult.GetAttribute("User"));
                    Assert.IsNotNull(testResult.GetAttribute("PointOrderNum"));
                    Assert.AreEqual(master.GetAttribute("PointOrderNum"), testResult.GetAttribute("PointOrderNum"));


                    master.ReadStartElement("RouteItemForClient");
                    testResult.ReadStartElement("RouteItemForClient");



                    master.Read();
                    testResult.Read();

                    if (master.GetAttribute("xsi:nil") != null && master.GetAttribute("xsi:nil") == "true")
                    {
                        Assert.AreEqual("true", testResult.GetAttribute("xsi:nil"));
                        bool masterIdIsEmpty = master.IsEmptyElement, testResultIdIsEmpty = testResult.IsEmptyElement;
                        master.ReadStartElement("Id");
                        testResult.ReadStartElement("Id");

                        if (!masterIdIsEmpty)
                        {
                            master.ReadEndElement(); // of Id
                        }

                        if (!testResultIdIsEmpty)
                        {
                            testResult.ReadEndElement(); // of Id
                        }

                    }
                    else
                    {
                        Assert.AreEqual(master.ReadString(), testResult.ReadString());
                        master.ReadEndElement(); // of Id
                        testResult.ReadEndElement(); // of Id
                    }

                    master.ReadEndElement(); // of RouteItemForClient
                    testResult.ReadEndElement(); // of RouteItemForClient
                }
                master.ReadEndElement(); // of RouteItems
                testResult.ReadEndElement(); // of RouteItems
                master.ReadEndElement(); // of RoutePointForClient
                testResult.ReadEndElement(); // of RoutePointForClient
            }

            master.ReadEndElement(); // of Points
            testResult.ReadEndElement(); // of Points
            master.Close();
            testResult.Close();
            masterXml.Close();
        }

        [Test]
        public void DeserializeRouteItems()
        {
            RoutePointSerializer ser = new RoutePointSerializer(new DataConnection());
            TextReader reader = File.OpenText(_xmlFile1Path);
            List<RoutePoint> points = ser.DeserializeRoutePoints(reader.ReadToEnd());
            reader.Close();
            Assert.IsNotNull(points);
            Assert.AreEqual(2, points.Count);

            // route point 1
            Assert.AreEqual(0, points[0].Id);
            Assert.AreEqual("[0]&&[0]", points[0].Expression);
            Assert.AreEqual(1, points[0].OrderNum);

            Assert.AreEqual(2, points[0].RouteItems.Count);

            Assert.AreEqual(0, points[0].RouteItems[0].Id);
            Assert.AreEqual(0, points[0].RouteItems[0].User);
            Assert.AreEqual(_decisionType, points[0].RouteItems[0].RouteItemType);

            Assert.IsNull(points[0].RouteItems[1].Id);
            Assert.AreEqual(1, points[0].RouteItems[1].User);
            Assert.AreEqual(_decisionType, points[0].RouteItems[1].RouteItemType);


            // route point 2
            Assert.IsNull(points[1].Id);
            Assert.AreEqual("[0]", points[1].Expression);
            Assert.AreEqual(2, points[1].OrderNum);

            Assert.AreEqual(1, points[1].RouteItems.Count);

            Assert.IsNull(points[1].RouteItems[0].Id);
            Assert.AreEqual(1, points[1].RouteItems[0].User);
            Assert.AreEqual(_decisionType, points[1].RouteItems[0].RouteItemType);
        }

        [TestFixtureTearDown]
        public void TFTearDown()
        {
            UserHelper.Current = _storedUserHelper;
        }
    }
}
